//---------------------------------------------------------------------------------------------------------------------------------------------------
// Modified:    2025-03-11        Name: mki_lib6.h
//
// Purpose:     Prototypes and definitions of the Malz++Kassner CAD6interface Version 2025.0
//
//              Outdated prototypes and definitions for older versions of the CAD6interface can be found at the end of this file.
//
//              All prototypes and definitions stated in this file are subject to change without notice. Errors excepted. Be sure to check the
//              interface's version (as passed to MKI_PlugInInit) before accessing ANY of the interface procedures!
//
//              Plug-ins using this interface may be distributed without any license fee. If you would like to offer your plug-ins in a bundle
//              including the serving application, please contact Malz++Kassner at the following address:
//
//                  Malz++Kassner GmbH
//                  Am Westerberge 46
//                  38122 Braunschweig
//                  Germany
//
//                  Phone +49 531 400 137
//                  Fax   +49 531 400 138
//
//                  www.CAD6.com
//                  post@malz-kassner.com
//
//              For further information on how to create plug-ins based on the Malz++Kassner CAD6interface, see the interface documentation
//              that is available online at "https://www.cad6.com/interface".
//---------------------------------------------------------------------------------------------------------------------------------------------------

#ifdef _MSC_VER

#pragma warning( disable:4200 ) // Nonstandard extension used: zero-sized array in struct/union
#pragma warning( disable:4302 ) // 'type cast' : truncation from ... to ...
#pragma warning( disable:4311 ) // 'type cast' : pointer truncation from ... to ...
#pragma warning( disable:4312 ) // 'type cast' : conversion from ... to ... of greater size
#pragma warning( disable:4355 ) // 'this' : used in base member initializer list
#pragma warning( disable:4480 ) // Nonstandard extension used: specifying underlying type for enum
#pragma warning( disable:4706 ) // Assignment within conditional expression
#pragma warning( disable:4996 ) // 'xxx' was declared deprecated
#pragma warning( disable:6054 ) // 'xxx' might not zero-terminated

#endif

//---------------------------------------------------------------------------------------------------------------------------------------------------

#include  <math.h>  // Floating point arithmetics


//---------------------------------------------------------------------------------------------------------------------------------------------------

constexpr __int32 MKI_INTERFACE_VERSION = 6250; // Current Open CAD6interface version
constexpr __int32 MKI_MKD_VERSION       = 6250; // Current version of CAD6 file format

constexpr LPWSTR MKI_MAIN_WINDOW_CLASS    = L"MKCAD6_MainWClass";
constexpr LPWSTR MKI_STARTER_WINDOW_CLASS = L"MKCAD6_StarterWClass";


//---------------------------------------------------------------------------------------------------------------------------------------------------

#ifndef MKI_CLASS_TYPES
#define MKI_CLASS_TYPES(TYPE)                           \
typedef MKI_##TYPE* MKI_##TYPE##_PTR;                   \
typedef const MKI_##TYPE* const MKI_CONST_##TYPE##_PTR; \
typedef MKI_##TYPE& MKI_##TYPE##_REF;                   \
typedef const MKI_##TYPE& MKI_CONST_##TYPE##_REF;
#endif

//---------------------------------------------------------------------------------------------------------------------------------------------------

#ifdef  MKI_SERVER
#define MKI_INTERFACE_MEMBER  __declspec( dllexport )
#else
#define MKI_INTERFACE_MEMBER  __declspec( dllimport )
#endif

#define MKI_INTERFACE_EXPORT  extern "C" __declspec( dllexport )
#define MKI_INTERFACE_IMPORT  extern "C" __declspec( dllimport )


//---------------------------------------------------------------------------------------------------------------------------------------------------

#ifndef META_CREATEBITMAPINDIRECT
#define META_CREATEBITMAPINDIRECT   0x02FD
#endif

#ifndef WM_MOUSEWHEEL
#define WM_MOUSEWHEEL               0x020A
#endif

#ifndef WM_MOUSEHWHEEL
#define WM_MOUSEHWHEEL              0x020E
#endif

#ifndef WHEEL_DELTA
#define WHEEL_DELTA                 120
#endif

#ifndef WHEEL_PAGESCROLL
#define WHEEL_PAGESCROLL            (UINT_MAX)
#endif


//---------------------------------------------------------------------------------------------------------------------------------------------------

#define Dialog_SetDefID(hwndCtl, id)        ((void) SendMessage((hwndCtl), DM_SETDEFID, (WPARAM) (id), 0 ))
#define Control_SetFont(hwndCtl, hFont)     ((void) SendMessage((hwndCtl), WM_SETFONT, (WPARAM) (hFont), 0 ))
#define ComboBox_GetSelFieldHeight(hwndCtl) ((__int32) SendMessage((hwndCtl), CB_GETITEMHEIGHT, (WPARAM) (__int32) -1, 0 ))


#ifndef MKI_RESOURCE

//---------------------------------------------------------------------------------------------------------------------------------------------------

typedef ID2D1DeviceContext*   MKI_D2D_CONTEXT;
typedef ID2D1StrokeStyle*     MKI_D2D_STROKE;
typedef ID2D1Brush*           MKI_D2D_BRUSH;
typedef ID2D1SolidColorBrush* MKI_D2D_SOLID_BRUSH;
typedef ID2D1BitmapBrush*     MKI_D2D_BITMAP_BRUSH;
typedef ID2D1PathGeometry*    MKI_D2D_PATH;
typedef ID2D1GeometrySink*    MKI_D2D_GEOSINK;
typedef ID2D1Layer*           MKI_D2D_LAYER;
typedef ID2D1Bitmap1*         MKI_D2D_BITMAP;

typedef IDWriteFont*       MKI_D2D_FONT;
typedef IDWriteTextFormat* MKI_D2D_TEXTFORMAT;
typedef IDWriteTextLayout* MKI_D2D_TEXTLAYOUT;

typedef D2D1_POINT_2F       MKI_D2D_POINT;
typedef D2D1_TRIANGLE       MKI_D2D_TRIANGLE;
typedef D2D1_RECT_F         MKI_D2D_RECT;
typedef D2D1_BEZIER_SEGMENT MKI_D2D_BEZIER;

typedef IWICBitmap*       MKI_WIC_BITMAP;
typedef IWICBitmapSource* MKI_WIC_BITMAP_SOURCE;

//---------------------------------------------------------------------------------------------------------------------------------------------------

constexpr double MKI_MATH_SIN_15 = 0.25881904510252076234889883762405; // sin( 15 )
constexpr double MKI_MATH_COS_15 = 0.96592582628906828674974319972890; // cos( 15 )
constexpr double MKI_MATH_TAN_15 = 0.26794919243112270647255365849413; // tan( 15 )
constexpr double MKI_MATH_SIN_7  = 0.12186934340514748111289391923153; // sin( 7 )
constexpr double MKI_MATH_COS_7  = 0.99254615164132203498006158933058; // cos( 7 )
constexpr double MKI_MATH_TAN_7  = 0.12278456090290459113423113605286; // tan( 7 )
constexpr double MKI_MATH_SIN_30 = 0.5;                                // sin( 30 )
constexpr double MKI_MATH_COS_30 = 0.86602540378443864676372317075294; // cos( 30 )
constexpr double MKI_MATH_TAN_30 = 0.57735026918962576450914878050196; // tan( 30 )
constexpr double MKI_MATH_SIN_42 = 0.66262004821573747750310324234623; // sin( 41.5 )
constexpr double MKI_MATH_COS_42 = 0.74895572078900214550328995017687; // cos( 41.5 )
constexpr double MKI_MATH_TAN_42 = 0.88472526455594376872840733762145; // tan( 41.5 )

constexpr double MKI_MATH_01PI  = 0.31415926535897932384626433832795; // 0.1 pi
constexpr double MKI_MATH_025PI = 0.78539816339744830961566084581988; // 0.25 pi
constexpr double MKI_MATH_05PI  = 1.5707963267948966192313216916398;  // 0.5 pi
constexpr double MKI_MATH_PI    = 3.1415926535897932384626433832795;  // 1.0 pi
constexpr double MKI_MATH_15PI  = 4.7123889803846898576939650749193;  // 1.5 pi
constexpr double MKI_MATH_2PI   = 6.2831853071795864769252867665590;  // 2.0 pi
constexpr double MKI_MATH_25PI  = 7.8539816339744830961566084581988;  // 2.5 pi
constexpr double MKI_MATH_3PI   = 9.4247779607693797153879301498385;  // 3.0 pi
constexpr double MKI_MATH_35PI  = 10.995574287564276334619251841478;  // 3.5 pi
constexpr double MKI_MATH_4PI   = 12.566370614359172953850573533118;  // 4.0 pi

constexpr double MKI_MATH_E        = 2.7182818284590452353602874713527;  // e
constexpr double MKI_MATH_LOG10_2  = 0.30102999566398119521373889472449; // log10( 2 )
constexpr double MKI_MATH_LOG10_3  = 0.47712125471966243729502790325512; // log10( 3 )
constexpr double MKI_MATH_LOG10_4  = 0.60205999132796239042747778944899; // log10( 4 )
constexpr double MKI_MATH_LOG10_5  = 0.69897000433601880478626110527551; // log10( 5 )
constexpr double MKI_MATH_LOG10_6  = 0.77815125038364363250876679797961; // log10( 6 )
constexpr double MKI_MATH_LOG10_7  = 0.84509804001425683071221625859264; // log10( 7 )
constexpr double MKI_MATH_LOG10_8  = 0.90308998699194358564121668417348; // log10( 8 )
constexpr double MKI_MATH_LOG10_9  = 0.95424250943932487459005580651023; // log10( 9 )
constexpr double MKI_MATH_LOG10_10 = 1.0;                                // log10( 10 )
constexpr double MKI_MATH_PHI      = 1.61803398874989484820458683436564; // phi

constexpr double MKI_MATH_DEG_RAD = 0.017453292519943295769236907684886; // pi / 180
constexpr double MKI_MATH_GRA_RAD = 0.015707963267948966192313216916398; // pi / 200
constexpr double MKI_MATH_REL_RAD = 6.2831853071795864769252867665590;   // 2.0 * pi
constexpr double MKI_MATH_RAD_DEG = 57.295779513082320876798154814105;   // 180 / pi
constexpr double MKI_MATH_RAD_GRA = 63.661977236758134307553505349006;   // 200 / pi
constexpr double MKI_MATH_RAD_REL = 0.15915494309189533576888376337251;  // 0.5 / pi

constexpr double MKI_MATH_INCH_MM = 25.4;                               // Inch -> mm
constexpr double MKI_MATH_INCH_M  = 0.0254;                             // Inch -> m
constexpr double MKI_MATH_MM_INCH = 0.03937007874015748031496062992126; // mm -> Inch
constexpr double MKI_MATH_M_INCH  = 39.37007874015748031496062992126;   // m -> Inch

constexpr double MKI_MATH_MICROMETER = 0.001;                                 // Micrometer (1/1000mm)
constexpr double MKI_MATH_MILLIMETER = 1.0;                                   // Millimeter
constexpr double MKI_MATH_CENTIMETER = 10.0;                                  // Centimeter (10mm)
constexpr double MKI_MATH_DECIMETER  = 100.0;                                 // Decimeter (100mm)
constexpr double MKI_MATH_METER      = 1000.0;                                // Meter (1000mm)
constexpr double MKI_MATH_KILOMETER  = 1000000.0;                             // Kilometer (1000m)
constexpr double MKI_MATH_MIL        = 0.0254;                                // Mil (1/1000")
constexpr double MKI_MATH_INCH       = 25.4;                                  // Zoll
constexpr double MKI_MATH_FOOT       = 304.8;                                 // Foot (12")
constexpr double MKI_MATH_YARD       = 914.4;                                 // Yard (3ft)
constexpr double MKI_MATH_MILE       = 1609344.0;                             // Mile (1760yd)
constexpr double MKI_MATH_DECIPOINT  = 0.035277777777777777777777777777778;   // Decipoint (1/720")
constexpr double MKI_MATH_POINT      = 0.35277777777777777777777777777778;    // Point (1/72")
constexpr double MKI_MATH_DIDOT      = 0.37593984962406015037593984962406;    // Didot (1/2660m)
constexpr double MKI_MATH_CICERO     = 4.5112781954887218045112781954887;     // Cicero (12/2660m)
constexpr double MKI_MATH_MINUTE     = 0.0002908882086657215961539484614148;  // Minute (1/60)
constexpr double MKI_MATH_SECOND     = 0.00000484813681109535993589914102358; // Second (1/3600)

constexpr double MKI_MATH_ROOT_1_6     = 0.40824829046386301636621401245098; // sqrt( 1/6 )
constexpr double MKI_MATH_ROOT_1_2     = 0.70710678118654752440084436210485; // sqrt( 1/2 )
constexpr double MKI_MATH_ROOT_2_3     = 0.81649658092772603273242802490196; // sqrt( 2/3 )
constexpr double MKI_MATH_ROOT_3_2     = 1.2247448713915890490986420373529;  // sqrt( 3/2 )
constexpr double MKI_MATH_ROOT_2       = 1.4142135623730950488016887242097;  // sqrt( 2.0 )
constexpr double MKI_MATH_ROOT_3       = 1.7320508075688772935274463415059;  // sqrt( 3.0 )
constexpr double MKI_MATH_ROOT_ROOT_2  = 1.1892071150027210667174999705605;  // sqrt( sqrt( 2.0 ) )
constexpr double MKI_MATH_ARC90_BEZIER = 0.55228474983079339840225163227960; // 4/3 * ( sqrt( 2 ) - 1 )
constexpr double MKI_MATH_ARC45_BEZIER = 0.26521648983954400921546349685957; // 4/3 * ( 1 - cos( 22.5 ) ) / sin( 22.5 )

constexpr double MKI_MATH_CODATA_C  = 299792458.0;      // CODATA 2018: Speed of light in vacuum c [m/s]
constexpr double MKI_MATH_CODATA_e  = 1.602176634e-19;  // CODATA 2018: Elemtary charge e [C]
constexpr double MKI_MATH_CODATA_G  = 6.67430e-11;      // CODATA 2018: Newtonian constant of gravitation G [m/kg/s]
constexpr double MKI_MATH_CODATA_h  = 6.62607015e-34;   // CODATA 2018: Planck constant h [Js]
constexpr double MKI_MATH_CODATA_me = 9.1093837015e-31; // CODATA 2018: Electron mass me [kg]

constexpr double MKI_DOUBLE_NOVAL =  1.0e+300; // Invalid double value
constexpr double MKI_DOUBLE_MIN   = -1.0e+100; // Minimum double value
constexpr double MKI_DOUBLE_MAX   =  1.0e+100; // Maximum double value

constexpr __int16 MKI_INT16_NOVAL = -32767; // Invalid __int16 value
constexpr __int32 MKI_INT16_MIN   = -32768; // Minimum __int16 value
constexpr __int32 MKI_INT16_MAX   =  32767; // Maximum __int16 value

constexpr __int32 MKI_INT32_NOVAL = -2147483647;    // Invalid __int32 value
constexpr __int64 MKI_INT32_MIN   = -2147483648i64; // Minimum __int32 value
constexpr __int64 MKI_INT32_MAX   =  2147483647i64; // Maximum __int32 value

constexpr __int64 MKI_INT64_NOVAL = -9223372036854775807i64; // Invalid __int64 value
constexpr double  MKI_INT64_MIN   = -9223372036854775806.49; // Minimum __int64 value
constexpr double  MKI_INT64_MAX   =  9223372036854775807.49; // Maximum __int64 value


//---------------------------------------------------------------------------------------------------------------------------------------------------

constexpr LPSTR MKI_INI_65_HEADER   = "Malz++Kassner INI 6.50";
constexpr LPSTR MKI_INI_70_HEADER   = "Malz++Kassner INI 7.00";
constexpr LPSTR MKI_MKD_460_HEADER  = "Malz++Kassner CAD 4.60";
constexpr LPSTR MKI_MKD_500_HEADER  = "Malz++Kassner CAD 5.00";
constexpr LPSTR MKI_MKD_600_HEADER  = "Malz++Kassner CAD 6.00";
constexpr LPSTR MKI_MKD_610_HEADER  = "Malz++Kassner CAD 6.10";
constexpr LPSTR MKI_MKD_620_HEADER  = "Malz++Kassner CAD 6.20";
constexpr LPSTR MKI_MKD_630_HEADER  = "Malz++Kassner CAD 6.30";
constexpr LPSTR MKI_MKD_640_HEADER  = "Malz++Kassner CAD 6.40";
constexpr LPSTR MKI_MKD_650_HEADER  = "Malz++Kassner CAD 6.50";
constexpr LPSTR MKI_MKD_660_HEADER  = "Malz++Kassner CAD 6.60";
constexpr LPSTR MKI_MKD_661_HEADER  = "Malz++Kassner CAD 6.61";
constexpr LPSTR MKI_MKD_6100_HEADER = "Malz++Kassner CAD6.100";
constexpr LPSTR MKI_MKD_6110_HEADER = "Malz++Kassner CAD 6110";
constexpr LPSTR MKI_MKD_6120_HEADER = "Malz++Kassner CAD 6120";
constexpr LPSTR MKI_MKD_6121_HEADER = "Malz++Kassner CAD 6121";
constexpr LPSTR MKI_MKD_6130_HEADER = "Malz++Kassner CAD 6130";
constexpr LPSTR MKI_MKD_6132_HEADER = "Malz++Kassner CAD 6132";
constexpr LPSTR MKI_MKD_6140_HEADER = "Malz++Kassner CAD 6140";
constexpr LPSTR MKI_MKD_6142_HEADER = "Malz++Kassner CAD 6142";
constexpr LPSTR MKI_MKD_6150_HEADER = "Malz++Kassner CAD 6150";
constexpr LPSTR MKI_MKD_6160_HEADER = "Malz++Kassner CAD 6160";
constexpr LPSTR MKI_MKD_6161_HEADER = "Malz++Kassner CAD 6161";
constexpr LPSTR MKI_MKD_6170_HEADER = "Malz++Kassner CAD 6170";
constexpr LPSTR MKI_MKD_6171_HEADER = "Malz++Kassner CAD 6171";
constexpr LPSTR MKI_MKD_6180_HEADER = "Malz++Kassner CAD 6180";
constexpr LPSTR MKI_MKD_6181_HEADER = "Malz++Kassner CAD 6181";
constexpr LPSTR MKI_MKD_6182_HEADER = "Malz++Kassner CAD 6182";
constexpr LPSTR MKI_MKD_6190_HEADER = "Malz++Kassner CAD 6190";
constexpr LPSTR MKI_MKD_6191_HEADER = "Malz++Kassner CAD 6191";
constexpr LPSTR MKI_MKD_6200_HEADER = "Malz++Kassner CAD 6200";
constexpr LPSTR MKI_MKD_6201_HEADER = "Malz++Kassner CAD 6201";
constexpr LPSTR MKI_MKD_6210_HEADER = "Malz++Kassner CAD 6210";
constexpr LPSTR MKI_MKD_6211_HEADER = "Malz++Kassner CAD 6211";
constexpr LPSTR MKI_MKD_6212_HEADER = "Malz++Kassner CAD 6212";
constexpr LPSTR MKI_MKD_6220_HEADER = "Malz++Kassner CAD 6220";
constexpr LPSTR MKI_MKD_6222_HEADER = "Malz++Kassner CAD 6222";
constexpr LPSTR MKI_MKD_6230_HEADER = "Malz++Kassner CAD 6230";
constexpr LPSTR MKI_MKD_6231_HEADER = "Malz++Kassner CAD 6231";
constexpr LPSTR MKI_MKD_6232_HEADER = "Malz++Kassner CAD 6232";
constexpr LPSTR MKI_MKD_6240_HEADER = "Malz++Kassner CAD 6240";
constexpr LPSTR MKI_MKD_6242_HEADER = "Malz++Kassner CAD 6242";
constexpr LPSTR MKI_MKD_6250_HEADER = "Malz++Kassner CAD 6250";

constexpr LPSTR MKI_UTF8_BOM_TEXT  = "﻿"; // Identification of UTF-8 files (first 3 chars of the file)
constexpr char  MKI_UTF8_BOM_CHAR1 = '';   // Identification of UTF-8 files (char 1 of the file)
constexpr char  MKI_UTF8_BOM_CHAR2 = '';   // Identification of UTF-8 files (char 2 of the file)
constexpr char  MKI_UTF8_BOM_CHAR3 = '';   // Identification of UTF-8 files (char 3 of the file)

constexpr LPWSTR MKI_BLOCK_ID = L"*";       // Library name of internal block list

constexpr __int32 MKI_BLOCK_INDEX           = 999;      // Library index of internal block List
constexpr __int32 MKI_BLOCKS_PER_DRAWING    = 1000000;  // Maximum number of blocks per drawing / library
constexpr __int32 MKI_BLOCKS_PER_FONT       = 224;      // Maximum number of characters per font (Ansi 32..255)

constexpr __int32 MKI_NESTING_DEPTH_MAX     = 25;       // Maximum nesting depth of entities

constexpr __int32 MKI_POINTS_PER_PATCH      = 4;        // Maximum number of points per patch
constexpr __int32 MKI_ATTRIBS_PER_OBJECT    = 200;      // Maximum number of attributes per block / instance / object / outline / point
constexpr __int32 MKI_ATTRIBS_PER_PARTSLIST = 50;       // Maximum number of attributes per parts list
constexpr __int32 MKI_BYTES_PER_BINARY      = 144;      // Recommended maximum number of bytes per binary block

constexpr __int32 MKI_LENGTH_PATH           = 260;      // Maximum length of a filename with path
constexpr __int32 MKI_LENGTH_TEXT_SHORT     = 500;      // Maximum length of "short" texts
constexpr __int32 MKI_LENGTH_TEXT_LONG      = 30000;    // Maximum length of "long" texts
constexpr __int32 MKI_LENGTH_VARIABLE       = 500;      // Maximum length of variable texts
constexpr __int32 MKI_LENGTH_ATTRIBUTE      = 4000;     // Maximum length of attribute texts
constexpr __int32 MKI_LENGTH_BINARY         = 32700;    // Maximum length of binary data
constexpr __int32 MKI_LENGTH_FILE_ID        = 22;       // Maximum length of Malz++Kassner file identification (MKI_MKD_??_HEADER)
constexpr __int32 MKI_LENGTH_FILE_EXIT      = 64;       // Maximum length of Malz++Kassner file termination (=EXIT= plus CRC)

constexpr __int32 MKI_CUSTOMMENU_MAX        = 100;      // Maximum number of custom menus in toolbox and panel
constexpr __int32 MKI_CUSTOMMENU_SIZE       = 30;       // Number of entries per custom menu
constexpr __int32 MKI_TOOLBOX_COLUMNS       = 20;       // Maximum number of button columns in toolbox
constexpr __int32 MKI_TOOLBOX_ROWS          = 50;       // Maximum number of button rows in toolbox
constexpr __int32 MKI_TOOLBOX_MAX           = MKI_TOOLBOX_COLUMNS * MKI_TOOLBOX_ROWS;
constexpr __int32 MKI_PANEL_COLUMNS         = 2;        // Maximum number of toolbox button columns in panel
constexpr __int32 MKI_PANEL_ROWS            = 50;       // Maximum number of toolbox button rows in panel
constexpr __int32 MKI_PANEL_MAX             = MKI_PANEL_COLUMNS * MKI_PANEL_ROWS;
constexpr __int32 MKI_PARAMETER_MAX         = 30;       // Maximum number of buttons in parameter window
constexpr __int32 MKI_COMMANDBAR_MAX        = 50;       // Maximum number of command bar sections
constexpr __int32 MKI_BLOCKLIST_COLUMNS     = 10;       // Maximum number of button columns in block window
constexpr __int32 MKI_BLOCKLIST_ROWS        = 50;       // Maximum number of button rows in block window
constexpr __int32 MKI_BLOCKLIST_MAX         = MKI_BLOCKLIST_COLUMNS * MKI_BLOCKLIST_ROWS;
constexpr __int32 MKI_CUSTOMKEY_MAX         = 1000;     // Number of keys
constexpr __int32 MKI_VARIABLE_MAX          = 1000;     // Number of global variables
constexpr __int32 MKI_DEFAULT_MAX           = 16;       // Number of defaults (identical for pens & layers)
constexpr __int32 MKI_SYSTEM_MAX            = 100;      // Number of coordinate systems (plus STANDARD)
constexpr __int32 MKI_HATCHLINE_MAX         = 8;        // Number of lines per hatch line sequence
constexpr __int32 MKI_ORTHO_MAX             = 8;        // Number of ortho mode angles
constexpr __int32 MKI_PAGE_MAX              = 1000;     // Number of page definitions
constexpr __int32 MKI_PAGE_MODEL            = -1;       // Page index of Model Space
constexpr __int32 MKI_PAGE_NOVAL            = 9999;     // Invalid / undefined page number
constexpr __int32 MKI_HATCH_MAX             = 1000;     // Number of hatching types (plus STANDARD)
constexpr __int32 MKI_LINE_MAX              = 1000;     // Number of line patterns (plus STANDARD)
constexpr __int32 MKI_LINE_BYLAYER          = 9999;     // Index of "by layer" line pattern
constexpr __int32 MKI_LINEBLOCK_MAX         = 1000;     // Number of block line patterns
constexpr __int32 MKI_LINEPART_MAX          = 16;       // Number of line parts per line pattern
constexpr __int32 MKI_LINEBLOCKPART_MAX     = 8;        // Number of line parts per block line pattern
constexpr __int32 MKI_PEN_MAX               = 1000;     // Number of pens (plus STANDARD)
constexpr __int32 MKI_LAYER_MAX             = 2000;     // Number of layers (plus STANDARD)
constexpr __int32 MKI_INTERN_FILLMODE_MAX   = 5;        // Number of fill mode buttons
constexpr __int32 MKI_INTERN_LINEWIDTH_MAX  = 6;        // Number of line width buttons
constexpr __int32 MKI_INTERN_COLOR_MAX      = 10;       // Number of line/fill color buttons
constexpr __int32 MKI_HARMONY_COLORS        = 7;        // Number of colors in a color harmony basecolor
constexpr __int32 MKI_MIXER_COLORS          = 3;        // Number of colors in the color mixer
constexpr __int32 MKI_CUSTOMCOLOR_MAX       = 1000;     // Number of custom colors
constexpr __int32 MKI_TASK_MAX              = 1000;     // Number of tasks
constexpr __int32 MKI_WINDOW_MAX            = 4;        // Maximum number of concurrent drawing windows
constexpr __int32 MKI_WINDOW_VIEW           = 999;      // Window index of view window
constexpr __int32 MKI_RECENT_DRAWING_MAX    = 100;      // Maximum number of entries in the recent drawings list
constexpr __int32 MKI_RECENT_BLOCK_MAX      = 100;      // Maximum number of entries in the recent blocks list
constexpr __int32 MKI_RECENT_FONT_MAX       = 10;       // Maximum number of entries in the recent fonts list
constexpr __int32 MKI_FONT_MAX              = 20;       // Maximum number of fonts opened
constexpr __int32 MKI_COLLECTION_MAX        = 50;       // Maximum number of collections opened
constexpr __int32 MKI_ITEMS_PER_COLLECTION  = 100000;   // Maximum number of items per collection
constexpr __int32 MKI_LIBRARY_MAX           = 200;      // Maximum number of libraries opened
constexpr __int32 MKI_DATABASE_MAX          = 100;      // Maximum number of databases opened
constexpr __int32 MKI_DATABASE_INTERNAL_MAX = 10;       // Maximum number of internal databases per drawing
constexpr __int32 MKI_DATABASE_LINE_MAX     = 64000;    // Maximum length of one database line
constexpr __int32 MKI_DATABASE_COLUMN_MAX   = 500;      // Maximum number of columns in a database line
constexpr __int32 MKI_SERIAL_INDEX_MAX      = 10000;    // Maximum number of database entries for serial printing
constexpr __int32 MKI_SERIAL_FILTER_MAX     = 4;        // Maximum number of filtering criteria for serial printing
constexpr __int32 MKI_SERIAL_SORT_MAX       = 4;        // Maximum number of sorting criteria for serial printing
constexpr __int32 MKI_SERIAL_RULE_MAX       = 4;        // Maximum number of rules for serial printing
constexpr __int32 MKI_PRINTER_MAX           = 8;        // Maximum number of printer settings
constexpr __int32 MKI_PRINTER_POLYGON_MAX   = 100;      // Maximum number of printing area polygon points
constexpr __int32 MKI_BITMAP_MAX            = 1000;     // Maximum number of bitmaps opened
constexpr __int32 MKI_PROCESSOR_MAX         = 16;       // Maximum number of concurrently used processor cores
constexpr __int32 MKI_EXTERNAL_MAX          = 50;       // Maximum number of external drawing references
constexpr __int32 MKI_HYPERLINK_MAX         = 1000;     // Maximum number of hyperlinks
constexpr __int32 MKI_COMMENT_MAX           = 1000;     // Maximum number of comments
constexpr __int32 MKI_UNDO_MAX              = 1000;     // Maximum number of undo steps
constexpr __int32 MKI_POSNO_RANGE_MAX       = 10;       // Number of position number ranges
constexpr __int32 MKI_POSNO_MAX             = 10000;    // Maximum number of position numbers
constexpr __int32 MKI_SUBCLASS_MAX          = 1000;     // Maximum number of subclassed controls in a dialog
constexpr __int32 MKI_TEXTSTYLE_TOTAL       = 9;        // Number of text style definitions (<h1> .. <h9>)
constexpr __int32 MKI_DEFPATH_MAX           = 20;       // Number of standard paths per file type
constexpr __int32 MKI_FILTER_MAX            = 20;       // Maximum number of loaded filters
constexpr __int32 MKI_PLUGIN_MAX            = 50;       // Maximum number of loaded plug-ins
constexpr __int32 MKI_PLUGIN_MENU_MAX       = 200;      // Maximum number of items in a plug-in's menu
constexpr __int32 MKI_PLUGIN_SUBMENU_MAX    = 30;       // Maximum number of items in a plug-in's submenu
constexpr __int32 MKI_PLUGIN_POPUP_MAX      = 5000;     // Maximum number of items in a plug-in's popup menu
constexpr __int32 MKI_PLUGIN_EXT_MAX        = 100;      // Maximum number of extended object types at a time
constexpr __int32 MKI_PLUGIN_CONTEXT_MAX    = 10;       // Maximum number of context menu commands at a time
constexpr __int32 MKI_PLUGIN_BUTTON_MAX     = 50;       // Maximum number of plug-in window buttons
constexpr __int32 MKI_PLUGIN_TOTAL_MAX      = MKI_PLUGIN_MAX + 2 * MKI_FILTER_MAX;
constexpr __int32 MKI_INPUT_MAX             = 2000;     // Maximum number of points to be entered
constexpr __int32 MKI_INPUT_DEF_MAX         = 8;        // Maximum number of predefined point entries
constexpr __int32 MKI_INPUT_POLYGON_MAX     = 500;      // Maximum number of points in an identification polygon
constexpr __int32 MKI_POLYPOLY_MAX          = 30000;    // Maximum number of nested sub-polygons
constexpr __int32 MKI_DB_SIZE_MAX           = 32000;    // Maximum size of a data block in bytes
constexpr __int32 MKI_DRAWING_PREVIEW_SIZE  = 250;      // Size in pixels of a drawing preview
constexpr __int32 MKI_UNIQUE_ID_SIZE        = 16;       // Size of unique ID structure in bytes


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_DIMLINEPARAM_LINE  = 0,
  MKI_DIMLINEPARAM_ARC   = 1,
  MKI_DIMLINEPARAM_EARC  = 2,

  MKI_DIMLINEPARAM_TOTAL = 3, // Number of dimension parameter sets MKI_DIMLINE
};

enum
{
  MKI_DIMLARGEPARAM_LENGTH_P      = 0,
  MKI_DIMLARGEPARAM_LENGTH_OBJ    = 1,
  MKI_DIMLARGEPARAM_DISTANCE_P    = 2,
  MKI_DIMLARGEPARAM_DISTANCE_OBJ  = 3,
  MKI_DIMLARGEPARAM_RADIUS_P      = 4,
  MKI_DIMLARGEPARAM_RADIUS_OBJ    = 5,
  MKI_DIMLARGEPARAM_DIAMETER_P    = 6,
  MKI_DIMLARGEPARAM_DIAMETER_OBJ  = 7,
  MKI_DIMLARGEPARAM_ANGLE_P       = 8,
  MKI_DIMLARGEPARAM_ANGLE_LL      = 9,
  MKI_DIMLARGEPARAM_ANGLE_OBJ     = 10,
  MKI_DIMLARGEPARAM_ARCLENGTH_P   = 11,
  MKI_DIMLARGEPARAM_ARCLENGTH_OBJ = 12,

  MKI_DIMLARGEPARAM_TOTAL         = 13, // Number of dimension parameter sets MKI_DIMLARGE
};

enum
{
  MKI_DIMSMALLPARAM_COORDINATE = 0,
  MKI_DIMSMALLPARAM_AREA       = 1,
  MKI_DIMSMALLPARAM_PERIMETER  = 2,
  MKI_DIMSMALLPARAM_TEXT       = 3,

  MKI_DIMSMALLPARAM_TOTAL      = 4, // Number of dimension parameter sets MKI_DIMSMALL
};

//---------------------------------------------------------------------------------------------------------------------------------------------------
constexpr __int32 MKI_SYSTEMCOLOR_RGB_MIN = 2000;
constexpr __int32 MKI_SYSTEMCOLOR_RGB_MAX = MKI_SYSTEMCOLOR_RGB_MIN + 40;

constexpr __int32 MKI_SYSTEMCOLOR_CMYK_MIN = 2100;
constexpr __int32 MKI_SYSTEMCOLOR_CMYK_MAX = MKI_SYSTEMCOLOR_CMYK_MIN + 40;

constexpr __int32 MKI_SYSTEMCOLOR_GRAY_MIN = 2200;
constexpr __int32 MKI_SYSTEMCOLOR_GRAY_MAX = MKI_SYSTEMCOLOR_GRAY_MIN + 40;

constexpr __int32 MKI_SYSTEMCOLOR_LAB_MIN = 2300;
constexpr __int32 MKI_SYSTEMCOLOR_LAB_MAX = MKI_SYSTEMCOLOR_LAB_MIN + 40;

constexpr __int32 MKI_STANDARDCOLOR_N_MIN = 2400;
constexpr __int32 MKI_STANDARDCOLOR_N_MAX = MKI_STANDARDCOLOR_N_MIN + 86;

constexpr __int32 MKI_STANDARDCOLOR_K_MIN = 2500;
constexpr __int32 MKI_STANDARDCOLOR_K_MAX = MKI_STANDARDCOLOR_K_MIN + 86;

constexpr __int32 MKI_STANDARDCOLOR_R_MIN = 2600;
constexpr __int32 MKI_STANDARDCOLOR_R_MAX = MKI_STANDARDCOLOR_R_MIN + 195;

constexpr __int32 MKI_STANDARDCOLOR_P_MIN = 2800;
constexpr __int32 MKI_STANDARDCOLOR_P_MAX = MKI_STANDARDCOLOR_P_MIN + 658;

#define MKI_COLOR_RGB_WHITE       RGB( 255, 255, 255 )  // RGB for White        #0
#define MKI_COLOR_RGB_GRAY5       RGB( 242, 242, 242 )  // RGB for Gray      5%
#define MKI_COLOR_RGB_GRAY12      RGB( 224, 224, 224 )  // RGB for Gray     12,5%
#define MKI_COLOR_RGB_GRAY25      RGB( 192, 192, 192 )  // RGB for Gray     25%
#define MKI_COLOR_RGB_GRAY50      RGB( 128, 128, 128 )  // RGB for Gray     50%
#define MKI_COLOR_RGB_GRAY62      RGB(  96,  96,  96 )  // RGB for Gray     62,5%
#define MKI_COLOR_RGB_GRAY75      RGB(  64,  64,  64 )  // RGB for Gray     75%
#define MKI_COLOR_RGB_BLACK       RGB(   0,   0,   0 )  // RGB for Black
#define MKI_COLOR_RGB_RED100      RGB( 255,   0,   0 )  // RGB for Red     100% #8
#define MKI_COLOR_RGB_RED75       RGB( 192,   0,   0 )  // RGB for Red      75%
#define MKI_COLOR_RGB_RED50       RGB( 128,   0,   0 )  // RGB for Red      50%
#define MKI_COLOR_RGB_RED25       RGB(  64,   0,   0 )  // RGB for Red      25%
#define MKI_COLOR_RGB_YELLOW100   RGB( 255, 255,   0 )  // RGB for Yellow  100%
#define MKI_COLOR_RGB_YELLOW75    RGB( 192, 192,   0 )  // RGB for Yellow   75%
#define MKI_COLOR_RGB_YELLOW50    RGB( 128, 128,   0 )  // RGB for Yellow   50%
#define MKI_COLOR_RGB_YELLOW25    RGB(  64,  64,   0 )  // RGB for Yellow   25%
#define MKI_COLOR_RGB_GREEN100    RGB(   0, 255,   0 )  // RGB for Green   100% #16
#define MKI_COLOR_RGB_GREEN75     RGB(   0, 192,   0 )  // RGB for Green    75%
#define MKI_COLOR_RGB_GREEN50     RGB(   0, 128,   0 )  // RGB for Green    50%
#define MKI_COLOR_RGB_GREEN25     RGB(   0,  64,   0 )  // RGB for Green    25%
#define MKI_COLOR_RGB_CYAN100     RGB(   0, 255, 255 )  // RGB for Cyan    100%
#define MKI_COLOR_RGB_CYAN75      RGB(   0, 192, 192 )  // RGB for Cyan     75%
#define MKI_COLOR_RGB_CYAN50      RGB(   0, 128, 128 )  // RGB for Cyan     50%
#define MKI_COLOR_RGB_CYAN25      RGB(   0,  64,  64 )  // RGB for Cyan     25%
#define MKI_COLOR_RGB_BLUE100     RGB(   0,   0, 255 )  // RGB for Blue    100% #24
#define MKI_COLOR_RGB_BLUE75      RGB(   0,   0, 192 )  // RGB for Blue     75%
#define MKI_COLOR_RGB_BLUE50      RGB(   0,   0, 128 )  // RGB for Blue     50%
#define MKI_COLOR_RGB_BLUE25      RGB(   0,   0,  64 )  // RGB for Blue     25%
#define MKI_COLOR_RGB_MAGENTA100  RGB( 255,   0, 255 )  // RGB for Magenta 100%
#define MKI_COLOR_RGB_MAGENTA75   RGB( 192,   0, 192 )  // RGB for Magenta  75%
#define MKI_COLOR_RGB_MAGENTA50   RGB( 128,   0, 128 )  // RGB for Magenta  50%
#define MKI_COLOR_RGB_MAGENTA25   RGB(  64,   0,  64 )  // RGB for Magenta  25%
#define MKI_COLOR_RGB_BROWN       RGB( 153,  51,   0 )  // RGB for Brown        #32
#define MKI_COLOR_RGB_NEONRED     RGB( 229,   0, 102 )  // RGB for Neon Red
#define MKI_COLOR_RGB_ORANGE      RGB( 255, 153,   0 )  // RGB for Orange
#define MKI_COLOR_RGB_SAND        RGB( 255, 217, 166 )  // RGB for Sand
#define MKI_COLOR_RGB_YELLOWGREEN RGB( 179, 204,   0 )  // RGB for Yellow Green
#define MKI_COLOR_RGB_TURQUOISE   RGB(   0, 179, 179 )  // RGB for Turquoise
#define MKI_COLOR_RGB_JEANSBLUE   RGB(   0, 127, 191 )  // RGB for Jeans Blue
#define MKI_COLOR_RGB_PURPLE      RGB( 179,   0, 127 )  // RGB for Purple       #39

#define MKI_COLOR_RGB_FADEDPINK   RGB( 255, 204, 204 )  // RGB for Faded Pink
#define MKI_COLOR_RGB_WALNUT      RGB( 102,  51,   0 )  // RGB for Walnut
#define MKI_COLOR_RGB_DEEPYELLOW  RGB( 255, 204,   0 )  // RGB for Deep Yellow
#define MKI_COLOR_RGB_LTYELLOW    RGB( 255, 255, 102 )  // RGB for Light Yellow
#define MKI_COLOR_RGB_LTGREEN     RGB(  51, 204, 102 )  // RGB for Light Green
#define MKI_COLOR_RGB_SPRINGGREEN RGB(  51, 204,  51 )  // RGB for Spring Green
#define MKI_COLOR_RGB_BABYBLUE    RGB( 102, 153, 255 )  // RGB for Baby Blue
#define MKI_COLOR_RGB_DEEPBLUE    RGB(  51,   0, 204 )  // RGB for Deep Blue
#define MKI_COLOR_RGB_VIOLET      RGB( 204, 102, 204 )  // RGB for Violet
#define MKI_COLOR_RGB_PINK        RGB( 255, 153, 204 )  // RGB for Pink
#define MKI_COLOR_RGB_HOTPINK     RGB( 255,  51, 153 )  // RGB for Hot Pink
#define MKI_COLOR_RGB_LTORANGE    RGB( 255, 153,  51 )  // RGB for Light Orange
#define MKI_COLOR_RGB_TRANSPARENT RGB( 242, 240, 230 )  // RGB for Transparent

#define MKI_COLOR_CMYK_WHITE       mki_color_make_cmyk(   0,   0,   0,   0 )  // CMYK for White        #0
#define MKI_COLOR_CMYK_GRAY5       mki_color_make_cmyk(   0,   0,   0,  13 )  // CMYK for Gray      5%
#define MKI_COLOR_CMYK_GRAY12      mki_color_make_cmyk(   0,   0,   0,  32 )  // CMYK for Gray     12,5%
#define MKI_COLOR_CMYK_GRAY25      mki_color_make_cmyk(   0,   0,   0,  64 )  // CMYK for Gray     25%
#define MKI_COLOR_CMYK_GRAY50      mki_color_make_cmyk(   0,   0,   0, 127 )  // CMYK for Gray     50%
#define MKI_COLOR_CMYK_GRAY62      mki_color_make_cmyk(   0,   0,   0, 159 )  // CMYK for Gray     62,5%
#define MKI_COLOR_CMYK_GRAY75      mki_color_make_cmyk(   0,   0,   0, 191 )  // CMYK for Gray     75%
#define MKI_COLOR_CMYK_BLACK       mki_color_make_cmyk(   0,   0,   0, 255 )  // CMYK for Black
#define MKI_COLOR_CMYK_RED100      mki_color_make_cmyk(   0, 255, 255,   0 )  // CMYK for Red     100% #8
#define MKI_COLOR_CMYK_RED75       mki_color_make_cmyk(   0, 255, 255,  64 )  // CMYK for Red      75%
#define MKI_COLOR_CMYK_RED50       mki_color_make_cmyk(   0, 255, 255, 127 )  // CMYK for Red      50%
#define MKI_COLOR_CMYK_RED25       mki_color_make_cmyk(   0, 255, 255, 191 )  // CMYK for Red      25%
#define MKI_COLOR_CMYK_YELLOW100   mki_color_make_cmyk(   0,   0, 255,   0 )  // CMYK for Yellow  100%
#define MKI_COLOR_CMYK_YELLOW75    mki_color_make_cmyk(   0,   0, 255,  64 )  // CMYK for Yellow   75%
#define MKI_COLOR_CMYK_YELLOW50    mki_color_make_cmyk(   0,   0, 255, 127 )  // CMYK for Yellow   50%
#define MKI_COLOR_CMYK_YELLOW25    mki_color_make_cmyk(   0,   0, 255, 191 )  // CMYK for Yellow   25%
#define MKI_COLOR_CMYK_GREEN100    mki_color_make_cmyk( 255,   0, 255,   0 )  // CMYK for Green   100% #16
#define MKI_COLOR_CMYK_GREEN75     mki_color_make_cmyk( 255,   0, 255,  64 )  // CMYK for Green    75%
#define MKI_COLOR_CMYK_GREEN50     mki_color_make_cmyk( 255,   0, 255, 127 )  // CMYK for Green    50%
#define MKI_COLOR_CMYK_GREEN25     mki_color_make_cmyk( 255,   0, 255, 191 )  // CMYK for Green    25%
#define MKI_COLOR_CMYK_CYAN100     mki_color_make_cmyk( 255,   0,   0,   0 )  // CMYK for Cyan    100%
#define MKI_COLOR_CMYK_CYAN75      mki_color_make_cmyk( 255,   0,   0,  64 )  // CMYK for Cyan     75%
#define MKI_COLOR_CMYK_CYAN50      mki_color_make_cmyk( 255,   0,   0, 127 )  // CMYK for Cyan     50%
#define MKI_COLOR_CMYK_CYAN25      mki_color_make_cmyk( 255,   0,   0, 191 )  // CMYK for Cyan     25%
#define MKI_COLOR_CMYK_BLUE100     mki_color_make_cmyk( 255, 255,   0,   0 )  // CMYK for Blue    100% #24
#define MKI_COLOR_CMYK_BLUE75      mki_color_make_cmyk( 255, 255,   0,  64 )  // CMYK for Blue     75%
#define MKI_COLOR_CMYK_BLUE50      mki_color_make_cmyk( 255, 255,   0, 127 )  // CMYK for Blue     50%
#define MKI_COLOR_CMYK_BLUE25      mki_color_make_cmyk( 255, 255,   0, 191 )  // CMYK for Blue     25%
#define MKI_COLOR_CMYK_MAGENTA100  mki_color_make_cmyk(   0, 255,   0,   0 )  // CMYK for Magenta 100%
#define MKI_COLOR_CMYK_MAGENTA75   mki_color_make_cmyk(   0, 255,   0,  64 )  // CMYK for Magenta  75%
#define MKI_COLOR_CMYK_MAGENTA50   mki_color_make_cmyk(   0, 255,   0, 127 )  // CMYK for Magenta  50%
#define MKI_COLOR_CMYK_MAGENTA25   mki_color_make_cmyk(   0, 255,   0, 191 )  // CMYK for Magenta  25%
#define MKI_COLOR_CMYK_BROWN       mki_color_make_cmyk(   0, 204, 255, 102 )  // CMYK for Brown        #32
#define MKI_COLOR_CMYK_NEONRED     mki_color_make_cmyk(   0, 255, 102,   0 )  // CMYK for Neon Red
#define MKI_COLOR_CMYK_ORANGE      mki_color_make_cmyk(   0, 127, 255,   0 )  // CMYK for Orange
#define MKI_COLOR_CMYK_SAND        mki_color_make_cmyk(   0,  52, 102,   0 )  // CMYK for Sand
#define MKI_COLOR_CMYK_YELLOWGREEN mki_color_make_cmyk( 102,   0, 255,   0 )  // CMYK for Yellow Green
#define MKI_COLOR_CMYK_TURQUOISE   mki_color_make_cmyk( 204,   0,  51,   0 )  // CMYK for Turquoise
#define MKI_COLOR_CMYK_JEANSBLUE   mki_color_make_cmyk( 255, 102,   0,   0 )  // CMYK for Jeans Blue
#define MKI_COLOR_CMYK_PURPLE      mki_color_make_cmyk( 102, 229,   0,   0 )  // CMYK for Purple       #39

#define MKI_COLOR_GRAY_WHITE  mki_color_make_gray( 255 )  // GRAY for White     #0
#define MKI_COLOR_GRAY_99     mki_color_make_gray( 252 )  // GRAY for Gray  99%
#define MKI_COLOR_GRAY_98     mki_color_make_gray( 250 )  // GRAY for Gray  98%
#define MKI_COLOR_GRAY_97     mki_color_make_gray( 247 )  // GRAY for Gray  97%
#define MKI_COLOR_GRAY_96     mki_color_make_gray( 245 )  // GRAY for Gray  96%
#define MKI_COLOR_GRAY_95     mki_color_make_gray( 242 )  // GRAY for Gray  95%
#define MKI_COLOR_GRAY_94     mki_color_make_gray( 240 )  // GRAY for Gray  94%
#define MKI_COLOR_GRAY_93     mki_color_make_gray( 237 )  // GRAY for Gray  93%
#define MKI_COLOR_GRAY_92     mki_color_make_gray( 235 )  // GRAY for Gray  92% #8
#define MKI_COLOR_GRAY_91     mki_color_make_gray( 232 )  // GRAY for Gray  91%
#define MKI_COLOR_GRAY_90     mki_color_make_gray( 229 )  // GRAY for Gray  90%
#define MKI_COLOR_GRAY_89     mki_color_make_gray( 227 )  // GRAY for Gray  89%
#define MKI_COLOR_GRAY_88     mki_color_make_gray( 224 )  // GRAY for Gray  88%
#define MKI_COLOR_GRAY_87     mki_color_make_gray( 222 )  // GRAY for Gray  87%
#define MKI_COLOR_GRAY_86     mki_color_make_gray( 219 )  // GRAY for Gray  86%
#define MKI_COLOR_GRAY_85     mki_color_make_gray( 217 )  // GRAY for Gray  85%
#define MKI_COLOR_GRAY_82     mki_color_make_gray( 210 )  // GRAY for Gray  82% #16
#define MKI_COLOR_GRAY_80     mki_color_make_gray( 204 )  // GRAY for Gray  80%
#define MKI_COLOR_GRAY_77     mki_color_make_gray( 198 )  // GRAY for Gray  77%
#define MKI_COLOR_GRAY_75     mki_color_make_gray( 191 )  // GRAY for Gray  75%
#define MKI_COLOR_GRAY_72     mki_color_make_gray( 185 )  // GRAY for Gray  72%
#define MKI_COLOR_GRAY_70     mki_color_make_gray( 178 )  // GRAY for Gray  70%
#define MKI_COLOR_GRAY_67     mki_color_make_gray( 172 )  // GRAY for Gray  67%
#define MKI_COLOR_GRAY_65     mki_color_make_gray( 166 )  // GRAY for Gray  65%
#define MKI_COLOR_GRAY_62     mki_color_make_gray( 159 )  // GRAY for Gray  62% #24
#define MKI_COLOR_GRAY_60     mki_color_make_gray( 153 )  // GRAY for Gray  60%
#define MKI_COLOR_GRAY_57     mki_color_make_gray( 147 )  // GRAY for Gray  57%
#define MKI_COLOR_GRAY_55     mki_color_make_gray( 140 )  // GRAY for Gray  55%
#define MKI_COLOR_GRAY_52     mki_color_make_gray( 134 )  // GRAY for Gray  52%
#define MKI_COLOR_GRAY_50     mki_color_make_gray( 127 )  // GRAY for Gray  50%
#define MKI_COLOR_GRAY_45     mki_color_make_gray( 115 )  // GRAY for Gray  45%
#define MKI_COLOR_GRAY_40     mki_color_make_gray( 102 )  // GRAY for Gray  40% 
#define MKI_COLOR_GRAY_35     mki_color_make_gray(  89 )  // GRAY for Gray  35% #32
#define MKI_COLOR_GRAY_30     mki_color_make_gray(  76 )  // GRAY for Gray  30%
#define MKI_COLOR_GRAY_25     mki_color_make_gray(  64 )  // GRAY for Gray  25%
#define MKI_COLOR_GRAY_20     mki_color_make_gray(  51 )  // GRAY for Gray  20% 
#define MKI_COLOR_GRAY_15     mki_color_make_gray(  38 )  // GRAY for Gray  15%
#define MKI_COLOR_GRAY_10     mki_color_make_gray(  25 )  // GRAY for Gray  10%
#define MKI_COLOR_GRAY_5      mki_color_make_gray(  13 )  // GRAY for Gray   5%
#define MKI_COLOR_GRAY_BLACK  mki_color_make_gray(   0 )  // GRAY for Black     #39

#define MKI_COLOR_LAB_WHITE       mki_color_make_lab( 100.0,   0.0,   0.0 )  // LAB for White        #0
#define MKI_COLOR_LAB_GRAY5       mki_color_make_lab(  95.4,   0.0,   0.0 )  // LAB for Gray      5%
#define MKI_COLOR_LAB_GRAY12      mki_color_make_lab(  89.2,   0.0,   0.0 )  // LAB for Gray     12,5%
#define MKI_COLOR_LAB_GRAY25      mki_color_make_lab(  77.8,   0.0,   0.0 )  // LAB for Gray     25%
#define MKI_COLOR_LAB_GRAY50      mki_color_make_lab(  53.6,   0.0,   0.0 )  // LAB for Gray     50%
#define MKI_COLOR_LAB_GRAY62      mki_color_make_lab(  40.8,   0.0,   0.0 )  // LAB for Gray     62,5%
#define MKI_COLOR_LAB_GRAY75      mki_color_make_lab(  27.0,   0.0,   0.0 )  // LAB for Gray     75%
#define MKI_COLOR_LAB_BLACK       mki_color_make_lab(   0.0,   0.0,   0.0 )  // LAB for Black
#define MKI_COLOR_LAB_RED100      mki_color_make_lab(  53.2,  80.0,  67.2 )  // LAB for Red     100% #8
#define MKI_COLOR_LAB_RED75       mki_color_make_lab(  40.0,  64.6,  54.2 )  // LAB for Red      75%
#define MKI_COLOR_LAB_RED50       mki_color_make_lab(  25.6,  48.0,  38.0 )  // LAB for Red      50%
#define MKI_COLOR_LAB_RED25       mki_color_make_lab(   9.8,  29.8,  15.4 )  // LAB for Red      25%
#define MKI_COLOR_LAB_YELLOW100   mki_color_make_lab(  97.2, -21.6,  94.4 )  // LAB for Yellow  100%
#define MKI_COLOR_LAB_YELLOW75    mki_color_make_lab(  75.4, -17.4,  76.4 )  // LAB for Yellow   75%
#define MKI_COLOR_LAB_YELLOW50    mki_color_make_lab(  51.8, -13.0,  56.6 )  // LAB for Yellow   50%
#define MKI_COLOR_LAB_YELLOW25    mki_color_make_lab(  26.0,  -8.0,  34.8 )  // LAB for Yellow   25%
#define MKI_COLOR_LAB_GREEN100    mki_color_make_lab(  87.8, -86.2,  83.2 )  // LAB for Green   100% #16
#define MKI_COLOR_LAB_GREEN75     mki_color_make_lab(  67.8, -69.6,  67.2 )  // LAB for Green    75%
#define MKI_COLOR_LAB_GREEN50     mki_color_make_lab(  46.2, -51.6,  49.8 )  // LAB for Green    50%
#define MKI_COLOR_LAB_GREEN25     mki_color_make_lab(  22.6, -32.0,  30.2 )  // LAB for Green    25%
#define MKI_COLOR_LAB_CYAN100     mki_color_make_lab(  91.2, -48.0, -14.2 )  // LAB for Cyan    100%
#define MKI_COLOR_LAB_CYAN75      mki_color_make_lab(  70.6, -38.8, -11.4 )  // LAB for Cyan     75%
#define MKI_COLOR_LAB_CYAN50      mki_color_make_lab(  48.2, -28.8,  -8.4 )  // LAB for Cyan     50%
#define MKI_COLOR_LAB_CYAN25      mki_color_make_lab(  23.8, -17.8,  -5.2 )  // LAB for Cyan     25%
#define MKI_COLOR_LAB_BLUE100     mki_color_make_lab(  32.2,  79.2,-107.8 )  // LAB for Blue    100% #24
#define MKI_COLOR_LAB_BLUE75      mki_color_make_lab(  23.0,  64.0, -87.2 )  // LAB for Blue     75%
#define MKI_COLOR_LAB_BLUE50      mki_color_make_lab(  13.0,  47.6, -64.8 )  // LAB for Blue     50%
#define MKI_COLOR_LAB_BLUE25      mki_color_make_lab(   3.4,  23.4, -37.6 )  // LAB for Blue     25%
#define MKI_COLOR_LAB_MAGENTA100  mki_color_make_lab(  60.4,  98.2, -60.8 )  // LAB for Magenta 100%
#define MKI_COLOR_LAB_MAGENTA75   mki_color_make_lab(  45.6,  79.4, -49.2 )  // LAB for Magenta  75%
#define MKI_COLOR_LAB_MAGENTA50   mki_color_make_lab(  29.8,  59.0, -36.4 )  // LAB for Magenta  50%
#define MKI_COLOR_LAB_MAGENTA25   mki_color_make_lab(  12.4,  36.4, -22.6 )  // LAB for Magenta  25%
#define MKI_COLOR_LAB_BROWN       mki_color_make_lab(  36.2,  40.8,  48.0 )  // LAB for Brown        #32
#define MKI_COLOR_LAB_NEONRED     mki_color_make_lab(  49.0,  77.0,  10.8 )  // LAB for Neon Red
#define MKI_COLOR_LAB_ORANGE      mki_color_make_lab(  72.2,  30.2,  77.2 )  // LAB for Orange
#define MKI_COLOR_LAB_SAND        mki_color_make_lab(  88.8,   6.2,  30.0 )  // LAB for Sand
#define MKI_COLOR_LAB_YELLOWGREEN mki_color_make_lab(  77.8, -28.8,  77.6 )  // LAB for Yellow Green
#define MKI_COLOR_LAB_TURQUOISE   mki_color_make_lab(  66.2, -36.8, -10.8 )  // LAB for Turquoise
#define MKI_COLOR_LAB_JEANSBLUE   mki_color_make_lab(  50.6,  -5.4, -41.6 )  // LAB for Jeans Blue
#define MKI_COLOR_LAB_PURPLE      mki_color_make_lab(  39.8,  68.6, -19.4 )  // LAB for Purple       #39


//---------------------------------------------------------------------------------------------------------------------------------------------------
constexpr double MKI_LINEWIDTH_MIN     = 0.0;   // Minimum allowed line width in [mm]
constexpr double MKI_LINEWIDTH_MAX     = 1.0e5; // Maximum allowed line width in [mm]
constexpr double MKI_LINEWIDTH_FIXED   = 2.11;  // Maximum allowed line width in [mm] for non-zoomed line display in mappings
constexpr double MKI_LINEWIDTH_BYLAYER = -1.0;  // Default line width for "by layer" (any negative value will be interpreted as "by layer")

constexpr double MKI_COORD_MIN = -1.0e12; // Allowed coordinate range
constexpr double MKI_COORD_MAX = 1.0e12;

constexpr double MKI_EDIT_MIN  = 1.0e-10; // Allowed input range
constexpr double MKI_EDIT_MAX  = 1.0e10;


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_INTQUAD;
class MKI_QUAD;
class MKI_BLOCK_ENUM;
class MKI_DIMPARAM;
class MKI_ENTITY_BASE;
class MKI_MULTI_XPROPERTY;


//---------------------------------------------------------------------------------------------------------------------------------------------------
typedef MKI_ENTITY_BASE*             MKI_ENTITY_PTR;
typedef const MKI_ENTITY_BASE* const MKI_CONST_ENTITY_PTR;


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_POINT_BASE
{
  public:

  double x, 
         y;
};

//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_VECTOR_BASE
{
  public:

  double vx, 
         vy;
};

//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MATRIX_BASE
{
  public:

  double m11, 
         m12,
         m21, 
         m22,
         m31, 
         m32;
};

//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_RECT_BASE
{
  public:

  double x1, 
         y1, 
         x2, 
         y2;
};

//---------------------------------------------------------------------------------------------------------------------------------------------------
#define mki_asizeof( p )   ( sizeof( p ) / sizeof( char ) )
#define mki_wsizeof( p )   ( sizeof( p ) / sizeof( wchar_t ) )
#define mki_arraysize( p ) ( sizeof( p ) / sizeof( p[0] ) )


//---------------------------------------------------------------------------------------------------------------------------------------------------
template <class TYPE> void
mki_swap( TYPE& f_rData1, TYPE& f_rData2 )
{
  BYTE cBuffer[sizeof( TYPE )];
  CopyMemory( cBuffer,   &f_rData1, sizeof( TYPE ) );
  CopyMemory( &f_rData1, &f_rData2, sizeof( TYPE ) );
  CopyMemory( &f_rData2, cBuffer,   sizeof( TYPE ) );
}


//---------------------------------------------------------------------------------------------------------------------------------------------------
__forceinline __int32
mki_int32_loword( const __int32 f_nValue )
{
  return( f_nValue & 0xffff );
}

__forceinline __int32
mki_int32_hiword( const __int32 f_nValue )
{
  return( ( f_nValue >> 16 ) & 0xffff );
}

__forceinline __int32
mki_int32_make( const __int32 f_nValueHi, const __int32 f_nValueLo )
{
  return( ( ( f_nValueHi & 0xffff ) << 16 ) | ( f_nValueLo & 0xffff ) );
}

__forceinline __int16
mki_round16( const double f_dValue )
{
  if( f_dValue < 0 )
    return( (__int16) ( f_dValue - 0.5 ) );
  else
    return( (__int16) ( f_dValue + 0.5 ) );
}

__forceinline __int32
mki_round32( const double f_dValue )
{
  if( f_dValue < 0 )
    return( (__int32) ( f_dValue - 0.5 ) );
  else
    return( (__int32) ( f_dValue + 0.5 ) );
}

__forceinline __int64
mki_round64( const double f_dValue )
{
  if( f_dValue < 0 )
    return( (__int64) ( f_dValue - 0.5 ) );
  else
    return( (__int64) ( f_dValue + 0.5 ) );
}

__forceinline double
mki_abs( const double f_dValue )
{
  if( f_dValue < 0 )
    return( -f_dValue );
  else
    return( f_dValue );
}

__forceinline __int32
mki_abs( const __int32 f_nValue )
{
  if( f_nValue < 0 )
    return( -f_nValue );
  else
    return( f_nValue );
}

__forceinline __int64
mki_abs( const __int64 f_nValue )
{
  if( f_nValue < 0 )
    return( -f_nValue );
  else
    return( f_nValue );
}

__forceinline bool
mki_abs_smaller( const double f_dValue, const double f_dThreshold )
{
  return( f_dValue > -f_dThreshold && f_dValue < f_dThreshold );
}

__forceinline bool
mki_abs_smaller( const __int32 f_nValue, const __int32 f_nThreshold )
{
  return( f_nValue > -f_nThreshold && f_nValue < f_nThreshold );
}

__forceinline bool
mki_abs_smaller( const __int64 f_nValue, const __int64 f_nThreshold )
{
  return( f_nValue > -f_nThreshold && f_nValue < f_nThreshold );
}

__forceinline bool
mki_abs_larger( const double f_dValue, const double f_dThreshold )
{
  return( f_dValue < -f_dThreshold || f_dValue > f_dThreshold );
}

__forceinline bool
mki_abs_larger( const __int32 f_nValue, const __int32 f_nThreshold )
{
  return( f_nValue < -f_nThreshold || f_nValue > f_nThreshold );
}

__forceinline bool
mki_abs_larger( const __int64 f_nValue, const __int64 f_nThreshold )
{
  return( f_nValue < -f_nThreshold || f_nValue > f_nThreshold );
}


//---------------------------------------------------------------------------------------------------------------------------------------------------

typedef BYTE MKI_ALPHAVALUE;
MKI_CLASS_TYPES( ALPHAVALUE )

enum
{
  MKI_ALPHA_MIN     = 0,
  MKI_ALPHA_MAX     = 254,
  MKI_ALPHA_BYLAYER = 255,
};

__forceinline MKI_ALPHAVALUE
mki_opacity_to_alpha( const double f_dValue )
{
  return( 255 - (BYTE) mki_round32( 255.0 * f_dValue ) );
}

__forceinline double
mki_alpha_to_opacity( MKI_ALPHAVALUE f_bAlpha )
{
  return( ( 255 - f_bAlpha ) / 255.0 );
}


//---------------------------------------------------------------------------------------------------------------------------------------------------

typedef BYTE MKI_COLORMODEL;
MKI_CLASS_TYPES( COLORMODEL )

enum
{
  MKI_COLORMODE_RGB100  = 0,
  MKI_COLORMODE_RGB255  = 1,
  MKI_COLORMODE_CMYK100 = 2,
  MKI_COLORMODE_CMYK255 = 3,
  MKI_COLORMODE_GRAY100 = 4,
  MKI_COLORMODE_GRAY255 = 5,
  MKI_COLORMODE_HSB     = 6,
  MKI_COLORMODE_LAB     = 7,
  MKI_COLORMODE_BYLAYER = 8,
  MKI_COLORMODE_MAX     = 9,
};

typedef DWORD MKI_COLORVALUE;
MKI_CLASS_TYPES( COLORVALUE )

__forceinline BYTE
mki_color_get_cmyk_c( const MKI_COLORVALUE f_cColor )
{
  return( (BYTE) ( f_cColor >> 24 ) );
}

__forceinline BYTE
mki_color_get_cmyk_m( const MKI_COLORVALUE f_cColor )
{
  return( (BYTE) ( f_cColor >> 16 ) );
}

__forceinline BYTE
mki_color_get_cmyk_y( const MKI_COLORVALUE f_cColor )
{
  return( (BYTE) ( f_cColor >> 8 ) );
}

__forceinline BYTE
mki_color_get_cmyk_k( const MKI_COLORVALUE f_cColor )
{
  return( (BYTE) f_cColor );
}

__forceinline MKI_COLORVALUE
mki_color_make_cmyk( const DWORD f_dwValueC, const DWORD f_dwValueM, const DWORD f_dwValueY, const DWORD f_dwValueK )
{
  return( (MKI_COLORVALUE) ( ( f_dwValueC & 0xff ) << 24 ) | ( ( f_dwValueM & 0xff ) << 16 ) | ( ( f_dwValueY & 0xff ) << 8 ) | ( f_dwValueK & 0xff ) );
}

__forceinline BYTE
mki_color_get_gray_k( const MKI_COLORVALUE f_cColor )
{
  return( (BYTE) f_cColor );
}

__forceinline MKI_COLORVALUE
mki_color_make_gray( const DWORD f_dwValueK )
{
  return( (MKI_COLORVALUE) ( f_dwValueK & 0xff ) );
}

__forceinline WORD
mki_color_get_hsb_h( const MKI_COLORVALUE f_cColor )
{
  return( (WORD) ( f_cColor >> 16 ) & 0x1ff );
}

__forceinline BYTE
mki_color_get_hsb_s( const MKI_COLORVALUE f_cColor )
{
  return( (BYTE) ( f_cColor >> 8 ) );
}

__forceinline BYTE
mki_color_get_hsb_b( const MKI_COLORVALUE f_cColor )
{
  return( (BYTE) f_cColor );
}

__forceinline MKI_COLORVALUE
mki_color_make_hsb( const DWORD f_dwValueH, const DWORD f_dwValueS, const DWORD f_dwValueB )
{
  return( (MKI_COLORVALUE) ( ( f_dwValueH & 0x1ff ) << 16 ) | ( ( f_dwValueS & 0xff ) << 8 ) | ( f_dwValueB & 0xff ) );
}

__forceinline double
mki_color_get_lab_l( const MKI_COLORVALUE f_cColor )
{
  return( 0.2 * ( (WORD) ( f_cColor >> 22 ) & 0x1ff ) );
}

__forceinline double
mki_color_get_lab_a( const MKI_COLORVALUE f_cColor )
{
  return( 0.2 * ( (__int32) ( f_cColor >> 11 ) & 0x7ff ) - 200.0 );
}

__forceinline double
mki_color_get_lab_b( const MKI_COLORVALUE f_cColor )
{
  return( 0.2 * ( (__int32) ( f_cColor ) & 0x7ff ) - 200.0 );
}

__forceinline MKI_COLORVALUE
mki_color_make_lab( const double f_dValueL, const double f_dValueA, const double f_dValueB )
{
  return( (MKI_COLORVALUE) ( ( mki_round32( f_dValueL * 5.0          ) & 0x1ff ) << 22 ) | 
                           ( ( mki_round32( f_dValueA * 5.0 + 1000.0 ) & 0x7ff ) << 11 ) | 
                           ( ( mki_round32( f_dValueB * 5.0 + 1000.0 ) & 0x7ff ) ) );
}

__forceinline BYTE
mki_color_get_rgb_r( const MKI_COLORVALUE f_cColor )
{
  return( GetRValue( f_cColor ) );
}

__forceinline BYTE
mki_color_get_rgb_g( const MKI_COLORVALUE f_cColor )
{
  return( GetGValue( f_cColor ) );
}

__forceinline BYTE
mki_color_get_rgb_b( const MKI_COLORVALUE f_cColor )
{
  return( GetBValue( f_cColor ) );
}

__forceinline MKI_COLORVALUE
mki_color_make_rgb( const DWORD f_dwValueR, const DWORD f_dwValueG, const DWORD f_dwValueB )
{
  return( (MKI_COLORVALUE) RGB( f_dwValueR, f_dwValueG, f_dwValueB ) );
}


//---------------------------------------------------------------------------------------------------------------------------------------------------
template <unsigned f_unSize> class MKI_BITFIELD
{
  public:

  MKI_BITFIELD( void ){};

  //------------------------------------------------------------------

  operator BYTE*()
  {
    return( m_abData );
  }

  //------------------------------------------------------------------

  __forceinline unsigned
  GetBitSize( void ) const
  {
    return( f_unSize );
  }

  __forceinline size_t
  GetByteSize( void ) const
  {
    return( ( f_unSize + 7 ) / 8 );
  }

  __forceinline bool
  IsAllClear( void ) const
  {
    for( size_t unCount = 0; unCount < ( f_unSize + 7 ) / 8; unCount++ )
    {
      if( m_abData[unCount] )
        return( false );
    }
    return( true );
  }

  __forceinline bool
  IsIdentical( const MKI_BITFIELD<f_unSize>& f_rData ) const
  {
    for( size_t unCount = 0; unCount < ( f_unSize + 7 ) / 8; unCount++ )
    {
      if( m_abData[unCount] != f_rData.m_abData[unCount] )
        return( false );
    }
    return( true );
  }

  //------------------------------------------------------------------

  __forceinline void
  SetAll( void )
  {
    for( size_t unCount = 0; unCount < ( f_unSize + 7 ) / 8; unCount++ )
      m_abData[unCount] = 0xff;
  }

  __forceinline void
  ClearAll( void )
  {
    ZeroMemory( m_abData, ( f_unSize + 7 ) / 8 );
  }

  __forceinline void
  InvertAll( void )
  {
    for( size_t unCount = 0; unCount < ( f_unSize + 7 ) / 8; unCount++ )
      m_abData[unCount] ^= 0xff;
  }

  //------------------------------------------------------------------

  __forceinline void
  BitwiseOrAll( const MKI_BITFIELD<f_unSize>& f_rData )
  {
    for( size_t unCount = 0; unCount < ( f_unSize + 7 ) / 8; unCount++ )
      m_abData[unCount] |= f_rData.m_abData[unCount];
  }

  __forceinline void
  BitwiseAndAll( const MKI_BITFIELD<f_unSize>& f_rData )
  {
    for( size_t unCount = 0; unCount < ( f_unSize + 7 ) / 8; unCount++ )
      m_abData[unCount] &= f_rData.m_abData[unCount];
  }

  __forceinline void
  BitwiseXOrAll( const MKI_BITFIELD<f_unSize>& f_rData )
  {
    for( size_t unCount = 0; unCount < ( f_unSize + 7 ) / 8; unCount++ )
      m_abData[unCount] ^= f_rData.m_abData[unCount];
  }

  //------------------------------------------------------------------

  __forceinline void
  SetBit( const unsigned f_unBitNum )
  {
    if( f_unBitNum < f_unSize )
      m_abData[f_unBitNum >> 3] |= (BYTE) ( 0x0080 >> ( f_unBitNum & 7 ) );
  }

  __forceinline void
  ClearBit( const unsigned f_unBitNum )
  {
    if( f_unBitNum < f_unSize )
      m_abData[f_unBitNum >> 3] &= (BYTE) ( 0xff7f >> ( f_unBitNum & 7 ) );
  }

  __forceinline void
  InvertBit( const unsigned f_unBitNum )
  {
    if( f_unBitNum < f_unSize )
      m_abData[f_unBitNum >> 3] ^= (BYTE) ( 0x0080 >> ( f_unBitNum & 7 ) );
  }

  __forceinline bool
  IsBitSet( const unsigned f_unBitNum ) const
  {
    if( f_unBitNum < f_unSize )
      return( ( m_abData[f_unBitNum >> 3] & (BYTE) ( 0x0080 >> ( f_unBitNum & 7 ) ) ) != 0 );
    else
      return( false );
  }

  //------------------------------------------------------------------

  __forceinline void
  CopyBitsToField( const BYTE* f_pSource, const unsigned f_unSourceBit, const unsigned f_unDestinBit, const unsigned f_unBitCount )
  {
    unsigned  unSourceCurrent = f_unSourceBit,
              unDestinCurrent = f_unDestinBit,
              unBitsToCopy = f_unBitCount;

    while( unBitsToCopy-- > 0 && unDestinCurrent < f_unSize )
    {
      if( f_pSource[unSourceCurrent >> 3] & (BYTE) ( 0x0080 >> ( unSourceCurrent & 7 ) ) )
        m_abData[unDestinCurrent >> 3] |= (BYTE) ( 0x0080 >> ( unDestinCurrent & 7 ) );
      else
        m_abData[unDestinCurrent >> 3] &= (BYTE) ( 0xff7f >> ( unDestinCurrent & 7 ) );

      unSourceCurrent++;
      unDestinCurrent++;
    }
  }

  __forceinline void
  CopyFieldToBits( const unsigned f_unSourceBit, BYTE* f_pDestin, const unsigned f_unDestinBit, const unsigned f_unBitCount ) const
  {
    unsigned  unSourceCurrent = f_unSourceBit,
              unDestinCurrent = f_unDestinBit,
              unBitsToCopy = f_unBitCount;

    while( unBitsToCopy-- > 0 && unSourceCurrent < f_unSize )
    {
      if( m_abData[unSourceCurrent >> 3] & (BYTE) ( 0x0080 >> ( unSourceCurrent & 7 ) ) )
        f_pDestin[unDestinCurrent >> 3] |= (BYTE) ( 0x0080 >> ( unDestinCurrent & 7 ) );
      else
        f_pDestin[unDestinCurrent >> 3] &= (BYTE) ( 0xff7f >> ( unDestinCurrent & 7 ) );

      unSourceCurrent++;
      unDestinCurrent++;
    }
  }

  //------------------------------------------------------------------

  BYTE  m_abData[( f_unSize + 7 ) / 8];
};

//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_TIMER
{

public:

  MKI_TIMER( void )
  {
    MKI_TIMER::Reset( 0 );
  };

  MKI_TIMER( __int32 f_nMilliseconds )
  {
    MKI_TIMER::Reset( f_nMilliseconds );
  };

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  Reset( __int32 f_nMilliseconds );

  MKI_INTERFACE_MEMBER void
  Restart( void );

  MKI_INTERFACE_MEMBER bool
  IsExpired( void );

  MKI_INTERFACE_MEMBER double
  GetSecondsSinceStart( void );

  MKI_INTERFACE_MEMBER double
  GetSecondsSinceLast( void );

  MKI_INTERFACE_MEMBER __int32
  GetMilliSecondsSinceStart( void );

  MKI_INTERFACE_MEMBER __int32
  GetMilliSecondsSinceLast( void );

  MKI_INTERFACE_MEMBER __int64
  GetCountsSinceStart( void );

  MKI_INTERFACE_MEMBER __int64
  GetCountsSinceLast( void );

  MKI_INTERFACE_MEMBER __int64
  GetCountsPerSecond( void ) const;

  //------------------------------------------------------------------

private:

  __int64 m_nFrequency,
          m_nInterval,
          m_nTimeStart,
          m_nTimeEnd,
          m_nTimeLast;
};

MKI_CLASS_TYPES( TIMER )

//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_INTPOINT : public POINT
{
  public:

  MKI_INTPOINT( void ){};

  __forceinline
  MKI_INTPOINT( const __int32 f_nX, const __int32 f_nY )
  {
    x = f_nX;
    y = f_nY;
  }

  __forceinline MKI_INTPOINT&
  operator =( const SIZE& f_rVector )
  {
    x = f_rVector.cx;
    y = f_rVector.cy;
    return( *this );
  }

  __forceinline MKI_INTPOINT&
  operator =( const POINT& f_rPoint )
  {
    return( *this = *(MKI_INTPOINT*)&f_rPoint );
  }

  __forceinline bool
  operator ==( const MKI_INTPOINT& f_rPoint ) const
  { 
    return( x == f_rPoint.x &&
            y == f_rPoint.y );
  }

  __forceinline bool
  operator !=( const MKI_INTPOINT& f_rPoint ) const
  { 
    return( x != f_rPoint.x ||
            y != f_rPoint.y );
  }

  __forceinline void
  SetZero( void )
  {
    x = y = 0;
  }

  __forceinline bool
  IsZero( void ) const
  {
    return( x == 0 &&
            y == 0 );
  }

  __forceinline bool
  IsNonZero( void ) const
  {
    return( x != 0 ||
            y != 0 );
  }

  __forceinline void
  SetInvalid( void )
  {
    x = y = MKI_INT32_NOVAL;
  }

  __forceinline bool
  IsValid( void ) const
  {
    return( x != MKI_INT32_NOVAL &&
            y != MKI_INT32_NOVAL );
  }

  __forceinline bool
  IsInvalid( void ) const
  {
    return( x == MKI_INT32_NOVAL ||
            y == MKI_INT32_NOVAL );
  }

  __forceinline void
  Set( const __int32 f_nX, const __int32 f_nY )
  {
    x = f_nX;
    y = f_nY;
  }

  //------------------------------------------------------------------

  __forceinline void
  Add( const __int32 f_nX, const __int32 f_nY )
  {
    x += f_nX;
    y += f_nY;
  }

  __forceinline void
  Add( const SIZE& f_rVector )
  {
    x += f_rVector.cx;
    y += f_rVector.cy;
  }

  __forceinline void
  SetAdd( const POINT& f_rPoint, const SIZE& f_rVector )
  {
    x = f_rPoint.x + f_rVector.cx;
    y = f_rPoint.y + f_rVector.cy;
  }

  __forceinline void
  Sub( const __int32 f_nX, const __int32 f_nY )
  {
    x -= f_nX;
    y -= f_nY;
  }

  __forceinline void
  Sub( const SIZE& f_rVector )
  {
    x -= f_rVector.cx;
    y -= f_rVector.cy;
  }

  __forceinline void
  SetSub( const POINT& f_rPoint, const SIZE& f_rVector )
  {
    x = f_rPoint.x - f_rVector.cx;
    y = f_rPoint.y - f_rVector.cy;
  }

  __forceinline void
  SetCenter( const POINT& f_rPoint1, const POINT& f_rPoint2 )
  {
    x = ( f_rPoint1.x + f_rPoint2.x ) / 2;
    y = ( f_rPoint1.y + f_rPoint2.y ) / 2;
  }

  __forceinline void
  SetCenter( const RECT& f_rRect )
  {
    x = ( f_rRect.left + f_rRect.right ) / 2;
    y = ( f_rRect.top + f_rRect.bottom ) / 2;
  }
};

MKI_CLASS_TYPES( INTPOINT )

//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_INTVECTOR : public SIZE
{
  public:

  MKI_INTVECTOR( void ){};

  __forceinline
  MKI_INTVECTOR( const __int32 f_nX, const __int32 f_nY )
  {
    cx = f_nX;
    cy = f_nY;
  }

  __forceinline
  operator POINT*()
  {
    return( (POINT*) this );
  }

  __forceinline MKI_INTVECTOR&
  operator =( const SIZE& f_rSize )
  {
    return( *this = *(MKI_INTVECTOR*)&f_rSize );
  }

  __forceinline MKI_INTVECTOR&
  operator =( const POINT& f_rPoint )
  {
    cx = f_rPoint.x;
    cy = f_rPoint.y;
    return( *this );
  }

  __forceinline bool
  operator ==( const MKI_INTVECTOR& f_rVector ) const
  { 
    return( cx == f_rVector.cx &&
            cy == f_rVector.cy );
  }

  __forceinline bool
  operator !=( const MKI_INTVECTOR& f_rVector ) const
  { 
    return( cx != f_rVector.cx ||
            cy != f_rVector.cy );
  }

  __forceinline void
  SetZero( void )
  {
    cx = cy = 0;
  }

  __forceinline bool
  IsZero( void ) const
  {
    return( cx == 0 &&
            cy == 0 );
  }

  __forceinline bool
  IsNonZero( void ) const
  {
    return( cx != 0 ||
            cy != 0 );
  }

  __forceinline void
  SetInvalid( void )
  {
    cx = cy = MKI_INT32_NOVAL;
  }

  __forceinline bool
  IsValid( void ) const
  {
    return( cx != MKI_INT32_NOVAL &&
            cy != MKI_INT32_NOVAL );
  }

  __forceinline bool
  IsInvalid( void ) const
  {
    return( cx == MKI_INT32_NOVAL ||
            cy == MKI_INT32_NOVAL );
  }

  __forceinline void
  Set( const __int32 f_nX, const __int32 f_nY )
  {
    cx = f_nX;
    cy = f_nY;
  }

  //------------------------------------------------------------------

  __forceinline void
  Add( const __int32 f_nX, const __int32 f_nY )
  {
    cx += f_nX;
    cy += f_nY;
  }

  __forceinline void
  Add( const SIZE& f_rVector )
  {
    cx += f_rVector.cx;
    cy += f_rVector.cy;
  }

  __forceinline void
  SetAdd( const SIZE& f_rVector1, const SIZE& f_rVector2 )
  {
    cx = f_rVector1.cx + f_rVector2.cx;
    cy = f_rVector1.cy + f_rVector2.cy;
  }

  __forceinline void
  Sub( const __int32 f_nX, const __int32 f_nY )
  {
    cx -= f_nX;
    cy -= f_nY;
  }

  __forceinline void
  Sub( const SIZE& f_rVector )
  {
    cx -= f_rVector.cx;
    cy -= f_rVector.cy;
  }

  __forceinline void
  SetSub( const SIZE& f_rVector1, const SIZE& f_rVector2 )
  {
    cx = f_rVector1.cx - f_rVector2.cx;
    cy = f_rVector1.cy - f_rVector2.cy;
  }

  __forceinline void
  SetSub( const POINT& f_rPoint1, const POINT& f_rPoint2 )
  {
    cx = f_rPoint1.x - f_rPoint2.x;
    cy = f_rPoint1.y - f_rPoint2.y;
  }
};

MKI_CLASS_TYPES( INTVECTOR )

//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_INTRECT : public RECT
{
  public:

  MKI_INTRECT( void ){};

  __forceinline
  MKI_INTRECT( const __int32 f_nX1, const __int32 f_nY1, const __int32 f_nX2, const __int32 f_nY2 )
  {
    left   = f_nX1;
    top    = f_nY1;
    right  = f_nX2;
    bottom = f_nY2;
  }

  MKI_INTERFACE_MEMBER 
  MKI_INTRECT( const MKI_INTQUAD& f_rQuad );

  __forceinline
  operator POINT*()
  {
    return( (POINT*) this );
  }

  __forceinline MKI_INTRECT&
  operator =( const RECT& f_rRect )
  {
    return( *this = *(MKI_INTRECT*)&f_rRect );
  }

  __forceinline bool
  operator ==( const RECT& f_rRect ) const
  { 
    return( left   == f_rRect.left &&
            top    == f_rRect.top &&
            right  == f_rRect.right &&
            bottom == f_rRect.bottom );
  }

  __forceinline bool
  operator !=( const RECT& f_rRect ) const
  { 
    return( left   != f_rRect.left ||
            top    != f_rRect.top ||
            right  != f_rRect.right ||
            bottom != f_rRect.bottom );
  }

  __forceinline void
  SetZero( void )
  {
    left = top = right = bottom = 0;
  }

  __forceinline bool
  IsZero( void ) const
  {
    return( left   == 0 &&
            top    == 0 &&
            right  == 0 &&
            bottom == 0 );
  }

  __forceinline bool
  IsNonZero( void ) const
  {
    return( left   != 0 ||
            top    != 0 ||
            right  != 0 ||
            bottom != 0 );
  }

  __forceinline void
  SetInvalid( void )
  {
    left = top = right = bottom = MKI_INT32_NOVAL;
  }

  __forceinline bool
  IsValid( void ) const
  {
    return( left   != MKI_INT32_NOVAL &&
            top    != MKI_INT32_NOVAL &&
            right  != MKI_INT32_NOVAL &&
            bottom != MKI_INT32_NOVAL );
  }

  __forceinline bool
  IsInvalid( void ) const
  {
    return( left   == MKI_INT32_NOVAL ||
            top    == MKI_INT32_NOVAL ||
            right  == MKI_INT32_NOVAL ||
            bottom == MKI_INT32_NOVAL );
  }

  __forceinline void
  Set( const __int32 f_nX1, const __int32 f_nY1, const __int32 f_nX2, const __int32 f_nY2 )
  {
    left   = f_nX1;
    top    = f_nY1;
    right  = f_nX2;
    bottom = f_nY2;
  }

  __forceinline void
  Set( const POINT& f_rPoint1, const POINT& f_rPoint2 )
  {
    left   = f_rPoint1.x;
    top    = f_rPoint1.y;
    right  = f_rPoint2.x;
    bottom = f_rPoint2.y;
  }

  __forceinline void
  Set( const POINT& f_rPoint )
  {
    left = right  = f_rPoint.x;
    top  = bottom = f_rPoint.y;
  }

  MKI_INTERFACE_MEMBER void
  SetMinMax( const RECT& f_rRect );

  MKI_INTERFACE_MEMBER void
  SetMinMax( const __int32 f_nX1, const __int32 f_nY1, const __int32 f_nX2, const __int32 f_nY2 );

  MKI_INTERFACE_MEMBER void
  SetMinMax( const POINT& f_rPoint1, const POINT& f_rPoint2 );

  __forceinline void
  SetCombine( const RECT& f_rRect1, const RECT& f_rRect2 )
  {
    left   = min( f_rRect1.left,   f_rRect2.left );
    top    = min( f_rRect1.top,    f_rRect2.top );
    right  = max( f_rRect1.right,  f_rRect2.right );
    bottom = max( f_rRect1.bottom, f_rRect2.bottom );
  }

  __forceinline void
  SetIntersect( const RECT& f_rRect1, const RECT& f_rRect2 )
  {
    left   = max( f_rRect1.left,   f_rRect2.left );
    top    = max( f_rRect1.top,    f_rRect2.top );
    right  = min( f_rRect1.right,  f_rRect2.right );
    bottom = min( f_rRect1.bottom, f_rRect2.bottom );
  }

  //------------------------------------------------------------------

  __forceinline bool
  IsDisjoint( const RECT& f_rRect ) const
  {
    return( left   > f_rRect.right  ||
            top    > f_rRect.bottom ||
            right  < f_rRect.left   ||
            bottom < f_rRect.top    );
  }

  __forceinline bool
  IsOutside( const RECT& f_rRect ) const
  {
    return( left   < f_rRect.left   ||
            top    < f_rRect.top    ||
            right  > f_rRect.right  || 
            bottom > f_rRect.bottom );
  }

  __forceinline void
  Combine( const RECT& f_rRect )
  {
    if( f_rRect.left < left )
      left = f_rRect.left;
    if( f_rRect.right > right )
      right = f_rRect.right;
    if( f_rRect.top < top )
      top = f_rRect.top;
    if( f_rRect.bottom > bottom )
      bottom = f_rRect.bottom;
  }

  __forceinline void
  Intersect( const RECT& f_rRect )
  {
    if( f_rRect.left > left )
      left = f_rRect.left;
    if( f_rRect.right < right )
      right = f_rRect.right;
    if( f_rRect.top > top )
      top = f_rRect.top;
    if( f_rRect.bottom < bottom )
      bottom = f_rRect.bottom;
  }

  __forceinline void
  Include( const POINT& f_rPoint )
  {
    if( f_rPoint.x < left )
      left = f_rPoint.x;
    if( f_rPoint.x > right )
      right = f_rPoint.x;
    if( f_rPoint.y < top )
      top = f_rPoint.y;
    if( f_rPoint.y > bottom )
      bottom = f_rPoint.y;
  }

  __forceinline void
  Include( __int32 f_nX, __int32 f_nY )
  {
    if( f_nX < left )
      left = f_nX;
    if( f_nX > right )
      right = f_nX;
    if( f_nY < top )
      top = f_nY;
    if( f_nY > bottom )
      bottom = f_nY;
  }

  __forceinline bool
  Contains( const MKI_INTPOINT& f_rPoint ) const
  {
    return( f_rPoint.x >= left   &&
            f_rPoint.x <  right  && 
            f_rPoint.y >= top    && 
            f_rPoint.y <  bottom );
  }

  __forceinline bool
  Contains( const MKI_POINT_BASE& f_rPoint ) const
  {
    return( f_rPoint.x >= left   &&
            f_rPoint.x <  right  && 
            f_rPoint.y >= top    && 
            f_rPoint.y <  bottom );
  }

  __forceinline void
  Expand( const __int32 f_nAmount )
  {
    left   -= f_nAmount;
    top    -= f_nAmount;
    right  += f_nAmount;
    bottom += f_nAmount;
  }

  __forceinline void
  Expand( const __int32 f_nAmountX, const __int32 f_nAmountY )
  {
    left   -= f_nAmountX;
    top    -= f_nAmountY;
    right  += f_nAmountX;
    bottom += f_nAmountY;
  }

  __forceinline void
  Move( const SIZE& f_rVector )
  {
    left   += f_rVector.cx;
    top    += f_rVector.cy;
    right  += f_rVector.cx;
    bottom += f_rVector.cy;
  }

  __forceinline __int32
  GetMinX( void ) const
  {
    return( min( left, right ) );
  }

  __forceinline __int32
  GetMinY( void ) const
  {
    return( min( top, bottom ) );
  }

  __forceinline __int32
  GetMaxX( void ) const
  {
    return( max( left, right ) );
  }

  __forceinline __int32
  GetMaxY( void ) const
  {
    return( max( top, bottom ) );
  }

  __forceinline __int32
  GetAbsSizeX( void ) const
  {
    return( mki_abs( right - left ) );
  }

  __forceinline __int32
  GetAbsSizeY( void ) const
  {
    return( mki_abs( bottom - top ) );
  }

  __forceinline void
  GetAbsSize( SIZE& f_rSize ) const
  {
    f_rSize.cx = mki_abs( right - left );
    f_rSize.cy = mki_abs( bottom - top );
  }

  __forceinline void
  GetAbsSize( MKI_VECTOR_BASE& f_rSize ) const
  {
    f_rSize.vx = mki_abs( right - left );
    f_rSize.vy = mki_abs( bottom - top );
  }

  __forceinline __int32
  GetCenterX( void ) const
  {
    return( ( left + right ) / 2 );
  }

  __forceinline __int32
  GetCenterY( void ) const
  {
    return( ( top + bottom ) / 2 );
  }

  __forceinline void
  GetCenter( POINT& f_rCenter ) const
  {
    f_rCenter.x = ( left + right ) / 2;
    f_rCenter.y = ( top + bottom ) / 2;
  }

  __forceinline void
  GetCenter( MKI_POINT_BASE& f_rCenter ) const
  {
    f_rCenter.x = 0.5 * ( left + right );
    f_rCenter.y = 0.5 * ( top + bottom );
  }

  __forceinline void
  GetCenter( SIZE& f_rCenter ) const
  {
    f_rCenter.cx = ( left + right ) / 2;
    f_rCenter.cy = ( top + bottom ) / 2;
  }

  __forceinline void
  GetCenter( MKI_VECTOR_BASE& f_rCenter ) const
  {
    f_rCenter.vx = 0.5 * ( left + right );
    f_rCenter.vy = 0.5 * ( top + bottom );
  }

  __forceinline __int32
  GetArea( void ) const
  {
    return( mki_abs( ( right - left ) * ( bottom - top ) ) );
  }

};

MKI_CLASS_TYPES( INTRECT )

//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_INTQUAD
{
  public:

  MKI_INTQUAD( void ){};

  __forceinline
  MKI_INTQUAD( const __int32 f_nX1, const __int32 f_nY1, const __int32 f_nX2, const __int32 f_nY2, 
               const __int32 f_nX3, const __int32 f_nY3, const __int32 f_nX4, const __int32 f_nY4 )
  {
    m_cPoint1.x = f_nX1;
    m_cPoint1.y = f_nY1;
    m_cPoint2.x = f_nX2;
    m_cPoint2.y = f_nY2;
    m_cPoint3.x = f_nX3;
    m_cPoint3.y = f_nY3;
    m_cPoint4.x = f_nX4;
    m_cPoint4.y = f_nY4;
  }

  __forceinline
  MKI_INTQUAD( const POINT& f_rPoint1, const POINT& f_rPoint2,
               const POINT& f_rPoint3, const POINT& f_rPoint4 )
  {
    m_cPoint1 = f_rPoint1;
    m_cPoint2 = f_rPoint2;
    m_cPoint3 = f_rPoint3;
    m_cPoint4 = f_rPoint4;
  }

  __forceinline
  MKI_INTQUAD( const __int32 f_nX1, const __int32 f_nY1, const __int32 f_nX2, const __int32 f_nY2 )
  {
    m_cPoint1.x = m_cPoint2.x = f_nX1;
    m_cPoint1.y = m_cPoint4.y = f_nY1;
    m_cPoint2.y = m_cPoint3.y = f_nY2;
    m_cPoint3.x = m_cPoint4.x = f_nX2;
  }

  __forceinline
  MKI_INTQUAD( const RECT& f_rRect )
  {
    m_cPoint1.x = f_rRect.left;
    m_cPoint1.y = f_rRect.bottom;
    m_cPoint2.x = f_rRect.right;
    m_cPoint2.y = f_rRect.bottom;
    m_cPoint3.x = f_rRect.right;
    m_cPoint3.y = f_rRect.top;
    m_cPoint4.x = f_rRect.left;
    m_cPoint4.y = f_rRect.top;
  }

  __forceinline void
  SetZero( void )
  {
    ZeroMemory( this, sizeof( *this ) );
  }

  __forceinline bool
  IsZero( void ) const
  {
    return( m_cPoint1.x == 0 &&
            m_cPoint1.y == 0 &&
            m_cPoint2.x == 0 &&
            m_cPoint2.y == 0 &&
            m_cPoint3.x == 0 &&
            m_cPoint3.y == 0 &&
            m_cPoint4.x == 0 &&
            m_cPoint4.y == 0 );
  }

  __forceinline bool
  IsNonZero( void ) const
  {
    return( m_cPoint1.x != 0 ||
            m_cPoint1.y != 0 ||
            m_cPoint2.x != 0 ||
            m_cPoint2.y != 0 ||
            m_cPoint3.x != 0 ||
            m_cPoint3.y != 0 ||
            m_cPoint4.x != 0 ||
            m_cPoint4.y != 0 );
  }

  __forceinline
  operator POINT*()
  {
    return( (POINT*) this );
  }

  __forceinline MKI_INTQUAD&
  operator =( const RECT& f_rRect )
  {
    m_cPoint1.x = f_rRect.left;
    m_cPoint1.y = f_rRect.bottom;
    m_cPoint2.x = f_rRect.right;
    m_cPoint2.y = f_rRect.bottom;
    m_cPoint3.x = f_rRect.right;
    m_cPoint3.y = f_rRect.top;
    m_cPoint4.x = f_rRect.left;
    m_cPoint4.y = f_rRect.top;
    return( *this );
  }

  __forceinline void
  Set( const __int32 f_nX1, const __int32 f_nY1, const __int32 f_nX2, const __int32 f_nY2, 
       const __int32 f_nX3, const __int32 f_nY3, const __int32 f_nX4, const __int32 f_nY4 )
  {
    m_cPoint1.x = f_nX1;
    m_cPoint1.y = f_nY1;
    m_cPoint2.x = f_nX2;
    m_cPoint2.y = f_nY2;
    m_cPoint3.x = f_nX3;
    m_cPoint3.y = f_nY3;
    m_cPoint4.x = f_nX4;
    m_cPoint4.y = f_nY4;
  }

  __forceinline void
  Set( const __int32 f_nX1, const __int32 f_nY1, const __int32 f_nX2, const __int32 f_nY2 )
  {
    m_cPoint1.x = m_cPoint2.x = f_nX1;
    m_cPoint1.y = m_cPoint4.y = f_nY1;
    m_cPoint2.y = m_cPoint3.y = f_nY2;
    m_cPoint3.x = m_cPoint4.x = f_nX2;
  }

  //------------------------------------------------------------------

  MKI_INTPOINT m_cPoint1,
               m_cPoint2,
               m_cPoint3,
               m_cPoint4;
};

MKI_CLASS_TYPES( INTQUAD )

//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_POINT : public MKI_POINT_BASE
{
  public:

  MKI_POINT( void ){};

  __forceinline
  MKI_POINT( const MKI_POINT* f_pThis )
  {
    *this = *f_pThis;
  }

  __forceinline
  MKI_POINT( const MKI_INTPOINT& f_rPoint )
  {
    x = f_rPoint.x;
    y = f_rPoint.y;
  }

  __forceinline
  MKI_POINT( const double f_dX, const double f_dY )
  {
    x = f_dX;
    y = f_dY;
  }

  __forceinline
  MKI_POINT( const MKI_POINT& f_rPoint, const MKI_VECTOR_BASE& f_rVector )
  {
    x = f_rPoint.x + f_rVector.vx;
    y = f_rPoint.y + f_rVector.vy;
  }

  __forceinline bool
  operator ==( const MKI_POINT& f_rPoint ) const
  { 
    return( x == f_rPoint.x &&
            y == f_rPoint.y );
  }

  __forceinline bool
  operator !=( const MKI_POINT& f_rPoint ) const
  { 
    return( x != f_rPoint.x ||
            y != f_rPoint.y );
  }

  //------------------------------------------------------------------

  __forceinline void
  SetZero( void )
  {
    ZeroMemory( this, sizeof( *this ) );
  }

  __forceinline bool
  IsZero( void ) const
  {
    return( x == 0.0 &&
            y == 0.0 );
  }

  __forceinline bool
  IsNonZero( void ) const
  {
    return( x != 0.0 ||
            y != 0.0 );
  }

  __forceinline void
  SetInvalid( void )
  {
    x = y = MKI_DOUBLE_NOVAL;
  }

  __forceinline bool
  IsValid( void ) const
  {
    return( x != MKI_DOUBLE_NOVAL &&
            y != MKI_DOUBLE_NOVAL );
  }

  __forceinline bool
  IsInvalid( void ) const
  {
    return( x == MKI_DOUBLE_NOVAL ||
            y == MKI_DOUBLE_NOVAL );
  }

  __forceinline void
  Set( const double f_dX, const double f_dY )
  {
    x = f_dX;
    y = f_dY;
  }

  __forceinline void
  Set( const MKI_VECTOR_BASE& f_rVector )
  {
    x = f_rVector.vx;
    y = f_rVector.vy;
  }

  //------------------------------------------------------------------

  __forceinline void
  Add( const double f_dX, const double f_dY )
  {
    x += f_dX;
    y += f_dY;
  }

  __forceinline void
  Add( const MKI_VECTOR_BASE& f_rVector )
  {
    x += f_rVector.vx;
    y += f_rVector.vy;
  }

  __forceinline void
  Add( const double f_dFactor, const MKI_VECTOR_BASE& f_rVector )
  {
    x += f_dFactor * f_rVector.vx;
    y += f_dFactor * f_rVector.vy;
  }

  __forceinline void
  SetAdd( const MKI_POINT& f_rPoint, const MKI_VECTOR_BASE& f_rVector )
  {
    x = f_rPoint.x + f_rVector.vx;
    y = f_rPoint.y + f_rVector.vy;
  }

  __forceinline void
  SetAdd( const MKI_POINT& f_rPoint, const double f_dFactor, const MKI_VECTOR_BASE& f_rVector )
  {
    x = f_rPoint.x + f_dFactor * f_rVector.vx;
    y = f_rPoint.y + f_dFactor * f_rVector.vy;
  }

  __forceinline void
  Sub( const double f_dX, const double f_dY )
  {
    x -= f_dX;
    y -= f_dY;
  }

  __forceinline void
  Sub( const MKI_VECTOR_BASE& f_rVector )
  {
    x -= f_rVector.vx;
    y -= f_rVector.vy;
  }

  __forceinline void
  Sub( const double f_dFactor, const MKI_VECTOR_BASE& f_rVector )
  {
    x -= f_dFactor * f_rVector.vx;
    y -= f_dFactor * f_rVector.vy;
  }

  __forceinline void
  SetSub( const MKI_POINT& f_rPoint, const MKI_VECTOR_BASE& f_rVector )
  {
    x = f_rPoint.x - f_rVector.vx;
    y = f_rPoint.y - f_rVector.vy;
  }

  __forceinline void
  SetSub( const MKI_POINT& f_rPoint, const double f_dFactor, const MKI_VECTOR_BASE& f_rVector )
  {
    x = f_rPoint.x - f_dFactor * f_rVector.vx;
    y = f_rPoint.y - f_dFactor * f_rVector.vy;
  }

  __forceinline void
  SetCenter( const MKI_POINT& f_rPoint1, const MKI_POINT& f_rPoint2 )
  {
    x = 0.5 * ( f_rPoint1.x + f_rPoint2.x );
    y = 0.5 * ( f_rPoint1.y + f_rPoint2.y );
  }

  __forceinline void
  SetCenter( const MKI_RECT_BASE& f_rRect )
  {
    x = 0.5 * ( f_rRect.x1 + f_rRect.x2 );
    y = 0.5 * ( f_rRect.y1 + f_rRect.y2 );
  }

  __forceinline void
  SetArcCenter( const MKI_POINT& f_rPoint1, const MKI_POINT& f_rPoint2, const double f_dCurve )
  {
    x = 0.5 * ( f_rPoint1.x + f_rPoint2.x ) - f_dCurve * ( f_rPoint2.y - f_rPoint1.y );
    y = 0.5 * ( f_rPoint1.y + f_rPoint2.y ) + f_dCurve * ( f_rPoint2.x - f_rPoint1.x );
  }

  __forceinline void
  Multiply( const MKI_MATRIX_BASE& f_rMatrix )
  {
    double  dBackup = x;
    x = f_rMatrix.m11 * dBackup + f_rMatrix.m21 * y + f_rMatrix.m31;
    y = f_rMatrix.m12 * dBackup + f_rMatrix.m22 * y + f_rMatrix.m32;
  }

  __forceinline void
  SetMultiply( const MKI_MATRIX_BASE& f_rMatrix, const MKI_POINT& f_rPoint )
  {
    x = f_rMatrix.m11 * f_rPoint.x + f_rMatrix.m21 * f_rPoint.y + f_rMatrix.m31;
    y = f_rMatrix.m12 * f_rPoint.x + f_rMatrix.m22 * f_rPoint.y + f_rMatrix.m32;
  }

  __forceinline void
  SetMultiply( const MKI_MATRIX_BASE& f_rMatrix, const double f_dX, const double f_dY )
  {
    x = f_rMatrix.m11 * f_dX + f_rMatrix.m21 * f_dY + f_rMatrix.m31;
    y = f_rMatrix.m12 * f_dX + f_rMatrix.m22 * f_dY + f_rMatrix.m32;
  }

  __forceinline bool
  IsNear( const MKI_POINT& f_rPoint, const double f_dDistance ) const
  {
    return( f_rPoint.x - x < f_dDistance && 
            x - f_rPoint.x < f_dDistance &&
            f_rPoint.y - y < f_dDistance && 
            y - f_rPoint.y < f_dDistance );
  }

  __forceinline bool
  IsNotNear( const MKI_POINT& f_rPoint, const double f_dDistance ) const
  {
    return( f_rPoint.x - x >= f_dDistance || 
            x - f_rPoint.x >= f_dDistance ||
            f_rPoint.y - y >= f_dDistance || 
            y - f_rPoint.y >= f_dDistance );
  }

  __forceinline double
  SquareTo( const MKI_POINT& f_rPoint ) const
  {
    double dx = f_rPoint.x - x,
           dy = f_rPoint.y - y;
    return( dx * dx + dy * dy );
  }

  __forceinline double
  SquareTo( const double f_dX, const double f_dY ) const
  {
    double dx = f_dX - x,
           dy = f_dY - y;
    return( dx * dx + dy * dy );
  }

  __forceinline double
  LengthTo( const MKI_POINT& f_rPoint ) const
  {
    double dx = f_rPoint.x - x,
           dy = f_rPoint.y - y;
    return( sqrt( dx * dx + dy * dy ) );
  }

  __forceinline double
  LengthTo( const double f_dX, const double f_dY ) const
  {
    double dx = f_dX - x,
           dy = f_dY - y;
    return( sqrt( dx * dx + dy * dy ) );
  }

  MKI_INTERFACE_MEMBER double
  AngleTo( const MKI_POINT& f_rPoint ) const;

  MKI_INTERFACE_MEMBER double
  AngleTo( const double f_dX, const double f_dY ) const;

  //------------------------------------------------------------------

  __forceinline double&
  Curve( void )
  {
    return( x );
  }

  __forceinline double
  Curve( void ) const
  {
    return( x );
  }

  __forceinline double&
  Orient( void )
  {
    return( y );
  }

  __forceinline double
  Orient( void ) const
  {
    return( y );
  }

  __forceinline bool
  IsOrientPositive( void )
  {
    return( y >= 0.0 );
  }

  __forceinline bool
  IsOrientNegative( void )
  {
    return( y < 0.0 );
  }

};

MKI_CLASS_TYPES( POINT )

//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_VECTOR : public MKI_VECTOR_BASE
{
  public:

  MKI_VECTOR( void ){};

  __forceinline
  MKI_VECTOR( const MKI_VECTOR* f_pThis )
  {
    *this = *f_pThis;
  }

  __forceinline
  MKI_VECTOR( const double f_dX, const double f_dY )
  {
    vx = f_dX;
    vy = f_dY;
  }

  __forceinline
  MKI_VECTOR( const MKI_POINT_BASE& f_rPoint1, const MKI_POINT_BASE& f_rPoint2 )
  {
    vx = f_rPoint1.x - f_rPoint2.x;
    vy = f_rPoint1.y - f_rPoint2.y;
  }

  __forceinline bool
  operator ==( const MKI_VECTOR& f_rVector ) const
  { 
    return( vx == f_rVector.vx &&
            vy == f_rVector.vy );
  }

  __forceinline bool
  operator !=( const MKI_VECTOR& f_rVector ) const
  { 
    return( vx != f_rVector.vx ||
            vy != f_rVector.vy );
  }

  //------------------------------------------------------------------

  __forceinline void
  SetZero( void )
  {
    ZeroMemory( this, sizeof( *this ) );
  }

  __forceinline bool
  IsZero( void ) const
  {
    return( vx >= -MKI_EDIT_MIN && vx <= MKI_EDIT_MIN &&
            vy >= -MKI_EDIT_MIN && vy <= MKI_EDIT_MIN );
  }

  __forceinline bool
  IsNonZero( void ) const
  {
    return( vx < -MKI_EDIT_MIN || vx > MKI_EDIT_MIN ||
            vy < -MKI_EDIT_MIN || vy > MKI_EDIT_MIN );
  }

  __forceinline void
  SetInvalid( void )
  {
    vx = vy = MKI_DOUBLE_NOVAL;
  }

  __forceinline bool
  IsValid( void ) const
  {
    return( vx != MKI_DOUBLE_NOVAL &&
            vy != MKI_DOUBLE_NOVAL );
  }

  __forceinline bool
  IsInvalid( void ) const
  {
    return( vx == MKI_DOUBLE_NOVAL ||
            vy == MKI_DOUBLE_NOVAL );
  }

  __forceinline void
  Set( const double f_dX, const double f_dY )
  {
    vx = f_dX;
    vy = f_dY;
  }

  __forceinline void
  Set( const MKI_POINT_BASE& f_rPoint )
  {
    vx = f_rPoint.x;
    vy = f_rPoint.y;
  }

  //------------------------------------------------------------------

  __forceinline void
  Add( const double f_dX, const double f_dY )
  {
    vx += f_dX;
    vy += f_dY;
  }

  __forceinline void
  Add( const MKI_VECTOR& f_rVector )
  {
    vx += f_rVector.vx;
    vy += f_rVector.vy;
  }

  __forceinline void
  Add( const double f_dFactor, const MKI_VECTOR& f_rVector )
  {
    vx += f_dFactor * f_rVector.vx;
    vy += f_dFactor * f_rVector.vy;
  }

  __forceinline void
  SetAdd( const MKI_VECTOR& f_rVector1, const MKI_VECTOR& f_rVector2 )
  {
    vx = f_rVector1.vx + f_rVector2.vx;
    vy = f_rVector1.vy + f_rVector2.vy;
  }

  __forceinline void
  SetAdd( const MKI_VECTOR& f_rVector1, const double f_dFactor, const MKI_VECTOR& f_rVector2 )
  {
    vx = f_rVector1.vx + f_dFactor * f_rVector2.vx;
    vy = f_rVector1.vy + f_dFactor * f_rVector2.vy;
  }

  __forceinline void
  Sub( const double f_dX, const double f_dY )
  {
    vx -= f_dX;
    vy -= f_dY;
  }

  __forceinline void
  Sub( const MKI_VECTOR& f_rVector )
  {
    vx -= f_rVector.vx;
    vy -= f_rVector.vy;
  }

  __forceinline void
  SetSub( const MKI_VECTOR& f_rVector1, const MKI_VECTOR& f_rVector2 )
  {
    vx = f_rVector1.vx - f_rVector2.vx;
    vy = f_rVector1.vy - f_rVector2.vy;
  }

  __forceinline void
  SetSub( const MKI_POINT_BASE& f_rPoint1, const MKI_POINT_BASE& f_rPoint2 )
  {
    vx = f_rPoint1.x - f_rPoint2.x;
    vy = f_rPoint1.y - f_rPoint2.y;
  }

  __forceinline void
  Divide( const double f_dFactor )
  {
    if( f_dFactor )
    {
      vx /= f_dFactor;
      vy /= f_dFactor;
    }
  }

  __forceinline void
  Multiply( const double f_dFactor )
  {
    vx *= f_dFactor;
    vy *= f_dFactor;
  }

  __forceinline void
  Multiply( const MKI_MATRIX_BASE& f_rMatrix )
  {
    double  dBackup = vx;
    vx = f_rMatrix.m11 * dBackup + f_rMatrix.m21 * vy;
    vy = f_rMatrix.m12 * dBackup + f_rMatrix.m22 * vy;
  }

  __forceinline void
  SetMultiply( const double f_dFactor, const MKI_VECTOR& f_rVector )
  {
    vx = f_rVector.vx * f_dFactor;
    vy = f_rVector.vy * f_dFactor;
  }

  __forceinline void
  SetMultiply( const MKI_MATRIX_BASE& f_rMatrix, const MKI_VECTOR& f_rVector )
  {
    vx = f_rMatrix.m11 * f_rVector.vx + f_rMatrix.m21 * f_rVector.vy;
    vy = f_rMatrix.m12 * f_rVector.vx + f_rMatrix.m22 * f_rVector.vy;
  }

  MKI_INTERFACE_MEMBER void
  Rotate( const double f_dAngle );

  MKI_INTERFACE_MEMBER void
  SetRotate( const double f_dAngle, const MKI_VECTOR& f_rVector );

  __forceinline double
  Scalar( const MKI_VECTOR& f_rVector ) const
  {
    return( vx * f_rVector.vx + vy * f_rVector.vy );
  }

  __forceinline double
  Determinant( const MKI_VECTOR& f_rVector ) const
  {
    return( vx * f_rVector.vy - vy * f_rVector.vx );
  }

  __forceinline double
  Square( void ) const
  {
    return( vx * vx + vy * vy );
  }

  __forceinline double
  Length( void ) const
  {
    return( sqrt( vx * vx + vy * vy ) );
  }

  MKI_INTERFACE_MEMBER double
  Normalize( void );

  MKI_INTERFACE_MEMBER double
  Normalize( double f_dLength );

  MKI_INTERFACE_MEMBER double
  Angle( void ) const;

  MKI_INTERFACE_MEMBER double
  AngleTo( const MKI_VECTOR& f_rVector ) const;
};

MKI_CLASS_TYPES( VECTOR )

//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_POINTDATA
{
  public:

  __forceinline double&
  Curve( void )
  {
    return( m_cPoint.x );
  }

  __forceinline double
  Curve( void ) const
  {
    return( m_cPoint.x );
  }

  __forceinline double&
  Orient( void )
  {
    return( m_cPoint.y );
  }

  __forceinline double
  Orient( void ) const
  {
    return( m_cPoint.y );
  }

  __forceinline bool
  IsOrientPositive( void )
  {
    return( m_cPoint.y >= 0.0 );
  }

  __forceinline bool
  IsOrientNegative( void )
  {
    return( m_cPoint.y < 0.0 );
  }

  //------------------------------------------------------------------

  MKI_POINT m_cPoint;
  __int32   m_nType,
            m_nHandle,
            m_nOutline;
};

MKI_CLASS_TYPES( POINTDATA )

//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MATRIX : public MKI_MATRIX_BASE
{
  public:

  MKI_MATRIX( void ){};

  __forceinline
  MKI_MATRIX( const MKI_MATRIX* f_pThis )
  {
    *this = *f_pThis;
  }

  __forceinline
  MKI_MATRIX( const double f_dM11, const double f_dM12, const double f_dM21, const double f_dM22, const double f_dM31, const double f_dM32 )
  {
    m11 = f_dM11;
    m12 = f_dM12;
    m21 = f_dM21;
    m22 = f_dM22;
    m31 = f_dM31;
    m32 = f_dM32;
  }

  //------------------------------------------------------------------

  __forceinline double
  x( const double f_dX, const double f_dY ) const
  {
    return( m11 * f_dX + m21 * f_dY + m31 );
  }

  __forceinline double
  x( const MKI_POINT_BASE& f_rPoint ) const
  {
    return( m11 * f_rPoint.x + m21 * f_rPoint.y + m31 );
  }

  __forceinline double
  x( const MKI_VECTOR_BASE& f_rVector ) const
  {
    return( m11 * f_rVector.vx + m21 * f_rVector.vy + m31 );
  }

  __forceinline double
  y( const double f_dX, const double f_dY ) const
  {
    return( m12 * f_dX + m22 * f_dY + m32 );
  }

  __forceinline double
  y( const MKI_POINT_BASE& f_rPoint ) const
  {
    return( m12 * f_rPoint.x + m22 * f_rPoint.y + m32 );
  }

  __forceinline double
  y( const MKI_VECTOR_BASE& f_rVector ) const
  {
    return( m12 * f_rVector.vx + m22 * f_rVector.vy + m32 );
  }

  __forceinline MKI_POINT
  Point( const double f_dX, const double f_dY ) const
  {
    return( MKI_POINT( m11 * f_dX + m21 * f_dY + m31,
                       m12 * f_dX + m22 * f_dY + m32 ) );
  }

  __forceinline MKI_POINT
  Point( const MKI_POINT_BASE& f_rPoint ) const
  {
    return( MKI_POINT( m11 * f_rPoint.x + m21 * f_rPoint.y + m31,
                       m12 * f_rPoint.x + m22 * f_rPoint.y + m32 ) );
  }

  __forceinline MKI_POINT_REF
  GetTranslation( void ) const
  {
    return( *( (MKI_POINT_PTR) &m31 ) );
  }

  __forceinline void
  SetTranslation( MKI_CONST_POINT_REF f_rPoint )
  {
    *( (MKI_POINT_PTR) &m31 ) = f_rPoint;
  }

  __forceinline void
  SetTranslation( MKI_CONST_VECTOR_REF f_rVector )
  {
    *( (MKI_VECTOR_PTR) &m31 ) = f_rVector;
  }

  __forceinline void
  ApplyPoint( double& f_rdX, double& f_rdY ) const
  {
    double dValue = f_rdX;
    f_rdX = m11 * dValue + m21 * f_rdY + m31;
    f_rdY = m12 * dValue + m22 * f_rdY + m32;
  }

  __forceinline void
  ApplyPoint( MKI_POINT_BASE& f_rPoint ) const
  {
    double dValue = f_rPoint.x;
    f_rPoint.x = m11 * dValue + m21 * f_rPoint.y + m31;
    f_rPoint.y = m12 * dValue + m22 * f_rPoint.y + m32;
  }

  __forceinline void
  ApplyPoint( MKI_MATRIX& f_rMatrix ) const
  {
    double dValue = f_rMatrix.m31;
    f_rMatrix.m31 = m11 * dValue + m21 * f_rMatrix.m32 + m31;
    f_rMatrix.m32 = m12 * dValue + m22 * f_rMatrix.m32 + m32;
  }

  __forceinline MKI_POINT
  Offset( const double f_dX, const double f_dY ) const
  {
    return( MKI_POINT( m11 * f_dX + m21 * f_dY,
                       m12 * f_dX + m22 * f_dY ) );
  }

  __forceinline MKI_POINT
  Offset( const MKI_POINT_BASE& f_rPoint ) const
  {
    return( MKI_POINT( m11 * f_rPoint.x + m21 * f_rPoint.y,
                       m12 * f_rPoint.x + m22 * f_rPoint.y ) );
  }

  __forceinline double
  dx( const double f_dX, const double f_dY ) const
  {
    return( m11 * f_dX + m21 * f_dY );
  }

  __forceinline double
  dx( const MKI_POINT_BASE& f_rPoint ) const
  {
    return( m11 * f_rPoint.x + m21 * f_rPoint.y );
  }

  __forceinline double
  dx( const MKI_VECTOR_BASE& f_rVector ) const
  {
    return( m11 * f_rVector.vx + m21 * f_rVector.vy );
  }

  __forceinline double
  dy( const double f_dX, const double f_dY ) const
  {
    return( m12 * f_dX + m22 * f_dY );
  }

  __forceinline double
  dy( const MKI_POINT_BASE& f_rPoint ) const
  {
    return( m12 * f_rPoint.x + m22 * f_rPoint.y );
  }

  __forceinline double
  dy( const MKI_VECTOR_BASE& f_rVector ) const
  {
    return( m12 * f_rVector.vx + m22 * f_rVector.vy );
  }

  __forceinline MKI_VECTOR
  Vector( const double f_dX, const double f_dY ) const
  {
    return( MKI_VECTOR( m11 * f_dX + m21 * f_dY,
                        m12 * f_dX + m22 * f_dY ) );
  }

  __forceinline MKI_VECTOR
  Vector( const MKI_VECTOR_BASE& f_rVector ) const
  {
    return( MKI_VECTOR( m11 * f_rVector.vx + m21 * f_rVector.vy,
                        m12 * f_rVector.vx + m22 * f_rVector.vy ) );
  }

  //------------------------------------------------------------------

  __forceinline void
  Init( void )
  {
          m12 = 
    m21 = 
    m31 = m32 = 0.0;

    m11 = m22 = 1.0;
  }

  __forceinline void
  SetZero( void )
  {
    ZeroMemory( this, sizeof( *this ) );
  }

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  Rotate( const double f_dAlpha );

  MKI_INTERFACE_MEMBER void
  RotateFix( const double f_dAlpha );

  MKI_INTERFACE_MEMBER void
  Distort( const double f_dAlpha, const double f_dBeta );

  MKI_INTERFACE_MEMBER void
  DistortX( const double f_dAlpha );

  MKI_INTERFACE_MEMBER void
  DistortY( const double f_dBeta );

  MKI_INTERFACE_MEMBER void
  Move( const double f_dX, const double f_dY );

  MKI_INTERFACE_MEMBER void
  Move( const MKI_VECTOR_BASE& f_rVector );

  MKI_INTERFACE_MEMBER void
  Scale( const double f_dX, const double f_dY );

  MKI_INTERFACE_MEMBER void
  Scale( const double f_dFactor );

  MKI_INTERFACE_MEMBER void
  Scale( const MKI_VECTOR_BASE& f_rVector );

  MKI_INTERFACE_MEMBER void
  ScaleFix( const double f_dX, const double f_dY );

  MKI_INTERFACE_MEMBER void
  ScaleFix( const double f_dFactor );

  MKI_INTERFACE_MEMBER void
  ScaleFix( const MKI_VECTOR_BASE& f_rVector );

  MKI_INTERFACE_MEMBER void
  MirrorX( void );

  MKI_INTERFACE_MEMBER void
  MirrorY( void );

  MKI_INTERFACE_MEMBER void
  SetMultiply( const MKI_MATRIX& f_rBase, const MKI_MATRIX& f_rFactor );

  MKI_INTERFACE_MEMBER void
  Multiply( const MKI_MATRIX& f_rFactor );

  __forceinline double
  Determinant( void ) const
  {
    return( m11 * m22 - m12 * m21 );
  }

  MKI_INTERFACE_MEMBER bool
  SetInvert( const MKI_MATRIX& f_rSource );

  MKI_INTERFACE_MEMBER bool
  Invert( void );

  MKI_INTERFACE_MEMBER bool
  IsMirrorMatrix( void ) const;

  MKI_INTERFACE_MEMBER bool
  IsNonscaleMatrix( void ) const;

  MKI_INTERFACE_MEMBER bool
  IsDistortMatrix( void ) const;

  //------------------------------------------------------------------

  __forceinline double
  GetMoveX( void ) const
  {
    return( m31 );
  }

  __forceinline double
  GetMoveY( void ) const
  {
    return( m32 );
  }

  MKI_INTERFACE_MEMBER double
  GetScaling( void ) const;

  MKI_INTERFACE_MEMBER bool
  GetScaling( double& f_rdScaleX, double& f_rdScaleY ) const;

  MKI_INTERFACE_MEMBER double
  GetTextHeight( void ) const;

  MKI_INTERFACE_MEMBER double
  GetRotation( void ) const;

  MKI_INTERFACE_MEMBER double
  GetDistortion( void ) const;

  MKI_INTERFACE_MEMBER bool
  Separate( double& f_rdScaleX,   double& f_rdScaleY,
            double& f_rdRotation, double& f_rdDistortion,
            double& f_rdMoveX,    double& f_rdMoveY ) const;

  MKI_INTERFACE_MEMBER void
  Assemble( const double f_dScaleX,   const double f_dScaleY,
            const double f_dRotation, const double f_dDistortion,
            const double f_dMoveX,    const double f_dMoveY );

  MKI_INTERFACE_MEMBER bool
  SeparateText( double& f_rdHeight,   double& f_rdCompression,
                double& f_rdRotation, double& f_rdDistortion,
                double& f_rdMoveX,    double& f_rdMoveY ) const;

  MKI_INTERFACE_MEMBER void
  AssembleText( const double f_dHeight,   const double f_dCompression,
                const double f_dRotation, const double f_dDistortion,
                const double f_dMoveX,    const double f_dMoveY );

  MKI_INTERFACE_MEMBER bool
  IsStandard( void ) const;

  MKI_INTERFACE_MEMBER bool
  IsIdentical( const MKI_MATRIX& f_rMatrix ) const;
};

MKI_CLASS_TYPES( MATRIX )

//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_RECT : public MKI_RECT_BASE
{
  public:

  MKI_RECT( void ){};

  __forceinline
  MKI_RECT( const MKI_RECT* f_pThis )
  {
    *this = *f_pThis;
  }

  __forceinline
  MKI_RECT( const MKI_INTRECT& f_rRect )
  {
    x1 = f_rRect.left;
    y1 = f_rRect.top;
    x2 = f_rRect.right;
    y2 = f_rRect.bottom;
  }

  __forceinline
  MKI_RECT( const double f_dX1, const double f_dY1, const double f_dX2, const double f_dY2 )
  {
    x1 = f_dX1;
    y1 = f_dY1;
    x2 = f_dX2;
    y2 = f_dY2;
  }

  __forceinline
  MKI_RECT( const MKI_POINT_BASE& f_rPoint1, const MKI_POINT_BASE& f_rPoint2 )
  {
    x1 = f_rPoint1.x;
    y1 = f_rPoint1.y;
    x2 = f_rPoint2.x;
    y2 = f_rPoint2.y;
  }

  MKI_INTERFACE_MEMBER 
  MKI_RECT( const MKI_QUAD& f_rQuad );

  __forceinline
  operator MKI_POINT*()
  {
    return( (MKI_POINT*) this );
  }

  //------------------------------------------------------------------

  __forceinline void
  SetZero( void )
  {
    ZeroMemory( this, sizeof( *this ) );
  }

  __forceinline bool
  IsZero( void ) const
  {
    return( x1 == 0.0 &&
            y1 == 0.0 &&
            x2 == 0.0 &&
            y2 == 0.0 );
  }

  __forceinline bool
  IsNonZero( void ) const
  {
    return( x1 != 0.0 ||
            y1 != 0.0 ||
            x2 != 0.0 ||
            y2 != 0.0 );
  }

  __forceinline void
  SetInvalid( void )
  {
    x1 = y1 =  MKI_DOUBLE_NOVAL;
    x2 = y2 = -MKI_DOUBLE_NOVAL;
  }

  __forceinline void
  SetMaximum( void )
  {
    x1 = y1 = -MKI_DOUBLE_NOVAL;
    x2 = y2 =  MKI_DOUBLE_NOVAL;
  }

  __forceinline bool
  IsValid( void ) const
  {
    return( x1 != MKI_DOUBLE_NOVAL && x1 != -MKI_DOUBLE_NOVAL &&
            y1 != MKI_DOUBLE_NOVAL && y1 != -MKI_DOUBLE_NOVAL &&
            x2 != MKI_DOUBLE_NOVAL && x2 != -MKI_DOUBLE_NOVAL &&
            y2 != MKI_DOUBLE_NOVAL && y2 != -MKI_DOUBLE_NOVAL );
  }

  __forceinline bool
  IsInvalid( void ) const
  {
    return( x1 == MKI_DOUBLE_NOVAL || x1 == -MKI_DOUBLE_NOVAL ||
            y1 == MKI_DOUBLE_NOVAL || y1 == -MKI_DOUBLE_NOVAL ||
            x2 == MKI_DOUBLE_NOVAL || x2 == -MKI_DOUBLE_NOVAL ||
            y2 == MKI_DOUBLE_NOVAL || y2 == -MKI_DOUBLE_NOVAL );
  }

  __forceinline void
  Set( const double f_dX1, const double f_dY1, const double f_dX2, const double f_dY2 )
  {
    x1 = f_dX1;
    y1 = f_dY1;
    x2 = f_dX2;
    y2 = f_dY2;
  }

  __forceinline void
  Set( const MKI_POINT_BASE& f_rPoint1, const MKI_POINT_BASE& f_rPoint2 )
  {
    x1 = f_rPoint1.x;
    y1 = f_rPoint1.y;
    x2 = f_rPoint2.x;
    y2 = f_rPoint2.y;
  }

  __forceinline void
  Set( const MKI_POINT_BASE& f_rPoint )
  {
    x1 = x2 = f_rPoint.x;
    y1 = y2 = f_rPoint.y;
  }

  MKI_INTERFACE_MEMBER void
  SetMinMax( const MKI_RECT& f_rRect );

  MKI_INTERFACE_MEMBER void
  SetMinMax( const double f_dX1, const double f_dY1, const double f_dX2, const double f_dY2 );

  MKI_INTERFACE_MEMBER void
  SetMinMax( const MKI_POINT_BASE& f_rPoint1, const MKI_POINT_BASE& f_rPoint2 );

  MKI_INTERFACE_MEMBER void
  SetMinMax( const MKI_POINT_BASE* f_pPoint, __int32 f_nCount );

  MKI_INTERFACE_MEMBER void
  SetMinMax( MKI_CONST_POINTDATA_PTR f_pPoint, __int32 f_nCount );

  __forceinline void
  SetCombine( const MKI_RECT& f_rRect1, const MKI_RECT& f_rRect2 )
  {
    x1 = min( f_rRect1.x1, f_rRect2.x1 );
    y1 = min( f_rRect1.y1, f_rRect2.y1 );
    x2 = max( f_rRect1.x2, f_rRect2.x2 );
    y2 = max( f_rRect1.y2, f_rRect2.y2 );
  }

  __forceinline void
  SetIntersect( const MKI_RECT& f_rRect1, const MKI_RECT& f_rRect2 )
  {
    x1 = max( f_rRect1.x1, f_rRect2.x1 );
    y1 = max( f_rRect1.y1, f_rRect2.y1 );
    x2 = min( f_rRect1.x2, f_rRect2.x2 );
    y2 = min( f_rRect1.y2, f_rRect2.y2 );
  }

  //------------------------------------------------------------------

  __forceinline bool
  IsDisjoint( const MKI_RECT& f_rRect ) const
  {
    return( x1 > f_rRect.x2 ||
            y1 > f_rRect.y2 ||
            x2 < f_rRect.x1 ||
            y2 < f_rRect.y1 );
  }

  __forceinline bool
  IsOutside( const MKI_RECT& f_rRect ) const
  {
    return( x1 < f_rRect.x1 ||
            y1 < f_rRect.y1 ||
            x2 > f_rRect.x2 || 
            y2 > f_rRect.y2 );
  }

  __forceinline void
  Combine( const MKI_RECT& f_rRect )
  {
    if( f_rRect.x1 < x1 )
      x1 = f_rRect.x1;
    if( f_rRect.x2 > x2 )
      x2 = f_rRect.x2;
    if( f_rRect.y1 < y1 )
      y1 = f_rRect.y1;
    if( f_rRect.y2 > y2 )
      y2 = f_rRect.y2;
  }

  __forceinline void
  Intersect( const MKI_RECT& f_rRect )
  {
    if( f_rRect.x1 > x1 )
      x1 = f_rRect.x1;
    if( f_rRect.x2 < x2 )
      x2 = f_rRect.x2;
    if( f_rRect.y1 > y1 )
      y1 = f_rRect.y1;
    if( f_rRect.y2 < y2 )
      y2 = f_rRect.y2;
  }

  __forceinline void
  Include( const MKI_POINT_BASE& f_rPoint )
  {
    if( f_rPoint.x < x1 )
      x1 = f_rPoint.x;
    if( f_rPoint.x > x2 )
      x2 = f_rPoint.x;
    if( f_rPoint.y < y1 )
      y1 = f_rPoint.y;
    if( f_rPoint.y > y2 )
      y2 = f_rPoint.y;
  }

  __forceinline void
  Include( double d_fX, double f_dY )
  {
    if( d_fX < x1 )
      x1 = d_fX;
    if( d_fX > x2 )
      x2 = d_fX;
    if( f_dY < y1 )
      y1 = f_dY;
    if( f_dY > y2 )
      y2 = f_dY;
  }

  __forceinline bool
  Contains( const MKI_POINT& f_rPoint ) const
  {
    return( f_rPoint.x >= x1 &&
            f_rPoint.x <= x2 && 
            f_rPoint.y >= y1 && 
            f_rPoint.y <= y2 );
  }

  __forceinline void
  Expand( const double f_dAmount )
  {
    x1 -= f_dAmount;
    y1 -= f_dAmount;
    x2 += f_dAmount;
    y2 += f_dAmount;
  }

  __forceinline void
  Expand( const double f_dAmountX, const double f_dAmountY )
  {
    x1 -= f_dAmountX;
    y1 -= f_dAmountY;
    x2 += f_dAmountX;
    y2 += f_dAmountY;
  }

  __forceinline void
  Move( const MKI_VECTOR& f_rVector )
  {
    x1 += f_rVector.vx;
    y1 += f_rVector.vy;
    x2 += f_rVector.vx;
    y2 += f_rVector.vy;
  }

  __forceinline void
  Move( const MKI_MATRIX& f_rMatrix )
  {
    x1 += f_rMatrix.m31;
    y1 += f_rMatrix.m32;
    x2 += f_rMatrix.m31;
    y2 += f_rMatrix.m32;
  }

  MKI_INTERFACE_MEMBER void
  Multiply( const MKI_MATRIX& f_rMatrix );

  MKI_INTERFACE_MEMBER void
  SetMultiply( const MKI_MATRIX& f_rMatrix, const MKI_RECT& f_rRect );

  __forceinline double
  GetMinX( void ) const
  {
    return( min( x1, x2 ) );
  }

  __forceinline double
  GetMinY( void ) const
  {
    return( min( y1, y2 ) );
  }

  __forceinline double
  GetMaxX( void ) const
  {
    return( max( x1, x2 ) );
  }

  __forceinline double
  GetMaxY( void ) const
  {
    return( max( y1, y2 ) );
  }

  __forceinline double
  GetAbsSizeX( void ) const
  {
    return( mki_abs( x2 - x1 ) );
  }

  __forceinline double
  GetAbsSizeY( void ) const
  {
    return( mki_abs( y2 - y1 ) );
  }

  __forceinline void
  GetAbsSize( MKI_VECTOR_REF f_rSize ) const
  {
    f_rSize.vx = mki_abs( x2 - x1 );
    f_rSize.vy = mki_abs( y2 - y1 );
  }

  __forceinline double
  GetCenterX( void ) const
  {
    return( 0.5 * ( x1 + x2 ) );
  }

  __forceinline double
  GetCenterY( void ) const
  {
    return( 0.5 * ( y1 + y2 ) );
  }

  __forceinline void
  GetCenter( MKI_POINT_BASE& f_rCenter ) const
  {
    f_rCenter.x = 0.5 * ( x1 + x2 );
    f_rCenter.y = 0.5 * ( y1 + y2 );
  }

  __forceinline void
  GetCenter( MKI_VECTOR& f_rCenter ) const
  {
    f_rCenter.vx = 0.5 * ( x1 + x2 );
    f_rCenter.vy = 0.5 * ( y1 + y2 );
  }

  __forceinline double
  GetArea( void ) const
  {
    return( mki_abs( ( x2 - x1 ) * ( y2 - y1 ) ) );
  }
};

MKI_CLASS_TYPES( RECT )

//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_LINE
{
  public:

  MKI_LINE( void ){};

  __forceinline
  MKI_LINE( const MKI_LINE* f_pThis )
  {
    *this = *f_pThis;
  }

  __forceinline
  MKI_LINE( const double f_dX1, const double f_dY1, const double f_dX2, const double f_dY2 )
  {
    m_cPoint1.x = f_dX1;
    m_cPoint1.y = f_dY1;
    m_cPoint2.x = f_dX2;
    m_cPoint2.y = f_dY2;
  }

  __forceinline
  MKI_LINE( const MKI_POINT& f_rPoint1, const MKI_POINT& f_rPoint2 )
  {
    m_cPoint1 = f_rPoint1;
    m_cPoint2 = f_rPoint2;
  }

  __forceinline
  operator MKI_POINT*()
  {
    return( (MKI_POINT*) this );
  }

  //------------------------------------------------------------------

  __forceinline void
  SetZero( void )
  {
    ZeroMemory( this, sizeof( *this ) );
  }

  __forceinline bool
  IsZero( void ) const
  {
    return( m_cPoint1.x == 0.0 &&
            m_cPoint1.y == 0.0 &&
            m_cPoint2.x == 0.0 &&
            m_cPoint2.y == 0.0 );
  }

  __forceinline bool
  IsNonZero( void ) const
  {
    return( m_cPoint1.x != 0.0 ||
            m_cPoint1.y != 0.0 ||
            m_cPoint2.x != 0.0 ||
            m_cPoint2.y != 0.0 );
  }

  __forceinline void
  Set( const double f_dX1, const double f_dY1, const double f_dX2, const double f_dY2 )
  {
    m_cPoint1.x = f_dX1;
    m_cPoint1.y = f_dY1;
    m_cPoint2.x = f_dX2;
    m_cPoint2.y = f_dY2;
  }

  __forceinline void
  Set( const MKI_POINT& f_rPoint1, const MKI_POINT& f_rPoint2 )
  {
    m_cPoint1 = f_rPoint1;
    m_cPoint2 = f_rPoint2;
  }

  __forceinline void
  Set( const MKI_POINT& f_rPoint, const MKI_VECTOR& f_rVector )
  {
    m_cPoint1 = f_rPoint;
    m_cPoint2.SetAdd( f_rPoint, f_rVector );
  }

  //------------------------------------------------------------------

  MKI_POINT m_cPoint1, 
            m_cPoint2;
};

MKI_CLASS_TYPES( LINE )

//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_QUAD
{
  public:

  MKI_QUAD( void ){};

  __forceinline
  MKI_QUAD( const MKI_QUAD* f_pThis )
  {
    *this = *f_pThis;
  }

  __forceinline
  MKI_QUAD( const double f_dX1, const double f_dY1, const double f_dX2, const double f_dY2, 
            const double f_dX3, const double f_dY3, const double f_dX4, const double f_dY4 )
  {
    m_cPoint1.x = f_dX1;
    m_cPoint1.y = f_dY1;
    m_cPoint2.x = f_dX2;
    m_cPoint2.y = f_dY2;
    m_cPoint3.x = f_dX3;
    m_cPoint3.y = f_dY3;
    m_cPoint4.x = f_dX4;
    m_cPoint4.y = f_dY4;
  }

  __forceinline
  MKI_QUAD( const MKI_POINT& f_rPoint1, const MKI_POINT& f_rPoint2,
            const MKI_POINT& f_rPoint3, const MKI_POINT& f_rPoint4 )
  {
    m_cPoint1 = f_rPoint1;
    m_cPoint2 = f_rPoint2;
    m_cPoint3 = f_rPoint3;
    m_cPoint4 = f_rPoint4;
  }

  __forceinline
  MKI_QUAD( const double f_dX1, const double f_dY1, const double f_dX2, const double f_dY2 )
  {
    m_cPoint1.x = m_cPoint2.x = f_dX1;
    m_cPoint1.y = m_cPoint4.y = f_dY1;
    m_cPoint2.y = m_cPoint3.y = f_dY2;
    m_cPoint3.x = m_cPoint4.x = f_dX2;
  }

  __forceinline
  MKI_QUAD( const MKI_RECT& f_rRect )
  {
    m_cPoint1.x = m_cPoint2.x = f_rRect.x1;
    m_cPoint1.y = m_cPoint4.y = f_rRect.y1;
    m_cPoint3.x = m_cPoint4.x = f_rRect.x2;
    m_cPoint2.y = m_cPoint3.y = f_rRect.y2;
  }

  __forceinline
  operator MKI_POINT*()
  {
    return( (MKI_POINT*) this );
  }

  //------------------------------------------------------------------

  __forceinline void
  SetZero( void )
  {
    ZeroMemory( this, sizeof( *this ) );
  }

  __forceinline bool
  IsZero( void ) const
  {
    return( m_cPoint1.x == 0.0 &&
            m_cPoint1.y == 0.0 &&
            m_cPoint2.x == 0.0 &&
            m_cPoint2.y == 0.0 &&
            m_cPoint3.x == 0.0 &&
            m_cPoint3.y == 0.0 &&
            m_cPoint4.x == 0.0 &&
            m_cPoint4.y == 0.0 );
  }

  __forceinline bool
  IsNonZero( void ) const
  {
    return( m_cPoint1.x != 0.0 ||
            m_cPoint1.y != 0.0 ||
            m_cPoint2.x != 0.0 ||
            m_cPoint2.y != 0.0 ||
            m_cPoint3.x != 0.0 ||
            m_cPoint3.y != 0.0 ||
            m_cPoint4.x != 0.0 ||
            m_cPoint4.y != 0.0 );
  }

  __forceinline void
  Set( const double f_dX1, const double f_dY1, const double f_dX2, const double f_dY2, 
       const double f_dX3, const double f_dY3, const double f_dX4, const double f_dY4 )
  {
    m_cPoint1.x = f_dX1;
    m_cPoint1.y = f_dY1;
    m_cPoint2.x = f_dX2;
    m_cPoint2.y = f_dY2;
    m_cPoint3.x = f_dX3;
    m_cPoint3.y = f_dY3;
    m_cPoint4.x = f_dX4;
    m_cPoint4.y = f_dY4;
  }

  __forceinline void
  Set( const MKI_POINT& f_rPoint1, const MKI_POINT& f_rPoint2,
       const MKI_POINT& f_rPoint3, const MKI_POINT& f_rPoint4 )
  {
    m_cPoint1 = f_rPoint1;
    m_cPoint2 = f_rPoint2;
    m_cPoint3 = f_rPoint3;
    m_cPoint4 = f_rPoint4;
  }

  __forceinline void
  Set( const double f_dX1, const double f_dY1, const double f_dX2, const double f_dY2 )
  {
    m_cPoint1.x = m_cPoint2.x = f_dX1;
    m_cPoint1.y = m_cPoint4.y = f_dY1;
    m_cPoint3.x = m_cPoint4.x = f_dX2;
    m_cPoint2.y = m_cPoint3.y = f_dY2;
  }

  __forceinline void
  Set( const MKI_RECT& f_rRect )
  {
    m_cPoint1.x = m_cPoint2.x = f_rRect.x1;
    m_cPoint1.y = m_cPoint4.y = f_rRect.y1;
    m_cPoint3.x = m_cPoint4.x = f_rRect.x2;
    m_cPoint2.y = m_cPoint3.y = f_rRect.y2;
  }

  MKI_INTERFACE_MEMBER void
  Move( const MKI_VECTOR& f_rVector );

  MKI_INTERFACE_MEMBER void
  Multiply( const MKI_MATRIX& f_rMatrix );

  MKI_INTERFACE_MEMBER void
  SetMultiply( const MKI_MATRIX& f_rMatrix, const MKI_RECT& f_rRect );

  MKI_INTERFACE_MEMBER void
  SetMultiply( const MKI_MATRIX& f_rMatrix, const MKI_QUAD& f_rQuad );

  //------------------------------------------------------------------

  MKI_POINT m_cPoint1, 
            m_cPoint2, 
            m_cPoint3, 
            m_cPoint4;
};

MKI_CLASS_TYPES( QUAD )

//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_UNITARC
{
  public:

  MKI_UNITARC( void ){};

  __forceinline
  MKI_UNITARC( const MKI_UNITARC* f_pThis )
  {
    *this = *f_pThis;
  }

  __forceinline
  MKI_UNITARC( const MKI_POINT& f_rPoint1, const MKI_POINT& f_rPoint2, 
               const double f_dAngle1, const double f_dAngle2 )
  {
    m_cPoint1 = f_rPoint1;
    m_cPoint2 = f_rPoint2;
    m_dAngle1 = f_dAngle1;
    m_dAngle2 = f_dAngle2;
  }

  //------------------------------------------------------------------

  __forceinline void
  Init( void )
  {
    m_cPoint1.x = m_cPoint1.y =
    m_cPoint2.x = m_cPoint2.y = 0.0;

    m_dAngle1 = 0.0;
    m_dAngle2 = MKI_MATH_05PI;
  }

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  SplitCenter( MKI_UNITARC& f_rResult1, MKI_UNITARC& f_rResult2 ) const;

  //------------------------------------------------------------------

  MKI_POINT m_cPoint1,
            m_cPoint2;
  double    m_dAngle1,
            m_dAngle2;
};

MKI_CLASS_TYPES( UNITARC )

//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_CIRCLEARC
{
  public:

  MKI_CIRCLEARC( void ){};

  __forceinline
  MKI_CIRCLEARC( const MKI_CIRCLEARC* f_pThis )
  {
    *this = *f_pThis;
  }

  __forceinline
  MKI_CIRCLEARC( const MKI_POINT& f_rCenter, const MKI_POINT& f_rPoint1, const MKI_POINT& f_rPoint2, 
                 const double f_dRadius, const double f_dAngle1, const double f_dAngle2 )
  {
    m_cCenter = f_rCenter;
    m_cPoint1 = f_rPoint1;
    m_cPoint2 = f_rPoint2;

    m_dRadius = f_dRadius;

    m_dAngle1 = f_dAngle1;
    m_dAngle2 = f_dAngle2;
  }

  //------------------------------------------------------------------

  __forceinline void
  Init( void )
  {
    m_cCenter.x = m_cCenter.y =
    m_cPoint1.x = m_cPoint1.y =
    m_cPoint2.x = m_cPoint2.y = 0.0;

    m_dRadius = 0.0;

    m_dAngle1 = 0.0;
    m_dAngle2 = MKI_MATH_05PI;
  }

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  SplitCenter( MKI_CIRCLEARC& f_rResult1, MKI_CIRCLEARC& f_rResult2 ) const;

  //------------------------------------------------------------------

  MKI_POINT m_cCenter,
            m_cPoint1,
            m_cPoint2;
  double    m_dRadius,
            m_dAngle1,
            m_dAngle2;
};

MKI_CLASS_TYPES( CIRCLEARC )

//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_ELLIPSEARC
{
  public:

  MKI_ELLIPSEARC( void ){};

  __forceinline
  MKI_ELLIPSEARC( const MKI_ELLIPSEARC* f_pThis )
  {
    *this = *f_pThis;
  }

  __forceinline
  MKI_ELLIPSEARC( const MKI_POINT& f_rCenter, const MKI_POINT& f_rPoint1, const MKI_POINT& f_rPoint2, 
                  const double f_dVectorX, const double f_dVectorY, const double f_dAngle1, const double f_dAngle2 )
  {
    m_cCenter = f_rCenter;
    m_cPoint1 = f_rPoint1;
    m_cPoint2 = f_rPoint2;

    m_cVector.vx = f_dVectorX;
    m_cVector.vy = f_dVectorY;

    m_dAngle1 = f_dAngle1;
    m_dAngle2 = f_dAngle2;
  }

  //------------------------------------------------------------------

  __forceinline void
  Init( void )
  {
    m_cCenter.x = m_cCenter.y =
    m_cPoint1.x = m_cPoint1.y =
    m_cPoint2.x = m_cPoint2.y = 0.0;

    m_cVector.vx = m_cVector.vy = 0.0;

    m_dAngle1 = 0.0;
    m_dAngle2 = MKI_MATH_05PI;
  }

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  SplitCenter( MKI_ELLIPSEARC& f_rResult1, MKI_ELLIPSEARC& f_rResult2 ) const;

  //------------------------------------------------------------------

  MKI_POINT   m_cCenter,
              m_cPoint1,
              m_cPoint2;
  MKI_VECTOR  m_cVector;
  double      m_dAngle1,
              m_dAngle2;
};

MKI_CLASS_TYPES( ELLIPSEARC )

//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_QUADRATIC_BEZIER
{
  public:

  MKI_QUADRATIC_BEZIER( void ){};

  __forceinline
  MKI_QUADRATIC_BEZIER( const MKI_QUADRATIC_BEZIER* f_pThis )
  {
    *this = *f_pThis;
  }

  MKI_INTERFACE_MEMBER
  MKI_QUADRATIC_BEZIER( const MKI_LINE& f_rData );

  __forceinline
  MKI_QUADRATIC_BEZIER( const double f_dX1, const double f_dY1,
                        const double f_dX2, const double f_dY2, 
                        const double f_dX3, const double f_dY3 )
  {
    m_cPoint0.x = f_dX1;
    m_cPoint0.y = f_dY1;
    m_cPoint1.x = f_dX2;
    m_cPoint1.y = f_dY2;
    m_cPoint2.x = f_dX3;
    m_cPoint2.y = f_dY3;

    m_dT1 = 0.0;
    m_dT2 = 1.0;
  }

  __forceinline
  MKI_QUADRATIC_BEZIER( const MKI_POINT& f_rPoint1, 
                        const MKI_POINT& f_rPoint2,
                        const MKI_POINT& f_rPoint3 )
  {
    m_cPoint0 = f_rPoint1;
    m_cPoint1 = f_rPoint2;
    m_cPoint2 = f_rPoint3;

    m_dT1 = 0.0;
    m_dT2 = 1.0;
  }

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  GetDerivative( MKI_LINE& f_rResult ) const;

  MKI_INTERFACE_MEMBER void
  GetExtremities( MKI_POINT& f_rPoint ) const;

  MKI_INTERFACE_MEMBER void
  GetPoint( const double f_dT, MKI_POINT_REF f_rResult ) const;

  MKI_INTERFACE_MEMBER void
  Multiply( const MKI_MATRIX& f_rFactor );

  MKI_INTERFACE_MEMBER void
  SetMultiply( const MKI_MATRIX& f_rFactor, MKI_QUADRATIC_BEZIER& f_rBezier );

  //------------------------------------------------------------------

  MKI_POINT m_cPoint0,
            m_cPoint1,
            m_cPoint2;
  double m_dT1,
         m_dT2;
};

MKI_CLASS_TYPES( QUADRATIC_BEZIER )

//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_CUBIC_BEZIER
{
  public:

  MKI_CUBIC_BEZIER( void ){};

  __forceinline
  MKI_CUBIC_BEZIER( const MKI_CUBIC_BEZIER* f_pThis )
  {
    *this = *f_pThis;
  }

  MKI_INTERFACE_MEMBER
  MKI_CUBIC_BEZIER( const MKI_QUADRATIC_BEZIER& f_fData );

  __forceinline
  MKI_CUBIC_BEZIER( const double f_dX1, const double f_dY1, 
                    const double f_dX2, const double f_dY2, 
                    const double f_dX3, const double f_dY3, 
                    const double f_dX4, const double f_dY4 )
  {
    m_cData0.m_cPoint.x = f_dX1;
    m_cData0.m_cPoint.y = f_dY1;
    m_cData1.m_cPoint.x = f_dX2;
    m_cData1.m_cPoint.y = f_dY2;
    m_cData2.m_cPoint.x = f_dX3;
    m_cData2.m_cPoint.y = f_dY3;
    m_cData3.m_cPoint.x = f_dX4;
    m_cData3.m_cPoint.y = f_dY4;

    m_cData0.m_nOutline = m_cData1.m_nOutline = m_cData2.m_nOutline = m_cData3.m_nOutline = 0;
    m_cData0.m_nHandle  = m_cData1.m_nHandle  = m_cData2.m_nHandle  = m_cData3.m_nHandle  = -1;

    m_dT1 = 0.0;
    m_dT2 = 1.0;
  }

  __forceinline
  MKI_CUBIC_BEZIER( const MKI_POINT& f_rPoint1, 
                    const MKI_POINT& f_rPoint2,
                    const MKI_POINT& f_rPoint3, 
                    const MKI_POINT& f_rPoint4 )
  {
    m_cData0.m_cPoint = f_rPoint1;
    m_cData1.m_cPoint = f_rPoint2;
    m_cData2.m_cPoint = f_rPoint3;
    m_cData3.m_cPoint = f_rPoint4;

    m_cData0.m_nOutline = m_cData1.m_nOutline = m_cData2.m_nOutline = m_cData3.m_nOutline = 0;
    m_cData0.m_nHandle  = m_cData1.m_nHandle  = m_cData2.m_nHandle  = m_cData3.m_nHandle  = -1;

    m_dT1 = 0.0;
    m_dT2 = 1.0;
  }

  __forceinline
  MKI_CUBIC_BEZIER( const MKI_POINTDATA& f_rPoint1, 
                    const MKI_POINTDATA& f_rPoint2,
                    const MKI_POINTDATA& f_rPoint3, 
                    const MKI_POINTDATA& f_rPoint4 )
  {
    m_cData0 = f_rPoint1;
    m_cData1 = f_rPoint2;
    m_cData2 = f_rPoint3;
    m_cData3 = f_rPoint4;
    m_dT1 = 0.0;
    m_dT2 = 1.0;
  }

  //------------------------------------------------------------------

  __forceinline void
  Init( void )
  {
    m_cData0.m_cPoint.x = m_cData0.m_cPoint.y =
    m_cData1.m_cPoint.x = m_cData1.m_cPoint.y =
    m_cData2.m_cPoint.x = m_cData2.m_cPoint.y =
    m_cData3.m_cPoint.x = m_cData3.m_cPoint.y = 0.0;

    m_cData0.m_nOutline = m_cData1.m_nOutline = m_cData2.m_nOutline = m_cData3.m_nOutline = 0;
    m_cData0.m_nHandle  = m_cData1.m_nHandle  = m_cData2.m_nHandle  = m_cData3.m_nHandle  = -1;

    m_dT1 = 0.0;
    m_dT2 = 1.0;
  }

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  GetDerivative( MKI_QUADRATIC_BEZIER& f_rResult ) const;

  MKI_INTERFACE_MEMBER void
  GetExtremities( MKI_RECT& f_rRect ) const;

  MKI_INTERFACE_MEMBER void
  SplitCenter( MKI_CUBIC_BEZIER& f_rResult1, MKI_CUBIC_BEZIER& f_rResult2 ) const;

  MKI_INTERFACE_MEMBER void
  SplitOnce( const double f_dT, MKI_CUBIC_BEZIER& f_rResult1, MKI_CUBIC_BEZIER& f_rResult2 ) const;

  MKI_INTERFACE_MEMBER void
  SplitTwice( const double f_dT1, const double f_dT2, MKI_CUBIC_BEZIER& f_rResult1, MKI_CUBIC_BEZIER& f_rResult2, MKI_CUBIC_BEZIER& f_rResult3 ) const;

  MKI_INTERFACE_MEMBER void
  GetPoint( const double f_dT, MKI_POINT_REF f_rResult ) const;

  MKI_INTERFACE_MEMBER void
  GetRect( MKI_RECT_REF f_rResult ) const;

  MKI_INTERFACE_MEMBER void
  GetRectX( MKI_RECT_REF f_rResult ) const;

  MKI_INTERFACE_MEMBER void
  GetRectY( MKI_RECT_REF f_rResult ) const;

  MKI_INTERFACE_MEMBER double
  GetT( MKI_CONST_POINT_REF f_rPoint ) const;

  MKI_INTERFACE_MEMBER double
  SquareTo( MKI_CONST_POINT_REF f_rPoint ) const;

  MKI_INTERFACE_MEMBER double
  LengthTo( MKI_CONST_POINT_REF f_rPoint ) const;

  MKI_INTERFACE_MEMBER void
  FlatConstantDeltaT( const size_t f_unLineNum, MKI_POINT_PTR f_pResults ) const;

  MKI_INTERFACE_MEMBER size_t
  FlatConstantLength( double f_dLength, const size_t f_unLineMax, MKI_POINT_PTR f_pResults ) const;

  MKI_INTERFACE_MEMBER double
  Length( const size_t f_unLineNum ) const;

  MKI_INTERFACE_MEMBER void
  Multiply( const MKI_MATRIX& f_rFactor );

  MKI_INTERFACE_MEMBER void
  SetMultiply( const MKI_MATRIX& f_rFactor, MKI_CUBIC_BEZIER& f_rBezier );

  //------------------------------------------------------------------

  MKI_POINTDATA m_cData0,
                m_cData1,
                m_cData2,
                m_cData3;
  double m_dT1,
         m_dT2;
};

MKI_CLASS_TYPES( CUBIC_BEZIER )

//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PATCH
{
  public:

  __int32   m_nPointCount,
            m_nInvisible;
  MKI_POINT m_acPoint[MKI_POINTS_PER_PATCH];
};

MKI_CLASS_TYPES( PATCH )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_ELEM_MODE_DRAWING    = 0x00000000, // Element of the current drawing
  MKI_ELEM_MODE_LIBRARY    = 0x10000000, // Element of a library
  MKI_ELEM_MODE_EXTERNAL   = 0x20000000, // Element of an external reference
  MKI_ELEM_MODE_COLLECTION = 0x40000000, // Element of a collection
  MKI_ELEM_MODE_INVALID    = 0x80000000, // Invalid / undefined element
  MKI_ELEM_MODE_MASK       = 0xf0000000, // Mask for element type

  MKI_ELEM_INDEX_MASK      = 0x0000ffff, // Original element index in the library or external reference (0..10000)
  MKI_ELEM_ENTRY_MASK      = 0x0fff0000, // Number of library or external reference (0..1000)
};

__forceinline __int32
mki_elem_mode( const __int32 f_nValue )
{
  return( f_nValue & MKI_ELEM_MODE_MASK );
}

__forceinline __int32
mki_elem_index( const __int32 f_nValue )
{
  return( f_nValue & MKI_ELEM_INDEX_MASK );
}

__forceinline __int32
mki_elem_entry( const __int32 f_nValue )
{
  return( ( f_nValue & MKI_ELEM_ENTRY_MASK ) >> 16 );
}

__forceinline __int32
mki_elem_set_entry( const __int32 f_nValue )
{
  return( ( f_nValue << 16 ) & MKI_ELEM_ENTRY_MASK );
}

#endif

//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_MENUMODE_BASE       = 0x0000ffff, // Bits containing the command ID
  MKI_MENUMODE_FLAG       = 0x00ff0000, // Bits containing the commang flags

  MKI_MENUMODE_NULL       = 0x00000000, // Flag for non-functional entry
  MKI_MENUMODE_LABEL      = 0x00010000, // Flag for a main menu label
  MKI_MENUMODE_ENTRY      = 0x00020000, // Flag for a main menu entry
  MKI_MENUMODE_POPUPLABEL = 0x00040000, // Flag for a submenu label
  MKI_MENUMODE_POPUPENTRY = 0x00080000, // Flag for a submenu entry

  MKI_MENUMODE_PLUGIN     = 0x02000000, // Flag for a plug-in insertion point
  MKI_MENUMODE_CUSTOM     = 0x04000000, // Flag for a custom menu or menu entry
  MKI_MENUMODE_SEPARATOR  = 0x08000000, // Flag for separating line
  MKI_MENUMODE_END        = 0x80000000, // Flag for end-of-list
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
// The command IDs must be defined as #define instead of enum{ } or constexpr in order to be usable in a RC file

#define MKI_MENU_FIRST_ENTRY            100

#define MKI_MENU_FILE_FIRST             100
#define MKI_MENU_FILE_NEW                     100
#define MKI_MENU_FILE_OPEN                    101
#define MKI_MENU_FILE_SAVE                    102
#define MKI_MENU_FILE_SAVE_AS                 103
#define MKI_MENU_FILE_LIST                    104
#define MKI_MENU_FILE_IMPORT_SUB              105
#define MKI_MENU_FILE_IMPORT_MKD              106
#define MKI_MENU_FILE_IMPORT_EMF              107
#define MKI_MENU_FILE_IMPORT_WMF              108
#define MKI_MENU_FILE_IMPORT_BMP_FILE_POINT   109
#define MKI_MENU_FILE_IMPORT_BMP_UPDATE       110
#define MKI_MENU_FILE_EXPORT_SUB              111
#define MKI_MENU_FILE_EXPORT_MKD              112
#define MKI_MENU_FILE_EXPORT_EMF              113
#define MKI_MENU_FILE_EXPORT_WMF              114
#define MKI_MENU_FILE_EXPORT_BMP              115
#define MKI_MENU_FILE_EXPORT_BMP_RECT         116
#define MKI_MENU_FILE_EXTRA_SUB               117
#define MKI_MENU_FILE_EXTRA_ERASE             118
#define MKI_MENU_FILE_BACKUP_SAVE_DUP         119
#define MKI_MENU_FILE_BACKUP_LOAD_DUP         120
#define MKI_MENU_FILE_BACKUP_ARCHIVE          121
#define MKI_MENU_FILE_BACKUP_SETUP            122
#define MKI_MENU_FILE_PRINT                   123
#define MKI_MENU_FILE_PRINT_AREA_RECT         124
#define MKI_MENU_FILE_EXIT                    125
#define MKI_MENU_FILE_IMPORT_BMP_EMBED        126
#define MKI_MENU_FILE_BACKUP_ARCHIVE_SECURED  127
#define MKI_MENU_FILE_OPEN_NEW                128
#define MKI_MENU_FILE_REFERENCE_SUB           129
#define MKI_MENU_FILE_REFERENCE_INSERT        130
#define MKI_MENU_FILE_REFERENCE_UPDATE        131
#define MKI_MENU_FILE_EXTRA_SAMPLE            132
#define MKI_MENU_FILE_NEW_NEW                 133
#define MKI_MENU_FILE_BACKUP_MANUALLY         134
#define MKI_MENU_FILE_PDF_TASK                135
#define MKI_MENU_FILE_EXPORT_BMP_TASK         136
#define MKI_MENU_FILE_TASK_BMP_RECT           137
#define MKI_MENU_FILE_IMPORT_BMP_URL_POINT    138
#define MKI_MENU_FILE_BACKUP_MANAGE_LIST      139
#define MKI_MENU_FILE_PRINT_SUB               140
#define MKI_MENU_FILE_PRINT_AREA_POLYGON      141
#define MKI_MENU_FILE_EXPORT_BMP_POLYGON      142
#define MKI_MENU_FILE_TASK_BMP_POLYGON        143
#define MKI_MENU_FILE_EXTRA_CLEANUP           144
#define MKI_MENU_FILE_EXPORT_PLG_TASK         145
#define MKI_MENU_FILE_TASK_PLG_RECT           146
#define MKI_MENU_FILE_TASK_PLG_POLYGON        147
#define MKI_MENU_FILE_PRINT_TASK              148
#define MKI_MENU_FILE_REFERENCE_OPEN_IN_NEW   149
#define MKI_MENU_FILE_BACKUP_SUB              150
#define MKI_MENU_FILE_SAVE_AS_SUB             151
#define MKI_MENU_FILE_SAVE_AS_EMF             152
#define MKI_MENU_FILE_SAVE_AS_WMF             153
#define MKI_MENU_FILE_SAVE_AS_BMP             154
#define MKI_MENU_MANAGE_MAPPING_ALL_PAGES     155
#define MKI_MENU_MANAGE_MAPPING_MOVE          156
#define MKI_MENU_MANAGE_SYSTEM_ROTATE_RESET   157
#define MKI_MENU_MANAGE_SYSTEM_ROTATE_PLUS90  158
#define MKI_MENU_MANAGE_SYSTEM_ROTATE_180     159
#define MKI_MENU_MANAGE_SYSTEM_ROTATE_MINUS90 160
#define MKI_MENU_MANAGE_SYSTEM_ROTATE_EDGE    161
#define MKI_MENU_MANAGE_SYSTEM_ROTATE_PP      162
#define MKI_MENU_MODIFY_ROTATE_PLUS90         163
#define MKI_MENU_MODIFY_ROTATE_180            164
#define MKI_MENU_MODIFY_ROTATE_MINUS90        165
#define MKI_MENU_MODIFY_ROTATE_EDGE           166
#define MKI_MENU_MODIFY_ROTATE_PP             167
#define MKI_MENU_FILE_BACKUP_AUTO_ARCHIVE     168
#define MKI_MENU_MANAGE_ROTATE_SUB            169
#define MKI_MENU_MANAGE_ROTATE_ANGLE          170
#define MKI_MENU_MANAGE_ROTATE_0              171
#define MKI_MENU_MANAGE_ROTATE_PLUS15         172
#define MKI_MENU_MANAGE_ROTATE_PLUS90         173
#define MKI_MENU_MANAGE_ROTATE_180            174
#define MKI_MENU_MANAGE_ROTATE_MINUS90        175
#define MKI_MENU_MANAGE_ROTATE_MINUS15        176
#define MKI_MENU_MANAGE_ROTATE_EDGE           177
#define MKI_MENU_MANAGE_ROTATE_PP             178
#define MKI_MENU_MANAGE_ROTATE_RESET          179
#define MKI_MENU_MANAGE_MAPPING_SCALE         180
#define MKI_MENU_MANAGE_MAPPING_ROTATE        181
#define MKI_MENU_MANAGE_MAPPING_EDIT_START    182
#define MKI_MENU_MANAGE_MAPPING_EDIT_END      183
#define MKI_MENU_WINDOW_ZOOM                  184
#define MKI_MENU_FILE_AUTO_CONVERTER          185
#define MKI_MENU_FILE_IMPORT_SVG              186
#define MKI_MENU_FILE_EXPORT_SVG              187
#define MKI_MENU_FILE_SAVE_AS_SVG             188
#define MKI_MENU_FILE_IMPORT_BMP_FILE_RECT    189
#define MKI_MENU_FILE_IMPORT_BMP_URL_RECT     190
#define MKI_MENU_FILE_LAST              191

#define MKI_MENU_EDIT_FIRST             200
#define MKI_MENU_EDIT_UNDO                    200
#define MKI_MENU_EDIT_REDO                    201
#define MKI_MENU_EDIT_UNDO_ERASE              202
#define MKI_MENU_EDIT_CUT                     203
#define MKI_MENU_EDIT_COPY                    204
#define MKI_MENU_EDIT_PASTE                   205
#define MKI_MENU_EDIT_COMMENT_SUB             206
#define MKI_MENU_EDIT_COMMENT_CREATE_P        207
#define MKI_MENU_EDIT_COMMENT_PARAM           208
#define MKI_MENU_EDIT_MKD_DETAILS             209
#define MKI_MENU_MANAGE_QUEUE                 210
#define MKI_MENU_EDIT_VARIABLES               211
#define MKI_MENU_MANAGE_XPEN_SUB              212
#define MKI_MENU_MANAGE_XPEN_SELECT           213
#define MKI_MENU_MANAGE_XPEN_EDIT             214
#define MKI_MENU_MANAGE_XPEN_ASSIGN           215
#define MKI_MENU_MANAGE_XPEN_DISABLE          216
#define MKI_MENU_MANAGE_XPEN_ENABLE           217
#define MKI_MENU_MANAGE_XPEN_LOCK             218
#define MKI_MENU_MANAGE_XPEN_MELT             219
#define MKI_MENU_MANAGE_XPEN_IGNORE           220
#define MKI_MENU_MANAGE_XPEN_USE              221
#define MKI_MENU_MANAGE_XPEN_SHADE            222
#define MKI_MENU_MANAGE_XPEN_LIGHT            223
#define MKI_MENU_MANAGE_XPEN_HIDE             224
#define MKI_MENU_MANAGE_XPEN_HIDE_EXCEPT      225
#define MKI_MENU_MANAGE_XPEN_CONCENTRATE      226
#define MKI_MENU_EDIT_COMMENT_CREATE_RECT     227
#define MKI_MENU_EDIT_COMMENT_CREATE_CIRCLE   228
#define MKI_MENU_EDIT_COMMENT_DISPLAY         229
#define MKI_MENU_EDIT_COMMENT_FOREGROUND      230
#define MKI_MENU_EDIT_COMMENT_DELETE_ALL      231
#define MKI_MENU_EDIT_COMMENT_IMPORT          232
#define MKI_MENU_EDIT_COMMENT_EXPORT          233
#define MKI_MENU_EDIT_FIND_TEXT               234
#define MKI_MENU_EDIT_FIND_PROPERTY           235
#define MKI_MENU_EDIT_HYPERLINK_SUB           236
#define MKI_MENU_EDIT_HYPERLINK_CREATE        237
#define MKI_MENU_EDIT_HYPERLINK_RESOLVE       238
#define MKI_MENU_MANAGE_PAGELAYER_SUB         239
#define MKI_MENU_MANAGE_PAGELAYER_SHOW        240
#define MKI_MENU_MANAGE_PAGELAYER_GRAY        241
#define MKI_MENU_MANAGE_PAGELAYER_HIDE        242
#define MKI_MENU_MANAGE_PAGEDISPLAY_SUB       243
#define MKI_MENU_MANAGE_PAGEDISPLAY_ALL       244
#define MKI_MENU_MANAGE_PAGEDISPLAY_PAGES     245
#define MKI_MENU_MANAGE_PAGEDISPLAY_ACTIVE    246
#define MKI_MENU_MANAGE_PAGEDISPLAY_BOTH      247
#define MKI_MENU_MANAGE_PAGEDISPLAY_MODEL     248
#define MKI_MENU_MANAGE_PAGEDISPLAY_GRAY      249
#define MKI_MENU_MANAGE_PAGE_MODEL            250
#define MKI_MENU_MANAGE_PAGE_MAP_TO_MODEL     251
#define MKI_MENU_MANAGE_PAGE_RECENT           252
#define MKI_MENU_EDIT_CUT_X                   253
#define MKI_MENU_EDIT_COPY_X                  254
#define MKI_MENU_EDIT_PASTE_X                 255
#define MKI_MENU_GEO_SUB                      256
#define MKI_MENU_MANAGE_HIDE_SUB              257
#define MKI_MENU_MANAGE_HIDE_OBJ              258
#define MKI_MENU_MANAGE_HIDE_EXCEPT           259
#define MKI_MENU_MANAGE_HIDE_UNHIDE           260
#define MKI_MENU_DRAW_CIRCLE_RECT             261
#define MKI_MENU_DRAW_ELLIPSE_RECT            262
#define MKI_MENU_GEO_CIRCLE_RECT              263
#define MKI_MENU_GEO_ELLIPSE_RECT             264
#define MKI_MENU_MANAGE_TASKS                 265
#define MKI_MENU_GEO_FOREGROUND               266
#define MKI_MENU_GEO_TEMP                     267
#define MKI_MENU_TEMP_LINE_STANDARD           268
#define MKI_MENU_TEMP_LINE_LINE               269
#define MKI_MENU_TEMP_LINE_SUB                270
#define MKI_MENU_TEMP_ENDLESS_STANDARD        271
#define MKI_MENU_TEMP_ENDLESS_LINE            272
#define MKI_MENU_TEMP_ENDLESS_H               273
#define MKI_MENU_TEMP_ENDLESS_V               274
#define MKI_MENU_TEMP_ENDLESS_SUB             275
#define MKI_MENU_TEMP_CIRCLE_STANDARD         276
#define MKI_MENU_TEMP_CIRCLE_CIRCLE           277
#define MKI_MENU_TEMP_CIRCLE_SUB              278
#define MKI_MENU_TEMP_ARC_STANDARD            279
#define MKI_MENU_TEMP_ARC_CIRCLE              280
#define MKI_MENU_TEMP_ARC_SUB                 281
#define MKI_MENU_TEMP_ELLIPSE_SUB             282
#define MKI_MENU_TEMP_EARC_SUB                283
#define MKI_MENU_TEMP_OPTIONS_SUB             284
#define MKI_MENU_EDIT_OPTIONS_EXCHANGE        285
#define MKI_MENU_EDIT_UNDOLIST                286
#define MKI_MENU_EDIT_HYPERLINK_EXECUTE       287
#define MKI_MENU_EDIT_COMMENT_CREATE_POLY     288
#define MKI_MENU_EDIT_COMMENT_LIST            289
#define MKI_MENU_MANAGE_LAYER_ALL_DISPLAY     290
#define MKI_MENU_MANAGE_LAYER_ALL_OUTPUT      291
#define MKI_MENU_MANAGE_LAYER_ALL_ACTIVATE    292
#define MKI_MENU_MANAGE_PEN_ALL_DISPLAY       293
#define MKI_MENU_MANAGE_PEN_ALL_OUTPUT        294
#define MKI_MENU_MANAGE_PEN_ALL_ACTIVATE      295
#define MKI_MENU_EDIT_PINBOARD                296
#define MKI_MENU_EDIT_NOTES                   297
#define MKI_MENU_WINDOW_NOTES                 298
#define MKI_MENU_MANAGE_NOTIFICATIONS         299
#define MKI_MENU_EDIT_LAST              300

#define MKI_MENU_MANAGE_FIRST           300
#define MKI_MENU_MANAGE_PAGE_SUB              300
#define MKI_MENU_MANAGE_PAGE_NEW              301
#define MKI_MENU_MANAGE_PAGE_EDIT             302
#define MKI_MENU_MANAGE_PAGE_MOVE             303
#define MKI_MENU_WINDOW_UNDO                  304
#define MKI_MENU_WINDOW_LAYER                 305
#define MKI_MENU_WINDOW_PEN                   306
#define MKI_MENU_MANAGE_LAYER_SUB             307
#define MKI_MENU_MANAGE_MAPPING_TO_CLIP       308
#define MKI_MENU_MANAGE_LAYER_EDIT            309
#define MKI_MENU_MANAGE_LAYER_DEFAULT         310
#define MKI_MENU_MANAGE_LAYER_ALL_RESET       311
#define MKI_MENU_MANAGE_LAYER_UNDO            312
#define MKI_MENU_MANAGE_XLAYER_SUB            313
#define MKI_MENU_MANAGE_XLAYER_SELECT         314
#define MKI_MENU_MANAGE_XLAYER_EDIT           315
#define MKI_MENU_MANAGE_XLAYER_ASSIGN         316
#define MKI_MENU_MANAGE_XLAYER_DISABLE        317
#define MKI_MENU_MANAGE_XLAYER_ENABLE         318
#define MKI_MENU_MANAGE_XLAYER_LOCK           319
#define MKI_MENU_MANAGE_XLAYER_MELT           320
#define MKI_MENU_MANAGE_XLAYER_IGNORE         321
#define MKI_MENU_MANAGE_XLAYER_USE            322
#define MKI_MENU_MANAGE_XLAYER_SHADE          323
#define MKI_MENU_MANAGE_XLAYER_LIGHT          324
#define MKI_MENU_MANAGE_XLAYER_HIDE           325
#define MKI_MENU_MANAGE_XLAYER_HIDE_EXCEPT    326
#define MKI_MENU_MANAGE_XLAYER_CONCENTRATE    327
#define MKI_MENU_MANAGE_SYSTEM_SUB            328
//                                            329 - unused
#define MKI_MENU_MANAGE_SYSTEM_EDIT           330
#define MKI_MENU_MANAGE_SYSTEM_ORIGIN         331
#define MKI_MENU_MANAGE_SYSTEM_SHOW_DISPLAY   332
#define MKI_MENU_MANAGE_SYSTEM_EDIT_DISPLAY   333
#define MKI_MENU_MANAGE_SYSTEM_SHOW_POSITION  334
#define MKI_MENU_MANAGE_SYSTEM_EDIT_POSITION  335
#define MKI_MENU_MANAGE_PEN_SUB               336
//                                            337 - unused
#define MKI_MENU_MANAGE_PEN_EDIT              338
#define MKI_MENU_MANAGE_PEN_DEFAULT           339
#define MKI_MENU_MANAGE_PEN_LINE              340
#define MKI_MENU_MANAGE_ZOOM_SUB              341
#define MKI_MENU_MANAGE_ZOOM_AREA             342
#define MKI_MENU_MANAGE_ZOOM_OBJECTS          343
#define MKI_MENU_MANAGE_ZOOM_PAGE             344
#define MKI_MENU_MANAGE_ZOOM_ALL_PAGES        345
#define MKI_MENU_MANAGE_ZOOM_ORIGINAL         346
#define MKI_MENU_MANAGE_ZOOM_FACTOR           347
#define MKI_MENU_MANAGE_ZOOM_UNDO             348
#define MKI_MENU_MANAGE_ZOOM_PAN              349
#define MKI_MENU_MANAGE_ZOOM_PLUS             350
#define MKI_MENU_MANAGE_ZOOM_MINUS            351
#define MKI_MENU_MANAGE_SETTINGS_SUB          352
#define MKI_MENU_MANAGE_SETTINGS_SCREEN       353
#define MKI_MENU_MANAGE_SETTINGS_OUTPUT       354
#define MKI_MENU_MANAGE_SETTINGS_PRECISION    355
#define MKI_MENU_MANAGE_SETTINGS_COLORS       356
#define MKI_MENU_MANAGE_TEMPLATE_SUB          357
#define MKI_MENU_MANAGE_TEMPLATE_LOAD         358
#define MKI_MENU_MANAGE_TEMPLATE_SAVE_AS      359
#define MKI_MENU_MANAGE_TEMPLATE_SAVE_DEFAULT 360
#define MKI_MENU_EDIT_DRAWING_MANAGER         361
#define MKI_MENU_WINDOW_SYSTEM                362
#define MKI_MENU_WINDOW_ARRANGE               363
#define MKI_MENU_WINDOW_ENLARGE_2             364
#define MKI_MENU_WINDOW_ENLARGE_3             365
#define MKI_MENU_WINDOW_ENLARGE_4             366
#define MKI_MENU_WINDOW_VIEW                  367
#define MKI_MENU_WINDOW_PANEL                 368
#define MKI_MENU_EDIT_OPTIONS_RESET           369
#define MKI_MENU_MANAGE_PAGE_RESET            370
#define MKI_MENU_MANAGE_MAPPING_MODEL_PPP     371
#define MKI_MENU_WINDOW_TOOLBOX               372
#define MKI_MENU_WINDOW_COMMAND               373
#define MKI_MENU_WINDOW_BLOCK                 374
#define MKI_MENU_WINDOW_GUIDE                 375
#define MKI_MENU_EDIT_OPTIONS_SUB             376
#define MKI_MENU_EDIT_OPTIONS_WINDOW          377
#define MKI_MENU_EDIT_OPTIONS_GENERAL         378
#define MKI_MENU_EDIT_OPTIONS_PATHS           379
#define MKI_MENU_EDIT_OPTIONS_LOADSAVE        380
#define MKI_MENU_EDIT_OPTIONS_TIME            381
#define MKI_MENU_WINDOW_HATCH                 382
#define MKI_MENU_EDIT_OPTIONS_KEY             383
#define MKI_MENU_WINDOW_PAGE                  384
#define MKI_MENU_EDIT_OPTIONS_MOUSE           385
#define MKI_MENU_EDIT_OPTIONS_LOAD            386
#define MKI_MENU_EDIT_OPTIONS_SAVE_AS         387
#define MKI_MENU_MANAGE_PEN_ALL_RESET         388
#define MKI_MENU_MANAGE_PEN_UNDO              389
#define MKI_MENU_MANAGE_PAGE_NEXT             390
#define MKI_MENU_MANAGE_PAGE_PREV             391
#define MKI_MENU_MANAGE_PAGE_GOTO             392
#define MKI_MENU_FILE_EXTRA_REGISTER_EXT      393
#define MKI_MENU_EDIT_OPTIONS_SKIN            394
#define MKI_MENU_MANAGE_SETTINGS_RULES        395
#define MKI_MENU_MANAGE_SETTINGS_SERIAL       396
#define MKI_MENU_WINDOW_PLUGIN                397
#define MKI_MENU_MANAGE_TEMPLATE_IMPORT       398
#define MKI_MENU_WINDOW_LINE                  399
#define MKI_MENU_MANAGE_LAST            400

#define MKI_MENU_MODIFY_FIRST           400
#define MKI_MENU_MODIFY_PROPERTY_SUB          400
#define MKI_MENU_MODIFY_EDIT_TEXT             401
#define MKI_MENU_MODIFY_ERASE                 402
#define MKI_MENU_MODIFY_MODIFY                403
#define MKI_MENU_MODIFY_MOVE_SUB              404
#define MKI_MENU_MODIFY_MOVE_STANDARD         405
#define MKI_MENU_MODIFY_MOVE_PERP             406
#define MKI_MENU_MODIFY_MOVE_PARALLEL         407
#define MKI_MENU_MODIFY_MOVE_RELATIVE         408
#define MKI_MENU_MODIFY_SCALE_SUB             409
#define MKI_MENU_MODIFY_SCALE_FACTOR          410
#define MKI_MENU_MODIFY_SCALE_PROP            411
#define MKI_MENU_MODIFY_SCALE_ANY             412
#define MKI_MENU_MODIFY_SCALE_REFERENCE_NUM   413
#define MKI_MENU_MODIFY_ROTATE_SUB            414
#define MKI_MENU_MODIFY_ROTATE_CENTER         415
#define MKI_MENU_MODIFY_ROTATE_ANY            416
#define MKI_MENU_MODIFY_ROTATE_REFERENCE      417
#define MKI_MENU_MODIFY_MIRROR_SUB            418
#define MKI_MENU_MODIFY_MIRROR_H              419
#define MKI_MENU_MODIFY_MIRROR_V              420
#define MKI_MENU_MODIFY_MIRROR_LINE           421
#define MKI_MENU_MODIFY_DISTORT_SUB           422
#define MKI_MENU_MODIFY_DISTORT_H             423
#define MKI_MENU_MODIFY_DISTORT_V             424
#define MKI_MENU_MODIFY_ALIGN_SUB             425
#define MKI_MENU_MODIFY_ALIGN_C_PAGE_H        426
#define MKI_MENU_MODIFY_ALIGN_C_PAGE_V        427
#define MKI_MENU_MODIFY_ALIGN_C_PAGE_BOTH     428
#define MKI_MENU_MODIFY_ALIGN_C_RECT_H        429
#define MKI_MENU_MODIFY_ALIGN_C_RECT_V        430
#define MKI_MENU_MODIFY_ALIGN_C_RECT_BOTH     431
#define MKI_MENU_MODIFY_ALIGN_H_LEFT          432
#define MKI_MENU_MODIFY_ALIGN_H_CENTER        433
#define MKI_MENU_MODIFY_ALIGN_H_RIGHT         434
#define MKI_MENU_MODIFY_ALIGN_V_TOP           435
#define MKI_MENU_MODIFY_ALIGN_V_CENTER        436
#define MKI_MENU_MODIFY_ALIGN_V_BOTTOM        437
#define MKI_MENU_MODIFY_PMOVE_SUB             438
#define MKI_MENU_MODIFY_PMOVE_STANDARD        439
#define MKI_MENU_MODIFY_PMOVE_PERP            440
#define MKI_MENU_MODIFY_PMOVE_PARALLEL        441
#define MKI_MENU_MODIFY_PMOVE_RELATIVE        442
#define MKI_MENU_MODIFY_PMOVE_SINGLE          443
#define MKI_MENU_MODIFY_GROUP_SUB             444
#define MKI_MENU_MODIFY_GROUP_CREATE          445
#define MKI_MENU_MODIFY_GROUP_RESOLVE         446
#define MKI_MENU_MODIFY_ORDER_SUB             447
#define MKI_MENU_MODIFY_ORDER_BACK            448
#define MKI_MENU_MODIFY_ORDER_BEHIND          449
#define MKI_MENU_MODIFY_ORDER_BEFORE          450
#define MKI_MENU_MODIFY_ORDER_FRONT           451
#define MKI_MENU_MODIFY_PROPERTY_COPY         452
#define MKI_MENU_MODIFY_PROPERTY_DEFAULT      453
#define MKI_MENU_MODIFY_MIRROR_PP             454
#define MKI_MENU_MODIFY_PMOVE_ALIGN_SNAP      455
#define MKI_MENU_MODIFY_RESOLVE_EXT           456
#define MKI_MENU_MODIFY_SCALE_REFERENCE_PP    457
#define MKI_MENU_MODIFY_ROTATE_ALIGN_EDGES    458
#define MKI_MENU_MODIFY_FIT_IN                459
#define MKI_MENU_MODIFY_EDIT_INPLACE          460
#define MKI_MENU_MODIFY_PROTATE_ANY           461
#define MKI_MENU_MODIFY_GROUP_CREATE_FLAT     462
#define MKI_MENU_MODIFY_GROUP_RESOLVE_ALL     463
#define MKI_MENU_MODIFY_GROUP_ADD             464
#define MKI_MENU_MODIFY_GROUP_ADD_FLAT        465
#define MKI_MENU_MODIFY_GROUP_FROM_BLOCKS     466
#define MKI_MENU_MODIFY_PMOVE_REFERENCE_NUM   467
#define MKI_MENU_MODIFY_SCALE_P_FACTOR        468
#define MKI_MENU_MODIFY_PSCALE_SUB            469
#define MKI_MENU_MODIFY_PSCALE_LINEAR         470
#define MKI_MENU_MODIFY_PSCALE_RADIAL         471
#define MKI_MENU_MODIFY_SCALE_LINEAR          472
#define MKI_MENU_MODIFY_SCALE_RADIAL          473
#define MKI_MENU_GEO_BEAM_SUB                 474
#define MKI_MENU_GEO_BEAM_STANDARD            475
#define MKI_MENU_GEO_BEAM_LINE                476
#define MKI_MENU_GEO_BEAM_H                   477
#define MKI_MENU_GEO_BEAM_V                   478
#define MKI_MENU_GEO_BEAM_MIDPERP             479
#define MKI_MENU_GEO_BEAM_PERP                480
#define MKI_MENU_GEO_BEAM_PARALLEL            481
#define MKI_MENU_GEO_BEAM_PARALLEL_NUM        482
#define MKI_MENU_GEO_BEAM_ANGLE               483
#define MKI_MENU_GEO_BEAM_BISECTOR            484
#define MKI_MENU_GEO_BEAM_APPEND_TANGENT      485
#define MKI_MENU_GEO_BEAM_APPEND_PERP         486
#define MKI_MENU_MODIFY_PROPERTY_EDIT         487
#define MKI_MENU_MODIFY_PROPERTY_MANAGER      488
#define MKI_MENU_DRAW_HATCH_ALIGN             489
#define MKI_MENU_MODIFY_EDIT_SUB              490
#define MKI_MENU_MODIFY_EDIT_LOCAL_ATTRIB     491
#define MKI_MENU_MODIFY_EDIT_OUTLINE_ATTRIB   492
#define MKI_MENU_MODIFY_EDIT_POINT_ATTRIB     493
#define MKI_MENU_MODIFY_SCALE_LINEWIDTH       494
#define MKI_MENU_MODIFY_SCALE_TEXTSIZE        495
#define MKI_MENU_MODIFY_PROPERTY_RESOLVE      496
#define MKI_MENU_DRAW_HATCH_UPDATE            497
#define MKI_MENU_DRAW_HATCH_COPY_TO_OUTLINE   498
#define MKI_MENU_EDIT_OPTIONS_CROSS           499
#define MKI_MENU_MODIFY_LAST            500

#define MKI_MENU_DRAW_FIRST             500
#define MKI_MENU_DRAW_LINE_SUB                500
#define MKI_MENU_DRAW_LINE_STANDARD           501
#define MKI_MENU_DRAW_LINE_LINE               502
#define MKI_MENU_DRAW_LINE_H                  503
#define MKI_MENU_DRAW_LINE_V                  504
#define MKI_MENU_DRAW_LINE_MIDPERP            505
#define MKI_MENU_DRAW_LINE_PERP               506
#define MKI_MENU_DRAW_LINE_PARALLEL           507
#define MKI_MENU_DRAW_LINE_PARALLEL_NUM       508
#define MKI_MENU_DRAW_LINE_ANGLE              509
#define MKI_MENU_DRAW_LINE_BISECTOR           510
#define MKI_MENU_DRAW_LINE_CROSS              511
#define MKI_MENU_DRAW_LINE_TRACK              512
#define MKI_MENU_DRAW_LINE_DISTANT            513
#define MKI_MENU_DRAW_LINE_ZIGZAG             514
#define MKI_MENU_DRAW_TANGENT_SUB             515
#define MKI_MENU_DRAW_TANGENT_OP              516
#define MKI_MENU_DRAW_TANGENT_OPH             517
#define MKI_MENU_DRAW_TANGENT_OPV             518
#define MKI_MENU_DRAW_TANGENT_OPE             519
#define MKI_MENU_DRAW_TANGENT_OAE             520
#define MKI_MENU_DRAW_TANGENT_OO              521
#define MKI_MENU_DRAW_POLYGON_SUB             522
#define MKI_MENU_DRAW_POLYGON_TRIANGLE        523
#define MKI_MENU_DRAW_POLYGON_QUADRANGLE      524
#define MKI_MENU_DRAW_POLYGON_PARALLEL        525
#define MKI_MENU_DRAW_POLYGON_RECTANGLE       526
#define MKI_MENU_DRAW_POLYGON_TRACK           527
#define MKI_MENU_DRAW_POLYEDER_SUB            528
#define MKI_MENU_DRAW_POLYEDER_STANDARD       529
#define MKI_MENU_DRAW_POLYEDER_CIRCLE         530
#define MKI_MENU_DRAW_POLYEDER_CIRCUMCIRCLE   531
#define MKI_MENU_DRAW_POLYEDER_DIAMETER       532
#define MKI_MENU_DRAW_POLYEDER_SIDELENGTH     533
#define MKI_MENU_DRAW_POLYEDER_INNERRADIUS    534
#define MKI_MENU_DRAW_CIRCLE_SUB              535
#define MKI_MENU_DRAW_CIRCLE_STANDARD         536
#define MKI_MENU_DRAW_CIRCLE_CIRCLE           537
#define MKI_MENU_DRAW_CIRCLE_CIRCUMCIRCLE     538
#define MKI_MENU_DRAW_CIRCLE_DIAMETER         539
#define MKI_MENU_DRAW_CIRCLE_INCIRCLE         540
#define MKI_MENU_DRAW_CIRCLE_CONC             541
#define MKI_MENU_DRAW_CIRCLE_PERP             542
#define MKI_MENU_DRAW_CIRCLE_OOO              543
#define MKI_MENU_DRAW_CIRCLE_PPR              544
#define MKI_MENU_DRAW_CIRCLE_OPR              545
#define MKI_MENU_DRAW_CIRCLE_OOR              546
#define MKI_MENU_DRAW_ARC_SUB                 547
#define MKI_MENU_DRAW_ARC_STANDARD            548
#define MKI_MENU_DRAW_ARC_CIRCLE              549
#define MKI_MENU_DRAW_ARC_CIRCUMCIRCLE        550
#define MKI_MENU_DRAW_ARC_DIAMETER            551
#define MKI_MENU_DRAW_ARC_CONC_CIRCLE         552
#define MKI_MENU_DRAW_ARC_CONC_ARC            553
#define MKI_MENU_DRAW_ARC_PPR                 554
#define MKI_MENU_DRAW_ARC_OPR                 555
#define MKI_MENU_DRAW_ARC_OOR                 556
#define MKI_MENU_DRAW_ELLIPSE_SUB             557
#define MKI_MENU_DRAW_ELLIPSE_STANDARD        558
#define MKI_MENU_DRAW_ELLIPSE_ROTATED         559
#define MKI_MENU_DRAW_ELLIPSE_ANY             560
#define MKI_MENU_DRAW_ELLIPSE_ELLIPSE         561
#define MKI_MENU_DRAW_EARC_SUB                562
#define MKI_MENU_DRAW_EARC_STANDARD           563
#define MKI_MENU_DRAW_EARC_ANY                564
#define MKI_MENU_DRAW_EARC_ELLIPSE            565
#define MKI_MENU_DRAW_ARCMODE_SUB             566
#define MKI_MENU_DRAW_ARCMODE_ARC             567
#define MKI_MENU_DRAW_ARCMODE_SECTOR          568
#define MKI_MENU_DRAW_ARCMODE_SEGMENT         569
#define MKI_MENU_DRAW_ARCMODE_TOGGLE          570
#define MKI_MENU_DRAW_ARCMODE_DIRECTION       571
#define MKI_MENU_DRAW_FREEHAND                572
#define MKI_MENU_DRAW_SPLINE                  573
#define MKI_MENU_DRAW_LINE_CURVE              574
#define MKI_MENU_DRAW_POLYGON_SURFACE         575
#define MKI_MENU_DRAW_HATCH_SUB               576
#define MKI_MENU_DRAW_HATCH_OBJ               577
#define MKI_MENU_DRAW_HATCH_SURFACE           578
#define MKI_MENU_DRAW_GRAPH                   579
#define MKI_MENU_DRAW_HATCH_TYPE_EDIT         580
#define MKI_MENU_DRAW_HATCH_RESOLVE           581
#define MKI_MENU_DRAW_HATCH_ORIGIN            582
#define MKI_MENU_DRAW_LINE_APPEND_TANGENT     583
#define MKI_MENU_DRAW_LINE_APPEND_PERP        584
#define MKI_MENU_DRAW_ARC_APPEND_TANGENT      585
#define MKI_MENU_DRAW_ARC_APPEND_PERP         586
#define MKI_MENU_DRAW_BEZIER_SUB              587
#define MKI_MENU_DRAW_BEZIER_STANDARD         588
#define MKI_MENU_DRAW_BEZIER_APPEND_TANGENT   589
#define MKI_MENU_DRAW_BEZIER_APPEND_PERP      590
#define MKI_MENU_DRAW_CIRCLE_OPP              591
#define MKI_MENU_DRAW_CIRCLE_OOP              592
#define MKI_MENU_DRAW_POLYGON_MIDRECTANGLE    593
#define MKI_MENU_DRAW_POLYGON_ROTRECTANGLE    594
#define MKI_MENU_DRAW_POLYGON_RECTANGLE_PPP   595
#define MKI_MENU_DRAW_ARC_APPEND_TANGENT_NUM  596
#define MKI_MENU_DRAW_ARC_APPEND_PERP_NUM     597
#define MKI_MENU_DRAW_HATCH_EXTRACT_OUTLINE   598
#define MKI_MENU_DRAW_HATCH_REPLACE_OUTLINE   599
#define MKI_MENU_DRAW_LAST              600

#define MKI_MENU_GEO_FIRST              600
#define MKI_MENU_GEO_ENDLESS_SUB              600
#define MKI_MENU_GEO_ENDLESS_STANDARD         601
#define MKI_MENU_GEO_ENDLESS_LINE             602
#define MKI_MENU_GEO_ENDLESS_H                603
#define MKI_MENU_GEO_ENDLESS_V                604
#define MKI_MENU_GEO_ENDLESS_MIDPERP          605
#define MKI_MENU_GEO_ENDLESS_PERP             606
#define MKI_MENU_GEO_ENDLESS_PARALLEL         607
#define MKI_MENU_GEO_ENDLESS_PARALLEL_NUM     608
#define MKI_MENU_GEO_ENDLESS_ANGLE            609
#define MKI_MENU_GEO_ENDLESS_BISECTOR         610
#define MKI_MENU_GEO_ENDLESS_CROSS            611
#define MKI_MENU_GEO_TANGENT_SUB              612
#define MKI_MENU_GEO_TANGENT_OP               613
#define MKI_MENU_GEO_TANGENT_H                614
#define MKI_MENU_GEO_TANGENT_V                615
#define MKI_MENU_GEO_TANGENT_OA               616
#define MKI_MENU_GEO_TANGENT_OO               617
#define MKI_MENU_GEO_CIRCLE_SUB               618
#define MKI_MENU_GEO_CIRCLE_STANDARD          619
#define MKI_MENU_GEO_CIRCLE_CIRCLE            620
#define MKI_MENU_GEO_CIRCLE_CIRCUMCIRCLE      621
#define MKI_MENU_GEO_CIRCLE_DIAMETER          622
#define MKI_MENU_GEO_CIRCLE_INCIRCLE          623
#define MKI_MENU_GEO_CIRCLE_CONC              624
#define MKI_MENU_GEO_CIRCLE_PERP              625
#define MKI_MENU_GEO_CIRCLE_OOO               626
#define MKI_MENU_GEO_CIRCLE_PPR               627
#define MKI_MENU_GEO_CIRCLE_OPR               628
#define MKI_MENU_GEO_CIRCLE_OOR               629
#define MKI_MENU_GEO_ELLIPSE_SUB              630
#define MKI_MENU_GEO_ELLIPSE_STANDARD         631
#define MKI_MENU_GEO_ELLIPSE_ROTATED          632
#define MKI_MENU_GEO_ELLIPSE_ANY              633
#define MKI_MENU_GEO_ELLIPSE_ELLIPSE          634
#define MKI_MENU_GEO_DISPLAY                  635
#define MKI_MENU_GEO_LOCK                     636
#define MKI_MENU_GEO_BACKGROUND               637
#define MKI_MENU_GEO_MARK_SINGLE              638
#define MKI_MENU_GEO_MARK_SUB                 639
#define MKI_MENU_GEO_MARK_DISTANCE            640
#define MKI_MENU_GEO_MARK_OBJECT              641
#define MKI_MENU_GEO_MARK_ARRAY               642
#define MKI_MENU_MODIFY_COPY_RELATIVE         643
#define MKI_MENU_MODIFY_COPY_MARK             644
#define MKI_MENU_MODIFY_COPY_ROTATE           645
#define MKI_MENU_MODIFY_COPY_STEP             646
#define MKI_MENU_MODIFY_COPY_DISTANCE         647
#define MKI_MENU_MODIFY_COPY_OBJECT           648
#define MKI_MENU_MODIFY_COPY_ARRAY            649
#define MKI_MENU_GEO_DELETE                   650
#define MKI_MENU_GEO_ENDLESS_APPEND_TANGENT   651
#define MKI_MENU_GEO_ENDLESS_APPEND_PERP      652
#define MKI_MENU_GEO_DELETE_OBJ               653
#define MKI_MENU_GEO_CIRCLE_OPP               654
#define MKI_MENU_GEO_CIRCLE_OOP               655
#define MKI_MENU_GEO_IDLE                     656
#define MKI_MENU_TRIM_SURFACE_REMOVE          657
#define MKI_MENU_GEO_LINE_SUB                 658
#define MKI_MENU_GEO_LINE_STANDARD            659
#define MKI_MENU_GEO_LINE_LINE                660
#define MKI_MENU_GEO_LINE_H                   661
#define MKI_MENU_GEO_LINE_V                   662
#define MKI_MENU_GEO_LINE_MIDPERP             663
#define MKI_MENU_GEO_LINE_PERP                664
#define MKI_MENU_GEO_LINE_PARALLEL            665
#define MKI_MENU_GEO_LINE_PARALLEL_NUM        666
#define MKI_MENU_GEO_LINE_ANGLE               667
#define MKI_MENU_GEO_LINE_BISECTOR            668
#define MKI_MENU_GEO_LINE_CROSS               669
#define MKI_MENU_GEO_LINE_CURVE               670
#define MKI_MENU_GEO_LINE_APPEND_TANGENT      671
#define MKI_MENU_GEO_LINE_APPEND_PERP         672
#define MKI_MENU_GEO_POLYGON_SUB              673
#define MKI_MENU_GEO_POLYGON_TRIANGLE         674
#define MKI_MENU_GEO_POLYGON_QUADRANGLE       675
#define MKI_MENU_GEO_POLYGON_PARALLEL         676
#define MKI_MENU_GEO_POLYGON_RECTANGLE        677
#define MKI_MENU_GEO_POLYGON_SURFACE          678
#define MKI_MENU_GEO_POLYGON_MIDRECTANGLE     679
#define MKI_MENU_GEO_POLYGON_ROTRECTANGLE     680
#define MKI_MENU_GEO_POLYGON_RECTANGLE_PPP    681
#define MKI_MENU_GEO_ARC_SUB                  682
#define MKI_MENU_GEO_ARC_STANDARD             683
#define MKI_MENU_GEO_ARC_CIRCLE               684
#define MKI_MENU_GEO_ARC_CIRCUMCIRCLE         685
#define MKI_MENU_GEO_ARC_DIAMETER             686
#define MKI_MENU_GEO_ARC_CONC_CIRCLE          687
#define MKI_MENU_GEO_ARC_CONC_ARC             688
#define MKI_MENU_GEO_ARC_PPR                  689
#define MKI_MENU_GEO_ARC_OPR                  690
#define MKI_MENU_GEO_ARC_OOR                  691
#define MKI_MENU_GEO_ARC_APPEND_TANGENT       692
#define MKI_MENU_GEO_ARC_APPEND_PERP          693
#define MKI_MENU_GEO_ARC_APPEND_TANGENT_NUM   694
#define MKI_MENU_GEO_ARC_APPEND_PERP_NUM      695
#define MKI_MENU_GEO_EARC_SUB                 696
#define MKI_MENU_GEO_EARC_STANDARD            697
#define MKI_MENU_GEO_EARC_ANY                 698
#define MKI_MENU_GEO_EARC_ELLIPSE             699
#define MKI_MENU_GEO_LAST               700

#define MKI_MENU_TRIM_FIRST             700
#define MKI_MENU_TRIM_TRIM_SUB                700
#define MKI_MENU_TRIM_TRIM_CUTOUT             701
#define MKI_MENU_TRIM_TRIM_SPLIT              702
#define MKI_MENU_TRIM_TRIM_RESOLVE            703
#define MKI_MENU_TRIM_TRIM_LENGTH_P           704
#define MKI_MENU_TRIM_TRIM_LENGTH_OBJ         705
#define MKI_MENU_TRIM_TRIM_ANGLE_P            706
#define MKI_MENU_TRIM_TRIM_ANGLE_OBJ          707
#define MKI_MENU_TRIM_TRIM_ORIENTATION        708
#define MKI_MENU_TRIM_TRIM_MARGIN             709
#define MKI_MENU_TRIM_CURVE_EDIT              710
#define MKI_MENU_TRIM_CURVE_ASSEMBLE          711
#define MKI_MENU_TRIM_CONNECT_TANGENT         712
#define MKI_MENU_TRIM_SURFACE_SUB             713
#define MKI_MENU_TRIM_SURFACE_GENERATE        714
#define MKI_MENU_TRIM_SURFACE_OR              715
#define MKI_MENU_TRIM_SURFACE_AND             716
#define MKI_MENU_TRIM_SURFACE_MINUS           717
#define MKI_MENU_TRIM_SURFACE_COMBINE         718
#define MKI_MENU_TRIM_SURFACE_CUT             719
#define MKI_MENU_TRIM_SURFACE_START           720
#define MKI_MENU_TRIM_CHECK_DUPLICATE         721
#define MKI_MENU_TRIM_CHAMFER_SUB             722
#define MKI_MENU_TRIM_CHAMFER_OO              723
#define MKI_MENU_TRIM_CHAMFER_CORNER          724
#define MKI_MENU_TRIM_CHAMFER_ALL             725
#define MKI_MENU_TRIM_ROUND_SUB               726
#define MKI_MENU_TRIM_ROUND_OO                727
#define MKI_MENU_TRIM_ROUND_CORNER            728
#define MKI_MENU_TRIM_ROUND_ALL               729
#define MKI_MENU_TRIM_REMOVE_CORNER           730
#define MKI_MENU_TRIM_SURFACE_DETACH          731
#define MKI_MENU_TRIM_TRANSFORM_SPLINE        732
#define MKI_MENU_TRIM_CREATE_STRIPE           733
#define MKI_MENU_TRIM_TRANSFORM_SUB           734
#define MKI_MENU_TRIM_TRANSFORM_INVERT        735
#define MKI_MENU_TRIM_TRANSFORM_ZIGZAG        736
#define MKI_MENU_TRIM_TRANSFORM_CIRCLE        737
#define MKI_MENU_TRIM_TRANSFORM_ARC           738
#define MKI_MENU_TRIM_TRANSFORM_SECTOR        739
#define MKI_MENU_TRIM_TRANSFORM_SEGMENT       740
#define MKI_MENU_TRIM_TRANSFORM_ELLIPSE       741
#define MKI_MENU_TRIM_TRANSFORM_EARC          742
#define MKI_MENU_TRIM_TRANSFORM_ESECTOR       743
#define MKI_MENU_TRIM_TRANSFORM_ESEGMENT      744
#define MKI_MENU_TRIM_TRANSFORM_CURVE         745
#define MKI_MENU_TRIM_TRANSFORM_SURFACE       746
#define MKI_MENU_TRIM_CLIP_SUB                747
#define MKI_MENU_TRIM_CLIP_CREATE_INSIDE      748
#define MKI_MENU_TRIM_CLIP_CREATE_OUTSIDE     749
#define MKI_MENU_TRIM_CLIP_RESOLVE            750
#define MKI_MENU_TRIM_SURFACE_ASSEMBLE        751
#define MKI_MENU_TRIM_SURFACE_NO_OVERLAP      752
#define MKI_MENU_TRIM_CLIP_CREATE_BOTH        753
#define MKI_MENU_TRIM_CORRECT_SUB             754
#define MKI_MENU_TRIM_CORRECT_OO              755
#define MKI_MENU_TRIM_CORRECT_CORNER          756
#define MKI_MENU_TRIM_CORRECT_ALL             757
#define MKI_MENU_TRIM_TRIM_AUTOLENGTH         758
#define MKI_MENU_TRIM_TRIM_AUTOCUTOUT         759
#define MKI_MENU_TRIM_NOTCH_CORNER_SUB        760
#define MKI_MENU_TRIM_NOTCH_CORNER_OO         761
#define MKI_MENU_TRIM_NOTCH_CORNER_CORNER     762
#define MKI_MENU_TRIM_NOTCH_CORNER_ALL        763
#define MKI_MENU_TRIM_NOTCH_EDGE_SUB          764
#define MKI_MENU_TRIM_NOTCH_EDGE_P            765
#define MKI_MENU_TRIM_NOTCH_EDGE_NUM          766
#define MKI_MENU_TRIM_CURVE_SIMPLIFY          767
#define MKI_MENU_MANAGE_MAPPING_SUB           768
#define MKI_MENU_MANAGE_MAPPING_PAGE_RECT     769
#define MKI_MENU_MANAGE_MAPPING_PAGE_CIRCLE   770
#define MKI_MENU_MANAGE_MAPPING_PAGE_OBJ      771
#define MKI_MENU_MANAGE_MAPPING_MODEL_RECT    772
#define MKI_MENU_MANAGE_MAPPING_MODEL_CIRCLE  773
#define MKI_MENU_MANAGE_MAPPING_MODEL_OBJ     774
#define MKI_MENU_TRIM_CHECK_SUB               775
#define MKI_MENU_TRIM_CHECK_SEL               776
#define MKI_MENU_TRIM_CHECK_CLEAR             777
#define MKI_MENU_TRIM_TRANSFORM_GEO           778
#define MKI_MENU_TRIM_TRANSFORM_OBJ           779
#define MKI_MENU_TRIM_CLIP_EDIT_PROPERTY      780
#define MKI_MENU_TRIM_CLIP_EXTRACT_OUTLINE    781
#define MKI_MENU_TRIM_CLIP_REPLACE_OUTLINE    782
#define MKI_MENU_TRIM_EDIT_GEOMETRY           783
#define MKI_MENU_MANAGE_LAYER_FLAG1_DISPLAY   784
#define MKI_MENU_MANAGE_LAYER_FLAG2_DISPLAY   785
#define MKI_MENU_MANAGE_LAYER_FLAG3_DISPLAY   786
#define MKI_MENU_MANAGE_LAYER_FLAG4_DISPLAY   787
#define MKI_MENU_MANAGE_PEN_FLAG1_DISPLAY     788
#define MKI_MENU_MANAGE_PEN_FLAG2_DISPLAY     789
#define MKI_MENU_MANAGE_PEN_FLAG3_DISPLAY     790
#define MKI_MENU_MANAGE_PEN_FLAG4_DISPLAY     791
#define MKI_MENU_TRIM_SURFACE_SPLIT_SURFACE   792
#define MKI_MENU_TRIM_SURFACE_SPLIT_CURVE     793
#define MKI_MENU_DRAW_ARCMODE_POSITIVE        794
#define MKI_MENU_DRAW_ARCMODE_NEGATIVE        795
#define MKI_MENU_TRIM_CLIP_MOVE               796
#define MKI_MENU_TRIM_CLIP_SCALE              797
#define MKI_MENU_TRIM_CLIP_ROTATE             798
#define MKI_MENU_TRIM_TRIM_RADIUS             799
#define MKI_MENU_TRIM_LAST              800

#define MKI_MENU_TEXT_FIRST             800
#define MKI_MENU_TEXT_SUB1                    800
#define MKI_MENU_TEXT_STANDARD                801
#define MKI_MENU_TEXT_FRAME                   802
#define MKI_MENU_TEXT_REFERENCE               803
#define MKI_MENU_TEXT_TO_OBJECTS              804
#define MKI_MENU_TEXT_DIM_LINE_SUB            805
#define MKI_MENU_TEXT_DIM_LINE_LINE           806
#define MKI_MENU_TEXT_DIM_LINE_ARC            807
#define MKI_MENU_TEXT_DIM_SUB1                808
#define MKI_MENU_TEXT_DIM_LENGTH_P            809
#define MKI_MENU_TEXT_DIM_LENGTH_OBJ          810
#define MKI_MENU_TEXT_DIM_DISTANCE_P          811
#define MKI_MENU_TEXT_DIM_DISTANCE_OBJ        812
#define MKI_MENU_TEXT_DIM_RADIUS_P            813
#define MKI_MENU_TEXT_DIM_RADIUS_OBJ          814
#define MKI_MENU_TEXT_DIM_DIAMETER_P          815
#define MKI_MENU_TEXT_DIM_DIAMETER_OBJ        816
#define MKI_MENU_TEXT_DIM_ANGLE_P             817
#define MKI_MENU_TEXT_DIM_ANGLE_LL            818
#define MKI_MENU_TEXT_DIM_ANGLE_OBJ           819
#define MKI_MENU_TEXT_DIM_ARCLENGTH_P         820
#define MKI_MENU_TEXT_DIM_ARCLENGTH_OBJ       821
#define MKI_MENU_TEXT_DIM_COORDINATE          822
#define MKI_MENU_TEXT_DIM_SUB2                823
#define MKI_MENU_TEXT_DIM_AREA                824
#define MKI_MENU_TEXT_DIM_PERIMETER           825
#define MKI_MENU_TEXT_DIM_PARAMETER           826
#define MKI_MENU_TEXT_EDIT_LINE_SUB           827
#define MKI_MENU_TEXT_EDIT_LINE_ANGLE         828
#define MKI_MENU_TEXT_EDIT_LINE_POSITION      829
#define MKI_MENU_TEXT_EDIT_EXT_LINE           830
#define MKI_MENU_TEXT_EDIT_NUM_ANGLE          831
#define MKI_MENU_TEXT_EDIT_NUM_POSITION       832
#define MKI_MENU_TEXT_EDIT_NUM_UPDATE         833
#define MKI_MENU_TEXT_EDIT_NUM_TEXT           834
#define MKI_MENU_TEXT_CALC_SUB1               835
#define MKI_MENU_TEXT_CALC_POINT              836
#define MKI_MENU_TEXT_CALC_DISTANCE_PP        837
#define MKI_MENU_TEXT_CALC_AREA_PERIMETER     838
#define MKI_MENU_TEXT_MANAGE_FONTS            839
#define MKI_MENU_TEXT_CALC_DISTANCE_OP        840
#define MKI_MENU_TEXT_CALC_DISTANCE_OO        841
#define MKI_MENU_TEXT_CALC_RADIUS             842
#define MKI_MENU_TEXT_CALC_ANGLE              843
#define MKI_MENU_TEXT_SET_CENTROID            844
#define MKI_MENU_TEXT_STYLES                  845
#define MKI_MENU_TEXT_CALC_OBJECT             846
#define MKI_MENU_TEXT_TRANSFORM_STANDARD      847
#define MKI_MENU_TEXT_TRANSFORM_REFERENCE     848
#define MKI_MENU_TEXT_DIM_TEXT                849
#define MKI_MENU_TEXT_LINK_SET                850
#define MKI_MENU_TEXT_LINK_FIND               851
#define MKI_MENU_TEXT_LINK_ID                 852
#define MKI_MENU_TEXT_LINK_REMOVE             853
#define MKI_MENU_TEXT_CALC_SUB2               854
#define MKI_MENU_TEXT_CALC_TEXT_COORDINATE    855
#define MKI_MENU_TEXT_CALC_TEXT_AREA          856
#define MKI_MENU_TEXT_CALC_TEXT_PERIMETER     857
#define MKI_MENU_TEXT_LINKED_STANDARD         858
#define MKI_MENU_TEXT_LINKED_REFERENCE        859
#define MKI_MENU_TEXT_DIM_LINKED_TEXT         860
#define MKI_MENU_TEXT_ALIGN                   861
#define MKI_MENU_TEXT_RESOLVE_VARIABLES       862
#define MKI_MENU_TEXT_DIM_LINE_EARC           863
#define MKI_MENU_TRIM_CLIP_COPY_TO_OUTLINE    864
#define MKI_MENU_TEXT_LAST              865

#define MKI_MENU_LIBRARY_FIRST          900
#define MKI_MENU_LIBRARY_BLOCK_SUB            900
#define MKI_MENU_LIBRARY_BLOCK_INSERT         901
#define MKI_MENU_LIBRARY_BLOCK_CREATE         902
#define MKI_MENU_LIBRARY_BLOCK_CREATE_FRAME   903
#define MKI_MENU_LIBRARY_BLOCK_EDIT           904
#define MKI_MENU_LIBRARY_POSNO_SUB            905
#define MKI_MENU_LIBRARY_POSNO_ASSIGN         906
#define MKI_MENU_LIBRARY_POSNO_DELETE         907
#define MKI_MENU_LIBRARY_POSNO_REORDER        908
#define MKI_MENU_LIBRARY_PARTS_ALL            909
#define MKI_MENU_LIBRARY_CLEAN                910
#define MKI_MENU_LIBRARY_CONVERT              911
#define MKI_MENU_LIBRARY_BLOCK_RESOLVE        912
#define MKI_MENU_LIBRARY_REPLACE_BLOCK        913
#define MKI_MENU_LIBRARY_REPLACE_LIBRARY      914
#define MKI_MENU_LIBRARY_LIBRARY_LIST         915
#define MKI_MENU_LIBRARY_DATABASE_SUB         916
#define MKI_MENU_LIBRARY_DATABASE_LIST        917
#define MKI_MENU_LIBRARY_DATABASE_ADD_OBJ     918
#define MKI_MENU_LIBRARY_DATABASE_FIND_OBJ    919
#define MKI_MENU_LIBRARY_DATABASE_APPLY_SEL   920
#define MKI_MENU_LIBRARY_DATABASE_FIND        921
#define MKI_MENU_LIBRARY_DATABASE_UPDATE      922
#define MKI_MENU_LIBRARY_SAVE_ALL             923
#define MKI_MENU_LIBRARY_BLOCK_RESOLVE_ALL    924
#define MKI_MENU_LIBRARY_POSNO_CREATE         925
#define MKI_MENU_LIBRARY_PARTS_SUB            926
#define MKI_MENU_LIBRARY_PARTS_SEL            927
#define MKI_MENU_LIBRARY_BLOCK_ANONYMOUS      928
#define MKI_MENU_LIBRARY_BLOCK_ADD_TASK       929
#define MKI_MENU_LIBRARY_BLOCK_USE_TASK       930
#define MKI_MENU_LIBRARY_COLLECTION_LIST      931
#define MKI_MENU_LIBRARY_POSNO_OPTIONS        932
#define MKI_MENU_LIBRARY_PARTS_USE_TASK       933
#define MKI_MENU_LIBRARY_PARTS_TASK_ALL       934
#define MKI_MENU_LIBRARY_PARTS_TASK_RECT      935
#define MKI_MENU_LIBRARY_PARTS_TASK_POLY      936
#define MKI_MENU_LIBRARY_BLOCK_ALIGN          937
#define MKI_MENU_LIBRARY_BLOCK_INSTANCE_EDIT  938
#define MKI_MENU_LIBRARY_BLOCK_ADD_TASK_FRAME 939
#define MKI_MENU_LIBRARY_POSNO_EDIT           940
#define MKI_MENU_LIBRARY_LAST           941

#define MKI_MENU_EXTRA_FIRST            1000
#define MKI_MENU_EXTRA_ID_ALL                 1000
#define MKI_MENU_EXTRA_ID_PREVIOUS            1001
#define MKI_MENU_EXTRA_ID_SEL                 1002
#define MKI_MENU_EXTRA_SEL_SUB                1003
#define MKI_MENU_EXTRA_SEL_SET                1004
#define MKI_MENU_EXTRA_SEL_CLEAR              1005
#define MKI_MENU_EXTRA_SEL_INVERT             1006
#define MKI_MENU_EXTRA_ID_AREA_MODE           1007
#define MKI_MENU_EXTRA_COORD_ENTRY            1008
#define MKI_MENU_EXTRA_PREVIOUS_POINT         1009
#define MKI_MENU_EXTRA_EDIT_PARAMETERS        1010
#define MKI_MENU_EXTRA_DUPLICATE_TOGGLE       1011
#define MKI_MENU_EXTRA_ORTHO_TOGGLE           1012
#define MKI_MENU_EXTRA_ORTHO_PARAM            1013
#define MKI_MENU_EXTRA_SNAP_SUB               1014
#define MKI_MENU_EXTRA_SNAP_TOGGLE            1015
#define MKI_MENU_EXTRA_SNAP_CENTER            1016
#define MKI_MENU_EXTRA_SNAP_QUADRANT          1017
#define MKI_MENU_EXTRA_SNAP_EDGE              1018
#define MKI_MENU_EXTRA_SNAP_CORNER            1019
#define MKI_MENU_EXTRA_SNAP_INTERSECTION      1020
//                                            1021 - unused
#define MKI_MENU_EXTRA_ID_WINDOW              1022
#define MKI_MENU_EXTRA_SNAP_SPECIAL           1023
#define MKI_MENU_EXTRA_SNAP_RELATIVE          1024
#define MKI_MENU_EXTRA_ID_AREA_POLYGON        1025
#define MKI_MENU_EXTRA_SNAP_RADIUS            1026
#define MKI_MENU_EXTRA_KEY_SUB                1027
#define MKI_MENU_EXTRA_KEY_SPACE              1028
#define MKI_MENU_EXTRA_KEY_LBUTTON            1029
#define MKI_MENU_EXTRA_KEY_MBUTTON            1030
#define MKI_MENU_EXTRA_KEY_RBUTTON            1031
#define MKI_MENU_EXTRA_MENU_SUB               1032
#define MKI_MENU_EXTRA_KEY_LEFT               1033
#define MKI_MENU_EXTRA_KEY_RIGHT              1034
#define MKI_MENU_EXTRA_KEY_UP                 1035
#define MKI_MENU_EXTRA_KEY_DOWN               1036
#define MKI_MENU_EXTRA_KEY_HOME               1037
#define MKI_MENU_EXTRA_KEY_END                1038
#define MKI_MENU_EXTRA_KEY_PGUP               1039
#define MKI_MENU_EXTRA_KEY_PGDN               1040
#define MKI_MENU_EXTRA_MULTILINE_PARAM        1041
#define MKI_MENU_EXTRA_FILTER_TOGGLE          1042
#define MKI_MENU_EXTRA_FILTER_PARAM           1043
#define MKI_MENU_EXTRA_ID_SUB                 1044
#define MKI_MENU_EXTRA_ID_EXTENDED            1045
#define MKI_MENU_EXTRA_ID_PAGE                1046
#define MKI_MENU_EXTRA_ID_AREA_RECT           1047
#define MKI_MENU_EXTRA_ID_CANCEL              1048
#define MKI_MENU_EXTRA_FINISH_COMMAND         1049
#define MKI_MENU_EXTRA_CANCEL_COMMAND         1050
#define MKI_MENU_EXTRA_MENU_WORKFLOW          1051
#define MKI_MENU_EXTRA_MENU_POPUP             1052
#define MKI_MENU_EXTRA_MENU_TEXT              1053
#define MKI_MENU_EXTRA_DISPLAY_SUB            1054
#define MKI_MENU_EXTRA_DISPLAY_POINT          1055
#define MKI_MENU_EXTRA_DISPLAY_ORIENT         1056
#define MKI_MENU_EXTRA_DISPLAY_NUMBER         1057
#define MKI_MENU_EXTRA_MENU_CONTEXT           1058
#define MKI_MENU_EXTRA_ID_OVERRIDE_LOCK_IDLE  1059
#define MKI_MENU_EXTRA_FILTER_FROM_OBJECT     1060
#define MKI_MENU_EXTRA_DISPLAY_ATTRIB_L       1061
#define MKI_MENU_EXTRA_DISPLAY_ATTRIB_O       1062
#define MKI_MENU_EXTRA_DISPLAY_ATTRIB_P       1063
#define MKI_MENU_EXTRA_DISPLAY_ATTRIB_G       1064
#define MKI_MENU_EXTRA_LAST             1065

#define MKI_MENU_HELP_FIRST             1100
#define MKI_MENU_HELP_MANUAL                  1100
#define MKI_MENU_HELP_COMMAND                 1101
//                                            1102 - unused
#define MKI_MENU_HELP_VIDEO                   1103
#define MKI_MENU_HELP_REGISTER                1104
//                                            1105 - unused
#define MKI_MENU_HELP_ABOUT                   1106
#define MKI_MENU_HELP_ACTIVATE                1107
#define MKI_MENU_HELP_UPDATE                  1108
#define MKI_MENU_HELP_HOMEPAGE                1109
#define MKI_MENU_HELP_SUPPORT                 1110
#define MKI_MENU_HELP_SHOP                    1111
#define MKI_MENU_HELP_UNLOCK                  1112
#define MKI_MENU_HELP_WHATSNEW                1113
#define MKI_MENU_TEMP_LINE_H                  1114
#define MKI_MENU_TEMP_LINE_V                  1115
#define MKI_MENU_TEMP_LINE_MIDPERP            1116
#define MKI_MENU_TEMP_LINE_PERP               1117
#define MKI_MENU_TEMP_LINE_PARALLEL           1118
#define MKI_MENU_TEMP_LINE_PARALLEL_NUM       1119
#define MKI_MENU_TEMP_LINE_ANGLE              1120
#define MKI_MENU_TEMP_LINE_BISECTOR           1121
#define MKI_MENU_TEMP_LINE_APPEND_TANGENT     1122
#define MKI_MENU_TEMP_LINE_APPEND_PERP        1123
#define MKI_MENU_TEMP_LINE_TANGENT_OO         1124
#define MKI_MENU_TEMP_LINE_CROSS              1125
#define MKI_MENU_TEMP_ENDLESS_MIDPERP         1126
#define MKI_MENU_TEMP_ENDLESS_PERP            1127
#define MKI_MENU_TEMP_ENDLESS_PARALLEL        1128
#define MKI_MENU_TEMP_ENDLESS_PARALLEL_NUM    1129
#define MKI_MENU_TEMP_ENDLESS_ANGLE           1130
#define MKI_MENU_TEMP_ENDLESS_BISECTOR        1131
#define MKI_MENU_TEMP_ENDLESS_APPEND_TANGENT  1132
#define MKI_MENU_TEMP_ENDLESS_APPEND_PERP     1133
#define MKI_MENU_TEMP_ENDLESS_TANGENT_OP      1134
#define MKI_MENU_TEMP_ENDLESS_TANGENT_H       1135
#define MKI_MENU_TEMP_ENDLESS_TANGENT_V       1136
#define MKI_MENU_TEMP_ENDLESS_TANGENT_OO      1137
#define MKI_MENU_TEMP_ENDLESS_CROSS           1138
#define MKI_MENU_TEMP_CIRCLE_CIRCUMCIRCLE     1139
#define MKI_MENU_TEMP_CIRCLE_DIAMETER         1140
#define MKI_MENU_TEMP_CIRCLE_CONC             1141
#define MKI_MENU_TEMP_ARC_CIRCUMCIRCLE        1142
#define MKI_MENU_TEMP_ARC_DIAMETER            1143
#define MKI_MENU_TEMP_ARC_CONC_CIRCLE         1144
#define MKI_MENU_TEMP_ARC_CONC_ARC            1145
#define MKI_MENU_TEMP_ARC_APPEND_TANGENT      1146
#define MKI_MENU_TEMP_ARC_APPEND_PERP         1147
#define MKI_MENU_TEMP_ELLIPSE_STANDARD        1148
#define MKI_MENU_TEMP_ELLIPSE_ROTATED         1149
#define MKI_MENU_TEMP_ELLIPSE_ANY             1150
#define MKI_MENU_TEMP_ELLIPSE_ELLIPSE         1151
#define MKI_MENU_TEMP_ELLIPSE_RECT            1152
#define MKI_MENU_TEMP_EARC_STANDARD           1153
#define MKI_MENU_TEMP_EARC_ANY                1154
#define MKI_MENU_TEMP_EARC_ELLIPSE            1155
#define MKI_MENU_TEMP_OPTIONS_TO_GEO          1156
#define MKI_MENU_TEMP_OPTIONS_DELETE_ONE      1157
#define MKI_MENU_TEMP_OPTIONS_DELETE_ALL      1158
#define MKI_MENU_TEMP_OPTIONS_MODE_POINT      1159
#define MKI_MENU_TEMP_OPTIONS_MODE_COMMAND    1160
#define MKI_MENU_TEMP_OPTIONS_MODE_DRAWING    1161
#define MKI_MENU_TEMP_CIRCLE_RECT             1162
#define MKI_MENU_TEMP_CIRCLE_INCIRCLE         1163
#define MKI_MENU_HELP_LAST              1164

#define MKI_MENU_12_FIRST 1200
#define MKI_MENU_MANAGE_DIGITIZER_SUB         1200
#define MKI_MENU_MANAGE_DIGITIZER_BUTTON      1201
#define MKI_MENU_MANAGE_DIGITIZER_CALIBRATE   1202
#define MKI_MENU_MANAGE_DIGITIZER_ORIGIN      1203
#define MKI_MENU_MANAGE_DIGITIZER_LOAD        1204
#define MKI_MENU_MANAGE_DIGITIZER_SAVE_AS     1205
#define MKI_MENU_MANAGE_DIGITIZER_INFO        1206
#define MKI_MENU_MANAGE_DIGITIZER_OPTIONS     1207
#define MKI_MENU_MANAGE_DIGITIZER_EDIT        1208
#define MKI_MENU_12_LAST  1209

#define MKI_MENU_13_FIRST              1300
#define MKI_MENU_DRAW_EXT_SUB                 1300
#define MKI_MENU_DRAW_EXT_OVAL_C_C_R          1301
#define MKI_MENU_DRAW_EXT_OVAL_M_C_R          1302
#define MKI_MENU_DRAW_EXT_OVAL_E_E_C          1303
#define MKI_MENU_DRAW_EXT_OVAL_RECTANGLE      1304
#define MKI_MENU_DRAW_EXT_FREEHAND_DIRECT     1305
#define MKI_MENU_DRAW_EXT_BEZIER_POLYLINE     1306
#define MKI_MENU_DRAW_EXT_BEZIER_DEBOOR       1307
#define MKI_MENU_DRAW_EXT_BEZIER_TRANSFORM    1308
#define MKI_MENU_DRAW_EXT_BEZIER_SMOOTHEN     1309
#define MKI_MENU_DRAW_EXT_SPIRAL_STANDARD     1310
#define MKI_MENU_DRAW_EXT_ARROW_STRAIGHT      1311
#define MKI_MENU_TEXT_DIM_SUB3                1312
#define MKI_MENU_TEXT_DIM_INCREMENT_STANDARD  1313
#define MKI_MENU_TEXT_DIM_INCREMENT_ARCHITECT 1314
#define MKI_MENU_TEXT_DIM_REFERENCE_SINGLE    1315
#define MKI_MENU_TEXT_DIM_REFERENCE_MULTI     1316
#define MKI_MENU_TEXT_DIM_MOVE_POINT          1317
#define MKI_MENU_TEXT_DIM_ADD_POINT           1318
#define MKI_MENU_TEXT_DIM_REMOVE_POINT        1319
#define MKI_MENU_TEXT_DIM_EDIT_ALL            1320
#define MKI_MENU_TEXT_SUB2                    1321
#define MKI_MENU_TEXT_ISO1302                 1322
#define MKI_MENU_TEXT_ISO1101A_STANDARD       1323
#define MKI_MENU_TEXT_ISO1101A_REFERENCE      1324
#define MKI_MENU_TEXT_ISO1101B_STANDARD       1325
#define MKI_MENU_TEXT_ISO1101B_PERP           1326
#define MKI_MENU_TEXT_DIN6784                 1327
#define MKI_MENU_TEXT_DIN332T10               1328
#define MKI_MENU_TEXT_ON_PATH                 1329
#define MKI_MENU_DRAW_FILL_SUB                1330
#define MKI_MENU_DRAW_FILL_BLOCK_OBJECT       1331
#define MKI_MENU_DRAW_FILL_BLOCK_SURFACE      1332
#define MKI_MENU_DRAW_FILL_SET_ORIGIN         1333
#define MKI_MENU_DRAW_FILL_GRADIENT_OBJECT    1334
#define MKI_MENU_DRAW_FILL_GRADIENT_SURFACE   1335
#define MKI_MENU_DRAW_FILL_GRADIENT_OUTLINE   1336
#define MKI_MENU_DRAW_FILL_EXTRACT_OUTLINE    1337
#define MKI_MENU_DRAW_FILL_REPLACE_OUTLINE    1338
#define MKI_MENU_MODIFY_COLOR_SUB             1339
#define MKI_MENU_MODIFY_COLOR_LIGHTEN         1340
#define MKI_MENU_MODIFY_COLOR_DARKEN          1341
#define MKI_MENU_MODIFY_COLOR_TINT            1342
#define MKI_MENU_MODIFY_COLOR_MODEL           1343
#define MKI_MENU_MODIFY_COLOR_MATRIX_RGB      1344
#define MKI_MENU_MODIFY_COLOR_MATRIX_CMYK     1345
#define MKI_MENU_MODIFY_COLOR_MATRIX_GRAY     1346
#define MKI_MENU_MODIFY_COLOR_MATRIX_HSB      1347
#define MKI_MENU_MODIFY_COLOR_MATRIX_LAB      1348
#define MKI_MENU_MANAGE_ZOOM_PRESETS          1349
#define MKI_MENU_DRAW_FILL_COPY_TO_OUTLINE    1350
#define MKI_MENU_13_LAST               1351

#define MKI_MENU_EXTERN_FIRST           1400
#define MKI_MENU_EXTERN_PLUGIN_MIN            MKI_MENU_EXTERN_FIRST
#define MKI_MENU_EXTERN_PLUGIN_MAX            ( MKI_MENU_EXTERN_PLUGIN_MIN + MKI_PLUGIN_MAX * MKI_PLUGIN_MENU_MAX )
#define MKI_MENU_EXTERN_IMPORT_MIN            MKI_MENU_EXTERN_PLUGIN_MAX
#define MKI_MENU_EXTERN_IMPORT_MAX            ( MKI_MENU_EXTERN_IMPORT_MIN + MKI_FILTER_MAX )
#define MKI_MENU_EXTERN_EXPORT_MIN            MKI_MENU_EXTERN_IMPORT_MAX
#define MKI_MENU_EXTERN_EXPORT_MAX            ( MKI_MENU_EXTERN_EXPORT_MIN + MKI_FILTER_MAX )
#define MKI_MENU_EXTERN_SAVEAS_MIN            MKI_MENU_EXTERN_EXPORT_MAX
#define MKI_MENU_EXTERN_SAVEAS_MAX            ( MKI_MENU_EXTERN_SAVEAS_MIN + MKI_FILTER_MAX )
#define MKI_MENU_EXTERN_LAST            MKI_MENU_EXTERN_SAVEAS_MAX

#define MKI_MENU_LAST_ENTRY             MKI_MENU_EXTERN_LAST


//---------------------------------------------------------------------------------------------------------------------------------------------------
constexpr __int32 MKI_MENU_INTERN_FIRST = MKI_MENU_LAST_ENTRY;

constexpr __int32 MKI_MENU_INTERN_BLOCK_INSERT_MIN = MKI_MENU_INTERN_FIRST;
constexpr __int32 MKI_MENU_INTERN_BLOCK_INSERT_MAX = MKI_MENU_INTERN_BLOCK_INSERT_MIN + MKI_BLOCKLIST_MAX;
constexpr __int32 MKI_MENU_INTERN_BLOCK_EDIT_MIN   = MKI_MENU_INTERN_BLOCK_INSERT_MAX;
constexpr __int32 MKI_MENU_INTERN_BLOCK_EDIT_MAX   = MKI_MENU_INTERN_BLOCK_EDIT_MIN + MKI_BLOCKLIST_MAX;

constexpr __int32 MKI_MENU_INTERN_SYSTEM_SELECT_MIN = MKI_MENU_INTERN_BLOCK_EDIT_MAX;
constexpr __int32 MKI_MENU_INTERN_SYSTEM_SELECT_MAX = MKI_MENU_INTERN_SYSTEM_SELECT_MIN + MKI_SYSTEM_MAX + 1;
constexpr __int32 MKI_MENU_INTERN_SYSTEM_EDIT_MIN   = MKI_MENU_INTERN_SYSTEM_SELECT_MAX;
constexpr __int32 MKI_MENU_INTERN_SYSTEM_EDIT_MAX   = MKI_MENU_INTERN_SYSTEM_EDIT_MIN + MKI_SYSTEM_MAX + 1;

constexpr __int32 MKI_MENU_INTERN_HATCH_SELECT_MIN = MKI_MENU_INTERN_SYSTEM_EDIT_MAX;
constexpr __int32 MKI_MENU_INTERN_HATCH_SELECT_MAX = MKI_MENU_INTERN_HATCH_SELECT_MIN + MKI_HATCH_MAX + 1;
constexpr __int32 MKI_MENU_INTERN_HATCH_EDIT_MIN   = MKI_MENU_INTERN_HATCH_SELECT_MAX;
constexpr __int32 MKI_MENU_INTERN_HATCH_EDIT_MAX   = MKI_MENU_INTERN_HATCH_EDIT_MIN + MKI_HATCH_MAX + 1;

constexpr __int32 MKI_MENU_INTERN_LAYER_SELECT_MIN = MKI_MENU_INTERN_HATCH_EDIT_MAX;
constexpr __int32 MKI_MENU_INTERN_LAYER_SELECT_MAX = MKI_MENU_INTERN_LAYER_SELECT_MIN + MKI_LAYER_MAX + 1;
constexpr __int32 MKI_MENU_INTERN_LAYER_EDIT_MIN   = MKI_MENU_INTERN_LAYER_SELECT_MAX;
constexpr __int32 MKI_MENU_INTERN_LAYER_EDIT_MAX   = MKI_MENU_INTERN_LAYER_EDIT_MIN + MKI_LAYER_MAX + 1;

constexpr __int32 MKI_MENU_INTERN_PEN_SELECT_MIN = MKI_MENU_INTERN_LAYER_EDIT_MAX;
constexpr __int32 MKI_MENU_INTERN_PEN_SELECT_MAX = MKI_MENU_INTERN_PEN_SELECT_MIN + MKI_PEN_MAX + 1;
constexpr __int32 MKI_MENU_INTERN_PEN_EDIT_MIN   = MKI_MENU_INTERN_PEN_SELECT_MAX;
constexpr __int32 MKI_MENU_INTERN_PEN_EDIT_MAX   = MKI_MENU_INTERN_PEN_EDIT_MIN + MKI_PEN_MAX + 1;

constexpr __int32 MKI_MENU_INTERN_LINETYPE_SELECT_MIN     = MKI_MENU_INTERN_PEN_EDIT_MAX;
constexpr __int32 MKI_MENU_INTERN_LINETYPE_SELECT_MAX     = MKI_MENU_INTERN_LINETYPE_SELECT_MIN + MKI_LINE_MAX + 1;
constexpr __int32 MKI_MENU_INTERN_LINETYPE_SELECT_BYLAYER = MKI_MENU_INTERN_LINETYPE_SELECT_MAX;
constexpr __int32 MKI_MENU_INTERN_LINETYPE_EDIT_MIN       = MKI_MENU_INTERN_LINETYPE_SELECT_BYLAYER + 1;
constexpr __int32 MKI_MENU_INTERN_LINETYPE_EDIT_MAX       = MKI_MENU_INTERN_LINETYPE_EDIT_MIN + MKI_LINE_MAX + 1;

constexpr __int32 MKI_MENU_INTERN_PAGE_SELECT_MIN = MKI_MENU_INTERN_LINETYPE_EDIT_MAX;
constexpr __int32 MKI_MENU_INTERN_PAGE_SELECT_MAX = MKI_MENU_INTERN_PAGE_SELECT_MIN + MKI_PAGE_MAX + 1;
constexpr __int32 MKI_MENU_INTERN_PAGE_EDIT_MIN   = MKI_MENU_INTERN_PAGE_SELECT_MAX;
constexpr __int32 MKI_MENU_INTERN_PAGE_EDIT_MAX   = MKI_MENU_INTERN_PAGE_EDIT_MIN + MKI_PAGE_MAX + 1;

constexpr __int32 MKI_MENU_INTERN_LINEWIDTH_SELECT_MIN = MKI_MENU_INTERN_PAGE_EDIT_MAX;
constexpr __int32 MKI_MENU_INTERN_LINEWIDTH_SELECT_MAX = MKI_MENU_INTERN_LINEWIDTH_SELECT_MIN + MKI_INTERN_COLOR_MAX;
constexpr __int32 MKI_MENU_INTERN_LINEWIDTH_EDIT_MIN   = MKI_MENU_INTERN_LINEWIDTH_SELECT_MAX;
constexpr __int32 MKI_MENU_INTERN_LINEWIDTH_EDIT_MAX   = MKI_MENU_INTERN_LINEWIDTH_EDIT_MIN + MKI_INTERN_COLOR_MAX;

constexpr __int32 MKI_MENU_INTERN_LINECOLOR_SELECT_MIN = MKI_MENU_INTERN_LINEWIDTH_EDIT_MAX;
constexpr __int32 MKI_MENU_INTERN_LINECOLOR_SELECT_MAX = MKI_MENU_INTERN_LINECOLOR_SELECT_MIN + MKI_INTERN_COLOR_MAX;
constexpr __int32 MKI_MENU_INTERN_LINECOLOR_EDIT_MIN   = MKI_MENU_INTERN_LINECOLOR_SELECT_MAX;
constexpr __int32 MKI_MENU_INTERN_LINECOLOR_EDIT_MAX   = MKI_MENU_INTERN_LINECOLOR_EDIT_MIN + MKI_INTERN_COLOR_MAX;

constexpr __int32 MKI_MENU_INTERN_FILLCOLOR_SELECT_MIN = MKI_MENU_INTERN_LINECOLOR_EDIT_MAX;
constexpr __int32 MKI_MENU_INTERN_FILLCOLOR_SELECT_MAX = MKI_MENU_INTERN_FILLCOLOR_SELECT_MIN + MKI_INTERN_COLOR_MAX;
constexpr __int32 MKI_MENU_INTERN_FILLCOLOR_EDIT_MIN   = MKI_MENU_INTERN_FILLCOLOR_SELECT_MAX;
constexpr __int32 MKI_MENU_INTERN_FILLCOLOR_EDIT_MAX   = MKI_MENU_INTERN_FILLCOLOR_EDIT_MIN + MKI_INTERN_COLOR_MAX;

constexpr __int32 MKI_MENU_INTERN_FILLMODE_SELECT_MIN = MKI_MENU_INTERN_FILLCOLOR_EDIT_MAX;
constexpr __int32 MKI_MENU_INTERN_FILLMODE_SELECT_MAX = MKI_MENU_INTERN_FILLMODE_SELECT_MIN + MKI_INTERN_FILLMODE_MAX;

constexpr __int32 MKI_MENU_INTERN_CUSTOMMENU_MIN = MKI_MENU_INTERN_FILLMODE_SELECT_MAX;
constexpr __int32 MKI_MENU_INTERN_CUSTOMMENU_MAX = MKI_MENU_INTERN_CUSTOMMENU_MIN + MKI_CUSTOMMENU_MAX;

constexpr __int32 MKI_MENU_INTERN_TASK_NEW         = MKI_MENU_INTERN_CUSTOMMENU_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_IMPORT      = MKI_MENU_INTERN_CUSTOMMENU_MAX + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_EXPORT      = MKI_MENU_INTERN_CUSTOMMENU_MAX + 2;
constexpr __int32 MKI_MENU_INTERN_TASK_SELECT_MIN  = MKI_MENU_INTERN_CUSTOMMENU_MAX + 3;
constexpr __int32 MKI_MENU_INTERN_TASK_SELECT_MAX  = MKI_MENU_INTERN_TASK_SELECT_MIN + MKI_TASK_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_SELECT_LIST = MKI_MENU_INTERN_TASK_SELECT_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_EDIT_MIN    = MKI_MENU_INTERN_TASK_SELECT_LIST + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_EDIT_MAX    = MKI_MENU_INTERN_TASK_EDIT_MIN + MKI_TASK_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_EDIT_LIST   = MKI_MENU_INTERN_TASK_EDIT_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_EDIT_MULTI  = MKI_MENU_INTERN_TASK_EDIT_MAX + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_UPDATE_MIN  = MKI_MENU_INTERN_TASK_EDIT_MAX + 2;
constexpr __int32 MKI_MENU_INTERN_TASK_UPDATE_MAX  = MKI_MENU_INTERN_TASK_UPDATE_MIN + MKI_TASK_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_UPDATE_LIST = MKI_MENU_INTERN_TASK_UPDATE_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_DUP_MIN     = MKI_MENU_INTERN_TASK_UPDATE_LIST + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_DUP_MAX     = MKI_MENU_INTERN_TASK_DUP_MIN + MKI_TASK_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_DUP_LIST    = MKI_MENU_INTERN_TASK_DUP_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_MOVE_ALL    = MKI_MENU_INTERN_TASK_DUP_LIST + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_MOVE_MIN    = MKI_MENU_INTERN_TASK_MOVE_ALL + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_MOVE_MAX    = MKI_MENU_INTERN_TASK_MOVE_MIN + MKI_TASK_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_MOVE_LIST   = MKI_MENU_INTERN_TASK_MOVE_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_COPY_MIN    = MKI_MENU_INTERN_TASK_MOVE_LIST + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_COPY_MAX    = MKI_MENU_INTERN_TASK_COPY_MIN + MKI_TASK_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_COPY_LIST   = MKI_MENU_INTERN_TASK_COPY_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_RECT_MIN    = MKI_MENU_INTERN_TASK_COPY_LIST + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_RECT_MAX    = MKI_MENU_INTERN_TASK_RECT_MIN + MKI_TASK_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_RECT_LIST   = MKI_MENU_INTERN_TASK_RECT_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_SIZE_MIN    = MKI_MENU_INTERN_TASK_RECT_LIST + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_SIZE_MAX    = MKI_MENU_INTERN_TASK_SIZE_MIN + MKI_TASK_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_SIZE_LIST   = MKI_MENU_INTERN_TASK_SIZE_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_POLY_MIN    = MKI_MENU_INTERN_TASK_SIZE_LIST + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_POLY_MAX    = MKI_MENU_INTERN_TASK_POLY_MIN + MKI_TASK_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_POLY_LIST   = MKI_MENU_INTERN_TASK_POLY_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_POINT_MIN   = MKI_MENU_INTERN_TASK_POLY_LIST + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_POINT_MAX   = MKI_MENU_INTERN_TASK_POINT_MIN + MKI_TASK_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_POINT_LIST  = MKI_MENU_INTERN_TASK_POINT_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_ZOOM_MIN    = MKI_MENU_INTERN_TASK_POINT_LIST + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_ZOOM_MAX    = MKI_MENU_INTERN_TASK_ZOOM_MIN + MKI_TASK_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_ZOOM_LIST   = MKI_MENU_INTERN_TASK_ZOOM_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_DELETE_ALL  = MKI_MENU_INTERN_TASK_ZOOM_LIST + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_DELETE_MIN  = MKI_MENU_INTERN_TASK_DELETE_ALL + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_DELETE_MAX  = MKI_MENU_INTERN_TASK_DELETE_MIN + MKI_TASK_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_DELETE_LIST = MKI_MENU_INTERN_TASK_DELETE_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_BITMAP_ALL  = MKI_MENU_INTERN_TASK_DELETE_LIST + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_BITMAP_MIN  = MKI_MENU_INTERN_TASK_BITMAP_ALL + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_BITMAP_MAX  = MKI_MENU_INTERN_TASK_BITMAP_MIN + MKI_TASK_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_BITMAP_LIST = MKI_MENU_INTERN_TASK_BITMAP_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_PDF_ALL     = MKI_MENU_INTERN_TASK_BITMAP_LIST + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_PDF_MIN     = MKI_MENU_INTERN_TASK_PDF_ALL + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_PDF_MAX     = MKI_MENU_INTERN_TASK_PDF_MIN + MKI_TASK_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_PDF_LIST    = MKI_MENU_INTERN_TASK_PDF_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_BLOCK_ALL   = MKI_MENU_INTERN_TASK_PDF_LIST + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_BLOCK_MIN   = MKI_MENU_INTERN_TASK_BLOCK_ALL + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_BLOCK_MAX   = MKI_MENU_INTERN_TASK_BLOCK_MIN + MKI_TASK_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_BLOCK_LIST  = MKI_MENU_INTERN_TASK_BLOCK_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_PRINT_ALL   = MKI_MENU_INTERN_TASK_BLOCK_LIST + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_PRINT_MIN   = MKI_MENU_INTERN_TASK_PRINT_ALL + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_PRINT_MAX   = MKI_MENU_INTERN_TASK_PRINT_MIN + MKI_TASK_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_PRINT_LIST  = MKI_MENU_INTERN_TASK_PRINT_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_PARTS_ALL   = MKI_MENU_INTERN_TASK_PRINT_LIST + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_PARTS_MIN   = MKI_MENU_INTERN_TASK_PARTS_ALL + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_PARTS_MAX   = MKI_MENU_INTERN_TASK_PARTS_MIN + MKI_TASK_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_PARTS_LIST  = MKI_MENU_INTERN_TASK_PARTS_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_PLUGIN_ALL  = MKI_MENU_INTERN_TASK_PARTS_LIST + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_PLUGIN_MIN  = MKI_MENU_INTERN_TASK_PLUGIN_ALL + 1;
constexpr __int32 MKI_MENU_INTERN_TASK_PLUGIN_MAX  = MKI_MENU_INTERN_TASK_PLUGIN_MIN + MKI_TASK_MAX;
constexpr __int32 MKI_MENU_INTERN_TASK_PLUGIN_LIST = MKI_MENU_INTERN_TASK_PLUGIN_MAX;

constexpr __int32 MKI_MENU_INTERN_STARTUP    = MKI_MENU_INTERN_TASK_PLUGIN_LIST + 1;
constexpr __int32 MKI_MENU_INTERN_BATCHOPEN  = MKI_MENU_INTERN_TASK_PLUGIN_LIST + 2;
constexpr __int32 MKI_MENU_INTERN_NEWS       = MKI_MENU_INTERN_TASK_PLUGIN_LIST + 3;
constexpr __int32 MKI_MENU_INTERN_FILEUPDATE = MKI_MENU_INTERN_TASK_PLUGIN_LIST + 4;
constexpr __int32 MKI_MENU_INTERN_LAST       = MKI_MENU_INTERN_TASK_PLUGIN_LIST + 5;


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{                                     
  MKI_HELP_CONTENTS                   = 10000, // Index
  MKI_HELP_HOTSPOT_HELP               = 10001, // Help on Usage of Hotspot Dialog Windows
  MKI_HELP_HOTSPOT_ADDRESS            = 10002, // Malz++Kassner address

  MKI_HELP_TOPIC_REFERENCE            = 11000, // Introduction, Support & Information
  MKI_HELP_TOPIC_CONVENTIONS          = 11001, // Conventions used in the Reference
  MKI_HELP_TOPIC_REQUIREMENTS         = 11002, // System Requirements
  MKI_HELP_TOPIC_PLOTTER              = 11003, // Notes on Plotter Output
  MKI_HELP_TOPIC_DIGITIZER            = 11004, // Using Digitizers
  MKI_HELP_TOPIC_TUTORIAL             = 11005, // Tutorial
  MKI_HELP_TOPIC_LICENSES             = 11006, // Licenses

  MKI_HELP_TOPIC_WINDOW_MAIN          = 12000, // Main Window
  MKI_HELP_TOPIC_WINDOW_DRAW          = 12001, // Drawing Window
  MKI_HELP_TOPIC_WINDOW_VIEW          = 12002, // View Window
  MKI_HELP_TOPIC_WINDOW_PANEL         = 12003, // Panel
  MKI_HELP_TOPIC_WINDOW_COMMAND       = 12004, // Command Bar
  MKI_HELP_TOPIC_WINDOW_WORKFLOW      = 12005, // Workflow Manager
  MKI_HELP_TOPIC_WINDOW_SCALE         = 12006, // Ruler
  MKI_HELP_TOPIC_WINDOW_STATUS        = 12007, // Status Line
  MKI_HELP_TOPIC_WINDOW_BLOCK         = 12008, // Block Window
  MKI_HELP_TOPIC_WINDOW_TOOLBOX       = 12009, // Toolbox
  MKI_HELP_TOPIC_WINDOW_GUIDE         = 12010, // Guide Window
  MKI_HELP_TOPIC_WINDOW_POPUP         = 12011, // Popup Menu
  MKI_HELP_TOPIC_WINDOW_PLUGIN        = 12012, // Plug-In Window
  MKI_HELP_TOPIC_WINDOW_LAYOUT        = 12013, // Layout Window
  MKI_HELP_TOPIC_WINDOW_CONTEXT       = 12014, // Context Menu
  MKI_HELP_TOPIC_WINDOW_PARAMETER     = 12015, // Parameters
  MKI_HELP_TOPIC_WINDOW_ZOOM          = 12016, // Zoom Presets

  MKI_HELP_TOPIC_TRANSMISSION         = 13000, // Layers, Pens, and Transmission
  MKI_HELP_TOPIC_SYSTEMS              = 13001, // Coordinate Systems, Scales, Grids, etc.
  MKI_HELP_TOPIC_OPTIONS              = 13002, // Snapping, Duplicate, and other Options
  MKI_HELP_TOPIC_BLOCKS               = 13003, // Libraries, Blocks and Instances
  MKI_HELP_TOPIC_DIMENSIONS           = 13004, // Dimensions
  MKI_HELP_TOPIC_TEXTFORMAT           = 13005, // Text Format
  MKI_HELP_TOPIC_VARIABLES            = 13006, // Variables
  MKI_HELP_TOPIC_TERM                 = 13007, // Term Evaluation
  MKI_HELP_TOPIC_CALCULATE            = 13008, // Statements in Texts
  MKI_HELP_TOPIC_FILEOPT              = 13009, // File Format
  MKI_HELP_TOPIC_PLUGIN               = 13010, // The Plug-In Concept
  MKI_HELP_TOPIC_TEXTINPUT            = 13011, // Text Input
  MKI_HELP_TOPIC_PAGES                = 13012, // Model Space and Pages
  MKI_HELP_TOPIC_GROUPS               = 13013, // Groups
  MKI_HELP_TOPIC_TASKS                = 13014, // Tasks
  MKI_HELP_TOPIC_ATTRIBUTES           = 13015, // Attributes
  MKI_HELP_TOPIC_ADVANCED_TEXT        = 13016, // Advanced Text Capabilities
  MKI_HELP_TOPIC_PARAMETRIC           = 13017, // Parametric Compounds
  MKI_HELP_TOPIC_COLLECTION           = 13018, // Collections, Pinboard & Items

  MKI_HELP_TOPIC_ID_SINGLE            = 13100, // Single Identification
  MKI_HELP_TOPIC_ID_OBJECT            = 13101, // Object Identification
  MKI_HELP_TOPIC_ID_POINT             = 13102, // Point Identification
  MKI_HELP_TOPIC_EDIT_PARAM           = 13103, // Edit Command Parameters
  MKI_HELP_TOPIC_COORD_ENTRY          = 13104, // Coordinate Entry
  MKI_HELP_TOPIC_DUPLICATE            = 13105, // Duplicate
  MKI_HELP_TOPIC_MULTILINE            = 13106, // Multiline
  MKI_HELP_TOPIC_SURFACE              = 13107, // Generated Surfaces
  MKI_HELP_TOPIC_GRID                 = 13108, // Grid
  MKI_HELP_TOPIC_SNAP                 = 13109, // Snap
  MKI_HELP_TOPIC_GEO                  = 13110, // Construction Aid
  MKI_HELP_TOPIC_TEMP                 = 13111, // Temporary objects

  MKI_HELP_INTERN_BLOCK_INSERT        = 14000, // Internal Commands
  MKI_HELP_INTERN_BLOCK_EDIT          = 14001,
  MKI_HELP_INTERN_TASK_COPY           = 14002,
  MKI_HELP_INTERN_SYSTEM_SELECT       = 14003,
  MKI_HELP_INTERN_SYSTEM_EDIT         = 14004,
  MKI_HELP_INTERN_HATCH_SELECT        = 14005,
  MKI_HELP_INTERN_HATCH_EDIT          = 14006,
  MKI_HELP_INTERN_LAYER_SELECT        = 14007,
  MKI_HELP_INTERN_LAYER_EDIT          = 14008,
  MKI_HELP_INTERN_PEN_SELECT          = 14009,
  MKI_HELP_INTERN_PEN_EDIT            = 14010,
  MKI_HELP_INTERN_PAGE_SELECT         = 14011,
  MKI_HELP_INTERN_PAGE_EDIT           = 14012,
  MKI_HELP_INTERN_TASK_SELECT         = 14013,
  MKI_HELP_INTERN_TASK_PARTS          = 14014,
  MKI_HELP_INTERN_TASK_EDIT           = 14015,
  MKI_HELP_INTERN_TASK_DELETE         = 14016,
  MKI_HELP_INTERN_TASK_BITMAP         = 14017,
  MKI_HELP_INTERN_TASK_PDF            = 14018,
  MKI_HELP_INTERN_TASK_RECT           = 14019,
  MKI_HELP_INTERN_TASK_ZOOM           = 14020,
  MKI_HELP_INTERN_TASK_DUP            = 14021,
  MKI_HELP_INTERN_TASK_BLOCK          = 14022,
  MKI_HELP_INTERN_TASK_POLY           = 14023,
  MKI_HELP_INTERN_TASK_POINT          = 14024,
  MKI_HELP_INTERN_TASK_PRINT          = 14025,
  MKI_HELP_INTERN_TASK_PLUGIN         = 14026,
  MKI_HELP_INTERN_TASK_MOVE           = 14027,
  MKI_HELP_INTERN_TASK_UPDATE         = 14028,
  MKI_HELP_INTERN_TASK_SIZE           = 14029,

  MKI_HELP_DIALOG_XPROPERTY           = 15000, // Edit Properties
  MKI_HELP_DIALOG_TEXTSTANDARD        = 15001, // Standard Text
  MKI_HELP_DIALOG_TEXTFRAME           = 15002, // Frame Text
  MKI_HELP_DIALOG_ZIGZAG              = 15003, // Zig Zag Line
  MKI_HELP_DIALOG_DIMPARAM0           = 15004, // Dimension Line
  MKI_HELP_DIALOG_DIMDATA             = 15005, // General Dimension Parameters
  MKI_HELP_DIALOG_DIMPARAM1           = 15006, // Dimension with Line
  MKI_HELP_DIALOG_DIMPARAM2           = 15007, // Dimension without Line
  MKI_HELP_DIALOG_CHOOSECOLOR         = 15008, // Color Selection
  MKI_HELP_DIALOG_FILESELECT          = 15009, // File Selection
  MKI_HELP_DIALOG_ACTIVATE            = 15010, // Activation
  MKI_HELP_DIALOG_PRINTLAYOUT         = 15011, // Print Drawing
  MKI_HELP_DIALOG_OPTIONS_FILE        = 15012, // File / Clipboard Options
  MKI_HELP_DIALOG_CHOOSESECTIONS      = 15013, // Edit Single File Options
  MKI_HELP_DIALOG_BACKUPOPTIONS       = 15014, // Backup Options
  MKI_HELP_DIALOG_BITMAPEDIT          = 15015, // Bitmap Reference
  MKI_HELP_DIALOG_EXPORTBITMAP        = 15016, // Export Bitmap
  MKI_HELP_DIALOG_COMMENTOPTIONS      = 15017, // Comment Options
  MKI_HELP_DIALOG_DETAILSEDIT         = 15018, // Edit Details
  MKI_HELP_DIALOG_PAGEFORMAT          = 15019, // Edit Single Page
  MKI_HELP_DIALOG_LAYEREDIT           = 15020, // Edit Layers
  MKI_HELP_DIALOG_DETAILSVIEW         = 15021, // View Details
  MKI_HELP_DIALOG_LAYERDEFAULT        = 15022, // Layer Defaults
  MKI_HELP_DIALOG_SYSTEMEDIT          = 15023, // Edit Coordinate Systems
  MKI_HELP_DIALOG_SYSTEMPARAM         = 15024, // Coordinate Systems View
  MKI_HELP_DIALOG_TASKEDIT            = 15025, // Edit Task
  MKI_HELP_DIALOG_BACKUPLIST          = 15026, // Manage Backup List
  MKI_HELP_DIALOG_DIMTEXT0            = 15027, // Edit Dimension Text
  MKI_HELP_DIALOG_SYSTEMGRID          = 15028, // Coordinate Systems Grid
  MKI_HELP_DIALOG_PAGELIST            = 15029, // Edit Pages
  MKI_HELP_DIALOG_PENEDIT             = 15030, // Edit Pens
  MKI_HELP_DIALOG_PENDEFAULT          = 15031, // Pen Defaults
  MKI_HELP_DIALOG_LINEEDIT            = 15032, // Edit Line Types
  MKI_HELP_DIALOG_LINETYPE            = 15033, // Edit Single Line Type
  MKI_HELP_DIALOG_PROPERTY            = 15034, // Simple Properties
  MKI_HELP_DIALOG_DIMLAYOUT           = 15035, // Global Dimension Parameters
  MKI_HELP_DIALOG_DRAWINGMANAGER      = 15036, // Drawing Manager
  MKI_HELP_DIALOG_OPTIONS_GENERAL     = 15037, // General Options
  MKI_HELP_DIALOG_OPTIONS_PATHS       = 15038, // Path Options
  MKI_HELP_DIALOG_ORTHOLAYOUT         = 15039, // Orthogonal Mode
  MKI_HELP_DIALOG_SETTINGS_SCREEN     = 15040, // Display Settings
  MKI_HELP_DIALOG_SETTINGS_PRECISION  = 15041, // Precision Settings
  MKI_HELP_DIALOG_SETTINGS_OUTPUT     = 15042, // Output Settings
  MKI_HELP_DIALOG_SETTINGS_COLORS     = 15043, // Colors Settings
  MKI_HELP_DIALOG_OPTIONS_KEYBOARD    = 15044, // Keyboard Options
  MKI_HELP_DIALOG_OPTIONS_MOUSE       = 15045, // Mouse Options
  MKI_HELP_DIALOG_OPTIONS_SKIN        = 15046, // Skin Options
  MKI_HELP_DIALOG_MULTICOPY_LINEAR    = 15047, // Multiple Copy, Linear
  MKI_HELP_DIALOG_FONTHANDLING        = 15048, // Fonts
  MKI_HELP_DIALOG_INSTANCEEDIT        = 15049, // Instance
  MKI_HELP_DIALOG_TEXTSTANDARDINSERT  = 15050, // Create Standard Text
  MKI_HELP_DIALOG_TEXTFRAMEINSERT     = 15051, // Create Frame Text
  MKI_HELP_DIALOG_FONTDEF             = 15052, // Select Font
  MKI_HELP_DIALOG_CURVE               = 15053, // Curve / Surface Entry
  MKI_HELP_DIALOG_FINDPROP            = 15054, // Selection Parameters
  MKI_HELP_DIALOG_CHANGETEXT          = 15055, // Edit Text
  MKI_HELP_DIALOG_TEXTREFERENCE       = 15056, // Reference Text
  MKI_HELP_DIALOG_DIMTEXT1            = 15057, // Edit Dimension and Tolerances
  MKI_HELP_DIALOG_DIMTEXT2            = 15058, // Edit Coordinates and Tolerances
  MKI_HELP_DIALOG_HATCHEDIT           = 15059, // Edit Hatching Types
  MKI_HELP_DIALOG_HATCHTYPE           = 15060, // Edit Single Hatching Type
  MKI_HELP_DIALOG_COPYTEXT            = 15061, // Copy Text
  MKI_HELP_DIALOG_MULTILINEPARAM      = 15062, // Edit Line Sequence
  MKI_HELP_DIALOG_BLOCKINSERT         = 15063, // Insert Block
  MKI_HELP_DIALOG_BLOCKREAD           = 15064, // Create Block
  MKI_HELP_DIALOG_BLOCKEDIT           = 15065, // Edit Block
  MKI_HELP_DIALOG_BLOCKSELECT         = 15066, // Select Block
  MKI_HELP_DIALOG_ATTRIBUTELIST       = 15067, // Edit Attribute List
  MKI_HELP_DIALOG_ATTRIBUTEEDIT       = 15068, // Edit Single Attribute
  MKI_HELP_DIALOG_PARTSLIST1          = 15069, // Generate Parts List, Part 1
  MKI_HELP_DIALOG_PARTSLIST2          = 15070, // Generate Parts List, Part 2
  MKI_HELP_DIALOG_BLOCKCONVERT        = 15071, // External Blocks to Internal
  MKI_HELP_DIALOG_BLOCKREPLACE        = 15072, // Replace Block
  MKI_HELP_DIALOG_LIBRARYREPLACE      = 15073, // Replace Library
  MKI_HELP_DIALOG_LIBRARYHANDLING     = 15074, // Libraries
  MKI_HELP_DIALOG_LISTBOXSELECTION    = 15075, // List box selection
  MKI_HELP_DIALOG_CLIPPINGEDIT        = 15076, // Edit Clipping
  MKI_HELP_DIALOG_COMMENTEDIT         = 15077, // Edit Comment
  MKI_HELP_DIALOG_VARIABLELIST        = 15078, // Variables
  MKI_HELP_DIALOG_ALERT               = 15079, // General Message
  MKI_HELP_DIALOG_CHOICE              = 15080, // Multi-Object Selection
  MKI_HELP_DIALOG_EDIT                = 15081, // General Entry
  MKI_HELP_DIALOG_MULTICOPY_RADIAL    = 15082, // Multiple Copy, Radial
  MKI_HELP_DIALOG_TABLETBUTTON        = 15083, // Digitizer Buttons
  MKI_HELP_DIALOG_PARTSLISTOPTION     = 15084, // Generate Parts List Options
  MKI_HELP_DIALOG_FILEINFO            = 15085, // File Infos
  MKI_HELP_DIALOG_OPTIONS_WINDOWS     = 15086, // Window Options
  MKI_HELP_DIALOG_COPYPARAMDEFAULT    = 15087, // Copy Properties Parameters
  MKI_HELP_DIALOG_OPTIONS_TIME        = 15088, // Time Options
  MKI_HELP_DIALOG_SECURELAYOUT        = 15089, // Secure Layout
  MKI_HELP_DIALOG_DATABASE            = 15090, // Databases
  MKI_HELP_DIALOG_DATABASEOPTIONS     = 15091, // Database Options
  MKI_HELP_DIALOG_LAYERFLAGS          = 15092, // Layer Flags
  MKI_HELP_DIALOG_PENFLAGS            = 15093, // Pen Flags
  MKI_HELP_DIALOG_REFERENCEINSERT     = 15094, // Insert Reference
  MKI_HELP_DIALOG_REFERENCEEDIT       = 15095, // Reference
  MKI_HELP_DIALOG_FINDTEXT            = 15096, // Find Text
  //                                  = 15097, // - reserved
  MKI_HELP_DIALOG_SYSTEMFLAGS         = 15098, // Coordinate System Flags
  MKI_HELP_DIALOG_LINEFLAGS           = 15099, // Line Type Flags
  MKI_HELP_DIALOG_SNAPPARAM           = 15100, // Snap Parameters
  MKI_HELP_DIALOG_LIBRARYELEMENTS     = 15101, // Library Elements
  MKI_HELP_DIALOG_PRINTSERIAL         = 15102, // Serial Printing
  MKI_HELP_DIALOG_SETTINGS_RULES      = 15103, // Printing Rules Settings
  MKI_HELP_DIALOG_SETTINGS_SERIAL     = 15104, // Serial Printing Settings
  MKI_HELP_DIALOG_UPDATE              = 15105, // Online Update
  MKI_HELP_DIALOG_DRAWINGNEW          = 15106, // New Drawing
  MKI_HELP_DIALOG_NOTCHCORNER         = 15107, // Notch Corner
  MKI_HELP_DIALOG_NOTCHEDGE           = 15108, // Notch Edge
  MKI_HELP_DIALOG_PAGEFLAGS           = 15109, // Page Flags
  MKI_HELP_DIALOG_PRINTCAD6PDF        = 15110, // CAD6PDF Print Options
  MKI_HELP_DIALOG_COLOROPTIONS        = 15111, // Color Options
  MKI_HELP_DIALOG_DATABASECONTENT     = 15112, // View Database Content
  MKI_HELP_DIALOG_DATABASEEDIT        = 15113, // Edit Database
  MKI_HELP_DIALOG_DATABASESETTINGS    = 15114, // Database Settings
  MKI_HELP_DIALOG_DATABASEADDOBJECT   = 15115, // Add Object's Data to Database
  MKI_HELP_DIALOG_DATABASEAUTONAMES   = 15116, // Database Auto-Column Names
  MKI_HELP_DIALOG_DATABASEFINDOBJECT  = 15117, // Find Object Data
  MKI_HELP_DIALOG_DATABASESELECTION   = 15118, // Set Permanent Selection
  MKI_HELP_DIALOG_DATABASECOLUMN      = 15119, // Edit Database Column
  MKI_HELP_DIALOG_MAPPINGEDIT         = 15120, // Edit Mapping
  MKI_HELP_DIALOG_MAPPING             = 15121, // Mapping Parameters
  MKI_HELP_DIALOG_CORRECTCORNER       = 15122, // Notch Corner
  MKI_HELP_DIALOG_TABLETFIELD         = 15123, // Tablet field
  MKI_HELP_DIALOG_FINDINDATABASE      = 15124, // Find in Database
  MKI_HELP_DIALOG_TEXTSTYLELIST       = 15125, // Text Style List
  MKI_HELP_DIALOG_TEXTSTYLE           = 15126, // Edit Text Style
  MKI_HELP_DIALOG_SELECT              = 15127, // Element Selection
  MKI_HELP_DIALOG_MODELFORMAT         = 15128, // Edit Model Space
  MKI_HELP_DIALOG_DATABASESTAMP       = 15129, // Database Stamp
  MKI_HELP_DIALOG_RECENTDRAWING       = 15130, // Recent Drawing List
  MKI_HELP_DIALOG_FOLDERS             = 15131, // Manage Folders
  MKI_HELP_DIALOG_BLOCKFOLDERS        = 15132, // Choose Block Folder
  MKI_HELP_DIALOG_HATCHENTITY         = 15133, // Edit Hatching Entity
  MKI_HELP_DIALOG_MULTISELECT         = 15134, // Multiple Element Selection
  MKI_HELP_DIALOG_COMMENTLIST         = 15135, // Display Comment List
  MKI_HELP_DIALOG_COMMENTPROP         = 15136, // Edit Comment Properties
  MKI_HELP_DIALOG_COLLECTIONHANDLING  = 15137, // Manage Collections
  MKI_HELP_DIALOG_ITEMSELECT          = 15138, // Select Collection Item
  MKI_HELP_DIALOG_ITEMCREATE          = 15139, // Create Collection Item
  MKI_HELP_DIALOG_ITEMEDIT            = 15140, // Edit Collection Items
  MKI_HELP_DIALOG_PINBOARD            = 15141, // Pinboard List
  MKI_HELP_DIALOG_NOTESLIST           = 15142, // Notes List
  MKI_HELP_DIALOG_NOTESHANDLING       = 15143, // Manage Notes
  MKI_HELP_DIALOG_NOTESEDIT           = 15144, // Edit Note
  MKI_HELP_DIALOG_OBJECTMANAGER       = 15145, // Object Manager
  MKI_HELP_DIALOG_ATTRIBUTEMULTI      = 15146, // Edit Multiple Attribute
  MKI_HELP_DIALOG_SIMPLIFY            = 15147, // Simplify Curve / Surface
  MKI_HELP_DIALOG_RECENTBLOCK         = 15148, // Recent Block List
  MKI_HELP_DIALOG_POSNOOPTIONS        = 15149, // Position Number Options
  MKI_HELP_DIALOG_POSNOREORDER        = 15150, // Reorder Position Numbers
  MKI_HELP_DIALOG_POSNOCREATE         = 15151, // Create Position Number
  MKI_HELP_DIALOG_EXCHANGE_CLIPBOARD  = 15152, // Data Exchange Clipboard
  MKI_HELP_DIALOG_TEXTOPTIONS         = 15153, // Text Editing Options
  MKI_HELP_DIALOG_RESOLVEOPTIONS      = 15154, // Trim Resolve Options
  MKI_HELP_DIALOG_DRAWINGCLEANUP      = 15155, // Clean Up Drawing
  MKI_HELP_DIALOG_CHECKOBJECTPARAM    = 15156, // Check Objects Parameters
  MKI_HELP_DIALOG_CHECKOBJECTINFO     = 15157, // Check Objects Result
  MKI_HELP_DIALOG_CHECKDUPLICATEPARAM = 15158, // Check Duplicates Parameters
  MKI_HELP_DIALOG_BLOCKOPTIONS        = 15159, // Insert Block Options
  MKI_HELP_DIALOG_OPTIONS_CROSS       = 15160, // Crosshairs Options
  MKI_HELP_DIALOG_PENLAYEROPTIONS     = 15161, // Pen / Layer Dialog Options
  MKI_HELP_DIALOG_CUSTOMMENU          = 15162, // Edit Custom Menu
  MKI_HELP_DIALOG_AUTOCONVPRINTJOB    = 15163, // Auto Converter Print Job
  MKI_HELP_DIALOG_AUTOCONVFILES       = 15164, // Auto Converter Directory
  MKI_HELP_DIALOG_AUTOCONVTOLIB       = 15165, // Auto Converter Convert to Library
  MKI_HELP_DIALOG_AUTOCONVFROMLIB     = 15166, // Auto Converter Convert from Library
  MKI_HELP_DIALOG_AUTOCONVINDEXFILE   = 15167, // Auto Converter Index File
  MKI_HELP_DIALOG_AUTOCONVOPTIONS     = 15168, // Auto Converter General Options
  MKI_HELP_DIALOG_AUTOCONVEDIT        = 15169, // Auto Converter Edit Text
  MKI_HELP_DIALOG_AUTOCONVFONT        = 15170, // Auto Converter Input Font
  MKI_HELP_DIALOG_GRAPHFUNCTION       = 15171, // Graph Function
  MKI_HELP_DIALOG_GRAPHAXES           = 15172, // Graph Axes
  MKI_HELP_DIALOG_GRAPHUNIT           = 15173, // Graph Unit
  MKI_HELP_DIALOG_GRAPHTEXT_X         = 15174, // Graph Text X-Axis
  MKI_HELP_DIALOG_GRAPHTEXT_Y         = 15175, // Graph Text Y-Axis
  MKI_HELP_DIALOG_GRAPHARROW          = 15176, // Graph Arrow
  MKI_HELP_DIALOG_GRAPHFAMILY         = 15177, // Graph Family
  MKI_HELP_DIALOG_EXCHANGE_METAFILE   = 15178, // Data Exchange EMF/WMF
  MKI_HELP_DIALOG_EXCHANGE_PLUGINS    = 15179, // Data Exchange Filters
  MKI_HELP_DIALOG_EXCHANGE_EXPORTSVG  = 15180, // Data Exchange SVG Export
  MKI_HELP_DIALOG_EXCHANGE_IMPORTSVG  = 15181, // Data Exchange SVG Import
  MKI_HELP_DIALOG_UNDOLIST            = 15182, // Undo List
  MKI_HELP_DIALOG_FONTLIST            = 15183, // Font List
  MKI_HELP_DIALOG_EXCHANGE_IMPORTBMP  = 15184, // Data Exchange Bitmap Import
  MKI_HELP_DIALOG_PRINTDIRECTPDF      = 15185, // DirectPDF Print Options
};

//---------------------------------------------------------------------------------------------------------------------------------------------------

#ifndef MKI_RESOURCE

enum
{
  MKI_POINT_NO                    = 0,
  MKI_POINT_ANY                   = 1,
  MKI_POINT_ANY1                  = 2,
  MKI_POINT_ANY2                  = 3,
  MKI_POINT_START                 = 4,
  MKI_POINT_START1                = 5,
  MKI_POINT_START2                = 6,
  MKI_POINT_END                   = 7,
  MKI_POINT_END1                  = 8,
  MKI_POINT_END2                  = 9,
  MKI_POINT_REFERENCE             = 10,
  MKI_POINT_REFERENCE1            = 11,
  MKI_POINT_REFERENCE2            = 12,
  MKI_POINT_DESTINATION           = 13,
  MKI_POINT_CORNER1               = 14,
  MKI_POINT_CORNER2               = 15,
  MKI_POINT_CORNER3               = 16,
  MKI_POINT_CORNER4               = 17,
  MKI_POINT_CIRCLE1               = 18,
  MKI_POINT_CIRCLE2               = 19,
  MKI_POINT_CIRCLE3               = 20,
  MKI_POINT_ELLIPSE1              = 21,
  MKI_POINT_ELLIPSE2              = 22,
  MKI_POINT_FRAME                 = 23,
  MKI_POINT_EDGE_CENTER           = 24,
  MKI_POINT_EDGE_CENTER1          = 25,
  MKI_POINT_EDGE_CENTER2          = 26,
  MKI_POINT_CIRCLE_CENTER         = 27,
  MKI_POINT_CIRCLE_CENTER1        = 28,
  MKI_POINT_CIRCLE_CENTER2        = 29,
  MKI_POINT_RADIUS                = 30,
  MKI_POINT_ANGLE                 = 31,
  MKI_POINT_ANGLE1                = 32,
  MKI_POINT_ANGLE2                = 33,
  MKI_POINT_LINE_RELATIVE         = 34,
  MKI_POINT_PIVOT1                = 35,
  MKI_POINT_PIVOT2                = 36,
  MKI_POINT_CURVE_END             = 37,
  MKI_POINT_ARC                   = 38,
  MKI_POINT_ARC_END1              = 39,
  MKI_POINT_ARC_END2              = 40,
  MKI_POINT_ARC_RELATIVE          = 41,
  MKI_POINT_POSITION              = 42,
  MKI_POINT_ROTATION              = 43,
  MKI_POINT_SECTION1              = 44,
  MKI_POINT_SECTION2              = 45,
  MKI_POINT_SELECTAREA1           = 46,
  MKI_POINT_SELECTAREA2           = 47,
  MKI_POINT_DIM_LINE1             = 48,
  MKI_POINT_DIM_LINE2             = 49,
  MKI_POINT_DIM_LINE3             = 50,
  MKI_POINT_DIM_EXTLINE           = 51,
  MKI_POINT_DIM_NUM1              = 52,
  MKI_POINT_DIM_NUM2              = 53,
  MKI_POINT_FREEHAND              = 54,
  MKI_POINT_SYMBOL                = 55,
                                          // First ID that requires identification resolving
  MKI_POINT_PL                    = 56,   MKI_POINT_ID_SINGLE_FIRST = MKI_POINT_PL,
  MKI_POINT_PL1                   = 57,
  MKI_POINT_PL2                   = 58,
  MKI_POINT_PL3                   = 59,
  MKI_POINT_PC                    = 60,
  MKI_POINT_PC1                   = 61,
  MKI_POINT_PC2                   = 62,
  MKI_POINT_PC3                   = 63,
  MKI_POINT_PE                    = 64,
  MKI_POINT_PE1                   = 65,
  MKI_POINT_PE2                   = 66,
  MKI_POINT_PE3                   = 67,
  MKI_POINT_PA                    = 68,
  MKI_POINT_PA1                   = 69,
  MKI_POINT_PA2                   = 70,
  MKI_POINT_PA3                   = 71,
  MKI_POINT_PCE                   = 72,
  MKI_POINT_PCE1                  = 73,
  MKI_POINT_PCE2                  = 74,
  MKI_POINT_PCE3                  = 75,
  MKI_POINT_PCEA                  = 76,
  MKI_POINT_PCEA1                 = 77,
  MKI_POINT_PCEA2                 = 78,
  MKI_POINT_PCEA3                 = 79,
  MKI_POINT_PCEB                  = 80,
  MKI_POINT_PCEB1                 = 81,
  MKI_POINT_PCEB2                 = 82,
  MKI_POINT_PCEB3                 = 83,
  MKI_POINT_PLC                   = 84,
  MKI_POINT_PLC1                  = 85,
  MKI_POINT_PLC2                  = 86,
  MKI_POINT_PLC3                  = 87,
  MKI_POINT_PLE                   = 88,
  MKI_POINT_PLE1                  = 89,
  MKI_POINT_PLE2                  = 90,
  MKI_POINT_PLE3                  = 91,
  MKI_POINT_PLA                   = 92,
  MKI_POINT_PLA1                  = 93,
  MKI_POINT_PLA2                  = 94,
  MKI_POINT_PLA3                  = 95,
  MKI_POINT_PLBA                  = 96,
  MKI_POINT_PLBA1                 = 97,
  MKI_POINT_PLBA2                 = 98,
  MKI_POINT_PLBA3                 = 99,
  MKI_POINT_PLCE                  = 100,
  MKI_POINT_PLCE1                 = 101,
  MKI_POINT_PLCE2                 = 102,
  MKI_POINT_PLCE3                 = 103,
  MKI_POINT_PLCEA                 = 104,
  MKI_POINT_PLCEA1                = 105,
  MKI_POINT_PLCEA2                = 106,
  MKI_POINT_PLCEA3                = 107,
  MKI_POINT_PANY                  = 108,
  MKI_POINT_PANY1                 = 109,
  MKI_POINT_PANY2                 = 110,
  MKI_POINT_PANY3                 = 111,
  MKI_POINT_PGL                   = 112,
  MKI_POINT_PGC                   = 113,
  MKI_POINT_PGE                   = 114,
  MKI_POINT_PGCE                  = 115,
  MKI_POINT_PGLC                  = 116,
  MKI_POINT_PGLE                  = 117,
  MKI_POINT_PGLCE                 = 118,
  MKI_POINT_PFCE                  = 119,
  MKI_POINT_PFCE1                 = 120,
  MKI_POINT_PFCE2                 = 121,
  MKI_POINT_PFCE3                 = 122,
  MKI_POINT_PCURVE                = 123,
  MKI_POINT_PCURVE1               = 124,
  MKI_POINT_PCURVE2               = 125,
  MKI_POINT_PCURVE3               = 126,
  MKI_POINT_PEDGE                 = 127,
  MKI_POINT_PEDGE1                = 128,
  MKI_POINT_PEDGE2                = 129,
  MKI_POINT_PEDGE3                = 130,

  MKI_POINT_ID_LOCKED             = 131,
  MKI_POINT_ID_ENTITY             = 132,
  MKI_POINT_ID_LINKENTITY         = 133,
  MKI_POINT_ID_TEMPENTITY         = 134,
  MKI_POINT_ID_TEMPLINEAR         = 135,
  MKI_POINT_ID_TEMPFILLED         = 136,
  MKI_POINT_ID_OBJECT             = 137,
  MKI_POINT_ID_LINKOBJECT         = 138,
  MKI_POINT_ID_BASEOBJECT         = 139,
  MKI_POINT_ID_INSTANCE           = 140,
  MKI_POINT_ID_TRIM               = 141,
  MKI_POINT_ID_TRIM1              = 142,
  MKI_POINT_ID_TRIM2              = 143,
  MKI_POINT_ID_LINE               = 144,
  MKI_POINT_ID_LINE1              = 145,
  MKI_POINT_ID_LINE2              = 146,
  MKI_POINT_ID_CIRCLE             = 147,
  MKI_POINT_ID_CIRCLE1            = 148,
  MKI_POINT_ID_CIRCLE2            = 149,
  MKI_POINT_ID_ARC                = 150,
  MKI_POINT_ID_ELLIPSE            = 151,
  MKI_POINT_ID_EARC               = 152,
  MKI_POINT_ID_CEA                = 153,
  MKI_POINT_ID_CEA1               = 154,
  MKI_POINT_ID_CEA2               = 155,
  MKI_POINT_ID_CURVE              = 156,
  MKI_POINT_ID_SURFACE            = 157,
  MKI_POINT_ID_CURVE_SURFACE      = 158,
  MKI_POINT_ID_EDIT_CURVE         = 159,
  MKI_POINT_ID_FILLED             = 160,
  MKI_POINT_ID_FILLED1            = 161,
  MKI_POINT_ID_FILLED2            = 162,
  MKI_POINT_ID_TEXT               = 163,
  MKI_POINT_ID_ANYTEXT            = 164,
  MKI_POINT_ID_LINKTEXT           = 165,
  MKI_POINT_ID_DIMENSIONLINE      = 166,
  MKI_POINT_ID_DIMENSIONONLY      = 167,
  MKI_POINT_ID_CLIP               = 168,
  MKI_POINT_ID_MAPPING            = 169,
  MKI_POINT_ID_GROUP              = 170,
  MKI_POINT_ID_EXT                = 171,
  MKI_POINT_ID_COMMENT            = 172,
  MKI_POINT_ID_SIMPLE             = 173,
  MKI_POINT_ID_SIMPLE1            = 174,
  MKI_POINT_ID_SIMPLE2            = 175,
  MKI_POINT_ID_SIMPLE_MARK        = 176,
  MKI_POINT_ID_SIMPLE_MARK1       = 177,
  MKI_POINT_ID_SIMPLE_MARK2       = 178,
  MKI_POINT_ID_POINT              = 179,
  MKI_POINT_ID_TEMP               = 180,
  MKI_POINT_ID_REFERENCE          = 181,
  MKI_POINT_ID_HYPERLINK          = 182,
  MKI_POINT_ID_EXTFILL            = 183,
  MKI_POINT_ID_HATCH              = 184,
  MKI_POINT_ID_POSNO              = 185,
                                          // First ID that enables multi-object identification
  MKI_POINT_ID_MULTIANYTEXT       = 186,  MKI_POINT_ID_MULTI_FIRST = MKI_POINT_ID_MULTIANYTEXT,
  MKI_POINT_ID_MULTITEXT          = 187,
  MKI_POINT_ID_MULTITEXTSTANDARD  = 188,
  MKI_POINT_ID_MULTITEXTREFERENCE = 189,
  MKI_POINT_ID_MULTIDIMENSIONLINE = 190,
  MKI_POINT_ID_MULTIDIMENSIONONLY = 191,
  MKI_POINT_ID_MULTIINSTANCE      = 192,
  MKI_POINT_ID_MULTIGROUP         = 193,
  MKI_POINT_ID_MULTIPOSNO         = 194,
  MKI_POINT_ID_MULTIOBJ           = 195,
  MKI_POINT_ID_MULTIENTITY        = 196,
  MKI_POINT_ID_MULTILOCKED        = 197,
  MKI_POINT_ID_MULTIGEO           = 198,
  MKI_POINT_ID_MULTIFILLED        = 199,
  MKI_POINT_ID_MULTIHYPERLINK     = 200,
  MKI_POINT_ID_MULTICLIP          = 201,
  MKI_POINT_ID_MULTIEXT           = 202,
  MKI_POINT_ID_MULTIEXTFILL       = 203,
  MKI_POINT_ID_MULTIHATCH         = 204,  MKI_POINT_ID_MULTI_LAST = MKI_POINT_ID_MULTIHATCH,
  MKI_POINT_ID_MULTIPOINT         = 205,
  MKI_POINT_ID_MULTIATTRIBPOINT   = 206,
                                          
  MKI_POINT_TOTAL                 = 207,
};

enum
{
  MKI_POINTMODE_MASK         = 0x0000ffff, // Basic point types
  MKI_POINTMODE_NO_GRID      = 0x00010000, // The point does not support grids
  MKI_POINTMODE_NO_SNAP      = 0x00020000, // The point does not support snapping
  MKI_POINTMODE_EXT_SNAP     = 0x00040000, // The point uses external snapping
  MKI_POINTMODE_EXT_OBJECT   = 0x00080000, // The point uses external object identification
  MKI_POINTMODE_EXT_POINT    = 0x00100000, // The point uses external point identification
  MKI_POINTMODE_HOLLOW       = 0x00200000, // The point uses feedback display that supports "hollow" mode
  MKI_POINTMODE_NO_HIGHLIGHT = 0x00400000, // The point does not support highlighting
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_GUIDE_NULL           = 0x00000000, // No guide
  MKI_GUIDE_SINGLE_OBJ     = 0x00000001, // Single object identification
  MKI_GUIDE_GROUP_SUB      = 0x00000002, // Object within group choosing
  MKI_GUIDE_MULTI_OBJ      = 0x00000004, // Multiple object choosing
  MKI_GUIDE_SINGLE_POINT   = 0x00000008, // Single point identification
  MKI_GUIDE_MULTI_POINT    = 0x00000010, // Multiple point choosing
  MKI_GUIDE_PARAMETER      = 0x00000020, // Parameters are available
  MKI_GUIDE_ARC_DIRECTION  = 0x00000040, // Arc direction has effect
  MKI_GUIDE_ARC_MODE       = 0x00000080, // Arc mode has effect
  MKI_GUIDE_COORDINATES    = 0x00000100, // Coordinate entry makes sense
  MKI_GUIDE_SNAP           = 0x00000200, // Snapping has effect
  MKI_GUIDE_GRID           = 0x00000400, // Grid has effect
  MKI_GUIDE_DUPLICATE      = 0x00000800, // Duplicate has effect
  MKI_GUIDE_MULTILINE      = 0x00001000, // Multiline has effect
  MKI_GUIDE_ESC            = 0x00002000, // Last point entry may be reversed
  MKI_GUIDE_RBUTTON_CANCEL = 0x00004000, // Right mouse button cancels command
  MKI_GUIDE_RBUTTON_FINISH = 0x00008000, // Right mouse button finishes entry
  MKI_GUIDE_RBUTTON_OPTION = 0x00010000, // Right mouse button calls option dialog
  MKI_GUIDE_ENTRY          = 0x00020000, // Description of point entry
  MKI_GUIDE_RESET          = 0x20000000, // Resets guide content to empty
  MKI_GUIDE_DEFAULT        = 0x40000000, // Default descriptions
  MKI_GUIDE_EXTERNAL       = 0x80000000, // Additional external text
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_APPTYPE_INDUSTRIE = 0x00000001, // Plug-in runs on at least CAD6industrie
  MKI_APPTYPE_STUDIO    = 0x00000002, // Plug-in runs on at least CAD6studio
  MKI_APPTYPE_PRO       = 0x00000004, // Plug-in runs on at least CAD6pro
  MKI_APPTYPE_ECO       = 0x00000008, // Plug-in runs on CAD6eco
  MKI_APPTYPE_STARTER   = 0x00000010, // Plug-in runs on CAD6starter
  MKI_APPTYPE_TRIAL     = 0x00000100, // Plug-in runs on trial versions (during 30 days trial period)
  MKI_APPTYPE_DISABLED  = 0x00000200, // Plug-in runs on disabled versions
  MKI_APPTYPE_READONLY  = 0x00000400, // Plug-in runs on read-only viewer versions

  MKI_APPTYPE_ALL       = 0xffffffff, // Plug-in runs on all applications
};

enum
{
  MKI_APPSTATE_FREE     = 0, // Application is free
  MKI_APPSTATE_LICENSED = 1, // Application is licensed and activated
  MKI_APPSTATE_TRIAL    = 2, // Application is in a 30 days trial period
  MKI_APPSTATE_LOCKING  = 3, // Application will be locked at next start
  MKI_APPSTATE_LOCKED   = 4, // Application is locked after 30 trial period
  MKI_APPSTATE_EXPIRED  = 5, // Application is locked due to end of rental
  MKI_APPSTATE_OUTDATED = 6, // Application is locked due to outdated license
  MKI_APPSTATE_INACTIVE = 7, // Application is in a 5 or 30 days activation period
  MKI_APPSTATE_READONLY = 8, // Application is in read-only viewer mode
};

enum
{
  MKI_PLUGINMENU_FILE   = 0,    // Plug-in contains commands for menu "File"
  MKI_PLUGINMENU_EDIT   = 1,    // Plug-in contains commands for menu "Edit"
  MKI_PLUGINMENU_MANAGE = 2,    // Plug-in contains commands for menu "Manage"
  MKI_PLUGINMENU_MODIFY = 3,    // Plug-in contains commands for menu "Modify"
  MKI_PLUGINMENU_DRAW   = 4,    // Plug-in contains commands for menu "Draw"
  MKI_PLUGINMENU_GEO    = 5,    // Plug-in contains commands for menu "Construction"
  MKI_PLUGINMENU_TRIM   = 6,    // Plug-in contains commands for menu "Trim"
  MKI_PLUGINMENU_TEXT   = 7,    // Plug-in contains commands for menu "Text"
  MKI_PLUGINMENU_LIB    = 8,    // Plug-in contains commands for menu "Library"
  MKI_PLUGINMENU_EXTRA  = 9,    // Plug-in contains commands for menu "Extra"
  MKI_PLUGINMENU_WINDOW = 10,   // Plug-in contains commands for menu "Window"
  MKI_PLUGINMENU_HELP   = 11,   // Plug-in contains commands for menu "Help"
  MKI_PLUGINMENU_CUSTOM = 97,   // Plug-in contains commands for custom menu
  MKI_PLUGINMENU_IMPORT = 98,   // Plug-in contains import filters
  MKI_PLUGINMENU_EXPORT = 99,   // Plug-in contains export filters
  MKI_PLUGINMENU_NONE   = 100,  // Plug-in contains invisible commands
};

enum
{
  MKI_EXECMODE_USER          = 0,   // A command was selected by the user
  MKI_EXECMODE_SYSTEM        = 1,   // A command is restarted by the system
  MKI_EXECMODE_HELP          = 2,   // The help topic for a command shall be displayed
  MKI_EXECMODE_GET_PROFILE   = 3,   // The plug-in should read any drawing-dependent settings now
  MKI_EXECMODE_SET_PROFILE   = 4,   // The plug-in should write any drawing-dependent settings now
  MKI_EXECMODE_MENU_INIT     = 5,   // Evaluate menu initialization entry state
  MKI_EXECMODE_STATUS        = 6,   // Evaluate global plug-in status / availability
  MKI_EXECMODE_READY         = 7,   // Notification that all plug-ins and filters have been loaded
  MKI_EXECMODE_BUTTON_LCLICK = 8,   // A button was clicked with the left mouse button
  MKI_EXECMODE_BUTTON_MCLICK = 9,   // A button was clicked with the middle mouse button
  MKI_EXECMODE_BUTTON_RCLICK = 10,  // A button was clicked with the right mouse button
  MKI_EXECMODE_BUTTON_HELP   = 11,  // F1 was pressed while hovering a button
  MKI_EXECMODE_UNDO          = 12,  // The user executed the "undo" function
  MKI_EXECMODE_REDO          = 13,  // The user executed the "redo" function
  MKI_EXECMODE_CLEAR_UNDO    = 14,  // The undo data shall be cleaned
  MKI_EXECMODE_RUNNING       = 15,  // Notification that the application has finished starting up
  MKI_EXECMODE_INTERFACE     = 16,  // Call of a custom plug-in interface command (see plug-in documentation)
  MKI_EXECMODE_LOAD_COMPLETE = 17,  // Loading or merging a file has been completed
};

enum
{
  MKI_COMMAND_DIRECT   = 0, // Directly executed command (no mouse input required)
  MKI_COMMAND_LOCAL    = 1, // Local command with a fixed number of point entries
  MKI_COMMAND_FIXED    = 2, // Global command with a fixed number of point entries
  MKI_COMMAND_VARIABLE = 3, // Global command with a variable number of point entries
};

enum
{
  MKI_COMMANDMODE_MASK        = 0x0000ffff, // Basic command modes
  MKI_COMMANDMODE_TRANSPARENT = 0x00010000, // The command is "transparent", i.e. it only interrupts the active command
  MKI_COMMANDMODE_DUPLICATE   = 0x00020000, // The command uses the duplicate function
  MKI_COMMANDMODE_MULTILINE   = 0x00040000, // The command uses the multiline function
  MKI_COMMANDMODE_GROUP_SUB   = 0x00080000, // The command allows identification inside a group
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum : __int16
{
  MKI_DB_OWNER_DEFAULT = 0,
  MKI_DB_OWNER_MK      = 1000,
  MKI_DB_OWNER_GKG     = 1907,
  MKI_DB_OWNER_GM_     = 1908,
};

enum : __int16
{
  MKI_DB_TYPE_NATIVE    = 0,
  MKI_DB_TYPE_INT32     = 1,
  MKI_DB_TYPE_DOUBLE    = 2,
  MKI_DB_TYPE_POINT     = 3,
  MKI_DB_TYPE_COLOR     = 4,
  MKI_DB_TYPE_PROPERTY  = 5,
  MKI_DB_TYPE_XPROPERTY = 6,
  MKI_DB_TYPE_FONTDEF   = 7,
  MKI_DB_TYPE_TEXT      = 8,
  MKI_DB_TYPE_BINARY    = 9,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_DB_TEMP_MIN            = -100, // First number of a temporary data block
  MKI_DB_TEMP_T001           = -1,
  MKI_DB_TEMP_T002           = -2,
  MKI_DB_TEMP_T003           = -3,
  MKI_DB_TEMP_T004           = -4,
  MKI_DB_TEMP_T005           = -5,
  MKI_DB_TEMP_T006           = -6,
  MKI_DB_TEMP_T007           = -7,
  MKI_DB_TEMP_T008           = -8,
  MKI_DB_TEMP_MAX            = 0,    // Range end of temporary data blocks
                             
  MKI_DB_POINT_MIN           = 0,    // First number of a movable point data block
  MKI_DB_POINT_ANY           = 0,
  MKI_DB_POINT_START         = 1,
  MKI_DB_POINT_END           = 2,
  MKI_DB_POINT_CENTER        = 3,
  MKI_DB_POINT_RADIUS        = 4,
  MKI_DB_POINT_ANGLE         = 5,
  MKI_DB_POINT_VECTOR        = 6,
  MKI_DB_POINT_PIVOT1        = 7,
  MKI_DB_POINT_PIVOT2        = 8,
  MKI_DB_POINT_ARC           = 9,
  MKI_DB_POINT_MARK          = 10,
  MKI_DB_POINT_CORNER        = 11,
  MKI_DB_POINT_MAX           = 50,   // Range end of movable point data blocks
                             
  MKI_DB_FIXED_MIN           = 50,   // First number of a non-movable point block
  MKI_DB_FIXED_ANY           = 50,
  MKI_DB_FIXED_START         = 51,
  MKI_DB_FIXED_END           = 52,
  MKI_DB_FIXED_CENTER        = 53,
  MKI_DB_FIXED_RADIUS        = 54,
  MKI_DB_FIXED_ANGLE         = 55,
  MKI_DB_FIXED_VECTOR        = 56,
  MKI_DB_FIXED_PIVOT1        = 57,
  MKI_DB_FIXED_PIVOT2        = 58,
  MKI_DB_FIXED_ARC           = 59,
  MKI_DB_FIXED_MARK          = 60,
  MKI_DB_FIXED_CORNER        = 61,
  MKI_DB_FIXED_MAX           = 100,  // Range end of non-movable point blocks
                             
  MKI_DB_ALL_MIN             = 100,  // First number of a standard data block
  MKI_DB_ALL_CONSTANT        = 100,
  MKI_DB_ALL_ARC             = 101,
  MKI_DB_ALL_CURVE           = 102,
  MKI_DB_ALL_TEXT            = 110,
  MKI_DB_ALL_MAX             = 200,  // Range end of standard data blocks
                             
  MKI_DB_INFO_MIN            = 200,  // First number of an object-specific data block
  MKI_DB_INFO_DIMLINE        = 220,
  MKI_DB_INFO_DIMLARGE       = 225,
  MKI_DB_INFO_DIMSMALL       = 230,
  MKI_DB_INFO_TEXTSTANDARD   = 235,
  MKI_DB_INFO_TEXTFRAME      = 236,
  MKI_DB_INFO_TEXTREFERENCE  = 237,
  MKI_DB_INFO_COMMENTDATA    = 240,
  MKI_DB_INFO_BITMAPREF      = 243,
  MKI_DB_INFO_MAX            = 300,  // Range end of object-specific data blocks
                             
  MKI_DB_ATTRIB_MIN          = 300,  // First number of an attribute data block
  MKI_DB_ATTRIB_GLOBAL_MIN   = 300,
  MKI_DB_ATTRIB_GLOBAL_TEXT  = 300,
  MKI_DB_ATTRIB_GLOBAL_NUM   = 301,
  MKI_DB_ATTRIB_GLOBAL_URL   = 302,
  MKI_DB_ATTRIB_GLOBAL_MAX   = 400,
  MKI_DB_ATTRIB_LOCAL_MIN    = 400,
  MKI_DB_ATTRIB_LOCAL_TEXT   = 400,
  MKI_DB_ATTRIB_LOCAL_NUM    = 401,
  MKI_DB_ATTRIB_LOCAL_URL    = 402,
  MKI_DB_ATTRIB_LOCAL_MAX    = 500,
  MKI_DB_ATTRIB_OUTLINE_MIN  = 500,
  MKI_DB_ATTRIB_OUTLINE_TEXT = 500,
  MKI_DB_ATTRIB_OUTLINE_NUM  = 501,
  MKI_DB_ATTRIB_OUTLINE_URL  = 502,
  MKI_DB_ATTRIB_OUTLINE_MAX  = 600,
  MKI_DB_ATTRIB_POINT_MIN    = 600,
  MKI_DB_ATTRIB_POINT_TEXT   = 600,
  MKI_DB_ATTRIB_POINT_NUM    = 601,
  MKI_DB_ATTRIB_POINT_URL    = 602,
  MKI_DB_ATTRIB_POINT_MAX    = 700,
  MKI_DB_ATTRIB_MAX          = 700,  // Range end of attribute data blocks
                             
  MKI_DB_END                 = 999,

  MKI_DB_CUSTOM_MIN          = 1000,  // First number of a custom data block for extended objects
  MKI_DB_CUSTOM_MAX          = 30000, // Range end of custom data blocks for extended objects
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum : __int16
{
  MKI_ENTITYTYPE_OBJECT   = 0,
  MKI_ENTITYTYPE_INSTANCE = 1,
  MKI_ENTITYTYPE_BLOCK    = 2,
  MKI_ENTITYTYPE_CLIP     = 3,
  MKI_ENTITYTYPE_REF      = 4,
  MKI_ENTITYTYPE_PATCH    = 5,
  MKI_ENTITYTYPE_MAPPING  = 6,
  MKI_ENTITYTYPE_GROUP    = 7,
  MKI_ENTITYTYPE_HATCH    = 8,
  MKI_ENTITYTYPE_EXT      = 9,
  MKI_ENTITYTYPE_ATTRIB   = 10,

  MKI_ENTITYTYPE_END      = 999,
};

enum
{
  MKI_ENTITYMODE_NONE         = 0x00000000,
  MKI_ENTITYMODE_HIDDEN       = 0x00000001,
  MKI_ENTITYMODE_EXT_NO_IDENT = 0x00000002,
  MKI_ENTITYMODE_EXT_NO_SNAP  = 0x00000004,
  MKI_ENTITYMODE_EXT_NO_PRINT = 0x00000008,
};

enum
{
  MKI_EXTFLAG_NONE      = 0x00000000,
  MKI_EXTFLAG_CURVE     = 0x00000001,
  MKI_EXTFLAG_SURFACE   = 0x00000002,
  MKI_EXTFLAG_TEXT      = 0x00000004,
  MKI_EXTFLAG_ATTRIB    = 0x00000008,
  MKI_EXTFLAG_DIM       = 0x00000010,
  MKI_EXTFLAG_TEXTPROP  = 0x00000020,
  MKI_EXTFLAG_INSTPROP  = 0x00000040,
  MKI_EXTFLAG_MULTITEXT = 0x00000080,
  MKI_EXTFLAG_PARAM     = 0x00000100,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_OBJ_LINE          = 0,
  MKI_OBJ_HATCH         = 1,
  MKI_OBJ_BEZIER        = 2,

  MKI_OBJ_CIRCLE        = 5,
  MKI_OBJ_ARC           = 6,
  MKI_OBJ_SECTOR        = 7,
  MKI_OBJ_SEGMENT       = 8,

  MKI_OBJ_ZIGZAG        = 10,
  MKI_OBJ_SPLINE        = 11,
  MKI_OBJ_CURVE         = 12,
  MKI_OBJ_SURFACE       = 13,

  MKI_OBJ_ELLIPSE       = 15,
  MKI_OBJ_EARC          = 16,
  MKI_OBJ_ESECTOR       = 17,
  MKI_OBJ_ESEGMENT      = 18,

  MKI_OBJ_DLINE         = 20,
  MKI_OBJ_DARC          = 21,
  MKI_OBJ_DEARC         = 22,

  MKI_OBJ_DDISTANCE     = 25,
  MKI_OBJ_DRADIUS       = 26,
  MKI_OBJ_DDIAMETER     = 27,
  MKI_OBJ_DANGLE        = 28,
  MKI_OBJ_DARCLENGTH    = 29,
  MKI_OBJ_DCOORDINATE   = 30,
  MKI_OBJ_DAREA         = 31,
  MKI_OBJ_DPERIMETER    = 32,
  MKI_OBJ_DTEXT         = 33,

  MKI_OBJ_TEXTSTANDARD  = 35,
  MKI_OBJ_TEXTFRAME     = 36,
  MKI_OBJ_TEXTREFERENCE = 37,
  
  MKI_OBJ_HYPERLINK     = 39,
  MKI_OBJ_COMMENT       = 40,
  MKI_OBJ_MARK          = 41,
  // - reserved         = 42,
  MKI_OBJ_BITMAPREF     = 43,
  
  MKI_OBJ_GEOENDLESS    = 45,
  MKI_OBJ_GEOCIRCLE     = 46,
  MKI_OBJ_GEOELLIPSE    = 47,
  
  MKI_OBJ_GUIDECURVE    = 48,
  MKI_OBJ_CONTAINER     = 49,
  MKI_OBJ_CALLBACK      = 50,

  MKI_OBJ_GEOBEAM       = 53,
  MKI_OBJ_GEOLINE       = 54,
  MKI_OBJ_GEOCURVE      = 55,
  MKI_OBJ_GEOSURFACE    = 56,
  MKI_OBJ_GEOARC        = 57,
  MKI_OBJ_GEOSECTOR     = 58,
  MKI_OBJ_GEOSEGMENT    = 59,
  MKI_OBJ_GEOEARC       = 60,
  MKI_OBJ_GEOESECTOR    = 61,
  MKI_OBJ_GEOESEGMENT   = 62,

  MKI_OBJ_TOTAL         = 64,   // Number of "known" object types (must be a multiple of 8)
  
  MKI_OBJ_END           = 999,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  //----------------------------------------------
  // Company:             Malz++Kassner GmbH
  // Plug-In Owner ID:    MKI_DB_OWNER_MK
  // Plug-In Prefix:      MK
  MKI_OBJ_SAMPLE_STAR         = 0x0000, // SAMPLE_.DLL - Regular Star

  MKI_OBJ_EXT_DIM_INCREMENT1  = 0x0010, // Incremental Dimension, Standard
  MKI_OBJ_EXT_DIM_INCREMENT2  = 0x0011, // Incremental Dimension, Architectural
  MKI_OBJ_EXT_DIM_REFERENCE1  = 0x0012, // Reference Dimension, Single Line
  MKI_OBJ_EXT_DIM_REFERENCE2  = 0x0013, // Reference Dimension, Staggered

  MKI_OBJ_EXT_BEZIER_POLY     = 0x0020, // Bezier Curves, Polyline
  MKI_OBJ_EXT_BEZIER_DEBOOR   = 0x0021, // Bezier Curves, DeBoor
  MKI_OBJ_EXT_SPIRAL          = 0x0022, // Spiral
  MKI_OBJ_EXT_ARROW           = 0x0023, // Arrow

  MKI_OBJ_EXT_FILL_BLOCK      = 0x0030, // Block Fill
  MKI_OBJ_EXT_FILL_GRADIENT   = 0x0031, // Gradient Fill
  MKI_OBJ_EXT_FILL_OUTLINE    = 0x0032, // Gradient Outline

  MKI_OBJ_EXT_3D_ASSEMBLY     = 0x0040, // 3D Part
  MKI_OBJ_EXT_3D_MAPPING      = 0x0041, // 3D Mapping

  MKI_OBJ_EXT_TEXT_ON_PATH    = 0x0050, // Text on Path

  MKI_OBJ_EXT_LABEL_ISO1302   = 0x0060, // Surface Finish ISO 1302
  MKI_OBJ_EXT_LABEL_ISO1101A  = 0x0061, // Shape/Position Tolerance ISO 1101
  MKI_OBJ_EXT_LABEL_ISO1101B  = 0x0062, // Tolerance Datum ISO 1101
  MKI_OBJ_EXT_LABEL_DIN6784   = 0x0063, // Workpiece Edge DIN 6784
  MKI_OBJ_EXT_LABEL_DIN332    = 0x0064, // Center Hole DIN 332 T10

  MKI_OBJ_EXT_TABLE           = 0x0070, // Table

  MKI_OBJ_EXT_PARAMETRIC      = 0x0080, // Parametric Compound


  //----------------------------------------------
  // Company:             Gtting KG
  // Plug-In Owner ID:    1907
  // Plug-In Prefix:      GKG
  MKI_OBJ_FTS1_SEGMENT        = 0x0000, // GKGFTS1_.DLL - FTS Segment
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PLUGINID_MKTEXT2  = 0,   // Text on Path
  MKI_PLUGINID_MKMOD2   = 1,   // 3D Part
  MKI_PLUGINID_MKTEXT1  = 2,   // Lettering
  MKI_PLUGINID_MKDIM1   = 3,   // Incremental & Reference Dimensioning
  MKI_PLUGINID_MKDRAW1  = 4,   // Advanced Drawing
  MKI_PLUGINID_MKPART1  = 9,   // Parametric Compound
  MKI_PLUGINID_MKFILL1  = 301, // Texture / Color Gradient
  MKI_PLUGINID_MKTABLE1 = 612, // Table
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_OUTPUTTYPE_NULL    = 0x0000, // No output
  MKI_OUTPUTTYPE_OUTLINE = 0x0001, // Output of outline
  MKI_OUTPUTTYPE_FILLING = 0x0002, // Output of filling
  MKI_OUTPUTTYPE_OTHER   = 0x0004, // Output of other things
  MKI_OUTPUTTYPE_TEXT    = 0x0008, // Output of text
  MKI_OUTPUTTYPE_BITMAP  = 0x0010, // Output of bitmap
  MKI_OUTPUTTYPE_NONTEXT = 0x0007, // Any non-text output
  MKI_OUTPUTTYPE_ANY     = 0x001f, // Any output
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  //----------------------------------------------
  // Company:   Malz++Kassner GmbH
  // Owner ID:  MKI_DB_OWNER_MK

  MKI_COLLECTION_TYPE_BASE_MIN = 0,

  MKI_COLLECTION_TYPE_FILTER       = 0, // Property Search / Filter
  MKI_COLLECTION_TYPE_SEARCH       = 1, // Text Search
  MKI_COLLECTION_TYPE_COLOR        = 2, // Color Definition
  MKI_COLLECTION_TYPE_FONT         = 3, // Font
  MKI_COLLECTION_TYPE_PROPERTY     = 4, // Property Set
  MKI_COLLECTION_TYPE_XPROPERTY    = 5, // Extended Property Set
  MKI_COLLECTION_TYPE_ATTRIBUTE    = 6, // Attribute
  MKI_COLLECTION_TYPE_BITMAPEXPORT = 7, // Bitmap Export
  MKI_COLLECTION_TYPE_VARIABLE     = 8, // Variable

  MKI_COLLECTION_TYPE_LAYERSET     = 20, // Set of Layers
  MKI_COLLECTION_TYPE_PENSET       = 21, // Set of Pens
  MKI_COLLECTION_TYPE_LINESET      = 22, // Set of Line Patterns
  MKI_COLLECTION_TYPE_HATCHSET     = 23, // Set of Hatching Types
  MKI_COLLECTION_TYPE_SYSTEMSET    = 24, // Set of Coordinate Systems
  MKI_COLLECTION_TYPE_COLORSET     = 25, // Set of Colors for Screen Display
  MKI_COLLECTION_TYPE_HIDELAYERS   = 26, // Hidden Layer information

  MKI_COLLECTION_TYPE_TEXT         = 40, // Text
  MKI_COLLECTION_TYPE_LAYERNAME    = 41, // Layer name
  MKI_COLLECTION_TYPE_PENNAME      = 42, // Pen name
  MKI_COLLECTION_TYPE_LINENAME     = 43, // Line Pettern name
  MKI_COLLECTION_TYPE_HATCHNAME    = 44, // Hatching Type name
  MKI_COLLECTION_TYPE_SYSTEMNAME   = 45, // Coordinate System name
  MKI_COLLECTION_TYPE_FILENAME     = 46, // Filename
  MKI_COLLECTION_TYPE_BLOCK        = 47, // Library and Block name
  MKI_COLLECTION_TYPE_NOTE         = 48, // Note

  MKI_COLLECTION_TYPE_BASE_MAX = 100,

  MKI_COLLECTION_TYPE_PLUGIN_MIN = 100,

  MKI_COLLECTION_TYPE_FILLBLOCK     = 100, // Block Pattern Filling
  MKI_COLLECTION_TYPE_FILLGRADIENT  = 101, // Color Gradient Filling
  MKI_COLLECTION_TYPE_FILLOUTLINE   = 102, // Outline Color Gradient
  MKI_COLLECTION_TYPE_ARROW         = 110, // Arrow
  MKI_COLLECTION_TYPE_SVG_IMPORT    = 113, // SVG Import Options
  MKI_COLLECTION_TYPE_AUTOCONVERTER = 114, // Auto-Converter Options
  MKI_COLLECTION_TYPE_SVG_EXPORT    = 115, // SVG Export Options

  MKI_COLLECTION_TYPE_HPGL_EXPORT   = 116, // HP-GL/2 Export Options
  MKI_COLLECTION_TYPE_HPGL_IMPORT   = 117, // HP-GL/2 Import Options
  MKI_COLLECTION_TYPE_SHP_EXPORT    = 118, // ESRI Shape Export Options
  MKI_COLLECTION_TYPE_SHP_IMPORT    = 119, // ESRI Shape Import Options
  MKI_COLLECTION_TYPE_DXF_EXPORT    = 120, // DXF Export Options
  MKI_COLLECTION_TYPE_DXF_IMPORT    = 121, // DXF Import Options
  MKI_COLLECTION_TYPE_DWG_EXPORT    = 122, // DWG Export Options
  MKI_COLLECTION_TYPE_DWG_IMPORT    = 123, // DWG Import Options

  MKI_COLLECTION_TYPE_PLUGIN_MAX = 200,

};


//---------------------------------------------------------------------------------------------------------------------------------------------------
constexpr wchar_t MKI_CHAR_TAB      = L'';   // Identification character for "horizontal tab"
constexpr wchar_t MKI_CHAR_NEWLINE  = L'';   // Identification character for "new line"
constexpr char    MKI_CHAR_SYSTEM   = '*';    // Identification character for system names
constexpr char    MKI_CHAR_INTERN   = '#';    // Identification character for internal definitions
constexpr char    MKI_CHAR_VARIABLE = '~';    // Identification character for variables
constexpr char    MKI_CHAR_TERM_IN  = '{';    // Identification character for term beginning
constexpr char    MKI_CHAR_TERM_OUT = '}';    // Identification character for term end
constexpr char    MKI_CHAR_ESC      = '\\';   // Identification character for special characters
constexpr char    MKI_CHAR_GROUP    = 'G';    // Identification character for groups
constexpr char    MKI_CHAR_POSNO    = 'P';    // Identification character for position numbers
constexpr char    MKI_END_CHAR      = '\x04'; // End-of-list identification in text lists (char)

#define MKI_END_TEXTW L"\x04" // End-of-list identification in text lists (Unicode)
#define MKI_END_TEXTA "\x04"  // End-of-list identification in text lists (Ansi)


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_DIMMODE_NUMREFRESH      = 0x00000001,
  MKI_DIMMODE_NUMCENTERED     = 0x00000002,
  MKI_DIMMODE_NUMPLACING      = 0x00000004,
  MKI_DIMMODE_NUMROTATE       = 0x00000008,
  MKI_DIMMODE_ARROWSTARTFORM  = 0x00000010,
  MKI_DIMMODE_ARROWSTARTMODE  = 0x00000020,
  MKI_DIMMODE_ARROWENDFORM    = 0x00000040,
  MKI_DIMMODE_ARROWENDMODE    = 0x00000080,
  MKI_DIMMODE_EXTSTARTDISPLAY = 0x00000100,
  MKI_DIMMODE_EXTENDDISPLAY   = 0x00000200,
  MKI_DIMMODE_DIMFLAG         = 0x00000400,
  MKI_DIMMODE_LINEDISPLAY     = 0x00000800,
  MKI_DIMMODE_LINEORIENTATION = 0x00001000,
  MKI_DIMMODE_LINETYPE        = 0x00002000,
  MKI_DIMMODE_LINEDISTMODE    = 0x00004000,
  MKI_DIMMODE_LINEDISTANCE    = 0x00008000,
  MKI_DIMMODE_GLOBALPARAM     = 0x00010000,
  MKI_DIMMODE_CHARDISTANCE    = 0x00020000,
  MKI_DIMMODE_TABDISTANCE     = 0x00040000,
  MKI_DIMMODE_TEXTMODE        = 0x00080000,
  MKI_DIMMODE_LINEOFFSET      = 0x00100000,
  MKI_DIMMODE_NO_LINE_ALIGN   = 0x01000000,
  MKI_DIMMODE_NO_LINE_FREE    = 0x02000000,
  MKI_DIMMODE_NO_LINE_AUTO    = 0x04000000,
  MKI_DIMMODE_NO_NUM_FREE     = 0x08000000,
  MKI_DIMMODE_NO_NUM_OUTSIDE  = 0x10000000,
  MKI_DIMMODE_NO_NUM_DOUBLE   = 0x20000000,
  MKI_DIMMODE_ALL             = 0x00ffffff,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PROPMODE_ALL           = 0x00000000,
  MKI_PROPMODE_NO_PAGE       = 0x00000001,
  MKI_PROPMODE_NO_LAYER      = 0x00000002,
  MKI_PROPMODE_NO_PEN        = 0x00000004,
  MKI_PROPMODE_BYLAYER       = 0x00000008,
  MKI_PROPMODE_READONLY      = 0x00000010,
  MKI_PROPMODE_CURRENT_LAYER = 0x00000020,
  MKI_PROPMODE_CURRENT_PEN   = 0x00000040,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_LINEMODE_OPTIMIZE_LOCAL  = 0x00000000,
  MKI_LINEMODE_OPTIMIZE_GLOBAL = 0x00000001,
  MKI_LINEMODE_OPTIMIZE_MASK   = 0x0000000F,

  MKI_LINEMODE_ZOOM_APPLY  = 0x00000000,
  MKI_LINEMODE_ZOOM_IGNORE = 0x00000010,
  MKI_LINEMODE_ZOOM_MASK   = 0x000000F0,
                               
  MKI_LINEMODE_CAP_ROUND  = 0x00000000,
  MKI_LINEMODE_CAP_SQUARE = 0x00000100,
  MKI_LINEMODE_CAP_FLAT   = 0x00000200,
  MKI_LINEMODE_CAP_MASK   = 0x00000F00,
                          
  MKI_LINEMODE_JOIN_ROUND = 0x00000000,
  MKI_LINEMODE_JOIN_BEVEL = 0x00001000,
  MKI_LINEMODE_JOIN_MITER = 0x00002000,
  MKI_LINEMODE_JOIN_MASK  = 0x0000F000,

  MKI_LINEMODE_LINE_MASKS = MKI_LINEMODE_CAP_MASK | MKI_LINEMODE_JOIN_MASK,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_COLORHARMONY_UNKNOWN = 0,
  MKI_COLORHARMONY_2       = 1,
  MKI_COLORHARMONY_3       = 2,
  MKI_COLORHARMONY_3_60    = 3,
  MKI_COLORHARMONY_4       = 4,
  MKI_COLORHARMONY_4_60    = 5,
  MKI_COLORHARMONY_4_30    = 6, 
  MKI_COLORHARMONY_5       = 7,
  MKI_COLORHARMONY_5_30    = 8,
  MKI_COLORHARMONY_6       = 9,
  MKI_COLORHARMONY_6_30    = 10,
  MKI_COLORHARMONY_ANALOG  = 11,
  MKI_COLORHARMONY_SHADES  = 12,
  MKI_COLORHARMONY_MAX     = 13,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_FILLMODE_FRAMED        = 0,
  MKI_FILLMODE_FILLED        = 1,
  MKI_FILLMODE_FILLED_FRAMED = 2,
  MKI_FILLMODE_ERASER        = 3,
  MKI_FILLMODE_ERASER_FRAMED = 4,
  MKI_FILLMODE_MAX           = 5,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_DIMZEROMODE_STRIP = 0,
  MKI_DIMZEROMODE_ONE   = 1,
  MKI_DIMZEROMODE_ALL   = 2,
  MKI_DIMZEROMODE_MAX   = 3,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_DIMEXTLINE_RADIAL   = 0,
  MKI_DIMEXTLINE_PARALLEL = 1,
  MKI_DIMEXTLINE_MAX      = 2,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_SNAPMODE_CENTER       = 0x00000001,
  MKI_SNAPMODE_QUADRANT     = 0x00000002,
  MKI_SNAPMODE_EDGE         = 0x00000004,
  MKI_SNAPMODE_CORNER       = 0x00000008,
  MKI_SNAPMODE_INTERSECTION = 0x00000010,
  MKI_SNAPMODE_SPECIAL      = 0x00000080,
  MKI_SNAPMODE_RELATIVE     = 0x00000100,

  MKI_SNAPMODE_BUFFERED     = 0x000000f8, // Buffered snapping modes
  MKI_SNAPMODE_BASIC        = 0x000000ff, // Basic snapping modes
  MKI_SNAPMODE_EXTRA        = 0x00000f00, // Additional snapping Modes
  MKI_SNAPMODE_SECONDARY    = MKI_SNAPMODE_QUADRANT | MKI_SNAPMODE_EDGE | MKI_SNAPMODE_SPECIAL | MKI_SNAPMODE_RELATIVE,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_SNAPTYPE_NONE         = 0,

  MKI_SNAPTYPE_CENTER       = 1,
  MKI_SNAPTYPE_QUADRANT     = 2,
  MKI_SNAPTYPE_EDGE         = 3,
  MKI_SNAPTYPE_CORNER       = 4,
  MKI_SNAPTYPE_INTERSECTION = 5,
  MKI_SNAPTYPE_MARK         = 6,
  MKI_SNAPTYPE_PIVOT        = 7,
  MKI_SNAPTYPE_RADIUS       = 8,
  MKI_SNAPTYPE_VECTOR       = 9,
  MKI_SNAPTYPE_ANGLE        = 10,
  MKI_SNAPTYPE_DIMENSION    = 11,
  MKI_SNAPTYPE_TEXT         = 12,
  MKI_SNAPTYPE_INSTANCE     = 13,
  MKI_SNAPTYPE_BITMAP       = 14,
  MKI_SNAPTYPE_REFERENCE    = 15,
  MKI_SNAPTYPE_TASKFRAME    = 16,
  MKI_SNAPTYPE_PAGECORNER   = 17,
  MKI_SNAPTYPE_PAGEGUIDE    = 18,
  MKI_SNAPTYPE_ORIGIN       = 19,
  MKI_SNAPTYPE_USERINPUT    = 20,
  MKI_SNAPTYPE_ANY          = 21,

  MKI_SNAPTYPE_TOTAL        = 22,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_ORTHOMODE_USER = 0,
  MKI_ORTHOMODE_90   = 1,
  MKI_ORTHOMODE_45   = 2,
  MKI_ORTHOMODE_30   = 3,
  MKI_ORTHOMODE_22_5 = 4,
  MKI_ORTHOMODE_15   = 5,
  MKI_ORTHOMODE_5    = 6,
  MKI_ORTHOMODE_MAX  = 7,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_IDENTMODE_LINE       = 0x00000001,
  MKI_IDENTMODE_CIRCLE     = 0x00000002,
  MKI_IDENTMODE_ARC        = 0x00000004,
  MKI_IDENTMODE_ELLIPSE    = 0x00000008,
  MKI_IDENTMODE_EARC       = 0x00000010,
  MKI_IDENTMODE_GEOENDLESS = 0x00000020,
  MKI_IDENTMODE_GEOCIRCLE  = 0x00000040,
  MKI_IDENTMODE_GEOELLIPSE = 0x00000080,
  MKI_IDENTMODE_BEZIER     = 0x00000100,
  MKI_IDENTMODE_GEOLINE    = 0x00000200,
  MKI_IDENTMODE_GEOARC     = 0x00000400,
  MKI_IDENTMODE_GEOEARC    = 0x00000800,
  MKI_IDENTMODE_GEOBEZIER  = 0x00001000,
  MKI_IDENTMODE_GEOBEAM    = 0x00002000,

  MKI_IDENTMODE_SNAP       = 0x01000000,
  MKI_IDENTMODE_NOCHOICE   = 0x02000000,
  MKI_IDENTMODE_NOPROMPT   = 0x04000000,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_POSNO_MODE_NUMERICAL   = 0x00000001,
  MKI_POSNO_MODE_COPY_ATTRIB = 0x00000002,
  MKI_POSNO_MODE_EDIT_ATTRIB = 0x00000004,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_LINEDISPLAY_MINIMUM = 0, // Display lines with minimum width, no line pattern
  MKI_LINEDISPLAY_NARROW  = 1, // Display lines with minimum width, use line pattern
  MKI_LINEDISPLAY_FIXED   = 2, // Display lines with unscaled width, use line pattern
  MKI_LINEDISPLAY_SCALED  = 3, // Display lines with scaled width, use line pattern
  MKI_LINEDISPLAY_MAX     = 4,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_POINTDISPLAY_POINTS   = 0x00000001, // Display definition points
  MKI_POINTDISPLAY_ARROWS   = 0x00000002, // Display direction arrows
  MKI_POINTDISPLAY_NUMBERS  = 0x00000004, // Display object numbers
  MKI_POINTDISPLAY_ATTRIB_L = 0x00000008, // Display local attributes
  MKI_POINTDISPLAY_ATTRIB_O = 0x00000010, // Display outline attributes
  MKI_POINTDISPLAY_ATTRIB_P = 0x00000020, // Display point attributes
  MKI_POINTDISPLAY_ATTRIB_G = 0x00000040, // Display global attributes
  MKI_POINTDISPLAY_SELONLY  = 0x00000100, // Display all of the above only for permanently selected objects
  MKI_POINTDISPLAY_GEO      = 0x00000200, // Display all of the above also for Construction Aid objects
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_ARCMODE_ARC     = 0, // Circle / ellipse arc
  MKI_ARCMODE_SECTOR  = 1, // Circle / ellipse sector
  MKI_ARCMODE_SEGMENT = 2, // Circle / ellipse segment
  MKI_ARCMODE_MAX     = 3,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_GEOMODE_DEFAULT    = 0, // According to normal object order
  MKI_GEOMODE_BACKGROUND = 1, // All Construction Aid objects in background
  MKI_GEOMODE_FOREGROUND = 2, // All Construction Aid objects in foreground
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_TRACKMODE_OBJECTS   = 0x00000001, // Use standard drawing objects during surface generation / tracking
  MKI_TRACKMODE_TEXTS     = 0x00000002, // Use texts during surface generation / tracking
  MKI_TRACKMODE_HATCHINGS = 0x00000004, // Use hatchings during surface generation / tracking
  MKI_TRACKMODE_GEO       = 0x00000008, // Use Construction Aid objects during surface generation / tracking
  MKI_TRACKMODE_TEMP      = 0x00000010, // Use temporary objects during surface generation / tracking

  MKI_TRACKMODE_DEFAULT   = MKI_TRACKMODE_GEO | MKI_TRACKMODE_TEMP | MKI_TRACKMODE_OBJECTS,  // All usual types of objects
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_TRACKTYPE_UNDEFINED = 0, // Undefined object type
  MKI_TRACKTYPE_OBJECTS   = 1, // Standard drawing object
  MKI_TRACKTYPE_GEO       = 2, // Construction Aid object
  MKI_TRACKTYPE_TEMP      = 3, // Temporary object
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_OUTPUTFRAME_PAGE    = 0, // Page frame
  MKI_OUTPUTFRAME_OBJECTS = 1, // Object frame
  MKI_OUTPUTFRAME_MAX     = 2,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_OUTPUTPOINT_CROSS_90 = 0x00000001, // Vertical / horizontal cross
  MKI_OUTPUTPOINT_CROSS_45 = 0x00000002, // Diagonal cross
  MKI_OUTPUTPOINT_CIRCLE   = 0x00000004, // Circle
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_DISPLAYMODE_DEFAULT        = 0x00000000, // Display instances/drawings/extended objects with default mode
  MKI_DISPLAYMODE_SCALED_WIDTH   = 0x00000001, // Display instances/drawings/extended objects with scaled line widths
  MKI_DISPLAYMODE_NOPOINTS       = 0x00000002, // Display instances/drawings/extended objects without points
  MKI_DISPLAYMODE_NOSNAP         = 0x00000004, // Ignore clippings for snapping and identification
  MKI_DISPLAYMODE_CONFINED       = 0x00000008, // Display instances/drawings using their inherent layers/pens/linetypes/coordinate systems
  MKI_DISPLAYMODE_NOGEO          = 0x00000010, // Display instances/drawings without any Construction Aid objects
  MKI_DISPLAYMODE_ADAPT_ANGLE    = 0x00000020, // Rotate texts/instances/drawings according to current page or model space
  MKI_DISPLAYMODE_SCALED_LENGTH  = 0x00000040, // Display instances/drawings/extended objects with scaled line pattern lengths
  MKI_DISPLAYMODE_EMBEDDED       = 0x00000080, // Display local display data instead of referenced block's data (passive link)
  MKI_DISPLAYMODE_SCALED_DIMTEXT = 0x00000100, // For internal use only
  MKI_DISPLAYMODE_NOSMOOTHING    = 0x00000200, // Display extended objects without smoothing
  MKI_DISPLAYMODE_ADAPT_SCALE    = 0x00000400, // Scale texts/instances/drawings according to current page or model space
  MKI_DISPLAYMODE_NOLISTPOINTS   = 0x00000800, // Display extended objects without points in display list
  MKI_DISPLAYMODE_NOHATCHUPDATE  = 0x00001000, // Do not update dynamic hatchings' display list but update their display matrix instead
  MKI_DISPLAYMODE_NOHIDE         = 0x00002000, // Do not hide instances due to their layer or pen assignment (or that of the referenced block)
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_BACKGROUNDCOLOR_NONE    = -1,
  MKI_BACKGROUNDCOLOR_DEFAULT = 0,
  MKI_BACKGROUNDCOLOR_WHITE   = 1,
  MKI_BACKGROUNDCOLOR_GRAY    = 2,
  MKI_BACKGROUNDCOLOR_BLACK   = 3,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PAGELAYERS_SHOW = 0, // Display all layers
  MKI_PAGELAYERS_GRAY = 1, // Gray all layers that are hidden in the current page
  MKI_PAGELAYERS_HIDE = 2, // Hide and lock all layers that are hidden in the current page
  MKI_PAGELAYERS_MAX  = 3,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PAGEDISPLAY_CLASSIC         = 0, // Display all objects
  MKI_PAGEDISPLAY_PAGES_NO_MODEL  = 1, // Display all objects assigned to pages
  MKI_PAGEDISPLAY_LAYOUT          = 2, // Display all objects assigned to the active page
  MKI_PAGEDISPLAY_PAGE_PLUS_MODEL = 3, // Display all objects assigned to model space or active page
  MKI_PAGEDISPLAY_MODEL_ONLY      = 4, // Display all objects assigned to model space
  MKI_PAGEDISPLAY_MAX             = 5,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_BOUNDARYLINE_BITMAP   = 0x00000001, // Display boundary lines for bitmaps
  MKI_BOUNDARYLINE_TEXT     = 0x00000002, // Display boundary lines for frame texts
  MKI_BOUNDARYLINE_MAP      = 0x00000004, // Display boundary lines for mappings
  MKI_BOUNDARYLINE_CLIP     = 0x00000008, // Display boundary lines for clippings
  MKI_BOUNDARYLINE_HATCH    = 0x00000010, // Display boundary lines for hatchings
  MKI_BOUNDARYLINE_OVERFLOW = 0x00000020, // Display text overflow highlights
  MKI_BOUNDARYLINE_ALL      = 0x000000ff, // Display all boundary lines
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PAGEOUTPUT_ALL          = 0, // Output all objects
  MKI_PAGEOUTPUT_PAGES        = 1, // Output all objects assigned to pages
  MKI_PAGEOUTPUT_ACTIVE       = 2, // Output all objects assigned to the active page
  MKI_PAGEOUTPUT_MODEL_ACTIVE = 3, // Output all objects assigned to model space or active page
  MKI_PAGEOUTPUT_MODEL        = 4, // Output all objects assigned to model space
  MKI_PAGEOUTPUT_LIKE_SCREEN  = 5, // Output objects according to the screen settings
  MKI_PAGEOUTPUT_MAX          = 6,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PRINTDIALOG_DEFAULT  = 0x00000000, // Default behaviour
  MKI_PRINTDIALOG_NOPRINT  = 0x00000001, // Hide "Print" button
  MKI_PRINTDIALOG_TASK_PDF = 0x00000002, // Show only one virtual printer named "PDF Task" (no changes possible to device data)
  MKI_PRINTDIALOG_TASK_GDI = 0x00000004, // Show only one virtual printer named "GDI Task" (no changes possible to device data)
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_COLOROUTPUT_RGB  = 0, // Output colors as RGB
  MKI_COLOROUTPUT_GRAY = 1, // Output colors as grayscale
  MKI_COLOROUTPUT_MONO = 2, // Output colors as black and white
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_COMPARE_NONE         = 0x00000000, // Undefined
  MKI_COMPARE_EQUAL        = 0x00000001, // A == B
  MKI_COMPARE_NOT_EQUAL    = 0x00000002, // A != B
  MKI_COMPARE_LESS         = 0x00000003, // A <  B
  MKI_COMPARE_NOT_LESS     = 0x00000004, // A >= B
  MKI_COMPARE_GREATER      = 0x00000005, // A >  B
  MKI_COMPARE_NOT_GREATER  = 0x00000006, // A <= B
  MKI_COMPARE_EMPTY        = 0x00000007, // lstrlen( A ) == 0
  MKI_COMPARE_NOT_EMPTY    = 0x00000008, // lstrlen( A ) > 0
  MKI_COMPARE_CONTAINS     = 0x00000009, // strstr( A, B ) != nullptr
  MKI_COMPARE_NOT_CONTAINS = 0x0000000a, // strstr( A, B ) == nullptr
  MKI_COMPARE_PARTOF       = 0x0000000b, // strstr( B, A ) != nullptr
  MKI_COMPARE_NOT_PARTOF   = 0x0000000c, // strstr( B, A ) == nullptr
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_LOGICAL_NONE = 0x00000000, // Undefined
  MKI_LOGICAL_AND  = 0x00000001, // A & A
  MKI_LOGICAL_OR   = 0x00000002, // A | B
  MKI_LOGICAL_XOR  = 0x00000003, // A ^ B
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_WINDOWMODE_1   = 0, // 1 window
  MKI_WINDOWMODE_2X  = 1, // 2 windows (equally sized, left and right)
  MKI_WINDOWMODE_2Y  = 2, // 2 windows (equally sized, top and bottom)
  MKI_WINDOWMODE_3X  = 3, // 3 windows (1 large one, 2 small ones right to it)
  MKI_WINDOWMODE_3Y  = 4, // 3 windows (1 large one, 2 small ones below it)
  MKI_WINDOWMODE_4   = 5, // 4 windows (equally sized)
  MKI_WINDOWMODE_4X  = 6, // 4 windows (1 large one, 3 small ones right ot it)
  MKI_WINDOWMODE_4Y  = 7, // 4 windows (1 large one, 3 small ones below it)
  MKI_WINDOWMODE_MAX = 8,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_SETTING_DUPLICATE        = 0x00000000, // Duplicate status (true = Active, false = Inactive)
  MKI_SETTING_SNAPACTIVE       = 0x00000001, // Snap status (true = Active, false = Inactive)
  MKI_SETTING_SNAPMODE         = 0x00000002, // Snap modes (combination of MKI_SNAPMODE_???)
  MKI_SETTING_SNAPRADIUS       = 0x00000003, // Snap radius (5..50)
  MKI_SETTING_SNAPTOLERANCE    = 0x00000004, // Snap tolerance (0..10)
  MKI_SETTING_ORTHO            = 0x00000005, // Orthogonal mode (true = Active, false = Disabled)
  MKI_SETTING_ARCDIRECTION     = 0x00000006, // Arc direction (true = Positive, false = Negative)
  MKI_SETTING_ARCMODE          = 0x00000007, // Arc mode (0 = Arc, 1 = Sector, 2 = Segment)
  MKI_SETTING_AREAMODE         = 0x00000008, // Area mode (true = Inside, false = Partial)
  MKI_SETTING_GEODISPLAY       = 0x00000009, // Construction Aid display (true = Visible, false = Invisible)
  MKI_SETTING_GEOLOCKED        = 0x0000000a, // Construction Aid status (true = Locked, false = Editable)
  MKI_SETTING_GEOIDLE          = 0x0000000b, // Construction Aid status (true = Idle, false = Active)
  MKI_SETTING_GEOBACKGROUND    = 0x0000000c, // Construction Aid order (true = Background, false = Arbitrary)
  MKI_SETTING_PAGEAUTOASSIGN   = 0x0000000d, // Assign objects to active page (true = Yes, false = No)
  MKI_SETTING_MARKFORM         = 0x0000000e, // Mark display form (MKI_MARKFORM_*)
  MKI_SETTING_MARKSIZE         = 0x0000000f, // Mark display size
  MKI_SETTING_MARKZOOMABLE     = 0x00000010, // Mark size mode (true = zoomable, false = fixed)
  MKI_SETTING_MARKCOLORED      = 0x00000011, // Mark color mode (true = use line color, false = use system color)
  MKI_SETTING_GRIDFORM         = 0x00000012, // Grid display form (MKI_MARKFORM_*)
  MKI_SETTING_GRIDSIZE         = 0x00000013, // Grid display size

  MKI_SETTING_SMALLDLG_COLOR   = 0x00000100, // Use small version of color dialog (true = Yes, false = No)
  MKI_SETTING_SMALLDLG_TEXT    = 0x00000101, // Use small version of text dialog (true = Yes, false = No)
  MKI_SETTING_SMALLDLG_PROP    = 0x00000102, // Use small version of property dialog (true = Yes, false = No)
  MKI_SETTING_SMALLDLG_XPROP_P = 0x00000103, // Use small version of extended property dialog with passive pens (true = Yes, false = No)
  MKI_SETTING_SMALLDLG_XPROP_A = 0x00000104, // Use small version of extended property dialog with active pens (true = Yes, false = No)

  MKI_SETTING_COLOR_LAYERS     = 0x00001000, // Color of grayed layers
  MKI_SETTING_COLOR_GEOMETRY   = 0x00001001, // Color of default Construction Aid objects
  MKI_SETTING_COLOR_BLOCKS     = 0x00001002, // Color of block text display
  MKI_SETTING_COLOR_ERASERS    = 0x00001003, // Color of erasers
  MKI_SETTING_COLOR_POINTS     = 0x00001004, // Color of standard points
  MKI_SETTING_COLOR_MARKS      = 0x00001005, // Color of markings
  MKI_SETTING_COLOR_INPUT      = 0x00001006, // Color of input rubber lines
  MKI_SETTING_COLOR_OBJSEL     = 0x00001007, // Color of permanently selected objects
  MKI_SETTING_COLOR_OBJUSE     = 0x00001008, // Color of identified objects
  MKI_SETTING_COLOR_OBJIDENT   = 0x00001009, // Color of object highlight when identification is inaccurate
  MKI_SETTING_COLOR_POINTUSE   = 0x0000100a, // Color of selected points
  MKI_SETTING_COLOR_CURSOR1    = 0x0000100b, // Color of "large" cursor cross (active window)
  MKI_SETTING_COLOR_CURSOR2    = 0x0000100c, // Color of "small" cursor cross (inactive windows)
  MKI_SETTING_COLOR_PAGES      = 0x0000100d, // Color of pages
  MKI_SETTING_COLOR_DRAWAREA   = 0x0000100e, // Color of drawing area background
  MKI_SETTING_COLOR_HYPERLINKS = 0x0000100f, // Color of hyperlinks
  MKI_SETTING_COLOR_HIGHLIGHT  = 0x00001010, // Color of highlights
  MKI_SETTING_COLOR_TEMP       = 0x00001011, // Color of temporary objects
  MKI_SETTING_COLOR_NOTES      = 0x00001012, // Color of internal notes

  MKI_SETTING_COLOR_DIALOG_TEXT      = 0x00001100, // Skin-adapted color of window background
  MKI_SETTING_COLOR_DIALOG_BACK      = 0x00001101, // Skin-adapted color of window text
  MKI_SETTING_COLOR_DIALOG_GRAYTEXT  = 0x00001102, // Skin-adapted color of gray text
  MKI_SETTING_COLOR_CONTROL_TEXT     = 0x00001103, // Skin-adapted color of list control background
  MKI_SETTING_COLOR_CONTROL_BACK     = 0x00001104, // Skin-adapted color of list control text
  MKI_SETTING_COLOR_CONTROL_HIGHTEXT = 0x00001105, // Skin-adapted color of list control highlighted background
  MKI_SETTING_COLOR_CONTROL_HIGHBACK = 0x00001106, // Skin-adapted color of list control highlighted text
  MKI_SETTING_COLOR_CONTROL_GRAYTEXT = 0x00001107, // Skin-adapted color of list control gray text

  MKI_SETTING_DISPLAY_LINES       = 0x00002000, // Display line mode (LINEDISPLAY_?)
  MKI_SETTING_DISPLAY_FILLS       = 0x00002001, // Display fillings (true = Yes, false = No)
  MKI_SETTING_DISPLAY_ERASERS     = 0x00002002, // Display erasers dyed (true = Yes, false = No)
  MKI_SETTING_DISPLAY_TEXTS       = 0x00002003, // Display texts using original font (true = Yes, false = No)
  MKI_SETTING_DISPLAY_BOUNDARIES  = 0x00002004, // Display boundary lines (MKI_BOUNDARYLINE_*)
  MKI_SETTING_DISPLAY_VARS        = 0x00002005, // Evaluate variables (true = Yes, false = No)
  MKI_SETTING_DISPLAY_BITMAPS     = 0x00002006, // Display bitmaps (true = Yes, false = No)
  MKI_SETTING_DISPLAY_BLOCKS      = 0x00002007, // Display blocks (true = Yes, false = No)
  MKI_SETTING_DISPLAY_MARKS       = 0x00002008, // Display markings (true = Yes, false = No)
  MKI_SETTING_DISPLAY_POINT_SIZE  = 0x00002009, // Size of a point marker (__int32)
  MKI_SETTING_DISPLAY_POINTS      = 0x0000200a, // Display definition point markings (MKI_POINTDISPLAY_*)
  MKI_SETTING_DISPLAY_SKINSCALE   = 0x0000200b, // Current scaling factor in percent of skins elements (25..400)
  MKI_SETTING_DISPLAY_SCREENSCALE = 0x0000200c, // Current scaling factor in percent of the screen (50..200)
  MKI_SETTING_DISPLAY_DIALOGSCALE = 0x0000200d, // Current scaling factor in percent of dialogs and menus (25..400)
  MKI_SETTING_DISPLAY_OUTLINES    = 0x0000200e, // Display outlines (true = Yes, false = No)
  MKI_SETTING_DISPLAY_PERMONLY    = 0x0000200f, // Display only permanently selected objects (true = Yes, false = No)
  MKI_SETTING_DISPLAY_TERMS       = 0x00002010, // Evaluate terms in variables (true = Yes, false = No)
  MKI_SETTING_DISPLAY_REFERENCES  = 0x00002011, // Display references (true = Yes, false = No)
  MKI_SETTING_DISPLAY_HYPERLINKS  = 0x00002012, // Display hyperlinks dyed (true = Yes, false = No)
  MKI_SETTING_DISPLAY_PAGELAYERS  = 0x00002013, // Page layer display mode (MKI_PAGELAYERS_*)
  MKI_SETTING_DISPLAY_PAGEDISPLAY = 0x00002014, // Page object display mode (MKI_PAGEDISPLAY_*)
  MKI_SETTING_DISPLAY_PAGEGRAY    = 0x00002015, // Page object graying mode (true = Gray, false = Hidden)
  MKI_SETTING_DISPLAY_WINDOWMODE  = 0x00002016, // Drawing window arrangement mode (MKI_WINDOWMODE_*)

  MKI_SETTING_EXTERNAL_XPROP      = 0x00003000, // Display drawing using external properties (true = Yes, false = No)
  MKI_SETTING_LINE_OPTIMIZE       = 0x00003001, // Optimize line patterns (true = Yes, false = No)
  MKI_SETTING_FONT_RESOLVE        = 0x00003002, // Resolve TrueType font into curves (true = Yes, false = No)
  MKI_SETTING_FONT_MINIMUM        = 0x00003003, // Minimum TrueType font size (0..99)
  MKI_SETTING_FONT_DIALOG_REGULAR = 0x00003004, // Standard dialog window font (regular)
  MKI_SETTING_FONT_DIALOG_ITALIC  = 0x00003005, // Standard dialog window font (italic)
  MKI_SETTING_FONT_DIALOG_BOLD    = 0x00003006, // Standard dialog window font (bold)
  MKI_SETTING_FONT_DIALOG_STRIKE  = 0x00003007, // Standard dialog window font (strikeout)
  MKI_SETTING_FONT_DIALOG_SMALL   = 0x00003008, // Standard dialog window font (small)
  MKI_SETTING_FONT_DIALOG_HEIGHT  = 0x00003009, // Standard dialog window font height in pixels
  MKI_SETTING_FONT_SKIN_REGULAR   = 0x0000300a, // Standard skin font (regular)
  MKI_SETTING_FONT_SKIN_UNDERLINE = 0x0000300b, // Standard skin font (underlined)
  MKI_SETTING_FONT_SKIN_HEIGHT    = 0x0000300c, // Standard skin font height in pixels

  MKI_SETTING_OUTPUT_PAGELAYERS   = 0x00004000, // Page layer output mode (true = Show, false = Hide)
  MKI_SETTING_OUTPUT_PAGEDISPLAY  = 0x00004001, // Page object output mode (MKI_PAGEOUTPUT_*)

  MKI_SETTING_QUALITY_SCREEN  = 0x00005000, // Quality of screen display (unit is [1/1000mm], maximum quality is 1, minimum quality is 1000000)
  MKI_SETTING_QUALITY_PRINT   = 0x00005001, // Quality of print-out (unit is [1/1000mm], maximum quality is 1, minimum quality is 1000000)
  MKI_SETTING_QUALITY_CONVERT = 0x00005002, // Quality of conversion/export (unit is [1/1000mm], maximum quality is 1, minimum quality is 1000000)
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_CHANGECODE_NONE       = 0,
  MKI_CHANGECODE_SETTING    = 1,
  MKI_CHANGECODE_PRINTER    = 2,
  MKI_CHANGECODE_PROPERTY   = 3,
  MKI_CHANGECODE_TEXT       = 4,
  MKI_CHANGECODE_ATTRIBUTE  = 5,
  MKI_CHANGECODE_DELETE     = 6,
  MKI_CHANGECODE_MODIFY     = 7,
  MKI_CHANGECODE_ADD        = 8,
  MKI_CHANGECODE_COMMENT    = 9,
  MKI_CHANGECODE_VARIABLE   = 10,
  MKI_CHANGECODE_NOTE       = 11,
  MKI_CHANGECODE_COLLECTION = 12,
  MKI_CHANGECODE_DATABASE   = 13,
  MKI_CHANGECODE_BLOCK      = 14,
  MKI_CHANGECODE_TEXTSTYLE  = 15,
  MKI_CHANGECODE_POSNO      = 16,
  MKI_CHANGECODE_ELEMENT    = 17,
  MKI_CHANGECODE_HATCH      = 18,
  MKI_CHANGECODE_LAYER      = 19,
  MKI_CHANGECODE_LINE       = 20,
  MKI_CHANGECODE_PAGE       = 21,
  MKI_CHANGECODE_PEN        = 22,
  MKI_CHANGECODE_SYSTEM     = 23,
  MKI_CHANGECODE_BITMAP     = 24,
  MKI_CHANGECODE_FONT       = 25,
  MKI_CHANGECODE_COLOR      = 26,
  MKI_CHANGECODE_TASK       = 27,
  MKI_CHANGECODE_PLUGIN     = 28,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_FEEDBACK_IDLE      = 0,  // Nothing to do
  MKI_FEEDBACK_SNAP      = 1,  // No snap point
  MKI_FEEDBACK_OVERFLOW  = 2,  // Overflow during input
  MKI_FEEDBACK_POINT     = 3,  // Invalid point
  MKI_FEEDBACK_MOUSE     = 4,  // Undefined mouse click
  MKI_FEEDBACK_DATA      = 5,  // Invalid data
  MKI_FEEDBACK_OBJECT    = 6,  // Wrong object type
  MKI_FEEDBACK_UNDO      = 7,  // Undo complete
  MKI_FEEDBACK_REDO      = 8,  // Redo complete
  MKI_FEEDBACK_IDENT     = 9,  // Invalid identification
  MKI_FEEDBACK_FILTER    = 10, // Invalid choice (filter active)
  MKI_FEEDBACK_PAGE      = 11, // No page active
  MKI_FEEDBACK_NORESULT  = 12, // Calculation without result
  MKI_FEEDBACK_NOSURFACE = 13, // Surface generation without result
  MKI_FEEDBACK_MAX       = 14,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
typedef __int64 MKI_RESULTVALUE;
MKI_CLASS_TYPES( RESULTVALUE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum : __int64
{
  // Default value 0
  MKI_RESULT_OK                =  0, // Nothing to report

  // Errors are subsequent numerical values from 1 to 255
  MKI_RESULT_ERROR             =  1, // Error: Unspecified error
  MKI_RESULT_STOPPED           =  2, // Error: The operation was stopped (e.g. because the callback procedure returned false)
  MKI_RESULT_NOTHING           =  3, // Error: No resulting output or nothing to do
  MKI_RESULT_PARAMETER         =  4, // Error: Invalid parameter(s)
  MKI_RESULT_MEMORYFULL        =  5, // Error: Memory full
  MKI_RESULT_DISKFULL          =  6, // Error: Disk full
  MKI_RESULT_IDENT             =  7, // Error: Wrong / missing identification
  MKI_RESULT_INVALID           =  8, // Error: Invalid data
  MKI_RESULT_TYPE              =  9, // Error: Wrong type of entity or object
  MKI_RESULT_IMPOSSIBLE        = 10, // Error: No possible result
  MKI_RESULT_COMPLEX           = 11, // Error: Calculation too complex

  // Problems are individual bits 8 to 27
  MKI_RESULT_LIBRARY_MISSING   = 0x0000000000000100i64, // Problem: At least one requested library is missing
  MKI_RESULT_BLOCK_MISSING     = 0x0000000000000200i64, // Problem: At least one requested block is missing
  MKI_RESULT_REFERENCE_MISSING = 0x0000000000000400i64, // Problem: At least one requested reference is missing
  MKI_RESULT_BITMAP_MISSING    = 0x0000000000000800i64, // Problem: At least one requested bitmap is missing
  MKI_RESULT_FONT_MISSING      = 0x0000000000001000i64, // Problem: At least one requested font is missing
  MKI_RESULT_PLUGIN_MISSING    = 0x0000000000002000i64, // Problem: At least one requested plug-in is missing
  MKI_RESULT_PDF_INTERFACE     = 0x0000000000004000i64, // Problem: Unable to access PDF interface
  MKI_RESULT_PDF_LIBRARY       = 0x0000000000008000i64, // Problem: Unable to access PDF library
  MKI_RESULT_UNLICENSED        = 0x0000000000010000i64, // Problem: CAD6 not licensed, a watermark will occur
  MKI_RESULT_INACCURATE        = 0x0000000000020000i64, // Problem: Result is inaccurate
  MKI_RESULT_INCOMPLETE        = 0x0000000000040000i64, // Problem: Result is incomplete

  // Warnings are individual bits 28 to 47
  MKI_RESULT_UNDERLINE         = 0x0000000010000000i64, // Warning: Underlined text was resolved into outlines, underline ignored
  MKI_RESULT_STRIKEOUT         = 0x0000000020000000i64, // Warning: Strike-out text was resolved into outlines, strikeout ignored
  MKI_RESULT_OPACITY           = 0x0000000040000000i64, // Warning: Transparency might not work on Windows printers or when exporting to WMF/EMF
  MKI_RESULT_RGBCOLOR          = 0x0000000080000000i64, // Warning: RGB color used where CMYK would be better

  // Custom values are individual bits 48 to 63
  MKI_RESULT_CUSTOM1           = 0x0001000000000000i64, // Custom: Bit 48
  MKI_RESULT_CUSTOM2           = 0x0002000000000000i64, // Custom: Bit 49
  MKI_RESULT_CUSTOM3           = 0x0004000000000000i64, // Custom: Bit 50
  MKI_RESULT_CUSTOM4           = 0x0008000000000000i64, // Custom: Bit 51
  MKI_RESULT_CUSTOM5           = 0x0010000000000000i64, // Custom: Bit 52
  MKI_RESULT_CUSTOM6           = 0x0020000000000000i64, // Custom: Bit 53
  MKI_RESULT_CUSTOM7           = 0x0040000000000000i64, // Custom: Bit 54
  MKI_RESULT_CUSTOM8           = 0x0080000000000000i64, // Custom: Bit 55
  MKI_RESULT_CUSTOM9           = 0x0100000000000000i64, // Custom: Bit 56
  MKI_RESULT_CUSTOM10          = 0x0200000000000000i64, // Custom: Bit 57
  MKI_RESULT_CUSTOM11          = 0x0400000000000000i64, // Custom: Bit 58
  MKI_RESULT_CUSTOM12          = 0x0800000000000000i64, // Custom: Bit 59
  MKI_RESULT_CUSTOM13          = 0x1000000000000000i64, // Custom: Bit 60
  MKI_RESULT_CUSTOM14          = 0x2000000000000000i64, // Custom: Bit 61
  MKI_RESULT_CUSTOM15          = 0x4000000000000000i64, // Custom: Bit 62
  MKI_RESULT_CUSTOM16          = 0x8000000000000000i64, // Custom: Bit 63

  MKI_RESULTMASK_ERROR         = 0x00000000000000ffi64, // Mask for all errors
  MKI_RESULTMASK_PROBLEM       = 0x000000000fffff00i64, // Mask for all problems
  MKI_RESULTMASK_WARNING       = 0x0000fffff0000000i64, // Mask for all warnings
  MKI_RESULTMASK_CUSTOM        = 0xffff000000000000i64, // Mask for all bits reserved for custom use
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_NOTIFICATIONMODE_DEFAULT  = 0, // Default notification, only added to list
  MKI_NOTIFICATIONMODE_BLINKING = 1, // Blinking notification, displayed at cross-hair
  MKI_NOTIFICATIONMODE_AUTOREAD = 2, // Temporary notification, displayed at cross-hair, automatically marked as read
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_NOTIFICATIONTIME_LONG   = 10000, // 10 s
  MKI_NOTIFICATIONTIME_NORMAL = 5000,  // 5 s
  MKI_NOTIFICATIONTIME_SHORT  = 2000,  // 2 s
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_INIT_ERROR           = 0, // Any other error
  MKI_INIT_OK              = 1, // The plug-in was successfully initialized
  MKI_INIT_OLD_PLUGIN      = 2, // The plug-in is too old to work with the serving application
  MKI_INIT_OLD_APPLICATION = 3, // The serving application is too old to work with the plug-in
  MKI_INIT_BAD_INSTALL     = 4, // A required file or resource is missing
  MKI_INIT_INACTIVE        = 5, // The plug-in is inactive or the trial period has ended
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_INPUT_ERROR    = 0,  // Error during input processing
  MKI_INPUT_OVERFLOW = 1,  // Too many points entered
  MKI_INPUT_IGNORE   = 2,  // Ignore this point!
  MKI_INPUT_OK       = 3,  // OK
  MKI_INPUT_CANCEL   = 4,  // Cancel current command
  MKI_INPUT_FINISH   = 5,  // Finish current command
  MKI_INPUT_NEXT     = 6,  // Continue with next point
  MKI_INPUT_UNUSED   = 7,  // This point is not required
  MKI_INPUT_RESET    = 8,  // Reset current command
  MKI_INPUT_OBJTYPE  = 9,  // Invalid/unusable object type identified
  MKI_INPUT_IDLE     = 10, // Nothing to do with this point - re-enter!
  MKI_INPUT_SNAP     = 11, // No snapping point found - re-enter!
  MKI_INPUT_IDENT    = 12, // No valid identification - re-enter!
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_IMPORT_OK        = 0, // File was imported successfully
  MKI_IMPORT_UNKNOWN   = 1, // File is of unknown type, ask another import filter
  MKI_IMPORT_ERROR     = 2, // Error while importing the file
  MKI_IMPORT_DUPLICATE = 3, // File is already opened
  MKI_IMPORT_OLDFORMAT = 4, // File uses an old format
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_STANDARDPATH_INF = 0  * MKI_DEFPATH_MAX, // Standard path of setting files
  MKI_STANDARDPATH_MKD = 1  * MKI_DEFPATH_MAX, // Standard path of drawing files
  MKI_STANDARDPATH_MKL = 2  * MKI_DEFPATH_MAX, // Standard path of library files
  MKI_STANDARDPATH_EMF = 3  * MKI_DEFPATH_MAX, // Standard path of EMF files
  MKI_STANDARDPATH_WMF = 4  * MKI_DEFPATH_MAX, // Standard path of WMF files
  MKI_STANDARDPATH_BMP = 5  * MKI_DEFPATH_MAX, // Standard path of bitmap files
  MKI_STANDARDPATH_DIG = 6  * MKI_DEFPATH_MAX, // Standard path of digitizer setting files
  MKI_STANDARDPATH_FLT = 7  * MKI_DEFPATH_MAX, // Standard path of filter files
  MKI_STANDARDPATH_ATT = 8  * MKI_DEFPATH_MAX, // Standard path of attribute text files
  MKI_STANDARDPATH_DBF = 9  * MKI_DEFPATH_MAX, // Standard path of database files
  MKI_STANDARDPATH_PDF = 10 * MKI_DEFPATH_MAX, // Standard path of PDF files
  MKI_STANDARDPATH_COL = 11 * MKI_DEFPATH_MAX, // Standard path of collection files
  MKI_STANDARDPATH_SVG = 12 * MKI_DEFPATH_MAX, // Standard path of SVG files
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_FILEMODE_NULL          = 0x00000000,

  MKI_FILEMODE_HEADER        = 0x00000001,
  MKI_FILEMODE_TASK          = 0x00000002,
  MKI_FILEMODE_TEXTSTYLE     = 0x00000004,
  MKI_FILEMODE_DATABASE      = 0x00000008,
  MKI_FILEMODE_WINDOW        = 0x00000010,
  MKI_FILEMODE_DEFAULT       = 0x00000020,
  MKI_FILEMODE_PROFILE       = 0x00000040,
  MKI_FILEMODE_MODULE        = 0x00000080,
  MKI_FILEMODE_PAGE          = 0x00000100,
  MKI_FILEMODE_COLOR         = 0x00000200,
  MKI_FILEMODE_HATCH         = 0x00000400,
  MKI_FILEMODE_ITEM          = 0x00000800,
  MKI_FILEMODE_SYSTEM        = 0x00001000,
  MKI_FILEMODE_PEN           = 0x00002000,
  MKI_FILEMODE_LINE          = 0x00004000,
  MKI_FILEMODE_LAYER         = 0x00008000,
  MKI_FILEMODE_BITMAP        = 0x00010000,
  MKI_FILEMODE_VARIABLE      = 0x00020000,
  MKI_FILEMODE_PREVIEW       = 0x00040000,
  MKI_FILEMODE_ATTRIB        = 0x00080000,
  MKI_FILEMODE_BLOCK         = 0x00100000,
  MKI_FILEMODE_OBJECT        = 0x00200000,

  MKI_FILEMODE_COMPRESSED    = 0x01000000, // Load/Save compressed drawing
  MKI_FILEMODE_PASSWORD      = 0x02000000, // Scramble with password (only valid if combined with MKI_FILEMODE_COMPRESSED)

  MKI_FILEMODE_NODIALOG      = 0x04000000, // Do not display a progress indicator
  MKI_FILEMODE_NOREDRAW      = 0x08000000, // Avoid screen refresh after loading
  MKI_FILEMODE_USERSECTIONS  = 0x10000000, // Save all previously loaded sections plus...
  MKI_FILEMODE_IDENTIFIED    = 0x20000000, // Save only identified objects
  MKI_FILEMODE_REPLACE       = 0x40000000, // Replace elements when loading
  MKI_FILEMODE_UPDATEMATCH   = 0x80000000, // Update matching elements when loading
  MKI_FILEMODE_ALL           = 0x40ffffff, // Load/save all possible elements

  MKI_FILEMODE_DRAWING       = MKI_FILEMODE_ALL,
  MKI_FILEMODE_LIBRARYCACHE  = MKI_FILEMODE_HEADER | MKI_FILEMODE_ATTRIB | MKI_FILEMODE_SYSTEM | MKI_FILEMODE_LINE | MKI_FILEMODE_PEN | MKI_FILEMODE_LAYER,
  MKI_FILEMODE_TEMPLATE      = MKI_FILEMODE_ALL - MKI_FILEMODE_BLOCK - MKI_FILEMODE_OBJECT,
  MKI_FILEMODE_IMPORT        = MKI_FILEMODE_OBJECT | MKI_FILEMODE_BLOCK | MKI_FILEMODE_SYSTEM | MKI_FILEMODE_PEN | MKI_FILEMODE_LINE | MKI_FILEMODE_LAYER | MKI_FILEMODE_BITMAP | MKI_FILEMODE_VARIABLE | MKI_FILEMODE_BITMAP,

  MKI_FILEMODE_TOTAL         = 24,         // Number of "user-editable" bits in MKI_FILEMODE_*
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_FILEOPTIONS_NONE   = 0, // Type of file options to edit with MKI_DialogFileOptions
  MKI_FILEOPTIONS_LOAD   = 1,
  MKI_FILEOPTIONS_SAVE   = 2,
  MKI_FILEOPTIONS_NEW    = 3,
  MKI_FILEOPTIONS_IMPORT = 4,
  MKI_FILEOPTIONS_CLIP   = 5,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_FILESELECT_OPEN = 0, // Open file (file and path must exist)
  MKI_FILESELECT_SAVE = 1, // Save file (warning if file exists, path must exist)
  MKI_FILESELECT_PATH = 2, // Choose path (filename is ignored, path must exist)
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_MESSAGE_OK           = 0, // Button "OK" (default)
  MKI_MESSAGE_YESNO1       = 1, // Buttons "Yes" (default) and "No"
  MKI_MESSAGE_YESNO2       = 2, // Buttons "Yes" and "No" (default)
  MKI_MESSAGE_YESNOCANCEL1 = 3, // Buttons "Yes" (default), "No", and "Cancel"
  MKI_MESSAGE_YESNOCANCEL2 = 4, // Buttons "Yes", "No" (default), and "Cancel"
  MKI_MESSAGE_OKCANCEL1    = 5, // Button "OK" (default) and "Cancel"
  MKI_MESSAGE_OKCANCEL2    = 6, // Button "OK" and "Cancel" (default)
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_BUTTON_INVALID = 0, // The dialog failed due to invalid parameters
  MKI_BUTTON_OK      = 1, // Button "OK"
  MKI_BUTTON_CANCEL  = 2, // Button "Cancel"
  MKI_BUTTON_YES     = 3, // Button "Yes"
  MKI_BUTTON_NO      = 4, // Button "No"
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum : __int64
{
  MKI_ENUM_LINES            = 0x0000000000000000i64, // Curves / surfaces may neither be nested nor filled
  MKI_ENUM_FILLS            = 0x0000000000000001i64, // Curves / surfaces may be filled
  MKI_ENUM_ARCS             = 0x0000000000000002i64, // Curves / surfaces may contain circular arcs
  MKI_ENUM_BEZIERS          = 0x0000000000000004i64, // Curves / surfaces may contain Bzier curves
  MKI_ENUM_LINETYPES        = 0x0000000000000008i64, // Curves / surfaces may use line patterns (instead of resolving)
  MKI_ENUM_SOLID_CHARS      = 0x0000000000000010i64, // Solid texts will be passed as separate characters (plus a matrix)
  MKI_ENUM_OTHER_CHARS      = 0x0000000000000020i64, // Non-solid texts will be passed as separate characters (plus a matrix)
  MKI_ENUM_SIMPLE_TEXTS     = 0x0000000000000040i64, // Texts will be passed as single one-line-strings (format plus matrix)
  MKI_ENUM_COMPLEX_TEXTS    = 0x0000000000000080i64, // Texts will be passed as complex multi-line-strings (format plus matrix)
  MKI_ENUM_PATCHES          = 0x0000000000000100i64, // Patches will be passed directly (instead of converting them to a surface plus curves)
  MKI_ENUM_COLORMODES       = 0x0000000000000200i64, // Colors will not be converted to RGB
  MKI_ENUM_NO_BASELINE      = 0x0000000000000400i64, // Text baseline positions will be corrected to character cell top edge
  MKI_ENUM_PATCH_EDGES      = 0x0000000000000800i64, // All patch edges will be passed, even invisible ones
  MKI_ENUM_HYPERLINKS       = 0x0000000000001000i64, // Enumerate hyperlinks
  MKI_ENUM_COMMENTS         = 0x0000000000002000i64, // Enumerate comments

  MKI_ENUM_MATRIX           = 0x0000000000010000i64, // Display matrixes will be passed (instead of applied directly)
  MKI_ENUM_NATIVE           = 0x0000000000020000i64, // All entities will be passed in native format only (address)
  MKI_ENUM_BLOCKLINES       = 0x0000000000040000i64, // Curves / surfaces may use block-based line patterns (instead of resolving)
  MKI_ENUM_OPTIMIZELINES    = 0x0000000000080000i64, // Line patterns will be optimized
  MKI_ENUM_MARK_CORNERS     = 0x0000000000100000i64, // Corners inside polylines will be marked as MKI_DB_POINT_CORNER or MKI_DB_FIXED_CORNER

  MKI_ENUM_PLAIN            = 0x0000000001000000i64, // Instances will be resolved
  MKI_ENUM_STRUCTURED       = 0x0000000002000000i64, // Instances will not be resolved
  MKI_ENUM_OUTLINED         = 0x0000000004000000i64, // Bitmaps will be enumerated outlined
  MKI_ENUM_ENDLESS          = 0x0000000008000000i64, // Geometry lines will be exported as MKI_ENUMDATA_ENDLESS
  MKI_ENUM_BEAMS            = 0x0000000010000000i64, // Geometry beams will be exported as MKI_ENUMDATA_BEAM

  MKI_ENUM_USED_BLOCKS      = 0x0000000100000000i64, // Enumerate all referenced blocks
  MKI_ENUM_USED_FONTS       = 0x0000000200000000i64, // Enumerate all referenced fonts
  MKI_ENUM_USED_BITMAPS     = 0x0000000400000000i64, // Enumerate all referenced bitmaps
  MKI_ENUM_USED_REFS        = 0x0000000800000000i64, // Enumerate all referenced drawings
  MKI_ENUM_USED_GROUPS      = 0x0000001000000000i64, // Enumerate all referenced groups
  MKI_ENUM_USED_CLIPS       = 0x0000002000000000i64, // Enumerate all referenced clipping surfaces

  MKI_ENUM_NO_OBJECTS       = 0x0000010000000000i64, // Do NOT enumerate objects
  MKI_ENUM_NO_BLOCKS        = 0x0000020000000000i64, // Do NOT enumerate blocks
  MKI_ENUM_NO_CHARS         = 0x0000040000000000i64, // Do NOT enumerate characters
  MKI_ENUM_NO_REFS          = 0x0000080000000000i64, // Do NOT enumerate referenced drawings
  MKI_ENUM_NO_POINTS        = 0x0000100000000000i64, // Enumerate all used properties
  MKI_ENUM_NO_MAPCONTENTS   = 0x0000200000000000i64, // Do NOT enumerate mapping contents
  MKI_ENUM_NO_EXTS          = 0x0000400000000000i64, // Do NOT enumerate extended objects
  MKI_ENUM_NO_GROUPCONTENTS = 0x0000800000000000i64, // Do NOT enumerate group contents
  MKI_ENUM_NO_HATCHCONTENTS = 0x0001000000000000i64, // Do NOT enumerate hatch contents
  MKI_ENUM_NO_MODELSPACE    = 0x0002000000000000i64, // Do NOT enumerate top-level entities assigned to Model Space
  MKI_ENUM_NO_CLIPCONTENTS  = 0x0004000000000000i64, // Do NOT enumerate clipping contents

  MKI_ENUM_TO_SCREEN        = 0x0010000000000000i64, // Use screen settings to calculate properties (instead of output properties)
  MKI_ENUM_TO_PRINTER       = 0x0020000000000000i64, // Assume the streaming is used for printer output, i.e. ignore all non-printed objects

  MKI_ENUM_VISIBLE          = 0x0100000000000000i64, // Enumerate only entities that are not hidden
  MKI_ENUM_WINDOW           = 0x0200000000000000i64, // Enumerate only entities that are visible in the currently active window
  MKI_ENUM_HIDDEN           = 0x0400000000000000i64, // Enumerate all object types despite any output setting
  MKI_ENUM_GEOMETRY         = 0x0800000000000000i64, // Enumerate all construction aid objects despite any output setting
  MKI_ENUM_MODELSPACE       = 0x1000000000000000i64, // Enumerate entities from Model Space only
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_ENUMDATA_INVALID      = 0,  // EnumData contains no valid data
  MKI_ENUMDATA_CURVE        = 1,  // EnumData contains an open curve
  MKI_ENUMDATA_SURFACE      = 2,  // EnumData contains a closed surface
  MKI_ENUMDATA_MARK         = 3,  // EnumData contains a list of markings
  MKI_ENUMDATA_CHAR         = 4,  // EnumData contains a list of characters plus matrixes
  MKI_ENUMDATA_TEXT         = 5,  // EnumData contains a complete line of text
  MKI_ENUMDATA_ENDLESS      = 6,  // EnumData contains a list of endless lines
  MKI_ENUMDATA_BEAM         = 7,  // EnumData contains a list of beams
  MKI_ENUMDATA_INST         = 8,  // EnumData contains an instance
  MKI_ENUMDATA_REF          = 9,  // EnumData contains a drawing reference
  MKI_ENUMDATA_PATCH        = 10, // EnumData contains a patch
  MKI_ENUMDATA_BITMAP       = 11, // EnumData contains a bitmap reference
  MKI_ENUMDATA_HYPERLINK    = 12, // EnumData contains a hyperlink
  MKI_ENUMDATA_COMMENT      = 13, // EnumData contains a comment
  MKI_ENUMDATA_POSITION     = 14, // EnumData contains the current text position

  MKI_ENUMDATA_OBJ_START    = 20, // EnumData contains an object start
  MKI_ENUMDATA_OBJ_END      = 21, // EnumData contains an object end
  MKI_ENUMDATA_INST_START   = 22, // EnumData contains an instance data start
  MKI_ENUMDATA_INST_END     = 23, // EnumData contains an instance data end
  MKI_ENUMDATA_EMBED_START  = 24, // EnumData contains an instance embedded data start
  MKI_ENUMDATA_EMBED_END    = 25, // EnumData contains an instance embedded data end
  MKI_ENUMDATA_EXT_START    = 26, // EnumData contains an extended object start
  MKI_ENUMDATA_EXT_END      = 27, // EnumData contains an extended object end
  MKI_ENUMDATA_CLIP_START   = 28, // EnumData contains a clipping surface start
  MKI_ENUMDATA_CLIP_END     = 29, // EnumData contains a clipping surface end
  MKI_ENUMDATA_REF_START    = 30, // EnumData contains a drawing reference start
  MKI_ENUMDATA_REF_END      = 31, // EnumData contains a drawing reference end
  MKI_ENUMDATA_MAP_START    = 32, // EnumData contains a mapping start
  MKI_ENUMDATA_MAP_END      = 33, // EnumData contains a mapping end
  MKI_ENUMDATA_GROUP_START  = 34, // EnumData contains group data start
  MKI_ENUMDATA_GROUP_END    = 35, // EnumData contains group data end
  MKI_ENUMDATA_HATCH_START  = 36, // EnumData contains hatching data start
  MKI_ENUMDATA_HATCH_END    = 37, // EnumData contains hatching data end
  MKI_ENUMDATA_CHAR_START   = 38, // EnumData contains a character sequence start
  MKI_ENUMDATA_CHAR_END     = 39, // EnumData contains a character sequence end

  MKI_ENUMDATA_NATIVE_OBJ   = 50, // EnumData contains pointer to an object
  MKI_ENUMDATA_NATIVE_INST  = 51, // EnumData contains pointer to an instance
  MKI_ENUMDATA_NATIVE_BLOCK = 52, // EnumData contains pointer to a block
  MKI_ENUMDATA_NATIVE_EXT   = 53, // EnumData contains pointer to an extended object
  MKI_ENUMDATA_NATIVE_CLIP  = 54, // EnumData contains pointer to a clipping surface
  MKI_ENUMDATA_NATIVE_REF   = 55, // EnumData contains pointer to a drawing reference
  MKI_ENUMDATA_NATIVE_PATCH = 56, // EnumData contains pointer to a patch
  MKI_ENUMDATA_NATIVE_MAP   = 56, // EnumData contains pointer to a mapping
  MKI_ENUMDATA_NATIVE_GROUP = 57, // EnumData contains pointer to a group
  MKI_ENUMDATA_NATIVE_HATCH = 58, // EnumData contains pointer to a hatching

  MKI_ENUMDATA_USED_BLOCK   = 70, // EnumData contains information about a used block
  MKI_ENUMDATA_USED_FONT    = 71, // EnumData contains information about a used font
  MKI_ENUMDATA_USED_BITMAP  = 72, // EnumData contains information about a used bitmap
  MKI_ENUMDATA_USED_REF     = 73, // EnumData contains information about a used drawing reference
  MKI_ENUMDATA_USED_GROUP   = 74, // EnumData contains information about a used group
  MKI_ENUMDATA_USED_CLIP    = 75, // EnumData contains information about a used clipping surface
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_NOTIFY_MODIFY_INIT         = 1,  // An undo-aware, multi-entity modification process (modify, copy, delete, etc.) was started
  MKI_NOTIFY_MODIFY_FINISH       = 2,  // The previously started process was sucessfully finished, react accordingly
  MKI_NOTIFY_MODIFY_CANCEL       = 3,  // The previously started process was canceled by the user or due to an error, react accordingly

  MKI_NOTIFY_EXT_INIT            = 10, // Ext is to be initialised
  MKI_NOTIFY_EXT_CREATE          = 11, // Ext has been freshly created (allocated in memory)
  MKI_NOTIFY_EXT_MODIFY          = 12, // Ext is to be modified using a matrix
  MKI_NOTIFY_EXT_HIDE            = 13, // Ext is to be hidden (by undo/redo mechanism, not yet removed from memory)
  MKI_NOTIFY_EXT_SHOW            = 14, // Ext is to be shown again (by undo/redo mechanism)
  MKI_NOTIFY_EXT_DESTROY         = 15, // Ext is to be physically deleted (removed from memory)

  MKI_NOTIFY_EXT_GET_TITLEPTR    = 20, // Return general description of this type of Ext
  MKI_NOTIFY_EXT_GET_CONTENT     = 21, // Return textual description of this individual Ext
  MKI_NOTIFY_EXT_GET_SHORTTEXT   = 22, // Return contained text of this individual Ext (shortened)
  MKI_NOTIFY_EXT_GET_INFORMATION = 23, // Return information on this individual Ext to be displayed at the crosshairs
  MKI_NOTIFY_EXT_GET_PROPERTY    = 24, // Extract properties of this individual Ext and apply to the given multi-entity property set
  MKI_NOTIFY_EXT_CONTEXT_FILL    = 25, // Return a list of context commands available for this individual Ext
  MKI_NOTIFY_EXT_CONTEXT_APPLY   = 26, // Apply given context command to this individual Ext
  MKI_NOTIFY_EXT_CONTEXT_HELP    = 27, // Show help on the given context command

  MKI_NOTIFY_EXT_ENUM            = 30, // Ext shall be enumerated
  MKI_NOTIFY_EXT_SNAP            = 31, // A point in Ext is to be snapped
  MKI_NOTIFY_EXT_IDENT           = 32, // The outline of Ext is to be identified
  MKI_NOTIFY_EXT_POINT_IDENT     = 33, // Points in Ext are to be identified
  MKI_NOTIFY_EXT_CHECK           = 34, // Check Ext for correct content and adjust if necessary

  MKI_NOTIFY_ITEM_GET_TITLE      = 45, // Return general description of this collection or pinboard item type
  MKI_NOTIFY_ITEM_GET_CONTENT    = 46, // Return textual description of this specific collection or pinboard item
  MKI_NOTIFY_ITEM_COPY           = 47, // The user wants to copy this specific pinboard item to the collection
  MKI_NOTIFY_ITEM_EDIT_CONTENT   = 48, // The user wants to edit this specific collection or pinboard item

  MKI_NOTIFY_FIND_TEXT           = 50, // Return the contained text of this extended object in search-friendly form
  MKI_NOTIFY_EDIT_TEXT_SINGLE    = 51, // Let the user edit one text in Ext individually
  MKI_NOTIFY_EDIT_TEXT_MULTI     = 52, // Let the user edit all texts in Ext at once
  MKI_NOTIFY_VIEW_TEXT_MULTI     = 53, // Let the user view all texts in Ext at once (display a read-only dialog)
  MKI_NOTIFY_FIND_ATTRIBUTE      = 54, // Find the given attribute's value in Ext and copy it to Data
  MKI_NOTIFY_EDIT_ATTRIBUTE      = 55, // Let the user edit an attribute in Data
  MKI_NOTIFY_EDIT_PARAMETER      = 56, // Let the user edit the parameters of Ext
  MKI_NOTIFY_VIEW_PARAMETER      = 57, // Let the user view the parameters of Ext (display a read-only dialog)
  MKI_NOTIFY_LIST_START          = 58, // The display list of Ext is to be parsed, create temporary display list if necessary
  MKI_NOTIFY_LIST_END            = 59, // The display list of Ext has been parsed, remove temporary display list if necessary

  MKI_NOTIFY_PROPERTY_INIT       = 60, // Properties shall be edited, alloc and init structure, clear all use flags
  MKI_NOTIFY_PROPERTY_SETALL     = 61, // Set all use flags for the given extended object type
  MKI_NOTIFY_PROPERTY_FILL       = 62, // Fill Ext properties into the structure
  MKI_NOTIFY_PROPERTY_SINGLE     = 63, // Let the user edit the structure for Ext
  MKI_NOTIFY_PROPERTY_MULTIPLE   = 64, // Let the user edit the structure for Ext
  MKI_NOTIFY_PROPERTY_APPLY      = 65, // Apply structure to Ext
  MKI_NOTIFY_PROPERTY_EXIT       = 66, // Discard the structure
  MKI_NOTIFY_PROPERTY_BASEAPPLY  = 67, // Apply only base properties to Ext
                                 
  MKI_NOTIFY_DISPLAY_INIT        = 70, // A screen redraw is started
  MKI_NOTIFY_DISPLAY_EXT_START   = 71, // Extended object data is to be displayed on the screen
  MKI_NOTIFY_DISPLAY_CALLBACK    = 72, // A callback object is to be displayed on the screen
  MKI_NOTIFY_DISPLAY_EXT_END     = 73, // Extended object data is to be displayed on the screen
  MKI_NOTIFY_DISPLAY_EXIT        = 74, // A screen redraw is finished
  MKI_NOTIFY_DISPLAY_TEMP        = 75, // All temporary screen elements shall be redrawn
  MKI_NOTIFY_DISPLAY_IDENT       = 76, // All identification information shall be redrawn

  MKI_NOTIFY_UPDATE_DATABASE     = 80, // A database has been changed, update any data depending on it
  MKI_NOTIFY_UPDATE_ELEMENTS     = 81, // Drawing elements have been changed, update any data depending on it
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_NOTIFY_RESULT_UNKNOWN = 0, // Ext is unknown, ask another plug-in
  MKI_NOTIFY_RESULT_IGNORE  = 1, // Ext is known, but this ModifyMode is not supported
  MKI_NOTIFY_RESULT_DONE    = 2, // Task was performed successfully
  MKI_NOTIFY_RESULT_COPY    = 3, // Task was performed successfully by creating a modified copy
  MKI_NOTIFY_RESULT_FAIL    = 4, // MKI_NOTIFY_EXT_CHECK failed due to memory limitations or format errors
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PROFILE_RESULT_OK               = 0x00000000,
  MKI_PROFILE_RESULT_ERROR_READING    = 0x00000001,
  MKI_PROFILE_RESULT_ERROR_WRITING    = 0x00000002,
  MKI_PROFILE_RESULT_ITEM_NOT_FOUND   = 0x00000003,
  MKI_PROFILE_RESULT_ITEM_INVALID     = 0x00000004,
  MKI_PROFILE_RESULT_OUT_OF_MEMORY    = 0x00000005,
  MKI_PROFILE_RESULT_UNKNOWN_VERSION  = 0x00000006,
  MKI_PROFILE_RESULT_UNAVAILABLE      = 0x00000007,
  MKI_PROFILE_RESULT_MASK             = 0x000000ff,

  MKI_PROFILE_WARNING_LAYER_NOT_FOUND     = 0x00000100,
  MKI_PROFILE_WARNING_PEN_NOT_FOUND       = 0x00000200,
  MKI_PROFILE_WARNING_LINE_NOT_FOUND      = 0x00000400,
  MKI_PROFILE_WARNING_SYSTEM_NOT_FOUND    = 0x00000800,
  MKI_PROFILE_WARNING_EXTENDED_NOT_FOUND  = 0x00001000,
  MKI_PROFILE_WARNING_COLOR_NOT_FOUND     = 0x00002000,
  MKI_PROFILE_WARNING_MASK                = 0xffffff00,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_FLAG_NULL          = 0x0000, // No flag set

  MKI_FLAG_SELECT        = 0x0001, // Entity is permanently selected
  MKI_FLAG_IDENT         = 0x0002, // Entity is temporarily selected
  MKI_FLAG_USE           = 0x0004, // Entity is identified
  MKI_FLAG_POINT         = 0x0008, // At least one point is identified

  MKI_FLAG_SELCHANGE     = 0x0010, // The permanent selection has changed
  MKI_FLAG_USECHANGE     = 0x0020, // The temporary selection has changed
  MKI_FLAG_PREVIOUS      = 0x0040, // Entity was previously selected
  MKI_FLAG_INTERNAL      = 0x0080, // Entity is used for internal affairs

  MKI_FLAG_TEMP1         = 0x0100, // Temporary multi-purpose flag 1
  MKI_FLAG_TEMP2         = 0x0200, // Temporary multi-purpose flag 2
  MKI_FLAG_TEMP3         = 0x0400, // Temporary multi-purpose flag 3
  MKI_FLAG_TEMP4         = 0x0800, // Temporary multi-purpose flag 4
  MKI_FLAG_TEMP_ALL      = 0x0f00,

  MKI_FLAG_NODISPLAY     = 0x1000, // Entity will not be displayed at all
  MKI_FLAG_NOOUTPUT      = 0x2000, // Entity will not be output at all
  MKI_FLAG_LOCKED        = 0x4000, // Entity is completely locked
  MKI_FLAG_IDLE          = 0x8000, // Entity is completely idle

  MKI_FLAG_GROUP_COPY    = MKI_FLAG_PREVIOUS  | MKI_FLAG_SELECT,
  MKI_FLAG_GROUP_NO_USE  = MKI_FLAG_NODISPLAY | MKI_FLAG_LOCKED,
  MKI_FLAG_GROUP_NO_SNAP = MKI_FLAG_NODISPLAY | MKI_FLAG_IDLE,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_FLAG_EX_VARIABLES  = 0x00010000, // Entity contains variables
  MKI_FLAG_EX_UPDATE     = 0x00020000, // Entity has not been updated yet
  MKI_FLAG_EX_HIGHLIGHT  = 0x00040000, // Entity shall be highlighted

  MKI_FLAG_EX_NODISPLAY1 = 0x01000000, // Entity base will not be displayed
  MKI_FLAG_EX_NOOUTPUT1  = 0x02000000, // Entity base will not be output
  MKI_FLAG_EX_LOCKED1    = 0x04000000, // Entity base is locked
  MKI_FLAG_EX_IDLE1      = 0x08000000, // Entity base is idle

  MKI_FLAG_EX_NODISPLAY2 = 0x10000000, // Entity text will not be displayed
  MKI_FLAG_EX_NOOUTPUT2  = 0x20000000, // Entity text will not be output
  MKI_FLAG_EX_LOCKED2    = 0x40000000, // Entity text is locked
  MKI_FLAG_EX_IDLE2      = 0x80000000, // Entity text is idle

  MKI_FLAG_EX_BASE       = 0x00000fff,
  MKI_FLAG_EX_GROUP0     = MKI_FLAG_NODISPLAY | MKI_FLAG_NOOUTPUT | MKI_FLAG_LOCKED | MKI_FLAG_IDLE,
  MKI_FLAG_EX_GROUP1     = MKI_FLAG_EX_NODISPLAY1 | MKI_FLAG_EX_NOOUTPUT1 | MKI_FLAG_EX_LOCKED1 | MKI_FLAG_EX_IDLE1,
  MKI_FLAG_EX_GROUP2     = MKI_FLAG_EX_NODISPLAY2 | MKI_FLAG_EX_NOOUTPUT2 | MKI_FLAG_EX_LOCKED2 | MKI_FLAG_EX_IDLE2,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
constexpr char  MKI_MKD_COMMA   = ',';
constexpr char  MKI_MKD_SEMI    = ';';
constexpr char  MKI_MKD_COLOR   = '/';
constexpr char  MKI_MKD_ESCAPE  = '\\';
constexpr char  MKI_MKD_STRING  = '"';
constexpr char  MKI_MKD_KEYWORD = '=';
constexpr char  MKI_MKD_COMMENT = '|';
constexpr LPSTR MKI_MKD_NEWLINE = "\x0d\x0a";


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_MKD_KEY_UNKNOWN    = -1,
  MKI_MKD_KEY_EXIT       = 0,
  MKI_MKD_KEY_END        = 1,
  MKI_MKD_KEY_DRAWING    = 2,
  MKI_MKD_KEY_LIBRARY    = 3,
  MKI_MKD_KEY_ATTRIB     = 4,
  MKI_MKD_KEY_PAGE       = 5,
  MKI_MKD_KEY_DEFAULT    = 6,
  MKI_MKD_KEY_SYSTEM     = 7,
  MKI_MKD_KEY_PEN        = 8,
  MKI_MKD_KEY_LINE       = 9,
  MKI_MKD_KEY_LAYER      = 10,
  MKI_MKD_KEY_HATCH      = 11,
  MKI_MKD_KEY_BLOCK      = 12,
  MKI_MKD_KEY_OBJECT     = 13,
  MKI_MKD_KEY_COLOR      = 14,
  MKI_MKD_KEY_PROFILE    = 15,
  MKI_MKD_KEY_MODULE     = 16,
  MKI_MKD_KEY_WINDOW     = 17,
//                         18 - unused
  MKI_MKD_KEY_BITMAP     = 19,
  MKI_MKD_KEY_VARIABLE   = 20,
  MKI_MKD_KEY_PREVIEW    = 21,
  MKI_MKD_KEY_INFO       = 22,
  MKI_MKD_KEY_DATABASE   = 23,
  MKI_MKD_KEY_TEXTSTYLE  = 24,
  MKI_MKD_KEY_TASK       = 25,
  MKI_MKD_KEY_COLLECTION = 26,
  MKI_MKD_KEY_ITEM       = 27,
  MKI_MKD_KEY_MAX        = 28,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_MKD_ENTRY_COMMANDBAR_FILLCOLOR  = 0,
  MKI_MKD_ENTRY_COMMANDBAR_LINEWIDTH  = 1,
  MKI_MKD_ENTRY_COMMANDBAR_LINECOLOR  = 2,
  MKI_MKD_ENTRY_DISPLAY_LINES         = 3,
  MKI_MKD_ENTRY_DISPLAY_FILLS         = 4,
  MKI_MKD_ENTRY_DISPLAY_ERASERS       = 5,
  MKI_MKD_ENTRY_DISPLAY_TEXTS         = 6,
  MKI_MKD_ENTRY_DISPLAY_FRAMES        = 7,
  MKI_MKD_ENTRY_DISPLAY_VARIABLES     = 8,
  MKI_MKD_ENTRY_DISPLAY_BITMAPS       = 9,
  MKI_MKD_ENTRY_DISPLAY_BLOCKS        = 10,
  MKI_MKD_ENTRY_DISPLAY_MARKS         = 11,
  MKI_MKD_ENTRY_DISPLAY_GEOMETRY      = 12,
  MKI_MKD_ENTRY_DISPLAY_POINTS        = 13,
  MKI_MKD_ENTRY_DISPLAY_ARROWS        = 14,
  MKI_MKD_ENTRY_DISPLAY_NUMBERS       = 15,
  MKI_MKD_ENTRY_DISPLAY_SELONLY       = 16,
  MKI_MKD_ENTRY_DISPLAY_TRUETYPE      = 17,
  MKI_MKD_ENTRY_DISPLAY_EXTERN        = 18,
  MKI_MKD_ENTRY_DISPLAY_ARCS          = 19,
  MKI_MKD_ENTRY_DISPLAY_AREAS         = 20,
  MKI_MKD_ENTRY_LIBRARY_SCALE         = 21,
  MKI_MKD_ENTRY_DISPLAY_ORTHO         = 22,
  MKI_MKD_ENTRY_DISPLAY_DECIMAL       = 23,
  MKI_MKD_ENTRY_DISPLAY_THRESHOLD     = 24,
  MKI_MKD_ENTRY_DISPLAY_QUALITY       = 25,
  MKI_MKD_ENTRY_DISPLAY_HYPERLINKS    = 26,
  MKI_MKD_ENTRY_DISPLAY_PAGES         = 27,
  MKI_MKD_ENTRY_COLOR_LAYERS          = 28,
  MKI_MKD_ENTRY_COLOR_GEOMETRY        = 29,
  MKI_MKD_ENTRY_COLOR_BLOCKS          = 30,
  MKI_MKD_ENTRY_COLOR_ERASERS         = 31,
  MKI_MKD_ENTRY_COLOR_POINTS          = 32,
  MKI_MKD_ENTRY_COLOR_MARKS           = 33,
  MKI_MKD_ENTRY_COLOR_INPUT           = 34,
  MKI_MKD_ENTRY_COLOR_FLAG            = 35,
  MKI_MKD_ENTRY_COLOR_CROSS           = 36,
  MKI_MKD_ENTRY_COLOR_PAGES           = 37,
  MKI_MKD_ENTRY_COLOR_DRAWAREA        = 38,
  MKI_MKD_ENTRY_COLOR_HYPERLINKS      = 39,
  MKI_MKD_ENTRY_COLOR_NOTES           = 40,
  MKI_MKD_ENTRY_OUTPUT_FRAME          = 41,
  MKI_MKD_ENTRY_OUTPUT_BITMAP         = 42,
  MKI_MKD_ENTRY_OUTPUT_GEOMETRY       = 43,
  MKI_MKD_ENTRY_OUTPUT_GRID           = 44,
  MKI_MKD_ENTRY_OUTPUT_MARK           = 45,
  MKI_MKD_ENTRY_OUTPUT_TRUETYPE       = 46,
  MKI_MKD_ENTRY_OUTPUT_INTERN         = 47,
  MKI_MKD_ENTRY_OUTPUT_PAGES          = 48,
  MKI_MKD_ENTRY_BITMAP_COLORBITS      = 49,
  MKI_MKD_ENTRY_BITMAP_UNIT           = 50,
  MKI_MKD_ENTRY_BITMAP_MODE           = 51,
  MKI_MKD_ENTRY_BITMAP_QUALITY        = 52,
  MKI_MKD_ENTRY_BITMAP_SIZE           = 53,
  MKI_MKD_ENTRY_BITMAP_RESOLUTION     = 54,
  MKI_MKD_ENTRY_BITMAP_ANTIALIASE     = 55,
  MKI_MKD_ENTRY_MULTILINE             = 56,
  MKI_MKD_ENTRY_ORTHOMODE             = 57,
  MKI_MKD_ENTRY_DIMTEXT               = 58,
  MKI_MKD_ENTRY_DIMPARAM              = 59,
  MKI_MKD_ENTRY_DIMLINE_ARROWS        = 60,
  MKI_MKD_ENTRY_DIMLARGE_NUMBERS      = 61,
  MKI_MKD_ENTRY_DIMLARGE_ARROWS       = 62,
  MKI_MKD_ENTRY_DIMLARGE_LINES        = 63,
  MKI_MKD_ENTRY_DIMLARGE_FLAGS        = 64,
  MKI_MKD_ENTRY_DIMLARGE_TEXTS        = 65,
  MKI_MKD_ENTRY_DIMSMALL_NUMBERS      = 66,
  MKI_MKD_ENTRY_DIMSMALL_TEXTS        = 67,
  MKI_MKD_ENTRY_DIMCOORD_ARROW        = 68,
  MKI_MKD_ENTRY_DIMCOORD_FRAME        = 69,
  MKI_MKD_ENTRY_DIMLAYOUT_ARROWS      = 70,
  MKI_MKD_ENTRY_DIMLAYOUT_DIMMODE     = 71,
  MKI_MKD_ENTRY_DIMLAYOUT_LINEMODE    = 72,
  MKI_MKD_ENTRY_DIMLAYOUT_EDITTEXT    = 73,
  MKI_MKD_ENTRY_TEXTSTD_FONT          = 74,
  MKI_MKD_ENTRY_TEXTSTD_DISTANCE      = 75,
  MKI_MKD_ENTRY_TEXTSTD_HEIGHT        = 76,
  MKI_MKD_ENTRY_TEXTSTD_GEOMETRY      = 77,
  MKI_MKD_ENTRY_TEXTSTD_MODE          = 78,
  MKI_MKD_ENTRY_TEXTFRAME_FONT        = 79,
  MKI_MKD_ENTRY_TEXTFRAME_DISTANCE    = 80,
  MKI_MKD_ENTRY_TEXTFRAME_HEIGHT      = 81,
  MKI_MKD_ENTRY_TEXTFRAME_MODE        = 82,
  MKI_MKD_ENTRY_TEXTREF_FONT          = 83,
  MKI_MKD_ENTRY_TEXTREF_DISTANCE      = 84,
  MKI_MKD_ENTRY_TEXTREF_HEIGHT        = 85,
  MKI_MKD_ENTRY_TEXTREF_GEOMETRY      = 86,
  MKI_MKD_ENTRY_TEXTREF_MODE          = 87,
  MKI_MKD_ENTRY_TEXTREF_ARROW         = 88,
  MKI_MKD_ENTRY_TEXTREF_FRAME         = 89,
  MKI_MKD_ENTRY_COMMENT_COLOR         = 90,
  MKI_MKD_ENTRY_COMMENT_SIZE          = 91,
  MKI_MKD_ENTRY_COMMENT_DISPLAY       = 92,
  MKI_MKD_ENTRY_DATABASE_STAMP        = 93,
  MKI_MKD_ENTRY_ZIGZAG_INDENT         = 94,
  MKI_MKD_ENTRY_SERIAL_DATABASE       = 95,
  MKI_MKD_ENTRY_SERIAL_INDEX          = 96,
  MKI_MKD_ENTRY_SERIAL_FILTER         = 97,
  MKI_MKD_ENTRY_SERIAL_SORT           = 98,
  MKI_MKD_ENTRY_SERIAL_RULE           = 99,
  MKI_MKD_ENTRY_POSNUM_RANGEINDEX     = 100,
  MKI_MKD_ENTRY_PRINT_LAYOUTINDEX     = 101,
  MKI_MKD_ENTRY_POSNUM_FONT_FIRST     = 102,
  MKI_MKD_ENTRY_POSNUM_FONT_LAST      = MKI_MKD_ENTRY_POSNUM_FONT_FIRST + MKI_POSNO_RANGE_MAX,
  MKI_MKD_ENTRY_POSNUM_DISTANCE_FIRST = MKI_MKD_ENTRY_POSNUM_FONT_LAST,
  MKI_MKD_ENTRY_POSNUM_DISTANCE_LAST  = MKI_MKD_ENTRY_POSNUM_DISTANCE_FIRST + MKI_POSNO_RANGE_MAX,
  MKI_MKD_ENTRY_POSNUM_HEIGHT_FIRST   = MKI_MKD_ENTRY_POSNUM_DISTANCE_LAST,
  MKI_MKD_ENTRY_POSNUM_HEIGHT_LAST    = MKI_MKD_ENTRY_POSNUM_HEIGHT_FIRST + MKI_POSNO_RANGE_MAX,
  MKI_MKD_ENTRY_POSNUM_GEOMETRY_FIRST = MKI_MKD_ENTRY_POSNUM_HEIGHT_LAST,
  MKI_MKD_ENTRY_POSNUM_GEOMETRY_LAST  = MKI_MKD_ENTRY_POSNUM_GEOMETRY_FIRST + MKI_POSNO_RANGE_MAX,
  MKI_MKD_ENTRY_POSNUM_MODE_FIRST     = MKI_MKD_ENTRY_POSNUM_GEOMETRY_LAST,
  MKI_MKD_ENTRY_POSNUM_MODE_LAST      = MKI_MKD_ENTRY_POSNUM_MODE_FIRST + MKI_POSNO_RANGE_MAX,
  MKI_MKD_ENTRY_POSNUM_ARROW_FIRST    = MKI_MKD_ENTRY_POSNUM_MODE_LAST,
  MKI_MKD_ENTRY_POSNUM_ARROW_LAST     = MKI_MKD_ENTRY_POSNUM_ARROW_FIRST + MKI_POSNO_RANGE_MAX,
  MKI_MKD_ENTRY_POSNUM_FRAME_FIRST    = MKI_MKD_ENTRY_POSNUM_ARROW_LAST,
  MKI_MKD_ENTRY_POSNUM_FRAME_LAST     = MKI_MKD_ENTRY_POSNUM_FRAME_FIRST + MKI_POSNO_RANGE_MAX,
  MKI_MKD_ENTRY_POSNUM_VALUE_FIRST    = MKI_MKD_ENTRY_POSNUM_FRAME_LAST,
  MKI_MKD_ENTRY_POSNUM_VALUE_LAST     = MKI_MKD_ENTRY_POSNUM_VALUE_FIRST + MKI_POSNO_RANGE_MAX,
  MKI_MKD_ENTRY_PRINT_HEADER_FIRST    = MKI_MKD_ENTRY_POSNUM_VALUE_LAST,
  MKI_MKD_ENTRY_PRINT_HEADER_LAST     = MKI_MKD_ENTRY_PRINT_HEADER_FIRST + MKI_PRINTER_MAX,
  MKI_MKD_ENTRY_PRINT_FOOTER_FIRST    = MKI_MKD_ENTRY_PRINT_HEADER_LAST,
  MKI_MKD_ENTRY_PRINT_FOOTER_LAST     = MKI_MKD_ENTRY_PRINT_FOOTER_FIRST + MKI_PRINTER_MAX,
  MKI_MKD_ENTRY_PRINT_DRAFT_FIRST     = MKI_MKD_ENTRY_PRINT_FOOTER_LAST,
  MKI_MKD_ENTRY_PRINT_DRAFT_LAST      = MKI_MKD_ENTRY_PRINT_DRAFT_FIRST + MKI_PRINTER_MAX,
  MKI_MKD_ENTRY_PRINT_SELONLY_FIRST   = MKI_MKD_ENTRY_PRINT_DRAFT_LAST,
  MKI_MKD_ENTRY_PRINT_SELONLY_LAST    = MKI_MKD_ENTRY_PRINT_SELONLY_FIRST + MKI_PRINTER_MAX,
  MKI_MKD_ENTRY_PRINT_CENTERED_FIRST  = MKI_MKD_ENTRY_PRINT_SELONLY_LAST,
  MKI_MKD_ENTRY_PRINT_CENTERED_LAST   = MKI_MKD_ENTRY_PRINT_CENTERED_FIRST + MKI_PRINTER_MAX,
  MKI_MKD_ENTRY_PRINT_EDGES_FIRST     = MKI_MKD_ENTRY_PRINT_CENTERED_LAST,
  MKI_MKD_ENTRY_PRINT_EDGES_LAST      = MKI_MKD_ENTRY_PRINT_EDGES_FIRST + MKI_PRINTER_MAX,
  MKI_MKD_ENTRY_PRINT_PAGES_FIRST     = MKI_MKD_ENTRY_PRINT_EDGES_LAST,
  MKI_MKD_ENTRY_PRINT_PAGES_LAST      = MKI_MKD_ENTRY_PRINT_PAGES_FIRST + MKI_PRINTER_MAX,
  MKI_MKD_ENTRY_PRINT_ORIENT_FIRST    = MKI_MKD_ENTRY_PRINT_PAGES_LAST,
  MKI_MKD_ENTRY_PRINT_ORIENT_LAST     = MKI_MKD_ENTRY_PRINT_ORIENT_FIRST + MKI_PRINTER_MAX,
  MKI_MKD_ENTRY_PRINT_MODE_FIRST      = MKI_MKD_ENTRY_PRINT_ORIENT_LAST,
  MKI_MKD_ENTRY_PRINT_MODE_LAST       = MKI_MKD_ENTRY_PRINT_MODE_FIRST + MKI_PRINTER_MAX,
  MKI_MKD_ENTRY_PRINT_SCALE_FIRST     = MKI_MKD_ENTRY_PRINT_MODE_LAST,
  MKI_MKD_ENTRY_PRINT_SCALE_LAST      = MKI_MKD_ENTRY_PRINT_SCALE_FIRST + MKI_PRINTER_MAX,
  MKI_MKD_ENTRY_PRINT_BORDER_FIRST    = MKI_MKD_ENTRY_PRINT_SCALE_LAST,
  MKI_MKD_ENTRY_PRINT_BORDER_LAST     = MKI_MKD_ENTRY_PRINT_BORDER_FIRST + MKI_PRINTER_MAX,
  MKI_MKD_ENTRY_PRINT_SERIAL_FIRST    = MKI_MKD_ENTRY_PRINT_BORDER_LAST,
  MKI_MKD_ENTRY_PRINT_SERIAL_LAST     = MKI_MKD_ENTRY_PRINT_SERIAL_FIRST + MKI_PRINTER_MAX,
  MKI_MKD_ENTRY_MAX                   = MKI_MKD_ENTRY_PRINT_SERIAL_LAST,
  MKI_MKD_ENTRY_END                   = 999,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_MKD_INFO_PLACING = 0,
  MKI_MKD_INFO_MAX     = 1,
  MKI_MKD_INFO_END     = 999,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------

#ifndef MKI_MACROS_A_H

MKI_INTERFACE_MEMBER void
MKI_InitA( LPSTR f_pszDestin );

MKI_INTERFACE_MEMBER bool
MKI_EqualA( LPCSTR f_pszText1, LPCSTR f_pszText2 );

MKI_INTERFACE_MEMBER __int32
MKI_CompareA( LPCSTR f_pszText1, LPCSTR f_pszText2 );

MKI_INTERFACE_MEMBER void
MKI_TerminateA( LPSTR f_pszDestin, const size_t f_unMaxSize );

    template <size_t f_unSize> __forceinline void
    MKI_TerminateA( char (&f_rDestin)[f_unSize] )
    {
      MKI_TerminateA( (LPSTR) &f_rDestin, f_unSize );
    }

MKI_INTERFACE_MEMBER void
MKI_CopyPartA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszSource1, const size_t f_unCharCount );

    template <size_t f_unSize> __forceinline void
    MKI_CopyPartA( char (&f_rDestin)[f_unSize], LPCSTR f_pszSource1, const size_t f_unCharCount )
    {
      MKI_CopyPartA( (LPSTR) &f_rDestin, f_unSize, f_pszSource1, f_unCharCount );
    }

MKI_INTERFACE_MEMBER void
MKI_CopyA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszSource1 );

    template <size_t f_unSize> __forceinline void
    MKI_CopyA( char (&f_rDestin)[f_unSize], LPCSTR f_pszSource1 )
    {
      MKI_CopyA( (LPSTR) &f_rDestin, f_unSize, f_pszSource1 );
    }

MKI_INTERFACE_MEMBER void
MKI_CopyA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszSource1, LPCSTR f_pszSource2 );

    template <size_t f_unSize> __forceinline void
    MKI_CopyA( char (&f_rDestin)[f_unSize], LPCSTR f_pszSource1, LPCSTR f_pszSource2 )
    {
      MKI_CopyA( (LPSTR) &f_rDestin, f_unSize, f_pszSource1, f_pszSource2 );
    }

MKI_INTERFACE_MEMBER void
MKI_CopyA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszSource1, LPCSTR f_pszSource2, LPCSTR f_pszSource3 );

    template <size_t f_unSize> __forceinline void
    MKI_CopyA( char (&f_rDestin)[f_unSize], LPCSTR f_pszSource1, LPCSTR f_pszSource2, LPCSTR f_pszSource3 )
    {
      MKI_CopyA( (LPSTR) &f_rDestin, f_unSize, f_pszSource1, f_pszSource2, f_pszSource3 );
    }

MKI_INTERFACE_MEMBER void
MKI_CopyA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszSource1, LPCSTR f_pszSource2, LPCSTR f_pszSource3, LPCSTR f_pszSource4 );

    template <size_t f_unSize> __forceinline void
    MKI_CopyA( char (&f_rDestin)[f_unSize], LPCSTR f_pszSource1, LPCSTR f_pszSource2, LPCSTR f_pszSource3, LPCSTR f_pszSource4 )
    {
      MKI_CopyA( (LPSTR) &f_rDestin, f_unSize, f_pszSource1, f_pszSource2, f_pszSource3, f_pszSource4 );
    }

MKI_INTERFACE_MEMBER void
MKI_CopyA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszSource1, LPCSTR f_pszSource2, LPCSTR f_pszSource3, LPCSTR f_pszSource4, LPCSTR f_pszSource5 );

    template <size_t f_unSize> __forceinline void
    MKI_CopyA( char (&f_rDestin)[f_unSize], LPCSTR f_pszSource1, LPCSTR f_pszSource2, LPCSTR f_pszSource3, LPCSTR f_pszSource4, LPCSTR f_pszSource5 )
    {
      MKI_CopyA( (LPSTR) &f_rDestin, f_unSize, f_pszSource1, f_pszSource2, f_pszSource3, f_pszSource4, f_pszSource5 );
    }

MKI_INTERFACE_MEMBER void
MKI_CopyA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszSource1, LPCSTR f_pszSource2, LPCSTR f_pszSource3, LPCSTR f_pszSource4, LPCSTR f_pszSource5, LPCSTR f_pszSource6 );

    template <size_t f_unSize> __forceinline void
    MKI_CopyA( char (&f_rDestin)[f_unSize], LPCSTR f_pszSource1, LPCSTR f_pszSource2, LPCSTR f_pszSource3, LPCSTR f_pszSource4, LPCSTR f_pszSource5, LPCSTR f_pszSource6 )
    {
      MKI_CopyA( (LPSTR) &f_rDestin, f_unSize, f_pszSource1, f_pszSource2, f_pszSource3, f_pszSource4, f_pszSource5, LPCSTR f_pszSource6 );
    }

MKI_INTERFACE_MEMBER void
MKI_CopyA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszSource1, LPCSTR f_pszSource2, LPCSTR f_pszSource3, LPCSTR f_pszSource4, LPCSTR f_pszSource5, LPCSTR f_pszSource6, LPCSTR f_pszSource7 );

    template <size_t f_unSize> __forceinline void
    MKI_CopyA( char (&f_rDestin)[f_unSize], LPCSTR f_pszSource1, LPCSTR f_pszSource2, LPCSTR f_pszSource3, LPCSTR f_pszSource4, LPCSTR f_pszSource5, LPCSTR f_pszSource6, LPCSTR f_pszSource7 )
    {
      MKI_CopyA( (LPSTR) &f_rDestin, f_unSize, f_pszSource1, f_pszSource2, f_pszSource3, f_pszSource4, f_pszSource5, LPCSTR f_pszSource6, LPCSTR f_pszSource7 );
    }

MKI_INTERFACE_MEMBER void
MKI_CopyA( LPSTR f_pszDestin, const size_t f_unMaxSize, const char f_chChar );

    template <size_t f_unSize> __forceinline void 
    MKI_CopyA( char (&f_rDestin)[f_unSize], char f_chChar )
    {
      MKI_CopyA( (LPSTR) &f_rDestin, f_unSize, f_chChar );
    }

MKI_INTERFACE_MEMBER bool
MKI_JoinPartA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszSource1, const size_t f_unCharCount );

    template <size_t f_unSize> __forceinline bool 
    MKI_JoinPartA( char (&f_rDestin)[f_unSize], LPCSTR f_pszSource1, const size_t f_unCharCount )
    {
      return( MKI_JoinPartA( (LPSTR) &f_rDestin, f_unSize, f_pszSource1, f_unCharCount ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_JoinA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszSource1 );

    template <size_t f_unSize> __forceinline bool 
    MKI_JoinA( char (&f_rDestin)[f_unSize], LPCSTR f_pszSource1 )
    {
      return( MKI_JoinA( (LPSTR) &f_rDestin, f_unSize, f_pszSource1 ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_JoinA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszSource1, LPCSTR f_pszSource2 );

    template <size_t f_unSize> __forceinline bool 
    MKI_JoinA( char (&f_rDestin)[f_unSize], LPCSTR f_pszSource1, LPCSTR f_pszSource2 )
    {
      return( MKI_JoinA( (LPSTR) &f_rDestin, f_unSize, f_pszSource1, f_pszSource2 ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_JoinA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszSource1, LPCSTR f_pszSource2, LPCSTR f_pszSource3 );

    template <size_t f_unSize> __forceinline bool 
    MKI_JoinA( char (&f_rDestin)[f_unSize], LPCSTR f_pszSource1, LPCSTR f_pszSource2, LPCSTR f_pszSource3 )
    {
      return( MKI_JoinA( (LPSTR) &f_rDestin, f_unSize, f_pszSource1, f_pszSource2, f_pszSource3 ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_JoinA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszSource1, LPCSTR f_pszSource2, LPCSTR f_pszSource3, LPCSTR f_pszSource4 );

    template <size_t f_unSize> __forceinline bool 
    MKI_JoinA( char (&f_rDestin)[f_unSize], LPCSTR f_pszSource1, LPCSTR f_pszSource2, LPCSTR f_pszSource3, LPCSTR f_pszSource4 )
    {
      return( MKI_JoinA( (LPSTR) &f_rDestin, f_unSize, f_pszSource1, f_pszSource2, f_pszSource3, f_pszSource4 ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_JoinA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszSource1, LPCSTR f_pszSource2, LPCSTR f_pszSource3, LPCSTR f_pszSource4, LPCSTR f_pszSource5 );

    template <size_t f_unSize> __forceinline bool 
    MKI_JoinA( char (&f_rDestin)[f_unSize], LPCSTR f_pszSource1, LPCSTR f_pszSource2, LPCSTR f_pszSource3, LPCSTR f_pszSource4, LPCSTR f_pszSource5 )
    {
      return( MKI_JoinA( (LPSTR) &f_rDestin, f_unSize, f_pszSource1, f_pszSource2, f_pszSource3, f_pszSource4, f_pszSource5 ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_JoinA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszSource1, LPCSTR f_pszSource2, LPCSTR f_pszSource3, LPCSTR f_pszSource4, LPCSTR f_pszSource5, LPCSTR f_pszSource6 );

    template <size_t f_unSize> __forceinline bool 
    MKI_JoinA( char (&f_rDestin)[f_unSize], LPCSTR f_pszSource1, LPCSTR f_pszSource2, LPCSTR f_pszSource3, LPCSTR f_pszSource4, LPCSTR f_pszSource5, LPCSTR f_pszSource6 )
    {
      return( MKI_JoinA( (LPSTR) &f_rDestin, f_unSize, f_pszSource1, f_pszSource2, f_pszSource3, f_pszSource4, f_pszSource5, f_pszSource6 ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_JoinA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszSource1, LPCSTR f_pszSource2, LPCSTR f_pszSource3, LPCSTR f_pszSource4, LPCSTR f_pszSource5, LPCSTR f_pszSource6, LPCSTR f_pszSource7 );

    template <size_t f_unSize> __forceinline bool 
    MKI_JoinA( char (&f_rDestin)[f_unSize], LPCSTR f_pszSource1, LPCSTR f_pszSource2, LPCSTR f_pszSource3, LPCSTR f_pszSource4, LPCSTR f_pszSource5, LPCSTR f_pszSource6, LPCSTR f_pszSource7 )
    {
      return( MKI_JoinA( (LPSTR) &f_rDestin, f_unSize, f_pszSource1, f_pszSource2, f_pszSource3, f_pszSource4, f_pszSource5, f_pszSource6, f_pszSource7 ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_JoinA( LPSTR f_pszDestin, const size_t f_unMaxSize, const char f_chChar );

    template <size_t f_unSize> __forceinline bool 
    MKI_JoinA( char (&f_rDestin)[f_unSize], char f_chChar )
    {
      return( MKI_JoinA( (LPSTR) &f_rDestin, f_unSize, f_chChar ) );
    }

MKI_INTERFACE_MEMBER void
MKI_PathA( LPSTR f_pszDestin, const size_t f_unMaxSize );

    template <size_t f_unSize> __forceinline void
    MKI_PathA( char (&f_rDestin)[f_unSize] )
    {
      MKI_PathA( (LPSTR) &f_rDestin, f_unSize );
    }

MKI_INTERFACE_MEMBER void
MKI_MakePathA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszDrive, LPCSTR f_pszPath, LPCSTR f_pszFile, LPCSTR f_pszExt );

    template <size_t f_unSize> __forceinline void
    MKI_MakePathA( char (&f_rDestin)[f_unSize], LPCSTR f_pszDrive, LPCSTR f_pszPath, LPCSTR f_pszFile, LPCSTR f_pszExt )
    {
      MKI_MakePathA( (LPSTR) &f_rDestin, f_unSize, f_pszDrive, f_pszPath, f_pszFile, f_pszExt );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, LPCSTR f_psz1 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, LPCSTR f_psz1 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, LPCSTR f_psz1, LPCSTR f_psz2 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, LPCSTR f_psz1, LPCSTR f_psz2 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_psz2 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, LPCSTR f_psz1, LPCSTR f_psz2, LPCSTR f_psz3 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, LPCSTR f_psz1, LPCSTR f_psz2, LPCSTR f_psz3 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_psz2, f_psz3 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, LPCSTR f_psz1, LPCSTR f_psz2, LPCSTR f_psz3, LPCSTR f_psz4 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, LPCSTR f_psz1, LPCSTR f_psz2, LPCSTR f_psz3, LPCSTR f_psz4 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_psz2, f_psz3, f_psz4 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, LPCSTR f_psz1, LPCSTR f_psz2, LPCSTR f_psz3, LPCSTR f_psz4, LPCSTR f_psz5 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, LPCSTR f_psz1, LPCSTR f_psz2, LPCSTR f_psz3, LPCSTR f_psz4, LPCSTR f_psz5 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_psz2, f_psz3, f_psz4, f_psz5 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, LPCSTR f_psz1, LPCSTR f_psz2, LPCSTR f_psz3, LPCSTR f_psz4, LPCSTR f_psz5, LPCSTR f_psz6 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, LPCSTR f_psz1, LPCSTR f_psz2, LPCSTR f_psz3, LPCSTR f_psz4, LPCSTR f_psz5, LPCSTR f_psz6 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_psz2, f_psz3, f_psz4, f_psz5, f_psz6 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, LPCSTR f_psz1, LPCSTR f_psz2, LPCSTR f_psz3, LPCSTR f_psz4, LPCSTR f_psz5, LPCSTR f_psz6, LPCSTR f_psz7 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, LPCSTR f_psz1, LPCSTR f_psz2, LPCSTR f_psz3, LPCSTR f_psz4, LPCSTR f_psz5, LPCSTR f_psz6, LPCSTR f_psz7 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_psz2, f_psz3, f_psz4, f_psz5, f_psz6, f_psz7 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, LPCSTR f_psz1, LPCSTR f_psz2, LPCSTR f_psz3, LPCSTR f_psz4, LPCSTR f_psz5, LPCSTR f_psz6, LPCSTR f_psz7, LPCSTR f_psz8 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, LPCSTR f_psz1, LPCSTR f_psz2, LPCSTR f_psz3, LPCSTR f_psz4, LPCSTR f_psz5, LPCSTR f_psz6, LPCSTR f_psz7, LPCSTR f_psz8 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_psz2, f_psz3, f_psz4, f_psz5, f_psz6, f_psz7, f_psz8 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, LPCSTR f_psz1, LPCSTR f_psz2, LPCSTR f_psz3, LPCSTR f_psz4, LPCSTR f_psz5, LPCSTR f_psz6, LPCSTR f_psz7, LPCSTR f_psz8, LPCSTR f_psz9 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, LPCSTR f_psz1, LPCSTR f_psz2, LPCSTR f_psz3, LPCSTR f_psz4, LPCSTR f_psz5, LPCSTR f_psz6, LPCSTR f_psz7, LPCSTR f_psz8, LPCSTR f_psz9 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_psz2, f_psz3, f_psz4, f_psz5, f_psz6, f_psz7, f_psz8, f_psz9 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, LPCSTR f_psz1, LPCSTR f_psz2, LPCSTR f_psz3, LPCSTR f_psz4, LPCSTR f_psz5, LPCSTR f_psz6, LPCSTR f_psz7, LPCSTR f_psz8, LPCSTR f_psz9, LPCSTR f_psz10 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, LPCSTR f_psz1, LPCSTR f_psz2, LPCSTR f_psz3, LPCSTR f_psz4, LPCSTR f_psz5, LPCSTR f_psz6, LPCSTR f_psz7, LPCSTR f_psz8, LPCSTR f_psz9, LPCSTR f_psz10 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_psz2, f_psz3, f_psz4, f_psz5, f_psz6, f_psz7, f_psz8, f_psz9, f_psz10 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, __int32 f_n1 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, __int32 f_n1 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, __int32 f_n1, __int32 f_n2 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, __int32 f_n1, __int32 f_n2 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_n2 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_n2, f_n3 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_n2, f_n3, f_n4 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4, __int32 f_n5 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4, __int32 f_n5 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_n2, f_n3, f_n4, f_n5 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4, __int32 f_n5, __int32 f_n6 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4, __int32 f_n5, __int32 f_n6 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_n2, f_n3, f_n4, f_n5, f_n6 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4, __int32 f_n5, __int32 f_n6, __int32 f_n7 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4, __int32 f_n5, __int32 f_n6, __int32 f_n7 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_n2, f_n3, f_n4, f_n5, f_n6, f_n7 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4, __int32 f_n5, __int32 f_n6, __int32 f_n7, __int32 f_n8 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4, __int32 f_n5, __int32 f_n6, __int32 f_n7, __int32 f_n8 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_n2, f_n3, f_n4, f_n5, f_n6, f_n7, f_n8 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4, __int32 f_n5, __int32 f_n6, __int32 f_n7, __int32 f_n8, __int32 f_n9 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4, __int32 f_n5, __int32 f_n6, __int32 f_n7, __int32 f_n8, __int32 f_n9 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_n2, f_n3, f_n4, f_n5, f_n6, f_n7, f_n8, f_n9 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4, __int32 f_n5, __int32 f_n6, __int32 f_n7, __int32 f_n8, __int32 f_n9, __int32 f_n10 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4, __int32 f_n5, __int32 f_n6, __int32 f_n7, __int32 f_n8, __int32 f_n9, __int32 f_n10 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_n2, f_n3, f_n4, f_n5, f_n6, f_n7, f_n8, f_n9, f_n10 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, double f_d1 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, double f_d1 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_d1 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, double f_d1, double f_d2 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, double f_d1, double f_d2 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_d1, f_d2 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, double f_d1, double f_d2, double f_d3 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, double f_d1, double f_d2, double f_d3 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_d1, f_d2, f_d3 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_d1, f_d2, f_d3, f_d4 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4, double f_d5 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4, double f_d5 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_d1, f_d2, f_d3, f_d4, f_d5 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4, double f_d5, double f_d6 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4, double f_d5, double f_d6 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_d1, f_d2, f_d3, f_d4, f_d5, f_d6 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4, double f_d5, double f_d6, double f_d7 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4, double f_d5, double f_d6, double f_d7 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_d1, f_d2, f_d3, f_d4, f_d5, f_d6, f_d7 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4, double f_d5, double f_d6, double f_d7, double f_d8 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4, double f_d5, double f_d6, double f_d7, double f_d8 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_d1, f_d2, f_d3, f_d4, f_d5, f_d6, f_d7, f_d8 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4, double f_d5, double f_d6, double f_d7, double f_d8, double f_d9 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4, double f_d5, double f_d6, double f_d7, double f_d8, double f_d9 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_d1, f_d2, f_d3, f_d4, f_d5, f_d6, f_d7, f_d8, f_d9 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4, double f_d5, double f_d6, double f_d7, double f_d8, double f_d9, double f_d10 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4, double f_d5, double f_d6, double f_d7, double f_d8, double f_d9, double f_d10 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_d1, f_d2, f_d3, f_d4, f_d5, f_d6, f_d7, f_d8, f_d9, f_d10 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, double f_d1, LPCSTR f_psz2 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, double f_d1, LPCSTR f_psz2 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_d1, f_psz2 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, double f_d1, double f_d2, LPCSTR f_psz3 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, double f_d1, double f_d2, LPCSTR f_psz3 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_d1, f_d2, f_psz3 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, LPCSTR f_psz1, __int32 f_n2 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, LPCSTR f_psz1, __int32 f_n2 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_n2 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, LPCSTR f_psz1, LPCSTR f_psz2, __int32 f_n3 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, LPCSTR f_psz1, LPCSTR f_psz2, __int32 f_n3 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_psz2, f_n3 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, LPCSTR f_psz1, LPCSTR f_psz2, __int32 f_n3, __int32 f_n4 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, LPCSTR f_psz1, LPCSTR f_psz2, __int32 f_n3, __int32 f_n4 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_psz2, f_n3, f_n4 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, LPCSTR f_psz1, LPCSTR f_psz2, LPCSTR f_psz3, __int32 f_n4 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, LPCSTR f_psz1, LPCSTR f_psz2, LPCSTR f_psz3, __int32 f_n4 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_psz2, f_psz3, f_n4 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, LPCSTR f_psz1, __int32 f_n2, __int32 f_n3 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, LPCSTR f_psz1, __int32 f_n2, __int32 f_n3 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_n2, f_n3 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, LPCSTR f_psz1, __int32 f_n2, __int32 f_n3, LPCSTR f_psz4 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, LPCSTR f_psz1, __int32 f_n2, __int32 f_n3, LPCSTR f_psz4 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_n2, f_n3, f_psz4 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, LPCSTR f_psz1, __int32 f_n2, __int32 f_n3, __int32 f_n4 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, LPCSTR f_psz1, __int32 f_n2, __int32 f_n3, __int32 f_n4 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_n2, f_n3, f_n4 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, LPCSTR f_psz1, __int32 f_n2, LPCSTR f_psz3 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, LPCSTR f_psz1, __int32 f_n2, LPCSTR f_psz3 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_n2, f_psz3 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, LPCSTR f_psz1, __int32 f_n2, LPCSTR f_psz3, LPCSTR f_psz4 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, LPCSTR f_psz1, __int32 f_n2, LPCSTR f_psz3, LPCSTR f_psz4 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_n2, f_psz3, f_psz4 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, __int32 f_n1, LPCSTR f_psz2 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, __int32 f_n1, LPCSTR f_psz2 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_psz2 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, __int32 f_n1, LPCSTR f_psz2, LPCSTR f_psz3 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, __int32 f_n1, LPCSTR f_psz2, LPCSTR f_psz3 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_psz2, f_psz3 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, __int32 f_n1, LPCSTR f_psz2, LPCSTR f_psz3, LPCSTR f_psz4 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, __int32 f_n1, LPCSTR f_psz2, LPCSTR f_psz3, LPCSTR f_psz4 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_psz2, f_psz3, f_psz4 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, __int32 f_n1, __int32 f_n2, LPCSTR f_psz3 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, __int32 f_n1, __int32 f_n2, LPCSTR f_psz3 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_n2, f_psz3 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, LPCSTR f_psz4 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, LPCSTR f_psz4 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_n2, f_n3, f_psz4 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, __int32 f_n1, double f_d2 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, __int32 f_n1, double f_d2 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_d2 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, __int32 f_n1, __int32 f_n2, double f_d3 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, __int32 f_n1, __int32 f_n2, double f_d3 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_n2, f_d3 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, char f_ch1 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, char f_ch1 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_ch1 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, char f_ch1, char f_ch2 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, char f_ch1, char f_ch2 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_ch1, f_ch2 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, char f_ch1, LPCSTR f_psz2 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, char f_ch1, LPCSTR f_psz2 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_ch1, f_psz2 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, char f_ch1, LPCSTR f_psz2, LPCSTR f_psz3 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, char f_ch1, LPCSTR f_psz2, LPCSTR f_psz3 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_ch1, f_psz2, f_psz3 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintA( LPSTR f_pszDestin, const size_t f_unMaxSize, LPCSTR f_pszFormat, char f_ch1, char f_ch2, LPCSTR f_psz3 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintA( char (&f_rDestin)[f_unSize], LPCSTR f_pszFormat, char f_ch1, char f_ch2, LPCSTR f_psz3 )
    {
      return( MKI_PrintA( (LPSTR) &f_rDestin, f_unSize, f_pszFormat, f_ch1, f_ch2, f_psz3 ) );
    }

#endif

//---------------------------------------------------------------------------------------------------------------------------------------------------
// Unicode text handling

#ifndef MKI_MACROS_W_H

MKI_INTERFACE_MEMBER void
MKI_InitW( LPWSTR f_pszDestin );

MKI_INTERFACE_MEMBER bool
MKI_EqualW( LPCWSTR f_pszText1, LPCWSTR f_pszText2 );

MKI_INTERFACE_MEMBER __int32
MKI_CompareW( LPCWSTR f_pszText1, LPCWSTR f_pszText2 );

MKI_INTERFACE_MEMBER void
MKI_TerminateW( LPWSTR f_pszDestin, const size_t f_unMaxSize );

    template <size_t f_unSize> __forceinline void
    MKI_TerminateW( wchar_t (&f_rDestin)[f_unSize] )
    {
      MKI_TerminateW( (LPWSTR) &f_rDestin, f_unSize );
    }

MKI_INTERFACE_MEMBER void
MKI_CopyPartW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszSource1, const size_t f_unCharCount );

    template <size_t f_unSize> __forceinline void 
    MKI_CopyPartW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszSource1, const size_t f_unCharCount )
    {
      MKI_CopyPartW( (LPWSTR) &f_rDestin, f_unSize, f_pszSource1, f_unCharCount );
    }

MKI_INTERFACE_MEMBER void
MKI_CopyW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszSource1 );

    template <size_t f_unSize> __forceinline void 
    MKI_CopyW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszSource1 )
    {
      MKI_CopyW( (LPWSTR) &f_rDestin, f_unSize, f_pszSource1 );
    }

MKI_INTERFACE_MEMBER void
MKI_CopyW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszSource1, LPCWSTR f_pszSource2 );

    template <size_t f_unSize> __forceinline void 
    MKI_CopyW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszSource1, LPCWSTR f_pszSource2 )
    {
      MKI_CopyW( (LPWSTR) &f_rDestin, f_unSize, f_pszSource1, f_pszSource2 );
    }

MKI_INTERFACE_MEMBER void
MKI_CopyW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszSource1, LPCWSTR f_pszSource2, LPCWSTR f_pszSource3 );

    template <size_t f_unSize> __forceinline void 
    MKI_CopyW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszSource1, LPCWSTR f_pszSource2, LPCWSTR f_pszSource3 )
    {
      MKI_CopyW( (LPWSTR) &f_rDestin, f_unSize, f_pszSource1, f_pszSource2, f_pszSource3 );
    }

MKI_INTERFACE_MEMBER void
MKI_CopyW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszSource1, LPCWSTR f_pszSource2, LPCWSTR f_pszSource3, LPCWSTR f_pszSource4 );

    template <size_t f_unSize> __forceinline void 
    MKI_CopyW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszSource1, LPCWSTR f_pszSource2, LPCWSTR f_pszSource3, LPCWSTR f_pszSource4 )
    {
      MKI_CopyW( (LPWSTR) &f_rDestin, f_unSize, f_pszSource1, f_pszSource2, f_pszSource3, f_pszSource4 );
    }

MKI_INTERFACE_MEMBER void
MKI_CopyW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszSource1, LPCWSTR f_pszSource2, LPCWSTR f_pszSource3, LPCWSTR f_pszSource4, LPCWSTR f_pszSource5 );

    template <size_t f_unSize> __forceinline void 
    MKI_CopyW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszSource1, LPCWSTR f_pszSource2, LPCWSTR f_pszSource3, LPCWSTR f_pszSource4, LPCWSTR f_pszSource5 )
    {
      MKI_CopyW( (LPWSTR) &f_rDestin, f_unSize, f_pszSource1, f_pszSource2, f_pszSource3, f_pszSource4, f_pszSource5 );
    }

MKI_INTERFACE_MEMBER void
MKI_CopyW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszSource1, LPCWSTR f_pszSource2, LPCWSTR f_pszSource3, LPCWSTR f_pszSource4, LPCWSTR f_pszSource5, LPCWSTR f_pszSource6 );

    template <size_t f_unSize> __forceinline void 
    MKI_CopyW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszSource1, LPCWSTR f_pszSource2, LPCWSTR f_pszSource3, LPCWSTR f_pszSource4, LPCWSTR f_pszSource5, LPCWSTR f_pszSource6 )
    {
      MKI_CopyW( (LPWSTR) &f_rDestin, f_unSize, f_pszSource1, f_pszSource2, f_pszSource3, f_pszSource4, f_pszSource5, f_pszSource6 );
    }

MKI_INTERFACE_MEMBER void
MKI_CopyW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszSource1, LPCWSTR f_pszSource2, LPCWSTR f_pszSource3, LPCWSTR f_pszSource4, LPCWSTR f_pszSource5, LPCWSTR f_pszSource6, LPCWSTR f_pszSource7 );

    template <size_t f_unSize> __forceinline void 
    MKI_CopyW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszSource1, LPCWSTR f_pszSource2, LPCWSTR f_pszSource3, LPCWSTR f_pszSource4, LPCWSTR f_pszSource5, LPCWSTR f_pszSource6, LPCWSTR f_pszSource7 )
    {
      MKI_CopyW( (LPWSTR) &f_rDestin, f_unSize, f_pszSource1, f_pszSource2, f_pszSource3, f_pszSource4, f_pszSource5, f_pszSource6, f_pszSource7 );
    }

MKI_INTERFACE_MEMBER void
MKI_CopyW( LPWSTR f_pszDestin, const size_t f_unMaxSize, wchar_t f_chChar );

    template <size_t f_unSize> __forceinline void 
    MKI_CopyW( wchar_t (&f_rDestin)[f_unSize], wchar_t f_chChar )
    {
      MKI_CopyW( (LPWSTR) &f_rDestin, f_unSize, f_chChar );
    }

MKI_INTERFACE_MEMBER bool
MKI_JoinPartW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszSource1, const size_t f_unCharCount );

    template <size_t f_unSize> __forceinline void 
    MKI_JoinPartW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszSource1, const size_t f_unCharCount )
    {
      MKI_JoinPartW( (LPSTR) &f_rDestin, f_unSize, f_pszSource1, f_unCharCount );
    }

MKI_INTERFACE_MEMBER bool
MKI_JoinW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszSource1 );

    template <size_t f_unSize> __forceinline bool 
    MKI_JoinW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszSource1 )
    {
      return( MKI_JoinW( (LPWSTR) &f_rDestin, f_unSize, f_pszSource1 ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_JoinW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszSource1, LPCWSTR f_pszSource2 );

    template <size_t f_unSize> __forceinline bool 
    MKI_JoinW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszSource1, LPCWSTR f_pszSource2 )
    {
      return( MKI_JoinW( (LPWSTR) &f_rDestin, f_unSize, f_pszSource1, f_pszSource2 ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_JoinW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszSource1, LPCWSTR f_pszSource2, LPCWSTR f_pszSource3 );

    template <size_t f_unSize> __forceinline bool 
    MKI_JoinW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszSource1, LPCWSTR f_pszSource2, LPCWSTR f_pszSource3 )
    {
      return( MKI_JoinW( (LPWSTR) &f_rDestin, f_unSize, f_pszSource1, f_pszSource2, f_pszSource3 ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_JoinW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszSource1, LPCWSTR f_pszSource2, LPCWSTR f_pszSource3, LPCWSTR f_pszSource4 );

    template <size_t f_unSize> __forceinline bool 
    MKI_JoinW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszSource1, LPCWSTR f_pszSource2, LPCWSTR f_pszSource3, LPCWSTR f_pszSource4 )
    {
      return( MKI_JoinW( (LPWSTR) &f_rDestin, f_unSize, f_pszSource1, f_pszSource2, f_pszSource3, f_pszSource4 ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_JoinW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszSource1, LPCWSTR f_pszSource2, LPCWSTR f_pszSource3, LPCWSTR f_pszSource4, LPCWSTR f_pszSource5 );

    template <size_t f_unSize> __forceinline bool 
    MKI_JoinW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszSource1, LPCWSTR f_pszSource2, LPCWSTR f_pszSource3, LPCWSTR f_pszSource4, LPCWSTR f_pszSource5 )
    {
      return( MKI_JoinW( (LPWSTR) &f_rDestin, f_unSize, f_pszSource1, f_pszSource2, f_pszSource3, f_pszSource4, f_pszSource5 ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_JoinW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszSource1, LPCWSTR f_pszSource2, LPCWSTR f_pszSource3, LPCWSTR f_pszSource4, LPCWSTR f_pszSource5, LPCWSTR f_pszSource6 );

    template <size_t f_unSize> __forceinline bool 
    MKI_JoinW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszSource1, LPCWSTR f_pszSource2, LPCWSTR f_pszSource3, LPCWSTR f_pszSource4, LPCWSTR f_pszSource5, LPCWSTR f_pszSource6 )
    {
      return( MKI_JoinW( (LPWSTR) &f_rDestin, f_unSize, f_pszSource1, f_pszSource2, f_pszSource3, f_pszSource4, f_pszSource5, f_pszSource6 ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_JoinW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszSource1, LPCWSTR f_pszSource2, LPCWSTR f_pszSource3, LPCWSTR f_pszSource4, LPCWSTR f_pszSource5, LPCWSTR f_pszSource6, LPCWSTR f_pszSource7 );

    template <size_t f_unSize> __forceinline bool 
    MKI_JoinW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszSource1, LPCWSTR f_pszSource2, LPCWSTR f_pszSource3, LPCWSTR f_pszSource4, LPCWSTR f_pszSource5, LPCWSTR f_pszSource6, LPCWSTR f_pszSource7 )
    {
      return( MKI_JoinW( (LPWSTR) &f_rDestin, f_unSize, f_pszSource1, f_pszSource2, f_pszSource3, f_pszSource4, f_pszSource5, f_pszSource6, f_pszSource7 ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_JoinW( LPWSTR f_pszDestin, const size_t f_unMaxSize, wchar_t f_chChar );

    template <size_t f_unSize> __forceinline bool 
    MKI_JoinW( wchar_t (&f_rDestin)[f_unSize], wchar_t f_chChar )
    {
      return( MKI_JoinW( (LPWSTR) &f_rDestin, f_unSize, f_chChar ) );
    }

MKI_INTERFACE_MEMBER void
MKI_PathW( LPWSTR f_pszDestin, const size_t f_unMaxSize );

    template <size_t f_unSize> __forceinline void 
    MKI_PathW( wchar_t (&f_rDestin)[f_unSize] )
    {
      MKI_PathW( (LPWSTR) &f_rDestin, f_unSize );
    }

MKI_INTERFACE_MEMBER void
MKI_MakePathW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszDrive, LPCWSTR f_pszPath, LPCWSTR f_pszFile, LPCWSTR f_pszExt );

    template <size_t f_unSize> __forceinline void
    MKI_MakePathW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszDrive, LPCWSTR f_pszPath, LPCWSTR f_pszFile, LPCWSTR f_pszExt )
    {
      MKI_MakePathW( (LPWSTR) &f_rDestin, f_unSize, f_pszDrive, f_pszPath, f_pszFile, f_pszExt );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, LPCWSTR f_psz1 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, LPCWSTR f_psz1 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, LPCWSTR f_psz1, LPCWSTR f_psz2 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, LPCWSTR f_psz1, LPCWSTR f_psz2 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_psz2 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, LPCWSTR f_psz1, LPCWSTR f_psz2, LPCWSTR f_psz3 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, LPCWSTR f_psz1, LPCWSTR f_psz2, LPCWSTR f_psz3 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_psz2, f_psz3 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, LPCWSTR f_psz1, LPCWSTR f_psz2, LPCWSTR f_psz3, LPCWSTR f_psz4 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, LPCWSTR f_psz1, LPCWSTR f_psz2, LPCWSTR f_psz3, LPCWSTR f_psz4 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_psz2, f_psz3, f_psz4 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, LPCWSTR f_psz1, LPCWSTR f_psz2, LPCWSTR f_psz3, LPCWSTR f_psz4, LPCWSTR f_psz5 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, LPCWSTR f_psz1, LPCWSTR f_psz2, LPCWSTR f_psz3, LPCWSTR f_psz4, LPCWSTR f_psz5 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_psz2, f_psz3, f_psz4, f_psz5 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, LPCWSTR f_psz1, LPCWSTR f_psz2, LPCWSTR f_psz3, LPCWSTR f_psz4, LPCWSTR f_psz5, LPCWSTR f_psz6 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, LPCWSTR f_psz1, LPCWSTR f_psz2, LPCWSTR f_psz3, LPCWSTR f_psz4, LPCWSTR f_psz5, LPCWSTR f_psz6 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_psz2, f_psz3, f_psz4, f_psz5, f_psz6 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, LPCWSTR f_psz1, LPCWSTR f_psz2, LPCWSTR f_psz3, LPCWSTR f_psz4, LPCWSTR f_psz5, LPCWSTR f_psz6, LPCWSTR f_psz7 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, LPCWSTR f_psz1, LPCWSTR f_psz2, LPCWSTR f_psz3, LPCWSTR f_psz4, LPCWSTR f_psz5, LPCWSTR f_psz6, LPCWSTR f_psz7 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_psz2, f_psz3, f_psz4, f_psz5, f_psz6, f_psz7 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, LPCWSTR f_psz1, LPCWSTR f_psz2, LPCWSTR f_psz3, LPCWSTR f_psz4, LPCWSTR f_psz5, LPCWSTR f_psz6, LPCWSTR f_psz7, LPCWSTR f_psz8 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, LPCWSTR f_psz1, LPCWSTR f_psz2, LPCWSTR f_psz3, LPCWSTR f_psz4, LPCWSTR f_psz5, LPCWSTR f_psz6, LPCWSTR f_psz7, LPCWSTR f_psz8 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_psz2, f_psz3, f_psz4, f_psz5, f_psz6, f_psz7, f_psz8 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, LPCWSTR f_psz1, LPCWSTR f_psz2, LPCWSTR f_psz3, LPCWSTR f_psz4, LPCWSTR f_psz5, LPCWSTR f_psz6, LPCWSTR f_psz7, LPCWSTR f_psz8, LPCWSTR f_psz9 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, LPCWSTR f_psz1, LPCWSTR f_psz2, LPCWSTR f_psz3, LPCWSTR f_psz4, LPCWSTR f_psz5, LPCWSTR f_psz6, LPCWSTR f_psz7, LPCWSTR f_psz8, LPCWSTR f_psz9 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_psz2, f_psz3, f_psz4, f_psz5, f_psz6, f_psz7, f_psz8, f_psz9 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, LPCWSTR f_psz1, LPCWSTR f_psz2, LPCWSTR f_psz3, LPCWSTR f_psz4, LPCWSTR f_psz5, LPCWSTR f_psz6, LPCWSTR f_psz7, LPCWSTR f_psz8, LPCWSTR f_psz9, LPCWSTR f_psz10 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, LPCWSTR f_psz1, LPCWSTR f_psz2, LPCWSTR f_psz3, LPCWSTR f_psz4, LPCWSTR f_psz5, LPCWSTR f_psz6, LPCWSTR f_psz7, LPCWSTR f_psz8, LPCWSTR f_psz9, LPCWSTR f_psz10 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_psz2, f_psz3, f_psz4, f_psz5, f_psz6, f_psz7, f_psz8, f_psz9, f_psz10 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, __int32 f_n1 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, __int32 f_n1 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, __int32 f_n1, __int32 f_n2 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, __int32 f_n1, __int32 f_n2 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_n2 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_n2, f_n3 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_n2, f_n3, f_n4 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4, __int32 f_n5 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4, __int32 f_n5 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_n2, f_n3, f_n4, f_n5 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4, __int32 f_n5, __int32 f_n6 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4, __int32 f_n5, __int32 f_n6 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_n2, f_n3, f_n4, f_n5, f_n6 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4, __int32 f_n5, __int32 f_n6, __int32 f_n7 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4, __int32 f_n5, __int32 f_n6, __int32 f_n7 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_n2, f_n3, f_n4, f_n5, f_n6, f_n7 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4, __int32 f_n5, __int32 f_n6, __int32 f_n7, __int32 f_n8 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4, __int32 f_n5, __int32 f_n6, __int32 f_n7, __int32 f_n8 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_n2, f_n3, f_n4, f_n5, f_n6, f_n7, f_n8 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4, __int32 f_n5, __int32 f_n6, __int32 f_n7, __int32 f_n8, __int32 f_n9 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4, __int32 f_n5, __int32 f_n6, __int32 f_n7, __int32 f_n8, __int32 f_n9 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_n2, f_n3, f_n4, f_n5, f_n6, f_n7, f_n8, f_n9 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4, __int32 f_n5, __int32 f_n6, __int32 f_n7, __int32 f_n8, __int32 f_n9, __int32 f_n10 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, __int32 f_n4, __int32 f_n5, __int32 f_n6, __int32 f_n7, __int32 f_n8, __int32 f_n9, __int32 f_n10 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_n2, f_n3, f_n4, f_n5, f_n6, f_n7, f_n8, f_n9, f_n10 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, double f_d1 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, double f_d1 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_d1 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, double f_d1, double f_d2 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, double f_d1, double f_d2 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_d1, f_d2 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, double f_d1, double f_d2, double f_d3 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, double f_d1, double f_d2, double f_d3 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_d1, f_d2, f_d3 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_d1, f_d2, f_d3, f_d4 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4, double f_d5 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4, double f_d5 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_d1, f_d2, f_d3, f_d4, f_d5 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4, double f_d5, double f_d6 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4, double f_d5, double f_d6 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_d1, f_d2, f_d3, f_d4, f_d5, f_d6 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4, double f_d5, double f_d6, double f_d7 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4, double f_d5, double f_d6, double f_d7 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_d1, f_d2, f_d3, f_d4, f_d5, f_d6, f_d7 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4, double f_d5, double f_d6, double f_d7, double f_d8 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4, double f_d5, double f_d6, double f_d7, double f_d8 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_d1, f_d2, f_d3, f_d4, f_d5, f_d6, f_d7, f_d8 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4, double f_d5, double f_d6, double f_d7, double f_d8, double f_d9 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4, double f_d5, double f_d6, double f_d7, double f_d8, double f_d9 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_d1, f_d2, f_d3, f_d4, f_d5, f_d6, f_d7, f_d8, f_d9 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4, double f_d5, double f_d6, double f_d7, double f_d8, double f_d9, double f_d10 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, double f_d1, double f_d2, double f_d3, double f_d4, double f_d5, double f_d6, double f_d7, double f_d8, double f_d9, double f_d10 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_d1, f_d2, f_d3, f_d4, f_d5, f_d6, f_d7, f_d8, f_d9, f_d10 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, double f_d1, LPCWSTR f_psz2 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, double f_d1, LPCWSTR f_psz2 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_d1, f_psz2 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, double f_d1, double f_d2, LPCWSTR f_psz3 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, double f_d1, double f_d2, LPCWSTR f_psz3 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_d1, f_d2, f_psz3 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, LPCWSTR f_psz1, __int32 f_n2 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, LPCWSTR f_psz1, __int32 f_n2 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_n2 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, LPCWSTR f_psz1, LPCWSTR f_psz2, __int32 f_n3 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, LPCWSTR f_psz1, LPCWSTR f_psz2, __int32 f_n3 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_psz2, f_n3 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, LPCWSTR f_psz1, LPCWSTR f_psz2, __int32 f_n3, __int32 f_n4 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, LPCWSTR f_psz1, LPCWSTR f_psz2, __int32 f_n3, __int32 f_n4 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_psz2, f_n3, f_n4 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, LPCWSTR f_psz1, LPCWSTR f_psz2, LPCWSTR f_psz3, __int32 f_n4 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, LPCWSTR f_psz1, LPCWSTR f_psz2, LPCWSTR f_psz3, __int32 f_n4 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_psz2, f_psz3, f_n4 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, LPCWSTR f_psz1, __int32 f_n2, __int32 f_n3 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, LPCWSTR f_psz1, __int32 f_n2, __int32 f_n3 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_n2, f_n3 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, LPCWSTR f_psz1, __int32 f_n2, __int32 f_n3, LPCWSTR f_psz4 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, LPCWSTR f_psz1, __int32 f_n2, __int32 f_n3, LPCWSTR f_psz4 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_n2, f_n3, f_psz4 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, LPCWSTR f_psz1, __int32 f_n2, __int32 f_n3, __int32 f_n4 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, LPCWSTR f_psz1, __int32 f_n2, __int32 f_n3, __int32 f_n4 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_n2, f_n3, f_n4 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, LPCWSTR f_psz1, __int32 f_n2, LPCWSTR f_psz3 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, LPCWSTR f_psz1, __int32 f_n2, LPCWSTR f_psz3 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_n2, f_psz3 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, LPCWSTR f_psz1, __int32 f_n2, LPCWSTR f_psz3, LPCWSTR f_psz4 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, LPCWSTR f_psz1, __int32 f_n2, LPCWSTR f_psz3, LPCWSTR f_psz4 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_psz1, f_n2, f_psz3, f_psz4 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, __int32 f_n1, LPCWSTR f_psz2 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, __int32 f_n1, LPCWSTR f_psz2 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_psz2 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, __int32 f_n1, LPCWSTR f_psz2, LPCWSTR f_psz3 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, __int32 f_n1, LPCWSTR f_psz2, LPCWSTR f_psz3 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_psz2, f_psz3 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, __int32 f_n1, LPCWSTR f_psz2, LPCWSTR f_psz3, LPCWSTR f_psz4 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, __int32 f_n1, LPCWSTR f_psz2, LPCWSTR f_psz3, LPCWSTR f_psz4 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_psz2, f_psz3, f_psz4 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, __int32 f_n1, __int32 f_n2, LPCWSTR f_psz3 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, __int32 f_n1, __int32 f_n2, LPCWSTR f_psz3 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_n2, f_psz3 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, LPCWSTR f_psz4 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, __int32 f_n1, __int32 f_n2, __int32 f_n3, LPCWSTR f_psz4 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_n2, f_n3, f_psz4 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, __int32 f_n1, double f_d2 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, __int32 f_n1, double f_d2 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_d2 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, __int32 f_n1, __int32 f_n2, double f_d3 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, __int32 f_n1, __int32 f_n2, double f_d3 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_n1, f_n2, f_d3 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, wchar_t f_ch1 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, wchar_t f_ch1 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_ch1 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, wchar_t f_ch1, wchar_t f_ch2 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, wchar_t f_ch1, wchar_t f_ch2 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_ch1, f_ch2 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, wchar_t f_ch1, LPCWSTR f_psz2 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, wchar_t f_ch1, LPCWSTR f_psz2 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_ch1, f_psz2 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, wchar_t f_ch1, LPCWSTR f_psz2, LPCWSTR f_psz3 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, wchar_t f_ch1, LPCWSTR f_psz2, LPCWSTR f_psz3 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_ch1, f_psz2, f_psz3 ) );
    }

MKI_INTERFACE_MEMBER size_t
MKI_PrintW( LPWSTR f_pszDestin, const size_t f_unMaxSize, LPCWSTR f_pszFormat, wchar_t f_ch1, wchar_t f_ch2, LPCWSTR f_psz3 );

    template <size_t f_unSize> __forceinline size_t
    MKI_PrintW( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszFormat, wchar_t f_ch1, wchar_t f_ch2, LPCWSTR f_psz3 )
    {
      return( MKI_PrintW( (LPWSTR) &f_rDestin, f_unSize, f_pszFormat, f_ch1, f_ch2, f_psz3 ) );
    }

#endif

//---------------------------------------------------------------------------------------------------------------------------------------------------

#ifndef MKI_MACROS_W_H

MKI_INTERFACE_MEMBER bool
MKI_TextIsUTF8( const LPCSTR f_pszSource );

MKI_INTERFACE_MEMBER bool
MKI_ConvertAutoToUTF16( LPCSTR f_pszSource, LPWSTR f_pszDestin, const size_t f_unMaxSize );

    template <size_t f_unSize> __forceinline bool
    MKI_ConvertAutoToUTF16( LPCSTR f_pszSource, wchar_t (&f_rDestin)[f_unSize] )
    {
      return( MKI_ConvertAutoToUTF16( f_pszSource, (LPWSTR) &f_rDestin, f_unSize ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_ConvertAnsiToUTF16( LPCSTR f_pszSource, LPWSTR f_pszDestin, const size_t f_unMaxSize );

    template <size_t f_unSize> __forceinline bool
    MKI_ConvertAnsiToUTF16( LPCSTR f_pszSource, wchar_t (&f_rDestin)[f_unSize] )
    {
      return( MKI_ConvertAnsiToUTF16( f_pszSource, (LPWSTR) &f_rDestin, f_unSize ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_ConvertAnsiToUTF16( LPCSTR f_pszSource, const size_t f_unCharNum, LPWSTR f_pszDestin, const size_t f_unMaxSize );

    template <size_t f_unSize> __forceinline bool
    MKI_ConvertAnsiToUTF16( LPCSTR f_pszSource, const size_t f_unCharNum, wchar_t (&f_rDestin)[f_unSize] )
    {
      return( MKI_ConvertAnsiToUTF16( f_pszSource, f_unCharNum, (LPWSTR) &f_rDestin, f_unSize ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_ConvertUTF16ToUntaggedAnsi( LPCWSTR f_pszSource, LPSTR f_pszDestin, const size_t f_unMaxSize );

    template <size_t f_unSize> __forceinline bool
    MKI_ConvertUTF16ToUntaggedAnsi( LPCWSTR f_pszSource, char (&f_rDestin)[f_unSize] )
    {
      return( MKI_ConvertUTF16ToUntaggedAnsi( f_pszSource, f_rDestin, f_unSize ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_ConvertTaggedAnsiToUTF16( LPCSTR f_pszSource, LPWSTR f_pszDestin, const size_t f_unMaxSize );

    template <size_t f_unSize> __forceinline bool
    MKI_ConvertTaggedAnsiToUTF16( LPCSTR f_pszSource, wchar_t (&f_rDestin)[f_unSize] )
    {
      return( MKI_ConvertTaggedAnsiToUTF16( f_pszSource, f_rDestin, f_unSize ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_ConvertTaggedAnsiToUTF16( LPCSTR f_pszSource, const size_t f_unCharNum, LPWSTR f_pszDestin, const size_t f_unMaxSize );

    template <size_t f_unSize> __forceinline bool
    MKI_ConvertTaggedAnsiToUTF16( LPCSTR f_pszSource, const size_t f_unCharNum, wchar_t (&f_rDestin)[f_unSize] )
    {
      return( MKI_ConvertTaggedAnsiToUTF16( f_pszSource, f_unCharNum, f_rDestin, f_unSize ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_ConvertTaggedUTF16ToUTF16( LPCWSTR f_pszSource, LPWSTR f_pszDestin, const size_t f_unMaxSize );

    template <size_t f_unSize> __forceinline bool
    MKI_ConvertTaggedUTF16ToUTF16( LPCWSTR f_pszSource, wchar_t (&f_rDestin)[f_unSize] )
    {
      return( MKI_ConvertTaggedUTF16ToUTF16( f_pszSource, f_rDestin, f_unSize ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_ConvertTaggedUTF16ToUTF16( LPCWSTR f_pszSource, const size_t f_unCharNum, LPWSTR f_pszDestin, const size_t f_unMaxSize );

    template <size_t f_unSize> __forceinline bool
    MKI_ConvertTaggedUTF16ToUTF16( LPCWSTR f_pszSource, const size_t f_unCharNum, wchar_t (&f_rDestin)[f_unSize] )
    {
      return( MKI_ConvertTaggedUTF16toUTF16( f_pszSource, f_unCharNum, f_rDestin, f_unSize ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_ConvertUTF16ToAnsi( LPCWSTR f_pszSource, LPSTR f_pszDestin, const size_t f_unMaxSize );

    template <size_t f_unSize> __forceinline bool
    MKI_ConvertUTF16ToAnsi( LPCWSTR f_pszSource, char (&f_rDestin)[f_unSize] )
    {
      return( MKI_ConvertUTF16ToAnsi( f_pszSource, (LPSTR) &f_rDestin, f_unSize ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_ConvertUTF16ToAnsi( LPCWSTR f_pszSource, const size_t f_unCharNum, LPSTR f_pszDestin, const size_t f_unMaxSize );

    template <size_t f_unSize> __forceinline bool
    MKI_ConvertUTF16ToAnsi( LPCWSTR f_pszSource, const size_t f_unCharNum, char (&f_rDestin)[f_unSize] )
    {
      return( MKI_ConvertUTF16ToAnsi( f_pszSource, f_unCharNum, (LPSTR) &f_rDestin, f_unSize ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_ConvertUTF16ToTaggedAnsi( LPCWSTR f_pszSource, LPSTR f_pszDestin, const size_t f_unMaxSize );

    template <size_t f_unSize> __forceinline bool
    MKI_ConvertUTF16ToTaggedAnsi( LPCWSTR f_pszSource, char (&f_rDestin)[f_unSize] )
    {
      return( MKI_ConvertUTF16ToTaggedAnsi( f_pszSource, (LPSTR) &f_rDestin, f_unSize ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_ConvertUTF16ToTaggedAnsi( LPCWSTR f_pszSource, const size_t f_unCharNum, LPSTR f_pszDestin, const size_t f_unMaxSize );

    template <size_t f_unSize> __forceinline bool
    MKI_ConvertUTF16ToTaggedAnsi( LPCWSTR f_pszSource, const size_t f_unCharNum, char (&f_rDestin)[f_unSize] )
    {
      return( MKI_ConvertUTF16ToTaggedAnsi( f_pszSource, f_unCharNum, (LPSTR) &f_rDestin, f_unSize ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_ConvertUTF16ToUTF8( LPCWSTR f_pszSource, LPSTR f_pszDestin, const size_t f_unMaxSize );

    template <size_t f_unSize> __forceinline bool
    MKI_ConvertUTF16ToUTF8( LPCWSTR f_pszSource, char (&f_rDestin)[f_unSize] )
    {
      return( MKI_ConvertUTF16ToUTF8( f_pszSource, (LPSTR) &f_rDestin, f_unSize ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_ConvertUTF16ToUTF8( LPCWSTR f_pszSource, const size_t f_unCharNum, LPSTR f_pszDestin, const size_t f_unMaxSize );

    template <size_t f_unSize> __forceinline bool
    MKI_ConvertUTF16ToUTF8( LPCWSTR f_pszSource, const size_t f_unCharNum, char (&f_rDestin)[f_unSize] )
    {
      return( MKI_ConvertUTF16ToUTF8( f_pszSource, f_unCharNum, (LPSTR) &f_rDestin, f_unSize ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_ConvertUTF8ToUTF16( const LPCSTR f_pszSource, LPWSTR f_pszDestin, const size_t f_unMaxSize );

    template <size_t f_unSize> __forceinline bool
    MKI_ConvertUTF8ToUTF16( const LPCSTR f_pszSource, wchar_t (&f_rDestin)[f_unSize] )
    {
      return( MKI_ConvertUTF8ToUTF16( f_pszSource, (LPWSTR) &f_rDestin, f_unSize ) );
    }

MKI_INTERFACE_MEMBER bool
MKI_ConvertUTF8ToUTF16( const LPCSTR f_pszSource, const size_t f_unCharNum, LPWSTR f_pszDestin, const size_t f_unMaxSize );

    template <size_t f_unSize> __forceinline bool
    MKI_ConvertUTF8ToUTF16( const LPCSTR f_pszSource, const size_t f_unCharNum, wchar_t (&f_rDestin)[f_unSize] )
    {
      return( MKI_ConvertUTF8ToUTF16( f_pszSource, f_unCharNum, (LPWSTR) &f_rDestin, f_unSize ) );
    }

MKI_INTERFACE_MEMBER __int32
MKI_ConvertInt32ToUTF16( __int32 f_nSource, LPWSTR f_pszDestin, const size_t f_unMaxSize );

    template <size_t f_unSize> __forceinline __int32
    MKI_ConvertInt32ToUTF16( __int32 f_nSource, wchar_t (&f_rDestin)[f_unSize] )
    {
      return( MKI_ConvertInt32ToUTF16( f_nSource, (LPWSTR) &f_rDestin, f_unSize ) );
    }

MKI_INTERFACE_MEMBER __int32
MKI_ConvertUTF16ToInt32( const LPCWSTR f_pszSource, __int32& f_rDestin );

#endif

//---------------------------------------------------------------------------------------------------------------------------------------------------

    template <size_t f_unSize> __forceinline UINT
    MKI_DialogControlGetText( HWND f_hDlg, __int32 f_nDlgItem, char (&f_rDestin)[f_unSize] )
    {
      return( GetDlgItemTextA( f_hDlg, f_nDlgItem, (LPSTR) &f_rDestin, f_unSize ) );
    }

    template <size_t f_unSize> __forceinline UINT
    MKI_DialogControlGetText( HWND f_hDlg, __int32 f_nDlgItem, wchar_t (&f_rDestin)[f_unSize] )
    {
      return( GetDlgItemTextW( f_hDlg, f_nDlgItem, (LPWSTR) &f_rDestin, f_unSize ) );
    }

//---------------------------------------------------------------------------------------------------------------------------------------------------
typedef char MKI_FILENAMEA[MKI_LENGTH_PATH],
             MKI_DUMMYSTRA[MKI_LENGTH_PATH + MKI_LENGTH_TEXT_SHORT],
             MKI_STR8A    [8],
             MKI_STR16A   [16],
             MKI_STR32A   [32],
             MKI_STR64A   [64],
             MKI_STR128A  [128],
             MKI_STR256A  [256],
             MKI_STR512A  [512],
             MKI_STR1024A [1024],
             MKI_STR2048A [2048],
             MKI_STR4096A [4096],
             MKI_STR8192A [8192],
             MKI_STR16384A[16384],
             MKI_STR32768A[32768];
             
typedef wchar_t MKI_FILENAMEW[MKI_LENGTH_PATH],
                MKI_DUMMYSTRW[MKI_LENGTH_PATH + MKI_LENGTH_TEXT_SHORT],
                MKI_STR8W    [8],
                MKI_STR16W   [16],
                MKI_STR32W   [32],
                MKI_STR64W   [64],
                MKI_STR128W  [128],
                MKI_STR256W  [256],
                MKI_STR512W  [512],
                MKI_STR1024W [1024],
                MKI_STR2048W [2048],
                MKI_STR4096W [4096],
                MKI_STR8192W [8192],
                MKI_STR16384W[16384],
                MKI_STR32768W[32768];

typedef wchar_t MKI_STRSHORTW  [MKI_LENGTH_TEXT_SHORT+4],
                MKI_STRSHORTWx2[MKI_LENGTH_TEXT_SHORT*2+4],
                MKI_STRSHORTWx3[MKI_LENGTH_TEXT_SHORT*3+4],
                MKI_STRSHORTWx5[MKI_LENGTH_TEXT_SHORT*5+4],
                MKI_STRLONGW   [MKI_LENGTH_TEXT_LONG+4];


//---------------------------------------------------------------------------------------------------------------------------------------------------
typedef MKI_STR32W  MKI_ATTRIBUTENAME;
typedef wchar_t     MKI_ATTRIBUTEVALUE[MKI_LENGTH_ATTRIBUTE + 4];
typedef MKI_STR256W MKI_BLOCKNAME;
typedef MKI_STR32W  MKI_COLORNAME;
typedef MKI_STR256W MKI_COMMENTLONG;
typedef MKI_STR64W  MKI_COMMENTSHORT;
typedef MKI_STR64W  MKI_COLLECTIONNAME;
typedef MKI_STR64W  MKI_COLUMNNAME;
typedef MKI_STR64W  MKI_DATABASENAME;
typedef MKI_STR256W MKI_FILEREFERENCE;
typedef MKI_STR64W  MKI_FONTNAME;
typedef MKI_STR64W  MKI_HATCHNAME;
typedef MKI_STR64W  MKI_ITEMNAME;
typedef MKI_STR256W MKI_LAYERNAME;
typedef MKI_STR64W  MKI_LIBRARYNAME;
typedef MKI_STR64W  MKI_LINENAME;
typedef MKI_STR64W  MKI_PAGENAME;
typedef MKI_STR256W MKI_PENNAME;
typedef MKI_STR64W  MKI_TASKNAME;
typedef MKI_STR64W  MKI_SELFILTERNAME;
typedef MKI_STR64W  MKI_SYSTEMNAME;
typedef MKI_STR32W  MKI_VARIABLENAME;
typedef wchar_t     MKI_VARIABLEVALUE[MKI_LENGTH_VARIABLE + 4];


//---------------------------------------------------------------------------------------------------------------------------------------------------
#if GDIPVER >= 0x0100
typedef Gdiplus::Bitmap* MKI_BITMAP_PTR;
#else
typedef void* MKI_BITMAP_PTR;
#endif


//---------------------------------------------------------------------------------------------------------------------------------------------------
typedef MKI_BITFIELD<MKI_LAYER_MAX> MKI_HIDELAYERS;
MKI_CLASS_TYPES( HIDELAYERS )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_DIALOG_TEMPLATE
{
  public:

  MKI_DIALOG_TEMPLATE( void )
  { 
    m_pData = nullptr;
  }

  MKI_INTERFACE_MEMBER 
  MKI_DIALOG_TEMPLATE( HINSTANCE f_hInstance, LPCSTR f_pszTemplateName );

  MKI_INTERFACE_MEMBER 
  ~MKI_DIALOG_TEMPLATE();

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER LPDLGTEMPLATE
  Init( HINSTANCE f_hInstance, LPCSTR f_pszTemplateName );

  MKI_INTERFACE_MEMBER void
  Exit( void );

  //------------------------------------------------------------------

  __forceinline
  operator LPDLGTEMPLATE()
  {
    return( (LPDLGTEMPLATE) m_pData );
  }

  __forceinline
  operator LPVOID()
  {
    return( (LPVOID) m_pData );
  }

  __forceinline
  operator BYTE*()
  {
    return( m_pData );
  }

  __forceinline bool
  operator ==( const LPVOID f_pValue ) const
  { 
    return( m_pData == f_pValue );
  }

  __forceinline bool
  operator !=( const LPVOID f_pValue ) const
  { 
    return( m_pData != f_pValue );
  }

  //------------------------------------------------------------------

  private:

  // Disable copy constructor
  MKI_DIALOG_TEMPLATE( const MKI_DIALOG_TEMPLATE& f_rClass ) = delete;

  // Disable assignment operator
  MKI_DIALOG_TEMPLATE&
  operator =( const MKI_DIALOG_TEMPLATE& f_rClass ) = delete;

  //------------------------------------------------------------------

  BYTE* m_pData;
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_UNIQUE_ID
{
  public:

  MKI_UNIQUE_ID( void ){};

  MKI_INTERFACE_MEMBER void
  SetZero( void );

  MKI_INTERFACE_MEMBER bool
  IsZero( void ) const;

  MKI_INTERFACE_MEMBER bool
  IsNonZero( void ) const;

  MKI_INTERFACE_MEMBER bool
  IsIdentical( const MKI_UNIQUE_ID& f_rThat ) const;

  MKI_INTERFACE_MEMBER void
  Create( void );

  MKI_INTERFACE_MEMBER bool
  CreateIfZero( void );

  //------------------------------------------------------------------

  BYTE  m_abData[MKI_UNIQUE_ID_SIZE];
};

MKI_CLASS_TYPES( UNIQUE_ID )

//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_ITEM_HEADER
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  __int32 m_nOwnerID,
          m_nType,
          m_nVersion;

  MKI_ITEMNAME    m_szName;
  MKI_COMMENTLONG m_szComment;
};

MKI_CLASS_TYPES( ITEM_HEADER )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_FONTTYPE_INTERN   = 0,
  MKI_FONTTYPE_TRUETYPE = 1,
  MKI_FONTTYPE_DEVICE   = 2,
  MKI_FONTTYPE_MAX      = 3,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_FONTSTYLE_REGULAR   = 0x0000,
  MKI_FONTSTYLE_ITALIC    = 0x0001,
  MKI_FONTSTYLE_UNDERLINE = 0x0002,
  MKI_FONTSTYLE_OVERLINE  = 0x0004,
  MKI_FONTSTYLE_STRIKEOUT = 0x0008,
  MKI_FONTSTYLE_SETMASK   = 0xff00,
  MKI_FONTSTYLE_BASIC     = 0x0001,

  // Maximum number of concurrent font styles per font
  MKI_FONTSTYLE_MAX = 32,

  // Maximum number of font names
  MKI_FONTNAME_MAX  = 10000,
};

// Vertical position of overlining in dimension texts relative to font size
constexpr double MKI_FONT_OVERLINE_OFFSET = 1.05;


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_FONTDEF
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER bool
  IsIdentical( const MKI_FONTDEF& f_rFont ) const;

  MKI_INTERFACE_MEMBER bool
  IsFillable( void ) const;

  //------------------------------------------------------------------

  __int32       m_nFontType,
                m_nFontStyle,
                m_nFontWeight;
  MKI_FONTNAME  m_szFontName;
};

MKI_CLASS_TYPES( FONTDEF )

//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MULTI_BASE
{
  public:

  MKI_INTERFACE_MEMBER void
  SetNone( void );

  MKI_INTERFACE_MEMBER void
  SetAll( void );

  MKI_INTERFACE_MEMBER bool
  IsIndeterminate( const __int32 f_nFlag ) const;

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  Handle( bool& f_rCurrent, const bool f_fNew, const __int32 f_nFlag );

  MKI_INTERFACE_MEMBER void
  Handle( __int32& f_rCurrent, const __int32 f_nNew, const __int32 f_nBit, const __int32 f_nFlag );

  MKI_INTERFACE_MEMBER void
  Handle( __int32& f_rCurrent, const __int32 f_nNew, const __int32 f_nFlag );

  MKI_INTERFACE_MEMBER void
  Handle( __int32& f_rCurrent1, __int32& f_rCurrent2, const __int32 f_nNew1, const __int32 f_nNew2, const __int32 f_nFlag );

  MKI_INTERFACE_MEMBER void
  Handle( MKI_COLORVALUE_REF f_rCurrent, const MKI_COLORVALUE f_cNew, const __int32 f_nFlag );

  MKI_INTERFACE_MEMBER void
  Handle( MKI_COLORVALUE_REF f_rCurrent1, MKI_COLORMODEL_REF f_rCurrent2, 
          const MKI_COLORVALUE f_cNew1, const MKI_COLORMODEL f_cNew2, const __int32 f_nFlag );

  MKI_INTERFACE_MEMBER void
  Handle( BYTE& f_rCurrent, const BYTE f_cNew, const __int32 f_nFlag );

  MKI_INTERFACE_MEMBER void
  Handle( double& f_rCurrent, const double f_dNew, const __int32 f_nFlag );

  MKI_INTERFACE_MEMBER void
  Handle( double& f_rCurrent1, double& f_rCurrent2, const double f_dNew1, const double f_dNew2, const __int32 f_nFlag );

  MKI_INTERFACE_MEMBER void
  Handle( MKI_POINT_REF f_rCurrent, MKI_CONST_POINT_REF f_rNew, const __int32 f_nFlag );

  MKI_INTERFACE_MEMBER void
  Handle( MKI_VECTOR_REF f_rCurrent, MKI_CONST_VECTOR_REF f_rNew, const __int32 f_nFlag );

  MKI_INTERFACE_MEMBER void
  Handle( LPWSTR f_pszCurrent, const size_t f_unMaxSize, LPCWSTR f_pszNew, const __int32 f_nFlag );

    template <size_t f_unSize> __forceinline void 
    Handle( wchar_t (&f_rDestin)[f_unSize], LPCWSTR f_pszNew, const __int32 f_nFlag )
    {
      Handle( (LPWSTR) &f_rDestin, f_unSize, f_pszNew, f_nFlag );
    }

  MKI_INTERFACE_MEMBER void
  Handle( MKI_FONTDEF_REF f_rCurrent, MKI_CONST_FONTDEF_REF f_rNew, const __int32 f_nFlag );

  MKI_INTERFACE_MEMBER void
  Handle( MKI_HIDELAYERS_REF f_rCurrent, MKI_CONST_HIDELAYERS_REF f_rNew, const __int32 f_nFlag );

  //------------------------------------------------------------------

  bool    m_fValid;

  __int32 m_nUse,
          m_nInit,
          m_nMulti;
};

MKI_CLASS_TYPES( MULTI_BASE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_DEFPEN_DIMLINE          = 0,
  MKI_DEFPEN_DIMTEXT          = 1,
  MKI_DEFPEN_TEXT             = 2,
  MKI_DEFPEN_REFLINE          = 3,
  MKI_DEFPEN_GEOMETRY         = 4,
  MKI_DEFPEN_CENTERCROSS      = 5,
  MKI_DEFPEN_MIDPERPENDICULAR = 6,
  MKI_DEFPEN_DISTANT          = 7,
  MKI_DEFPEN_HATCH            = 8,
  MKI_DEFPEN_MAPPING          = 9,
  MKI_DEFPEN_SURFACE          = 10,
  MKI_DEFPEN_MAX              = 11,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_DEFLAYER_MARK             = 0,
  MKI_DEFLAYER_SURFACE          = 1,
  MKI_DEFLAYER_DIM              = 2,
  MKI_DEFLAYER_TEXT             = 3,
  MKI_DEFLAYER_HATCH            = 4,
  MKI_DEFLAYER_BLOCK            = 5,
  MKI_DEFLAYER_GEOMETRY         = 6,
  //                            = 7 - unused
  MKI_DEFLAYER_DISTANT          = 8,
  MKI_DEFLAYER_EXTERNAL         = 9,
  MKI_DEFLAYER_MAPPING          = 10,
  MKI_DEFLAYER_BITMAP           = 11,
  MKI_DEFLAYER_CENTERCROSS      = 12,
  MKI_DEFLAYER_MIDPERPENDICULAR = 13,
  MKI_DEFLAYER_MAX              = 14,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_DEFAULTDEF
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  __int32 m_anPens  [MKI_DEFAULT_MAX],
          m_anLayers[MKI_DEFAULT_MAX];
};

MKI_CLASS_TYPES( DEFAULTDEF )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_ZOOMDEF
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_POINT m_cPos;     // View center point
  double    m_dZoom,    // View zoom
            m_dRotate;  // View rotation
  __int32   m_nPage;    // Page index
};

MKI_CLASS_TYPES( ZOOMDEF )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_WINDOWSYSTEM
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  __int32 m_anWindow[MKI_WINDOW_MAX],
          m_nView;
};

MKI_CLASS_TYPES( WINDOWSYSTEM )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_WINDOWZOOM
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_ZOOMDEF m_acWindow[MKI_WINDOW_MAX],
              m_cView;
};

MKI_CLASS_TYPES( WINDOWZOOM )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_WINDOWDEF
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  __int32 m_nSystem;
  bool    m_fDisplayGrid,
          m_fPositionGrid;

  MKI_ZOOMDEF m_cZoom;
};

MKI_CLASS_TYPES( WINDOWDEF )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_VIEWMODE_PLAIN           = 0,
  MKI_VIEWMODE_ISOMETRIC_LEFT  = 1,
  MKI_VIEWMODE_ISOMETRIC_RIGHT = 2,
  MKI_VIEWMODE_ISOMETRIC_TOP   = 3,
  MKI_VIEWMODE_DIMETRIC1_LEFT  = 4,
  MKI_VIEWMODE_DIMETRIC1_RIGHT = 5,
  MKI_VIEWMODE_DIMETRIC1_TOP   = 6,
  MKI_VIEWMODE_DIMETRIC2_LEFT  = 7,
  MKI_VIEWMODE_DIMETRIC2_RIGHT = 8,
  MKI_VIEWMODE_DIMETRIC2_TOP   = 9,
  MKI_VIEWMODE_CABINET_RIGHT   = 10,
  MKI_VIEWMODE_CABINET_TOP     = 11,
  MKI_VIEWMODE_CAVALIER_RIGHT  = 12,
  MKI_VIEWMODE_CAVALIER_TOP    = 13,
  MKI_VIEWMODE_MAX             = 14,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_ORIGIN_CUSTOM       = 0,
  MKI_ORIGIN_TOP_LEFT     = 1,
  MKI_ORIGIN_TOP_EDGE     = 2,
  MKI_ORIGIN_TOP_RIGHT    = 3,
  MKI_ORIGIN_LEFT_EDGE    = 4,
  MKI_ORIGIN_CENTER       = 5,
  MKI_ORIGIN_RIGHT_EDGE   = 6,
  MKI_ORIGIN_BOTTOM_LEFT  = 7,
  MKI_ORIGIN_BOTTOM_EDGE  = 8,
  MKI_ORIGIN_BOTTOM_RIGHT = 9,
  MKI_ORIGIN_MAX          = 10,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_GRID_OFF       = 0,
  MKI_GRID_CARTESIAN = 1,
  MKI_GRID_ISOMETRIC = 2,
  MKI_GRID_DIMETRIC1 = 3,
  MKI_GRID_DIMETRIC2 = 4,
  MKI_GRID_MAX       = 5,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_FRACTION_FLOAT                = 0,
  MKI_FRACTION_MIXED                = 1,
  MKI_FRACTION_INCH_FLOAT           = 2,
  MKI_FRACTION_INCH_MIXED           = 3,
  MKI_FRACTION_YARD_FLOAT           = 4,
  MKI_FRACTION_YARD_MIXED           = 5,
  MKI_FRACTION_DEGREE_FLOAT         = 6,
  MKI_FRACTION_DEGREE_MIXED         = 7,
  MKI_FRACTION_ARCHITECT_FLOAT      = 8,
  MKI_FRACTION_ARCHITECT_CM         = 9,
  MKI_FRACTION_ARCHITECT_HALF_CM    = 10,
  MKI_FRACTION_ARCHITECT_QUARTER_CM = 11,
  MKI_FRACTION_ARCHITECT_TENTH_CM   = 12,
  MKI_FRACTION_MAX                  = 13,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_LENGTHUNIT_UM   = 0,
  MKI_LENGTHUNIT_MM   = 1,
  MKI_LENGTHUNIT_CM   = 2,
  MKI_LENGTHUNIT_DM   = 3,
  MKI_LENGTHUNIT_M    = 4,
  MKI_LENGTHUNIT_KM   = 5,
  MKI_LENGTHUNIT_MIL  = 6,
  MKI_LENGTHUNIT_INCH = 7,
  MKI_LENGTHUNIT_FOOT = 8,
  MKI_LENGTHUNIT_YARD = 9,
  MKI_LENGTHUNIT_MILE = 10,
  MKI_LENGTHUNIT_DP   = 11,
  MKI_LENGTHUNIT_PT   = 12,
  MKI_LENGTHUNIT_BP   = 13,
  MKI_LENGTHUNIT_CIC  = 14,
  MKI_LENGTHUNIT_MAX  = 15,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_ANGLEUNIT_DEG = 0,
  MKI_ANGLEUNIT_GRA = 1,
  MKI_ANGLEUNIT_RAD = 2,
  MKI_ANGLEUNIT_REL = 3,
  MKI_ANGLEUNIT_GEO = 4,
  MKI_ANGLEUNIT_MAX = 5,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_SYSTEMDEF
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( bool f_fImperial );

  MKI_INTERFACE_MEMBER __int32
  GetLengthUnit( void ) const;

  MKI_INTERFACE_MEMBER bool
  IsImperial( void ) const;

  MKI_INTERFACE_MEMBER bool
  IsArchitect( void ) const;

  //------------------------------------------------------------------

  MKI_SYSTEMNAME   m_szSystemName;
  MKI_COMMENTSHORT m_szSystemComment;

  double     m_dSystemRotate,
             m_dSystemScale;
  __int32    m_nSystemOption,
             m_nSystemOriginMode;
  MKI_POINT  m_cSystemOrigin;
  __int32    m_nSystemLenUnit,
             m_nSystemLineUnit,
             m_nSystemAngleUnit,
             m_nSystemFraction,
             m_nSystemPrecision;  
  __int32    m_nSystemGridMode;
  MKI_VECTOR m_cSystemGridSize;
  __int32    m_nSystemSnapMode;
  MKI_VECTOR m_cSystemSnapSize;
  MKI_MATRIX m_cSystemMMToUnit,
             m_cSystemUnitToMM;
};

MKI_CLASS_TYPES( SYSTEMDEF )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MULTI_SYSTEMDEF : public MKI_MULTI_BASE
{

public:

  enum
  {
    USE_SCALE       = 0x00000001,
    USE_LENGTH_MODE = 0x00000002,
    USE_ANGLE_MODE  = 0x00000004,
    USE_DIGITS      = 0x00000008,
    USE_ORIGIN_X    = 0x00000010,
    USE_ORIGIN_Y    = 0x00000020,
    USE_VIEW_MODE   = 0x00000040,
    USE_VIEW_ROTATE = 0x00000080,
    USE_LENGTH_UNIT = 0x00000100,
    USE_LINE_UNIT   = 0x00000200,
    USE_ANGLE_UNIT  = 0x00000400,
    USE_GRID        = 0x00000800,
    USE_SNAP        = 0x00001000,
  };

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  Fill( MKI_CONST_SYSTEMDEF_REF f_rSystemDef );

  MKI_INTERFACE_MEMBER void
  Apply( MKI_SYSTEMDEF_REF f_rSystemDef, bool f_fCopyAll ) const;

  //------------------------------------------------------------------

  MKI_SYSTEMDEF m_cData;
};

MKI_CLASS_TYPES( MULTI_SYSTEMDEF )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_LINEBLOCKENTRY
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  double  m_dLength;

  MKI_LIBRARYNAME m_szLibraryName;
  MKI_BLOCKNAME   m_szBlockName;

  bool    m_fNotFound;
};

MKI_CLASS_TYPES( LINEBLOCKENTRY )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_LINEBLOCKDEF
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER bool
  IsIdentical( const MKI_LINEBLOCKDEF& f_rData ) const;

  //------------------------------------------------------------------

  __int32  m_nBlockCount,
           m_nBlockMode;

  MKI_LINEBLOCKENTRY m_acBlockList[MKI_LINEBLOCKPART_MAX];
};

MKI_CLASS_TYPES( LINEBLOCKDEF )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_LINETYPE_STANDARD   = 0x00000000,
  MKI_LINETYPE_ABSOLUTE   = 0x00000001,
  MKI_LINETYPE_USELINE    = 0x00000002,
  MKI_LINETYPE_USEBLOCK   = 0x00000004,
  MKI_LINETYPE_ROTATED    = 0x00000008,
  MKI_LINETYPE_SCALED     = 0x00000010,
  MKI_LINETYPE_FLAG_COLOR = 0x00000020,
  MKI_LINETYPE_FLAG_WIDTH = 0x00000040,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_LINEDEF
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_LINEDEF( void );

  MKI_INTERFACE_MEMBER 
  ~MKI_LINEDEF();

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  Exit( void );

  MKI_INTERFACE_MEMBER bool
  Attach( const MKI_LINEBLOCKDEF& f_rData );

  MKI_INTERFACE_MEMBER bool
  Copy( const MKI_LINEDEF& f_rClass );

  MKI_INTERFACE_MEMBER bool
  CopyToLibrary( const LPCWSTR f_pszLibraryName, const MKI_LINEDEF& f_rClass );

  MKI_INTERFACE_MEMBER bool
  CopyFromLibrary( const LPCWSTR f_pszLibraryName, const MKI_LINEDEF& f_rClass );

  //------------------------------------------------------------------

  MKI_LINENAME     m_szLineName;
  MKI_COMMENTSHORT m_szLineComment;

  __int32  m_nLineNum,
           m_nLineMode;
  double   m_adLineData[MKI_LINEPART_MAX];

  MKI_LINEBLOCKDEF_PTR m_pLineBlock;

  //------------------------------------------------------------------

  private:

  // Disable copy constructor
  MKI_INTERFACE_MEMBER
  MKI_LINEDEF( const MKI_LINEDEF& f_rClass ) = delete;

  // Disable assignment operator
  MKI_LINEDEF&
  operator =( const MKI_LINEDEF& f_rClass ) = delete;
};

MKI_CLASS_TYPES( LINEDEF )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PROPERTY
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  CopyFromProperty( const MKI_PROPERTY& f_rSource, const __int32 f_nFlag );

  MKI_INTERFACE_MEMBER bool
  IsIdenticalBase( const MKI_PROPERTY& f_rProperty ) const;

  MKI_INTERFACE_MEMBER bool
  IsIdenticalFlag( const MKI_PROPERTY& f_rProperty, const __int32 f_nFlag ) const;

  __forceinline bool
  IsFilled( void ) const
  {
    return( m_nFillMode == MKI_FILLMODE_FILLED ||
            m_nFillMode == MKI_FILLMODE_FILLED_FRAMED );
  };

  __forceinline bool
  IsErased( void ) const
  {
    return( m_nFillMode == MKI_FILLMODE_ERASER ||
            m_nFillMode == MKI_FILLMODE_ERASER_FRAMED );
  };

  __forceinline bool
  IsOpaque( void ) const
  {
    return( m_nFillMode != MKI_FILLMODE_FRAMED );
  };

  __forceinline bool
  IsFramed( void ) const
  {
    return( m_nFillMode != MKI_FILLMODE_FILLED &&
            m_nFillMode != MKI_FILLMODE_ERASER );
  };

  //------------------------------------------------------------------

  __int32        m_nFillMode;
  MKI_COLORVALUE m_cFillColor,
                 m_cLineColor;
  __int32        m_nLineType;
  double         m_dLineWidth;
  __int32        m_nLineMode;
  MKI_COLORMODEL m_cFillColorMode;
  MKI_ALPHAVALUE m_cFillAlpha;
  MKI_COLORMODEL m_cLineColorMode;
  MKI_ALPHAVALUE m_cLineAlpha;
  double         m_dLineTypeScale;
};

MKI_CLASS_TYPES( PROPERTY )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_NAMED_PROPERTY
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_PROPERTY  m_cData;
  MKI_LINENAME  m_szLineName;
};

MKI_CLASS_TYPES( NAMED_PROPERTY )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_USE_NULL           = 0x00000000,
  MKI_USE_PEN            = 0x00000001,
  MKI_USE_FILLMODE       = 0x00000002,
  MKI_USE_FILLCOLOR      = 0x00000004,
  MKI_USE_LINECOLOR      = 0x00000008,
  MKI_USE_LINEWIDTH      = 0x00000010,
  MKI_USE_LINETYPE       = 0x00000020,
  MKI_USE_LAYER          = 0x00000040,
  MKI_USE_LINEMODECAPS   = 0x00000080,
  MKI_USE_PAGE           = 0x00000100,
  MKI_USE_FILLALPHA      = 0x00000200,
  MKI_USE_LINEALPHA      = 0x00000400,
  MKI_USE_LINETYPESCALE  = 0x00000800,
  MKI_USE_LINEMODEOPT    = 0x00001000,
  MKI_USE_LINEMODEZOOM   = 0x00002000,

  MKI_USE_PROPERTY       = 0x00003ebe,  // All except page, layer, pen
  MKI_USE_PROP_PEN       = 0x00003ebf,  // All except page, layer
  MKI_USE_ALL            = 0x00003fff,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_XPROPERTY
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  CopyFromProperty( MKI_CONST_PROPERTY_REF f_rSource );

  MKI_INTERFACE_MEMBER void
  CopyFromProperty( MKI_CONST_PROPERTY_REF f_rSource, const __int32 f_nFlag );

  MKI_INTERFACE_MEMBER void
  CopyToProperty( MKI_PROPERTY_REF f_rDestin, const __int32 f_nFlag ) const;

  MKI_INTERFACE_MEMBER void
  CopyFromXProperty( const MKI_XPROPERTY& f_rSource, const __int32 f_nFlag );

  MKI_INTERFACE_MEMBER void
  InheritXProperty( const MKI_XPROPERTY& f_rSource );

  MKI_INTERFACE_MEMBER void
  ResolvePenLayer( void );

  MKI_INTERFACE_MEMBER bool
  IsStandard( bool f_fIgnorePage ) const;

  MKI_INTERFACE_MEMBER bool
  IsIdenticalBase( const MKI_XPROPERTY& f_rXProperty ) const;

  MKI_INTERFACE_MEMBER bool
  IsIdenticalFlag( const MKI_XPROPERTY& f_rXProperty ) const;

  MKI_INTERFACE_MEMBER void
  ApplyDefault( __int32 f_nPenDefault, __int32 f_nLayerDefault );

  __forceinline bool
  IsFilled( void ) const
  {
    return( m_nFillMode == MKI_FILLMODE_FILLED ||
            m_nFillMode == MKI_FILLMODE_FILLED_FRAMED );
  };

  __forceinline bool
  IsErased( void ) const
  {
    return( m_nFillMode == MKI_FILLMODE_ERASER ||
            m_nFillMode == MKI_FILLMODE_ERASER_FRAMED );
  };

  __forceinline bool
  IsOpaque( void ) const
  {
    return( m_nFillMode != MKI_FILLMODE_FRAMED );
  };

  __forceinline bool
  IsFramed( void ) const
  {
    return( m_nFillMode != MKI_FILLMODE_FILLED &&
            m_nFillMode != MKI_FILLMODE_ERASER );
  };

  MKI_INTERFACE_MEMBER bool
  IsVisible( void );

  MKI_INTERFACE_MEMBER bool
  IsPrinted( void );

  MKI_INTERFACE_MEMBER bool
  IsLocked( void );

  MKI_INTERFACE_MEMBER bool
  IsSnapped( void );

  MKI_INTERFACE_MEMBER bool
  IsGrayed( void );

  //------------------------------------------------------------------

  __int32        m_nFlag, // Transmission flag (MKI_USE_*)
                 m_nPen,
                 m_nFillMode;
  MKI_COLORVALUE m_cFillColor,
                 m_cLineColor;
  __int32        m_nLineType;
  double         m_dLineWidth;
  __int32        m_nLineMode,
                 m_nLayer,
                 m_nPage;
  MKI_COLORMODEL m_cFillColorMode;
  MKI_ALPHAVALUE m_cFillAlpha;
  MKI_COLORMODEL m_cLineColorMode;
  MKI_ALPHAVALUE m_cLineAlpha;
  double         m_dLineTypeScale;

  __int32 m_nFromPen,    // Internal Data - do not modify!
          m_nFromLayer;  // Internal Data - do not modify!
};

MKI_CLASS_TYPES( XPROPERTY )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_NAMED_XPROPERTY
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_XPROPERTY m_cData;

  __int32 m_nUseFlag,
          m_nUseProp;

  MKI_PENNAME   m_szPenName;
  MKI_LINENAME  m_szLineName;
  MKI_LAYERNAME m_szLayerName;
};

MKI_CLASS_TYPES( NAMED_XPROPERTY )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PATCHPROPERTY
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  CopyFromXProperty( MKI_CONST_XPROPERTY_REF f_rSource );

  MKI_INTERFACE_MEMBER void
  CopyToXProperty( MKI_XPROPERTY_REF f_rDestin ) const;

  __forceinline bool
  IsFilled( void ) const
  {
    return( m_nFillMode == MKI_FILLMODE_FILLED ||
            m_nFillMode == MKI_FILLMODE_FILLED_FRAMED );
  };

  __forceinline bool
  IsErased( void ) const
  {
    return( m_nFillMode == MKI_FILLMODE_ERASER ||
            m_nFillMode == MKI_FILLMODE_ERASER_FRAMED );
  };

  __forceinline bool
  IsOpaque( void ) const
  {
    return( m_nFillMode != MKI_FILLMODE_FRAMED );
  };

  __forceinline bool
  IsFramed( void ) const
  {
    return( m_nFillMode != MKI_FILLMODE_FILLED &&
            m_nFillMode != MKI_FILLMODE_ERASER );
  };

  //------------------------------------------------------------------

  __int32        m_nPen,
                 m_nFillMode,
                 m_nLayer;
  MKI_COLORVALUE m_cFillColor,
                 m_cLineColor;
  MKI_COLORMODEL m_cFillColorMode;
  MKI_ALPHAVALUE m_cFillAlpha;
  MKI_COLORMODEL m_cLineColorMode;
  MKI_ALPHAVALUE m_cLineAlpha;
  double         m_dLineWidth;
};

MKI_CLASS_TYPES( PATCHPROPERTY )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_LAYERPROPERTY
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  Apply( const MKI_LAYERPROPERTY& f_rData );

  //------------------------------------------------------------------

  MKI_PROPERTY m_cProperty;
  __int32      m_nFlag; // Transmission flag (MKI_USE_*)
};

MKI_CLASS_TYPES( LAYERPROPERTY )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_LAYERDATA
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER bool
  IsIdentical( const MKI_LAYERDATA& f_rData ) const;

  MKI_INTERFACE_MEMBER void
  Apply( const MKI_LAYERDATA& f_rData );

  //------------------------------------------------------------------

  MKI_LAYERPROPERTY m_cData[MKI_LAYER_MAX];
};

MKI_CLASS_TYPES( LAYERDATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_MODELMODE_DIMENSION        = 0x00000001,
  MKI_MODELMODE_REFARROW         = 0x00000002,
  MKI_MODELMODE_LINETYPE         = 0x00000004,
  MKI_MODELMODE_HATCH            = 0x00000008,
  MKI_MODELMODE_APPLY_PAGELAYER  = 0x00000010,
  MKI_MODELMODE_APPLY_MAPLAYER   = 0x00000020,
};

enum
{
  MKI_MAPEDITMODE_FOREGROUND  = 0x00000001,
  MKI_MAPEDITMODE_IDLE        = 0x00000002,
  MKI_MAPEDITMODE_DYE         = 0x00000004,
  MKI_MAPEDITMODE_TRANSPARENT = 0x00000008,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MODELDEF
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  double    m_dModelScale;  // Display scale
  __int32   m_nModelMode,   // Display scale application mode
            m_nModelSystem; // Assigned coordinate system

  __int32          m_nRecentPage;   // Recent page
  MKI_WINDOWZOOM   m_cRecentZoom;   // Recent views
  MKI_WINDOWSYSTEM m_cRecentSystem; // Recent coordinate systems
  MKI_HIDELAYERS   m_afHideLayers;  // Hidden layers

  __int32        m_nMapEditMode;  // Mapping editing mode
  MKI_COLORVALUE m_cMapEditColor; // Mapping editing color
};

MKI_CLASS_TYPES( MODELDEF )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PAGETYPE_CUSTOM      = 0,
  MKI_PAGETYPE_DIN_A4      = 1,
  MKI_PAGETYPE_DIN_A3      = 2,
  MKI_PAGETYPE_DIN_A3_2    = 3,
  MKI_PAGETYPE_DIN_A3_1    = 4,
  MKI_PAGETYPE_DIN_A3_0    = 5,
  MKI_PAGETYPE_DIN_A2      = 6,
  MKI_PAGETYPE_DIN_A2_1    = 7,
  MKI_PAGETYPE_DIN_A2_0    = 8,
  MKI_PAGETYPE_DIN_A1      = 9,
  MKI_PAGETYPE_DIN_A1_0    = 10,
  MKI_PAGETYPE_DIN_A0      = 11,
  MKI_PAGETYPE_DIN_2A0     = 12,
  MKI_PAGETYPE_ISO_A4x3    = 13,
  MKI_PAGETYPE_ISO_A4x4    = 14,
  MKI_PAGETYPE_ISO_A4x5    = 15,
  MKI_PAGETYPE_ISO_A4x6    = 16,
  MKI_PAGETYPE_ISO_A3x3    = 17,
  MKI_PAGETYPE_ISO_A3x4    = 18,
  MKI_PAGETYPE_ISO_A2x3    = 19,
  MKI_PAGETYPE_DIN_B5      = 20,
  MKI_PAGETYPE_DIN_B4      = 21,
  MKI_PAGETYPE_DIN_B3      = 22,
  MKI_PAGETYPE_DIN_B2      = 23,
  MKI_PAGETYPE_DIN_B1      = 24,
  MKI_PAGETYPE_DIN_B0      = 25,
  MKI_PAGETYPE_DIN_C5      = 26,
  MKI_PAGETYPE_DIN_C4      = 27,
  MKI_PAGETYPE_DIN_C3      = 28,
  MKI_PAGETYPE_DIN_C2      = 29,
  MKI_PAGETYPE_DIN_C1      = 30,
  MKI_PAGETYPE_DIN_C0      = 31,
  MKI_PAGETYPE_US_HALF     = 32,
  MKI_PAGETYPE_ANSI_A      = 33,
  MKI_PAGETYPE_US_LETTER   = 33,
  MKI_PAGETYPE_ANSI_B      = 34,
  MKI_PAGETYPE_US_TABLOID  = 34,
  MKI_PAGETYPE_ANSI_C      = 35,
  MKI_PAGETYPE_ANSI_D      = 36,
  MKI_PAGETYPE_ANSI_E      = 37,
  MKI_PAGETYPE_US_LEGAL    = 38,
  MKI_PAGETYPE_DIN_A8      = 39,
  MKI_PAGETYPE_DIN_A7      = 40,
  MKI_PAGETYPE_DIN_A6      = 41,
  MKI_PAGETYPE_DIN_A5      = 42,
  MKI_PAGETYPE_DIN_B6      = 43,
  MKI_PAGETYPE_DIN_C7      = 44,
  MKI_PAGETYPE_DIN_C6      = 45,
  MKI_PAGETYPE_DIN_LONG    = 46,
  MKI_PAGETYPE_DIN_A3_PLUS = 47,
  MKI_PAGETYPE_MAX         = 48,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PAGEORIENT_CUSTOM    = 0,
  MKI_PAGEORIENT_PORTRAIT  = 1,
  MKI_PAGEORIENT_LANDSCAPE = 2,
  MKI_PAGEORIENT_MAX       = 3,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PAGEMODE_STANDARD       = 0x00000000,
  MKI_PAGEMODE_FORCE_PAGESIZE = 0x00000001,
  MKI_PAGEMODE_LINE_SCALED    = 0x00000002,
  MKI_PAGEMODE_DISABLED       = 0x00000004,
  MKI_PAGEMODE_TAB_COLOR      = 0x00000008,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PAGEGUIDE_ACTIVE       = 0x00000001,
  MKI_PAGEGUIDE_MARGINLEFT   = 0x00000002,
  MKI_PAGEGUIDE_MARGINTOP    = 0x00000004,
  MKI_PAGEGUIDE_MARGINRIGHT  = 0x00000008,
  MKI_PAGEGUIDE_MARGINBOTTOM = 0x00000010,
  MKI_PAGEGUIDE_COLUMNFRAME  = 0x00000020,
  MKI_PAGEGUIDE_COLUMNCENTER = 0x00000040,
  MKI_PAGEGUIDE_LINES        = 0x00000080,
  MKI_PAGEGUIDE_COLOR        = 0x00000100,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PAGEDEF
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_COMMENTSHORT  m_szPageComment;

  __int32    m_nPageType,   // Page type (MKI_PAGETYPE_*)
             m_nPageOrient, // Page orientation (MKI_PAGEORIENT_*)
             m_nPageOrigin; // Page insert point location (MKI_ORIGIN_*)
  MKI_POINT  m_cPagePos;    // Page insertion point
  MKI_VECTOR m_cPageSize;   // Page size in [mm]
  double     m_dPageRotate, // Page rotation angle
             m_dPageScale;  // Page scaling
  __int32    m_nPageMode,   // Page modes (MKI_PAGEMODE_*)
             m_nPageSystem; // Assigned coordinate system (for rotation)

  MKI_WINDOWZOOM   m_cRecentZoom;   // Recent views
  MKI_WINDOWSYSTEM m_cRecentSystem; // Recent coordinate systems
  MKI_HIDELAYERS   m_afHideLayers;  // Hidden layers

  __int32  m_nGuideMode,    // Guide line visibility (MKI_PAGEGUIDE_*)
           m_nGuideColumns; // Number of columns
  MKI_RECT m_cMargin;       // Page margin in [mm]
  double   m_dColumnSpace,  // Space between adjacent columns
           m_dLineDistance; // Text line distance

  MKI_COLORVALUE m_cGuideColor, // Guide line color
                 m_cTabColor;   // Tab button color
};

MKI_CLASS_TYPES( PAGEDEF )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MULTI_PAGEDEF : public MKI_MULTI_BASE
{

public:

  enum
  {
    USE_COMMENT      = 0x00000001,
    USE_ORIGIN       = 0x00000002,
    USE_SIZE         = 0x00000004,
    USE_ROTATE       = 0x00000008,
    USE_SCALE        = 0x00000010,
    USE_LAYERS       = 0x00000020,
    USE_SYSTEM       = 0x00000040,
    USE_FORCEPAGE    = 0x00000080,
    USE_LINESCALE    = 0x00000100,
    USE_MARGINLEFT   = 0x00000200,
    USE_MARGINTOP    = 0x00000400,
    USE_MARGINRIGHT  = 0x00000800,
    USE_MARGINBOTTOM = 0x00001000,
    USE_COLUMNS      = 0x00002000,
    USE_MARGINCOLUMN = 0x00004000,
    USE_LINEDISTANCE = 0x00008000,
    USE_COLOR        = 0x00010000,
    USE_SHOWTABCOLOR = 0x00020000,
    USE_TABCOLOR     = 0x00040000,
    USE_DISABLED     = 0x00080000,
  };

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  Fill( MKI_CONST_PAGEDEF_REF f_rPageDef );

  MKI_INTERFACE_MEMBER void
  Apply( MKI_PAGEDEF_REF f_rPageDef, bool f_fCopyAll ) const;

  //------------------------------------------------------------------

  MKI_MULTI_BASE m_cGuideBase;

  MKI_PAGEDEF m_cData;
};

MKI_CLASS_TYPES( MULTI_PAGEDEF )


//---------------------------------------------------------------------------------------------------------------------------------------------------
// Basic pen and layer modes
enum
{
  MKI_BASEMODE_NULL            = 0x00000000,
  MKI_BASEMODE_PROPS_IDENTICAL = 0x00000001,
  MKI_BASEMODE_SHOW_COMMENTS   = 0x00000002,
  MKI_BASEMODE_PASSIVE         = 0x00000004,
  MKI_BASEMODE_DEFAULT         = MKI_BASEMODE_SHOW_COMMENTS,

  MKI_BASEMODE_EX_KEEP_SCREEN  = 0x00010000,  // Copy screen properties to output properties
  MKI_BASEMODE_EX_KEEP_OUTPUT  = 0x00020000,  // Copy output properties to screen properties
  MKI_BASEMODE_EX_APPLY_PROPS  = 0x00040000,  // Apply all properties to the current drawing's entities before switching to passive mode
};

// Pen and layer modes
enum
{
  MKI_PENLAYERMODE_DISPLAY = 0x00000001,
  MKI_PENLAYERMODE_OUTPUT  = 0x00000002,
  MKI_PENLAYERMODE_LOCK    = 0x00000004,
  MKI_PENLAYERMODE_IDLE    = 0x00000008,
  MKI_PENLAYERMODE_GRAY    = 0x00000010,
  MKI_PENLAYERMODE_NOLIST  = 0x00000020,
  MKI_PENLAYERMODE_FLAG1   = 0x00000100,
  MKI_PENLAYERMODE_FLAG2   = 0x00000200,
  MKI_PENLAYERMODE_FLAG3   = 0x00000400,
  MKI_PENLAYERMODE_FLAG4   = 0x00000800,

  MKI_PENLAYERMODE_DEFAULT = 0x00000003,
  MKI_PENLAYERMODE_FLAGS   = 0x00000f00,
  MKI_PENLAYERMODE_ALL     = 0x00000f3f,

  MKI_PENLAYERMODE2_DISPLAY = 0x00010000,
  MKI_PENLAYERMODE2_OUTPUT  = 0x00020000,
  MKI_PENLAYERMODE2_LOCK    = 0x00040000,
  MKI_PENLAYERMODE2_IDLE    = 0x00080000,
  MKI_PENLAYERMODE2_GRAY    = 0x00100000,
  MKI_PENLAYERMODE2_NOLIST  = 0x00200000,
  MKI_PENLAYERMODE2_FLAG1   = 0x01000000,
  MKI_PENLAYERMODE2_FLAG2   = 0x02000000,
  MKI_PENLAYERMODE2_FLAG3   = 0x04000000,
  MKI_PENLAYERMODE2_FLAG4   = 0x08000000,

  MKI_PENLAYERMODE2_FLAGS   = 0x0f000000,
  MKI_PENLAYERMODE2_ALL     = 0x0f3f0000,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PENDEF
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_PENNAME      m_szPenName;
  MKI_COMMENTSHORT m_szPenComment;

  MKI_PROPERTY m_cPenIntern,
               m_cPenExtern;

  __int32  m_nPenLayer,
           m_nPenMode;    // Pen modes (MKI_PENLAYERMODE_*)
};

MKI_CLASS_TYPES( PENDEF )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_LAYERDEF
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_LAYERNAME    m_szLayerName;
  MKI_COMMENTSHORT m_szLayerComment;

  MKI_XPROPERTY m_cLayerIntern,
                m_cLayerExtern;

  __int32   m_nLayerMode;   // Layer modes (MKI_PENLAYERMODE_*)
};

MKI_CLASS_TYPES( LAYERDEF )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_HATCHLINE
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER bool
  IsIdentical( const MKI_HATCHLINE& f_rData ) const;

  //------------------------------------------------------------------

  double        m_dDistance;
  MKI_XPROPERTY m_cXProperty;
  bool          m_fUse;
};

MKI_CLASS_TYPES( HATCHLINE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_HATCHLINELIST
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER bool
  IsIdentical( const MKI_HATCHLINELIST& f_rData ) const;

  //------------------------------------------------------------------

  MKI_HATCHLINE m_acData[MKI_HATCHLINE_MAX];
};

MKI_CLASS_TYPES( HATCHLINELIST )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_HATCHDEF
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER bool
  IsIdentical( const MKI_HATCHDEF& f_rData ) const;

  //------------------------------------------------------------------

  MKI_HATCHNAME    m_szHatchName;
  MKI_COMMENTSHORT m_szHatchComment;

  bool              m_fHatchLines1Active;
  double            m_dHatchLines1Rotate;
  MKI_HATCHLINELIST m_cHatchLines1;
  __int32           m_nHatchLineIndex1;

  bool              m_fHatchLines2Active;
  double            m_dHatchLines2Rotate;
  MKI_HATCHLINELIST m_cHatchLines2;
  __int32           m_nHatchLineIndex2;

  bool            m_fHatchBlockActive;
  MKI_LIBRARYNAME m_szHatchLibraryName;
  MKI_BLOCKNAME   m_szHatchBlockName;
  double          m_dHatchBlockRotate,
                  m_dHatchBlockScale,
                  m_dHatchXStep1,
                  m_dHatchXStep2,
                  m_dHatchYStep1,
                  m_dHatchYStep2,
                  m_dHatchLineStep1,
                  m_dHatchLineStep2;

  double m_dHatchRotate,
         m_dHatchOffset1,
         m_dHatchOffset2;

  bool      m_fHatchLocalActive;
  MKI_POINT m_cHatchLocalOrigin;
  double    m_dHatchLocalRotate;
};

MKI_CLASS_TYPES( HATCHDEF )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_COLORDEF
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_COLORNAME  m_szColorName;
  MKI_COLORVALUE m_cColorValue;
  MKI_COLORMODEL m_cColorMode;
  __int32        m_nColorIndex,
                 m_nColorFade;
};

MKI_CLASS_TYPES( COLORDEF )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_VARIABLEDEF
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_VARIABLENAME  m_szName;
  MKI_COMMENTLONG   m_szComment;
  MKI_VARIABLEVALUE m_szValue;
};

MKI_CLASS_TYPES( VARIABLEDEF )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_TEXTSTLYEMODE_KEEP  = 0,
  MKI_TEXTSTLYEMODE_SET   = 1,
  MKI_TEXTSTLYEMODE_CLEAR = 2,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_TEXTSTYLEDEF
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  __forceinline bool
  IsFilled( void ) const
  {
    return( m_nFillMode == MKI_FILLMODE_FILLED ||
            m_nFillMode == MKI_FILLMODE_FILLED_FRAMED );
  };

  __forceinline bool
  IsErased( void ) const
  {
    return( m_nFillMode == MKI_FILLMODE_ERASER ||
            m_nFillMode == MKI_FILLMODE_ERASER_FRAMED );
  };

  __forceinline bool
  IsOpaque( void ) const
  {
    return( m_nFillMode != MKI_FILLMODE_FRAMED );
  };

  __forceinline bool
  IsFramed( void ) const
  {
    return( m_nFillMode != MKI_FILLMODE_FILLED &&
            m_nFillMode != MKI_FILLMODE_ERASER );
  };

  //------------------------------------------------------------------

  __int32 m_nUseProp;

  MKI_COLORVALUE m_cFillColor,  
                 m_cLineColor;
  MKI_COLORMODEL m_cFillColorMode,
                 m_cLineColorMode;

  __int32 m_nFillMode;

  bool        m_fUseFont;
  MKI_FONTDEF m_cFont;

  __int32 m_nModeBold,      // Bold mode (MKI_TEXTSTLYEMODE_*)
          m_nModeItalic,    // Italic mode (MKI_TEXTSTLYEMODE_*)
          m_nModeUnderline, // Underline mode (MKI_TEXTSTLYEMODE_*)
          m_nModeStrikeout; // Strikeout mode (MKI_TEXTSTLYEMODE_*)

  double m_dScale,
         m_dOffset,
         m_dMarginL,
         m_dMarginR;
};

MKI_CLASS_TYPES( TEXTSTYLEDEF )

//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PRINTPDF_FLAG_NONE        = 0x00000000, // No special options
  MKI_PRINTPDF_FLAG_LAYERS      = 0x00000001, // Export layer information
  MKI_PRINTPDF_FLAG_ANONYMIZE   = 0x00000002, // Anonymize layer names
  MKI_PRINTPDF_FLAG_ALLLAYERS   = 0x00000004, // Output all layers (even hidden)
  MKI_PRINTPDF_FLAG_NO_PRINT    = 0x00000010, // No printing allowed
  MKI_PRINTPDF_FLAG_NO_COPY     = 0x00000020, // No copying allowed
  MKI_PRINTPDF_FLAG_NO_MODIFY   = 0x00000040, // No modifying allowed
  MKI_PRINTPDF_FLAG_VIEW        = 0x00000100, // Open the created PDF file using the currently associated application
  MKI_PRINTPDF_FLAG_OVERPRINT   = 0x00000200, // Overprint black text
  MKI_PRINTPDF_FLAG_WARNCOLOR   = 0x00000400, // Report any use of non-CMYK and non-gray colors
  MKI_PRINTPDF_FLAG_CLIPPING    = 0x00000800, // Clip curves and surfaces against the page's frame (time-consuming)
  MKI_PRINTPDF_FLAG_GAMMA       = 0x00001000, // Apply gamma correction
  MKI_PRINTPDF_FLAG_NO_COMPRESS = 0x00002000, // Do not compress object streams
  MKI_PRINTPDF_FLAG_COLORSPACE  = 0x00004000, // Embed RGB color space profile
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PRINTPDF_TYPE_V17   = 0, // Create PDF 1.7 file
  MKI_PRINTPDF_TYPE_PDFA1 = 1, // Create PDF/A-1b compliant file (based on PDF 1.4, no transparency, no layers, no CMYK colors)
  MKI_PRINTPDF_TYPE_PDFX3 = 2, // No longer supported, defaults to MKI_PRINTPDF_TYPE_PDFX4
  MKI_PRINTPDF_TYPE_PDFX1 = 3, // No longer supported, defaults to MKI_PRINTPDF_TYPE_PDFX4
  MKI_PRINTPDF_TYPE_PDFX4 = 4, // Create PDF/X4 compliant file (print production, no layers, only CMYK and grayscale)
  MKI_PRINTPDF_TYPE_V17X8 = 5, // Create PDF 1.7 ext 8 file
  MKI_PRINTPDF_TYPE_V20   = 6, // Create PDF 2.0 file
  MKI_PRINTPDF_TYPE_PDFA2 = 7, // Create PDF/A-2b compliant file (based on PDF 1.7, transparency, layers, no CMYK colors)
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PRINTPDF_PAGE_DEFAULT      = 0, // Page display modes
  MKI_PRINTPDF_PAGE_1PAGE        = 1,
  MKI_PRINTPDF_PAGE_2PAGELEFT    = 2,
  MKI_PRINTPDF_PAGE_2PAGERIGHT   = 3,
  MKI_PRINTPDF_PAGE_1COLUMN      = 4,
  MKI_PRINTPDF_PAGE_2COLUMNLEFT  = 5,
  MKI_PRINTPDF_PAGE_2COLUMNRIGHT = 6,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PRINTPDF_3D_PLAIN = 0, // 3D output modes
  MKI_PRINTPDF_3D_BOTH  = 1,
  MKI_PRINTPDF_3D_ONLY  = 2,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PRINTPDF_U3D_NAMES    = 0x00000001, // U3D export modes
  MKI_PRINTPDF_U3D_OPTIMIZE = 0x00000002,
  MKI_PRINTPDF_U3D_NORMALS  = 0x00000004,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PRINT_PDF
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_VECTOR m_cPageSize;     // Page size in [mm]
  __int32    m_nPageType,     // Page type (MKI_PAGETYPE_*)
             m_nPageOrient,   // Page orientation (MKI_PAGEORIENT_*)
             m_nPageDisplay,  // Default page display mode (MKI_PRINTPDF_PAGE_*)
             m_nOutputType,   // Type of PDF to be output (MKI_PRINTPDF_TYPE_*)
             m_nOutputFlag,   // Output options (MKI_PRINTPDF_FLAG_*)
             m_nOutputMode3D, // 3D output mode (MKI_PRINTPDF_3D_*)
             m_nExportMode3D, // Export mode for U3D export (MKI_PRINTPDF_U3D_*)
             m_nBitmapType,   // Bitmap file type and compression mode (MKI_BITMAPMODE_*)
             m_nGamma,        // Gamma correction value
             m_nUnused;       // Currently unused, reserved for later use

  MKI_FILENAMEW m_szProfileNameX4;  // Color profile for PDF/X4
};

MKI_CLASS_TYPES( PRINT_PDF )

//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PRINTOUTPUT_DIRECT           = 0, // Output directly to the device
  MKI_PRINTOUTPUT_USER_FILE        = 1, // Output to the preset filename
  MKI_PRINTOUTPUT_DRAWING_PATH     = 2, // Output to the drawing's path with the preset filename
  MKI_PRINTOUTPUT_USER_PATH        = 3, // Output to the preset file path
  MKI_PRINTOUTPUT_PDF_USER_FILE    = 4, // Output to a PDF file with the preset filename
  MKI_PRINTOUTPUT_PDF_DRAWING_PATH = 5, // Output to a PDF file in the drawing's path
  MKI_PRINTOUTPUT_PDF_USER_PATH    = 6, // Output to a PDF file in the preset path
  MKI_PRINTOUTPUT_MAX              = 7,
};

enum
{
  MKI_PRINTERTYPE_GDI       = 0,  // Standard Windows GDI printer
  MKI_PRINTERTYPE_CAD6PDF   = 1,  // Full-featured CAD6PDF export
  MKI_PRINTERTYPE_DIRECTPDF = 2,  // DirectPDF export
  MKI_PRINTERTYPE_MAX       = 3,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PRINT_DEVICE
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_STR16W m_szShortName;  // Short name of the printer

  MKI_FILEREFERENCE m_szDeviceName,   // Name of the printer
                    m_szDevicePort,   // Port of the printer
                    m_szDeviceDriver; // Printer driver
  MKI_COMMENTLONG   m_szComment;      // Comment on the printer

  MKI_FILENAMEW m_szOutputPath, // Predefined output path
                m_szOutputFile; // Predefined output file

  __int32 m_nOutputMode,  // Output file mode (MKI_PRINTOUTPUT_*)
          m_nHideFlags,   // Flags that shall mark hidden layers/pens (MKI_PENLAYERMODE_FLAG*)
          m_nType;        // Printer type

  MKI_PRINT_PDF m_cPrintDirectPDF,  // DirectPDF export settings (MKI_PRINTERTYPE_*)
                m_cPrintCAD6PDF;    // CAD6PDF export settings (MKI_PRINTERTYPE_*)
};

MKI_CLASS_TYPES( PRINT_DEVICE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PRINTMODE_SINGLE    = 0, // Print drawing on one sheet
  MKI_PRINTMODE_MULTIPLE  = 1, // Print drawing on multiple sheets
  MKI_PRINTMODE_FIT_SCALE = 2, // Scale drawing to fit on current sheet
  MKI_PRINTMODE_FIT_PAPER = 3, // Find a paper large enough for the drawing
  MKI_PRINTMODE_MAX       = 4,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PRINTOPTION_DRAFT           = 0x00000001, // Draft mode
  MKI_PRINTOPTION_SELECTED        = 0x00000002, // Print selected objects only
  MKI_PRINTOPTION_CENTERED        = 0x00000004, // Center print-out on page(s)
  MKI_PRINTOPTION_EDGES           = 0x00000008, // Print cutting edges
  MKI_PRINTOPTION_COLLATE         = 0x00000010, // Collate copies
  MKI_PRINTOPTION_SCALE_LINES     = 0x00000020, // Scale line widths
  MKI_PRINTOPTION_KNOWN_PAGES     = 0x00000040, // Use predefined paper sizes only
  MKI_PRINTOPTION_SPLIT           = 0x00000080, // Start a new job after the each page
  MKI_PRINTOPTION_RULES           = 0x00000100, // Apply rules after each printing process
  MKI_PRINTOPTION_SERIAL_DATABASE = 0x00000200, // Use database for serial printing
  MKI_PRINTOPTION_SERIAL_SPLIT    = 0x00000400, // Start a new job after the given number of database entries
  MKI_PRINTOPTION_DISABLED        = 0x00000800, // Print disabled pages, too
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PRINTORIENT_PORTRAIT  = 0, // Portrait (height > width)
  MKI_PRINTORIENT_LANDSCAPE = 1, // Landscape (height < width)
  MKI_PRINTORIENT_MAX       = 2,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PRINTRANGE_ALL      = 0, // All pages frames will be printed
  MKI_PRINTRANGE_CURRENT  = 1, // Only the currently active page will be printed
  MKI_PRINTRANGE_INTERVAL = 2, // All pages in the given range will be printed
  MKI_PRINTRANGE_OBJECTS  = 3, // All objects will be printed (regardless of pages)
  MKI_PRINTRANGE_MAX      = 4,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PRINTSERIAL_JOB    = 0, // Add the current job's number to the output filename
  MKI_PRINTSERIAL_RECORD = 1, // Add the first record's number to the output filename
  MKI_PRINTSERIAL_ENTRY  = 2, // Add the first record's entry to the output filename
  MKI_PRINTSERIAL_MAX    = 3,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PRINT_ENTRY
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER bool
  operator ==( const MKI_PRINT_ENTRY& f_rData ) const;

  MKI_INTERFACE_MEMBER bool
  operator !=( const MKI_PRINT_ENTRY& f_rData ) const;

  //------------------------------------------------------------------

  MKI_COMMENTLONG m_szHeaderL,  // Right-aligned header text
                  m_szHeaderC,  // Centered header text
                  m_szHeaderR,  // Left-aligned header text
                  m_szFooterL,  // Right-aligned footer text
                  m_szFooterC,  // Centered footer text
                  m_szFooterR;  // Left-aligned footer text

  double   m_dScale,          // Scaling
           m_dOffset;         // Additional paper size offset
  MKI_RECT m_cBorder;         // Page frames in [mm]

  __int32  m_nCopyNum,        // Number of copies
           m_nOptions,        // Printing options (MKI_PRINTOPTION_*)
           m_nPrintMode,      // Printing mode (MKI_PRINTMODE_*)
           m_nPageOrient,     // Paper orientation (MKI_PRINTORIENT_*)
           m_nPageRangeMode;  // Page range mode (MKI_PRINTRANGE_*)

  MKI_COMMENTLONG m_szPageRangeText; // Page range text (e.g. 1, 6-9, 4-1)

  __int32  m_nSerialMin,      // First database entry to print
           m_nSerialMax,      // Last database entry to print
           m_nSerialPerJob,   // Number of database entries per job
           m_nSerialFileMode; // Indicates which information shall be added to the output filename (MKI_PRINTSERIAL_*)

  MKI_COLUMNNAME m_szSerialEntry; // Name of a database record entry to be added to the output filename
};

MKI_CLASS_TYPES( PRINT_ENTRY )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PRINT_DATA
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_PRINT_DATA( void );

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_PRINT_ENTRY m_acData[MKI_PRINTER_MAX]; // One printer's parameters
  __int32         m_nIndex;                  // Index of currently active printer

  bool            m_fUseSection;  // true: Section printing
  MKI_RECT        m_cSection;     // Section to print in [mm]

  __int32         m_nPolyCount;                             // Number of printing area polygon points
  MKI_POINT       m_acPolyPoints[MKI_PRINTER_POLYGON_MAX];  // Printing area polygon points in [mm]
};

MKI_CLASS_TYPES( PRINT_DATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_BITMAPMODE_BMP          = 0x00000010, // Export in BMP format, uncompressed
  MKI_BITMAPMODE_PNG          = 0x00000020, // Export in PNG format, compressed
  MKI_BITMAPMODE_PNG_NOALPHA  = 0x00000021, // Export in PNG format, compressed, no alpha channel
  MKI_BITMAPMODE_JPEG_HIGHEST = 0x00000030, // Export in JPEG format, highest quality/lowest compression
  MKI_BITMAPMODE_JPEG_HIGH    = 0x00000031, // Export in JPEG format, high quality/low compression
  MKI_BITMAPMODE_JPEG_MEDIUM  = 0x00000032, // Export in JPEG format, medium quality/medium compression
  MKI_BITMAPMODE_JPEG_LOW     = 0x00000033, // Export in JPEG format, low quality/good compression
  MKI_BITMAPMODE_JPEG_LOWEST  = 0x00000034, // Export in JPEG format, lowest quality/best compression
  MKI_BITMAPMODE_TIFF         = 0x00000040, // Export in TIFF format, uncompressed
  MKI_BITMAPMODE_TIFF_LZW     = 0x00000041, // Export in TIFF format, LZW compressed
  MKI_BITMAPMODE_TIFF_CCITT3  = 0x00000042, // Export in TIFF format, CCITT Fax 3 compressed
  MKI_BITMAPMODE_TIFF_CCITT4  = 0x00000043, // Export in TIFF format, CCITT Fax 4 compressed
  MKI_BITMAPMODE_TIFF_ZIP     = 0x00000044, // Export in TIFF format, ZIP compressed
  MKI_BITMAPMODE_GIF          = 0x00000050, // Export in GIF format, uncompressed
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_BITMAPSIZE_RESOLUTION   = 0, // Based on resolution
  MKI_BITMAPSIZE_PROPORTIONAL = 1, // Based on pixel size, proportional
  MKI_BITMAPSIZE_ARBITRARY    = 2, // Based on pixel size, arbitrary
  MKI_BITMAPSIZE_MAX          = 3,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_BITMAPRESOLUTION_INCH = 0, // Bitmap resolution is given in pixels per inch
  MKI_BITMAPRESOLUTION_MM   = 1, // Bitmap resolution is given in pixels per mm
  MKI_BITMAPRESOLUTION_MAX  = 2,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_BITMAPANTIALIASING_NONE    = 0, // No antialiasing
  MKI_BITMAPANTIALIASING_2_TIMES = 1, // Antialiasing by 2 times oversampling 
  MKI_BITMAPANTIALIASING_3_TIMES = 2, // Antialiasing by 3 times oversampling 
  MKI_BITMAPANTIALIASING_4_TIMES = 3, // Antialiasing by 4 times oversampling 
  MKI_BITMAPANTIALIASING_5_TIMES = 4, // Antialiasing by 5 times oversampling 
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_BITMAPOPTION_NONE        = 0x0000, // No option
  MKI_BITMAPOPTION_NOSMOOTH    = 0x0001, // Objects and texts are drawn without antialiasing
  MKI_BITMAPOPTION_TRANSPARENT = 0x0002, // The bitmap gets a transparent background
  MKI_BITMAPOPTION_GAMMA       = 0x0004, // Adapt colors (gamma)
  MKI_BITMAPOPTION_PROFILE     = 0x0008, // Add RGB color profile to bitmap
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BITMAP_EXPORT
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_BITMAP_EXPORT( void );

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  __int32       m_nColorBits,      // Number of color bits (1,8,24,32)
                m_nFileType,       // Type of bitmap to save (MKI_BITMAPMODE_*)
                m_nPixelMode;      // Size determination mode (MKI_BITMAPSIZE_*)
  MKI_INTVECTOR m_cPixels;         // Predefined size of bitmap
  __int32       m_nResolutionMode; // Resolution mode (MKI_BITMAPRESOLUTION_*)
  MKI_VECTOR    m_cResolution;     // Predefined resolution of the bitmap
  __int32       m_nAntialiase,     // Antialiasing mode for 24-bit bitmaps (MKI_BITMAPANTIALIASING_*)
                m_nOptions,        // Export options (MKI_BITMAPOPTION_*)
                m_nGamma;          // Gamma correction (-100..100)
};

MKI_CLASS_TYPES( BITMAP_EXPORT )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_COLLECTION_DATA
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_COLLECTION_DATA( void );

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_COLLECTIONNAME m_szCollectionName;  // Collection name for options
  MKI_ITEMNAME       m_szItemName;        // Item name for options
};

MKI_CLASS_TYPES( COLLECTION_DATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_CREATEBLOCK_DATA
{
  public:

  enum
  {
    MODE_SHOW_ATTRIBUTES  = 0x00000001,
    MODE_SHOW_PREVIEW     = 0x00000002,
    MODE_SHOW_NOVARIABLES = 0x00000004,

    MODE_REPLACE_OBJECTS     = 0x00000010,
    MODE_STANDARD_ATTRIBUTES = 0x00000020,
    MODE_EDIT_ATTRIBUTES     = 0x00000040,
    MODE_KEEP_ATTRIBUTES     = 0x00000080,
    MODE_RESOLVE_PROPERTIES  = 0x00000100,
    MODE_FIX_PROPERTIES      = 0x00000200,
    MODE_EDIT_PROPERTIES     = 0x00000400,
    MODE_EDIT_BLOCK_ONLY     = 0x00000800,
  };

  enum
  {
    DIALOGMODE_NONE           = 0x00000000,
    DIALOGMODE_REPLACE        = 0x00000001,
    DIALOGMODE_STANDARDATTRIB = 0x00000002,
    DIALOGMODE_EDITATTRIB     = 0x00000004,
    DIALOGMODE_RESOLVEPROP    = 0x00000008,
    DIALOGMODE_EDITPROP       = 0x00000010,
    DIALOGMODE_COMMENT        = 0x00000020,
    DIALOGMODE_FIXPROP        = 0x00000040,
    DIALOGMODE_EDITBLOCKONLY  = 0x00000080,
    DIALOGMODE_ALL            = 0xffffffff,
  };

  MKI_INTERFACE_MEMBER
  MKI_CREATEBLOCK_DATA( void );

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  InitAnonymous( void );
  
  //------------------------------------------------------------------

  MKI_LIBRARYNAME m_szLibraryName;  // Current library name
  MKI_BLOCKNAME   m_szBlockName;    // Current block name
  MKI_COMMENTLONG m_szComment;      // Current comment

  __int32 m_nMode;
};

MKI_CLASS_TYPES( CREATEBLOCK_DATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARTSLIST_ATTRIB
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_PARTSLIST_ATTRIB( void );

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  __int32           m_nCount;                             // Number of names in the list
  MKI_ATTRIBUTENAME m_acData[MKI_ATTRIBS_PER_PARTSLIST];  // List of selected attribute names
};

MKI_CLASS_TYPES( PARTSLIST_ATTRIB )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PARTSLIST_FILE_ANSI = 0,
  MKI_PARTSLIST_FILE_UTF8 = 1,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PARTSLIST_USE_ALL       = 0,
  MKI_PARTSLIST_USE_SELECTION = 1,
  MKI_PARTSLIST_USE_LIBRARY   = 2,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PARTSLIST_PARSE_TOP_ONLY    = 0,
  MKI_PARTSLIST_PARSE_ALL_LEVELS  = 1,
  MKI_PARTSLIST_PARSE_BOTTOM_ONLY = 2,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PARTSLIST_ARRANGE_INST_SINGLE   = 0,
  MKI_PARTSLIST_ARRANGE_INST_GROUPED  = 1,
  MKI_PARTSLIST_ARRANGE_POSNO_SINGLE  = 2,
  MKI_PARTSLIST_ARRANGE_POSNO_GROUPED = 3,
  MKI_PARTSLIST_ARRANGE_ALL_SINGLE    = 4,
  MKI_PARTSLIST_ARRANGE_TOTAL         = 5,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARTSLIST_DATA
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_PARTSLIST_DATA( void );

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_LIBRARYNAME m_szLibraryName;  // Library name
  MKI_FILENAMEW   m_szFileName;     // Output file name
  MKI_STR32W      m_szNAText,       // Replacement text
                  m_szNANumber;     // Replacement number

  __int32 m_nFileType,        // MKI_PARTSLIST_FILE_*
          m_nUseMode,         // MKI_PARTSLIST_USE_*
          m_nParseMode,       // MKI_PARTSLIST_PARSE_*
          m_nArrangeMode,     // MKI_PARTSLIST_ARRANGE_*
          m_nRanges,          // Bit-coded position number ranges
          m_nTextDelimiter,   // Text delimiterd
          m_nRecordSeparator, // Data record separator
          m_nFieldSeparator;  // Field separator
  bool    m_fHeadline;        // Output column headers

  MKI_PARTSLIST_ATTRIB m_acSelection[MKI_PARTSLIST_ARRANGE_TOTAL]; // Chosen attributes for each groupmode
};

MKI_CLASS_TYPES( PARTSLIST_DATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_TASKMODE_NULL          = 0x00000000, // None
  // Section 1: Layers
  MKI_TASKMODE_LAYER_MODE    = 0x00000001, // Save layer modes
  MKI_TASKMODE_LAYER_INTERN  = 0x00000002, // Save internal layer properties
  MKI_TASKMODE_LAYER_EXTERN  = 0x00000004, // Save external layer properties
  MKI_TASKMODE_LAYER_DEFAULT = 0x00000008, // Save layer default
  MKI_TASKMODE_LAYER_MASK    = 0x0000000f, // All layer modes
  // Section 2: Pens
  MKI_TASKMODE_PEN_MODE      = 0x00000010, // Save pen modes
  MKI_TASKMODE_PEN_INTERN    = 0x00000020, // Save internal pen properties
  MKI_TASKMODE_PEN_EXTERN    = 0x00000040, // Save external pen properties
  MKI_TASKMODE_PEN_DEFAULT   = 0x00000080, // Save pen default
  MKI_TASKMODE_PEN_MASK      = 0x000000f0, // All pen modes
  // Section 3: Active elements
  MKI_TASKMODE_ACTIVE_LAYER  = 0x00000100, // Save active layer
  MKI_TASKMODE_ACTIVE_PEN    = 0x00000200, // Save active pen
  MKI_TASKMODE_ACTIVE_PAGE   = 0x00000400, // Save active page
  MKI_TASKMODE_ACTIVE_SYSTEM = 0x00000800, // Save active coordinate system
  MKI_TASKMODE_ACTIVE_HATCH  = 0x00001000, // Save active hatching
  MKI_TASKMODE_ACTIVE_MASK   = 0x00001f00, // All active modes
  // Section 4: Display settings
  MKI_TASKMODE_DISPLAY_LINE  = 0x00010000, // Save line display mode
  MKI_TASKMODE_DISPLAY_POINT = 0x00020000, // Save point display mode
  MKI_TASKMODE_DISPLAY_MASK  = 0x00030000, // All display modes
  // Section 5: Output settings
  MKI_TASKMODE_OUTPUT_LAYER  = 0x00100000, // Save layer output mode
  MKI_TASKMODE_OUTPUT_PAGE   = 0x00200000, // Save page output mode
  MKI_TASKMODE_OUTPUT_COLOR  = 0x00400000, // Save color output mode
  MKI_TASKMODE_OUTPUT_MASK   = 0x00700000, // All output modes
  // Section 6: Export settings
  MKI_TASKMODE_EXPORT_PDF    = 0x01000000, // Save PDF export parameters (MKI_PRINT_PDF)
  MKI_TASKMODE_EXPORT_BITMAP = 0x02000000, // Save bitmap export parameters (MKI_BITMAP_EXPORT)
  MKI_TASKMODE_EXPORT_BLOCK  = 0x04000000, // Create block parameters (MKI_CREATEBLOCK_DATA)
  MKI_TASKMODE_EXPORT_PRINT  = 0x08000000, // Print parameters (MKI_PRINT_ENTRY)
  MKI_TASKMODE_EXPORT_PLUGIN = 0x10000000, // Export via plug-in
  MKI_TASKMODE_EXPORT_PARTS  = 0x40000000, // Create parts list (MKI_PARTSLIST_DATA)
  MKI_TASKMODE_EXPORT_MASK   = 0x5f000000, // All export modes

  MKI_TASKMODE_FLAT_FILENAME = 0x20000000, // Resolve variables and statements in filename
  MKI_TASKMODE_APPLY_FILTER  = 0x00008000, // Apply given filter during creation / export

  MKI_TASKMODE_ALL           = 0x7f739fff, // All
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_TASKDEF
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_TASKDEF( void );

  MKI_INTERFACE_MEMBER 
  ~MKI_TASKDEF();

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  Free( void );

  MKI_INTERFACE_MEMBER bool
  Copy( const MKI_TASKDEF& f_rClass );

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER bool
  HasFrame( void ) const;

  MKI_INTERFACE_MEMBER bool
  HasSize( void ) const;

  //------------------------------------------------------------------

  __int32           m_nMode;          // Elements to be used (MKI_TASKMODE_*)

  MKI_TASKNAME      m_szName;         // Name of the task
  MKI_COMMENTSHORT  m_szComment;      // Comment

  MKI_FILENAMEW     m_szFileName;     // Filename  for export uses (printing, PDF file, bitmap export)
  MKI_RECT          m_cRect,          // Rectangle for export uses (printing, PDF file, bitmap export, create block)
                    m_cSize;          // Optional size information for export uses (create frame block)

  __int32           m_nLayerCount,    // Number of layers in Name/Intern/Extern/Mode (if not nullptr)
                    m_nLayerMode;     // Layer modes to be used (MKI_PENLAYERMODE_*)
  MKI_LAYERNAME*    m_pszLayerName;   // List of names
  __int32*          m_pnLayerMode;    // List of modes (MKI_PENLAYERMODE_*)
  MKI_XPROPERTY_PTR m_pcLayerIntern,  // List of screen properties
                    m_pcLayerExtern;  // List of output properties
  __int32           m_anLayerDefault[MKI_DEFAULT_MAX];  // Default assignments

  __int32           m_nPenCount,      // Number of pens in Name/Intern/Extern/Mode (if not nullptr)
                    m_nPenMode;       // Pen modes to be used (MKI_PENLAYERMODE_*)
  MKI_PENNAME*      m_pszPenName;     // List of names
  __int32*          m_pnPenMode;      // List of modes (MKI_PENLAYERMODE_*)
  MKI_PROPERTY_PTR  m_pcPenIntern,    // List of screen properties
                    m_pcPenExtern;    // List of output properties
  __int32           m_anPenDefault[MKI_DEFAULT_MAX];    // Default assignments

  __int32 m_nActiveLayer,       // Currently active layer
          m_nActivePen,         // Currently active pen
          m_nActivePage,        // Currently active page
          m_nActiveHatch,       // Currently active hatching
          m_nActiveViewSystem,  // Currently active coordinate systems
          m_anActiveDrawSystem[MKI_WINDOW_MAX]; 

  __int32 m_nDisplayLine,  // Line pattern and width display mode (MKI_LINEDISPLAY_*)
          m_nDisplayPoint; // Point display mode (MKI_POINTDISPLAY_*)

  __int32 m_nOutputLayer,  // Layer output mode (MKI_PAGELAYERS_*)
          m_nOutputPage,   // Page output mode (MKI_PAGEOUTPUT_*)
          m_nOutputColor;  // Color output mode (MKI_COLOROUTPUT_*)

  MKI_FILEREFERENCE     m_szExportName;     // Printing device name
  MKI_PRINT_ENTRY       m_cExportPrint;     // Printer settings
  MKI_PRINT_PDF         m_cExportDirectPDF, // DirectPDF export settings
                        m_cExportCAD6PDF;   // CAD6PDF export settings
  __int32               m_nExportType,      // PDF printer type
                        m_nExportHideFlags; // Flags to be hidden
  MKI_BITMAP_EXPORT     m_cExportBitmap;    // Bitmap export settings
  MKI_CREATEBLOCK_DATA  m_cExportBlock;     // Create block settings
  MKI_COLLECTION_DATA   m_cExportPlugIn;    // Plug-In export settings
  MKI_PARTSLIST_DATA    m_cExportPartsList; // Create parts list settings
  MKI_POINT             m_cExportPoint;     // Create block insert point
  MKI_COLLECTION_DATA   m_cFilter;          // Filter to be applied

  __int32   m_nPolyCount;                             // Number of polygon points
  MKI_POINT m_acPolyPoints[MKI_PRINTER_POLYGON_MAX];  // List of polygon points

  private:

  // Disable copy constructor
  MKI_TASKDEF( const MKI_TASKDEF& f_rClass ) = delete;

  // Disable assignment operator
  MKI_TASKDEF&
  operator =( const MKI_TASKDEF& f_rClass ) = delete;

};

MKI_CLASS_TYPES( TASKDEF )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_FILE_HEADER
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  __int64 m_nCreated,
          m_nModified,
          m_nExpired;

  MKI_COMMENTSHORT m_szTitle,
                   m_szTheme,
                   m_szCreator,
                   m_szModifier;
  MKI_COMMENTLONG  m_szComment;
};

MKI_CLASS_TYPES( FILE_HEADER )

//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_TOKEN_DATA
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  char m_szNewline[4],
       m_chComma,
       m_chSemi,
       m_chColor,
       m_chEscape,
       m_chString,
       m_chKeyword,
       m_chComment;
};

MKI_CLASS_TYPES( TOKEN_DATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_MARKFORM_CROSS_BOX    = 0,
  MKI_MARKFORM_CROSS_CIRCLE = 1,
  MKI_MARKFORM_BOX          = 2,
  MKI_MARKFORM_CIRCLE       = 3,
  MKI_MARKFORM_DIAGONAL     = 4,
  MKI_MARKFORM_ORTHOGONAL   = 5,
  MKI_MARKFORM_LINE         = 6,
  MKI_MARKFORM_POINT        = 7,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_ARROWMODE_INSIDE  = 0,
  MKI_ARROWMODE_OUTSIDE = 1,
  MKI_ARROWMODE_AUTO    = 2,
  MKI_ARROWMODE_MAX     = 3,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_ARROWFORM_NONE          = 0,
  MKI_ARROWFORM_TRIANGLE_FILL = 1,
  MKI_ARROWFORM_TRIANGLE      = 2,
  MKI_ARROWFORM_TRIANGLE_OPEN = 3,
  MKI_ARROWFORM_DIAGONAL      = 4,
  MKI_ARROWFORM_CIRCLE_FILL   = 5,
  MKI_ARROWFORM_CIRCLE        = 6,
  MKI_ARROWFORM_CIRCLE_LARGE  = 7,
  MKI_ARROWFORM_MAX           = 8,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_DIMLINE
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER bool
  IsIdentical( const MKI_DIMLINE& f_rData ) const;

  //------------------------------------------------------------------

  __int32 m_nArrowStartForm,  // Start arrow form (MKI_ARROWFORM_*)
          m_nArrowStartMode,  // Start arrow mode (MKI_ARROWMODE_*)
          m_nArrowEndForm,    // End arrow form (MKI_ARROWFORM_*)
          m_nArrowEndMode;    // End arrow mode (MKI_ARROWMODE_*)
};

MKI_CLASS_TYPES( DIMLINE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_NUMROTATE_NONE     = 0x00000000,
  MKI_NUMROTATE_PARALLEL = 0x00000001,
  MKI_NUMROTATE_DIN      = 0x00000002,
  MKI_NUMROTATE_180      = 0x00000004,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_NUMREFRESH_NONE       = 0x00000000,
  MKI_NUMREFRESH_AUTOMATIC  = 0x00000001,
  MKI_NUMREFRESH_DOUBLE     = 0x00000002,
  MKI_NUMREFRESH_BACKGROUND = 0x00000004,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_DIMTYPE_STANDARD  = 0,
  MKI_DIMTYPE_ARBITRARY = 1,
  MKI_DIMTYPE_HALF      = 2,
  MKI_DIMTYPE_MAX       = 3,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_DIMDISTANCE_ARBITRARY = 0,
  MKI_DIMDISTANCE_FIXED     = 1,
  MKI_DIMDISTANCE_MULTIPLE  = 2,
  MKI_DIMDISTANCE_MAX       = 3,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_DIMLINE_PARALLEL   = 0,
  MKI_DIMLINE_ARBITRARY  = 1,
  MKI_DIMLINE_HORIZONTAL = 2,
  MKI_DIMLINE_VERTICAL   = 3,
  MKI_DIMLINE_AUTOMATIC  = 4,
  MKI_DIMLINE_MAX        = 5,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_DIMFLAG_NONE         = 0x00000000,
  MKI_DIMFLAG_RADIUSEXTEND = 0x00000001,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_DIMSMALL
{
  public:

  MKI_INTERFACE_MEMBER void
  InitParam( bool f_fIsAngle );

  MKI_INTERFACE_MEMBER void
  CopyParam( const MKI_DIMPARAM& f_rData, bool f_fIsAngle );

  MKI_INTERFACE_MEMBER void
  InitSmall( bool f_fUseGlobal, bool f_fIsAngle );

  MKI_INTERFACE_MEMBER bool
  IsIdenticalSmall( const MKI_DIMSMALL& f_rData ) const;

  //------------------------------------------------------------------

  MKI_FONTDEF   m_cTextFont;
  MKI_XPROPERTY m_cTextXProperty;
  MKI_UNIQUE_ID m_cLinkedID;

  double  m_dTextSize1,
          m_dTextSize2,
          m_dCharDistance,
          m_dTabDistance;
  __int32 m_nTextMode,      // Horizontal text alignment (MKI_TEXTMODE_*)
          m_nTextVertical,  // Vertical text alignment (MKI_TEXTVERTICAL_*)
          m_nTextOrder,     // Text running order (MKI_TEXTORDER_*)
          m_nNumMode,       // Number display mode (MKI_FRACTION_*)
          m_nNumPrecision,  // Number of fractional digits
          m_nNumRefresh,    // Dimension text updating mode (MKI_NUMREFRESH_*)
          m_nNumRotate,     // Dimension text rotation mode (MKI_NUMROTATE_*)
          m_nSystem,        // Index of coordinate system
          m_nDimFlag,       // Dimension modes (MKI_DIMFLAG_*)
          m_nUnused;        // Internal Data - do not modify!
};

MKI_CLASS_TYPES( DIMSMALL )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_DIMPARAM : public MKI_DIMSMALL
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  Init( const MKI_DIMSMALL& f_rDimSmall, bool f_fIsAngle );

  //------------------------------------------------------------------

  __int32 m_nAngleMode,      // Number display mode for angles (MKI_FRACTION_*)
          m_nAnglePrecision; // Fractional digits for angles
};

MKI_CLASS_TYPES( DIMPARAM )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_NUMPLACING_NONE   = 0,
  MKI_NUMPLACING_TIGHT  = 1,
  MKI_NUMPLACING_INSIDE = 2,
  MKI_NUMPLACING_START  = 3,
  MKI_NUMPLACING_END    = 4,
  MKI_NUMPLACING_MAX    = 5,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_DIMLARGE : public MKI_DIMSMALL
{
  public:

  MKI_INTERFACE_MEMBER void
  InitLarge( bool f_fUseGlobal, bool f_fIsAngle );

  MKI_INTERFACE_MEMBER bool
  IsIdenticalLarge( const MKI_DIMLARGE& f_rData ) const;

  //------------------------------------------------------------------

  bool    m_fNumCentered;
  __int32 m_nNumPlacing,      // Placing of dimension text (MKI_NUMPLACING_*)
          m_nArrowStartForm,  // Start arrow form (MKI_ARROWFORM_*)
          m_nArrowStartMode,  // Start arrow mode (MKI_ARROWMODE_*)
          m_nArrowEndForm,    // End arrow form (MKI_ARROWFORM_*)
          m_nArrowEndMode;    // End arrow mode (MKI_ARROWMODE_*)
  bool    m_fExtStartDisplay,
          m_fExtEndDisplay,
          m_fLineDisplay;
  __int32 m_nLineOrientation, // Dimension line orientation (MKI_DIMLINE_*)
          m_nLineDimType,     // Dimension line type (MKI_DIMTYPE_*)
          m_nLineDistMode;    // Dimension line distance mode (MKI_DIMDISTANCE_*)
  double  m_dLineDistance,
          m_dLineStartOffset,
          m_dLineEndOffset;
};

MKI_CLASS_TYPES( DIMLARGE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_DIMTEXTSMALL
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER bool
  AppendDimensionText( MKI_BLOCK_ENUM& f_rData ) const;

  MKI_INTERFACE_MEMBER bool
  AppendFullDimension( MKI_BLOCK_ENUM& f_rData ) const;

  //------------------------------------------------------------------

  MKI_STRSHORTW m_szPreText,
                m_szDimension,
                m_szPostText,
                m_szTolerance1,
                m_szTolerance2;

  bool  m_fUpdate;
};

MKI_CLASS_TYPES( DIMTEXTSMALL )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_DIMTEXTLARGE
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER bool
  AppendFullDimension( MKI_BLOCK_ENUM& f_rData ) const;

  //------------------------------------------------------------------

  MKI_STRSHORTW m_szPreText,
                m_szDimension1,
                m_szMiddleText,
                m_szDimension2,
                m_szPostText,
                m_szTolerance1,
                m_szTolerance2;

  bool  m_fUpdate;
};

MKI_CLASS_TYPES( DIMTEXTLARGE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_TEXTMODE_LEFT   = 0,
  MKI_TEXTMODE_CENTER = 1,
  MKI_TEXTMODE_RIGHT  = 2,
  MKI_TEXTMODE_BLOCK  = 3,
  MKI_TEXTMODE_MAX    = 4,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_TEXTVERTICAL_FRAMETOP    = 0,
  MKI_TEXTVERTICAL_FRAMECENTER = 1,
  MKI_TEXTVERTICAL_FRAMEBOTTOM = 2,
  MKI_TEXTVERTICAL_LINETOP     = 3,
  MKI_TEXTVERTICAL_LINECENTER  = 4,
  MKI_TEXTVERTICAL_LINEBOTTOM  = 5,
  MKI_TEXTVERTICAL_MAX         = 6,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_TEXTORDER_LATIN  = 0,
  MKI_TEXTORDER_ARABIC = 1,
  MKI_TEXTORDER_KANJI  = 2,
  MKI_TEXTORDER_MAX    = 3,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_TEXTSTANDARD
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER bool
  IsIdentical( const MKI_TEXTSTANDARD& f_rData ) const;

  //------------------------------------------------------------------

  MKI_FONTDEF   m_cTextFont;
  MKI_XPROPERTY m_cTextXProperty;
  MKI_UNIQUE_ID m_cLinkedID;
  MKI_MATRIX    m_cTextMatrix;

  double  m_dCharDistance,
          m_dTabDistance,
          m_dLineDistance;
  __int32 m_nTextMode,      // Horizontal text alignment (MKI_TEXTMODE_*)
          m_nTextVertical,  // Vertical text alignment (MKI_TEXTVERTICAL_*)
          m_nTextOrder,     // Text running order (MKI_TEXTORDER_*)
          m_nDisplayMode,   // General display mode (MKI_DISPLAYMODE_*)
          m_nSystem,        // Coordinate system
          m_anUnused;       // Internal Data - do not modify!
};

MKI_CLASS_TYPES( TEXTSTANDARD )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_TEXTFRAME
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER bool
  IsIdentical( const MKI_TEXTFRAME& f_rData ) const;

  //------------------------------------------------------------------

  MKI_FONTDEF   m_cTextFont;
  MKI_XPROPERTY m_cTextXProperty;
  MKI_UNIQUE_ID m_cLinkedID;

  double  m_dTextSize,
          m_dCharDistance,
          m_dTabDistance,
          m_dLineDistance;
  __int32 m_nTextMode,      // Horizontal text alignment (MKI_TEXTMODE_*)
          m_nTextVertical,  // Vertical text alignment (MKI_TEXTVERTICAL_*)
          m_nTextOrder,     // Text running order (MKI_TEXTORDER_*)
          m_nSystem,        // Coordinate system
          m_anUnused[2];    // Internal Data - do not modify!
};

MKI_CLASS_TYPES( TEXTFRAME )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_REFARROW_POLYLINE   = 0,
  MKI_REFARROW_LINE       = 1,
  MKI_REFARROW_ARROW_45   = 2,
  MKI_REFARROW_TEXT_45    = 3,
  MKI_REFARROW_ARROW_90   = 4,
  MKI_REFARROW_TEXT_90    = 5,
  MKI_REFARROW_HORIZONTAL = 6,
  MKI_REFARROW_VERTICAL   = 7,
  MKI_REFARROW_NONE       = 8,
  MKI_REFARROW_MAX        = 9,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_REFFRAME_RECTANGLE = 0,
  MKI_REFFRAME_RHOMB     = 1,
  MKI_REFFRAME_CIRCLE    = 2,
  MKI_REFFRAME_ELLIPSE   = 3,
  MKI_REFFRAME_ROUNDRECT = 4,
  MKI_REFFRAME_NONE      = 5,
  MKI_REFFRAME_HALO      = 6,
  MKI_REFFRAME_MAX       = 7,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_TEXTREFERENCE
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER bool
  IsIdentical( const MKI_TEXTREFERENCE& f_rData ) const;

  //------------------------------------------------------------------

  __int32 m_nArrowForm,   // Arrow form (MKI_ARROWFORM_*)
          m_nArrowMode,   // Reference arrow (MKI_REFARROW_*)
          m_nFrameForm;   // Reference frame (MKI_REFFRAME_*)
  double  m_dFrameOffset; // Offset between text and reference frame in [mm]
};

MKI_CLASS_TYPES( TEXTREFERENCE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_COMMENT_SYMBOL_PIN    = 0,
  MKI_COMMENT_SYMBOL_BUBBLE = 1,
  MKI_COMMENT_SYMBOL_ARROW  = 2,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_COMMENT_PROGRESS_NEW      = 0,
  MKI_COMMENT_PROGRESS_WORKING  = 1,
  MKI_COMMENT_PROGRESS_SOLVED   = 2,
  MKI_COMMENT_PROGRESS_CRITICAL = 3,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_COMMENTDATA
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER bool
  IsIdentical( const MKI_COMMENTDATA& f_rData ) const;

  //------------------------------------------------------------------

  MKI_COMMENTSHORT m_szTopic; // Topic text

  __int32 m_nSymbol,    // Symbol   (MKI_COMMENT_SYMBOL_*)
          m_nProgress;  // Progress (MKI_COMMENT_PROGRESS_*)
};

MKI_CLASS_TYPES( COMMENTDATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_BITMAPDISPLAY_DEFAULT  = 0x00000000, // Standard display
  MKI_BITMAPDISPLAY_MASK     = 0x00000001, // Use transparent color
  MKI_BITMAPDISPLAY_NOSMOOTH = 0x00000002, // Disable smoothing for transparent bitmaps - no longer supported!
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BITMAP_EFFECT
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  __int32 m_nGamma,
          m_nBrightness,
          m_nContrast,
          m_nIntensity,
          m_nDisplayMode; // Bitmap display mode (MKI_BITMAPDISPLAY_*)

  MKI_COLORVALUE m_cMaskColor;
};

MKI_CLASS_TYPES( BITMAP_EFFECT )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_BITMAPFIT_NONE    = 0x00000000,
  MKI_BITMAPFIT_INSIDE  = 0x00000001,
  MKI_BITMAPFIT_OUTSIDE = 0x00000002,
  MKI_BITMAPFIT_COVER   = 0x00000003,
};

enum
{
  MKI_BITMAPALIGN_BL = 0x00000000,
  MKI_BITMAPALIGN_BC = 0x00000001,
  MKI_BITMAPALIGN_BR = 0x00000002,
  MKI_BITMAPALIGN_CL = 0x00000003,
  MKI_BITMAPALIGN_CC = 0x00000004,
  MKI_BITMAPALIGN_CR = 0x00000005,
  MKI_BITMAPALIGN_TL = 0x00000006,
  MKI_BITMAPALIGN_TC = 0x00000007,
  MKI_BITMAPALIGN_TR = 0x00000008,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BITMAPREF
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER bool
  IsIdentical( const MKI_BITMAPREF& f_rData ) const;

  MKI_INTERFACE_MEMBER bool
  UsesFitting( void ) const;

  //------------------------------------------------------------------

  MKI_FILEREFERENCE m_szBitmapName;
  MKI_MATRIX        m_cDisplayMatrix;
  MKI_BITMAP_EFFECT m_cEffect;
  __int32           m_nFitMode,
                    m_nAlignMode;
  MKI_VECTOR        m_cFitSize;

  __int32 m_nListIndex; // Internal Data - do not modify!
};

MKI_CLASS_TYPES( BITMAPREF )


//---------------------------------------------------------------------------------------------------------------------------------------------------
typedef __int32 MKI_OFFSET;

__forceinline static MKI_OFFSET
mki_db_alignsize( MKI_OFFSET f_nSize )
{
  return( ( f_nSize + 7 ) & (~7) );
};

__forceinline static MKI_OFFSET
mki_db_textsize( LPCWSTR f_pszText )
{
  return( ( ( lstrlenW( f_pszText ) * sizeof( wchar_t ) ) + 8 ) & (~7) );
};

//---------------------------------------------------------------------------------------------------------------------------------------------------
#define MKI_DB_SIZE_POINT    ( mki_db_alignsize( sizeof( MKI_BLOCK_POINT ) + sizeof( MKI_POINT ) ) )
#define MKI_DB_SIZE_CONSTANT ( mki_db_alignsize( sizeof( MKI_BLOCK_DOUBLE ) + sizeof( double ) ) )
#define MKI_DB_SIZE_ARC      ( mki_db_alignsize( sizeof( MKI_BLOCK_DOUBLE ) + sizeof( double ) ) )
#define MKI_DB_SIZE_CURVE    ( mki_db_alignsize( sizeof( MKI_BLOCK_DOUBLE ) + sizeof( double ) + sizeof( double ) ) )
#define MKI_DB_SIZE_TEXT     ( mki_db_alignsize( sizeof( MKI_BLOCK_TEXT ) ) )
#define MKI_DB_SIZE_END      ( mki_db_alignsize( sizeof( MKI_BLOCK_BASE ) ) )
#define MKI_DB_SIZE_BASE     ( mki_db_alignsize( sizeof( MKI_BLOCK_BASE ) ) )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_HEADER
{
  public:

  MKI_OFFSET m_nSize;

  __int16 m_nIdent,
          m_nFlag,
          m_nBlockOwner,
          m_nBlockType,
          m_nElemType,
          m_nElemCount;
};

MKI_CLASS_TYPES( BLOCK_HEADER )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_BASE
{
  public:

  __forceinline bool
  IsNotEndOfList( void ) const
  {
    return( m_cHeader.m_nSize >= MKI_DB_SIZE_BASE && m_cHeader.m_nSize <= MKI_DB_SIZE_MAX && 
            m_cHeader.m_nBlockType != MKI_DB_END );
  }

  __forceinline bool
  IsPoint( void ) const
  {
    return( m_cHeader.m_nBlockType >= MKI_DB_POINT_MIN && m_cHeader.m_nBlockType < MKI_DB_POINT_MAX );
  }

  __forceinline bool
  IsFixedPoint( void ) const
  {
    return( m_cHeader.m_nBlockType >= MKI_DB_FIXED_MIN && m_cHeader.m_nBlockType < MKI_DB_FIXED_MAX );
  }

  __forceinline bool
  IsAnyPoint( void ) const
  {
    return( m_cHeader.m_nBlockType >= MKI_DB_POINT_MIN && m_cHeader.m_nBlockType < MKI_DB_FIXED_MAX );
  }

  __forceinline bool
  IsAnyPointOrInfo( void ) const
  {
    return( m_cHeader.m_nBlockType >= MKI_DB_TEMP_MIN && m_cHeader.m_nBlockType < MKI_DB_ALL_MAX );
  }

  __forceinline bool
  IsAttribute( void ) const
  {
    return( m_cHeader.m_nBlockType >= MKI_DB_ATTRIB_MIN && m_cHeader.m_nBlockType < MKI_DB_ATTRIB_MAX );
  }

  __forceinline bool
  IsGlobalAttribute( void ) const
  {
    return( m_cHeader.m_nBlockType >= MKI_DB_ATTRIB_GLOBAL_MIN && m_cHeader.m_nBlockType < MKI_DB_ATTRIB_GLOBAL_MAX );
  }

  __forceinline bool
  IsLocalAttribute( void ) const
  {
    return( m_cHeader.m_nBlockType >= MKI_DB_ATTRIB_LOCAL_MIN && m_cHeader.m_nBlockType < MKI_DB_ATTRIB_LOCAL_MAX );
  }

  __forceinline bool
  IsBaseAttribute( void ) const
  {
    return( m_cHeader.m_nBlockType >= MKI_DB_ATTRIB_GLOBAL_MIN &&
            m_cHeader.m_nBlockType <  MKI_DB_ATTRIB_LOCAL_MAX );
  }

  __forceinline bool
  IsOutlineAttribute( void ) const
  {
    return( m_cHeader.m_nBlockType >= MKI_DB_ATTRIB_OUTLINE_MIN && m_cHeader.m_nBlockType < MKI_DB_ATTRIB_OUTLINE_MAX );
  }

  __forceinline bool
  IsPointAttribute( void ) const
  {
    return( m_cHeader.m_nBlockType >= MKI_DB_ATTRIB_POINT_MIN && m_cHeader.m_nBlockType < MKI_DB_ATTRIB_POINT_MAX );
  }

  __forceinline bool
  IsOutlineOrPointAttribute( void ) const
  {
    return( m_cHeader.m_nBlockType >= MKI_DB_ATTRIB_OUTLINE_MIN && m_cHeader.m_nBlockType < MKI_DB_ATTRIB_POINT_MAX );
  }

  __forceinline bool
  IsNotAttribute( void ) const
  {
    return( ( m_cHeader.m_nBlockType >= MKI_DB_TEMP_MIN && m_cHeader.m_nBlockType < MKI_DB_ATTRIB_MIN ) ||
            m_cHeader.m_nBlockType >= MKI_DB_ATTRIB_MAX );
  }

  __forceinline bool
  IsNotBaseAttribute( void ) const
  {
    return( ( m_cHeader.m_nBlockType >= MKI_DB_TEMP_MIN && m_cHeader.m_nBlockType < MKI_DB_ATTRIB_GLOBAL_MIN ) ||
            ( m_cHeader.m_nBlockType >= MKI_DB_ATTRIB_OUTLINE_MIN && m_cHeader.m_nBlockType < MKI_DB_ATTRIB_MAX ) ||
            ( m_cHeader.m_nBlockType >= MKI_DB_CUSTOM_MIN && m_cHeader.m_nBlockType < MKI_DB_CUSTOM_MAX ) );
  }

  //------------------------------------------------------------------

  __forceinline __int16
  GetFlag( void ) const
  {
    return( m_cHeader.m_nFlag );
  }

  __forceinline bool
  IsFlagSet( const __int16 f_nFlag ) const
  {
    return( ( m_cHeader.m_nFlag & f_nFlag ) != 0 );
  }

  __forceinline bool
  IsFlagClear( const __int16 f_nFlag ) const
  {
    return( ( m_cHeader.m_nFlag & f_nFlag ) == 0 );
  }

  __forceinline void
  SetFlag( const __int16 f_hFlag )
  {
    m_cHeader.m_nFlag = f_hFlag;
  }

  __forceinline void
  SetFlagBits( const __int16 f_hFlag )
  {
    m_cHeader.m_nFlag |= f_hFlag;
  }

  __forceinline void
  ClearFlagBits( const __int16 f_hFlag )
  {
    m_cHeader.m_nFlag &= ~f_hFlag;
  }

  __forceinline void
  InvertFlagBits( const __int16 f_hFlag )
  {
    m_cHeader.m_nFlag ^= f_hFlag;
  }

  //------------------------------------------------------------------

  MKI_BLOCK_HEADER m_cHeader;

};

MKI_CLASS_TYPES( BLOCK_BASE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
typedef MKI_BLOCK_BASE*             MKI_BLOCK_PTR;
typedef const MKI_BLOCK_BASE* const MKI_CONST_BLOCK_PTR;


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_INT32 : public MKI_BLOCK_BASE
{
  public:

  // ElemType = 1, ElemCount = Number of LONGs ( >0 )
  __int32 m_pData[];
};

MKI_CLASS_TYPES( BLOCK_INT32 )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_DOUBLE : public MKI_BLOCK_BASE
{
  public:

  // ElemType = 2, ElemCount = Number of DOUBLEs ( >0 )
  double m_pData[];
};

MKI_CLASS_TYPES( BLOCK_DOUBLE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_POINT : public MKI_BLOCK_BASE
{
  public:

  // ElemType = 3, ElemCount = Number of MKI_POINTs ( >0 )
  MKI_POINT m_pData[];
};

MKI_CLASS_TYPES( BLOCK_POINT )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_COLOR : public MKI_BLOCK_BASE
{
  public:

  // ElemType = 4, ElemCount = Number of MKI_COLORVALUEs ( >0 )
  MKI_COLORVALUE m_pData[];
};

MKI_CLASS_TYPES( BLOCK_COLOR )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_PROPERTY : public MKI_BLOCK_BASE
{
  public:

  // ElemType = 5, ElemCount = Number of MKI_PROPERTYs ( >0 )
  MKI_PROPERTY m_pData[];
};

MKI_CLASS_TYPES( BLOCK_PROPERTY )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_XPROPERTY : public MKI_BLOCK_BASE
{
  public:

  // ElemType = 6, ElemCount = Number of MKI_XPROPERTYs ( >0 )
  MKI_XPROPERTY m_pData[];
};

MKI_CLASS_TYPES( BLOCK_XPROPERTY )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_FONTDEF : public MKI_BLOCK_BASE
{
  public:

  // ElemType = 7, ElemCount = Number of MKI_FONTDEFs ( >0 )
  MKI_FONTDEF m_pData[];
};

MKI_CLASS_TYPES( BLOCK_FONTDEF )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_TEXT : public MKI_BLOCK_BASE
{
  public:

  // ElemType = 8, ElemCount = Text length ( <0 static, >0 dynamic )
  wchar_t m_pText[];
};

MKI_CLASS_TYPES( BLOCK_TEXT )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_BINARY : public MKI_BLOCK_BASE
{
  public:

  // ElemType = 9, ElemCount = Data length ( >0 )
  BYTE m_pData[];
};

MKI_CLASS_TYPES( BLOCK_BINARY )


//---------------------------------------------------------------------------------------------------------------------------------------------------
// Native Data Blocks
class MKI_BLOCK_ATTRIBUTE : public MKI_BLOCK_BASE
{
  public:

  // ElemCount = Attribute length ( <0 static, >0 dynamic )
  MKI_ATTRIBUTENAME m_szName;
  wchar_t           m_pValue[];
};

MKI_CLASS_TYPES( BLOCK_ATTRIBUTE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_DIMLINE : public MKI_BLOCK_BASE
{
  public:

  MKI_DIMLINE m_cData;
};

MKI_CLASS_TYPES( BLOCK_DIMLINE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_DIMSMALL : public MKI_BLOCK_BASE
{
  public:

  MKI_DIMSMALL m_cData;
};

MKI_CLASS_TYPES( BLOCK_DIMSMALL )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_DIMLARGE : public MKI_BLOCK_BASE
{
  public:

  MKI_DIMLARGE m_cData;
};

MKI_CLASS_TYPES( BLOCK_DIMLARGE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_TEXTSTANDARD : public MKI_BLOCK_BASE
{
  public:

  MKI_TEXTSTANDARD m_cData;
};

MKI_CLASS_TYPES( BLOCK_TEXTSTANDARD )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_TEXTFRAME : public MKI_BLOCK_BASE
{
  public:

  MKI_TEXTFRAME m_cData;
};

MKI_CLASS_TYPES( BLOCK_TEXTFRAME )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_TEXTREFERENCE : public MKI_BLOCK_BASE
{
  public:

  MKI_TEXTREFERENCE m_cData;
};

MKI_CLASS_TYPES( BLOCK_TEXTREFERENCE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_COMMENTDATA : public MKI_BLOCK_BASE
{
  public:

  MKI_COMMENTDATA m_cData;
};

MKI_CLASS_TYPES( BLOCK_COMMENTDATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_BITMAPREF : public MKI_BLOCK_BASE
{
  public:

  MKI_BITMAPREF m_cData;
};

MKI_CLASS_TYPES( BLOCK_BITMAPREF )


//---------------------------------------------------------------------------------------------------------------------------------------------------
// Internal Data Blocks
class MKI_BLOCK_T001 : public MKI_BLOCK_BASE
{
  public:

  double m_dArcAngle;
};

MKI_CLASS_TYPES( BLOCK_T001 )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_T002 : public MKI_BLOCK_BASE
{
  public:

  double m_dStartAngle,
         m_dArcAngle;
};

MKI_CLASS_TYPES( BLOCK_T002 )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_T003 : public MKI_BLOCK_BASE
{
  public:

  double m_dArrowStartAngle,
         m_dArrowEndAngle;
};

MKI_CLASS_TYPES( BLOCK_T003 )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_T004 : public MKI_BLOCK_BASE
{
  public:

  double m_dArcAngle,
         m_dStartAngle,
         m_dEndAngle,
         m_dArrowStartAngle,
         m_dArrowEndAngle;
};

MKI_CLASS_TYPES( BLOCK_T004 )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_T005 : public MKI_BLOCK_BASE
{
  public:

  MKI_STR32W  m_szExtraText;
  double      m_dDimValue,
              m_dArrowStartAngle,
              m_dArrowEndAngle,
              m_dNumberAngle;

  MKI_POINT   m_cPoint1, // Position of arrow 1 / start point of dimension
              m_cPoint2, // Position of arrow 2 / end point of dimension
              m_cPoint3, // End point of extension line 1 (x1,y1)
              m_cPoint4, // End point of extension line 2 (x2,y2)
              m_cPoint7; // Insertion point of number

  MKI_QUAD    m_cBackQuad;  // Four corner points of the underlying text quandrangle
  double      m_dGap1,      // Gap for the dimension text in interval [0,1] of dimension line length
              m_dGap2;
};

MKI_CLASS_TYPES( BLOCK_T005 )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_T006 : public MKI_BLOCK_BASE
{
  public:

  double  m_dDimValue,
          m_dArcAngle,
          m_dStartAngle,
          m_dEndAngle,
          m_dArrowStartAngle,
          m_dArrowEndAngle,
          m_dNumberAngle;

  MKI_POINT m_cPoint1, // Position of arrow 1 / start point of dimension
            m_cPoint2, // Position of arrow 2 / end point of dimension
            m_cPoint3, // End point of extension line 1 (x1,y1)
            m_cPoint4, // End point of extension line 2 (x2,y2)
            m_cPoint5, // Center point of dimension line arc
            m_cPoint6, // Radius point of dimension line arc
            m_cPoint7; // Insertion point of number

  MKI_QUAD  m_cBackQuad;  // Four corner points of the underlying text quandrangle
  double    m_dGap1,      // Gap for the dimension text in interval [0,1] of dimension line length
            m_dGap2;
};

MKI_CLASS_TYPES( BLOCK_T006 )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_T007 : public MKI_BLOCK_BASE
{
  public:

  double  m_dDimValue1,
          m_dDimValue2,
          m_dArrowAngle,
          m_dNumberAngle;

  MKI_POINT m_cPoint1,  // Corner point 1 / Center point
            m_cPoint2,  // Corner point 2 / Radius point / Vector 1
            m_cPoint3,  // Corner point 3                / Vector 2
            m_cPoint4,  // Corner point 4
            m_cPoint5,  // Corner point 5
            m_cPoint6,  // Corner point 6
            m_cPoint7,  // Corner point 7
            m_cPoint8,  // Corner point 8
            m_cPoint9,  // Start point of reference line
            m_cPoint10, // Bending point of reference line
            m_cPoint11; // Insertion point of number

  MKI_QUAD  m_cBackQuad;  // Four corner points of the underlying text quandrangle
};

MKI_CLASS_TYPES( BLOCK_T007 )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_T008 : public MKI_BLOCK_BASE
{
  public:
};

MKI_CLASS_TYPES( BLOCK_T008 )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_ENTITY_MEMORY
{
  public:

  size_t m_nUnits, // Internal Data - do not modify!
         m_nBytes; // Internal Data - do not modify!

  MKI_ENTITY_MEMORY* m_pPrev; // Internal Data - do not modify!
  MKI_ENTITY_MEMORY* m_pNext; // Internal Data - do not modify!
};

constexpr __int32 MKI_UNDO_MIN_INFINITY = -2147000000;  // - infinite undo time
constexpr __int32 MKI_UNDO_MAX_INFINITY =  2147000000;  // + infinite undo time

MKI_CLASS_TYPES( ENTITY_MEMORY )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_ENTITY_HEADER
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_RECT m_cRect;      // Internal Data - do not modify!
  __int32  m_nIdent,     // Internal Data - do not modify!
           m_nFlag,      // Internal Data - do not modify!
           m_nUndo1,     // Internal Data - do not modify!
           m_nUndo2,     // Internal Data - do not modify!
           m_nReserved1, // Internal Data - do not modify!
           m_nReserved2; // Internal Data - do not modify!

  MKI_ENTITY_PTR m_pLinkFirst, // Internal Data - do not modify!
                 m_pLinkLast,  // Internal Data - do not modify!
                 m_pDataFirst, // Internal Data - do not modify!
                 m_pDataLast;  // Internal Data - do not modify!

  __int32       m_nEntityOwner,
                m_nEntityType;
  MKI_UNIQUE_ID m_cUniqueID;
  __int32       m_nEntityMode;

  __int16          m_nPlugInNum;     // Internal Data - do not modify!
  unsigned __int16 m_fDataValid :1,  // Internal Data - do not modify!
                   m_fFrameValid:1,  // Internal Data - do not modify!
                   m_fNotFound  :1,  // Internal Data - do not modify!
                   m_afUnused   :13; // Internal Data - do not modify!
};

MKI_CLASS_TYPES( ENTITY_HEADER )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_ENTITY_BASE
{
  public:

  MKI_INTERFACE_MEMBER void
  InitHeader( __int32 f_nEntityType );

  MKI_INTERFACE_MEMBER MKI_ENTITY_PTR
  GetRoot( void ) const;

  MKI_INTERFACE_MEMBER __int32
  GetDepth( void ) const;

  MKI_INTERFACE_MEMBER __int64
  GetElemCount( void ) const;

  MKI_INTERFACE_MEMBER bool
  HasLinkedID( void ) const;

  MKI_INTERFACE_MEMBER bool
  GetLinkedID( MKI_UNIQUE_ID_REF f_rID ) const;

  MKI_INTERFACE_MEMBER bool
  SetLinkedID( MKI_CONST_UNIQUE_ID_REF f_rID );

  MKI_INTERFACE_MEMBER bool
  GetTitle( MKI_STR256W& f_rszText ) const;

  MKI_INTERFACE_MEMBER bool
  GetLongTitle( MKI_STR256W& f_rszText ) const;

  MKI_INTERFACE_MEMBER void
  GetMultiXProperty( MKI_MULTI_XPROPERTY& f_rData ) const;

  MKI_INTERFACE_MEMBER size_t
  GetHeaderSize( void ) const;

  MKI_INTERFACE_MEMBER LPBYTE
  GetDataAddress( void ) const;

  MKI_INTERFACE_MEMBER __int32
  GetDataType( void ) const;

  MKI_INTERFACE_MEMBER __int32
  GetOutputType( void ) const;

  MKI_INTERFACE_MEMBER bool
  AllowsOutlineOrPointAttributes( void ) const;

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER bool
  IsHidden( void ) const;

  MKI_INTERFACE_MEMBER bool
  IsExt3D( void ) const;
  
  MKI_INTERFACE_MEMBER bool
  IsExt3DAssembly( void ) const;
  
  MKI_INTERFACE_MEMBER bool
  IsExt3DMapping( void ) const;
  
  MKI_INTERFACE_MEMBER bool
  IsExtGradient( void ) const;
  
  MKI_INTERFACE_MEMBER bool
  IsExtParametric( void ) const;

  //------------------------------------------------------------------

  __forceinline __int32
  GetFlag( void ) const
  {
    return( m_cHeader.m_nFlag );
  }

  __forceinline bool
  IsFlagSet( const __int32 f_nFlag ) const
  {
    return( ( m_cHeader.m_nFlag & f_nFlag ) != 0 );
  }

  __forceinline bool
  IsFlagClear( const __int32 f_nFlag ) const
  {
    return( ( m_cHeader.m_nFlag & f_nFlag ) == 0 );
  }

  __forceinline void
  SetFlag( const __int32 f_nFlag )
  {
    m_cHeader.m_nFlag = f_nFlag;
  }

  __forceinline void
  SetFlagBits( const __int32 f_nFlag )
  {
    m_cHeader.m_nFlag |= f_nFlag;
  }

  __forceinline void
  ClearFlagBits( const __int32 f_nFlag )
  {
    m_cHeader.m_nFlag &= ~f_nFlag;
  }

  __forceinline void
  InvertFlagBits( const __int32 f_nFlag )
  {
    m_cHeader.m_nFlag ^= f_nFlag;
  }

  //------------------------------------------------------------------

  MKI_ENTITY_MEMORY m_cMemory;
  MKI_ENTITY_PTR    m_pParent;
  MKI_ENTITY_HEADER m_cHeader;
};

MKI_CLASS_TYPES( ENTITY_BASE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_ENTITY_ATTRIB : public MKI_ENTITY_BASE
{
  public:

  BYTE m_pData[];
};

MKI_CLASS_TYPES( ENTITY_ATTRIB )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_ENTITY_OBJECT : public MKI_ENTITY_BASE
{
  public:

  MKI_ENTITY_OBJECT( void )
  {
    m_cHeader.m_nEntityType = MKI_ENTITYTYPE_OBJECT;
  };

  //------------------------------------------------------------------

  static MKI_INTERFACE_MEMBER bool
  IsGeoObject( __int32 f_nObjectType );

  MKI_INTERFACE_MEMBER bool
  IsGeoObject( void ) const;

  static MKI_INTERFACE_MEMBER bool
  IsTextOnlyObject( __int32 f_nObjectType );

  MKI_INTERFACE_MEMBER bool
  IsTextOnlyObject( void ) const;

  static MKI_INTERFACE_MEMBER bool
  IsSurface( __int32 f_nObjectType );

  MKI_INTERFACE_MEMBER bool
  IsSurface( void ) const;

  static MKI_INTERFACE_MEMBER bool
  IsBasicObject( __int32 f_nObjectType );

  MKI_INTERFACE_MEMBER bool
  IsBasicObject( void ) const;

  static MKI_INTERFACE_MEMBER bool
  IsSpecialObject( __int32 f_nObjectType );

  MKI_INTERFACE_MEMBER bool
  IsSpecialObject( void ) const;

  static MKI_INTERFACE_MEMBER __int32
  GetOutputType( __int32 f_nObjType );

  MKI_INTERFACE_MEMBER __int32
  GetOutputType( void ) const;

  MKI_INTERFACE_MEMBER __int32
  GetOutlineCount( void ) const;

  static MKI_INTERFACE_MEMBER bool
  AllowsOutlineOrPointAttributes( __int32 f_nObjType );

  MKI_INTERFACE_MEMBER bool
  AllowsOutlineOrPointAttributes( void ) const;

  //------------------------------------------------------------------

  // EntityType = MKI_ENTITYTYPE_OBJECT
  MKI_XPROPERTY m_cXProperty;

  __int32 m_nObjectType;

  BYTE m_pData[];
};

MKI_CLASS_TYPES( ENTITY_OBJECT )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_ENTITY_BLOCK : public MKI_ENTITY_BASE
{
  public:

  MKI_INTERFACE_MEMBER bool
  IsIntern( void ) const;

  MKI_INTERFACE_MEMBER bool
  IsGroup( void ) const;

  MKI_INTERFACE_MEMBER bool
  IsPosNo( void ) const;

  MKI_INTERFACE_MEMBER MKI_BLOCK_ATTRIBUTE_PTR
  GetPosNo( __int32 f_nRange );

  //------------------------------------------------------------------

  // EntityType = MKI_ENTITYTYPE_BLOCK
  MKI_XPROPERTY m_cXProperty;

  MKI_BLOCKNAME   m_szBlockName;
  MKI_COMMENTLONG m_szComment;

  MKI_RECT  m_cBlockRect;

  BYTE m_pData[];

};

MKI_CLASS_TYPES( ENTITY_BLOCK )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_ENTITY_INSTANCE : public MKI_ENTITY_BASE
{
  public:

  MKI_INTERFACE_MEMBER bool
  IsIntern( void ) const;

  MKI_INTERFACE_MEMBER bool
  IsGroup( void ) const;

  MKI_INTERFACE_MEMBER bool
  IsPosNo( void ) const;

  MKI_INTERFACE_MEMBER MKI_BLOCK_ATTRIBUTE_PTR
  GetPosNo( __int32 f_nRange );

  //------------------------------------------------------------------

  // EntityType = MKI_ENTITYTYPE_INSTANCE
  MKI_XPROPERTY m_cXProperty;

  MKI_LIBRARYNAME m_szLibraryName;
  MKI_BLOCKNAME   m_szBlockName;
  MKI_COMMENTLONG m_szComment;

  MKI_MATRIX  m_cDisplayMatrix;
  __int32     m_nDisplayMode;

  MKI_ENTITY_BLOCK_PTR  m_pBlock;      // Internal Data - do not modify!
  __int32               m_nListIndex,  // Internal Data - do not modify!
                        m_nBlockIndex; // Internal Data - do not modify!

  BYTE m_pData[];
};

MKI_CLASS_TYPES( ENTITY_INSTANCE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_ENTITY_CLIP : public MKI_ENTITY_BASE
{
  public:

  // EntityType = MKI_ENTITYTYPE_CLIP
  MKI_XPROPERTY m_cXProperty;

  MKI_MATRIX  m_cDisplayMatrix;
  __int32     m_nDisplayMode;

  MKI_HIDELAYERS m_afHideLayers;  

  BYTE m_pData[];
};

MKI_CLASS_TYPES( ENTITY_CLIP )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_ENTITY_REF : public MKI_ENTITY_BASE
{
  public:

  // EntityType = MKI_ENTITYTYPE_REF
  MKI_XPROPERTY m_cXProperty;

  MKI_FILEREFERENCE m_szDrawingName;

  MKI_MATRIX  m_cDisplayMatrix;
  __int32     m_nDisplayMode;

  MKI_HIDELAYERS m_afHideLayers;

  BYTE m_pData[];
};

MKI_CLASS_TYPES( ENTITY_REF )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PATCH_INVISIBLE1     = 0x0001,
  MKI_PATCH_INVISIBLE2     = 0x0002,
  MKI_PATCH_INVISIBLE3     = 0x0004,
  MKI_PATCH_INVISIBLE4     = 0x0008,
  MKI_PATCH_INVISIBLE_MASK = 0x00ff,

  MKI_PATCH_INACTIVE1      = 0x0100,
  MKI_PATCH_INACTIVE2      = 0x0200,
  MKI_PATCH_INACTIVE3      = 0x0400,
  MKI_PATCH_INACTIVE4      = 0x0800,
  MKI_PATCH_INACTIVE_MASK  = 0xff00,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_ENTITY_PATCH : public MKI_ENTITY_BASE
{
  public:

  __forceinline bool
  IsFilled( void ) const
  {
    return( m_cProperty.m_nFillMode == MKI_FILLMODE_FILLED ||
            m_cProperty.m_nFillMode == MKI_FILLMODE_FILLED_FRAMED );
  };

  __forceinline bool
  IsErased( void ) const
  {
    return( m_cProperty.m_nFillMode == MKI_FILLMODE_ERASER ||
            m_cProperty.m_nFillMode == MKI_FILLMODE_ERASER_FRAMED );
  };

  __forceinline bool
  IsOpaque( void ) const
  {
    return( m_cProperty.m_nFillMode != MKI_FILLMODE_FRAMED );
  };

  __forceinline bool
  IsFramed( void ) const
  {
    return( m_cProperty.m_nFillMode != MKI_FILLMODE_FILLED &&
            m_cProperty.m_nFillMode != MKI_FILLMODE_ERASER );
  };

  // EntityType = MKI_ENTITYTYPE_PATCH
  MKI_PATCHPROPERTY m_cProperty;

  __int32   m_nPointCount,
            m_nInvisible;
  MKI_POINT m_pPoint[];
};

MKI_CLASS_TYPES( ENTITY_PATCH )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_ENTITY_MAPPING : public MKI_ENTITY_BASE
{
  public:

  // EntityType = MKI_ENTITYTYPE_MAPPING
  MKI_XPROPERTY m_cXProperty;

  MKI_MATRIX  m_cDisplayMatrix;
  __int32     m_nDisplayMode;

  MKI_HIDELAYERS m_afHideLayers;
  MKI_LAYERDATA m_cLayerData;

  BYTE m_pData[];
};

MKI_CLASS_TYPES( ENTITY_MAPPING )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_ENTITY_GROUP : public MKI_ENTITY_BASE
{
  public:

  MKI_INTERFACE_MEMBER bool
  IsPosNo( void ) const;

  MKI_INTERFACE_MEMBER MKI_BLOCK_ATTRIBUTE_PTR
  GetPosNo( __int32 f_nRange );

  //------------------------------------------------------------------

  MKI_COMMENTLONG m_szComment;

  BYTE m_pData[];

};

MKI_CLASS_TYPES( ENTITY_GROUP )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_ENTITY_HATCH : public MKI_ENTITY_BASE
{
  public:

  // EntityType = MKI_ENTITYTYPE_HATCH
  MKI_XPROPERTY m_cXProperty;

  MKI_MATRIX  m_cDisplayMatrix;
  __int32     m_nDisplayMode;

  MKI_HATCHDEF  m_cHatchDef;

  BYTE m_pData[];
};

MKI_CLASS_TYPES( ENTITY_HATCH )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_ENTITY_EXT : public MKI_ENTITY_BASE
{
  public:

  // EntityType = MKI_ENTITYTYPE_EXT
  MKI_XPROPERTY m_cXProperty;

  MKI_MATRIX  m_cDisplayMatrix;
  __int32     m_nDisplayMode,
              m_nExtType,
              m_nExtFlag,
              m_nVersion;

  __int64     m_nUserData1,   // Custom Temporary Data, will not be stored in a file
              m_nUserData2;   // Custom Temporary Data, will not be stored in a file

  BYTE m_pData[];
};

MKI_CLASS_TYPES( ENTITY_EXT )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_ENTITYLIST_ENTITIES    = 0,  // Entity list of the drawing
  MKI_ENTITYLIST_BLOCKS      = 1,  // Block list of the drawing
  MKI_ENTITYLIST_PLUGINS     = 2,  // Plug-in data list
  MKI_ENTITYLIST_FONTBLOCKS  = 3,  // Block lists of fonts
  MKI_ENTITYLIST_LIBBLOCKS   = (MKI_ENTITYLIST_FONTBLOCKS + MKI_FONT_MAX),      // Block lists of libraries
  MKI_ENTITYLIST_REFENTITIES = (MKI_ENTITYLIST_LIBBLOCKS + MKI_LIBRARY_MAX),    // Entity lists of drawing references
  MKI_ENTITYLIST_REFBLOCKS   = (MKI_ENTITYLIST_REFENTITIES + MKI_EXTERNAL_MAX), // Block lists of drawing references
  MKI_ENTITYLIST_TOTAL       = (MKI_ENTITYLIST_REFBLOCKS + MKI_EXTERNAL_MAX),
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_ENTITYENUM_TOP   = 0, // Enumerate all chosen entities and top-level roots
  MKI_ENTITYENUM_FLAGS = 1, // Enumerate all chosen entities and roots, plus all chosen leafs of non-chosen roots
  MKI_ENTITYENUM_LEAFS = 2, // Enumerate all entities and leafs, but no roots
  MKI_ENTITYENUM_ROOTS = 3, // Enumarate all chosen objects, plus all roots that are chosen themselves and/or contain chosen leafs
  MKI_ENTITYENUM_ALL   = 4, // Enumerate all entities, all roots, all leafs of all roots
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_ENTITY_ENUM
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_ENTITY_ENUM( void );

  MKI_INTERFACE_MEMBER
  ~MKI_ENTITY_ENUM();

  MKI_INTERFACE_MEMBER bool
  IsValid( void ) const;

  MKI_INTERFACE_MEMBER MKI_ENTITY_PTR
  Reset( MKI_CONST_ENTITY_PTR f_pEntity, __int32 f_nFlag1, __int32 f_nFlag2, bool f_fValid, __int32 f_nEnumMode = MKI_ENTITYENUM_TOP );

  MKI_INTERFACE_MEMBER MKI_ENTITY_PTR
  GetFirst( __int32 f_nListIndex, __int32 f_nFlag1, __int32 f_nFlag2, bool f_fValid, __int32 f_nEnumMode = MKI_ENTITYENUM_TOP );

  MKI_INTERFACE_MEMBER MKI_ENTITY_PTR
  GetNext( MKI_CONST_ENTITY_PTR f_pEntity );

  MKI_INTERFACE_MEMBER MKI_ENTITY_PTR
  GetLast( __int32 f_nListIndex, __int32 f_nFlag1, __int32 f_nFlag2, bool f_fValid, __int32 f_nEnumMode = MKI_ENTITYENUM_TOP );

  MKI_INTERFACE_MEMBER MKI_ENTITY_PTR
  GetPrevious( MKI_CONST_ENTITY_PTR f_pEntity );

  private:

  void* m_pData;
};

MKI_CLASS_TYPES( ENTITY_ENUM )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_ATTRIBUTE_ENTRY
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  __forceinline bool
  IsAttribute( void ) const
  {
    return( m_nBlockType >= MKI_DB_ATTRIB_MIN &&
            m_nBlockType <  MKI_DB_ATTRIB_MAX );
  }

  __forceinline bool
  IsLocalAttribute( void ) const
  {
    return( m_nBlockType >= MKI_DB_ATTRIB_LOCAL_MIN &&
            m_nBlockType <  MKI_DB_ATTRIB_LOCAL_MAX );
  }

  __forceinline bool
  IsGlobalAttribute( void ) const
  {
    return( m_nBlockType >= MKI_DB_ATTRIB_GLOBAL_MIN &&
            m_nBlockType <  MKI_DB_ATTRIB_GLOBAL_MAX );
  }

  __forceinline bool
  IsBaseAttribute( void ) const
  {
    return( m_nBlockType >= MKI_DB_ATTRIB_GLOBAL_MIN &&
            m_nBlockType <  MKI_DB_ATTRIB_LOCAL_MAX );
  }

  __forceinline bool
  IsOutlineAttribute( void ) const
  {
    return( m_nBlockType >= MKI_DB_ATTRIB_OUTLINE_MIN &&
            m_nBlockType <  MKI_DB_ATTRIB_OUTLINE_MAX );
  }

  __forceinline bool
  IsPointAttribute( void ) const
  {
    return( m_nBlockType >= MKI_DB_ATTRIB_POINT_MIN &&
            m_nBlockType <  MKI_DB_ATTRIB_POINT_MAX );
  }

  __forceinline bool
  IsOutlineOrPointAttribute( void ) const
  {
    return( m_nBlockType >= MKI_DB_ATTRIB_OUTLINE_MIN &&
            m_nBlockType <  MKI_DB_ATTRIB_POINT_MAX );
  }

  //------------------------------------------------------------------

  __int16            m_nIdent,
                     m_nBlockType;
  MKI_ATTRIBUTENAME  m_szName;
  MKI_ATTRIBUTEVALUE m_szValue;
};

MKI_CLASS_TYPES( ATTRIBUTE_ENTRY )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_ATTRIBUTE_MODE_ALL     = 0,
  MKI_ATTRIBUTE_MODE_BASE    = 1,
  MKI_ATTRIBUTE_MODE_GLOBAL  = 2,
  MKI_ATTRIBUTE_MODE_LOCAL   = 3,
  MKI_ATTRIBUTE_MODE_OUTLINE = 4,
  MKI_ATTRIBUTE_MODE_POINT   = 5,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_ATTRIBUTE_DATA
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_ATTRIBUTE_DATA( void );

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER __int32
  GetIndex( __int16 f_nBlockType, const LPCWSTR f_pszName ) const;

  MKI_INTERFACE_MEMBER __int32
  Add( __int16 f_nBlockType, const LPCWSTR f_pszName, const LPCWSTR f_pszValue, bool f_fReplace );

  MKI_INTERFACE_MEMBER bool
  Remove( __int32 f_nIndex );

  //------------------------------------------------------------------

  HWND  m_hWnd;

  MKI_LIBRARYNAME m_szLibraryName;
  MKI_BLOCKNAME   m_szBlockName;

  __int32 m_nDialogMode,
          m_nCount,
          m_nActive;
  bool    m_fIndexIsLocked;

  MKI_ATTRIBUTE_ENTRY m_acList[MKI_ATTRIBS_PER_OBJECT];
};

MKI_CLASS_TYPES( ATTRIBUTE_DATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BLOCK_ENUM
{
  public:

  MKI_BLOCK_ENUM( MKI_CONST_ENTITY_PTR f_pEntity )
  {
    m_pEntity = (MKI_ENTITY_PTR) f_pEntity;
    m_pData = (MKI_BLOCK_PTR) ( m_pBase = (LPBYTE) f_pEntity->GetDataAddress() );
    m_nOffset = 0;
    m_nHeader = (MKI_OFFSET) f_pEntity->GetHeaderSize();
  }

  MKI_BLOCK_ENUM( MKI_CONST_ENTITY_OBJECT_PTR f_pObj )
  {
    m_pEntity = (MKI_ENTITY_PTR) f_pObj;
    m_pData = (MKI_BLOCK_PTR) ( m_pBase = (LPBYTE) f_pObj->m_pData );
    m_nOffset = 0;
    m_nHeader = sizeof( MKI_ENTITY_OBJECT );
  }

  MKI_BLOCK_ENUM( MKI_CONST_ENTITY_INSTANCE_PTR f_pInst )
  {
    m_pEntity = (MKI_ENTITY_PTR) f_pInst;
    m_pData = (MKI_BLOCK_PTR) ( m_pBase = (LPBYTE) f_pInst->m_pData );
    m_nOffset = 0;
    m_nHeader = sizeof( MKI_ENTITY_INSTANCE );
  }

  MKI_BLOCK_ENUM( MKI_CONST_ENTITY_CLIP_PTR f_pClip )
  {
    m_pEntity = (MKI_ENTITY_PTR) f_pClip;
    m_pData = (MKI_BLOCK_PTR) ( m_pBase = (LPBYTE) f_pClip->m_pData );
    m_nOffset = 0;
    m_nHeader = sizeof( MKI_ENTITY_CLIP );
  }

  MKI_BLOCK_ENUM( MKI_CONST_ENTITY_REF_PTR f_pRef )
  {
    m_pEntity = (MKI_ENTITY_PTR) f_pRef;
    m_pData = (MKI_BLOCK_PTR) ( m_pBase = (LPBYTE) f_pRef->m_pData );
    m_nOffset = 0;
    m_nHeader = sizeof( MKI_ENTITY_REF );
  }

  MKI_BLOCK_ENUM( MKI_CONST_ENTITY_MAPPING_PTR f_pMap )
  {
    m_pEntity = (MKI_ENTITY_PTR) f_pMap;
    m_pData = (MKI_BLOCK_PTR) ( m_pBase = (LPBYTE) f_pMap->m_pData );
    m_nOffset = 0;
    m_nHeader = sizeof( MKI_ENTITY_MAPPING );
  }

  MKI_BLOCK_ENUM( MKI_CONST_ENTITY_GROUP_PTR f_pGroup )
  {
    m_pEntity = (MKI_ENTITY_PTR) f_pGroup;
    m_pData = (MKI_BLOCK_PTR) ( m_pBase = (LPBYTE) f_pGroup->m_pData );
    m_nOffset = 0;
    m_nHeader = sizeof( MKI_ENTITY_GROUP );
  }

  MKI_BLOCK_ENUM( MKI_CONST_ENTITY_HATCH_PTR f_pHatch )
  {
    m_pEntity = (MKI_ENTITY_PTR) f_pHatch;
    m_pData = (MKI_BLOCK_PTR) ( m_pBase = (LPBYTE) f_pHatch->m_pData );
    m_nOffset = 0;
    m_nHeader = sizeof( MKI_ENTITY_HATCH );
  }

  MKI_BLOCK_ENUM( MKI_CONST_ENTITY_EXT_PTR f_pExt )
  {
    m_pEntity = (MKI_ENTITY_PTR) f_pExt;
    m_pData = (MKI_BLOCK_PTR) ( m_pBase = (LPBYTE) f_pExt->m_pData );
    m_nOffset = 0;
    m_nHeader = sizeof( MKI_ENTITY_EXT );
  }

  MKI_BLOCK_ENUM( MKI_CONST_ENTITY_BLOCK_PTR f_pBlock )
  {
    m_pEntity = (MKI_ENTITY_PTR) f_pBlock;
    m_pData = (MKI_BLOCK_PTR) ( m_pBase = (LPBYTE) f_pBlock->m_pData );
    m_nOffset = 0;
    m_nHeader = sizeof( MKI_ENTITY_BLOCK );
  }

  MKI_BLOCK_ENUM( MKI_CONST_ENTITY_ATTRIB_PTR f_pAttrib )
  {
    m_pEntity = (MKI_ENTITY_PTR) f_pAttrib;
    m_pData = (MKI_BLOCK_PTR) ( m_pBase = (LPBYTE) f_pAttrib->m_pData );
    m_nOffset = 0;
    m_nHeader = sizeof( MKI_ENTITY_ATTRIB );
  }

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER __int16
  ApplyOffset( void );

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER __int16
  SetAndApplyOffset( const MKI_OFFSET f_nOffset );

  MKI_INTERFACE_MEMBER __int16
  SetAndApplyDifference( MKI_CONST_ENTITY_OBJECT_PTR f_pObj, MKI_CONST_BLOCK_PTR f_pBlock );

  MKI_INTERFACE_MEMBER __int16
  SetBaseAndResetOffset( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  MKI_INTERFACE_MEMBER __int16
  SetBaseAndResetOffset( MKI_CONST_ENTITY_INSTANCE_PTR f_pInst );

  MKI_INTERFACE_MEMBER __int16
  SetBaseAndResetOffset( MKI_CONST_ENTITY_CLIP_PTR f_pClip );

  MKI_INTERFACE_MEMBER __int16
  SetBaseAndResetOffset( MKI_CONST_ENTITY_REF_PTR f_pRef );

  MKI_INTERFACE_MEMBER __int16
  SetBaseAndResetOffset( MKI_CONST_ENTITY_MAPPING_PTR f_pMap );

  MKI_INTERFACE_MEMBER __int16
  SetBaseAndResetOffset( MKI_CONST_ENTITY_GROUP_PTR f_pGroup );

  MKI_INTERFACE_MEMBER __int16
  SetBaseAndResetOffset( MKI_CONST_ENTITY_HATCH_PTR f_pHatch );

  MKI_INTERFACE_MEMBER __int16
  SetBaseAndResetOffset( MKI_CONST_ENTITY_EXT_PTR f_pExt );

  MKI_INTERFACE_MEMBER __int16
  SetBaseAndResetOffset( MKI_CONST_ENTITY_BLOCK_PTR f_pBlock );

  MKI_INTERFACE_MEMBER __int16
  SetBaseAndResetOffset( MKI_CONST_ENTITY_ATTRIB_PTR f_pAttrib );

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER bool
  PreliminaryTerminate( void );

  MKI_INTERFACE_MEMBER bool
  TerminateAndSetSize( void );

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER bool
  AddSizeToOffset( void );

  MKI_INTERFACE_MEMBER bool
  SetNext( void );

  MKI_INTERFACE_MEMBER bool
  SetNext( const MKI_BLOCK_ENUM& f_rEnum );

  MKI_INTERFACE_MEMBER bool
  ParseBezier( MKI_BLOCK_ENUM& f_rData1, MKI_BLOCK_ENUM& f_rData2 ) const;

  MKI_INTERFACE_MEMBER bool
  ParseArc( MKI_BLOCK_ENUM& f_rData ) const;

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER bool
  CopyAllDataBlocks( const MKI_BLOCK_ENUM& f_rEnum );

  MKI_INTERFACE_MEMBER bool
  CopyAllDataBlocks( MKI_CONST_ENTITY_PTR f_pSource );

  MKI_INTERFACE_MEMBER bool
  CopyPointDataBlocks( const MKI_BLOCK_ENUM& f_rEnum );

  MKI_INTERFACE_MEMBER bool
  CopyPointDataBlocks( MKI_CONST_ENTITY_PTR f_pSource );

  MKI_INTERFACE_MEMBER bool
  CopyNonAttribDataBlocks( MKI_CONST_ENTITY_PTR f_pSource );

  MKI_INTERFACE_MEMBER bool
  CopyNonBaseAttribDataBlocks( MKI_CONST_ENTITY_PTR f_pSource );

  MKI_INTERFACE_MEMBER bool
  CopyLocalAttributes( MKI_CONST_ENTITY_PTR f_pSource );

  MKI_INTERFACE_MEMBER bool
  CopyBaseAttributes( MKI_CONST_ENTITY_PTR f_pSource );

  MKI_INTERFACE_MEMBER bool
  CopyOutlineAttributes( MKI_CONST_ENTITY_PTR f_pSource, __int32 f_nOutlineIndex );

  MKI_INTERFACE_MEMBER bool
  CopyPointAttributes( MKI_CONST_ENTITY_PTR f_pSource, __int32 f_nPointHandle );

  MKI_INTERFACE_MEMBER bool
  AppendLocalAttributes( MKI_CONST_ATTRIBUTE_DATA_REF f_rAttributes );

  MKI_INTERFACE_MEMBER bool
  AppendBaseAttributes( MKI_CONST_ATTRIBUTE_DATA_REF f_rAttributes );

  MKI_INTERFACE_MEMBER bool
  AppendOutlineAttributes( MKI_CONST_ATTRIBUTE_DATA_REF f_rAttributes );

  MKI_INTERFACE_MEMBER bool
  AppendPointAttributes( MKI_CONST_ATTRIBUTE_DATA_REF f_rAttributes );

  //------------------------------------------------------------------

  __forceinline bool
  IsNotEndOfList( void ) const
  {
    return( m_pData->IsNotEndOfList() );
  }

  __forceinline bool
  IsPoint( void ) const
  {
    return( m_pData->IsPoint() );
  }

  __forceinline bool
  IsFixedPoint( void ) const
  {
    return( m_pData->IsFixedPoint() );
  }

  __forceinline bool
  IsAnyPoint( void ) const
  {
    return( m_pData->IsAnyPoint() );
  }

  __forceinline bool
  IsAnyPointOrInfo( void ) const
  {
    return( m_pData->IsAnyPointOrInfo() );
  }

  __forceinline bool
  IsAttribute( void ) const
  {
    return( m_pData->IsAttribute() );
  }

  __forceinline bool
  IsGlobalAttribute( void ) const
  {
    return( m_pData->IsGlobalAttribute() );
  }

  __forceinline bool
  IsLocalAttribute( void ) const
  {
    return( m_pData->IsLocalAttribute() );
  }

  __forceinline bool
  IsBaseAttribute( void ) const
  {
    return( m_pData->IsBaseAttribute() );
  }

  __forceinline bool
  IsOutlineAttribute( void ) const
  {
    return( m_pData->IsOutlineAttribute() );
  }

  __forceinline bool
  IsPointAttribute( void ) const
  {
    return( m_pData->IsPointAttribute() );
  }

  __forceinline bool
  IsOutlineOrPointAttribute( void ) const
  {
    return( m_pData->IsOutlineOrPointAttribute() );
  }

  __forceinline bool
  IsNotAttribute( void ) const
  {
    return( m_pData->IsNotAttribute() );
  }

  __forceinline bool
  IsNotBaseAttribute( void ) const
  {
    return( m_pData->IsNotBaseAttribute() );
  }

  //------------------------------------------------------------------

  __forceinline MKI_BLOCK_PTR
  GetAddress( void ) const
  {
    return( m_pData );
  }

  __forceinline MKI_BLOCK_HEADER_REF
  GetHeader( void ) const
  {
    return( m_pData->m_cHeader );
  }

  __forceinline MKI_OFFSET
  GetOffset( void ) const
  {
    return( m_nOffset );
  }

  __forceinline __int16
  GetFlag( void ) const
  {
    return( m_pData->GetFlag() );
  }

  __forceinline bool
  IsFlagSet( const __int16 f_nFlag ) const
  {
    return( m_pData->IsFlagSet( f_nFlag ) );
  }

  __forceinline bool
  IsFlagClear( const __int16 f_nFlag ) const
  {
    return( m_pData->IsFlagClear( f_nFlag ) );
  }

  __forceinline MKI_OFFSET
  GetSize( void ) const
  {
    return( m_pData->m_cHeader.m_nSize );
  }

  __forceinline __int16
  GetIdent( void ) const
  {
    return( m_pData->m_cHeader.m_nIdent );
  }

  __forceinline __int16
  GetElemType( void ) const
  {
    return( m_pData->m_cHeader.m_nElemType );
  }

  __forceinline __int16
  GetElemCount( void ) const
  {
    return( m_pData->m_cHeader.m_nElemCount );
  }

  __forceinline __int16
  GetBlockOwner( void ) const
  {
    return( m_pData->m_cHeader.m_nBlockOwner );
  }

  __forceinline __int16
  GetBlockType( void ) const
  {
    return( m_pData->m_cHeader.m_nBlockType );
  }

  //------------------------------------------------------------------

  __forceinline void
  SetFlag( const __int16 f_hFlag )
  {
    m_pData->SetFlag( f_hFlag );
  }

  __forceinline void
  SetFlagBits( const __int16 f_hFlag )
  {
    m_pData->SetFlagBits( f_hFlag );
  }

  __forceinline void
  ClearFlagBits( const __int16 f_hFlag )
  {
    m_pData->ClearFlagBits( f_hFlag );
  }

  __forceinline void
  InvertFlagBits( const __int16 f_hFlag )
  {
    m_pData->InvertFlagBits( f_hFlag );
  }

  __forceinline void
  SetAlignSize( const size_t f_nSize )
  {
    m_pData->m_cHeader.m_nSize = mki_db_alignsize( (MKI_OFFSET) f_nSize );
  }

  __forceinline void
  SetIdent( const __int16 f_hIdent )
  {
    m_pData->m_cHeader.m_nIdent = f_hIdent;
  }

  __forceinline void
  SetElemType( const __int16 f_hElemType )
  {
    m_pData->m_cHeader.m_nElemType = f_hElemType;
  }

  __forceinline void
  SetElemCount( const __int16 f_hElemCount )
  {
    m_pData->m_cHeader.m_nElemCount = f_hElemCount;
  }

  __forceinline void
  SetBlockOwner( const __int16 f_hBlockOwner )
  {
    m_pData->m_cHeader.m_nBlockOwner = f_hBlockOwner;
  }

  __forceinline void
  SetBlockType( const __int16 f_hBlockType )
  {
    m_pData->m_cHeader.m_nBlockType = f_hBlockType;
  }

  //------------------------------------------------------------------

  __forceinline double&
  Constant( void ) const
  {
    return( ( (MKI_BLOCK_DOUBLE_PTR) m_pData )->m_pData[0] );
  }

  __forceinline double&
  Orient( void ) const
  {
    return( ( (MKI_BLOCK_DOUBLE_PTR) m_pData )->m_pData[0] );
  }

  __forceinline bool
  IsOrientPositive( void ) const
  {
    return( ( (MKI_BLOCK_DOUBLE_PTR) m_pData )->m_pData[0] >= 0.0 );
  }

  __forceinline bool
  IsOrientNegative( void ) const
  {
    return( ( (MKI_BLOCK_DOUBLE_PTR) m_pData )->m_pData[0] < 0.0 );
  }

  __forceinline double&
  Curve( void ) const
  {
    return( ( (MKI_BLOCK_DOUBLE_PTR) m_pData )->m_pData[1] );
  }

  __forceinline MKI_ATTRIBUTENAME&
  AttributeName( void ) const
  {
    return( ( (MKI_BLOCK_ATTRIBUTE_PTR) m_pData )->m_szName );
  }

  __forceinline LPWSTR
  AttributeValue( void ) const
  {
    return( ( (MKI_BLOCK_ATTRIBUTE_PTR) m_pData )->m_pValue );
  }

  __forceinline __int32*
  Int32( void ) const
  {
    return( ( (MKI_BLOCK_INT32_PTR) m_pData )->m_pData );
  }

  __forceinline double*
  Double( void ) const
  {
    return( ( (MKI_BLOCK_DOUBLE_PTR) m_pData )->m_pData );
  }

  __forceinline MKI_COLORVALUE_PTR
  Color( void ) const
  {
    return( ( (MKI_BLOCK_COLOR_PTR) m_pData )->m_pData );
  }

  __forceinline MKI_PROPERTY_PTR
  Property( void ) const
  {
    return( ( (MKI_BLOCK_PROPERTY_PTR) m_pData )->m_pData );
  }

  __forceinline MKI_XPROPERTY_PTR
  XProperty( void ) const
  {
    return( ( (MKI_BLOCK_XPROPERTY_PTR) m_pData )->m_pData );
  }

  __forceinline MKI_FONTDEF_PTR
  FontDef( void ) const
  {
    return( ( (MKI_BLOCK_FONTDEF_PTR) m_pData )->m_pData );
  }

  __forceinline LPWSTR
  Text( void ) const
  {
    return( ( (MKI_BLOCK_TEXT_PTR) m_pData )->m_pText );
  }

  __forceinline LPBYTE
  Binary( void ) const
  {
    return( ( (MKI_BLOCK_BINARY_PTR) m_pData )->m_pData );
  }

  __forceinline MKI_POINT_REF
  Point( void ) const
  {
    return( ( (MKI_BLOCK_POINT_PTR) m_pData )->m_pData[0] );
  }

  __forceinline MKI_POINT_REF
  Point( const __int32 f_nIndex ) const
  {
    return( ( (MKI_BLOCK_POINT_PTR) m_pData )->m_pData[f_nIndex] );
  }

  __forceinline MKI_DIMLINE_REF
  DimLine( void ) const
  {
    return( ( (MKI_BLOCK_DIMLINE_PTR) m_pData )->m_cData );
  }

  __forceinline MKI_DIMLARGE_REF
  DimLarge( void ) const
  {
    return( ( (MKI_BLOCK_DIMLARGE_PTR) m_pData )->m_cData );
  }

  __forceinline MKI_DIMSMALL_REF
  DimSmall( void ) const
  {
    return( ( (MKI_BLOCK_DIMSMALL_PTR) m_pData )->m_cData );
  }

  __forceinline MKI_TEXTSTANDARD_REF
  TextStandard( void ) const
  {
    return( ( (MKI_BLOCK_TEXTSTANDARD_PTR) m_pData )->m_cData );
  }

  __forceinline MKI_TEXTFRAME_REF
  TextFrame( void ) const
  {
    return( ( (MKI_BLOCK_TEXTFRAME_PTR) m_pData )->m_cData );
  }

  __forceinline MKI_TEXTREFERENCE_REF
  TextReference( void ) const
  {
    return( ( (MKI_BLOCK_TEXTREFERENCE_PTR) m_pData )->m_cData );
  }

  __forceinline MKI_COMMENTDATA_REF
  CommentData( void ) const
  {
    return( ( (MKI_BLOCK_COMMENTDATA_PTR) m_pData )->m_cData );
  }

  __forceinline MKI_BITMAPREF_REF
  BitmapRef( void ) const
  {
    return( ( (MKI_BLOCK_BITMAPREF_PTR) m_pData )->m_cData );
  }

  //------------------------------------------------------------------

  __forceinline MKI_BLOCK_T001_REF
  Temp001( void ) const
  {
    return( (MKI_BLOCK_T001_REF) *m_pData );
  }

  __forceinline MKI_BLOCK_T002_REF
  Temp002( void ) const
  {
    return( (MKI_BLOCK_T002_REF) *m_pData );
  }

  __forceinline MKI_BLOCK_T003_REF
  Temp003( void ) const
  {
    return( (MKI_BLOCK_T003_REF) *m_pData );
  }

  __forceinline MKI_BLOCK_T004_REF
  Temp004( void ) const
  {
    return( (MKI_BLOCK_T004_REF) *m_pData );
  }

  __forceinline MKI_BLOCK_T005_REF
  Temp005( void ) const
  {
    return( (MKI_BLOCK_T005_REF) *m_pData );
  }

  __forceinline MKI_BLOCK_T006_REF
  Temp006( void ) const
  {
    return( (MKI_BLOCK_T006_REF) *m_pData );
  }

  __forceinline MKI_BLOCK_T007_REF
  Temp007( void ) const
  {
    return( (MKI_BLOCK_T007_REF) *m_pData );
  }

  __forceinline MKI_BLOCK_T008_REF
  Temp008( void ) const
  {
    return( (MKI_BLOCK_T008_REF) *m_pData );
  }

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER bool
  AppendDataBlock( const MKI_BLOCK_ENUM& f_rEnum );

  MKI_INTERFACE_MEMBER bool
  AppendPoint( __int32 f_nType, const double f_dX, const double f_dY );

  MKI_INTERFACE_MEMBER bool
  AppendPoint( __int32 f_nType, MKI_CONST_POINT_REF f_rPoint );

  MKI_INTERFACE_MEMBER bool
  AppendOrientArc( const bool f_fOrient );

  MKI_INTERFACE_MEMBER bool
  AppendOrientArc( const double f_dOrient );

  MKI_INTERFACE_MEMBER bool
  AppendOrientCurve( const bool f_fOrient, const double f_dCurve );

  MKI_INTERFACE_MEMBER bool
  AppendOrientCurve( const double f_dOrient, const double f_dCurve );

  MKI_INTERFACE_MEMBER bool
  AppendConstant( const double f_dValue );

  MKI_INTERFACE_MEMBER bool
  AppendTextLong( const LPCWSTR f_pszText );

  MKI_INTERFACE_MEMBER bool
  AppendTextShort( const LPCWSTR f_pszText, bool f_fStaticLength = false );

  MKI_INTERFACE_MEMBER bool
  AppendAttribute( __int32 f_nType, const LPCWSTR f_pszName, const LPCWSTR f_pszValue, bool f_fStaticLength = false );

  MKI_INTERFACE_MEMBER bool
  AppendAny( __int32 f_nType, const size_t f_nSize );

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  InitPoint( __int32 f_nType, const double f_dX, const double f_dY );

  MKI_INTERFACE_MEMBER void
  InitPoint( __int32 f_nType, MKI_CONST_POINT_REF f_rPoint );

  MKI_INTERFACE_MEMBER void
  InitInt32( __int32 f_nType, __int32 f_nData );

  MKI_INTERFACE_MEMBER void
  InitInt32( __int32 f_nType, __int32* f_pData, __int32 f_nCount );

  MKI_INTERFACE_MEMBER void
  InitDouble( __int32 f_nType, double f_dData );

  MKI_INTERFACE_MEMBER void
  InitDouble( __int32 f_nType, double* f_pData, __int32 f_nCount );

  MKI_INTERFACE_MEMBER void
  InitOrientCurve( const bool f_fOrient, const double f_dCurve );

  MKI_INTERFACE_MEMBER void
  InitText( __int16 f_hCount );

  MKI_INTERFACE_MEMBER void
  InitEnd( void );

  MKI_INTERFACE_MEMBER void
  InitAttribute( __int32 f_nType );

  MKI_INTERFACE_MEMBER void
  InitBinary( __int32 f_nType, const void* f_pData, __int32 f_nSize );

  //------------------------------------------------------------------

private:

  // Disable default constructor
  MKI_BLOCK_ENUM( void ) = delete;

  //------------------------------------------------------------------

  MKI_ENTITY_PTR  m_pEntity;
  LPBYTE          m_pBase;
  MKI_BLOCK_PTR   m_pData;
  MKI_OFFSET      m_nHeader,
                  m_nOffset;
};

MKI_CLASS_TYPES( BLOCK_ENUM )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_LINE
{
  public:

  MKI_INTERFACE_MEMBER 
  MKI_PARSE_OBJ_LINE( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_POINT_PTR m_pStart,
                      m_pEnd;
  __int32 m_nHandleStart,
          m_nHandleEnd;

  __forceinline MKI_POINT_REF PointStart( void ) { return( m_pStart->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointEnd( void ) { return( m_pEnd->m_pData[0] ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_LINE( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_LINE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_ZIGZAG
{
  public:

  MKI_INTERFACE_MEMBER 
  MKI_PARSE_OBJ_ZIGZAG( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_POINT_PTR m_pStart,
                      m_pEnd;
  __int32 m_nHandleStart,
          m_nHandleEnd;
  MKI_BLOCK_DOUBLE_PTR m_pData;

  __forceinline MKI_POINT_REF PointStart( void ) { return( m_pStart->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointEnd( void ) { return( m_pEnd->m_pData[0] ); }
  __forceinline double& Distance( void ) { return( m_pData->m_pData[0] ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_ZIGZAG( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_ZIGZAG )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_CIRCLE
{
  public:

  MKI_INTERFACE_MEMBER 
  MKI_PARSE_OBJ_CIRCLE( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_POINT_PTR m_pCenter,
                      m_pRadius;
  __int32 m_nHandleCenter,
          m_nHandleRadius;

  __forceinline MKI_POINT_REF PointCenter( void ) { return( m_pCenter->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointRadius( void ) { return( m_pRadius->m_pData[0] ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_CIRCLE( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_CIRCLE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_ARC
{
  public:

  MKI_INTERFACE_MEMBER 
  MKI_PARSE_OBJ_ARC( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_POINT_PTR m_pCenter,
                      m_pRadius,
                      m_pAngle1,
                      m_pAngle2;
  __int32 m_nHandleCenter,
          m_nHandleRadius,
          m_nHandleAngle1,
          m_nHandleAngle2;
  MKI_BLOCK_DOUBLE_PTR m_pOrient;
  MKI_BLOCK_T001_PTR m_pTemp;

  __forceinline MKI_POINT_REF PointCenter( void ) { return( m_pCenter->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointRadius( void ) { return( m_pRadius->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointAngle1( void ) { return( m_pAngle1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointAngle2( void ) { return( m_pAngle2->m_pData[0] ); }
  __forceinline double& Orient( void ) { return( m_pOrient->m_pData[0] ); }
  __forceinline bool IsOrientPositive( void ) const { return( m_pOrient->m_pData[0] >= 0.0 ); }
  __forceinline bool IsOrientNegative( void ) const { return( m_pOrient->m_pData[0] < 0.0 ); }
  __forceinline double& ArcAngle( void ) { return( m_pTemp->m_dArcAngle ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_ARC( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_ARC )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_ELLIPSE
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_PARSE_OBJ_ELLIPSE( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_POINT_PTR m_pCenter,
                      m_pVector1,
                      m_pVector2;
  __int32 m_nHandleCenter,
          m_nHandleVector1,
          m_nHandleVector2;

  __forceinline MKI_POINT_REF PointCenter( void ) { return( m_pCenter->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointVector1( void ) { return( m_pVector1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointVector2( void ) { return( m_pVector2->m_pData[0] ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_ELLIPSE( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_ELLIPSE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_EARC
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_PARSE_OBJ_EARC( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_POINT_PTR m_pCenter,
                      m_pVector1,
                      m_pVector2,
                      m_pAngle1,
                      m_pAngle2;
  __int32 m_nHandleCenter,
          m_nHandleVector1,
          m_nHandleVector2,
          m_nHandleAngle1,
          m_nHandleAngle2;
  MKI_BLOCK_DOUBLE_PTR m_pOrient;
  MKI_BLOCK_T002_PTR m_pTemp;

  __forceinline MKI_POINT_REF PointCenter( void ) { return( m_pCenter->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointVector1( void ) { return( m_pVector1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointVector2( void ) { return( m_pVector2->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointAngle1( void ) { return( m_pAngle1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointAngle2( void ) { return( m_pAngle2->m_pData[0] ); }
  __forceinline double& Orient( void ) { return( m_pOrient->m_pData[0] ); }
  __forceinline bool IsOrientPositive( void ) const { return( m_pOrient->m_pData[0] >= 0.0 ); }
  __forceinline bool IsOrientNegative( void ) const { return( m_pOrient->m_pData[0] < 0.0 ); }
  __forceinline double& StartAngle( void ) { return( m_pTemp->m_dStartAngle ); }
  __forceinline double& ArcAngle( void ) { return( m_pTemp->m_dArcAngle ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_EARC( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_EARC )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_DLINE
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_PARSE_OBJ_DLINE( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_POINT_PTR m_pStart,
                      m_pEnd;
  __int32 m_nHandleStart,
          m_nHandleEnd;
  MKI_BLOCK_DIMLINE_PTR m_pData;
  MKI_BLOCK_T003_PTR m_pTemp;

  __forceinline MKI_POINT_REF PointStart( void ) { return( m_pStart->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointEnd( void ) { return( m_pEnd->m_pData[0] ); }
  __forceinline MKI_DIMLINE_REF DimLine( void ) { return( m_pData->m_cData ); }
  __forceinline double& ArrowStartAngle( void ) { return( m_pTemp->m_dArrowStartAngle ); }
  __forceinline double& ArrowEndAngle( void ) { return( m_pTemp->m_dArrowEndAngle ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_DLINE( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_DLINE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_DARC
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_PARSE_OBJ_DARC( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_POINT_PTR m_pCenter,
                      m_pRadius,
                      m_pAngle1,
                      m_pAngle2;
  __int32 m_nHandleCenter,
          m_nHandleRadius,
          m_nHandleAngle1,
          m_nHandleAngle2;
  MKI_BLOCK_DOUBLE_PTR m_pOrient;
  MKI_BLOCK_DIMLINE_PTR m_pData;
  MKI_BLOCK_T004_PTR m_pTemp;

  __forceinline MKI_POINT_REF PointCenter( void ) { return( m_pCenter->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointRadius( void ) { return( m_pRadius->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointAngle1( void ) { return( m_pAngle1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointAngle2( void ) { return( m_pAngle2->m_pData[0] ); }
  __forceinline double& Orient( void ) { return( m_pOrient->m_pData[0] ); }
  __forceinline bool IsOrientPositive( void ) const { return( m_pOrient->m_pData[0] >= 0.0 ); }
  __forceinline bool IsOrientNegative( void ) const { return( m_pOrient->m_pData[0] < 0.0 ); }
  __forceinline MKI_DIMLINE_REF DimLine( void ) { return( m_pData->m_cData ); }
  __forceinline double& ArcAngle( void ) { return( m_pTemp->m_dArcAngle ); }
  __forceinline double& StartAngle( void ) { return( m_pTemp->m_dStartAngle ); }
  __forceinline double& EndAngle( void ) { return( m_pTemp->m_dEndAngle ); }
  __forceinline double& ArrowStartAngle( void ) { return( m_pTemp->m_dArrowStartAngle ); }
  __forceinline double& ArrowEndAngle( void ) { return( m_pTemp->m_dArrowEndAngle ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_DARC( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_DARC )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_DEARC
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_PARSE_OBJ_DEARC( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_POINT_PTR m_pCenter,
                      m_pVector1,
                      m_pVector2,
                      m_pAngle1,
                      m_pAngle2;
  __int32 m_nHandleCenter,
          m_nHandleVector1,
          m_nHandleVector2,
          m_nHandleAngle1,
          m_nHandleAngle2;
  MKI_BLOCK_DOUBLE_PTR m_pOrient;
  MKI_BLOCK_DIMLINE_PTR m_pData;
  MKI_BLOCK_T004_PTR m_pTemp;

  __forceinline MKI_POINT_REF PointCenter( void ) { return( m_pCenter->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointVector1( void ) { return( m_pVector1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointVector2( void ) { return( m_pVector2->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointAngle1( void ) { return( m_pAngle1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointAngle2( void ) { return( m_pAngle2->m_pData[0] ); }
  __forceinline double& Orient( void ) { return( m_pOrient->m_pData[0] ); }
  __forceinline bool IsOrientPositive( void ) const { return( m_pOrient->m_pData[0] >= 0.0 ); }
  __forceinline bool IsOrientNegative( void ) const { return( m_pOrient->m_pData[0] < 0.0 ); }
  __forceinline MKI_DIMLINE_REF DimLine( void ) { return( m_pData->m_cData ); }
  __forceinline double& ArcAngle( void ) { return( m_pTemp->m_dArcAngle ); }
  __forceinline double& StartAngle( void ) { return( m_pTemp->m_dStartAngle ); }
  __forceinline double& EndAngle( void ) { return( m_pTemp->m_dEndAngle ); }
  __forceinline double& ArrowStartAngle( void ) { return( m_pTemp->m_dArrowStartAngle ); }
  __forceinline double& ArrowEndAngle( void ) { return( m_pTemp->m_dArrowEndAngle ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_DEARC( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_DEARC )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_DGENERIC5
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_PARSE_OBJ_DGENERIC5( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_POINT_PTR m_pPoint1,
                      m_pPoint2,
                      m_pLine1,
                      m_pLine2,
                      m_pNumber1,
                      m_pNumber2,
                      m_pExtension1,
                      m_pExtension2;
  MKI_BLOCK_TEXT_PTR m_pText1,
                     m_pText2,
                     m_pText3,
                     m_pTolerance1,
                     m_pTolerance2;
  MKI_BLOCK_DIMLARGE_PTR m_pData;
  MKI_BLOCK_T005_PTR m_pTemp;

  __forceinline MKI_POINT_REF Point1( void ) { return( m_pPoint1->m_pData[0] ); }
  __forceinline MKI_POINT_REF Point2( void ) { return( m_pPoint2->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointLine1( void ) { return( m_pLine1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointLine2( void ) { return( m_pLine2->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointNumber1( void ) { return( m_pNumber1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointNumber2( void ) { return( m_pNumber2->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointExtension1( void ) { return( m_pExtension1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointExtension2( void ) { return( m_pExtension2->m_pData[0] ); }
  __forceinline LPWSTR Text1( void ) { return( m_pText1->m_pText ); }
  __forceinline LPWSTR Text2( void ) { return( m_pText2->m_pText ); }
  __forceinline LPWSTR Text3( void ) { return( m_pText3->m_pText ); }
  __forceinline LPWSTR Tolerance1( void ) { return( m_pTolerance1->m_pText ); }
  __forceinline LPWSTR Tolerance2( void ) { return( m_pTolerance2->m_pText ); }
  __forceinline MKI_DIMLARGE_REF DimLarge( void ) { return( m_pData->m_cData ); }
  __forceinline MKI_BLOCK_T005_REF Temp( void ) { return( *m_pTemp ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_DGENERIC5( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_DGENERIC5 )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_DDISTANCE
{
  public:

  MKI_INTERFACE_MEMBER 
  MKI_PARSE_OBJ_DDISTANCE( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_POINT_PTR m_pStart,
                      m_pEnd,
                      m_pLine1,
                      m_pLine2,
                      m_pLine3,
                      m_pNumber1,
                      m_pNumber2,
                      m_pExtension1,
                      m_pExtension2;
  MKI_BLOCK_TEXT_PTR m_pText1,
                     m_pText2,
                     m_pText3,
                     m_pTolerance1,
                     m_pTolerance2;
  MKI_BLOCK_DIMLARGE_PTR m_pData;
  MKI_BLOCK_T005_PTR m_pTemp;

  //------------------------------------------------------------------

  __forceinline MKI_POINT_REF PointStart( void ) { return( m_pStart->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointEnd( void ) { return( m_pEnd->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointLine1( void ) { return( m_pLine1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointLine2( void ) { return( m_pLine2->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointLine3( void ) { return( m_pLine3->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointNumber1( void ) { return( m_pNumber1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointNumber2( void ) { return( m_pNumber2->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointExtension1( void ) { return( m_pExtension1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointExtension2( void ) { return( m_pExtension2->m_pData[0] ); }
  __forceinline LPWSTR Text1( void ) { return( m_pText1->m_pText ); }
  __forceinline LPWSTR Text2( void ) { return( m_pText2->m_pText ); }
  __forceinline LPWSTR Text3( void ) { return( m_pText3->m_pText ); }
  __forceinline LPWSTR Tolerance1( void ) { return( m_pTolerance1->m_pText ); }
  __forceinline LPWSTR Tolerance2( void ) { return( m_pTolerance2->m_pText ); }
  __forceinline MKI_DIMLARGE_REF DimLarge( void ) { return( m_pData->m_cData ); }
  __forceinline MKI_BLOCK_T005_REF Temp( void ) { return( *m_pTemp ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_DDISTANCE( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_DDISTANCE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_DRADIUS
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_PARSE_OBJ_DRADIUS( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_POINT_PTR m_pStart,
                      m_pEnd,
                      m_pLine1,
                      m_pLine2,
                      m_pNumber1,
                      m_pNumber2,
                      m_pExtension1,
                      m_pExtension2;
  MKI_BLOCK_TEXT_PTR m_pText1,
                     m_pText2,
                     m_pText3,
                     m_pTolerance1,
                     m_pTolerance2;
  MKI_BLOCK_DIMLARGE_PTR m_pData;
  MKI_BLOCK_T005_PTR m_pTemp;

  __forceinline MKI_POINT_REF PointStart( void ) { return( m_pStart->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointEnd( void ) { return( m_pEnd->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointLine1( void ) { return( m_pLine1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointLine2( void ) { return( m_pLine2->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointNumber1( void ) { return( m_pNumber1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointNumber2( void ) { return( m_pNumber2->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointExtension1( void ) { return( m_pExtension1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointExtension2( void ) { return( m_pExtension2->m_pData[0] ); }
  __forceinline LPWSTR Text1( void ) { return( m_pText1->m_pText ); }
  __forceinline LPWSTR Text2( void ) { return( m_pText2->m_pText ); }
  __forceinline LPWSTR Text3( void ) { return( m_pText3->m_pText ); }
  __forceinline LPWSTR Tolerance1( void ) { return( m_pTolerance1->m_pText ); }
  __forceinline LPWSTR Tolerance2( void ) { return( m_pTolerance2->m_pText ); }
  __forceinline MKI_DIMLARGE_REF DimLarge( void ) { return( m_pData->m_cData ); }
  __forceinline MKI_BLOCK_T005_REF Temp( void ) { return( *m_pTemp ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_DRADIUS( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_DRADIUS )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_DDIAMETER
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_PARSE_OBJ_DDIAMETER( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_POINT_PTR m_pCenter,
                      m_pRadius,
                      m_pLine1,
                      m_pLine2,
                      m_pNumber1,
                      m_pNumber2,
                      m_pExtension1,
                      m_pExtension2;
  MKI_BLOCK_TEXT_PTR m_pText1,
                     m_pText2,
                     m_pText3,
                     m_pTolerance1,
                     m_pTolerance2;
  MKI_BLOCK_DIMLARGE_PTR m_pData;
  MKI_BLOCK_T005_PTR m_pTemp;

  __forceinline MKI_POINT_REF PointCenter( void ) { return( m_pCenter->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointRadius( void ) { return( m_pRadius->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointLine1( void ) { return( m_pLine1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointLine2( void ) { return( m_pLine2->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointNumber1( void ) { return( m_pNumber1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointNumber2( void ) { return( m_pNumber2->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointExtension1( void ) { return( m_pExtension1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointExtension2( void ) { return( m_pExtension2->m_pData[0] ); }
  __forceinline LPWSTR Text1( void ) { return( m_pText1->m_pText ); }
  __forceinline LPWSTR Text2( void ) { return( m_pText2->m_pText ); }
  __forceinline LPWSTR Text3( void ) { return( m_pText3->m_pText ); }
  __forceinline LPWSTR Tolerance1( void ) { return( m_pTolerance1->m_pText ); }
  __forceinline LPWSTR Tolerance2( void ) { return( m_pTolerance2->m_pText ); }
  __forceinline MKI_DIMLARGE_REF DimLarge( void ) { return( m_pData->m_cData ); }
  __forceinline MKI_BLOCK_T005_REF Temp( void ) { return( *m_pTemp ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_DDIAMETER( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_DDIAMETER )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_DGENERIC6
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_PARSE_OBJ_DGENERIC6( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_POINT_PTR m_pPoint1,
                      m_pPoint2,
                      m_pLine,
                      m_pNumber1,
                      m_pNumber2,
                      m_pExtension1,
                      m_pExtension2;
  MKI_BLOCK_TEXT_PTR m_pText1,
                     m_pText2,
                     m_pText3,
                     m_pTolerance1,
                     m_pTolerance2;
  MKI_BLOCK_DIMLARGE_PTR m_pData;
  MKI_BLOCK_T006_PTR m_pTemp;

  __forceinline MKI_POINT_REF Point1( void ) { return( m_pPoint1->m_pData[0] ); }
  __forceinline MKI_POINT_REF Point2( void ) { return( m_pPoint2->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointLine( void ) { return( m_pLine->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointNumber1( void ) { return( m_pNumber1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointNumber2( void ) { return( m_pNumber2->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointExtension1( void ) { return( m_pExtension1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointExtension2( void ) { return( m_pExtension2->m_pData[0] ); }
  __forceinline LPWSTR Text1( void ) { return( m_pText1->m_pText ); }
  __forceinline LPWSTR Text2( void ) { return( m_pText2->m_pText ); }
  __forceinline LPWSTR Text3( void ) { return( m_pText3->m_pText ); }
  __forceinline LPWSTR Tolerance1( void ) { return( m_pTolerance1->m_pText ); }
  __forceinline LPWSTR Tolerance2( void ) { return( m_pTolerance2->m_pText ); }
  __forceinline MKI_DIMLARGE_REF DimLarge( void ) { return( m_pData->m_cData ); }
  __forceinline MKI_BLOCK_T006_REF Temp( void ) { return( *m_pTemp ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_DGENERIC6( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_DGENERIC6 )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_DANGLE
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_PARSE_OBJ_DANGLE( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_POINT_PTR m_pStart1,
                      m_pEnd1,
                      m_pStart2,
                      m_pEnd2,
                      m_pLine,
                      m_pNumber1,
                      m_pNumber2,
                      m_pExtension1,
                      m_pExtension2;
  MKI_BLOCK_TEXT_PTR m_pText1,
                     m_pText2,
                     m_pText3,
                     m_pTolerance1,
                     m_pTolerance2;
  MKI_BLOCK_DIMLARGE_PTR m_pData;
  MKI_BLOCK_T006_PTR m_pTemp;

  __forceinline MKI_POINT_REF PointStart1( void ) { return( m_pStart1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointEnd1( void ) { return( m_pEnd1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointStart2( void ) { return( m_pStart2->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointEnd2( void ) { return( m_pEnd2->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointLine( void ) { return( m_pLine->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointNumber1( void ) { return( m_pNumber1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointNumber2( void ) { return( m_pNumber2->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointExtension1( void ) { return( m_pExtension1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointExtension2( void ) { return( m_pExtension2->m_pData[0] ); }
  __forceinline LPWSTR Text1( void ) { return( m_pText1->m_pText ); }
  __forceinline LPWSTR Text2( void ) { return( m_pText2->m_pText ); }
  __forceinline LPWSTR Text3( void ) { return( m_pText3->m_pText ); }
  __forceinline LPWSTR Tolerance1( void ) { return( m_pTolerance1->m_pText ); }
  __forceinline LPWSTR Tolerance2( void ) { return( m_pTolerance2->m_pText ); }
  __forceinline MKI_DIMLARGE_REF DimLarge( void ) { return( m_pData->m_cData ); }
  __forceinline MKI_BLOCK_T006_REF Temp( void ) { return( *m_pTemp ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_DANGLE( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_DANGLE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_DARCLENGTH
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_PARSE_OBJ_DARCLENGTH( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_POINT_PTR m_pCenter,
                      m_pRadius,
                      m_pAngle1,
                      m_pAngle2;
  MKI_BLOCK_DOUBLE_PTR m_pOrient;
  MKI_BLOCK_POINT_PTR m_pLine,
                      m_pNumber1,
                      m_pNumber2,
                      m_pExtension1,
                      m_pExtension2;
  MKI_BLOCK_TEXT_PTR m_pText1,
                     m_pText2,
                     m_pText3,
                     m_pTolerance1,
                     m_pTolerance2;
  MKI_BLOCK_DIMLARGE_PTR m_pData;
  MKI_BLOCK_T006_PTR m_pTemp;

  __forceinline MKI_POINT_REF PointCenter( void ) { return( m_pCenter->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointRadius( void ) { return( m_pRadius->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointAngle1( void ) { return( m_pAngle1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointAngle2( void ) { return( m_pAngle2->m_pData[0] ); }
  __forceinline double& Orient( void ) { return( m_pOrient->m_pData[0] ); }
  __forceinline bool IsOrientPositive( void ) const { return( m_pOrient->m_pData[0] >= 0.0 ); }
  __forceinline bool IsOrientNegative( void ) const { return( m_pOrient->m_pData[0] < 0.0 ); }
  __forceinline MKI_POINT_REF PointLine( void ) { return( m_pLine->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointNumber1( void ) { return( m_pNumber1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointNumber2( void ) { return( m_pNumber2->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointExtension1( void ) { return( m_pExtension1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointExtension2( void ) { return( m_pExtension2->m_pData[0] ); }
  __forceinline LPWSTR Text1( void ) { return( m_pText1->m_pText ); }
  __forceinline LPWSTR Text2( void ) { return( m_pText2->m_pText ); }
  __forceinline LPWSTR Text3( void ) { return( m_pText3->m_pText ); }
  __forceinline LPWSTR Tolerance1( void ) { return( m_pTolerance1->m_pText ); }
  __forceinline LPWSTR Tolerance2( void ) { return( m_pTolerance2->m_pText ); }
  __forceinline MKI_DIMLARGE_REF DimLarge( void ) { return( m_pData->m_cData ); }
  __forceinline MKI_BLOCK_T006_REF Temp( void ) { return( *m_pTemp ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_DARCLENGTH( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_DARCLENGTH )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_DCOORDINATE
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_PARSE_OBJ_DCOORDINATE( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_POINT_PTR m_pStart,
                      m_pSplit,
                      m_pNumber1,
                      m_pNumber2;
  MKI_BLOCK_TEXT_PTR m_pText1,
                     m_pText2,
                     m_pText3,
                     m_pText4,
                     m_pText5,
                     m_pTolerance1,
                     m_pTolerance2;
  MKI_BLOCK_DIMSMALL_PTR m_pData;
  MKI_BLOCK_TEXTREFERENCE_PTR m_pTextRef;
  MKI_BLOCK_T007_PTR m_pTemp;

  __forceinline MKI_POINT_REF PointStart( void ) { return( m_pStart->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointSplit( void ) { return( m_pSplit->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointNumber1( void ) { return( m_pNumber1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointNumber2( void ) { return( m_pNumber2->m_pData[0] ); }
  __forceinline LPWSTR Text1( void ) { return( m_pText1->m_pText ); }
  __forceinline LPWSTR Text2( void ) { return( m_pText2->m_pText ); }
  __forceinline LPWSTR Text3( void ) { return( m_pText3->m_pText ); }
  __forceinline LPWSTR Text4( void ) { return( m_pText4->m_pText ); }
  __forceinline LPWSTR Text5( void ) { return( m_pText5->m_pText ); }
  __forceinline LPWSTR Tolerance1( void ) { return( m_pTolerance1->m_pText ); }
  __forceinline LPWSTR Tolerance2( void ) { return( m_pTolerance2->m_pText ); }
  __forceinline MKI_DIMSMALL_REF DimSmall( void ) { return( m_pData->m_cData ); }
  __forceinline MKI_TEXTREFERENCE_REF TextReference( void ) { return( m_pTextRef->m_cData ); }
  __forceinline MKI_BLOCK_T007_REF Temp( void ) { return( *m_pTemp ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_DCOORDINATE( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_DCOORDINATE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_DGENERIC7
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_PARSE_OBJ_DGENERIC7( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_POINT_PTR m_pNumber1,
                      m_pNumber2;
  MKI_BLOCK_DOUBLE_PTR m_pValue;
  MKI_BLOCK_TEXT_PTR m_pText1,
                     m_pText2,
                     m_pText3,
                     m_pTolerance1,
                     m_pTolerance2;
  MKI_BLOCK_DIMSMALL_PTR m_pData;
  MKI_BLOCK_T007_PTR m_pTemp;

  __forceinline MKI_POINT_REF PointNumber1( void ) { return( m_pNumber1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointNumber2( void ) { return( m_pNumber2->m_pData[0] ); }
  __forceinline double& Value( void ) { return( m_pValue->m_pData[0] ); }
  __forceinline LPWSTR Text1( void ) { return( m_pText1->m_pText ); }
  __forceinline LPWSTR Text2( void ) { return( m_pText2->m_pText ); }
  __forceinline LPWSTR Text3( void ) { return( m_pText3->m_pText ); }
  __forceinline LPWSTR Tolerance1( void ) { return( m_pTolerance1->m_pText ); }
  __forceinline LPWSTR Tolerance2( void ) { return( m_pTolerance2->m_pText ); }
  __forceinline MKI_DIMSMALL_REF DimSmall( void ) { return( m_pData->m_cData ); }
  __forceinline MKI_BLOCK_T007_REF Temp( void ) { return( *m_pTemp ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_DGENERIC7( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_DGENERIC7 )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_DAREA
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_PARSE_OBJ_DAREA( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_POINT_PTR m_pNumber1,
                      m_pNumber2;
  MKI_BLOCK_DOUBLE_PTR m_pArea;
  MKI_BLOCK_TEXT_PTR m_pText1,
                     m_pText2,
                     m_pText3,
                     m_pTolerance1,
                     m_pTolerance2;
  MKI_BLOCK_DIMSMALL_PTR m_pData;
  MKI_BLOCK_T007_PTR m_pTemp;

  __forceinline MKI_POINT_REF PointNumber1( void ) { return( m_pNumber1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointNumber2( void ) { return( m_pNumber2->m_pData[0] ); }
  __forceinline double& ValueArea( void ) { return( m_pArea->m_pData[0] ); }
  __forceinline LPWSTR Text1( void ) { return( m_pText1->m_pText ); }
  __forceinline LPWSTR Text2( void ) { return( m_pText2->m_pText ); }
  __forceinline LPWSTR Text3( void ) { return( m_pText3->m_pText ); }
  __forceinline LPWSTR Tolerance1( void ) { return( m_pTolerance1->m_pText ); }
  __forceinline LPWSTR Tolerance2( void ) { return( m_pTolerance2->m_pText ); }
  __forceinline MKI_DIMSMALL_REF DimSmall( void ) { return( m_pData->m_cData ); }
  __forceinline MKI_BLOCK_T007_REF Temp( void ) { return( *m_pTemp ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_DAREA( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_DAREA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_DPERIMETER
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_PARSE_OBJ_DPERIMETER( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_POINT_PTR m_pNumber1,
                      m_pNumber2;
  MKI_BLOCK_DOUBLE_PTR m_pPerimeter;
  MKI_BLOCK_TEXT_PTR m_pText1,
                     m_pText2,
                     m_pText3,
                     m_pTolerance1,
                     m_pTolerance2;
  MKI_BLOCK_DIMSMALL_PTR m_pData;
  MKI_BLOCK_T007_PTR m_pTemp;

  __forceinline MKI_POINT_REF PointNumber1( void ) { return( m_pNumber1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointNumber2( void ) { return( m_pNumber2->m_pData[0] ); }
  __forceinline double& ValuePerimeter( void ) { return( m_pPerimeter->m_pData[0] ); }
  __forceinline LPWSTR Text1( void ) { return( m_pText1->m_pText ); }
  __forceinline LPWSTR Text2( void ) { return( m_pText2->m_pText ); }
  __forceinline LPWSTR Text3( void ) { return( m_pText3->m_pText ); }
  __forceinline LPWSTR Tolerance1( void ) { return( m_pTolerance1->m_pText ); }
  __forceinline LPWSTR Tolerance2( void ) { return( m_pTolerance2->m_pText ); }
  __forceinline MKI_DIMSMALL_REF DimSmall( void ) { return( m_pData->m_cData ); }
  __forceinline MKI_BLOCK_T007_REF Temp( void ) { return( *m_pTemp ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_DPERIMETER( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_DPERIMETER )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_DTEXT
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_PARSE_OBJ_DTEXT( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_POINT_PTR m_pNumber1,
                      m_pNumber2;
  MKI_BLOCK_TEXT_PTR m_pText,
                     m_pTolerance1,
                     m_pTolerance2;
  MKI_BLOCK_DIMSMALL_PTR m_pData;
  MKI_BLOCK_T007_PTR m_pTemp;

  __forceinline MKI_POINT_REF PointNumber1( void ) { return( m_pNumber1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointNumber2( void ) { return( m_pNumber2->m_pData[0] ); }
  __forceinline LPWSTR Text( void ) { return( m_pText->m_pText ); }
  __forceinline LPWSTR Tolerance1( void ) { return( m_pTolerance1->m_pText ); }
  __forceinline LPWSTR Tolerance2( void ) { return( m_pTolerance2->m_pText ); }
  __forceinline MKI_DIMSMALL_REF DimSmall( void ) { return( m_pData->m_cData ); }
  __forceinline MKI_BLOCK_T007_REF Temp( void ) { return( *m_pTemp ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_DTEXT( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_DTEXT )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_TEXTSTANDARD
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_PARSE_OBJ_TEXTSTANDARD( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_TEXT_PTR m_pText;
  MKI_BLOCK_TEXTSTANDARD_PTR m_pTextStd;

  __forceinline LPWSTR Text( void ) { return( m_pText->m_pText ); }
  __forceinline MKI_TEXTSTANDARD_REF TextStandard( void ) { return( m_pTextStd->m_cData ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_TEXTSTANDARD( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_TEXTSTANDARD )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_TEXTFRAME
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_PARSE_OBJ_TEXTFRAME( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_POINT_PTR m_pStart,
                      m_pVector1,
                      m_pVector2;
  MKI_BLOCK_TEXT_PTR m_pText;
  MKI_BLOCK_TEXTFRAME_PTR m_pTextFrm;

  __forceinline MKI_POINT_REF PointStart( void ) { return( m_pStart->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointVector1( void ) { return( m_pVector1->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointVector2( void ) { return( m_pVector2->m_pData[0] ); }
  __forceinline LPWSTR Text( void ) { return( m_pText->m_pText ); }
  __forceinline MKI_TEXTFRAME_REF TextFrame( void ) { return( m_pTextFrm->m_cData ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_TEXTFRAME( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_TEXTFRAME )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_TEXTREFERENCE
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_PARSE_OBJ_TEXTREFERENCE( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_POINT_PTR m_pStart,
                      m_pSplit;
  MKI_BLOCK_TEXT_PTR m_pText;
  MKI_BLOCK_TEXTSTANDARD_PTR m_pTextStd;
  MKI_BLOCK_TEXTREFERENCE_PTR m_pTextRef;
  MKI_BLOCK_T007_PTR m_pTemp;

  __forceinline MKI_POINT_REF PointStart( void ) { return( m_pStart->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointSplit( void ) { return( m_pSplit->m_pData[0] ); }
  __forceinline LPWSTR Text( void ) { return( m_pText->m_pText ); }
  __forceinline MKI_TEXTSTANDARD_REF TextStandard( void ) { return( m_pTextStd->m_cData ); }
  __forceinline MKI_TEXTREFERENCE_REF TextReference( void ) { return( m_pTextRef->m_cData ); }
  __forceinline MKI_BLOCK_T007_REF Temp( void ) { return( *m_pTemp ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_TEXTREFERENCE( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_TEXTREFERENCE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_HYPERLINK
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_PARSE_OBJ_HYPERLINK( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_TEXT_PTR m_pURL;
  MKI_BLOCK_PTR m_pCurve;

  __forceinline LPWSTR LinkURL( void ) { return( m_pURL->m_pText ); }
  __forceinline MKI_BLOCK_PTR CurveData( void ) { return( m_pCurve ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_HYPERLINK( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_HYPERLINK )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_COMMENT
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_PARSE_OBJ_COMMENT( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_POINT_PTR m_pRef;
  MKI_BLOCK_TEXT_PTR m_pText;
  MKI_BLOCK_COMMENTDATA_PTR m_pData;
  MKI_BLOCK_PTR m_pCurve;

  __forceinline MKI_POINT_REF PointRef( void ) { return( m_pRef->m_pData[0] ); }
  __forceinline LPWSTR Text( void ) { return( m_pText->m_pText ); }
  __forceinline MKI_COMMENTDATA_REF CommentData( void ) { return( m_pData->m_cData ); }
  __forceinline MKI_BLOCK_PTR CurveData( void ) { return( m_pCurve ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_COMMENT( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_COMMENT )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PARSE_OBJ_BITMAPREF
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_PARSE_OBJ_BITMAPREF( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_BLOCK_BITMAPREF_PTR m_pData;

  __forceinline MKI_BITMAPREF_REF BitmapRef( void ) { return( m_pData->m_cData ); }

  //------------------------------------------------------------------

  private:

  // Disable default constructor
  MKI_PARSE_OBJ_BITMAPREF( void ) = delete;
};

MKI_CLASS_TYPES( PARSE_OBJ_BITMAPREF )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MULTI_XPROPERTY : public MKI_MULTI_BASE
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  Fill( MKI_CONST_XPROPERTY_REF f_rXProperty );

  MKI_INTERFACE_MEMBER void
  Apply( MKI_XPROPERTY_REF f_rXProperty, bool f_fCopyAll ) const;

  //------------------------------------------------------------------

  MKI_MULTI_BASE m_cFlagBase;

  MKI_XPROPERTY m_cData;
};

MKI_CLASS_TYPES( MULTI_XPROPERTY )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MULTI_TEXTSTANDARD : public MKI_MULTI_BASE
{
  public:

  enum
  {
    USE_FONT         = 0x00000001,
    USE_CHARDISTANCE = 0x00000002,
    USE_TABDISTANCE  = 0x00000004,
    USE_LINEDISTANCE = 0x00000008,
    USE_HEIGHT       = 0x00000010,
    USE_COMPRESSION  = 0x00000020,
    USE_ROTATION     = 0x00000040,
    USE_DISTORTION   = 0x00000080,
    USE_XPOS         = 0x00000100,
    USE_YPOS         = 0x00000200,
    USE_TEXTMODE     = 0x00000400,
    USE_ORDER        = 0x00000800,
    USE_ADAPTANGLE   = 0x00001000,
    USE_VERTICAL     = 0x00002000,
    USE_SYSTEM       = 0x00004000,
    USE_ADAPTSCALE   = 0x00008000,

    USE_MATRIX       = USE_HEIGHT | USE_COMPRESSION | USE_ROTATION | USE_DISTORTION | USE_XPOS | USE_YPOS,
  };

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  Fill( MKI_CONST_TEXTSTANDARD_REF f_rTextStandard );

  MKI_INTERFACE_MEMBER void
  Apply( MKI_TEXTSTANDARD_REF f_rTextStandard, bool f_fCopyAll ) const;

  //------------------------------------------------------------------

  MKI_FONTDEF m_cTextFont;
  double      m_dCharDistance,
              m_dTabDistance,
              m_dLineDistance,
              m_dHeight,
              m_dCompression,
              m_dRotation,
              m_dDistortion;
  MKI_POINT   m_cPointUnits;
  __int32     m_nTextMode,
              m_nTextVertical,
              m_nTextOrder,
              m_nDisplayMode,
              m_nSystem;
};

MKI_CLASS_TYPES( MULTI_TEXTSTANDARD )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MULTI_TEXTFRAME : public MKI_MULTI_BASE
{
  public:

  enum
  {
    USE_FONT         = 0x00000001,
    USE_CHARDISTANCE = 0x00000002,
    USE_TABDISTANCE  = 0x00000004,
    USE_LINEDISTANCE = 0x00000008,
    USE_SIZE         = 0x00000010,
    USE_TEXTMODE     = 0x00000020,
    USE_ORDER        = 0x00000040,
    USE_VERTICAL     = 0x00000080,
    USE_SYSTEM       = 0x00000100,
  };

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  Fill( MKI_CONST_TEXTFRAME_REF f_rTextFrame );

  MKI_INTERFACE_MEMBER void
  Apply( MKI_TEXTFRAME_REF f_rTextFrame, bool f_fCopyAll ) const;

  //------------------------------------------------------------------

  MKI_TEXTFRAME m_cData;
};

MKI_CLASS_TYPES( MULTI_TEXTFRAME )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MULTI_TEXTREFERENCE : public MKI_MULTI_BASE
{
  public:

  enum
  {
    USE_ARROWFORM   = 0x00000001,
    USE_ARROWMODE   = 0x00000002,
    USE_FRAMEFORM   = 0x00000004,
    USE_FRAMEOFFSET = 0x00000008,
  };

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  Fill( MKI_CONST_TEXTREFERENCE_REF f_rTextReference );

  MKI_INTERFACE_MEMBER void
  Apply( MKI_TEXTREFERENCE_REF f_rTextReference, bool f_fCopyAll ) const;

  //------------------------------------------------------------------

  MKI_TEXTREFERENCE m_cData;
};

MKI_CLASS_TYPES( MULTI_TEXTREFERENCE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MULTI_ZIGZAG : public MKI_MULTI_BASE
{
  public:

  enum
  {
    USE_INDENT = 0x00000001,
  };

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  Fill( const double f_dIndent );

  MKI_INTERFACE_MEMBER void
  Apply( double& f_rdIndent, bool f_fCopyAll ) const;

  //------------------------------------------------------------------

  double  m_dIndent;
};

MKI_CLASS_TYPES( MULTI_ZIGZAG )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MULTI_DIMPARAM : public MKI_MULTI_BASE
{
  public:

  enum
  {
    USE_FONT         = 0x00000001,
    USE_SIZE1        = 0x00000002,
    USE_SIZE2        = 0x00000004,
    USE_LENGTHMODE   = 0x00000008,
    USE_SYSTEM       = 0x00000010,
    USE_CHARDISTANCE = 0x00000020,
    USE_TABDISTANCE  = 0x00000040,
    USE_TEXTMODE     = 0x00000080,
    USE_ORDER        = 0x00000100,
    USE_VERTICAL     = 0x00000200,
    USE_ANGLEMODE    = 0x00000400,
  };

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  Fill( MKI_CONST_DIMSMALL_REF f_rDimSmall, bool f_fIsAngle );

  MKI_INTERFACE_MEMBER void
  Apply( MKI_DIMSMALL_REF f_rDimSmall, bool f_fIsAngle, bool f_fCopyAll ) const;

  //------------------------------------------------------------------

  MKI_DIMPARAM m_cData;
};

MKI_CLASS_TYPES( MULTI_DIMPARAM )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MULTI_DIMLINE : public MKI_MULTI_BASE
{
  public:

  enum
  {
    USE_STARTFORM = 0x00000001,
    USE_STARTMODE = 0x00000002,
    USE_ENDFORM   = 0x00000004,
    USE_ENDMODE   = 0x00000008,
  };

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  Fill( MKI_CONST_DIMLINE_REF f_rDimLine );

  MKI_INTERFACE_MEMBER void
  Apply( MKI_DIMLINE_REF f_rDimLine, bool f_fCopyAll ) const;

  //------------------------------------------------------------------

  MKI_DIMLINE m_cData;
};

MKI_CLASS_TYPES( MULTI_DIMLINE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MULTI_DIMLARGE : public MKI_MULTI_BASE
{
  public:

  enum
  {
    USE_NUMREFRESH   = 0x00000001,
    USE_NUMCENTERED  = 0x00000002,
    USE_NUMPLACING   = 0x00000004,
    USE_NUMROTATE    = 0x00000008,
    USE_STARTFORM    = 0x00000010,
    USE_STARTMODE    = 0x00000020,
    USE_ENDFORM      = 0x00000040,
    USE_ENDMODE      = 0x00000080,
    USE_EXTSTART     = 0x00000100,
    USE_EXTEND       = 0x00000200,
    USE_LINEDISPLAY  = 0x00000400,
    USE_LINEORIENT   = 0x00000800,
    USE_LINEDIMTYPE  = 0x00001000,
    USE_LINEDISTMODE = 0x00002000,
    USE_LINEDISTANCE = 0x00004000,
    USE_STARTOFFSET  = 0x00008000,
    USE_ENDOFFSET    = 0x00010000,
    USE_RADIUSEXTEND = 0x00020000,
  };

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  Fill( MKI_CONST_DIMLARGE_REF f_rDimLarge );

  MKI_INTERFACE_MEMBER void
  Apply( MKI_DIMLARGE_REF f_rDimLarge, bool f_fCopyAll ) const;

  //------------------------------------------------------------------

  MKI_DIMLARGE m_cData;
};

MKI_CLASS_TYPES( MULTI_DIMLARGE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MULTI_DIMSMALL : public MKI_MULTI_BASE
{
  public:

  enum
  {
    USE_NUMREFRESH = 0x00010000,
    USE_NUMROTATE  = 0x00020000,
  };

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  Fill( MKI_CONST_DIMSMALL_REF f_rDimSmall );

  MKI_INTERFACE_MEMBER void
  Apply( MKI_DIMSMALL_REF f_rDimSmall, bool f_fCopyAll ) const;

  //------------------------------------------------------------------

  MKI_DIMSMALL m_cData;
};

MKI_CLASS_TYPES( MULTI_DIMSMALL )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MULTI_COMMENT : public MKI_MULTI_BASE
{
  public:

  enum
  {
    USE_SYMBOL   = 0x00000001,
    USE_PROGRESS = 0x00000002,
  };

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  Fill( MKI_CONST_COMMENTDATA_REF f_rComment );

  MKI_INTERFACE_MEMBER void
  Apply( MKI_COMMENTDATA_REF f_rComment, bool f_fCopyAll ) const;

  //------------------------------------------------------------------

  __int32 m_nSymbol,
          m_nProgress;
};

MKI_CLASS_TYPES( MULTI_COMMENT )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MULTI_BITMAPREF : public MKI_MULTI_BASE
{
  public:

  enum
  {
    USE_NAME       = 0x00000001,
    USE_XSCALE     = 0x00000002,
    USE_YSCALE     = 0x00000004,
    USE_ROTATION   = 0x00000008,
    USE_DISTORTION = 0x00000010,
    USE_XPOS       = 0x00000020,
    USE_YPOS       = 0x00000040,
    USE_GAMMA      = 0x00000080,
    USE_BRIGHTNESS = 0x00000100,
    USE_CONTRAST   = 0x00000200,
    USE_INTENSITY  = 0x00000400,
    USE_MASK       = 0x00000800,
    USE_MASKCOLOR  = 0x00001000,
    USE_FITMODE    = 0x00002000,
    USE_ALIGNMODE  = 0x00004000,

    USE_MATRIX     = USE_XSCALE | USE_YSCALE | USE_ROTATION | USE_DISTORTION | USE_XPOS | USE_YPOS,
  };

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  Fill( MKI_CONST_BITMAPREF_REF f_rBitmapRef );

  MKI_INTERFACE_MEMBER bool 
  Apply( MKI_BITMAPREF_REF f_rBitmapRef, bool f_fCopyAll ) const;

  MKI_INTERFACE_MEMBER bool
  GetBitmapInfo( MKI_DUMMYSTRW& f_rString ) const;

  //------------------------------------------------------------------

  MKI_FILEREFERENCE m_szBitmapName;

  MKI_VECTOR m_cScaling;
  double     m_dRotation,
             m_dDistortion;
  MKI_POINT  m_cPointUnits;

  MKI_BITMAP_EFFECT m_cEffect;
  __int32           m_nFitMode,
                    m_nAlignMode;

  MKI_INTVECTOR m_cSize;       // For information only, do not modify!
  __int32       m_nColorBits;  // For information only, do not modify!
  MKI_VECTOR    m_cResolution; // For information only, do not modify!
  bool          m_fIsLoaded,   // For information only, do not modify!
                m_fHasAlpha,   // For information only, do not modify!
                m_fIsMono,     // For information only, do not modify!
                m_fBaseCMYK;   // For information only, do not modify!
};

MKI_CLASS_TYPES( MULTI_BITMAPREF )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MULTI_INSTANCE : public MKI_MULTI_BASE
{
  public:

  enum
  {
    USE_LIBRARYNAME  = 0x00000001,
    USE_BLOCKNAME    = 0x00000002,
    USE_XSCALE       = 0x00000004,
    USE_YSCALE       = 0x00000008,
    USE_ROTATION     = 0x00000010,
    USE_DISTORTION   = 0x00000020,
    USE_XPOS         = 0x00000040,
    USE_YPOS         = 0x00000080,
    USE_SCALEDWIDTH  = 0x00000100,
    USE_CONFINED     = 0x00000200,
    USE_NOGEO        = 0x00000400,
    USE_ADAPTANGLE   = 0x00000800,
    USE_SCALEDLENGTH = 0x00001000,
    USE_EMBEDDED     = 0x00002000,
    USE_COMMENT      = 0x00004000,
    USE_ADAPTSCALE   = 0x00008000,
    USE_NOHIDE       = 0x00010000,

    USE_NAMES        = USE_LIBRARYNAME | USE_BLOCKNAME,
    USE_MATRIX       = USE_XSCALE | USE_YSCALE | USE_ROTATION | USE_DISTORTION | USE_XPOS | USE_YPOS,
  };

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  Fill( MKI_CONST_ENTITY_INSTANCE_PTR f_pInst );

  MKI_INTERFACE_MEMBER bool
  Apply( MKI_ENTITY_INSTANCE_PTR f_pInst, bool f_fCopyAll ) const;

  //------------------------------------------------------------------

  MKI_LIBRARYNAME m_szLibraryName;
  MKI_BLOCKNAME   m_szBlockName;
  MKI_COMMENTLONG m_szComment;

  MKI_VECTOR m_cScaling;
  double     m_dRotation,
             m_dDistortion;
  MKI_POINT  m_cPointUnits;
  __int32    m_nDisplayMode;
  bool       m_fInitAttrib,
             m_fInitEmbedded;
};

MKI_CLASS_TYPES( MULTI_INSTANCE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MULTI_CLIP : public MKI_MULTI_BASE
{
  public:

  enum
  {
    USE_XSCALE       = 0x00000001,
    USE_YSCALE       = 0x00000002,
    USE_ROTATION     = 0x00000004,
    USE_DISTORTION   = 0x00000008,
    USE_XPOS         = 0x00000010,
    USE_YPOS         = 0x00000020,
    USE_SCALEDWIDTH  = 0x00000040,
    USE_NOPOINTS     = 0x00000080,
    USE_NOSNAP       = 0x00000100,
    USE_HIDELAYERS   = 0x00000200,
    USE_SCALEDLENGTH = 0x00000400,
    USE_NOHIDE       = 0x00000800,

    USE_MATRIX       = USE_XSCALE | USE_YSCALE | USE_ROTATION | USE_DISTORTION | USE_XPOS | USE_YPOS,
  };

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  Fill( MKI_CONST_ENTITY_CLIP_PTR f_pClip );

  MKI_INTERFACE_MEMBER void
  Apply( MKI_ENTITY_CLIP_PTR f_pClip, bool f_fCopyAll ) const;

  //------------------------------------------------------------------

  MKI_VECTOR m_cScaling;
  double     m_dRotation,
             m_dDistortion;
  MKI_POINT  m_cPointUnits;
  __int32    m_nDisplayMode;

  MKI_HIDELAYERS m_afHideLayers;
};

MKI_CLASS_TYPES( MULTI_CLIP )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MULTI_GROUP : public MKI_MULTI_BASE
{
  public:

  enum
  {
    USE_COMMENT = 0x00000001,
  };

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  Fill( MKI_CONST_ENTITY_GROUP_PTR f_pGroup );

  MKI_INTERFACE_MEMBER void
  Apply( MKI_ENTITY_GROUP_PTR f_pGroup, bool f_fCopyAll ) const;

  //------------------------------------------------------------------

  MKI_COMMENTLONG m_szComment;
};

MKI_CLASS_TYPES( MULTI_GROUP )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MULTI_REF : public MKI_MULTI_BASE
{
  public:

  enum
  {
    USE_NAME         = 0x00000001,  
    USE_XSCALE       = 0x00000002,  
    USE_YSCALE       = 0x00000004,  
    USE_ROTATION     = 0x00000008,  
    USE_DISTORTION   = 0x00000010,  
    USE_XPOS         = 0x00000020,  
    USE_YPOS         = 0x00000040,  
    USE_SCALEDWIDTH  = 0x00000080,  
    USE_CONFINED     = 0x00000100,  
    USE_NOGEO        = 0x00000200,  
    USE_ADAPTANGLE   = 0x00000400,  
    USE_SCALEDLENGTH = 0x00000800,
    USE_ADAPTSCALE   = 0x00001000,  
    USE_NOHIDE       = 0x00002000,  
    USE_HIDELAYERS   = 0x00004000,  

    USE_MATRIX       = USE_XSCALE | USE_YSCALE | USE_ROTATION | USE_DISTORTION | USE_XPOS | USE_YPOS,
  };

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  Fill( MKI_CONST_ENTITY_REF_PTR f_pRef );

  MKI_INTERFACE_MEMBER bool
  Apply( MKI_ENTITY_REF_PTR f_pRef, bool f_fCopyAll ) const;

  //------------------------------------------------------------------

  MKI_FILEREFERENCE m_szDrawingName;

  MKI_VECTOR m_cScaling;
  double     m_dRotation,
             m_dDistortion;
  MKI_POINT  m_cPointUnits;
  __int32    m_nDisplayMode;

  MKI_HIDELAYERS m_afHideLayers;
};

MKI_CLASS_TYPES( MULTI_REF )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MULTI_MAPPING : public MKI_MULTI_BASE
{
  public:

  enum
  {
    USE_XSCALE       = 0x00000001,
    USE_YSCALE       = 0x00000002,
    USE_ROTATION     = 0x00000004,
    USE_DISTORTION   = 0x00000008,
    USE_XPOS         = 0x00000010,
    USE_YPOS         = 0x00000020,
    USE_SCALEDWIDTH  = 0x00000040,
    USE_NOSNAP       = 0x00000080,
    USE_NOGEO        = 0x00000100,
    USE_HIDELAYERS   = 0x00000200,
    USE_SCALEDLENGTH = 0x00000400,
    USE_ADAPTSCALE   = 0x00000800,

    USE_MATRIX       = USE_XSCALE | USE_YSCALE | USE_ROTATION | USE_DISTORTION | USE_XPOS | USE_YPOS,
  };

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  Fill( MKI_CONST_ENTITY_MAPPING_PTR f_pMap );

  MKI_INTERFACE_MEMBER void
  Apply( MKI_ENTITY_MAPPING_PTR f_pMap, bool f_fCopyAll ) const;

  //------------------------------------------------------------------

  MKI_VECTOR m_cScaling;
  double     m_dRotation,
             m_dDistortion;
  MKI_POINT  m_cPointUnits;
  __int32    m_nDisplayMode;

  MKI_HIDELAYERS m_afHideLayers;
  MKI_LAYERDATA m_cLayerData;
};

MKI_CLASS_TYPES( MULTI_MAPPING )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MULTI_HATCH : public MKI_MULTI_BASE
{
  public:

  enum
  {
    USE_SCALEDWIDTH   = 0x00000001,
    USE_NOSNAP        = 0x00000002,
    USE_NOHATCHUPDATE = 0x00000004,
    USE_SCALEDLENGTH  = 0x00000008,
    USE_HATCHING      = 0x00000010,
  };

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  Fill( MKI_CONST_ENTITY_HATCH_PTR f_pHatch );

  MKI_INTERFACE_MEMBER bool
  Apply( MKI_ENTITY_HATCH_PTR f_pHatch, bool f_fCopyAll ) const;

  //------------------------------------------------------------------

  __int32       m_nDisplayMode;
  MKI_HATCHDEF  m_cHatchDef;
};

MKI_CLASS_TYPES( MULTI_HATCH )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MULTI_EXT
{
  public:

  __int32 m_nPlugIn;

  MKI_COMMENTSHORT   m_szDescription;
  MKI_ENTITY_EXT_PTR m_pExt;
};

MKI_CLASS_TYPES( MULTI_EXT )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_USETYPE_FIX      = 0,
  MKI_USETYPE_TRANSMIT = 1,
  MKI_USETYPE_SET      = 2,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MULTI_INFO
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_MULTI_INFO( void );

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  bool  m_fExtValid,
        m_fForceMultiple;

  __int32 m_nDimMode,
          m_nPropMode,
          m_nExtCount,
          m_nEntityCount,
          m_nUseType;
  __int64 m_nTotalSize;

  MKI_UNIQUE_ID m_cUniqueID,
                m_cLinkedID;

  MKI_MULTI_XPROPERTY     m_acXProperty[2]; // 0 - Object, 1 - Text
  MKI_MULTI_TEXTSTANDARD  m_cTextStandard;
  MKI_MULTI_TEXTFRAME     m_cTextFrame;
  MKI_MULTI_TEXTREFERENCE m_cTextReference;
  MKI_MULTI_ZIGZAG        m_cZigZag;
  MKI_MULTI_DIMLINE       m_cDimLine;
  MKI_MULTI_DIMPARAM      m_cDimTextParam;
  MKI_MULTI_DIMSMALL      m_cDimText;
  MKI_MULTI_DIMPARAM      m_cDimParam;
  MKI_MULTI_DIMLARGE      m_cDimLarge;
  MKI_MULTI_DIMSMALL      m_cDimSmall;
  MKI_MULTI_COMMENT       m_cComment;
  MKI_MULTI_BITMAPREF     m_cBitmapRef;
  MKI_MULTI_INSTANCE      m_cInstance;
  MKI_MULTI_CLIP          m_cClipping;
  MKI_MULTI_GROUP         m_cGroup;
  MKI_MULTI_REF           m_cReference;
  MKI_MULTI_MAPPING       m_cMapping;
  MKI_MULTI_HATCH         m_cHatch;
  MKI_MULTI_EXT_PTR       m_pExtData;
};

MKI_CLASS_TYPES( MULTI_INFO )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_INSERTBLOCK_DATA
{
  public:

  enum
  {
    MODE_SHOW_ATTRIBUTES  = 0x00000001,
    MODE_SHOW_PREVIEW     = 0x00000002,
    MODE_SHOW_NOVARIABLES = 0x00000004,

    MODE_TRANSMIT_LAYER   = 0x00000010,
    MODE_USE_BLOCK_LAYER  = 0x00000020,
    MODE_TRANSMIT_PEN     = 0x00000040,
    MODE_USE_BLOCK_PEN    = 0x00000080,
    MODE_COPY_ATTRIBUTES  = 0x00000100,
    MODE_EDIT_ATTRIBUTES  = 0x00000200,
    MODE_COPY_COMMENT     = 0x00000400,
    MODE_ADAPT_SCALE      = 0x00000800,
    MODE_MANUAL_ROTATE    = 0x00001000,
    MODE_RESOLVE          = 0x00002000,
    MODE_GROUPED          = 0x00004000,
    MODE_CONVERT          = 0x00008000,
  };

  enum
  {
    DIALOGMODE_NONE             = 0x00000000,
    DIALOGMODE_LIBRARY          = 0x00000001,
    DIALOGMODE_SCALE            = 0x00000002,
    DIALOGMODE_ROTATE           = 0x00000004,
    DIALOGMODE_TRANSMITLAYER    = 0x00000008,
    DIALOGMODE_USEBLOCKLAYER    = 0x00000010,
    DIALOGMODE_TRANSMITPEN      = 0x00000020,
    DIALOGMODE_USEBLOCKPEN      = 0x00000040,
    DIALOGMODE_COPYATTRIB       = 0x00000080,
    DIALOGMODE_EDITATTRIB       = 0x00000100,
    DIALOGMODE_USEBLOCKSCALE    = 0x00000200,
    DIALOGMODE_RESOLVE          = 0x00000400,
    DIALOGMODE_DISPLAY_SCALED   = 0x00000800,
    DIALOGMODE_DISPLAY_NOGEO    = 0x00001000,
    DIALOGMODE_DISPLAY_CONFINED = 0x00002000,
    DIALOGMODE_DISPLAY_ADAPT    = 0x00004000,
    DIALOGMODE_DISPLAY_EMBEDDED = 0x00008000,
    DIALOGMODE_DISPLAY_NOHIDE   = 0x00010000,
    DIALOGMODE_EMBED            = 0x00020000,
    DIALOGMODE_CONVERT          = 0x00040000,
    DIALOGMODE_COMMENT          = 0x01000000,
    DIALOGMODE_ALL              = 0xffffffff,
  };

  MKI_INTERFACE_MEMBER
  MKI_INSERTBLOCK_DATA( void );

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_LIBRARYNAME m_szLibraryName;  // Current library name
  MKI_BLOCKNAME   m_szBlockName;    // Current block name
  MKI_COMMENTLONG m_szComment;      // Current comment

  double  m_dScale,       // Scale
          m_dRotate;      // Rotation
  __int32 m_nMode,        // Modes
          m_nDisplayMode; // Display mode
};

MKI_CLASS_TYPES( INSERTBLOCK_DATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
constexpr __int32 MKI_SELFILTER_VALUE_MAX = 10; // Number of values per selection filter entry
constexpr __int32 MKI_SELFILTER_ENTRY_MAX = 5;  // Number of entries per selection filter


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_SELFILTER_TYPE_UNDEFINED    = -1,
  MKI_SELFILTER_TYPE_LAYER        = 0,  // -> MKI_SELFILTER_LAYER
  MKI_SELFILTER_TYPE_PEN          = 1,  // -> MKI_SELFILTER_PEN
  MKI_SELFILTER_TYPE_LINETYPE     = 2,  // -> MKI_SELFILTER_LINETYPE
  MKI_SELFILTER_TYPE_SYSTEM       = 3,  // -> MKI_SELFILTER_SYSTEM
  MKI_SELFILTER_TYPE_ENTITYTYPE   = 4,  // -> MKI_SELFILTER_ENTITYTYPE
  MKI_SELFILTER_TYPE_OBJECTTYPE   = 5,  // -> MKI_SELFILTER_OBJECTTYPE
  MKI_SELFILTER_TYPE_LINEWIDTH    = 6,  // -> MKI_SELFILTER_INTERVAL
  MKI_SELFILTER_TYPE_TEXTSIZE     = 7,  // -> MKI_SELFILTER_INTERVAL
  MKI_SELFILTER_TYPE_TEXTROTATE   = 8,  // -> MKI_SELFILTER_INTERVAL
  MKI_SELFILTER_TYPE_LINECOLOR    = 9,  // -> MKI_SELFILTER_COLOR
  MKI_SELFILTER_TYPE_FILLCOLOR    = 10, // -> MKI_SELFILTER_COLOR
  MKI_SELFILTER_TYPE_FILLMODE     = 11, // -> MKI_SELFILTER_VALUE
  MKI_SELFILTER_TYPE_LINECAPS     = 12, // -> MKI_SELFILTER_VALUE
  MKI_SELFILTER_TYPE_TEXTMODE     = 13, // -> MKI_SELFILTER_VALUE (MKI_TEXTMODE_*)
  MKI_SELFILTER_TYPE_TEXTORDER    = 14, // -> MKI_SELFILTER_VALUE (MKI_TEXTORDER_*)
  MKI_SELFILTER_TYPE_FONTFAMILY   = 15, // -> MKI_SELFILTER_VALUE (ANSI_CHARSET, BALTIC_CHARSET, CHINESEBIG5_CHARSET, etc.)
  MKI_SELFILTER_TYPE_FONTSTYLE    = 16, // -> MKI_SELFILTER_VALUE (upper word contains weight, lower word contains MKI_FONTSTYLE_*)
  MKI_SELFILTER_TYPE_FONTNAME     = 17, // -> MKI_SELFILTER_TEXT ("Arial", "Times New Roman", etc.; .TextMode is unused)
  MKI_SELFILTER_TYPE_TEXT         = 18, // -> MKI_SELFILTER_TEXT
  MKI_SELFILTER_TYPE_RECORD       = 19, // -> MKI_SELFILTER_RECORD
  MKI_SELFILTER_TYPE_EXTTYPE      = 20, // -> MKI_SELFILTER_EXTTYPE
  MKI_SELFILTER_TYPE_TEXTVERTICAL = 21, // -> MKI_SELFILTER_VALUE (MKI_TEXTVERTICAL_*)
  MKI_SELFILTER_TYPE_PAGE         = 22, // -> MKI_SELFILTER_PAGE
  MKI_SELFILTER_TYPE_LINEALPHA    = 23, // -> MKI_SELFILTER_INTERVAL
  MKI_SELFILTER_TYPE_FILLALPHA    = 24, // -> MKI_SELFILTER_INTERVAL
  MKI_SELFILTER_TYPE_UNIQUEID     = 25, // -> MKI_SELFILTER_TEXT ("837c7ef7564ab4e57a64b886c4df62a4"; .TextMode is unused)
  MKI_SELFILTER_TYPE_LINKEDID     = 26, // -> MKI_SELFILTER_TEXT ("837c7ef7564ab4e57a64b886c4df62a4"; .TextMode is unused)
  MKI_SELFILTER_TYPE_LENGTH       = 27, // -> MKI_SELFILTER_INTERVAL
  MKI_SELFILTER_TYPE_RADIUS       = 28, // -> MKI_SELFILTER_INTERVAL
  MKI_SELFILTER_TYPE_ANGLE        = 29, // -> MKI_SELFILTER_INTERVAL
  MKI_SELFILTER_TYPE_SCALING      = 30, // -> MKI_SELFILTER_INTERVAL
  MKI_SELFILTER_TYPE_AREA         = 31, // -> MKI_SELFILTER_INTERVAL

  MKI_SELFILTER_TYPE_MAX          = 32,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_SELFILTER_MODE_OBJECT  = 0x00000001, // Use object properties
  MKI_SELFILTER_MODE_TEXT    = 0x00000002, // Use text properties
  MKI_SELFILTER_MODE_RECURSE = 0x00000100, // Resursive search
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_SEARCH_MODE_SENSITIVE   = 0x00000001, // Case sensitive
  MKI_SEARCH_MODE_NOTAGS      = 0x00000002, // Tags will be ignored
  MKI_SEARCH_MODE_NOVARS      = 0x00000004, // Variables will not be resolved
  MKI_SEARCH_MODE_PARTIAL     = 0x00000008, // Partial search
  MKI_SEARCH_MODE_RECURSE     = 0x00000010, // Recursive search
  MKI_SEARCH_MODE_VISIBLE     = 0x00000020, // Only visible objects
  MKI_SEARCH_MODE_EXPRESSION  = 0x00000040, // Search text contains an expression (e.g. "~Value~ >= 0 && ~Value~ < 1.0")
  MKI_SEARCH_MODE_OBJECTPROP  = 0x00000080, // Use object properties
  MKI_SEARCH_MODE_TEXTPROP    = 0x00000100, // Use text properties
  MKI_SEARCH_MODE_TEXT        = 0x00010000, // Standard texts
  MKI_SEARCH_MODE_DIMENSION   = 0x00020000, // Dimension number
  MKI_SEARCH_MODE_ATTRIBVALUE = 0x00040000, // Attribute values
  MKI_SEARCH_MODE_BLOCKNAME   = 0x00080000, // Block names
  MKI_SEARCH_MODE_LIBRARYNAME = 0x00100000, // Library names
  MKI_SEARCH_MODE_BITMAPNAME  = 0x00200000, // Bitmap names
  MKI_SEARCH_MODE_FONTNAME    = 0x00400000, // Font names
  MKI_SEARCH_MODE_DRAWINGNAME = 0x00800000, // Drawing names
  MKI_SEARCH_MODE_HYPERLINK   = 0x01000000, // Hyperlink texts
  MKI_SEARCH_MODE_COMMENT     = 0x02000000, // Comment texts
  MKI_SEARCH_MODE_ATTRIBNAME  = 0x04000000, // Attribute names
  MKI_SEARCH_MODE_LAYERNAME   = 0x08000000, // Layer names
  MKI_SEARCH_MODE_PENNAME     = 0x10000000, // Pen names
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
#define MKI_SELFILTER_MAX_SIZE  sizeof( MKI_SELFILTER_RECORD )  // Largest possible selection filter entry


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_SELFILTER_HEADER
{
  public:

  __int32 m_nType,
          m_nSize;
  bool    m_fInvert;
};

MKI_CLASS_TYPES( SELFILTER_HEADER )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_SELFILTER_BASE
{
  public:

  MKI_SELFILTER_HEADER m_cHeader;
};

MKI_CLASS_TYPES( SELFILTER_BASE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_SELFILTER_PAGE : public MKI_SELFILTER_BASE
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_BITFIELD<MKI_PAGE_MAX+1> m_afBits;
};

MKI_CLASS_TYPES( SELFILTER_PAGE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_SELFILTER_LAYER : public MKI_SELFILTER_BASE
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_BITFIELD<MKI_LAYER_MAX+1> m_afBits;
};

MKI_CLASS_TYPES( SELFILTER_LAYER )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_SELFILTER_PEN : public MKI_SELFILTER_BASE
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_BITFIELD<MKI_PEN_MAX+1> m_afBits;
};

MKI_CLASS_TYPES( SELFILTER_PEN )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_SELFILTER_LINETYPE : public MKI_SELFILTER_BASE
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_BITFIELD<MKI_LINE_MAX+1> m_afBits;
};

MKI_CLASS_TYPES( SELFILTER_LINETYPE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_SELFILTER_SYSTEM : public MKI_SELFILTER_BASE
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_BITFIELD<MKI_SYSTEM_MAX+1> m_afBits;
};

MKI_CLASS_TYPES( SELFILTER_SYSTEM )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_SELFILTER_OBJECTTYPE : public MKI_SELFILTER_BASE
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_BITFIELD<MKI_OBJ_TOTAL> m_afBits;
};

MKI_CLASS_TYPES( SELFILTER_OBJECTTYPE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_SELFILTER_ENTITYTYPE : public MKI_SELFILTER_BASE
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  bool m_fObject,
       m_fInstance,
       m_fGroup,
       m_fPosNo,
       m_fClip,
       m_fReference,
       m_fExtended,
       m_fMapping,
       m_fHatch;
};

MKI_CLASS_TYPES( SELFILTER_ENTITYTYPE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_SELFILTER_INTERVAL : public MKI_SELFILTER_BASE
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( __int32 f_nType );

  //------------------------------------------------------------------

  __int32 m_nCount;
  double  m_adMinValue[MKI_SELFILTER_VALUE_MAX],
          m_adMaxValue[MKI_SELFILTER_VALUE_MAX];
};

MKI_CLASS_TYPES( SELFILTER_INTERVAL )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_SELFILTER_COLOR : public MKI_SELFILTER_BASE
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( __int32 f_nType );

  //------------------------------------------------------------------

  __int32        m_nCount;
  MKI_COLORVALUE m_acValue[MKI_SELFILTER_VALUE_MAX];
  MKI_COLORMODEL m_acMode [MKI_SELFILTER_VALUE_MAX];
};

MKI_CLASS_TYPES( SELFILTER_COLOR )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_SELFILTER_VALUE : public MKI_SELFILTER_BASE
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( __int32 f_nType );

  //------------------------------------------------------------------

  __int32 m_nCount;
  __int32 m_anValue[MKI_SELFILTER_VALUE_MAX];
};

MKI_CLASS_TYPES( SELFILTER_VALUE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_SELFILTER_TEXT : public MKI_SELFILTER_BASE
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( __int32 f_nType );

  //------------------------------------------------------------------

  __int32       m_nCount;
  MKI_STRSHORTW m_aszText   [MKI_SELFILTER_VALUE_MAX];
  __int32       m_anTextMode[MKI_SELFILTER_VALUE_MAX];
};

MKI_CLASS_TYPES( SELFILTER_TEXT )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_SELFILTER_RECORD : public MKI_SELFILTER_BASE
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  __int32       m_nCount;
  MKI_STRSHORTW m_aszText      [MKI_SELFILTER_VALUE_MAX];
  __int32       m_anTextMode   [MKI_SELFILTER_VALUE_MAX],
                m_anColumnIndex[MKI_SELFILTER_VALUE_MAX];
};

MKI_CLASS_TYPES( SELFILTER_RECORD )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_SELFILTER_EXTTYPE : public MKI_SELFILTER_BASE
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_BITFIELD<MKI_PLUGIN_EXT_MAX> m_afBits;
};

MKI_CLASS_TYPES( SELFILTER_EXTTYPE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_SELFILTER_DATA
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  __int32                m_nMode; // MKI_SELFILTER_MODE_*
  MKI_SELFILTER_BASE_PTR m_apData[MKI_SELFILTER_ENTRY_MAX];
};

MKI_CLASS_TYPES( SELFILTER_DATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_PLUGINMODE_INVALID    = 0,
  MKI_PLUGINMODE_DEACTIVATE = 1,
  MKI_PLUGINMODE_TESTING    = 2,
  MKI_PLUGINMODE_UNLOCKED   = 3,
  MKI_PLUGINMODE_FREE       = 4,
  MKI_PLUGINMODE_CUSTOM     = 5,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PLUGIN_CONTAINER
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_STR32W m_szRegistry;
  MKI_STR32A m_szBuildDate;
  LPWSTR     m_pszCaption,
             m_pszVendor,
             m_pszCopyright;
  WORD       m_wProductMin,
             m_wProductMax,
             m_wCompileYear,
             m_wCompileMonth,
             m_wCompileDay;
  MKI_STR64A m_szUserName,
             m_szUserCode;
  __int64    m_nUserTime;
  __int32    m_nUserMode,
             m_nUserDays;
};

MKI_CLASS_TYPES( PLUGIN_CONTAINER )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_DATABASETYPE_ANY = 0, // Any / default format
  MKI_DATABASETYPE_CSV = 1, // Comma-separated value format
  MKI_DATABASETYPE_DBF = 2, // DBase binary format
  MKI_DATABASETYPE_MAX = 3,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_DATABASEOPTION_NONE                        = 0x00000000, // None
  MKI_DATABASEOPTION_FIRSTROW                    = 0x00000001, // First row contains column titles
  MKI_DATABASEOPTION_AUTOSAVE                    = 0x00000002, // Save internal database to external file when saving drawing
  MKI_DATABASEOPTION_APPLY_AUTOCOLUMNS_UPON_OPEN = 0x00000004, // Apply auto-columns upon opening or updating of the database
  MKI_DATABASEOPTION_UTF8                        = 0x00000008, // Database is in UTF-8 format (not ANSI)
  MKI_DATABASEOPTION_FORCE_DECIMAL_POINT         = 0x00000010, // Always use 'point' as decimal separator
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_COLUMNTYPE_TEXT     = 0, // Text
  MKI_COLUMNTYPE_INTEGER  = 1, // Integer value
  MKI_COLUMNTYPE_NUMERIC  = 2, // Numeric value 
  MKI_COLUMNTYPE_FLOATING = 3, // Floating point number
  MKI_COLUMNTYPE_BOOL     = 4, // Boolean value
  MKI_COLUMNTYPE_DATE     = 5, // Date
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_COLUMNASSIGN_NONE              = 0,  // No auto-column
  MKI_COLUMNASSIGN_OBJECT_ID         = 1,  // Unique object identifier
  MKI_COLUMNASSIGN_OBJECT_COUNTER    = 2,  // Object counter
  MKI_COLUMNASSIGN_OBJECT_TYPE       = 3,  // Object type (MKI_OBJ_* or -1 for block instance)
  MKI_COLUMNASSIGN_OBJECT_NAME       = 4,  // Object type name or "block instance"
  MKI_COLUMNASSIGN_PAGE              = 5,  // Page number (negative if assigned to no page)
  MKI_COLUMNASSIGN_PAGE_COMMENT      = 6,  // Page comment
  MKI_COLUMNASSIGN_LAYER             = 7,  // Layer name
  MKI_COLUMNASSIGN_LAYER_COMMENT     = 8,  // Layer comment
  MKI_COLUMNASSIGN_PEN               = 9,  // Pen name
  MKI_COLUMNASSIGN_PEN_COMMENT       = 10, // Pen comment
  MKI_COLUMNASSIGN_LINE_TYPE         = 11, // Line type name
  MKI_COLUMNASSIGN_LINE_WIDTH        = 12, // Line width in [mm]
  MKI_COLUMNASSIGN_LINE_COLOR_NATIVE = 13, // Native color description for line: TTRRGGBB (translucency value followed by red/green/blue color values, range for all values: 0..255)
  MKI_COLUMNASSIGN_LINE_COLOR_ACI    = 14, // ACI color value for line
  MKI_COLUMNASSIGN_FILL_COLOR_NATIVE = 15, // Native color description for filling: TTRRGGBB (translucency value followed by red/green/blue color values, range for all values: 0..255)
  MKI_COLUMNASSIGN_FILL_COLOR_ACI    = 16, // ACI color value for filling
  MKI_COLUMNASSIGN_AREA              = 17, // Area
  MKI_COLUMNASSIGN_PERIMETER         = 18, // Perimeter
  MKI_COLUMNASSIGN_INSERT_X          = 19, // X coordinate [in current length unit] of insertion point (block instance, marking, and text/dimension objects)
  MKI_COLUMNASSIGN_INSERT_Y          = 20, // Y coordinate [in current length unit] of insertion point (block instance, marking, and text/dimension objects)
  MKI_COLUMNASSIGN_LABEL             = 21, // Name of block, Text "Marking", or content of text/dimension object
  MKI_COLUMNASSIGN_ROTATION          = 22, // Rotation of block instance or standard/reference text object in [deg]
  MKI_COLUMNASSIGN_HEIGHT_SCALING    = 23, // Scaling of block instance or font height of standard/reference text object
  MKI_COLUMNASSIGN_SELECTION         = 24, // Permanent selection status: "ON/OFF" or 1/0 depending on the column's type
  MKI_COLUMNASSIGN_ATTRIB_CONTENT    = 25, // Only valid for instances, this column assignment may occur more than once in the same database. The column title matches the attribute's name.
  MKI_COLUMNASSIGN_MANUAL_INPUT      = 26, // This column assignment may occur more than once in the same database. When this assignment is present and the user has selected a single object to be added to the database then the user is prompted to enter a text or numerical value manually depending on the column's type.
  MKI_COLUMNASSIGN_MAX               = 27,

  MKI_COLUMNASSIGN_CUSTOM_MAX        = 50, // Maximum number of saved column names for attribute and manual columns
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_DELIMITER_DOUBLE    = 0, // "
  MKI_DELIMITER_SINGLE    = 1, // '
  MKI_DELIMITER_NONE      = 2, // None
  MKI_DELIMITER_INTERNAL  = 3, // 0x001f
  MKI_DELIMITER_MAX       = 4,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_SEPARATOR_SEMICOLON = 0, // ;
  MKI_SEPARATOR_COMMA     = 1, // ,
  MKI_SEPARATOR_TAB       = 2, // <TAB>
  MKI_SEPARATOR_LF        = 3, // <LF>
  MKI_SEPARATOR_CR        = 4, // <CR>
  MKI_SEPARATOR_CR_LF     = 5, // <CR><LF>
  MKI_SEPARATOR_MAX       = 6,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_CELLMODE_NONE      = 0x00000000, // No special mode
  MKI_CELLMODE_DELIMITED = 0x00000001, // Cell contains delimited text
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_DATABASE_CELL
{
  public: 

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  LPCWSTR m_pszContent;
  __int32 m_nLength,
          m_nMode;
};

MKI_CLASS_TYPES( DATABASE_CELL )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_DATABASE_COLUMN
{
  public: 

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  __int32 m_nType,
          m_nAutoAssign,
          m_nMaxLength,
          m_nDecimalPlaces;
};

MKI_CLASS_TYPES( DATABASE_COLUMN )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_DATABASE_DATA
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_FILENAMEW       m_szFileName;
  MKI_DATABASENAME    m_szDatabaseName;
  MKI_DATABASE_COLUMN m_acColumnData[MKI_DATABASE_COLUMN_MAX];

  __int32 m_nColumnCount,
          m_nFileType,
          m_nTextDelimiter,
          m_nRecordSeparator,
          m_nFieldSeparator,
          m_nOptions;
};

MKI_CLASS_TYPES( DATABASE_DATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
// Definitions for Entity Enumeration
class MKI_GEO_OBJECT
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_GEO_OBJECT( void );

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER bool
  CopyObj( MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

  //------------------------------------------------------------------

  MKI_ENTITY_PTR m_pEntity;

  LPWSTR m_pszDescription;

  MKI_RECT  m_cRect;
  MKI_POINT m_cPoint1,
            m_cPoint2,
            m_cPoint3,
            m_cPoint4,
            m_cPoint5;

  __int32 m_nHandle1,
          m_nHandle2,
          m_nHandle3,
          m_nHandle4,
          m_nHandle5,

          m_nObjType,
          m_nPen,
          m_nLayer,
          m_nFlag,
          m_nVisible,
          m_nOutlineOld,
          m_nOutlineNew;

  double m_dLen,
         m_dStart,
         m_dArc;

  bool m_fOrient,
       m_fValid;
};

MKI_CLASS_TYPES( GEO_OBJECT )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_SNAP_POINT
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_SNAP_POINT( void );

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_MATRIX  m_cMatrix;
  MKI_POINT   m_cClickPoint,
              m_cBestPoint;

  double  m_dRadius,
          m_dDistance;
  __int32 m_nMode,
          m_nBestType;
  LPWSTR  m_pszDescription1,
          m_pszDescription2;
  bool    m_fStopped;
};

MKI_CLASS_TYPES( SNAP_POINT )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_POINTIDENTDATA
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_POINTIDENTDATA( void );

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_POINT_PTR m_pPoints;

  __int32 m_nCount,
          m_nFlag;
  double  m_dRadius;
};

MKI_CLASS_TYPES( POINTIDENTDATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_IDENTDATA
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_IDENTDATA( void );

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER bool
  AddSorted( MKI_CONST_GEO_OBJECT_REF f_rGeoObject );

  //------------------------------------------------------------------

  MKI_POINT m_cClickPoint;

  double  m_dRadius;
  __int32 m_nMode,
          m_nCount,
          m_nCurrent,
          m_nMax;

  MKI_GEO_OBJECT_PTR m_pList;
};

MKI_CLASS_TYPES( IDENTDATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_SIMPLIFY_DATA
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_SIMPLIFY_DATA( void );

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  SetDefault( void );

  //------------------------------------------------------------------

  bool    m_fActive;
  double  m_dLineMin,
          m_dLineMax,
          m_dLineRel,
          m_dLineAbs,
          m_dCircleMin,
          m_dCircleMax,
          m_dCircleRel,
          m_dCircleAbs;
};

MKI_CLASS_TYPES( SIMPLIFY_DATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_ENUMDEF_DATA
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER __int32
  ClipToRectangle( MKI_CONST_RECT_REF f_rRect, bool f_fMaintainFillings );

  MKI_INTERFACE_MEMBER __int32
  ClipToRectangle( MKI_CONST_RECT_REF f_rRect, MKI_POINTDATA_PTR f_pResult, __int32 f_nSize, bool f_fMaintainFillings );

  MKI_INTERFACE_MEMBER __int32
  GetOrientation( __int32 f_nStartIndex ) const;

  MKI_INTERFACE_MEMBER __int32
  IsPointInside( __int32 f_nStartIndex, MKI_CONST_POINT_REF f_rPoint ) const;

  MKI_INTERFACE_MEMBER __int32
  IsPointLeft( __int32 f_nStartIndex, MKI_CONST_POINT_REF f_rPoint ) const;

  //------------------------------------------------------------------

  __int32 m_nEnumData,
          m_nEnumCount,
          m_nEnumFlag1,
          m_nEnumFlag2;

  MKI_RESULTVALUE m_nEnumResult;

  __int64 m_nEnumMode;
  bool    m_fEnumStopped;

  MKI_XPROPERTY m_cOriginalXProperty,
                m_cResolvedXProperty;
  double        m_dLineLengthScale;
  MKI_FONTDEF   m_cFont;

  double  m_dCharDistance,
          m_dTabDistance,
          m_dLineDistance;
  __int32 m_nTextMode,
          m_nTextVertical,
          m_nTextOrder,
          m_nSystem;

  MKI_BITMAP_EFFECT m_cBitmapEffect;
  MKI_POINTDATA_PTR m_pPointData;
  MKI_MATRIX_PTR    m_pMatrix;
  MKI_POINT         m_cTextPos;
  __int32           m_nChar;
  LPCWSTR           m_pszText,
                    m_pszTitle,
                    m_pszName;

  MKI_ENTITY_OBJECT_PTR   m_pObj;
  MKI_ENTITY_INSTANCE_PTR m_pInst;
  MKI_ENTITY_BLOCK_PTR    m_pBlock;
  MKI_ENTITY_CLIP_PTR     m_pClip;
  MKI_ENTITY_REF_PTR      m_pRef;
  MKI_ENTITY_PATCH_PTR    m_pPatch;
  MKI_ENTITY_MAPPING_PTR  m_pMap;
  MKI_ENTITY_GROUP_PTR    m_pGroup;
  MKI_ENTITY_HATCH_PTR    m_pHatch;
  MKI_ENTITY_EXT_PTR      m_pExt;

  MKI_RECT  m_cEntityRect;
  __int32   m_nObjectType;

  MKI_HIDELAYERS m_afHideLayers;
  MKI_LAYERDATA  m_cLayerData;
};

MKI_CLASS_TYPES( ENUMDEF_DATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_ENUMATTRIB_DATA
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_RESULTVALUE m_nEnumResult;

  __int16 m_nType;
  LPCWSTR m_pszName,
          m_pszValue;

  MKI_ENTITY_PTR m_pEntity;
  
  __int32 m_nAttribHandle,
          m_nStartIndex,
          m_nStartHandle,
          m_nPointIndex,
          m_nPointHandle,
          m_nOutlineIndex;
};

MKI_CLASS_TYPES( ENUMATTRIB_DATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_ENTITYSTACK
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER bool
  Add( MKI_CONST_ENTITY_PTR f_pEntity );

  MKI_INTERFACE_MEMBER MKI_ENTITY_PTR
  GetTop( void ) const;

  MKI_INTERFACE_MEMBER MKI_ENTITY_EXT_PTR
  GetTopExt( void ) const;

  MKI_INTERFACE_MEMBER MKI_ENTITY_INSTANCE_PTR
  GetInstance( void ) const;

  __forceinline bool
  IsEmpty( void ) const
  {
    return( m_nCount == 0 );
  }

  __forceinline bool
  IsFull( void ) const
  {
    return( m_nCount >= MKI_NESTING_DEPTH_MAX );
  }

  //------------------------------------------------------------------

  MKI_ENTITY_PTR  m_acStack[MKI_NESTING_DEPTH_MAX];
  __int32       m_nCount;
};

MKI_CLASS_TYPES( ENTITYSTACK )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BUFFER
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER bool
  Alloc( __int32 f_nSize, bool f_fClosedOnly );

  MKI_INTERFACE_MEMBER bool
  AddEnumerate( __int32 f_nFlag, __int64 f_nEnumMode, MKI_TIMER_PTR f_pTimer );

  MKI_INTERFACE_MEMBER bool
  AddTemporary( __int64 f_nEnumMode, MKI_TIMER_PTR f_pTimer );

  MKI_INTERFACE_MEMBER bool
  AddEntity( MKI_CONST_ENTITY_PTR f_pEntity, __int64 f_nEnumMode );

  MKI_INTERFACE_MEMBER bool
  Prepare( double f_dTolerance, MKI_TIMER_PTR f_pTimer );

  MKI_INTERFACE_MEMBER bool
  Free( void );

  //------------------------------------------------------------------

  void*    m_pBuffer;
  MKI_RECT m_cElemRect;
  __int32  m_nElemMax,
           m_nElemCount,
           m_nPolygonCount;
  bool     m_fClosedOnly,
           m_fInterruptible;
};

MKI_CLASS_TYPES( BUFFER )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_SEARCH_DATA
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_ENTITY_PTR m_pEntity;

  bool    m_fIsDrawing;
  __int32 m_nLibraryNum;

  MKI_ENTITY_BLOCK_PTR m_pBlock;
  MKI_ENTITY_CLIP_PTR  m_pClip;
  MKI_ENTITY_REF_PTR   m_pRef;
  MKI_ENTITY_GROUP_PTR m_pGroup;
  MKI_ENTITY_EXT_PTR   m_pExt;
};

MKI_CLASS_TYPES( SEARCH_DATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
typedef bool (*MKI_ENUMOBJECT_PROC)( MKI_CONST_ENUMDEF_DATA_PTR f_pEnumData, void* f_pUserData );
typedef bool (*MKI_ENUMATTRIB_PROC)( MKI_CONST_ENUMATTRIB_DATA_PTR f_pEnumData, void* f_pUserData );
typedef bool (*MKI_ENUMIDENT_PROC) ( MKI_ENTITY_PTR f_pEntity, void* f_pUserData );
typedef bool (*MKI_ENUMPOINTS_PROC) ( MKI_ENTITY_PTR f_pEntity, MKI_BLOCK_PTR f_pBlock, void* f_pUserData );
typedef bool (*MKI_ENUMEXT_PROC)( MKI_ENTITY_EXT_PTR f_pExt, void* f_pEnumData, void* f_pUserData );


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_ENUMEXT_DATA
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_RESULTVALUE m_nEnumResult;

  MKI_ENUMEXT_PROC m_pfnCallback;
  void*            m_pUserData;
  __int32          m_nEnumMode;
};

MKI_CLASS_TYPES( ENUMEXT_DATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
typedef bool (*MKI_HATCHOPENPATH_PROC) ( MKI_CONST_XPROPERTY_PTR f_pXProperty, void* f_pUserData );
typedef bool (*MKI_HATCHADDOBJECT_PROC) ( MKI_CONST_GEO_OBJECT_PTR f_pGeoObject, __int32 f_nScanLineID, void* f_pUserData );
typedef bool (*MKI_HATCHCLOSEPATH_PROC) ( void* f_pUserData );


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_SUBCLASS_DATA;
typedef void (*MKI_SUBCLASS_PROC)( MKI_SUBCLASS_DATA* f_pData );


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_SUBCLASS_DATA
{
  public:

  MKI_SUBCLASS_PROC m_pfnDrawProc,
                    m_pfnMenuProc;

  WNDPROC m_pfnWindowProc;
  HWND    m_hDlg,
          m_hControl;
  __int32 m_nCtlID,
          m_nIconID;
  void*   m_pUserData;
};

MKI_CLASS_TYPES( SUBCLASS_DATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_SUBCLASS_LIST
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_SUBCLASS_LIST( void );

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER void
  AddControl( HWND f_hDlg, __int32 f_nCtlID,
              MKI_SUBCLASS_PROC f_fnDrawProc, MKI_SUBCLASS_PROC f_fnMenuProc = nullptr, void* f_pUserData = nullptr );

  MKI_INTERFACE_MEMBER void
  AddControlRange( HWND f_hDlg, __int32 f_nCtlID, __int32 f_nRange, 
                   MKI_SUBCLASS_PROC f_fnDrawProc, MKI_SUBCLASS_PROC f_fnMenuProc = nullptr, void* f_pUserData = nullptr );

  MKI_INTERFACE_MEMBER void
  AddIconButton( HWND f_hDlg, __int32 f_nCtlID, 
                 int f_nIconID, MKI_SUBCLASS_PROC f_fnMenuProc = nullptr, void* f_pUserData = nullptr );

  MKI_INTERFACE_MEMBER void
  AddDialogControls( HWND f_hDlg );

  MKI_INTERFACE_MEMBER void
  AddPropertySheetControls( HWND f_hPropSheet );

  //------------------------------------------------------------------

  MKI_SUBCLASS_DATA m_acData[MKI_SUBCLASS_MAX];
  __int32           m_nCount;
};

MKI_CLASS_TYPES( SUBCLASS_LIST )


//---------------------------------------------------------------------------------------------------------------------------------------------------
typedef void (*MKI_CALLBACK_PROC)( HANDLE f_hHandle, void* f_pData );


//---------------------------------------------------------------------------------------------------------------------------------------------------
typedef void (*MKI_HOOKPOSITION_PROC)( __int32 f_nWindowNum, MKI_POINT_PTR f_pPosition );
typedef void (*MKI_HOOKMOUSE_PROC)( __int32 f_nWindowNum, UINT f_unMessage, UINT f_unButtons, __int32 f_nX, __int32 n_fY );
typedef void (*MKI_HOOKKEY_PROC)( __int32 f_nWindowNum, UINT f_unMessage, __int32 f_nKey, __int32 f_pData );


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_FILTER_MODE_NULL     = 0x00000000,
  MKI_FILTER_MODE_MERGE    = 0x00000001,
  MKI_FILTER_MODE_QUIET    = 0x00000002,
  MKI_FILTER_MODE_AUTOCONV = 0x00000004,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_HIGHLIGHT_NONE      = 0x00000000,
  MKI_HIGHLIGHT_SELECTION = 0x00000001,
  MKI_HIGHLIGHT_TEMP      = 0x00000002,
  MKI_HIGHLIGHT_FEEDBACK  = 0x00000004,
  MKI_HIGHLIGHT_HOVER     = 0x00000008,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BITMAP_DATA
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_FILENAMEW m_szFileName,
                m_szLocalFileName;
  void*         m_pBitmap;
  __int32       m_nColorBits;
  MKI_INTVECTOR m_cSize;
  MKI_VECTOR    m_cMMPerPixel;
  bool          m_fEmbedded;
};

MKI_CLASS_TYPES( BITMAP_DATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_CONTAINS_GEOMETRY   = 0x00000001,
  MKI_CONTAINS_COMMENT    = 0x00000002,
  MKI_CONTAINS_EXTENDED   = 0x00000004,
  MKI_CONTAINS_PARAMETRIC = 0x00000008,
  MKI_CONTAINS_3D         = 0x00000010,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_DISPLAYDATA
{
  public:

  MKI_INTERFACE_MEMBER
  MKI_DISPLAYDATA( void );

  MKI_INTERFACE_MEMBER void
  Init( void );

  MKI_INTERFACE_MEMBER bool
  ControlDrawInit( HWND f_hControl );

  MKI_INTERFACE_MEMBER void
  ControlDrawExit( void );

  MKI_INTERFACE_MEMBER void
  DrawCoordinatePoint( MKI_CONST_POINT_REF f_rPoint, bool f_fIsSelected, bool f_fIsIdle, const D2D1_COLOR_F* f_pColor = nullptr );

  MKI_INTERFACE_MEMBER void
  DrawCoordinateLine( MKI_CONST_POINT_REF f_rStart, MKI_CONST_POINT_REF f_rEnd, const D2D1_COLOR_F* f_pColor = nullptr );

  MKI_INTERFACE_MEMBER void
  DrawCoordinateBezier( MKI_CONST_POINT_REF f_rStart, MKI_CONST_POINT_REF f_rPivot1, MKI_CONST_POINT_REF f_rPivot2, MKI_CONST_POINT_REF f_rEnd, const D2D1_COLOR_F* f_pColor = nullptr );

  MKI_INTERFACE_MEMBER void
  DrawCoordinateCircle( MKI_CONST_POINT_REF f_rCenter, MKI_CONST_POINT_REF f_rRadius, const D2D1_COLOR_F* f_pColor = nullptr );

  MKI_INTERFACE_MEMBER void
  DrawCoordinateEllipse( MKI_CONST_POINT_REF f_rCenter, MKI_CONST_POINT_REF f_rVector1, MKI_CONST_POINT_REF f_rVector2, const D2D1_COLOR_F* f_pColor = nullptr );

  MKI_INTERFACE_MEMBER void
  DrawPixelPoint( double f_dX, double f_dY, bool f_fIsSelected, bool f_fIsIdle, const D2D1_COLOR_F* f_pColor = nullptr );

  MKI_INTERFACE_MEMBER void
  DrawPixelMark( double f_dX, double f_dY, const D2D1_COLOR_F* f_pColor = nullptr );

  MKI_INTERFACE_MEMBER void
  DrawPixelLine( double f_dX1, double f_dY1, double f_dX2, double f_dY2, const D2D1_COLOR_F& f_rColor );

  MKI_INTERFACE_MEMBER void
  DrawPixelEndless( double f_dX1, double f_dY1, double f_dX2, double f_dY2, const D2D1_COLOR_F& f_rColor );

  MKI_INTERFACE_MEMBER void
  DrawPixelBezier( double f_dX1, double f_dY1, double f_dX2, double f_dY2,
                   double f_dX3, double f_dY3, double f_dX4, double f_dY4, const D2D1_COLOR_F& f_rColor );

  MKI_INTERFACE_MEMBER void
  DrawPixelRect( double f_dX1, double f_dY1, double f_dX2, double f_dY2, const D2D1_COLOR_F* f_pFillColor, const D2D1_COLOR_F* f_pEdgeColor );

  MKI_INTERFACE_MEMBER void
  DrawPixelEllipse( double f_dX1, double f_dY1, double f_dX2, double f_dY2, double f_dX3, double f_dY3, 
                    const D2D1_COLOR_F* f_pFillColor, const D2D1_COLOR_F* f_pEdgeColor );

  MKI_INTERFACE_MEMBER void
  DrawPixelPolygon( MKI_CONST_POINT_PTR f_pPoints, __int32 f_nPointCount, const D2D1_COLOR_F* f_pFillColor, const D2D1_COLOR_F* f_pEdgeColor );

  MKI_INTERFACE_MEMBER void
  DrawPixelPatch( MKI_CONST_PATCH_REF f_rPatch, const D2D1_COLOR_F* f_pFillColor, const D2D1_COLOR_F* f_pEdgeColor );

  MKI_INTERFACE_MEMBER __int32
  DrawPixelText( double f_dX, double f_dY, const LPCWSTR f_pszText, __int32 f_nTextMode, __int32 f_nTextVertical,
                 const D2D1_COLOR_F& f_rTextColor, const D2D1_COLOR_F* f_pBackColor = nullptr );

  MKI_INTERFACE_MEMBER void
  DrawPixelOrigin( double f_dX, double f_dY, MKI_COLORVALUE f_cColor, const LPCWSTR f_pszText );

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER bool
  IsVisible( MKI_CONST_XPROPERTY_REF f_rXProperty );

  MKI_INTERFACE_MEMBER bool
  IsPrinted( MKI_CONST_XPROPERTY_REF f_rXProperty );

  MKI_INTERFACE_MEMBER bool
  IsLocked( MKI_CONST_XPROPERTY_REF f_rXProperty );

  MKI_INTERFACE_MEMBER bool
  IsSnapped( MKI_CONST_XPROPERTY_REF f_rXProperty );

  MKI_INTERFACE_MEMBER bool
  IsGrayed( MKI_CONST_XPROPERTY_REF f_rXProperty );

  //------------------------------------------------------------------

  void*           m_pEngine;
  MKI_D2D_CONTEXT m_pDC;
  MKI_ENTITY_PTR  m_pEntity;
  HWND            m_hDrawWnd;
  HDC             m_hDrawDC;
  MKI_INTRECT     m_cDrawRect,
                  m_cViewRect;
  MKI_XPROPERTY   m_cXProperty;
  MKI_MATRIX      m_cMatrix;
  __int32         m_nHighlight,
                  m_nContains;
  bool            m_fAnyDraw;
};

MKI_CLASS_TYPES( DISPLAYDATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_UPDATE_DATABASE
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_DATABASENAME m_szDatabaseName;
  bool             m_fRedraw;
};

MKI_CLASS_TYPES( UPDATE_DATABASE )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_UPDATE_ELEMENTS
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  __int32 m_nChanges;
};

MKI_CLASS_TYPES( UPDATE_ELEMENTS )


//---------------------------------------------------------------------------------------------------------------------------------------------------
typedef __int32 (*MKI_PLUGINCOMMAND_PROC)( __int32 f_nCommandID, __int32 f_nExecMode, void* f_pData );
typedef __int32 (*MKI_PLUGINNOTIFY_PROC)( MKI_ENTITY_EXT_PTR f_pExt, __int32 f_nNotifyMode, void* f_pData );
typedef __int32 (*MKI_PLUGINIMPORT_PROC)( const LPCWSTR f_pszFileName, __int32 f_nFilterMode, void* f_pData );
typedef bool (*MKI_PLUGINEXPORT_PROC)( const LPCWSTR f_pszFileName, __int32 f_nFilterMode, __int32 f_nFlag, 
                                       const LPCWSTR f_pszCollectionName, const LPCWSTR f_pszItemName, void* f_pData );
typedef __int32 (*MKI_PLUGINREQUEST_PROC)( const LPCWSTR f_pszRequest, LPWSTR f_pszBuffer, const size_t f_nBufferSize );


//---------------------------------------------------------------------------------------------------------------------------------------------------
typedef __int32 (*MKI_INPUTPOINTINIT_PROC)( __int32 f_nCommandID, __int32 f_nPointIndex, MKI_CONST_POINT_REF f_rPoint );
typedef bool (*MKI_INPUTPOINTMOVE_PROC)( __int32 f_nCommandID, __int32 f_nPointIndex, MKI_CONST_POINT_REF f_rPoint );
typedef __int32 (*MKI_INPUTPOINTEXIT_PROC)( __int32 f_nCommandID, __int32 f_nPointIndex, MKI_CONST_POINT_REF f_rPoint );
typedef void (*MKI_INPUTDISPLAY_PROC)( __int32 f_nCommandID, __int32 f_nPointIndex, MKI_DISPLAYDATA_PTR f_pData );
typedef bool (*MKI_INPUTPARAMETER_PROC)( __int32 f_nCommandID, __int32 f_nMode );
typedef __int32 (*MKI_INPUTDIALOG_PROC)( __int32 f_nCommandID, __int32 f_nPointIndex );
typedef __int32 (*MKI_INPUTCANCEL_PROC)( __int32 f_nCommandID, __int32 f_nPointCount );
typedef void (*MKI_INPUTFINISH_PROC)( __int32 f_nCommandID, __int32 f_nPointCount );


//---------------------------------------------------------------------------------------------------------------------------------------------------
constexpr __int32 MKI_ICON_MENU_FRAME  = 6;
constexpr __int32 MKI_ICON_MENU_SIZE   = 48;
constexpr __int32 MKI_ICON_MENU_OFFSET = MKI_ICON_MENU_SIZE + 2 * MKI_ICON_MENU_FRAME;

constexpr __int32 MKI_ICON_COMMAND_FRAME  = 10;
constexpr __int32 MKI_ICON_COMMAND_SIZE   = 80;
constexpr __int32 MKI_ICON_COMMAND_OFFSET = MKI_ICON_COMMAND_SIZE + 2 * MKI_ICON_COMMAND_FRAME;

constexpr __int32 MKI_ICON_BITMAP_FRAME  = 15;
constexpr __int32 MKI_ICON_BITMAP_SIZE   = 420;
constexpr __int32 MKI_ICON_BITMAP_OFFSET = MKI_ICON_BITMAP_SIZE + 2 * MKI_ICON_BITMAP_FRAME;


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_ICON_ID_FILETYPE_MKD = 0,
  MKI_ICON_ID_FILETYPE_DXF = 1,
  MKI_ICON_ID_FILETYPE_DWG = 2,

  MKI_ICON_ID_PAGEMODE_CUSTOM    = 3,
  MKI_ICON_ID_PAGEMODE_PORTRAIT  = 4,
  MKI_ICON_ID_PAGEMODE_LANDSCAPE = 5,

  MKI_ICON_ID_REFARROW_POLYLINE   = 6,
  MKI_ICON_ID_REFARROW_LINE       = 7,
  MKI_ICON_ID_REFARROW_ARROW_45   = 8,
  MKI_ICON_ID_REFARROW_TEXT_45    = 9,
  MKI_ICON_ID_REFARROW_ARROW_90   = 10,
  MKI_ICON_ID_REFARROW_TEXT_90    = 11,
  MKI_ICON_ID_REFARROW_HORIZONTAL = 12,
  MKI_ICON_ID_REFARROW_VERTICAL   = 13,
  MKI_ICON_ID_REFARROW_NONE       = 14,

  MKI_ICON_ID_GRID_OFF       = 15,
  MKI_ICON_ID_GRID_CARTESIAN = 16,
  MKI_ICON_ID_GRID_ISOMETRIC = 17,
  MKI_ICON_ID_GRID_DIMETRIC1 = 18,
  MKI_ICON_ID_GRID_DIMETRIC2 = 19,

  MKI_ICON_ID_ARROW_L_NONE          = 20,
  MKI_ICON_ID_ARROW_L_TRIANGLE_FILL = 21,
  MKI_ICON_ID_ARROW_L_TRIANGLE      = 22,
  MKI_ICON_ID_ARROW_L_TRIANGLE_OPEN = 23,
  MKI_ICON_ID_ARROW_L_DIAGONAL      = 24,
  MKI_ICON_ID_ARROW_L_CIRCLE_FILL   = 25,
  MKI_ICON_ID_ARROW_L_CIRCLE        = 26,
  MKI_ICON_ID_ARROW_L_CIRCLE_LARGE  = 27,

  MKI_ICON_ID_ARROW_R_NONE          = 30,
  MKI_ICON_ID_ARROW_R_TRIANGLE_FILL = 31,
  MKI_ICON_ID_ARROW_R_TRIANGLE      = 32,
  MKI_ICON_ID_ARROW_R_TRIANGLE_OPEN = 33,
  MKI_ICON_ID_ARROW_R_DIAGONAL      = 34,
  MKI_ICON_ID_ARROW_R_CIRCLE_FILL   = 35,
  MKI_ICON_ID_ARROW_R_CIRCLE        = 36,
  MKI_ICON_ID_ARROW_R_CIRCLE_LARGE  = 37,

  MKI_ICON_ID_REFFRAME_RECTANGLE = 40,
  MKI_ICON_ID_REFFRAME_RHOMB     = 41,
  MKI_ICON_ID_REFFRAME_CIRCLE    = 42,
  MKI_ICON_ID_REFFRAME_ELLIPSE   = 43,
  MKI_ICON_ID_REFFRAME_ROUNDRECT = 44,
  MKI_ICON_ID_REFFRAME_NONE      = 45,
  MKI_ICON_ID_REFFRAME_HALO      = 46,

  MKI_ICON_ID_TEXTORDER_LATIN  = 47,
  MKI_ICON_ID_TEXTORDER_ARABIC = 48,
  MKI_ICON_ID_TEXTORDER_KANJI  = 49,

  MKI_ICON_ID_TEXTMODE_LEFT   = 50,
  MKI_ICON_ID_TEXTMODE_CENTER = 51,
  MKI_ICON_ID_TEXTMODE_RIGHT  = 52,
  MKI_ICON_ID_TEXTMODE_BLOCK  = 53,

  MKI_ICON_ID_FILLMODE_FRAMED        = 54,
  MKI_ICON_ID_FILLMODE_FILLED        = 55,
  MKI_ICON_ID_FILLMODE_FILLED_FRAMED = 56,
  MKI_ICON_ID_FILLMODE_ERASER        = 57,
  MKI_ICON_ID_FILLMODE_ERASER_FRAMED = 58,
  MKI_ICON_ID_FILLMODE_NONE          = 59,

  MKI_ICON_ID_TEXTVERTICAL_EXTENTTOP    = 60,
  MKI_ICON_ID_TEXTVERTICAL_EXTENTCENTER = 61,
  MKI_ICON_ID_TEXTVERTICAL_EXTENTBOTTOM = 62,
  MKI_ICON_ID_TEXTVERTICAL_LINETOP      = 63,
  MKI_ICON_ID_TEXTVERTICAL_LINECENTER   = 64,
  MKI_ICON_ID_TEXTVERTICAL_LINEBOTTOM   = 65,
  MKI_ICON_ID_TEXTVERTICAL_FRAMETOP     = 66,
  MKI_ICON_ID_TEXTVERTICAL_FRAMECENTER  = 67,
  MKI_ICON_ID_TEXTVERTICAL_FRAMEBOTTOM  = 68,

  MKI_ICON_ID_LINECAP_ROUND  = 70,
  MKI_ICON_ID_LINECAP_SQUARE = 71,
  MKI_ICON_ID_LINECAP_FLAT   = 72,

  MKI_ICON_ID_LINEJOIN_ROUND = 73,
  MKI_ICON_ID_LINEJOIN_BEVEL = 74,
  MKI_ICON_ID_LINEJOIN_MITER = 75,

  MKI_ICON_ID_DIMZEROMODE_STRIP = 76,
  MKI_ICON_ID_DIMZEROMODE_ONE   = 77,
  MKI_ICON_ID_DIMZEROMODE_ALL   = 78,

  MKI_ICON_ID_DIMTYPE_STANDARD  = 80,
  MKI_ICON_ID_DIMTYPE_ARBITRARY = 81,
  MKI_ICON_ID_DIMTYPE_HALF      = 82,

  MKI_ICON_ID_DIMLINE_PARALLEL   = 83,
  MKI_ICON_ID_DIMLINE_ARBITRARY  = 84,
  MKI_ICON_ID_DIMLINE_HORIZONTAL = 85,
  MKI_ICON_ID_DIMLINE_VERTICAL   = 86,
  MKI_ICON_ID_DIMLINE_AUTOMATIC  = 87,

  MKI_ICON_ID_DIMEXTLINE_RADIAL   = 88,
  MKI_ICON_ID_DIMEXTLINE_PARALLEL = 89,

  MKI_ICON_ID_DIMDISTANCE_ARBITRARY = 90,
  MKI_ICON_ID_DIMDISTANCE_FIXED     = 91,
  MKI_ICON_ID_DIMDISTANCE_MULTIPLE  = 92,

  MKI_ICON_ID_CORNERMODE_INSIDE  = 93,
  MKI_ICON_ID_CORNERMODE_OUTSIDE = 94,
  MKI_ICON_ID_CORNERMODE_TANGENT = 95,

  MKI_ICON_ID_NOTCHMODE_SINGLE = 96,
  MKI_ICON_ID_NOTCHMODE_DOUBLE = 97,
  MKI_ICON_ID_NOTCHMODE_FRIDGE = 98,

  MKI_ICON_ID_NUMPLACING_NONE   = 100,
  MKI_ICON_ID_NUMPLACING_TIGHT  = 101,
  MKI_ICON_ID_NUMPLACING_INSIDE = 102,
  MKI_ICON_ID_NUMPLACING_START  = 103,
  MKI_ICON_ID_NUMPLACING_END    = 104,

  MKI_ICON_ID_EDGEMODE_ANGULAR    = 105,
  MKI_ICON_ID_EDGEMODE_POINTED_HW = 106,
  MKI_ICON_ID_EDGEMODE_POINTED_HA = 107,
  MKI_ICON_ID_EDGEMODE_POINTED_WA = 108,
  MKI_ICON_ID_EDGEMODE_ROUND      = 109,

  MKI_ICON_ID_WINDOWMODE_1   = 110,
  MKI_ICON_ID_WINDOWMODE_2X  = 111,
  MKI_ICON_ID_WINDOWMODE_2Y  = 112,
  MKI_ICON_ID_WINDOWMODE_3X  = 113,
  MKI_ICON_ID_WINDOWMODE_3Y  = 114,
  MKI_ICON_ID_WINDOWMODE_4   = 115,
  MKI_ICON_ID_WINDOWMODE_4X  = 116,
  MKI_ICON_ID_WINDOWMODE_4Y  = 117,

  MKI_ICON_ID_FOLDER_PLUS  = 118,
  MKI_ICON_ID_FOLDER_MINUS = 119,

  MKI_ICON_ID_SNAPMODE_CENTER       = 120,
  MKI_ICON_ID_SNAPMODE_QUADRANT     = 121,
  MKI_ICON_ID_SNAPMODE_EDGE         = 122,
  MKI_ICON_ID_SNAPMODE_CORNER       = 123,
  MKI_ICON_ID_SNAPMODE_INTERSECTION = 124,
  MKI_ICON_ID_SNAPMODE_OTHERS       = 125,
  MKI_ICON_ID_SNAPMODE_RELATIVE     = 126,

  MKI_ICON_ID_FILTER       = 129,
  MKI_ICON_ID_TEXTOVERFLOW = 130,

  MKI_ICON_ID_COMMENT_PIN1 = 131,
  MKI_ICON_ID_COMMENT_PIN2 = 132,
  MKI_ICON_ID_COMMENT_PIN3 = 133,

  MKI_ICON_ID_PLUGIN          = 134,
  MKI_ICON_ID_PANMARK         = 135,
  MKI_ICON_ID_PLACE_REFERENCE = 136,
  MKI_ICON_ID_PLACE_BITMAP    = 137,
  MKI_ICON_ID_HOURGLASS       = 138,
  MKI_ICON_ID_JOYSTICK        = 139,

  MKI_ICON_ID_F2_ARCORIENT_POSITIVE = 140,
  MKI_ICON_ID_F2_ARCORIENT_NEGATIVE = 141,
  MKI_ICON_ID_F2_ARCMODE_ARC        = 142,
  MKI_ICON_ID_F2_ARCMODE_SECTOR     = 143,
  MKI_ICON_ID_F2_ARCMODE_SEGMENT    = 144,
  MKI_ICON_ID_F4_GRID               = 145,
  MKI_ICON_ID_F5_ORTHO              = 146,
  MKI_ICON_ID_F6_SNAP               = 147,
  MKI_ICON_ID_F6_AREA_INSIDE        = 148,
  MKI_ICON_ID_F6_AREA_PARTIAL       = 149,
  MKI_ICON_ID_F7_DIMORIENT_H        = 150,
  MKI_ICON_ID_F7_DIMORIENT_V        = 151,
  MKI_ICON_ID_F7_MULTILINE          = 152,
  MKI_ICON_ID_F7_DIMINVERT_SMALL    = 153,
  MKI_ICON_ID_F7_DIMINVERT_LARGE    = 154,
  MKI_ICON_ID_F7_DUPLICATE          = 155,
  MKI_ICON_ID_TAB_EXTIDENT          = 156,
  MKI_ICON_ID_TAB_DISPLAY_FILLED    = 157,
  MKI_ICON_ID_TAB_DISPLAY_HOLLOW    = 158,
  MKI_ICON_ID_OVERRIDE_LOCK_IDLE    = 159,

  MKI_ICON_ID_MODE_DISPLAY = 160,
  MKI_ICON_ID_MODE_OUTPUT  = 161,
  MKI_ICON_ID_MODE_LOCK    = 162,
  MKI_ICON_ID_MODE_IDLE    = 163,
  MKI_ICON_ID_MODE_GRAY    = 164,
  MKI_ICON_ID_MODE_FLAG1   = 165,
  MKI_ICON_ID_MODE_FLAG2   = 166,
  MKI_ICON_ID_MODE_FLAG3   = 167,
  MKI_ICON_ID_MODE_FLAG4   = 168,

  MKI_ICON_ID_ESC_PARAMETERS = 169,

  MKI_ICON_ID_CHECK_SMALL    = 170,
  MKI_ICON_ID_CHECK_ACTIVE   = 171,
  MKI_ICON_ID_CHECK_INACTIVE = 172,
  MKI_ICON_ID_ARROW_ACTIVE   = 173,
  MKI_ICON_ID_ARROW_INACTIVE = 174,

  MKI_ICON_ID_SIGN_DANGER = 175,
  MKI_ICON_ID_SIGN_INFO   = 176,
  MKI_ICON_ID_SIGN_PIN    = 177,

  MKI_ICON_ID_MODE_NO_LOCK = 178,
  MKI_ICON_ID_MODE_NO_IDLE = 179,

  MKI_ICON_ID_NEWS_0 = 180,

  MKI_ICON_ID_NEWS_READ_1 = 181,
  MKI_ICON_ID_NEWS_READ_2 = 182,
  MKI_ICON_ID_NEWS_READ_3 = 183,
  MKI_ICON_ID_NEWS_READ_4 = 184,
  MKI_ICON_ID_NEWS_READ_5 = 185,
  MKI_ICON_ID_NEWS_READ_6 = 186,
  MKI_ICON_ID_NEWS_READ_7 = 187,
  MKI_ICON_ID_NEWS_READ_8 = 188,
  MKI_ICON_ID_NEWS_READ_9 = 189,
  MKI_ICON_ID_NEWS_READ_X = 190,

  MKI_ICON_ID_NEWS_UNREAD_1 = 191,
  MKI_ICON_ID_NEWS_UNREAD_2 = 192,
  MKI_ICON_ID_NEWS_UNREAD_3 = 193,
  MKI_ICON_ID_NEWS_UNREAD_4 = 194,
  MKI_ICON_ID_NEWS_UNREAD_5 = 195,
  MKI_ICON_ID_NEWS_UNREAD_6 = 196,
  MKI_ICON_ID_NEWS_UNREAD_7 = 197,
  MKI_ICON_ID_NEWS_UNREAD_8 = 198,
  MKI_ICON_ID_NEWS_UNREAD_9 = 199,
  MKI_ICON_ID_NEWS_UNREAD_X = 200,

  MKI_ICON_ID_NOTES_0 = 201,
  MKI_ICON_ID_NOTES_1 = 202,
  MKI_ICON_ID_NOTES_2 = 203,
  MKI_ICON_ID_NOTES_3 = 204,
  MKI_ICON_ID_NOTES_4 = 205,
  MKI_ICON_ID_NOTES_5 = 206,
  MKI_ICON_ID_NOTES_6 = 207,
  MKI_ICON_ID_NOTES_7 = 208,
  MKI_ICON_ID_NOTES_8 = 209,
  MKI_ICON_ID_NOTES_9 = 210,
  MKI_ICON_ID_NOTES_X = 211,

  MKI_ICON_ID_MARK_0 = 212,
  MKI_ICON_ID_MARK_1 = 213,
  MKI_ICON_ID_MARK_2 = 214,
  MKI_ICON_ID_MARK_3 = 215,
  MKI_ICON_ID_MARK_4 = 216,
  MKI_ICON_ID_MARK_5 = 217,
  MKI_ICON_ID_MARK_6 = 218,
  MKI_ICON_ID_MARK_7 = 219,

  MKI_ICON_ID_ARROW_1_0  = 220,
  MKI_ICON_ID_ARROW_1_1  = 221,
  MKI_ICON_ID_ARROW_1_2  = 222,
  MKI_ICON_ID_ARROW_1_3  = 223,
  MKI_ICON_ID_ARROW_1_4  = 224,
  MKI_ICON_ID_ARROW_1_5  = 225,
  MKI_ICON_ID_ARROW_1_6  = 226,
  MKI_ICON_ID_ARROW_1_7  = 227,
  MKI_ICON_ID_ARROW_1_8  = 228,
  MKI_ICON_ID_ARROW_1_9  = 229,
  MKI_ICON_ID_ARROW_1_10 = 230,
  MKI_ICON_ID_ARROW_1_11 = 231,
  MKI_ICON_ID_ARROW_1_12 = 232,
  MKI_ICON_ID_ARROW_1_13 = 233,
  MKI_ICON_ID_ARROW_1_14 = 234,
  MKI_ICON_ID_ARROW_1_15 = 235,
  MKI_ICON_ID_ARROW_1_16 = 236,
  MKI_ICON_ID_ARROW_1_17 = 237,
  MKI_ICON_ID_ARROW_1_18 = 238,
  MKI_ICON_ID_ARROW_1_19 = 239,
  MKI_ICON_ID_ARROW_1_20 = 240,
  MKI_ICON_ID_ARROW_1_21 = 241,
  MKI_ICON_ID_ARROW_1_22 = 242,
  MKI_ICON_ID_ARROW_1_23 = 243,
  MKI_ICON_ID_ARROW_1_24 = 244,
  MKI_ICON_ID_ARROW_2_0  = 245,
  MKI_ICON_ID_ARROW_2_1  = 246,
  MKI_ICON_ID_ARROW_2_2  = 247,
  MKI_ICON_ID_ARROW_2_3  = 248,
  MKI_ICON_ID_ARROW_2_4  = 249,
  MKI_ICON_ID_ARROW_2_5  = 250,
  MKI_ICON_ID_ARROW_2_6  = 251,
  MKI_ICON_ID_ARROW_2_7  = 252,
  MKI_ICON_ID_ARROW_2_8  = 253,
  MKI_ICON_ID_ARROW_2_9  = 254,
  MKI_ICON_ID_ARROW_2_10 = 255,
  MKI_ICON_ID_ARROW_2_11 = 256,
  MKI_ICON_ID_ARROW_2_12 = 257,
  MKI_ICON_ID_ARROW_2_13 = 258,
  MKI_ICON_ID_ARROW_2_14 = 259,
  MKI_ICON_ID_ARROW_2_15 = 260,
  MKI_ICON_ID_ARROW_2_16 = 261,
  MKI_ICON_ID_ARROW_2_17 = 262,
  MKI_ICON_ID_ARROW_2_18 = 263,
  MKI_ICON_ID_ARROW_2_19 = 264,
  MKI_ICON_ID_ARROW_2_20 = 265,
  MKI_ICON_ID_ARROW_2_21 = 266,
  MKI_ICON_ID_ARROW_2_22 = 267,
  MKI_ICON_ID_ARROW_2_23 = 268,
  MKI_ICON_ID_ARROW_2_24 = 269,

  MKI_ICON_ID_TEXT_R_PARALLEL = 270,
  MKI_ICON_ID_TEXT_R_NONE     = 271,
  MKI_ICON_ID_TEXT_V_BASELINE = 272,
  MKI_ICON_ID_TEXT_V_TOP      = 273,
  MKI_ICON_ID_TEXT_V_BOTTOM   = 274,
  MKI_ICON_ID_TEXT_V_CENTER   = 275,
  MKI_ICON_ID_TEXT_H_LEFT     = 276,
  MKI_ICON_ID_TEXT_H_CENTER   = 277,
  MKI_ICON_ID_TEXT_H_RIGHT    = 278,
  MKI_ICON_ID_TEXT_H_BLOCK    = 279,

  MKI_ICON_ID_LABEL_ISO1302  = 280,
  MKI_ICON_ID_LABEL_ISO1101A = 281,
  MKI_ICON_ID_LABEL_ISO1101B = 282,
  MKI_ICON_ID_LABEL_DIN6784  = 283,
  MKI_ICON_ID_LABEL_DIN332   = 284,

  MKI_ICON_ID_FILL_STRIPES = 285,
  MKI_ICON_ID_FILL_CIRCLES = 286,
  MKI_ICON_ID_FILL_SQUARES = 287,
  MKI_ICON_ID_FILL_RAYS    = 288,

  MKI_ICON_ID_LOCK    = 289,
  MKI_ICON_ID_PIN_OFF = 290,
  MKI_ICON_ID_PIN_ON  = 291,
  MKI_ICON_ID_PLUS    = 292,
  MKI_ICON_ID_RENAME  = 293,

  MKI_ICON_ID_MODE2_DISPLAY = 294,
  MKI_ICON_ID_MODE2_OUTPUT  = 295,
  MKI_ICON_ID_MODE2_LOCK    = 296,
  MKI_ICON_ID_MODE2_IDLE    = 297,
  MKI_ICON_ID_MODE2_GRAY    = 298,
  MKI_ICON_ID_MODE2_FLAG1   = 299,
  MKI_ICON_ID_MODE2_FLAG2   = 300,
  MKI_ICON_ID_MODE2_FLAG3   = 301,
  MKI_ICON_ID_MODE2_FLAG4   = 302,

  // Indirect definitions (duplicate uses)
  MKI_ICON_ID_CROSS            = MKI_ICON_ID_FILLMODE_NONE,
  MKI_ICON_ID_COMMENT_NEW      = MKI_ICON_ID_MODE_DISPLAY,
  MKI_ICON_ID_COMMENT_WORKING  = MKI_ICON_ID_HOURGLASS,
  MKI_ICON_ID_COMMENT_SOLVED   = MKI_ICON_ID_CHECK_ACTIVE,
  MKI_ICON_ID_COMMENT_CRITICAL = MKI_ICON_ID_SIGN_DANGER,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_ICON_DATA
{
  public:

  MKI_BITMAP_PTR  m_pIconBitmap;

  __int32 m_nIconXOffset,
          m_nIconYOffset,
          m_nIconXSize,
          m_nIconYSize,
          m_nIconMode;
};

MKI_CLASS_TYPES( ICON_DATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_MENU_ENTRY
{
  public:

  LPWSTR m_pszDescription,
         m_pszMenuEntry;
};

MKI_CLASS_TYPES( MENU_ENTRY )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_COMMAND_ENTRY
{
  public:

  __int32 m_nCommandMode,
          m_nPointNumber,
          m_anPointTypes   [MKI_INPUT_DEF_MAX],
          m_anPointRelative[MKI_INPUT_DEF_MAX];
  LPWSTR  m_apszPointNames [MKI_INPUT_DEF_MAX];
};

MKI_CLASS_TYPES( COMMAND_ENTRY )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_COMMAND_DATA
{
  public:

  __int32           m_nType;
  MKI_MENU_ENTRY    m_cMenuData;
  MKI_COMMAND_ENTRY m_cInputData;
  MKI_ICON_DATA     m_cIconData;
};

MKI_CLASS_TYPES( COMMAND_DATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_POPUP_ENTRY
{
  public:

  LPWSTR  m_pszText;
  __int32 m_nCommand;
  bool    m_fGrayed,
          m_fChecked;
  __int16 m_nShortcut;
  __int32 m_nSubCount;

  MKI_POPUP_ENTRY* m_pSubList;

};

MKI_CLASS_TYPES( POPUP_ENTRY )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_TOOLTIP_ENTRY
{
  public:

  __int32 m_nCtlID;
  LPCWSTR m_pszText;
};

MKI_CLASS_TYPES( TOOLTIP_ENTRY )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_TOOLTIP_LIST
{

public:

//---------------------------------------------------------------------------------------------------------------------------------------------------
  
  MKI_INTERFACE_MEMBER 
  MKI_TOOLTIP_LIST( void );

  ~MKI_TOOLTIP_LIST()
  {
    MKI_TOOLTIP_LIST::Exit();
  }

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  SetTooltipList( HWND f_hDlg, MKI_TOOLTIP_ENTRY_PTR f_pListOfEntries1, MKI_TOOLTIP_ENTRY_PTR f_pListOfEntries2 = nullptr );

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER bool
  Init( void );

  MKI_INTERFACE_MEMBER void
  Activate( void );

  MKI_INTERFACE_MEMBER void
  Exit( void );

  //------------------------------------------------------------------

  MKI_INTERFACE_MEMBER void
  AddTooltipList( HWND f_hDlg, MKI_TOOLTIP_ENTRY_PTR f_pListOfEntries1, MKI_TOOLTIP_ENTRY_PTR f_pListOfEntries2 = nullptr );

  MKI_INTERFACE_MEMBER bool
  AddRectText( HWND f_hWnd, MKI_INTRECT& f_rRect, LPCWSTR f_szText );

  MKI_INTERFACE_MEMBER bool
  AddControlText( HWND f_hDlg, __int32 f_nCtlID, LPCWSTR f_pszText );

  MKI_INTERFACE_MEMBER bool
  AddControlLayer( HWND f_hDlg, __int32 f_nCtlID, MKI_CONST_LAYERDEF_REF f_rLayer, bool f_fIdentical, bool f_fMulti );

  MKI_INTERFACE_MEMBER bool
  AddControlPen( HWND f_hDlg, __int32 f_nCtlID, MKI_CONST_PENDEF_REF f_rPen, bool f_fIdentical, bool f_fPassive, bool f_fMulti );

  MKI_INTERFACE_MEMBER bool
  AddControlLine( HWND f_hDlg, __int32 f_nCtlID, MKI_CONST_LINEDEF_REF f_rLine, bool f_fMulti );

  MKI_INTERFACE_MEMBER bool
  AddControlSystem( HWND f_hDlg, __int32 f_nCtlID, MKI_CONST_SYSTEMDEF_REF f_rSystem, bool f_fMulti );

  MKI_INTERFACE_MEMBER bool
  AddControlSystem( HWND f_hDlg, __int32 f_nCtlID, __int32 f_nSystem, bool f_fMulti );

  MKI_INTERFACE_MEMBER bool
  AddControlHatch( HWND f_hDlg, __int32 f_nCtlID, MKI_CONST_HATCHDEF_REF f_rHatch, bool f_fMulti );

  MKI_INTERFACE_MEMBER bool
  AddControlModelOrPage( HWND f_hDlg, __int32 f_nCtlID, MKI_CONST_PAGEDEF_REF f_rPage, bool f_fMulti );

  MKI_INTERFACE_MEMBER bool
  AddControlModelOrPage( HWND f_hDlg, __int32 f_nCtlID, MKI_CONST_MODELDEF_REF f_rModel, bool f_fMulti );

  MKI_INTERFACE_MEMBER bool
  AddControlModelOrPage( HWND f_hDlg, __int32 f_nCtlID, __int32 f_nPageIndex, bool f_fMulti );

  //------------------------------------------------------------------

  HWND    m_hTooltip;   // Tooltip handle
  __int32 m_nMaxLength; // Maximum text length
};

MKI_CLASS_TYPES( TOOLTIP_LIST )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_BUTTONSTATE_NULL       = 0x00000000,
  MKI_BUTTONSTATE_PRESSED    = 0x00000001,
  MKI_BUTTONSTATE_IDLE       = 0x00000002,
  MKI_BUTTONSTATE_DOWN       = 0x00000004,
  MKI_BUTTONSTATE_DEFAULT    = 0x00000008,
  MKI_BUTTONSTATE_COMMAND    = 0x00000010,
  MKI_BUTTONSTATE_POPUP      = 0x00000020,
  MKI_BUTTONSTATE_CHECKED    = 0x00000040,
  MKI_BUTTONSTATE_HIGHLIGHT  = 0x00000080,

  MKI_BUTTONSTATE_NO_LBUTTON = 0x00001000,
  MKI_BUTTONSTATE_NO_MBUTTON = 0x00002000,
  MKI_BUTTONSTATE_NO_RBUTTON = 0x00004000,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_BUTTON_DATA
{
  public:

  __int32 m_nType,
          m_nButtonState;
  LPWSTR  m_pszButtonText,
          m_pszOptionText;

  MKI_ICON_DATA m_cIconData;
};

MKI_CLASS_TYPES( BUTTON_DATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_EXTENDED_DATA
{
  public:

  __int16 m_nOwnerID;
  __int32 m_nExtType;
  LPWSTR  m_pszExtTitle;
};

MKI_CLASS_TYPES( EXTENDED_DATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_CONTEXT_MENU
{
  public:

  MKI_CONTEXT_MENU( void )
  {
    MKI_CONTEXT_MENU::Init();
  }

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_POINT m_cPoint;

  __int32 m_nCount,
          m_anCommandID[MKI_PLUGIN_CONTEXT_MAX];
  LPWSTR  m_aszMenuText[MKI_PLUGIN_CONTEXT_MAX];
};

MKI_CLASS_TYPES( CONTEXT_MENU )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_CONTEXT_COMMAND
{
  public:

  MKI_CONTEXT_COMMAND( void )
  {
    MKI_CONTEXT_COMMAND::Init();
  }

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  MKI_POINT m_cPoint;
  __int32   m_nCommandID;
};

MKI_CLASS_TYPES( CONTEXT_COMMAND )


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PLUGIN_PROC
{
  public:

  MKI_INPUTPOINTINIT_PROC m_pfnInputPointInitProc;
  MKI_INPUTPOINTMOVE_PROC m_pfnInputPointMoveProc;
  MKI_INPUTPOINTEXIT_PROC m_pfnInputPointExitProc;
  MKI_INPUTDISPLAY_PROC   m_pfnInputDisplayProc;
  MKI_INPUTPARAMETER_PROC m_pfnInputParameterProc;
  MKI_INPUTDIALOG_PROC    m_pfnInputDialogProc;
  MKI_INPUTCANCEL_PROC    m_pfnInputCancelProc;
  MKI_INPUTFINISH_PROC    m_pfnInputFinishProc;
};

MKI_CLASS_TYPES( PLUGIN_PROC )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_EXPORTCAPABILITIES_BASIC  = 0x00000000,
  MKI_EXPORTCAPABILITIES_LAYOUT = 0x00000001,
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PLUGIN_ID
{
  public:

  MKI_INTERFACE_MEMBER void
  Init( void );

  //------------------------------------------------------------------

  __int16 m_nOwnerID,
          m_nPlugInID;
  __int32 m_nAppType;

  MKI_PLUGIN_CONTAINER_PTR m_pContainer;
  MKI_PLUGIN_PROC          m_cPlugInProc;
  MKI_COMMAND_DATA         m_cPlugInData;
  MKI_COMMAND_DATA_PTR     m_pCommandData;
  MKI_BUTTON_DATA_PTR      m_pButtonData;
  MKI_EXTENDED_DATA_PTR    m_pExtendedData;

  LPWSTR m_pszFileType,
         m_pszFileExtension,
         m_pszFileDefault;

  __int32 m_nOptionsOwnerID,
          m_nOptionsItemType,
          m_nExportCapabilities;
};

MKI_CLASS_TYPES( PLUGIN_ID )


//---------------------------------------------------------------------------------------------------------------------------------------------------
typedef __int32 (*MKI_PLUGININIT_PROC)( HINSTANCE f_hMainInst, HWND f_hMainWnd, __int32 f_nInterfaceVersion, MKI_PLUGIN_ID_REF f_rPlugInID );
typedef bool (*MKI_PLUGINEXIT_PROC)( bool f_fSaveSettings );


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_PLUGIN_DATA
{
  public:

  MKI_PLUGIN_ID          m_cID;
  HINSTANCE              m_hHandle;
  MKI_PLUGININIT_PROC    m_pfnProcInit;
  MKI_PLUGINEXIT_PROC    m_pfnProcExit;
  MKI_PLUGINCOMMAND_PROC m_pfnProcCommand;
  MKI_PLUGINNOTIFY_PROC  m_pfnProcNotify;
  MKI_PLUGINIMPORT_PROC  m_pfnProcImport;
  MKI_PLUGINEXPORT_PROC  m_pfnProcExport;
  MKI_PLUGINREQUEST_PROC m_pfnProcRequest;
};

MKI_CLASS_TYPES( PLUGIN_DATA )


//---------------------------------------------------------------------------------------------------------------------------------------------------
enum
{
  MKI_CONTEXT_PINBOARD_PUT      = 1,
  MKI_CONTEXT_PINBOARD_GET      = 2,
  MKI_CONTEXT_COLLECTION_PUT    = 3,
  MKI_CONTEXT_COLLECTION_GET    = 4,
  MKI_CONTEXT_COLLECTION_MANAGE = 5,
};

#ifndef MKI_SERVER      // Only visible outside the serving application!

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlA( WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlB( WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlC( WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlD( WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlE( WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlF( WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlG( WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlH( WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlI( WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlJ( WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlK( WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlL( WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlM( WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlN( WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlO( WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlP( WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlQ( WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlR( WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlS( WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlT( WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlU( WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlV( WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlW( WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlX( WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlY( WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_KeyIsCtrlZ( WPARAM f_wParam, LPARAM f_lParam );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER void*
MKI_HeapAlloc( const size_t f_unSize );

MKI_INTERFACE_MEMBER void
MKI_HeapFree( void* f_pData );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER void
MKI_EntityCloseAll( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_OBJECT_PTR
MKI_ObjectOpen( __int32 f_nType );

MKI_INTERFACE_MEMBER MKI_ENTITY_OBJECT_PTR
MKI_ObjectGetAddress( void );

MKI_INTERFACE_MEMBER __int32
MKI_ObjectGetFree( void );

MKI_INTERFACE_MEMBER bool
MKI_ObjectAddDataBlock( __int32 f_nType, __int16 f_nElemType, __int16 f_nElemCount, const void* const f_pData, __int32 f_nBytes );

MKI_INTERFACE_MEMBER bool
MKI_ObjectAddPoint( __int32 f_nType, double f_dX, double f_dY );

MKI_INTERFACE_MEMBER bool
MKI_ObjectAddPoint( __int32 f_nType, MKI_CONST_POINT_REF f_rPoint );

MKI_INTERFACE_MEMBER bool
MKI_ObjectAddInt32( __int32 f_nType, __int32 f_nValue );

MKI_INTERFACE_MEMBER bool
MKI_ObjectAddInt32( __int32 f_nType, __int32* f_pElemData, __int16 f_nElemCount );

MKI_INTERFACE_MEMBER bool
MKI_ObjectAddDouble( __int32 f_nType, double f_dValue );

MKI_INTERFACE_MEMBER bool
MKI_ObjectAddDouble( __int32 f_nType, double* f_pElemData, __int16 f_nElemCount );

MKI_INTERFACE_MEMBER bool
MKI_ObjectAddConstant( double f_dValue );

MKI_INTERFACE_MEMBER bool
MKI_ObjectAddOrient( double f_dOrient );

MKI_INTERFACE_MEMBER bool
MKI_ObjectAddCurve( double f_dOrient, double f_dCurve );

MKI_INTERFACE_MEMBER bool
MKI_ObjectAddTextShort( const LPCWSTR f_pszText, bool f_fStaticLength );

MKI_INTERFACE_MEMBER bool
MKI_ObjectAddTextLong( const LPCWSTR f_pszText );

MKI_INTERFACE_MEMBER bool
MKI_ObjectAddXProperty( __int32 f_nType, MKI_CONST_XPROPERTY_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_ObjectAddDimLine( MKI_CONST_DIMLINE_REF f_rData );

MKI_INTERFACE_MEMBER MKI_XPROPERTY_PTR
MKI_ObjectAddDimSmall( MKI_CONST_DIMSMALL_REF f_rData, bool f_fUseGlobal, bool f_fIsAngle );

MKI_INTERFACE_MEMBER MKI_XPROPERTY_PTR
MKI_ObjectAddDimLarge( MKI_CONST_DIMLARGE_REF f_rData, bool f_fUseGlobal, bool f_fIsAngle );

MKI_INTERFACE_MEMBER MKI_XPROPERTY_PTR
MKI_ObjectAddTextStandard( MKI_CONST_TEXTSTANDARD_REF f_rData );

MKI_INTERFACE_MEMBER MKI_XPROPERTY_PTR
MKI_ObjectAddTextFrame( MKI_CONST_TEXTFRAME_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_ObjectAddTextReference( MKI_CONST_TEXTREFERENCE_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_ObjectAddCommentData( MKI_CONST_COMMENTDATA_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_ObjectAddBitmapRef( MKI_CONST_BITMAPREF_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_ObjectAddAttribute( __int32 f_nType, const LPCWSTR f_pszName, const LPCWSTR f_pszText );

MKI_INTERFACE_MEMBER bool
MKI_ObjectCopyOneDataBlock( MKI_CONST_BLOCK_ENUM_REF f_rEnum );

MKI_INTERFACE_MEMBER bool
MKI_ObjectCopyAllDataBlocks( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_ObjectCopyPointDataBlocks( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_ObjectCopyNonAttribDataBlocks( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_ObjectCopyNonBaseAttribDataBlocks( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_ObjectCopyBaseAttributes( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_ObjectCopyOutlineAttributes( MKI_CONST_ENTITY_PTR f_pRefEntity, __int32 f_nOutlineIndex );

MKI_INTERFACE_MEMBER bool
MKI_ObjectCopyPointAttributes( MKI_CONST_ENTITY_PTR f_pRefEntity, __int32 f_nPointHandle );

MKI_INTERFACE_MEMBER bool
MKI_ObjectAddEnd( void );

MKI_INTERFACE_MEMBER bool
MKI_ObjectAddPreliminaryEnd( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_OBJECT_PTR
MKI_ObjectAddContainer( MKI_ENTITY_OBJECT_PTR f_pRefObj, MKI_ENTITY_OBJECT_PTR f_pNewObj );

MKI_INTERFACE_MEMBER bool
MKI_ObjectCopyLinkList( MKI_ENTITY_OBJECT_PTR f_pRefObj, MKI_ENTITY_PTR f_pFirstObj );

MKI_INTERFACE_MEMBER bool
MKI_ObjectUpdateFrame( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_OBJECT_PTR
MKI_ObjectInsert( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER MKI_ENTITY_OBJECT_PTR
MKI_ObjectReplace( MKI_CONST_ENTITY_PTR f_pRefEntity, bool f_fDuplicate );

MKI_INTERFACE_MEMBER void
MKI_ObjectClose( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_OBJECT_PTR
MKI_ObjectFastInsert( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER MKI_ENTITY_CLIP_PTR
MKI_ClipOpen( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_CLIP_PTR
MKI_ClipGetAddress( void );

MKI_INTERFACE_MEMBER __int32
MKI_ClipGetFree( void );

MKI_INTERFACE_MEMBER __int32
MKI_ClipGetListCount( void );

MKI_INTERFACE_MEMBER bool
MKI_ClipAddPoint( __int32 f_nType, double f_dX, double f_dY );

MKI_INTERFACE_MEMBER bool
MKI_ClipAddPoint( __int32 f_nType, MKI_CONST_POINT_REF f_rPoint );

MKI_INTERFACE_MEMBER bool
MKI_ClipAddCurve( double f_dOrient, double f_dCurve );

MKI_INTERFACE_MEMBER bool
MKI_ClipAddAttribute( __int32 f_nType, const LPCWSTR f_pszName, const LPCWSTR f_pszText );

MKI_INTERFACE_MEMBER bool
MKI_ClipCopyAllDataBlocks( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_ClipCopyPointDataBlocks( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_ClipCopyNonAttribDataBlocks( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_ClipCopyNonBaseAttribDataBlocks( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_ClipCopyBaseAttributes( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_ClipCopyOutlineAttributes( MKI_CONST_ENTITY_PTR f_pRefEntity, __int32 f_nOutlineIndex );

MKI_INTERFACE_MEMBER bool
MKI_ClipCopyPointAttributes( MKI_CONST_ENTITY_PTR f_pRefEntity, __int32 f_nPointHandle );

MKI_INTERFACE_MEMBER bool
MKI_ClipAddEnd( void );

MKI_INTERFACE_MEMBER bool
MKI_ClipAddPreliminaryEnd( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_CLIP_PTR
MKI_ClipInsert( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER void
MKI_ClipClose( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_CLIP_PTR
MKI_ClipFastInsert( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER MKI_ENTITY_REF_PTR
MKI_RefOpen( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_REF_PTR
MKI_RefGetAddress( void );

MKI_INTERFACE_MEMBER __int32
MKI_RefGetFree( void );

MKI_INTERFACE_MEMBER bool
MKI_RefAddAttribute( __int32 f_nType, const LPCWSTR f_pszName, const LPCWSTR f_pszText );

MKI_INTERFACE_MEMBER bool
MKI_RefCopyBaseAttributes( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_RefAddEnd( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_REF_PTR
MKI_RefInsert( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER void
MKI_RefClose( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_REF_PTR
MKI_RefFastInsert( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER MKI_ENTITY_MAPPING_PTR
MKI_MappingOpen( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_MAPPING_PTR
MKI_MappingGetAddress( void );

MKI_INTERFACE_MEMBER __int32
MKI_MappingGetFree( void );

MKI_INTERFACE_MEMBER bool
MKI_MappingAddPoint( __int32 f_nType, double f_dX, double f_dY );

MKI_INTERFACE_MEMBER bool
MKI_MappingAddPoint( __int32 f_nType, MKI_CONST_POINT_REF f_rPoint );

MKI_INTERFACE_MEMBER bool
MKI_MappingAddCurve( double f_dOrient, double f_dCurve );

MKI_INTERFACE_MEMBER bool
MKI_MappingAddAttribute( __int32 f_nType, const LPCWSTR f_pszName, const LPCWSTR f_pszValue );

MKI_INTERFACE_MEMBER bool
MKI_MappingCopyAllDataBlocks( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_MappingCopyPointDataBlocks( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_MappingCopyNonAttribDataBlocks( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_MappingCopyNonBaseAttribDataBlocks( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_MappingCopyBaseAttributes( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_MappingCopyOutlineAttributes( MKI_CONST_ENTITY_PTR f_pRefEntity, __int32 f_nOutlineIndex );

MKI_INTERFACE_MEMBER bool
MKI_MappingCopyPointAttributes( MKI_CONST_ENTITY_PTR f_pRefEntity, __int32 f_nPointHandle );

MKI_INTERFACE_MEMBER bool
MKI_MappingAddEnd( void );

MKI_INTERFACE_MEMBER bool
MKI_MappingAddPreliminaryEnd( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_MAPPING_PTR
MKI_MappingInsert( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER void
MKI_MappingClose( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_MAPPING_PTR
MKI_MappingFastInsert( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER MKI_ENTITY_HATCH_PTR
MKI_HatchOpen( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_HATCH_PTR
MKI_HatchGetAddress( void );

MKI_INTERFACE_MEMBER __int32
MKI_HatchGetFree( void );

MKI_INTERFACE_MEMBER bool
MKI_HatchAddPoint( __int32 f_nType, double f_dX, double f_dY );

MKI_INTERFACE_MEMBER bool
MKI_HatchAddPoint( __int32 f_nType, MKI_CONST_POINT_REF f_rPoint );

MKI_INTERFACE_MEMBER bool
MKI_HatchAddCurve( double f_dOrient, double f_dCurve );
  
MKI_INTERFACE_MEMBER bool
MKI_HatchCopyOneDataBlock( MKI_CONST_BLOCK_ENUM_REF f_rEnum );

MKI_INTERFACE_MEMBER bool
MKI_HatchAddAttribute( __int32 f_nType, const LPCWSTR f_pszName, const LPCWSTR f_pszValue );

MKI_INTERFACE_MEMBER bool
MKI_HatchCopyAllDataBlocks( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_HatchCopyPointDataBlocks( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_HatchCopyNonAttribDataBlocks( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_HatchCopyNonBaseAttribDataBlocks( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_HatchCopyBaseAttributes( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_HatchCopyOutlineAttributes( MKI_CONST_ENTITY_PTR f_pRefEntity, __int32 f_nOutlineIndex );

MKI_INTERFACE_MEMBER bool
MKI_HatchCopyPointAttributes( MKI_CONST_ENTITY_PTR f_pRefEntity, __int32 f_nPointHandle );

MKI_INTERFACE_MEMBER bool
MKI_HatchAddEnd( void );

MKI_INTERFACE_MEMBER bool
MKI_HatchAddPreliminaryEnd( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_HATCH_PTR
MKI_HatchInsert( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER void
MKI_HatchClose( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_HATCH_PTR
MKI_HatchFastInsert( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER MKI_ENTITY_EXT_PTR
MKI_ExtOpen( __int16 f_nOwnerID, __int32 f_nType, __int32 f_nFlag, __int32 f_nVersion );

MKI_INTERFACE_MEMBER MKI_ENTITY_EXT_PTR
MKI_ExtDataOpen( MKI_ENTITY_EXT_PTR f_pExt );

MKI_INTERFACE_MEMBER MKI_ENTITY_EXT_PTR
MKI_ExtListOpen( MKI_ENTITY_EXT_PTR f_pExt );

MKI_INTERFACE_MEMBER MKI_ENTITY_EXT_PTR
MKI_ExtGetAddress( void );

MKI_INTERFACE_MEMBER __int32
MKI_ExtGetFree( void );

MKI_INTERFACE_MEMBER bool
MKI_ExtAddDataBlock( __int32 f_nType, __int16 f_nElemType, __int16 f_nElemCount, const void* const f_pData, __int32 f_nBytes );

MKI_INTERFACE_MEMBER bool
MKI_ExtAddBinary( __int32 f_nType, const void* const f_pData, const size_t f_nSize );

MKI_INTERFACE_MEMBER bool
MKI_ExtAddPoint( __int32 f_nType, double f_dX, double f_dY );

MKI_INTERFACE_MEMBER bool
MKI_ExtAddPoint( __int32 f_nType, MKI_CONST_POINT_REF f_rPoint );

MKI_INTERFACE_MEMBER bool
MKI_ExtAddInt32( __int32 f_nType, __int32 f_nValue );

MKI_INTERFACE_MEMBER bool
MKI_ExtAddInt32( __int32 f_nType, __int32* f_pElemData, __int16 f_nElemCount );

MKI_INTERFACE_MEMBER bool
MKI_ExtAddDouble( __int32 f_nType, double f_dValue );

MKI_INTERFACE_MEMBER bool
MKI_ExtAddDouble( __int32 f_nType, double* f_pElemData, __int16 f_nElemCount );

MKI_INTERFACE_MEMBER bool
MKI_ExtAddText( __int32 f_nType, const LPCWSTR f_pszText, const size_t f_nSize, bool f_fStaticLength );

MKI_INTERFACE_MEMBER bool
MKI_ExtAddXProperty( __int32 f_nType, MKI_CONST_XPROPERTY_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_ExtAddAttribute( __int32 f_nType, const LPCWSTR f_pszName, const LPCWSTR f_pszValue );

MKI_INTERFACE_MEMBER bool
MKI_ExtCopyAllDataBlocks( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_ExtCopyPointDataBlocks( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_ExtCopyNonAttribDataBlocks( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_ExtCopyNonBaseAttribDataBlocks( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_ExtCopyBaseAttributes( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_ExtCopyOutlineAttributes( MKI_CONST_ENTITY_PTR f_pRefEntity, __int32 f_nOutlineIndex );

MKI_INTERFACE_MEMBER bool
MKI_ExtCopyPointAttributes( MKI_CONST_ENTITY_PTR f_pRefEntity, __int32 f_nPointHandle );

MKI_INTERFACE_MEMBER bool
MKI_ExtAddEnd( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_OBJECT_PTR
MKI_ExtDataAddObject( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_OBJECT_PTR
MKI_ExtDataAddContainer( MKI_ENTITY_OBJECT_PTR f_pObj );

MKI_INTERFACE_MEMBER MKI_ENTITY_PTR
MKI_ExtDataAddEntity( MKI_ENTITY_PTR f_pEntity );

MKI_INTERFACE_MEMBER MKI_ENTITY_PTR
MKI_ExtListAddEntity( MKI_ENTITY_PTR f_pEntity );

MKI_INTERFACE_MEMBER bool
MKI_ExtCopyDataList( MKI_CONST_ENTITY_EXT_PTR f_pRefExt );

MKI_INTERFACE_MEMBER bool
MKI_ExtCopyLinkList( MKI_CONST_ENTITY_EXT_PTR f_pRefExt );

MKI_INTERFACE_MEMBER MKI_ENTITY_EXT_PTR
MKI_ExtInsert( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER void
MKI_ExtClose( void );

MKI_INTERFACE_MEMBER void
MKI_ExtDataClose( bool f_fReplace );

MKI_INTERFACE_MEMBER void
MKI_ExtListClose( bool f_fReplace );

MKI_INTERFACE_MEMBER MKI_ENTITY_EXT_PTR
MKI_ExtFastInsert( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER MKI_ENTITY_EXT_PTR
MKI_ExtTempListOpen( __int16 f_nPlugInNum );

MKI_INTERFACE_MEMBER MKI_ENTITY_PTR
MKI_ExtTempListAddEntity( MKI_ENTITY_EXT_PTR f_pTemp, MKI_ENTITY_PTR f_pEntity );

MKI_INTERFACE_MEMBER void
MKI_ExtTempListDelete( MKI_ENTITY_PTR f_pList );

MKI_INTERFACE_MEMBER void
MKI_ExtTempListClose( MKI_ENTITY_EXT_PTR f_pTemp );

MKI_INTERFACE_MEMBER MKI_ENTITY_INSTANCE_PTR
MKI_InstanceOpen( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_INSTANCE_PTR
MKI_InstanceGetAddress( void );

MKI_INTERFACE_MEMBER __int32
MKI_InstanceGetFree( void );

MKI_INTERFACE_MEMBER bool
MKI_InstanceAddAttribute( __int32 f_nType, const LPCWSTR f_pszName, const LPCWSTR f_pszValue );

MKI_INTERFACE_MEMBER bool
MKI_InstanceCopyBaseAttributes( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_InstanceAddEnd( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_INSTANCE_PTR
MKI_InstanceInsert( MKI_CONST_ENTITY_PTR f_pRefEntity, bool f_fEmbedded );

MKI_INTERFACE_MEMBER void
MKI_InstanceClose( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_INSTANCE_PTR
MKI_InstanceFastInsert( MKI_CONST_ENTITY_PTR f_pRefEntity, bool f_fEmbedded );

MKI_INTERFACE_MEMBER MKI_ENTITY_BLOCK_PTR
MKI_BlockOpen( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_BLOCK_PTR
MKI_BlockGetAddress( void );

MKI_INTERFACE_MEMBER __int32
MKI_BlockGetFree( void );

MKI_INTERFACE_MEMBER bool
MKI_BlockAddAttribute( __int32 f_nType, const LPCWSTR f_pszName, const LPCWSTR f_pszValue );

MKI_INTERFACE_MEMBER bool
MKI_BlockCopyBaseAttributes( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER MKI_ENTITY_PTR
MKI_BlockAddEntity( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_BlockAddEnd( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_BLOCK_PTR
MKI_BlockInsert( const LPCWSTR f_pszLibraryName, bool f_fBoosted );

MKI_INTERFACE_MEMBER void
MKI_BlockClose( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_BLOCK_PTR
MKI_BlockFastInsert( const LPCWSTR f_pszLibraryName, bool f_fBoosted );

MKI_INTERFACE_MEMBER void
MKI_BlockUpdateList( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_GROUP_PTR
MKI_GroupOpen( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_GROUP_PTR
MKI_GroupGetAddress( void );

MKI_INTERFACE_MEMBER __int32
MKI_GroupGetFree( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_PTR
MKI_GroupAddEntity( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_GroupAddAttribute( __int32 f_nType, const LPCWSTR f_pszName, const LPCWSTR f_pszValue );

MKI_INTERFACE_MEMBER bool
MKI_GroupCopyBaseAttributes( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_GroupAddEnd( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_GROUP_PTR
MKI_GroupInsert( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER void
MKI_GroupClose( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_GROUP_PTR
MKI_GroupFastInsert( MKI_CONST_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_PatchInsert( __int32 f_nPointNum, __int32 f_nInvisible, MKI_CONST_POINT_PTR f_pPoints, MKI_CONST_PATCHPROPERTY_REF f_rProperty );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER __int32
MKI_BitmapGetNumber( void );

MKI_INTERFACE_MEMBER bool
MKI_BitmapGetName( __int32 f_nBitmapIndex, MKI_FILENAMEW& f_rBitmapName );

MKI_INTERFACE_MEMBER bool
MKI_BitmapCalcFrame( __int32 f_nBitmapIndex, MKI_CONST_MATRIX_PTR f_pMatrix, MKI_RECT_REF f_rFrame );

MKI_INTERFACE_MEMBER bool
MKI_BitmapGetData( __int32 f_nBitmapIndex, MKI_BITMAP_DATA_REF f_rBitmapData );

MKI_INTERFACE_MEMBER __int32
MKI_BitmapGetIndex( const LPCWSTR f_pszBitmapName );

MKI_INTERFACE_MEMBER bool
MKI_BitmapReadFromFile( const LPCWSTR f_pszFileName, MKI_FILENAMEW& f_rszLocalName, bool f_fEmbedded );

MKI_INTERFACE_MEMBER bool
MKI_BitmapWriteToFile( __int32 f_nBitmapIndex, const LPCWSTR f_pszFileName, __int32 f_nFileType );

MKI_INTERFACE_MEMBER bool
MKI_BitmapDelete( __int32 f_nBitmapIndex );

MKI_INTERFACE_MEMBER __int32
MKI_BitmapLibraryGetIndex( const LPCWSTR f_pszLibraryName, const LPCWSTR f_pszBitmapName );

MKI_INTERFACE_MEMBER __int32
MKI_BitmapLibraryGetNumber( const LPCWSTR f_pszLibraryName );

MKI_INTERFACE_MEMBER bool
MKI_BitmapLibraryCalcFrame( const LPCWSTR f_pszLibraryName, __int32 f_nBitmapIndex, MKI_CONST_MATRIX_PTR f_pMatrix, MKI_RECT_REF f_rFrame );

MKI_INTERFACE_MEMBER bool
MKI_BitmapLibraryGetName( const LPCWSTR f_pszLibraryName, __int32 f_nBitmapIndex, MKI_FILENAMEW& f_rBitmapName );

MKI_INTERFACE_MEMBER bool
MKI_BitmapLibraryGetData( const LPCWSTR f_pszLibraryName, __int32 f_nBitmapIndex, MKI_BITMAP_DATA_REF f_rBitmapData );

MKI_INTERFACE_MEMBER bool
MKI_BitmapLibraryReadFromFile( const LPCWSTR f_pszLibraryName, const LPCWSTR f_pszFileName, MKI_FILENAMEW& f_rszLocalName, bool f_fEmbedded );

MKI_INTERFACE_MEMBER bool
MKI_BitmapLibraryWriteToFile( const LPCWSTR f_pszLibraryName, __int32 f_nBitmapIndex, const LPCWSTR f_pszFileName, __int32 f_nFileType );

MKI_INTERFACE_MEMBER bool
MKI_BitmapLibraryDelete( const LPCWSTR f_pszLibraryName, __int32 f_nBitmapIndex );

MKI_INTERFACE_MEMBER MKI_BITMAP_PTR
MKI_BitmapLoadImage( const LPCWSTR f_pszBitmapName );

MKI_INTERFACE_MEMBER HBITMAP
MKI_BitmapFlattenImage( MKI_BITMAP_PTR f_pBitmap, __int32 f_nSourceX, __int32 f_nSourceY, __int32 f_nSourceDX, __int32 f_nSourceDY,
                                                  __int32 f_nDestinDX, __int32 f_nDestinDY, MKI_COLORVALUE f_cBackColor );

MKI_INTERFACE_MEMBER void
MKI_BitmapFreeImage( MKI_BITMAP_PTR f_pBitmap );

MKI_INTERFACE_MEMBER MKI_BITMAP_PTR
MKI_BitmapLoadResourcePNG( HINSTANCE f_hInstance, const LPCSTR f_pszResName );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER bool
MKI_ModelGet( MKI_MODELDEF_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_ModelSet( MKI_CONST_MODELDEF_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_PageGetByIndex( __int32 f_nIndex, MKI_PAGEDEF_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_PageGetQuadRect( __int32 f_nIndex, MKI_QUAD_PTR f_pQuad, MKI_RECT_PTR f_pRect );

MKI_INTERFACE_MEMBER bool
MKI_PageSetByIndex( __int32 f_nIndex, MKI_CONST_PAGEDEF_REF f_rData );

MKI_INTERFACE_MEMBER __int32
MKI_PageGetActive( void );

MKI_INTERFACE_MEMBER bool
MKI_PageSetActive( __int32 f_nIndex, bool f_fRedraw, bool f_fResetZoom );

MKI_INTERFACE_MEMBER __int32
MKI_PageGetNumber( void );

MKI_INTERFACE_MEMBER __int32
MKI_PageAdd( MKI_CONST_PAGEDEF_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_PageDelete( __int32 f_nIndex );

MKI_INTERFACE_MEMBER bool
MKI_HatchGetOrigin( MKI_POINT_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_HatchGetName( __int32 f_nIndex, MKI_HATCHNAME& f_rName );

MKI_INTERFACE_MEMBER __int32
MKI_HatchGetIndex( const LPCWSTR f_pszName );

MKI_INTERFACE_MEMBER bool
MKI_HatchGetByIndex( __int32 f_nIndex, MKI_HATCHDEF_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_HatchSetByIndex( __int32 f_nIndex, MKI_CONST_HATCHDEF_REF f_rData );

MKI_INTERFACE_MEMBER __int32
MKI_HatchGetActive( void );

MKI_INTERFACE_MEMBER bool
MKI_HatchSetActive( __int32 f_nIndex );

MKI_INTERFACE_MEMBER __int32
MKI_HatchGetNumber( void );

MKI_INTERFACE_MEMBER __int32
MKI_SystemGetCurrent( MKI_SYSTEMDEF_REF f_rData );

MKI_INTERFACE_MEMBER __int32
MKI_SystemGetIndex( const LPCWSTR f_pszName );

MKI_INTERFACE_MEMBER __int32
MKI_SystemAdd( MKI_CONST_SYSTEMDEF_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_SystemGetName( __int32 f_nIndex, MKI_SYSTEMNAME& f_rName );

MKI_INTERFACE_MEMBER bool
MKI_SystemGetByIndex( __int32 f_nIndex, MKI_SYSTEMDEF_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_SystemSetByIndex( __int32 f_nIndex, MKI_CONST_SYSTEMDEF_REF f_rData );

MKI_INTERFACE_MEMBER __int32
MKI_SystemGetActive( __int32 f_nWindowNum );

MKI_INTERFACE_MEMBER bool
MKI_SystemSetActive( __int32 f_nWindowNum, __int32 f_nIndex );

MKI_INTERFACE_MEMBER __int32
MKI_SystemGetNumber( void );

MKI_INTERFACE_MEMBER bool
MKI_SystemGetLibraryByIndex( const LPCWSTR f_pszLibraryName, __int32 f_nIndex, MKI_SYSTEMDEF_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_SystemSetLibraryByIndex( const LPCWSTR f_pszLibraryName, __int32 f_nIndex, MKI_CONST_SYSTEMDEF_REF f_rData );

MKI_INTERFACE_MEMBER __int32
MKI_PenGetBaseMode( void );

MKI_INTERFACE_MEMBER bool
MKI_PenSetBaseMode( __int32 f_nMode );

MKI_INTERFACE_MEMBER __int32
MKI_PenGetIndex( const LPCWSTR f_pszName );

MKI_INTERFACE_MEMBER __int32
MKI_PenAdd( MKI_CONST_PENDEF_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_PenGetName( __int32 f_nIndex, MKI_PENNAME& f_rName );

MKI_INTERFACE_MEMBER bool
MKI_PenGetByIndex( __int32 f_nIndex, MKI_PENDEF_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_PenSetByIndex( __int32 f_nIndex, MKI_CONST_PENDEF_REF f_rData );

MKI_INTERFACE_MEMBER __int32
MKI_PenGetActive( void );

MKI_INTERFACE_MEMBER bool
MKI_PenSetActive( __int32 f_nIndex, bool f_fApply );

MKI_INTERFACE_MEMBER __int32
MKI_PenGetNumber( void );

MKI_INTERFACE_MEMBER __int32
MKI_PenGetLibraryIndex( const LPCWSTR f_pszLibraryName, const LPCWSTR f_pszName );

MKI_INTERFACE_MEMBER bool
MKI_PenGetLibraryByIndex( const LPCWSTR f_pszLibraryName, __int32 f_nIndex, MKI_PENDEF_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_PenSetLibraryByIndex( const LPCWSTR f_pszLibraryName, __int32 f_nIndex, MKI_CONST_PENDEF_REF f_rData );

MKI_INTERFACE_MEMBER __int32
MKI_LineGetIndex( const LPCWSTR f_pszName );

MKI_INTERFACE_MEMBER __int32
MKI_LineAdd( MKI_CONST_LINEDEF_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_LineGetName( __int32 f_nIndex, MKI_LINENAME& f_rName );

MKI_INTERFACE_MEMBER bool
MKI_LineGetByIndex( __int32 f_nIndex, MKI_LINEDEF_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_LineSetByIndex( __int32 f_nIndex, MKI_CONST_LINEDEF_REF f_rData );

MKI_INTERFACE_MEMBER __int32
MKI_LineGetNumber( void );

MKI_INTERFACE_MEMBER bool
MKI_LineGetLibraryByIndex( const LPCWSTR f_pszLibraryName, __int32 f_nIndex, MKI_LINEDEF_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_LineSetLibraryByIndex( const LPCWSTR f_pszLibraryName, __int32 f_nIndex, MKI_CONST_LINEDEF_REF f_rData );

MKI_INTERFACE_MEMBER __int32
MKI_LayerGetBaseMode( void );

MKI_INTERFACE_MEMBER bool
MKI_LayerSetBaseMode( __int32 f_nMode );

MKI_INTERFACE_MEMBER __int32
MKI_LayerGetIndex( const LPCWSTR f_pszName );

MKI_INTERFACE_MEMBER __int32
MKI_LayerAdd( MKI_CONST_LAYERDEF_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_LayerGetName( __int32 f_nIndex, MKI_LAYERNAME& f_rName );

MKI_INTERFACE_MEMBER bool
MKI_LayerGetByIndex( __int32 f_nIndex, MKI_LAYERDEF_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_LayerSetByIndex( __int32 f_nIndex, MKI_CONST_LAYERDEF_REF f_rData );

MKI_INTERFACE_MEMBER __int32
MKI_LayerGetActive( void );

MKI_INTERFACE_MEMBER bool
MKI_LayerSetActive( __int32 f_nIndex, bool f_fApply );

MKI_INTERFACE_MEMBER __int32
MKI_LayerGetNumber( void );

MKI_INTERFACE_MEMBER __int32
MKI_LayerGetLibraryIndex( const LPCWSTR f_pszLibraryName, const LPCWSTR f_pszName );

MKI_INTERFACE_MEMBER bool
MKI_LayerGetLibraryByIndex( const LPCWSTR f_pszLibraryName, __int32 f_nIndex, MKI_LAYERDEF_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_LayerSetLibraryByIndex( const LPCWSTR f_pszLibraryName, __int32 f_nIndex, MKI_CONST_LAYERDEF_REF f_rData );

MKI_INTERFACE_MEMBER __int32
MKI_VariableGetNumber( void );

MKI_INTERFACE_MEMBER bool
MKI_VariableGetValue( const LPCWSTR f_pszName, MKI_VARIABLEVALUE& f_rData );

MKI_INTERFACE_MEMBER bool
MKI_VariableGetComment( const LPCWSTR f_pszName, MKI_COMMENTSHORT& f_rData );

MKI_INTERFACE_MEMBER bool
MKI_VariableAdd( const LPCWSTR f_pszName, const LPCWSTR f_pszData );

MKI_INTERFACE_MEMBER __int32
MKI_ColorGetNumber( void );

MKI_INTERFACE_MEMBER __int32
MKI_ColorGetIndex( const LPCWSTR f_pszName );

MKI_INTERFACE_MEMBER __int32
MKI_ColorFindValue( MKI_COLORVALUE f_cColor, MKI_COLORMODEL f_cColorMode );

MKI_INTERFACE_MEMBER __int32
MKI_ColorAdd( MKI_CONST_COLORDEF_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_ColorGetByIndex( __int32 f_nIndex, MKI_COLORDEF_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_ColorSetByIndex( __int32 f_nIndex, MKI_CONST_COLORDEF_REF f_rData );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER MKI_ATTRIBUTE_DATA_PTR
MKI_AttributeBuffer( void );

MKI_INTERFACE_MEMBER __int32
MKI_AttributeGetBlock( const LPCWSTR f_pszBlockName, const LPCWSTR f_pszLibraryName, MKI_ATTRIBUTE_DATA_PTR f_pData );

MKI_INTERFACE_MEMBER __int32
MKI_AttributeGetLocal( MKI_CONST_ENTITY_PTR f_pEntity, MKI_ATTRIBUTE_DATA_PTR f_pData, bool f_fGlobal );

MKI_INTERFACE_MEMBER __int32
MKI_AttributeGetOutline( MKI_CONST_ENTITY_PTR f_pEntity, __int32 f_nOutlineIndex, MKI_ATTRIBUTE_DATA_PTR f_pData );

MKI_INTERFACE_MEMBER __int32
MKI_AttributeGetPoint( MKI_CONST_ENTITY_PTR f_pEntity, __int32 f_nPointHandle, MKI_ATTRIBUTE_DATA_PTR f_pData );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER __int32
MKI_ImportGetNumber( void );

MKI_INTERFACE_MEMBER bool
MKI_ImportGetData( __int32 f_nIndex, MKI_PLUGIN_DATA_REF f_rPlugInData );

MKI_INTERFACE_MEMBER __int32
MKI_ExportGetNumber( void );

MKI_INTERFACE_MEMBER bool
MKI_ExportGetData( __int32 f_nIndex, MKI_PLUGIN_DATA_REF f_rPlugInData );

MKI_INTERFACE_MEMBER __int32
MKI_PlugInGetNumber( void );

MKI_INTERFACE_MEMBER bool
MKI_PlugInGetData( __int32 f_nIndex, MKI_PLUGIN_DATA_REF f_rPlugInData );

MKI_INTERFACE_MEMBER bool
MKI_PlugInGetState( __int16 f_nOwnerID, __int16 f_nPlugInID );

MKI_INTERFACE_MEMBER bool
MKI_PlugInCallInterface( __int16 f_nOwnerID, __int16 f_nPlugInID, __int32 f_nCommandID, void *f_pData );

MKI_INTERFACE_MEMBER bool
MKI_CommandAvailable( __int32 f_nCommandID );

MKI_INTERFACE_MEMBER void
MKI_CommandInternal( __int32 f_nCommandID, bool f_nAddToList );

MKI_INTERFACE_MEMBER bool
MKI_CommandExternal( __int16 f_nOwnerID, __int16 f_nPlugInID, __int32 f_nCommandID, bool f_nAddToList );

MKI_INTERFACE_MEMBER bool
MKI_CommandUpdateMenu( void );

MKI_INTERFACE_MEMBER __int32
MKI_UndoGetCount( void );

MKI_INTERFACE_MEMBER void
MKI_UndoInitProcess( void );

MKI_INTERFACE_MEMBER void
MKI_UndoCancelProcess( void );

MKI_INTERFACE_MEMBER void
MKI_UndoFinishProcess( __int32 f_nCommandID, const LPCWSTR f_pszInfo, __int32 f_nCode );

MKI_INTERFACE_MEMBER void
MKI_UndoUpdateLinks( void );

MKI_INTERFACE_MEMBER void
MKI_UndoSetPrevious( void );

MKI_INTERFACE_MEMBER void
MKI_UndoGetNewFrame( MKI_RECT_REF f_rFrame );

MKI_INTERFACE_MEMBER bool
MKI_HookPositionStart( MKI_HOOKPOSITION_PROC f_fnCallback );

MKI_INTERFACE_MEMBER void
MKI_HookPositionEnd( MKI_HOOKPOSITION_PROC f_fnCallback );

MKI_INTERFACE_MEMBER bool
MKI_HookMouseStart( MKI_HOOKMOUSE_PROC f_fnCallback );

MKI_INTERFACE_MEMBER void
MKI_HookMouseEnd( MKI_HOOKMOUSE_PROC f_fnCallback );

MKI_INTERFACE_MEMBER bool
MKI_HookKeyStart( MKI_HOOKKEY_PROC f_fnCallback );

MKI_INTERFACE_MEMBER void
MKI_HookKeyEnd( MKI_HOOKKEY_PROC f_fnCallback );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER __int32
MKI_DatabaseGetNumber( bool f_fInternalDatabase );

MKI_INTERFACE_MEMBER __int32
MKI_DatabaseGetIndex( const LPCWSTR f_pszDatabaseName );

MKI_INTERFACE_MEMBER bool
MKI_DatabaseGetDisplayName( __int32 f_nDatabaseIndex, MKI_DATABASENAME& f_rDisplayName );

MKI_INTERFACE_MEMBER bool
MKI_DatabaseGetData( __int32 f_nDatabaseIndex, MKI_DATABASE_DATA_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_DatabaseSetData( __int32 f_nDatabaseIndex, MKI_CONST_DATABASE_DATA_REF f_rData );

MKI_INTERFACE_MEMBER __int32
MKI_DatabaseOpen( MKI_CONST_DATABASE_DATA_REF f_rData, bool f_fInternalDatabase );

MKI_INTERFACE_MEMBER __int32
MKI_DatabaseCreate( MKI_CONST_DATABASE_DATA_REF f_rData, bool f_InternalDatabase );

MKI_INTERFACE_MEMBER bool
MKI_DatabaseSave( __int32 f_nDatabaseIndex, const LPCWSTR f_pszFileName, __int32 f_nType );

MKI_INTERFACE_MEMBER bool
MKI_DatabaseRemove( __int32 f_nDatabaseIndex );

MKI_INTERFACE_MEMBER __int32
MKI_DatabaseGetActive( void );

MKI_INTERFACE_MEMBER bool
MKI_DatabaseSetActive( __int32 f_nDatabaseIndex );

MKI_INTERFACE_MEMBER __int32
MKI_DatabaseCleanUp( __int32 f_nDatabaseIndex );

MKI_INTERFACE_MEMBER bool
MKI_DatabaseCopy( __int32 f_nTargetDatabaseIndex, __int32 f_nRefDatabaseIndex );

MKI_INTERFACE_MEMBER __int32
MKI_DatabaseColumnMerge( __int32 f_nTargetDatabaseIndex, __int32 f_nTargetOriginalRowCount, __int32 f_nRefDatabaseIndex, __int32 f_nRefColumnIndex );

MKI_INTERFACE_MEMBER __int32
MKI_DatabaseUpdateObjectData( __int32 f_nDatabaseIndex );

MKI_INTERFACE_MEMBER bool
MKI_DatabaseSort( __int32 f_nDatabaseIndex, __int32 f_nColumnIndex );

MKI_INTERFACE_MEMBER __int32
MKI_DatabaseColumnGetCount( __int32 f_nDatabaseIndex );

MKI_INTERFACE_MEMBER __int32
MKI_DatabaseColumnGetWidth( __int32 f_nDatabaseIndex, __int32 f_nColumnIndex );

MKI_INTERFACE_MEMBER bool
MKI_DatabaseColumnRemove( __int32 f_nDatabaseIndex, __int32 f_nColumnIndex );

MKI_INTERFACE_MEMBER __int32
MKI_DatabaseColumnAdd( __int32 f_nDatabaseIndex, MKI_DATABASE_COLUMN_REF f_rColumnData, __int32 f_nColumnIndex, __int32 f_nCellMode );

MKI_INTERFACE_MEMBER __int32
MKI_DatabaseColumnFind( __int32 f_nDatabaseIndex, LPCWSTR f_pszFindText, bool f_fPartial );

MKI_INTERFACE_MEMBER __int32
MKI_DatabaseRowGetCount( __int32 f_nDatabaseIndex );

MKI_INTERFACE_MEMBER bool
MKI_DatabaseRowRemove( __int32 f_nDatabaseIndex, __int32 f_nRowIndex );

MKI_INTERFACE_MEMBER bool
MKI_DatabaseRowAdd( __int32 f_nDatabaseIndex, __int32 f_nRowIndex, __int32 f_nCellMode );

MKI_INTERFACE_MEMBER __int32
MKI_DatabaseRowFind( __int32 f_nDatabaseIndex, __int32 f_nColumnIndex, LPCWSTR f_pszFindText, bool f_fPartial );

MKI_INTERFACE_MEMBER bool
MKI_DatabaseCellGet( __int32 f_nDatabaseIndex, __int32 f_nRowIndex, __int32 f_nColumnIndex, MKI_DATABASE_CELL_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_DatabaseCellSet( __int32 f_nDatabaseIndex, __int32 f_nRowIndex, __int32 f_nColumnIndex, MKI_CONST_DATABASE_CELL_REF f_rData );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadKeyOpen( const LPCWSTR f_pszKeyName );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadData( const LPCWSTR f_pszValueName, void* f_pValue, const size_t f_unMaxSize );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadString( const LPCWSTR f_pszValueName, LPSTR f_pszValue, const size_t f_unMaxSize );

#endif

    template <size_t f_unSize> __forceinline bool
    MKI_ProfileReadString( const LPCWSTR f_pszValueName, char (&f_rDestin)[f_unSize] )
    {
      return( MKI_ProfileReadString( f_pszValueName, (LPSTR) &f_rDestin, f_unSize ) );
    }

#ifndef MKI_SERVER

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadString( const LPCWSTR f_pszValueName, LPWSTR f_pszValue, const size_t f_unMaxSize );

#endif

    template <size_t f_unSize> __forceinline bool
    MKI_ProfileReadString( const LPCWSTR f_pszValueName, wchar_t (&f_rDestin)[f_unSize] )
    {
      return( MKI_ProfileReadString( f_pszValueName, (LPWSTR) &f_rDestin, f_unSize ) );
    }

#ifndef MKI_SERVER

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadBool( const LPCWSTR f_pszValueName, bool& f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadInt32( const LPCWSTR f_pszValueName, __int32& f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadInt64( const LPCWSTR f_pszValueName, __int64& f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadColor( const LPCWSTR f_pszValueName, MKI_COLORVALUE_REF f_rColor, MKI_COLORMODEL_REF f_rColorModel );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadIntPoint( const LPCWSTR f_pszValueName, MKI_INTPOINT_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadIntVector( const LPCWSTR f_pszValueName, MKI_INTVECTOR_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadIntRect( const LPCWSTR f_pszValueName, MKI_INTRECT_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadDouble( const LPCWSTR f_pszValueName, double& f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadPoint( const LPCWSTR f_pszValueName, MKI_POINT_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadVector( const LPCWSTR f_pszValueName, MKI_VECTOR_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadRect( const LPCWSTR f_pszValueName, MKI_RECT_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadMatrix( const LPCWSTR f_pszValueName, MKI_MATRIX_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadProperty( const LPCWSTR f_pszValueName, MKI_PROPERTY_REF f_rValue );

MKI_INTERFACE_MEMBER __int32
MKI_ProfileReadNamedProperty( const LPCWSTR f_pszValueName, MKI_NAMED_PROPERTY_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadXProperty( const LPCWSTR f_pszValueName, MKI_XPROPERTY_REF f_rValue );

MKI_INTERFACE_MEMBER __int32
MKI_ProfileReadNamedXProperty( const LPCWSTR f_pszValueName, MKI_NAMED_XPROPERTY_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadAttribute( const LPCWSTR f_pszValueName, MKI_ATTRIBUTE_ENTRY_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadVariable( const LPCWSTR f_pszValueName, MKI_VARIABLEDEF_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadFontDef( const LPCWSTR f_pszValueName, MKI_FONTDEF_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadDimLine( const LPCWSTR f_pszValueName, MKI_DIMLINE_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadDimSmall( const LPCWSTR f_pszValueName, MKI_DIMSMALL_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadDimLarge( const LPCWSTR f_pszValueName, MKI_DIMLARGE_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadTextStandard( const LPCWSTR f_pszValueName, MKI_TEXTSTANDARD_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadTextReference( const LPCWSTR f_pszValueName, MKI_TEXTREFERENCE_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadTextFrame( const LPCWSTR f_pszValueName, MKI_TEXTFRAME_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadPrintDevice( const LPCWSTR f_pszValueName, MKI_PRINT_DEVICE_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadPrintData( const LPCWSTR f_pszValueName, MKI_PRINT_DATA_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadBitmapExport( const LPCWSTR f_pszValueName, MKI_BITMAP_EXPORT_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileReadSimplifyData( const LPCWSTR f_pszValueName, MKI_SIMPLIFY_DATA_REF f_rValue );

MKI_INTERFACE_MEMBER void
MKI_ProfileReadKeyClose( void );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteKeyOpen( const LPCWSTR f_pszKeyName );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteClear( void );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteString( const LPCWSTR f_pszValueName, const LPCSTR f_pszValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteString( const LPCWSTR f_pszValueName, const LPCWSTR f_pszValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteBool( const LPCWSTR f_pszValueName, bool f_fValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteInt32( const LPCWSTR f_pszValueName, __int32 f_nValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteInt64( const LPCWSTR f_pszValueName, __int64 f_nValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteColor( const LPCWSTR f_pszValueName, MKI_COLORVALUE f_cColor, MKI_COLORMODEL f_cColorModel );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteIntPoint( const LPCWSTR f_pszValueName, MKI_CONST_INTPOINT_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteIntVector( const LPCWSTR f_pszValueName, MKI_CONST_INTVECTOR_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteIntRect( const LPCWSTR f_pszValueName, MKI_CONST_INTRECT_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteDouble( const LPCWSTR f_pszValueName, double f_dValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWritePoint( const LPCWSTR f_pszValueName, MKI_CONST_POINT_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteVector( const LPCWSTR f_pszValueName, MKI_CONST_VECTOR_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteRect( const LPCWSTR f_pszValueName, MKI_CONST_RECT_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteMatrix( const LPCWSTR f_pszValueName, MKI_CONST_MATRIX_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteProperty( const LPCWSTR f_pszValueName, MKI_CONST_PROPERTY_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteNamedProperty( const LPCWSTR f_pszValueName, MKI_CONST_PROPERTY_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteNamedProperty( const LPCWSTR f_pszValueName, MKI_CONST_NAMED_PROPERTY_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteXProperty( const LPCWSTR f_pszValueName, MKI_CONST_XPROPERTY_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteNamedXProperty( const LPCWSTR f_pszValueName, MKI_CONST_XPROPERTY_REF f_rValue, __int32 f_nUseFlag, __int32 f_nUseProp );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteNamedXProperty( const LPCWSTR f_pszValueName, MKI_CONST_NAMED_XPROPERTY_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteAttribute( const LPCWSTR f_pszValueName, MKI_CONST_ATTRIBUTE_ENTRY_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteVariable( const LPCWSTR f_pszValueName, MKI_CONST_VARIABLEDEF_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteFontDef( const LPCWSTR f_pszValueName, MKI_CONST_FONTDEF_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteDimLine( const LPCWSTR f_pszValueName, MKI_CONST_DIMLINE_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteDimSmall( const LPCWSTR f_pszValueName, MKI_CONST_DIMSMALL_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteDimLarge( const LPCWSTR f_pszValueName, MKI_CONST_DIMLARGE_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteTextStandard( const LPCWSTR f_pszValueName, MKI_CONST_TEXTSTANDARD_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteTextReference( const LPCWSTR f_pszValueName, MKI_CONST_TEXTREFERENCE_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteTextFrame( const LPCWSTR f_pszValueName, MKI_CONST_TEXTFRAME_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWritePrintDevice( const LPCWSTR f_pszValueName, MKI_CONST_PRINT_DEVICE_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWritePrintData( const LPCWSTR f_pszValueName, MKI_CONST_PRINT_DATA_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteBitmapExport( const LPCWSTR f_pszValueName, MKI_CONST_BITMAP_EXPORT_REF f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_ProfileWriteSimplifyData( const LPCWSTR f_pszValueName, MKI_CONST_SIMPLIFY_DATA_REF f_rValue );

MKI_INTERFACE_MEMBER void
MKI_ProfileWriteKeyClose( void );

MKI_INTERFACE_MEMBER __int32
MKI_ProfileDrawingGetData( __int16 f_nOwnerID, __int32 f_nProfileID, void* f_pProfileData, const size_t f_unMaxSize );

MKI_INTERFACE_MEMBER MKI_ENTITY_OBJECT_PTR
MKI_ProfileDrawingGetObject( __int16 f_nOwnerID, __int32 f_nProfileID );

MKI_INTERFACE_MEMBER bool
MKI_ProfileDrawingSetData( __int16 f_nOwnerID, __int32 f_nProfileID, const void* const f_pProfileData, const size_t f_unSize );

MKI_INTERFACE_MEMBER bool
MKI_ProfileDrawingSetObject( __int16 f_nOwnerID, __int32 f_nProfileID, MKI_CONST_ENTITY_OBJECT_PTR f_pObj );

MKI_INTERFACE_MEMBER INT_PTR
MKI_SettingGet( __int32 f_nMode );

MKI_INTERFACE_MEMBER void
MKI_SettingSet( __int32 f_nMode, INT_PTR f_nValue );

MKI_INTERFACE_MEMBER __int32
MKI_MaxEntityData( void );

MKI_INTERFACE_MEMBER __int32
MKI_MaxEntityPoints( void );

MKI_INTERFACE_MEMBER __int32
MKI_MaxPolygonPoints( void );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER __int32
MKI_CollectionGetNumber( void );

MKI_INTERFACE_MEMBER bool
MKI_CollectionGetInfo( __int32 f_nCollectionNum, MKI_FILENAMEW& f_rszFileName, MKI_FILE_HEADER_PTR f_pFileHeader, bool* f_pIsReadOnly );

MKI_INTERFACE_MEMBER __int32
MKI_CollectionGetItemNumber( const LPCWSTR f_pszCollectionName, __int32 f_nOwnerID, __int32 f_nType );

MKI_INTERFACE_MEMBER __int32
MKI_CollectionReadItemOpen( HWND f_hWindow, const LPCWSTR f_pszCaption, __int32 f_nOwnerID, __int32 f_nType );

MKI_INTERFACE_MEMBER __int32
MKI_CollectionReadItemOpen( const LPCWSTR f_pszCollectionName, __int32 f_nOwnerID, __int32 f_nType, const LPCWSTR f_pszItemName );

MKI_INTERFACE_MEMBER __int32
MKI_CollectionReadPinboardOpen( HWND f_hWindow, __int32 f_nOwnerID, __int32 f_nType );

MKI_INTERFACE_MEMBER __int32
MKI_CollectionReadPinboardOpen( __int32 f_nOwnerID, __int32 f_nType, const LPCWSTR f_pszItemName );

MKI_INTERFACE_MEMBER void
MKI_CollectionReadClose( HWND f_hWindow, __int32 f_nResult );

MKI_INTERFACE_MEMBER bool
MKI_CollectionWriteItemOpen( HWND f_hWindow, const LPCWSTR f_pszCaption, __int32 f_nOwnerID, __int32 f_nType, __int32 f_nVersion );

MKI_INTERFACE_MEMBER bool
MKI_CollectionWritePinboardOpen( __int32 f_nOwnerID, __int32 f_nType, __int32 f_nVersion );

MKI_INTERFACE_MEMBER bool
MKI_CollectionWriteClose( bool f_fApply );

MKI_INTERFACE_MEMBER bool
MKI_CollectionDeleteItem( const LPCWSTR f_pszCollectionName, __int32 f_nOwnerID, __int32 f_nType, const LPCWSTR f_pszItemName );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER bool
MKI_DrawingNewFile( __int32 f_nOpenFlags, bool f_fPrompt );

MKI_INTERFACE_MEMBER bool
MKI_DrawingOpenFile( const LPCWSTR f_pszFileName, __int32 f_nOpenFlags, bool f_fSetName, bool f_fMerge, bool f_fPrompt );

MKI_INTERFACE_MEMBER bool
MKI_DrawingSaveFile( const LPCWSTR f_pszFileName, __int32 f_nSaveFlags, bool f_fPrompt );

MKI_INTERFACE_MEMBER bool
MKI_DrawingArchiveFile( const LPCWSTR f_pszFileName, bool f_fPrompt, bool f_fCompressed );

MKI_INTERFACE_MEMBER void
MKI_DrawingSetChanged( __int32 f_nCode );

MKI_INTERFACE_MEMBER bool
MKI_DrawingGetInfo( MKI_FILENAMEW& f_rszFileName, MKI_FILE_HEADER_PTR f_pFileHeader, bool* f_pDrawingChanged );

MKI_INTERFACE_MEMBER bool
MKI_DrawingSetInfo( const LPCWSTR f_pszFileName, MKI_CONST_FILE_HEADER_PTR f_pFileHeader );

MKI_INTERFACE_MEMBER __int32
MKI_LibraryGetNumber( void );

MKI_INTERFACE_MEMBER bool
MKI_LibraryGetInfoByIndex( __int32 f_nLibraryNum, MKI_FILENAMEW& f_rszFileName, MKI_FILE_HEADER_PTR f_pFileHeader, bool* f_pIsReadOnly );

MKI_INTERFACE_MEMBER bool
MKI_LibraryGetInfoByName( const LPCWSTR f_pszLibraryName, MKI_FILENAMEW& f_rszFileName, MKI_FILE_HEADER_PTR f_pFileHeader, bool* f_pIsReadOnly );

MKI_INTERFACE_MEMBER const LPCWSTR
MKI_LibraryGetDisplayName( const LPCWSTR f_pszLibraryName );

MKI_INTERFACE_MEMBER __int32
MKI_WindowGetCurrentData( MKI_DISPLAYDATA_REF f_rData );

MKI_INTERFACE_MEMBER __int32
MKI_WindowGetNumber( void );

MKI_INTERFACE_MEMBER __int32
MKI_WindowGetActive( void );

MKI_INTERFACE_MEMBER __int32
MKI_WindowGetIdent( void );

MKI_INTERFACE_MEMBER bool
MKI_WindowSetActive( __int32 f_nWindowNum );

MKI_INTERFACE_MEMBER bool
MKI_WindowGetRect( __int32 f_nWindowNum, MKI_INTRECT_REF f_rRect, MKI_INTPOINT_REF f_rOffset );

MKI_INTERFACE_MEMBER bool
MKI_WindowGetMatrix( __int32 f_nWindowNum, MKI_MATRIX_PTR f_pMatrixMMToPixel, MKI_MATRIX_PTR f_pMatrixPixelToMM );

MKI_INTERFACE_MEMBER double
MKI_WindowGetPixelPerMM( __int32 f_nWindowNum );

MKI_INTERFACE_MEMBER bool
MKI_WindowGetViewZoom( __int32 f_nWindowNum, MKI_ZOOMDEF_REF f_rZoom );

MKI_INTERFACE_MEMBER bool
MKI_WindowGetViewRect( __int32 f_nWindowNum, MKI_RECT_REF f_rRect );

MKI_INTERFACE_MEMBER bool
MKI_WindowSetViewAll( __int32 f_nWindowNum, bool f_fRedraw );

MKI_INTERFACE_MEMBER bool
MKI_WindowSetViewPage( __int32 f_nWindowNum, bool f_fRedraw );

MKI_INTERFACE_MEMBER bool
MKI_WindowSetViewZoom( __int32 f_nWindowNum, MKI_CONST_ZOOMDEF_REF f_rZoom, bool f_fRedraw );

MKI_INTERFACE_MEMBER bool
MKI_WindowSetViewArea( __int32 f_nWindowNum, MKI_CONST_POINT_REF f_rPoint1, MKI_CONST_POINT_REF f_rPoint2, bool f_fRedraw );

MKI_INTERFACE_MEMBER bool
MKI_WindowSetViewQuad( __int32 f_nWindowNum, MKI_CONST_QUAD_REF f_rQuad, bool f_fRedraw );

MKI_INTERFACE_MEMBER void
MKI_WindowRedraw( __int32 f_nWindowNum );

MKI_INTERFACE_MEMBER void
MKI_DrawWindowAll( void );

MKI_INTERFACE_MEMBER void
MKI_DrawWindowArea( MKI_CONST_RECT_REF f_rData );

MKI_INTERFACE_MEMBER void
MKI_DrawWindowSelection( bool f_fClearFlag );

MKI_INTERFACE_MEMBER void
MKI_DrawNewObjects( void );

MKI_INTERFACE_MEMBER void
MKI_DrawClear( HWND f_hWindow );

MKI_INTERFACE_MEMBER void
MKI_DrawBlock( HWND f_hWindow, const LPCWSTR f_pszBlockName, const LPCWSTR f_pszLibraryName );

MKI_INTERFACE_MEMBER void
MKI_DrawEntity( HWND f_hWindow, MKI_CONST_ENTITY_PTR f_pEntity, __int32 f_nBackground, const LPCWSTR f_pszText );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER bool
MKI_DrawingBitmapExport( const LPCWSTR f_pszFileName, MKI_BITMAP_EXPORT_REF f_rExportData, MKI_CONST_RECT_PTR f_pFrame );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_DrawingPrint( MKI_PRINT_DATA_PTR f_pPrintData );

MKI_INTERFACE_MEMBER bool
MKI_PrinterGetInfo( __int32 f_nIndex, MKI_PRINT_DEVICE_REF f_rDeviceData );

MKI_INTERFACE_MEMBER bool
MKI_PrinterSetInfo( __int32 f_nIndex, MKI_CONST_PRINT_DEVICE_REF f_rDeviceData );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER void
MKI_DialogHelpTopic( HWND f_hWindow, const LPCWSTR f_pszPlugInID, DWORD f_dwHelpTopicID );

MKI_INTERFACE_MEMBER bool
MKI_DialogEnterIdle( HWND f_hWindow, WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_DialogPopupRegister( HWND f_hWindow );

MKI_INTERFACE_MEMBER bool
MKI_DialogPopupUnregister( HWND f_hWindow );

MKI_INTERFACE_MEMBER void
MKI_DialogPopupActivate( HWND f_hWindow, WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER void
MKI_DialogPopupMove( HWND f_hWindow );

MKI_INTERFACE_MEMBER void
MKI_DialogPopupSetCursor( HWND f_hWindow, WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER void
MKI_DialogInitPropertySheet( HWND f_hPropSheet, MKI_SUBCLASS_LIST_PTR f_pSubclassList );

MKI_INTERFACE_MEMBER void
MKI_DialogPropertySheetNoCancel( HWND f_hPropSheet );

MKI_INTERFACE_MEMBER bool
MKI_DialogInitSlider( HWND f_hWindow, __int32 f_nCtlID, __int32 f_nMin, __int32 f_nMax, __int32 f_nStep, const double f_dValue );

MKI_INTERFACE_MEMBER INT_PTR
MKI_DialogHandleDefaultMessages( HWND f_hDlg, UINT f_unMessage, WPARAM f_wParam, LPARAM f_lParam );

MKI_INTERFACE_MEMBER bool
MKI_DialogHandleSlider( HWND f_hWindow, WPARAM f_wParam, LPARAM f_lParam,
                        __int32 f_nCtlID, __int32 f_nMin, __int32 f_nMax, __int32 f_nStep, __int32& f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_DialogHandleSpinInt64( HWND f_hWindow, WPARAM f_wParam, LPARAM f_lParam, 
                           __int32 f_nCtlPlus, __int32 f_nCtlMinus, __int32 f_nCtlEdit, __int64 f_nMin, __int64 f_nMax, __int64 f_nStep );

MKI_INTERFACE_MEMBER bool
MKI_DialogHandleSpinDouble( HWND f_hWindow, WPARAM f_wParam, LPARAM f_lParam,
                            __int32 f_nPlusCtl, __int32 f_nMinusCtl, __int32 f_nEditCtl, double f_dMin, double f_dMax, double f_dStep );

MKI_INTERFACE_MEMBER void
MKI_DialogInitCheck( HWND f_hWindow, __int32 f_nCtlID );

MKI_INTERFACE_MEMBER void
MKI_DialogInitCheck( HWND f_hWindow, __int32 f_nCtlID, const bool f_fValue );

MKI_INTERFACE_MEMBER void
MKI_DialogInitCheck( HWND f_hWindow, __int32 f_nCtlID, const __int32 f_nValue );

MKI_INTERFACE_MEMBER void
MKI_DialogInitMultiCheck( HWND f_hWindow, __int32 f_nCtlIDmin, __int32 f_nCtlIDmax, const __int32 f_nValue );

MKI_INTERFACE_MEMBER void
MKI_DialogHandleCheckSet( HWND f_hWindow, __int32 f_nCtlID, bool& f_rfValue );

MKI_INTERFACE_MEMBER void
MKI_DialogHandleCheckSet( HWND f_hWindow, __int32 f_nCtlID, __int32& f_rnValue, __int32 f_nBitMask );

MKI_INTERFACE_MEMBER void
MKI_DialogHandleCheckClear( HWND f_hWindow, __int32 f_nCtlID, bool& f_rfValue );

MKI_INTERFACE_MEMBER void
MKI_DialogHandleCheckClear( HWND f_hWindow, __int32 f_nCtlID, __int32& f_rnValue, __int32 f_nBitMask );

MKI_INTERFACE_MEMBER void
MKI_DialogHandleCheckInvert( HWND f_hWindow, __int32 f_nCtlID, bool& f_rfValue );

MKI_INTERFACE_MEMBER void
MKI_DialogHandleCheckInvert( HWND f_hWindow, __int32 f_nCtlID, __int32& f_rnValue, __int32 f_nBitMask );

MKI_INTERFACE_MEMBER void
MKI_DialogHandleMultiCheckInvert( HWND f_hWindow, __int32 f_nCtlIDmin, __int32 f_nCtlIDmax, __int32 f_nCtlID, __int32& f_rnValue );

MKI_INTERFACE_MEMBER void
MKI_DialogInitRadio( HWND f_hWindow, __int32 f_nCtlIDfalse, __int32 f_nCtlIDtrue, const bool f_fValue );

MKI_INTERFACE_MEMBER void
MKI_DialogInitRadio( HWND f_hWindow, __int32 f_nCtlIDmin, __int32 f_nCtlIDmax );

MKI_INTERFACE_MEMBER void
MKI_DialogInitRadio( HWND f_hWindow, __int32 f_nCtlIDmin, __int32 f_nCtlIDmax, const __int32 f_nValue );

MKI_INTERFACE_MEMBER void
MKI_DialogHandleRadio( HWND f_hWindow, __int32 f_nCtlIDfalse, __int32 f_nCtlIDtrue, __int32 f_nCtlID, bool& f_rfValue );

MKI_INTERFACE_MEMBER void
MKI_DialogHandleRadio( HWND f_hWindow, __int32 f_nCtlIDmin, __int32 f_nCtlIDmax, __int32 f_nCtlID, __int32& f_rnValue );

MKI_INTERFACE_MEMBER __int32
MKI_DialogCollectionShowMenu( HWND f_hWindow, __int32 f_nCtlID, __int32 f_nType, const LPCWSTR f_pszTypeName, bool f_fAllowPut, bool f_fAllowGet );

MKI_INTERFACE_MEMBER bool
MKI_DialogCollectionHandleText( HWND f_hWindow, __int32 f_nCtlID, __int32 f_nEditID );

MKI_INTERFACE_MEMBER bool
MKI_DialogCollectionHandleName( HWND f_hWindow, __int32 f_nCtlID, __int32 f_nType, const LPCWSTR f_pszNameIn, LPWSTR f_pszNameOut, const size_t f_unMaxSize );

#endif

    template <size_t f_unSize> __forceinline bool
    MKI_DialogCollectionHandleName( HWND f_hWindow, __int32 f_nCtlID, __int32 f_nType, const LPCWSTR f_pszNameIn, wchar_t (&f_rNameOut)[f_unSize] )
    {
      return( MKI_DialogCollectionHandleName( f_hWindow, f_nCtlID, f_nType, f_pszNameIn, (LPWSTR) &f_rNameOut, f_unSize ) );
    }

#ifndef MKI_SERVER

MKI_INTERFACE_MEMBER bool
MKI_DialogCollectionHandleColor( HWND f_hWindow, __int32 f_nCtlID,
                                 MKI_COLORVALUE f_cColorValueIn, MKI_COLORMODEL f_cColorModelIn, MKI_COLORDEF_PTR f_pColorOut );

MKI_INTERFACE_MEMBER bool
MKI_DialogCollectionHandleFile( HWND f_hWindow, __int32 f_nCtlID, const LPCWSTR f_pszFileNameIn, MKI_FILENAMEW* f_pFileNameOut );

MKI_INTERFACE_MEMBER bool
MKI_DialogCollectionHandleBlock( HWND f_hWindow, __int32 f_nCtlID, const LPCWSTR f_pszLibraryIn, const LPCWSTR f_pszBlockIn,
                                 MKI_LIBRARYNAME* f_pLibraryOut, MKI_BLOCKNAME* f_pBlockOut );

MKI_INTERFACE_MEMBER void
MKI_DialogComboboxAdjust( HWND f_hWindow, __int32 f_nCtlID );

MKI_INTERFACE_MEMBER void
MKI_DialogComboboxFill( HWND f_hWindow, __int32 f_nCtlID, const LPWSTR* f_apszTextList, __int32 f_nActive );

MKI_INTERFACE_MEMBER void
MKI_DialogCustomButtonColor( HWND f_hWindow, MKI_COLORVALUE f_cColor, MKI_COLORMODEL f_cColorMode, bool f_fLineOnly );

MKI_INTERFACE_MEMBER void
MKI_DialogCustomButtonMenu( HWND f_hWindow, const LPCWSTR f_pszText );

MKI_INTERFACE_MEMBER void
MKI_DialogCustomButtonLayer( HWND f_hWindow, const LPCWSTR f_pszText, __int32 f_nLayerMode );

MKI_INTERFACE_MEMBER void
MKI_DialogCustomButtonLayer( HWND f_hWindow, __int32 f_nIndex );

MKI_INTERFACE_MEMBER void
MKI_DialogCustomButtonPen( HWND f_hWindow, const LPCWSTR f_pszText, __int32 f_nPenMode );

MKI_INTERFACE_MEMBER void
MKI_DialogCustomButtonPen( HWND f_hWindow, __int32 f_nIndex );

MKI_INTERFACE_MEMBER void
MKI_DialogCustomListboxColor( const LPDRAWITEMSTRUCT f_pDrawItem, MKI_COLORVALUE f_cColor, MKI_COLORMODEL f_cColorMode, const LPCWSTR f_pszText );

MKI_INTERFACE_MEMBER void
MKI_DialogCustomIconControl( HWND f_hWindow, __int32 f_nCtlID, MKI_CONST_ICON_DATA_REF f_rIconData );

MKI_INTERFACE_MEMBER void
MKI_DialogCustomIconControl( HWND f_hWindow, __int32 f_nCtlID, __int32 f_nIconID );

MKI_INTERFACE_MEMBER bool
MKI_DialogCustomBitmapControl( HWND f_hWindow, __int32 f_nCtlID, MKI_CONST_ICON_DATA_REF f_rIconData );

MKI_INTERFACE_MEMBER bool
MKI_DialogCustomBitmapControl( HWND f_hWindow, __int32 f_nCtlID, const LPCWSTR f_pszFileName );

MKI_INTERFACE_MEMBER __int32
MKI_DialogTrackPopup( HWND f_hWindow, MKI_CONST_POPUP_ENTRY_PTR f_pEntryList, __int32 f_nEntryCount, __int32 f_nCommandID, MKI_CONST_INTRECT_PTR f_pRect );

MKI_INTERFACE_MEMBER __int32
MKI_DialogBoxParam( HINSTANCE f_hInstance, LPCSTR f_pszTemplateName, HWND f_hWindow, DLGPROC f_fnDialogFunc, LPARAM f_dwInitParam );

MKI_INTERFACE_MEMBER void
MKI_DialogCenter( HWND f_hWindow, HICON f_hIcon );

MKI_INTERFACE_MEMBER __int32
MKI_DialogMessage( const LPCWSTR f_pszText, const LPCWSTR f_pszCaption, __int32 f_nMode );

MKI_INTERFACE_MEMBER __int32
MKI_DialogCheckMessage( const LPCWSTR f_pszText, const LPCWSTR f_pszCaption, __int32 f_nMode, const LPCWSTR f_pszCheckText, bool& f_rCheckState );

MKI_INTERFACE_MEMBER bool
MKI_DialogString( HWND f_hWindow, const LPCWSTR f_pszCaption, const LPCWSTR f_pszPrefix, LPWSTR f_szText, const size_t f_unMaxSize );

MKI_INTERFACE_MEMBER bool
MKI_DialogName( HWND f_hWindow, const LPCWSTR f_pszCaption, const LPCWSTR f_pszPrefix, LPWSTR f_szName, const size_t f_unMaxSize );

MKI_INTERFACE_MEMBER bool
MKI_DialogInt32( HWND f_hWindow, const LPCWSTR f_pszCaption, const LPCWSTR f_pszPrefix, __int32& f_rValue, __int32 f_nMin, __int32 f_nMax );

MKI_INTERFACE_MEMBER bool
MKI_DialogInt64( HWND f_hWindow, const LPCWSTR f_pszCaption, const LPCWSTR f_pszPrefix, __int64& f_rValue, __int64 f_nMin, __int64 f_nMax );

MKI_INTERFACE_MEMBER bool
MKI_DialogDouble( HWND f_hWindow, const LPCWSTR f_pszCaption, const LPCWSTR f_pszPrefix, LPWSTR f_pszText, double& f_rValue );

MKI_INTERFACE_MEMBER bool
MKI_DialogAngle( HWND f_hWindow, const LPCWSTR f_pszCaption, const LPCWSTR f_pszPrefix, LPWSTR f_pszText, double& f_rAngle );

MKI_INTERFACE_MEMBER bool
MKI_DialogLength( HWND f_hWindow, const LPCWSTR f_pszCaption, const LPCWSTR f_pszPrefix, LPWSTR f_pszText, double& f_rLength );

MKI_INTERFACE_MEMBER bool
MKI_DialogWidth( HWND f_hWindow, const LPCWSTR f_pszCaption, const LPCWSTR f_pszPrefix, LPWSTR f_pszText, double& f_rWidth );

MKI_INTERFACE_MEMBER bool
MKI_DialogSelection( HWND f_hWindow, const LPCWSTR f_pszCaption );

MKI_INTERFACE_MEMBER bool
MKI_DialogColor( HWND f_hWindow, const LPCWSTR f_pszCaption, MKI_COLORVALUE_REF f_rData, MKI_COLORMODEL_PTR f_pColorMode, bool f_fByLayer );

MKI_INTERFACE_MEMBER bool
MKI_DialogFontDef( HWND f_hWindow, const LPCWSTR f_pszCaption, MKI_FONTDEF_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_DialogProperty( HWND f_hWindow, const LPCWSTR f_pszCaption, MKI_PROPERTY_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_DialogXProperty( HWND f_hWindow, const LPCWSTR f_pszCaption, MKI_XPROPERTY_REF f_rXProperty, __int32 f_nUseType, __int32 f_nPropMode );

MKI_INTERFACE_MEMBER bool
MKI_DialogXPropertyMulti( HWND f_hWindow, const LPCWSTR f_pszCaption, MKI_MULTI_XPROPERTY_REF f_rXProperty, __int32 f_nUseType, __int32 f_nPropMode );

MKI_INTERFACE_MEMBER bool
MKI_DialogAnyEntity( HWND f_hWindow, const LPCWSTR f_pszCaption, MKI_MULTI_INFO_REF f_rData, bool* f_pfDuplicate, __int32* f_pnIdentical );

MKI_INTERFACE_MEMBER bool
MKI_DialogAnyEntityMulti( HWND f_hWindow, const LPCWSTR f_pszCaption, MKI_MULTI_INFO_REF f_rData, bool* f_pfDuplicate, __int32* f_pnIdentical );

MKI_INTERFACE_MEMBER bool
MKI_DialogDimLine( HWND f_hWindow, const LPCWSTR f_pszCaption, MKI_DIMLINE_REF f_rDimLine, __int32 f_nDimMode );

MKI_INTERFACE_MEMBER bool
MKI_DialogDimLineMulti( HWND f_hWindow, const LPCWSTR f_pszCaption, MKI_MULTI_DIMLINE_REF f_rDimLine, __int32 f_nDimMode );

MKI_INTERFACE_MEMBER bool
MKI_DialogDimSmall( HWND f_hWindow, const LPCWSTR f_pszCaption, 
                    MKI_DIMSMALL_REF f_rDimSmall, MKI_DIMPARAM_REF f_rDimParam, bool f_fUseGlobal, __int32 f_nDimMode );

MKI_INTERFACE_MEMBER bool
MKI_DialogDimSmallMulti( HWND f_hWindow, const LPCWSTR f_pszCaption,
                         MKI_MULTI_DIMSMALL_REF f_rDimSmall, MKI_MULTI_DIMPARAM_REF f_rDimParam, bool f_fUseGlobal, __int32 f_nDimMode );

MKI_INTERFACE_MEMBER bool
MKI_DialogDimLarge( HWND f_hWindow, const LPCWSTR f_pszCaption, 
                    MKI_DIMLARGE_REF f_rDimLarge, MKI_DIMPARAM_REF f_rDimParam, bool f_fUseGlobal, __int32 f_nDimMode );

MKI_INTERFACE_MEMBER bool
MKI_DialogDimLargeMulti( HWND f_hWindow, const LPCWSTR f_pszCaption, 
                         MKI_MULTI_DIMLARGE_REF f_rDimLarge, MKI_MULTI_DIMPARAM_REF f_rDimParam, bool f_fUseGlobal, __int32 f_nDimMode );

MKI_INTERFACE_MEMBER bool
MKI_DialogDimTextSmall( HWND f_hWindow, const LPCWSTR f_pszCaption, MKI_DIMTEXTSMALL_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_DialogDimTextLarge( HWND f_hWindow, const LPCWSTR f_pszCaption, MKI_DIMTEXTLARGE_REF f_rData );

MKI_INTERFACE_MEMBER bool
MKI_DialogTextStandard( HWND f_hWindow, const LPCWSTR f_pszCaption,
                        MKI_TEXTSTANDARD_PTR f_pTextStandard, MKI_TEXTREFERENCE_PTR f_pTextReference, 
                        LPWSTR f_pszEditText, const size_t f_unMaxSize, bool f_fAllowCurrentSystem, bool f_fReadOnly );

MKI_INTERFACE_MEMBER bool
MKI_DialogTextStandardMulti( HWND f_hWindow, const LPCWSTR f_pszCaption,
                             MKI_MULTI_TEXTSTANDARD_PTR f_pTextStandard, MKI_MULTI_TEXTREFERENCE_PTR f_pTextReference );

MKI_INTERFACE_MEMBER bool
MKI_DialogTextFrame( HWND f_hWindow, const LPCWSTR f_pszCaption, MKI_TEXTFRAME_REF f_rTextFrame,
                     LPWSTR f_pszEditText, const size_t f_unMaxSize, bool f_fAllowCurrentSystem, bool f_fReadOnly );

MKI_INTERFACE_MEMBER bool
MKI_DialogTextFrameMulti( HWND f_hWindow, const LPCWSTR f_pszCaption, MKI_MULTI_TEXTFRAME_REF f_rTextFrame );

MKI_INTERFACE_MEMBER bool
MKI_DialogFileOptions( HWND f_hWindow, const LPCWSTR f_pszCaption,
                       __int32 *f_pCurrentSet, __int32 f_nDefaultMode, bool f_fMerge );

MKI_INTERFACE_MEMBER __int32
MKI_DialogFileSelection( HWND f_hWindow, const LPCWSTR f_pszCaption,
                         MKI_FILENAMEW& f_rszFileName, __int32 f_nFileMode,
                         const LPCWSTR f_pszFilterString, __int32 f_nFilterIndex, const LPCWSTR f_pszDefaultExtension,
                         MKI_CALLBACK_PROC f_fnHelpCallback, void* f_pHelpData,
                         MKI_CALLBACK_PROC f_fnAboutCallback, void* f_pAboutData,
                         MKI_CALLBACK_PROC f_fnOptionsCallback, void* f_pOptionsData );

MKI_INTERFACE_MEMBER bool
MKI_DialogCreateBlock( HWND f_hWindow, const LPCWSTR f_pszCaption,
                       MKI_CREATEBLOCK_DATA_REF f_rData, __int32 f_nDialogMode );

MKI_INTERFACE_MEMBER bool
MKI_DialogInsertBlock( HWND f_hWindow, const LPCWSTR f_pszCaption,
                       MKI_INSERTBLOCK_DATA_REF f_rData, __int32 f_nDialogMode );

MKI_INTERFACE_MEMBER bool
MKI_DialogDatabaseColumns( HWND f_hWindow, const LPCWSTR f_pszCaption, __int32 f_nDatabaseIndex );

MKI_INTERFACE_MEMBER bool
MKI_DialogPrint( HWND f_hWindow, const LPCWSTR f_pszCaption, MKI_PRINT_DATA_REF f_rPrintData, __int32 f_nDialogMode );

MKI_INTERFACE_MEMBER bool
MKI_DialogSerialPrint( HWND f_hWindow, const LPCWSTR f_pszCaption, MKI_PRINT_DATA_REF f_rPrintData );

MKI_INTERFACE_MEMBER bool
MKI_DialogBitmapExport( HWND f_hWindow, const LPCWSTR f_pszCaption, MKI_BITMAP_EXPORT_REF f_rExportData, bool f_fIsDrawing );

MKI_INTERFACE_MEMBER __int32
MKI_DialogSelectLine( HWND f_hWindow, MKI_CONST_INTRECT_PTR f_pButtonRect, __int32 f_nCurrent, __int32 f_nBasicID, __int32 f_nByLayerID );

MKI_INTERFACE_MEMBER __int32
MKI_DialogSelectSystem( HWND f_hWindow, MKI_CONST_INTRECT_PTR f_pButtonRect, __int32 f_nCurrent, __int32 f_nBasicID, __int32 f_nExtraID, __int32 f_nEditID );

MKI_INTERFACE_MEMBER __int32
MKI_DialogSelectHatch( HWND f_hWindow, MKI_CONST_INTRECT_PTR f_pButtonRect, __int32 f_nCurrent, __int32 f_nBasicID, __int32 f_nExtraID, __int32 f_nEditID );

MKI_INTERFACE_MEMBER __int32
MKI_DialogSelectLayer( HWND f_hWindow, MKI_CONST_INTRECT_PTR f_pButtonRect, __int32 f_nCurrent, __int32 f_nBasicID, __int32 f_nExtraID, __int32 f_nEditID );

MKI_INTERFACE_MEMBER __int32
MKI_DialogSelectPen( HWND f_hWindow, MKI_CONST_INTRECT_PTR f_pButtonRect, __int32 f_nCurrent, __int32 f_nBasicID, __int32 f_nExtraID, __int32 f_nEditID );

MKI_INTERFACE_MEMBER __int32
MKI_DialogSelectPage( HWND f_hWindow, MKI_CONST_INTRECT_PTR f_pButtonRect, __int32 f_nCurrent, __int32 f_nBasicID, __int32 f_nExtraID, __int32 f_nEditID );

MKI_INTERFACE_MEMBER __int32
MKI_DialogSelectMultiPage( HWND f_hWindow, const LPCWSTR f_pszCaption, bool* f_pfSelected );

MKI_INTERFACE_MEMBER bool
MKI_DialogSelectVariable( HWND f_hWindow, MKI_CONST_INTRECT_PTR f_pButtonRect, __int32 f_nTextCtlID );

MKI_INTERFACE_MEMBER bool
MKI_DialogSelectVariableEx( HWND f_hWindow, MKI_CONST_INTRECT_PTR f_pButtonRect, __int32 f_nTextCtlID,
                            MKI_VARIABLEDEF_PTR f_pExtraVariables, __int32 f_nExtraCount );

MKI_INTERFACE_MEMBER bool
MKI_DialogCollectionManage( HWND f_hWindow, const LPCWSTR f_pszCaption, __int32 f_nOwnerID, __int32 f_nType );

MKI_INTERFACE_MEMBER bool
MKI_DialogTrimGenerate( HWND f_hWindow, const LPCWSTR f_pszCaption, 
                        __int32* f_pMode, double* f_pTolerance, MKI_SIMPLIFY_DATA_PTR f_pSimplify );

MKI_INTERFACE_MEMBER bool
MKI_DialogTrimSimplify( HWND f_hWindow, const LPCWSTR f_pszCaption, MKI_SIMPLIFY_DATA_PTR f_pSimplify );

MKI_INTERFACE_MEMBER __int32
MKI_DialogSelectEntity( MKI_CONST_ENTITY_PTR* f_pEntityList, const __int32* f_pPointTypeList, __int32 f_nListSize, MKI_CONST_MATRIX_PTR f_pMatrix );

MKI_INTERFACE_MEMBER void
MKI_DialogUpdateGuide( __int32 f_nGuideMode, const LPCWSTR f_pszGuideText );

MKI_INTERFACE_MEMBER void
MKI_DialogUpdateButtons( void );

MKI_INTERFACE_MEMBER bool
MKI_DialogShowProgress( const LPCWSTR f_pszTitle, const LPCWSTR f_pszText, bool f_fHasPercentBar );

MKI_INTERFACE_MEMBER bool
MKI_DialogProgressIsVisible( void );

MKI_INTERFACE_MEMBER bool
MKI_DialogProgressIsPending( void );

MKI_INTERFACE_MEMBER void
MKI_DialogUpdateProgress( const LPCWSTR f_pszText1, const LPCWSTR f_pszText2, __int64 f_nDone, __int64 f_nTotal );

MKI_INTERFACE_MEMBER void
MKI_DialogHideProgress( void );

MKI_INTERFACE_MEMBER void
MKI_DialogClearProgress( void );

MKI_INTERFACE_MEMBER bool
MKI_DialogIsCancelled( void );

MKI_INTERFACE_MEMBER bool
MKI_DialogShowMasterProgress( const LPCWSTR f_pszTitle );

MKI_INTERFACE_MEMBER void
MKI_DialogUpdateMasterProgress( const LPCWSTR f_pszText1, const LPCWSTR f_pszText2, __int64 f_nDone, __int64 f_nTotal );

MKI_INTERFACE_MEMBER void
MKI_DialogHideMasterProgress( void );

MKI_INTERFACE_MEMBER bool
MKI_DialogMasterIsCancelled( void );

MKI_INTERFACE_MEMBER bool
MKI_DialogPlugInTest( HWND f_hMainWnd, MKI_PLUGIN_CONTAINER_REF f_rContainerData, bool f_fSilent, HICON f_hIcon );

MKI_INTERFACE_MEMBER bool
MKI_DialogPlugInNoTest( HWND f_hMainWnd, MKI_PLUGIN_CONTAINER_REF f_rContainerData, bool f_fSilent, HICON f_hIcon );

MKI_INTERFACE_MEMBER void
MKI_DialogPlugInAbout( HWND f_hMainWnd, MKI_PLUGIN_CONTAINER_REF f_rContainerData, MKI_WIC_BITMAP f_hCustomBitmap );

MKI_INTERFACE_MEMBER void
MKI_DialogControlShow( HWND f_hWindow, __int32 f_nCtlID, bool f_fVisible = true );

MKI_INTERFACE_MEMBER void
MKI_DialogControlHide( HWND f_hWindow, __int32 f_nCtlID );

MKI_INTERFACE_MEMBER void
MKI_DialogControlEnable( HWND f_hWindow, __int32 f_nCtlID, bool f_fEnabled = true );

MKI_INTERFACE_MEMBER void
MKI_DialogControlDisable( HWND f_hWindow, __int32 f_nCtlID );

MKI_INTERFACE_MEMBER bool
MKI_DialogControlGetInt32( HWND f_hWindow, __int32 f_nEditID, __int32& f_rnValue );

MKI_INTERFACE_MEMBER bool
MKI_DialogControlGetInt32( HWND f_hWindow, __int32 f_nEditID, __int32& f_rnValue, 
                           __int32 f_nMin, __int32 f_nMax, const LPCWSTR f_pszError );

MKI_INTERFACE_MEMBER bool
MKI_DialogControlGetInt64( HWND f_hWindow, __int32 f_nEditID, __int64& f_rnValue );

MKI_INTERFACE_MEMBER bool
MKI_DialogControlGetInt64( HWND f_hWindow, __int32 f_nEditID, __int64& f_rnValue,
                           __int64 f_nMin, __int64 f_nMax, const LPCWSTR f_pszError );

MKI_INTERFACE_MEMBER bool
MKI_DialogControlGetPoint( HWND f_hWindow, __int32 f_nDlgItemX, __int32 f_nDlgItemY, MKI_POINT_REF f_rPoint );

MKI_INTERFACE_MEMBER void
MKI_DialogControlSetText( HWND f_hWindow, __int32 f_nTextCtlID, const LPCSTR f_pszText );

MKI_INTERFACE_MEMBER void
MKI_DialogControlSetText( HWND f_hWindow, __int32 f_nTextCtlID, const LPCWSTR f_pszText );

MKI_INTERFACE_MEMBER void
MKI_DialogControlSetInt32( HWND f_hWindow, __int32 f_nTextCtlID, const __int32 f_nValue );

MKI_INTERFACE_MEMBER void
MKI_DialogControlSetInt64( HWND f_hWindow, __int32 f_nTextCtlID, const __int64 f_nValue );

MKI_INTERFACE_MEMBER void
MKI_DialogControlSetDouble( HWND f_hWindow, __int32 f_nTextCtlID, const double f_dValue );

MKI_INTERFACE_MEMBER void
MKI_DialogControlSetDoubleEx( HWND f_hWindow, __int32 f_nTextCtlID, const double f_dValue, __int32 f_nPrecision );

MKI_INTERFACE_MEMBER void
MKI_DialogControlSetPoint( HWND f_hWindow, __int32 f_nTextCtlIDX, __int32 f_nUnitCtlIDX, __int32 f_nTextCtlIDY, __int32 f_nUnitCtlIDY, MKI_CONST_POINT_REF f_rPoint );

MKI_INTERFACE_MEMBER void
MKI_DialogControlSetCoordinate( HWND f_hWindow, __int32 f_nTextCtlID, __int32 f_nUnitCtlID, const double f_dValue );

MKI_INTERFACE_MEMBER void
MKI_DialogControlSetLength( HWND f_hWindow, __int32 f_nTextCtlID, __int32 f_nUnitCtlID, const double f_dValue );

MKI_INTERFACE_MEMBER void
MKI_DialogControlSetArea( HWND f_hWindow, __int32 f_nTextCtlID, __int32 f_nUnitCtlID, const double f_dValue );

MKI_INTERFACE_MEMBER void
MKI_DialogControlSetWidth( HWND f_hWindow, __int32 f_nTextCtlID, __int32 f_nUnitCtlID, const double f_dValue );

MKI_INTERFACE_MEMBER void
MKI_DialogControlSetAngle( HWND f_hWindow, __int32 f_nTextCtlID, __int32 f_nUnitCtlID, const double f_dValue );

MKI_INTERFACE_MEMBER bool
MKI_ConvertIntString32( LPWSTR f_pszText, __int32 f_nValue );

MKI_INTERFACE_MEMBER bool
MKI_ConvertIntString64( LPWSTR f_pszText, __int64 f_nValue );

MKI_INTERFACE_MEMBER bool
MKI_ConvertNumberString( LPWSTR f_pszText, __int64 f_nNumber );

MKI_INTERFACE_MEMBER bool
MKI_ConvertBytesString( LPWSTR f_pszText, __int64 f_nValue );

MKI_INTERFACE_MEMBER bool
MKI_ConvertColorString( MKI_COLORVALUE f_cColor, MKI_COLORMODEL f_cColorMode, MKI_DUMMYSTRW& f_rszText );

MKI_INTERFACE_MEMBER bool
MKI_ConvertDoubleString( LPWSTR f_pszText, double f_dValue );

MKI_INTERFACE_MEMBER bool
MKI_ConvertDoubleStringEx( LPWSTR f_pszText, double f_dValue, __int32 f_nTotalLen, __int32 f_nFractionLen );

MKI_INTERFACE_MEMBER bool
MKI_ConvertLengthString( LPWSTR f_pszText, double f_dValue, LPWSTR f_pszUnit );

MKI_INTERFACE_MEMBER bool
MKI_ConvertLengthStringEx( LPWSTR f_pszText, double f_dValue, __int32 f_nTotalLen, __int32 f_nFractionLen, LPWSTR f_pszUnit );

MKI_INTERFACE_MEMBER bool
MKI_ConvertLengthDisplay( LPWSTR f_pszText, double f_dValue );

MKI_INTERFACE_MEMBER bool
MKI_ConvertAreaString( LPWSTR f_pszText, double f_dValue, LPWSTR f_pszUnit );

MKI_INTERFACE_MEMBER bool
MKI_ConvertAreaStringEx( LPWSTR f_pszText, double f_dValue, __int32 f_nTotalLen, __int32 f_nFractionLen, LPWSTR f_pszUnit );

MKI_INTERFACE_MEMBER bool
MKI_ConvertAreaDisplay( LPWSTR f_pszText, double f_dValue );

MKI_INTERFACE_MEMBER bool
MKI_ConvertWidthString( LPWSTR f_pszText, double f_dValue, LPWSTR f_pszUnit );

MKI_INTERFACE_MEMBER bool
MKI_ConvertWidthStringEx( LPWSTR f_pszText, double f_dValue, __int32 f_nTotalLen, __int32 f_nFractionLen, LPWSTR f_pszUnit );

MKI_INTERFACE_MEMBER bool
MKI_ConvertWidthDisplay( LPWSTR f_pszText, double f_dValue );

MKI_INTERFACE_MEMBER bool
MKI_ConvertAngleString( LPWSTR f_pszText, double f_dValue, LPWSTR f_pszUnit );

MKI_INTERFACE_MEMBER bool
MKI_ConvertAngleStringEx( LPWSTR f_pszText, double f_dValue, __int32 f_nTotalLen, __int32 f_nFractionLen, LPWSTR f_pszUnit );

MKI_INTERFACE_MEMBER bool
MKI_ConvertAngleDisplay( LPWSTR f_pszText, double f_dValue );

MKI_INTERFACE_MEMBER bool
MKI_ConvertStringInt32( __int32& f_rValue, const LPCWSTR f_pszText );

MKI_INTERFACE_MEMBER bool
MKI_ConvertStringInt64( __int64& f_rValue, const LPCWSTR f_pszText );

MKI_INTERFACE_MEMBER bool
MKI_ConvertStringDouble( double& f_rValue, const LPCWSTR f_pszText, HWND f_hParentWindow );

MKI_INTERFACE_MEMBER bool
MKI_ConvertStringDoubleFast( double& f_rValue, const LPCWSTR f_pszText );

MKI_INTERFACE_MEMBER bool
MKI_ConvertStringLength( double& f_rValue, const LPCWSTR f_pszText, HWND f_hParentWindow );

MKI_INTERFACE_MEMBER bool
MKI_ConvertStringLengthFast( double& f_rValue, const LPCWSTR f_pszText );

MKI_INTERFACE_MEMBER bool
MKI_ConvertStringArea( double& f_rValue, const LPCWSTR f_pszText, HWND f_hParentWindow );

MKI_INTERFACE_MEMBER bool
MKI_ConvertStringAreaFast( double& f_rValue, const LPCWSTR f_pszText );

MKI_INTERFACE_MEMBER bool
MKI_ConvertStringWidth( double& f_rValue, const LPCWSTR f_pszText, HWND f_hParentWindow );

MKI_INTERFACE_MEMBER bool
MKI_ConvertStringWidthFast( double& f_rValue, const LPCWSTR f_pszText );

MKI_INTERFACE_MEMBER bool
MKI_ConvertStringAngle( double& f_rValue, const LPCWSTR f_pszText, HWND f_hParentWindow );

MKI_INTERFACE_MEMBER bool
MKI_ConvertStringAngleFast( double& f_rValue, const LPCWSTR f_pszText );

MKI_INTERFACE_MEMBER bool
MKI_ConvertVariables( const LPCWSTR f_pszText, LPWSTR f_pszBuffer, const size_t f_nBufferSize, 
                      MKI_CONST_ENTITYSTACK_PTR f_pStack, __int32 f_nSystem, MKI_CONST_UNIQUE_ID_PTR f_pLinkedID );

#endif

    template <size_t f_unSize> __forceinline bool
    MKI_ConvertVariables( const LPCWSTR f_pszText, wchar_t (&f_rDestin)[f_unSize], 
                          MKI_CONST_ENTITYSTACK_PTR f_pStack, __int32 f_nSystem, MKI_CONST_UNIQUE_ID_PTR f_pLinkedID )
    {
      return( MKI_ConvertVariables( f_pszText, (LPWSTR) &f_rDestin, f_unSize, f_pStack, f_nSystem, f_pLinkedID ) );
    }

#ifndef MKI_SERVER

MKI_INTERFACE_MEMBER bool
MKI_ConvertVariablesEx( const LPCWSTR f_pszText, LPWSTR f_pszBuffer, const size_t f_nBufferSize, 
                        MKI_CONST_ENTITYSTACK_PTR f_pStack, __int32 f_nSystem, MKI_CONST_UNIQUE_ID_PTR f_pLinkedID,
                        MKI_VARIABLEDEF_PTR f_pExtraVariables, __int32 f_nExtraCount, __int32 f_nDigits );

#endif

    template <size_t f_unSize> __forceinline bool
    MKI_ConvertVariablesEx( const LPCWSTR f_pszText, wchar_t (&f_rDestin)[f_unSize], 
                            MKI_CONST_ENTITYSTACK_PTR f_pStack, __int32 f_nSystem, MKI_CONST_UNIQUE_ID_PTR f_pLinkedID,
                            MKI_VARIABLEDEF_PTR f_pExtraVariables, __int32 f_nExtraCount, __int32 f_nDigits )
    {
      return( MKI_ConvertVariablesEx( f_pszText, (LPWSTR) &f_rDestin, f_unSize, f_pStack, f_nSystem, f_pLinkedID, f_pExtraVariables, f_nExtraCount, f_nDigits ) );
    }

#ifndef MKI_SERVER

MKI_INTERFACE_MEMBER MKI_COLORVALUE
MKI_ColorConvertMode( MKI_COLORVALUE f_cInputColor, MKI_COLORMODEL f_cInputColorMode, MKI_COLORMODEL f_cOutputColorMode );

MKI_INTERFACE_MEMBER MKI_COLORVALUE
MKI_ColorFade( MKI_COLORVALUE f_cColor, MKI_COLORMODEL f_cColorMode, double f_dColorFade );

MKI_INTERFACE_MEMBER D2D1_COLOR_F
MKI_ColorConvertD2D( MKI_COLORVALUE f_cColorRGB, double f_dAlpha );

MKI_INTERFACE_MEMBER D2D1_COLOR_F
MKI_ColorConvertD2D( MKI_COLORVALUE f_cColor, MKI_COLORMODEL f_cColorMode, double f_dAlpha );

MKI_INTERFACE_MEMBER double
MKI_ColorBrightness( MKI_COLORVALUE f_cColor, MKI_COLORMODEL f_cColorMode );

MKI_INTERFACE_MEMBER MKI_COLORVALUE
MKI_ColorMix( MKI_COLORVALUE f_cColor1, MKI_COLORVALUE f_cColor2, MKI_COLORMODEL f_cColorMode, double f_dAmount );

MKI_INTERFACE_MEMBER DWORD
MKI_ColorGetSortValue( MKI_COLORVALUE f_cColor, MKI_COLORMODEL f_cColorMode );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER _locale_t&
MKI_GetLocale( void );

MKI_INTERFACE_MEMBER __int32
MKI_GetApplicationType( void );

MKI_INTERFACE_MEMBER __int32
MKI_GetApplicationState( void );

MKI_INTERFACE_MEMBER bool
MKI_ApplicationIsDisabled( void );

MKI_INTERFACE_MEMBER bool
MKI_GetVersionString( MKI_DUMMYSTRW& f_rszText );

MKI_INTERFACE_MEMBER bool
MKI_GetBuildDate( const LPCSTR f_pszDate, MKI_STR32A& f_rszText );

MKI_INTERFACE_MEMBER bool
MKI_GetCommandTitle( MKI_DUMMYSTRW& f_rszText, __int32 f_nCommandID );

MKI_INTERFACE_MEMBER bool
MKI_GetCommandIcon( __int32 f_nCommandID, MKI_ICON_DATA_REF f_rIconData );

MKI_INTERFACE_MEMBER __int32
MKI_GetProcessorCount( void );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER bool
MKI_CalcTextFrame( MKI_CONST_TEXTSTANDARD_REF f_rData, const LPCWSTR f_pszText, MKI_RECT_REF f_rFrame );

MKI_INTERFACE_MEMBER MKI_ENTITY_BLOCK_PTR
MKI_CalcBlockFrame( const LPCWSTR f_pszBlockName, const LPCWSTR f_pszLibraryName,
                    MKI_CONST_MATRIX_PTR f_pMatrix, MKI_RECT_PTR f_pFrame, bool f_fAutoLoad );

MKI_INTERFACE_MEMBER bool
MKI_CalcRefFrame( const LPCWSTR f_pszDrawingName, __int32 f_nDisplayMode,
                  MKI_CONST_MATRIX_PTR f_pMatrix, MKI_RECT_PTR f_pFrame );

MKI_INTERFACE_MEMBER bool
MKI_CalcArcFrame( MKI_CONST_POINT_REF f_rCenter, 
                  MKI_CONST_POINT_REF f_rAngle1, MKI_CONST_POINT_REF f_rAngle2, bool f_fPositive,
                  MKI_CONST_MATRIX_PTR f_pMatrix, MKI_RECT_REF f_rFrame );

MKI_INTERFACE_MEMBER bool
MKI_CalcObjectArea( MKI_CONST_ENTITY_OBJECT_PTR f_pObj, double& f_rResult );

MKI_INTERFACE_MEMBER bool
MKI_CalcObjectCentroid( MKI_CONST_ENTITY_OBJECT_PTR f_pObj, MKI_POINT_REF f_rResult );

MKI_INTERFACE_MEMBER bool
MKI_CalcObjectPerimeter( MKI_CONST_ENTITY_OBJECT_PTR f_pObj, double& f_rResult );

MKI_INTERFACE_MEMBER bool
MKI_CalcEntityDistance( MKI_CONST_ENTITY_PTR f_pEntity, MKI_CONST_POINT_REF f_rPoint, double& f_rResult );

MKI_INTERFACE_MEMBER bool
MKI_CalcPointInsideObject( MKI_CONST_ENTITY_OBJECT_PTR f_pObj,
                           MKI_CONST_POINT_REF f_rPoint, double f_dTolerance, bool& f_rUncertain );

MKI_INTERFACE_MEMBER __int32
MKI_CalcSegmentNumber( double f_dLength );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER double
MKI_CalcArcAngle( MKI_CONST_POINT_REF f_rCenter, 
                  MKI_CONST_POINT_REF f_rAngle1, MKI_CONST_POINT_REF f_rAngle2, bool f_fPositive );

MKI_INTERFACE_MEMBER double
MKI_CalcArcBisection( MKI_CONST_POINT_REF f_rCenter, 
                      MKI_CONST_POINT_REF f_rAngle1, MKI_CONST_POINT_REF f_rAngle2, bool f_fPositive );

MKI_INTERFACE_MEMBER double
MKI_CalcArcCurvature( MKI_CONST_POINT_REF f_rCenter,
                      MKI_CONST_POINT_REF f_rStart, MKI_CONST_POINT_REF f_rEnd );

MKI_INTERFACE_MEMBER double
MKI_CalcEllipseAngle( MKI_CONST_POINT_REF f_rCenter, MKI_CONST_POINT_REF f_rVector1, MKI_CONST_POINT_REF f_rVector2, 
                      MKI_CONST_POINT_REF f_rAngle, MKI_POINT_PTR f_pResult );

MKI_INTERFACE_MEMBER bool
MKI_CalcDistance( MKI_CONST_GEO_OBJECT_REF f_rGeoObj, MKI_CONST_POINT_REF f_rPoint, double& f_rResult );

MKI_INTERFACE_MEMBER __int32
MKI_CalcIntersection( MKI_CONST_GEO_OBJECT_REF f_rGeoObj1, MKI_CONST_GEO_OBJECT_REF f_rGeoObj2, MKI_POINT_PTR f_pResults );

MKI_INTERFACE_MEMBER __int32
MKI_CalcPerpendicular( MKI_CONST_GEO_OBJECT_REF f_rGeoObj, MKI_CONST_POINT_REF f_rPoint, MKI_POINT_PTR f_pResults );

MKI_INTERFACE_MEMBER __int32
MKI_CalcTangent( MKI_CONST_GEO_OBJECT_REF f_rGeoObj1, MKI_CONST_GEO_OBJECT_REF f_rGeoObj2, MKI_POINT_PTR f_pResults );

MKI_INTERFACE_MEMBER __int32
MKI_CalcRadiusFit( MKI_CONST_GEO_OBJECT_REF f_rGeoObj1,
                   MKI_CONST_GEO_OBJECT_REF f_rGeoObj2, double f_dRadius, __int32 f_nObjType, MKI_GEO_OBJECT_PTR f_pResults );

MKI_INTERFACE_MEMBER __int32
MKI_CalcIncircle( MKI_CONST_POINT_REF f_rPoint1, MKI_CONST_POINT_REF f_rPoint2,
                  MKI_CONST_POINT_REF f_rPoint3, MKI_POINT_REF f_rCenter, MKI_POINT_REF f_rRadius );

MKI_INTERFACE_MEMBER __int32
MKI_CalcCircumcircle( MKI_CONST_POINT_REF f_rPoint1, MKI_CONST_POINT_REF f_rPoint2,
                      MKI_CONST_POINT_REF f_rPoint3, MKI_POINT_REF f_rCenter );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_CalcEquidistant( MKI_CONST_ENTITY_OBJECT_PTR f_pObj1, MKI_ENTITY_OBJECT_PTR f_pObj2,
                     double f_dDistance, bool f_fRounded, bool f_fResolve, 
                     MKI_CONST_SIMPLIFY_DATA_PTR f_pSimplify, MKI_CONST_POINT_REF f_rReferencePoint );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_CalcEquidistantEx( MKI_CONST_ENTITY_OBJECT_PTR f_pObj1, MKI_ENTITY_OBJECT_PTR f_pObj2,
                       double f_dDistance, bool f_fRounded, bool f_fResolve, 
                       MKI_CONST_SIMPLIFY_DATA_PTR f_pSimplify, MKI_CONST_POINT_PTR f_pReferencePoint, bool& f_rfInside );

MKI_INTERFACE_MEMBER __int32
MKI_CalcBezier( MKI_CONST_GEO_OBJECT_REF f_rGeoObj, MKI_POINTDATA_PTR f_pResults );

MKI_INTERFACE_MEMBER bool
MKI_CalcPointInsidePolygon( MKI_CONST_POINT_PTR f_pPoints, __int32 f_nSize, MKI_CONST_POINT_REF f_rPoint );

MKI_INTERFACE_MEMBER bool
MKI_CalcPointInsideArea( MKI_CONST_BUFFER_REF f_rData, MKI_CONST_POINT_REF f_rPoint );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_EnumerateAll( __int32 f_nEnumFlag1, __int32 f_nEnumFlag2, __int64 f_nEnumMode, MKI_ENUMOBJECT_PROC f_fnCallback, void* f_pUserData );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_EnumerateAllRect( __int32 f_nEnumFlag1, __int32 f_nEnumFlag2, __int64 f_nEnumMode, MKI_ENUMOBJECT_PROC f_fnCallback, 
                      MKI_CONST_RECT_REF f_rEnumRect, void* f_pUserData );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_EnumerateAllExt( __int32 f_nEnumFlag1, __int32 f_nEnumFlag2, MKI_ENUMOBJECT_PROC f_fnCallback, void* f_pUserData );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_EnumeratePageRect( __int32 f_nEnumFlag1, __int32 f_nEnumFlag2, __int64 f_nEnumMode, MKI_ENUMOBJECT_PROC f_fnCallback, 
                       MKI_CONST_RECT_REF f_rEnumRect, __int32 f_nPageIndex, void* f_pUserData );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_EnumeratePage( __int32 f_nEnumFlag1, __int32 f_nEnumFlag2, __int64 f_nEnumMode, MKI_ENUMOBJECT_PROC f_fnCallback,
                   __int32 f_nPageIndex, void* f_pUserData );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_EnumerateNative( MKI_ENUMOBJECT_PROC f_fnCallback, void* f_pUserData );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_EnumerateLibrary( const LPCWSTR f_pszLibraryName, __int64 f_nEnumMode, MKI_ENUMOBJECT_PROC f_fnCallback, void* f_pUserData );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_EnumerateEntity( MKI_CONST_ENTITY_PTR f_pEntity, __int64 f_nEnumMode, MKI_ENUMOBJECT_PROC f_fnCallback, void* f_pUserData );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_EnumerateEntityAttrib( MKI_CONST_ENTITY_PTR f_pEntity, __int32 f_nMode, MKI_ENUMATTRIB_PROC f_fnCallback, 
                           void* f_pUserData, __int32* f_pAttribCount );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_EnumerateBlock( const LPCWSTR f_pszBlockName, const LPCWSTR f_pszLibraryName, MKI_CONST_XPROPERTY_PTR f_pXProperty, 
                    __int64 f_nEnumMode, MKI_ENUMOBJECT_PROC f_fnCallback, void* f_pUserData );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_EnumerateBlockAttrib( const LPCWSTR f_pszBlockName, const LPCWSTR f_pszLibraryName, __int32 f_nMode,
                          MKI_ENUMATTRIB_PROC f_fnCallback, void* f_pUserData, __int32* f_pAttribCount );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_EnumerateChar( MKI_CONST_FONTDEF_PTR f_pFont, __int32 f_nCharIndex, MKI_CONST_MATRIX_PTR f_pMatrix,
                   __int64 f_nEnumMode, MKI_ENUMOBJECT_PROC f_fnCallback, void* f_pUserData );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_EnumerateExt( __int16 f_nOwnerID, __int16 f_nPlugInID, MKI_ENTITY_EXT_PTR f_pExt, 
                  __int32 f_nEnumMode, MKI_ENUMEXT_PROC f_fnCallback, void* f_pUserData );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_EnumerateGroup( MKI_ENTITY_GROUP_PTR f_pGroup, MKI_ENUMIDENT_PROC f_fnCallback, bool f_fAll, void* f_pUserData );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_EnumerateIdent( MKI_ENUMIDENT_PROC f_fnCallback, bool f_fReversed, void* f_pUserData );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_EnumerateFlag(  __int32 f_nEnumFlag1, __int32 f_nEnumFlag2,
                   MKI_ENUMIDENT_PROC f_fnCallback, bool f_fReversed, void* f_pUserData );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_EnumeratePoints( MKI_ENUMPOINTS_PROC f_fnCallback, void* f_pUserData );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_EnumerateHatchLine( MKI_CONST_BUFFER_REF f_rData, MKI_CONST_HATCHLINELIST_REF f_rHatchLines,
                        double f_dHatchTotalRotate, double f_dHatchOffset1, double f_dHatchOffset2,
                        MKI_CONST_POINT_PTR f_pHatchOrigin, MKI_CONST_XPROPERTY_PTR f_pXProperty,
                        MKI_HATCHOPENPATH_PROC f_fnOpenPathCallback,
                        MKI_HATCHADDOBJECT_PROC f_fnAddObjectCallback,
                        MKI_HATCHCLOSEPATH_PROC f_fnClosePathCallback, 
                        void* f_pUserData );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_EnumerateHatchBlock( MKI_CONST_BUFFER_REF f_rData, MKI_CONST_HATCHDEF_REF f_rHatchDef,
                         MKI_CONST_POINT_PTR f_pHatchOrigin, MKI_CONST_XPROPERTY_PTR f_pXProperty,
                         MKI_HATCHOPENPATH_PROC f_fnOpenPathCallback,
                         MKI_HATCHADDOBJECT_PROC f_fnAddObjectCallback,
                         MKI_HATCHCLOSEPATH_PROC f_fnClosePathCallback, 
                         void* f_pUserData );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER MKI_ENTITY_PTR
MKI_EntityBuffer( void );

MKI_INTERFACE_MEMBER MKI_ENTITY_OBJECT_PTR
MKI_ContainerBuffer( MKI_CONST_XPROPERTY_PTR f_pXProperty );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER void
MKI_InputDrawPoint( MKI_CONST_POINT_REF f_rPoint );

MKI_INTERFACE_MEMBER void
MKI_InputDrawFadedRect( MKI_CONST_POINT_REF f_rPoint1, MKI_CONST_POINT_REF f_rPoint2 );

MKI_INTERFACE_MEMBER void
MKI_InputDrawLine( MKI_CONST_POINT_REF f_rPoint1, MKI_CONST_POINT_REF f_rPoint2 );

MKI_INTERFACE_MEMBER void
MKI_InputDrawEndless( MKI_CONST_POINT_REF f_rPoint1, MKI_CONST_POINT_REF f_rPoint2 );

MKI_INTERFACE_MEMBER void
MKI_InputDrawBeam( MKI_CONST_POINT_REF f_rPoint1, MKI_CONST_POINT_REF f_rPoint2 );

MKI_INTERFACE_MEMBER void
MKI_InputDrawBezier( MKI_CONST_POINT_REF f_rPoint1, MKI_CONST_POINT_REF f_rPoint2,
                     MKI_CONST_POINT_REF f_rPoint3, MKI_CONST_POINT_REF f_rPoint4 );

MKI_INTERFACE_MEMBER void
MKI_InputDrawCurve( MKI_CONST_POINT_PTR f_pPoints, const __int32* f_pTypes, __int32 f_nPointCount, bool f_fClosed );

MKI_INTERFACE_MEMBER void
MKI_InputDrawFrame( MKI_CONST_POINT_REF f_rPoint1, MKI_CONST_POINT_REF f_rPoint2,
                    MKI_CONST_POINT_REF f_rPoint3, MKI_CONST_POINT_REF f_rPoint4 );

MKI_INTERFACE_MEMBER void
MKI_InputDrawCircle( MKI_CONST_POINT_REF f_rCenter, MKI_CONST_POINT_REF f_rRadius );

MKI_INTERFACE_MEMBER void
MKI_InputDrawCircleArc( MKI_CONST_POINT_REF f_rCenter,
                        MKI_CONST_POINT_REF f_rAngle1, MKI_CONST_POINT_REF f_rAngle2, bool f_fPositive );

MKI_INTERFACE_MEMBER void
MKI_InputDrawEllipse( MKI_CONST_POINT_REF f_rCenter, 
                      MKI_CONST_POINT_REF f_rVector1, MKI_CONST_POINT_REF f_rVector2 );

MKI_INTERFACE_MEMBER void
MKI_InputDrawEllipseArc( MKI_CONST_POINT_REF f_rCenter, 
                         MKI_CONST_POINT_REF f_rVector1, MKI_CONST_POINT_REF f_rVector2,
                         MKI_CONST_POINT_REF f_rAngle1, MKI_CONST_POINT_REF f_rAngle2, bool f_fPositive );

MKI_INTERFACE_MEMBER void
MKI_InputDrawPatch( MKI_CONST_PATCH_REF f_rPatch, const D2D1_COLOR_F* f_pFillColor, const D2D1_COLOR_F* f_pEdgeColor );

MKI_INTERFACE_MEMBER void
MKI_InputDrawReference( MKI_CONST_GEO_OBJECT_REF f_rGeoObject, const D2D1_COLOR_F* f_pColor );

MKI_INTERFACE_MEMBER void
MKI_InputDrawTempEntity( bool f_fSecondary );

MKI_INTERFACE_MEMBER void
MKI_InputDrawCursorText( const LPCWSTR f_pszText );

MKI_INTERFACE_MEMBER void
MKI_InputFeedbackInitRect( MKI_CONST_RECT_REF f_rRect, MKI_CONST_MATRIX_REF f_rMatrix );

MKI_INTERFACE_MEMBER void
MKI_InputFeedbackInitQuad( MKI_CONST_QUAD_REF f_rQuad, MKI_CONST_MATRIX_REF f_rMatrix );

MKI_INTERFACE_MEMBER void
MKI_InputFeedbackSetBlock( const LPCWSTR f_pszBlockName, const LPCWSTR f_pszLibraryName, __int32 f_nDisplayMode );

MKI_INTERFACE_MEMBER void
MKI_InputFeedbackSetFlag( __int32 f_nFlag );

MKI_INTERFACE_MEMBER void
MKI_InputFeedbackSetEntity( MKI_ENTITY_PTR f_pEntity );

MKI_INTERFACE_MEMBER void
MKI_InputFeedbackDisplay( MKI_CONST_MATRIX_REF f_rMatrix, bool f_fDrawFrame );

MKI_INTERFACE_MEMBER void
MKI_InputSetPoint( __int32 f_nPointIndex, MKI_CONST_POINT_REF f_rPoint );

MKI_INTERFACE_MEMBER bool
MKI_InputGetLastPoint( MKI_POINT_REF f_rPoint );

MKI_INTERFACE_MEMBER __int32
MKI_InputGetIdentData( __int32 f_nPointIndex, MKI_GEO_OBJECT_REF f_rGeoObject );

MKI_INTERFACE_MEMBER __int32
MKI_InputGetIdentObject( __int32 f_nPointIndex, MKI_GEO_OBJECT_REF f_rGeoObject );

MKI_INTERFACE_MEMBER MKI_ENTITY_PTR
MKI_InputGetIdentAddress( __int32 f_nPointIndex );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_InputGenerateSurfaceStep1( __int32* f_pMode, double* f_pTolerance, MKI_BUFFER_REF f_rBuffer );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_InputGenerateSurfaceStep2( __int32 f_nPointIndex, __int32 f_nPointCount, MKI_BUFFER_REF f_rBuffer, 
                               double* f_pTolerance, MKI_CONST_SIMPLIFY_DATA_PTR f_pSimplify, bool f_fSecondary );

MKI_INTERFACE_MEMBER MKI_ENTITY_PTR
MKI_InputGetTempEntity( bool f_fSecondary );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER bool
MKI_EditSearchID( MKI_CONST_UNIQUE_ID_REF f_rUniqueID, MKI_SEARCH_DATA_REF f_rSearchData, bool f_fIncludeLibraries );

MKI_INTERFACE_MEMBER bool
MKI_EditIdentStart( void );

MKI_INTERFACE_MEMBER void
MKI_EditIdentSetEntity( MKI_ENTITY_PTR f_pEntity );

MKI_INTERFACE_MEMBER bool
MKI_EditIdentClear( void );

MKI_INTERFACE_MEMBER bool
MKI_EditIdentFinish( void );

MKI_INTERFACE_MEMBER __int32
MKI_EditIdentCount( MKI_RECT_PTR f_pFrame );

MKI_INTERFACE_MEMBER void
MKI_EditIdentRedraw( void );

MKI_INTERFACE_MEMBER bool
MKI_EditIdentMatrix( MKI_CONST_MATRIX_PTR f_pMatrix, __int32 f_nScaleMode, bool f_fDuplicate );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_EditIdentEnumModify( MKI_ENUMOBJECT_PROC f_fnCallback, bool f_fDuplicate, void* f_pUserData );

MKI_INTERFACE_MEMBER bool
MKI_EditIdentDuplicate( bool f_fDuplicate );

MKI_INTERFACE_MEMBER void
MKI_EditMarkDeleted( MKI_ENTITY_PTR f_pEntity );

MKI_INTERFACE_MEMBER void
MKI_EditRefresh( MKI_ENTITY_PTR f_pEntity );

MKI_INTERFACE_MEMBER void
MKI_EditRefreshAll( void );

MKI_INTERFACE_MEMBER bool
MKI_EditMoveBefore( MKI_ENTITY_PTR f_pEntity, MKI_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_EditMoveBehind( MKI_ENTITY_PTR f_pEntity, MKI_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_EditMoveBefore( MKI_ENTITY_PTR f_pParent, MKI_ENTITY_PTR f_pEntity, MKI_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_EditMoveBehind( MKI_ENTITY_PTR f_pParent, MKI_ENTITY_PTR f_pEntity, MKI_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER MKI_ENTITY_PTR
MKI_EditLinkBefore( MKI_ENTITY_PTR f_pEntity, MKI_ENTITY_PTR f_pRefEntity, bool f_fDuplicate );

MKI_INTERFACE_MEMBER MKI_ENTITY_PTR
MKI_EditLinkBehind( MKI_ENTITY_PTR f_pEntity, MKI_ENTITY_PTR f_pRefEntity, bool f_fDuplicate );

MKI_INTERFACE_MEMBER MKI_ENTITY_PTR
MKI_EditCopyBefore( MKI_ENTITY_PTR f_pEntity, MKI_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER MKI_ENTITY_PTR
MKI_EditCopyBehind( MKI_ENTITY_PTR f_pEntity, MKI_ENTITY_PTR f_pRefEntity );

MKI_INTERFACE_MEMBER bool
MKI_EditBlockResetProperties( const LPCWSTR f_pszBlockName, const LPCWSTR f_pszLibraryName );

MKI_INTERFACE_MEMBER bool
MKI_EditInstanceResolve( MKI_CONST_ENTITY_INSTANCE_PTR f_pInst, bool f_fIsPositionNumber );

MKI_INTERFACE_MEMBER bool
MKI_EditClipResolve( MKI_CONST_ENTITY_CLIP_PTR f_pClip );

MKI_INTERFACE_MEMBER bool
MKI_EditGroupResolve( MKI_CONST_ENTITY_GROUP_PTR f_pGroup, bool f_fAll );

MKI_INTERFACE_MEMBER bool
MKI_EditExtResolve( MKI_CONST_ENTITY_EXT_PTR f_pExt );

MKI_INTERFACE_MEMBER MKI_ENTITY_GROUP_PTR
MKI_EditExtToGroup( MKI_CONST_ENTITY_EXT_PTR f_pExt, const LPCWSTR f_pszComment );

MKI_INTERFACE_MEMBER __int32
MKI_EditPointsCount( MKI_RECT_PTR f_pFrame );

MKI_INTERFACE_MEMBER bool
MKI_EditPointsMatrix( MKI_CONST_MATRIX_PTR f_pMatrix, bool f_fDuplicate );

MKI_INTERFACE_MEMBER void
MKI_EditEntityPointsMatrix( MKI_ENTITY_PTR f_pEntity, MKI_CONST_MATRIX_PTR f_pMatrix );

MKI_INTERFACE_MEMBER bool
MKI_EditXPropertyApply( MKI_XPROPERTY_REF f_rTarget, MKI_CONST_XPROPERTY_REF f_rModify, __int32 f_nFlagMask, __int32 f_nPropMask );

MKI_INTERFACE_MEMBER bool
MKI_EditXPropertyResolve( MKI_CONST_XPROPERTY_REF f_rSource, MKI_XPROPERTY_REF f_rTarget );

MKI_INTERFACE_MEMBER bool
MKI_EditXPropertyInherit( MKI_ENTITY_PTR f_pEntity, MKI_CONST_XPROPERTY_REF f_rModify );

MKI_INTERFACE_MEMBER bool
MKI_EditScanForEscape( bool f_fNoPrompt = false );

MKI_INTERFACE_MEMBER bool
MKI_PhysicalDeleteBlock( const LPCWSTR f_pszBlockName, const LPCWSTR f_pszLibraryName, bool f_fOnlyIfUnused );

MKI_INTERFACE_MEMBER bool
MKI_PhysicalDeleteEntity( MKI_ENTITY_PTR f_pEntity );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_TransformCurveSurface( MKI_CONST_ENTITY_OBJECT_PTR f_pObj1, MKI_ENTITY_OBJECT_PTR f_pObj2 );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_TransformSurface( MKI_CONST_ENTITY_OBJECT_PTR f_pObj1, MKI_ENTITY_OBJECT_PTR f_pObj2 );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_TransformDistortable( MKI_CONST_ENTITY_PTR f_pEntity1, MKI_ENTITY_PTR f_pEntity2 );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_TransformInvert( MKI_CONST_ENTITY_OBJECT_PTR f_pObj1, MKI_ENTITY_OBJECT_PTR f_pObj2 );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_TransformSimplify( MKI_CONST_ENTITY_OBJECT_PTR f_pObj1, MKI_ENTITY_OBJECT_PTR f_pObj2, MKI_CONST_SIMPLIFY_DATA_PTR f_pSimplify );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_TransformReverseOneOutline( MKI_CONST_ENTITY_OBJECT_PTR f_pObj1, MKI_ENTITY_OBJECT_PTR f_pObj2, __int32 f_nSubIndex );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_TransformReverseAllOutlines( MKI_CONST_ENTITY_OBJECT_PTR f_pObj1, MKI_ENTITY_OBJECT_PTR f_pObj2 );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_TransformStartPoint( MKI_CONST_ENTITY_OBJECT_PTR f_pObj1, MKI_ENTITY_OBJECT_PTR f_pObj2, MKI_CONST_POINT_REF f_rPoint );

MKI_INTERFACE_MEMBER MKI_RESULTVALUE
MKI_TransformObjectType( MKI_CONST_ENTITY_OBJECT_PTR f_pObj1, MKI_ENTITY_OBJECT_PTR f_pObj2, __int32 f_nObjType );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER bool
MKI_FileGetStandardPath( __int32 f_nPathID, MKI_FILENAMEW& f_rszPathName );

MKI_INTERFACE_MEMBER void
MKI_FileSetCurrentFile( const LPCWSTR f_pszFileName );

MKI_INTERFACE_MEMBER bool
MKI_FileSetExtension( const LPCWSTR f_pszOldName, MKI_FILENAMEW& f_rszNewName, const LPCWSTR f_pszExtension );

MKI_INTERFACE_MEMBER bool
MKI_FileGetExtension( const LPCWSTR f_pszFullName, MKI_FILENAMEW& f_rszExtension );

MKI_INTERFACE_MEMBER bool
MKI_FileShortName( const LPCWSTR f_pszFullName, MKI_FILENAMEW& f_rszShortName );

MKI_INTERFACE_MEMBER bool
MKI_FilePathName( const LPCWSTR f_pszFullName, MKI_FILENAMEW& f_rszPath );

MKI_INTERFACE_MEMBER bool
MKI_FileBaseName( const LPCWSTR f_pszFullName, MKI_FILENAMEW& f_rszBase );

MKI_INTERFACE_MEMBER bool
MKI_FileFullPath( const LPCWSTR f_pszOldName, MKI_FILENAMEW& f_rszNewName );

MKI_INTERFACE_MEMBER bool
MKI_FileApplicationPath( const LPCWSTR f_pszOldName, MKI_FILENAMEW& f_rszNewName );

MKI_INTERFACE_MEMBER bool
MKI_FileAllUsersPath( const LPCWSTR f_pszOldName, MKI_FILENAMEW& f_rszNewName );

MKI_INTERFACE_MEMBER bool
MKI_FileUserPath( const LPCWSTR f_pszOldName, MKI_FILENAMEW& f_rszNewName );

MKI_INTERFACE_MEMBER bool
MKI_FileFindLocal( const LPCWSTR f_pszOldName, MKI_FILENAMEW& f_rszNewName );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER bool
MKI_FileExist( const LPCWSTR f_pszFileName );

MKI_INTERFACE_MEMBER bool
MKI_FileDelete( const LPCWSTR f_pszFileName );

MKI_INTERFACE_MEMBER bool
MKI_FileGrantUserAccess( const LPCWSTR f_pszFileName );

MKI_INTERFACE_MEMBER bool
MKI_FileIsLocked( const LPCWSTR f_pszFileName );

MKI_INTERFACE_MEMBER bool
MKI_FileOpen( HANDLE* f_pFileHandle, const LPCWSTR f_pszFileName );

MKI_INTERFACE_MEMBER bool
MKI_FileCreate( HANDLE* f_pFileHandle, const LPCWSTR f_pszFileName );

MKI_INTERFACE_MEMBER bool
MKI_FileAppend( HANDLE* f_pFileHandle, const LPCWSTR f_pszFileName );

MKI_INTERFACE_MEMBER __int64
MKI_FileSize( HANDLE f_hFileHandle );

MKI_INTERFACE_MEMBER bool
MKI_FileCheckUTF8( HANDLE f_hFileHandle );

MKI_INTERFACE_MEMBER size_t
MKI_FileUnbufferedRead( HANDLE f_hFileHandle, void* f_pData, const size_t f_unSize );

MKI_INTERFACE_MEMBER size_t
MKI_FileUnbufferedWrite( HANDLE f_hFileHandle, const void* const f_pData, const size_t f_unSize );

MKI_INTERFACE_MEMBER __int32
MKI_FileUnbufferedWriteAsUTF8( HANDLE f_hFileHandle, const LPCWSTR f_pszText );

MKI_INTERFACE_MEMBER __int32
MKI_FileUnbufferedWriteAsUTF8( HANDLE f_hFileHandle, const LPCWSTR f_pszText, const size_t f_unLength );

MKI_INTERFACE_MEMBER __int32
MKI_FileUnbufferedWriteAsAnsi( HANDLE f_hFileHandle, const LPCWSTR f_pszText );

MKI_INTERFACE_MEMBER __int32
MKI_FileUnbufferedWriteAsAnsi( HANDLE f_hFileHandle, const LPCWSTR f_pszText, const size_t f_unLength );

MKI_INTERFACE_MEMBER bool
MKI_FileSetPointer( HANDLE f_hFileHandle, __int64 f_nOffset, DWORD f_dwMode );

MKI_INTERFACE_MEMBER bool
MKI_FileClose( HANDLE f_hFileHandle );

MKI_INTERFACE_MEMBER bool
MKI_FileCopy( const LPCWSTR f_pszSourceName, const LPCWSTR f_pszDestinName );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER bool
MKI_FileWriteInitNull( void );

MKI_INTERFACE_MEMBER bool
MKI_FileWriteInitDisk( HANDLE f_hFileHandle, bool f_fGenerateCRC, bool f_fIsUTF8 );

MKI_INTERFACE_MEMBER bool
MKI_FileWriteInitMemory( HGLOBAL f_hMemory, const size_t f_nMemorySize );

MKI_INTERFACE_MEMBER bool
MKI_FileWriteByteData( const void* const f_pData, const size_t f_unSize );

MKI_INTERFACE_MEMBER bool
MKI_FileWriteTextData( const LPCSTR f_pszData );

MKI_INTERFACE_MEMBER bool
MKI_FileWriteFlush( void );

MKI_INTERFACE_MEMBER void
MKI_FileWriteExit( void );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER void
MKI_FileWriteNewline( void );

MKI_INTERFACE_MEMBER void
MKI_FileWriteComma( void );

MKI_INTERFACE_MEMBER void
MKI_FileWriteSemi( void );

MKI_INTERFACE_MEMBER void
MKI_FileWriteComment( const LPCSTR f_pszValue );

MKI_INTERFACE_MEMBER void
MKI_FileWriteKeyword( __int32 f_nKeyword );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER void
MKI_FileWriteString( const LPCWSTR f_pszValue );

MKI_INTERFACE_MEMBER void
MKI_FileWriteBinary( const void* const f_pValue, const size_t f_unSize );

MKI_INTERFACE_MEMBER void
MKI_FileWriteBool( bool f_fValue );

MKI_INTERFACE_MEMBER void
MKI_FileWriteInt16( __int16 f_nValue );

MKI_INTERFACE_MEMBER void
MKI_FileWriteInt32( __int32 f_nValue );

MKI_INTERFACE_MEMBER void
MKI_FileWriteInt64( __int64 f_nValue );

MKI_INTERFACE_MEMBER void
MKI_FileWriteDouble( double f_dValue );

MKI_INTERFACE_MEMBER void
MKI_FileWriteColor( MKI_COLORVALUE f_cValue, MKI_COLORMODEL f_cColorMode );

MKI_INTERFACE_MEMBER void
MKI_FileWriteFontdef( MKI_CONST_FONTDEF_REF f_rFont );

MKI_INTERFACE_MEMBER void
MKI_FileWriteXProperty( MKI_CONST_XPROPERTY_REF f_rXProperty );

MKI_INTERFACE_MEMBER void
MKI_FileWriteProperty( MKI_CONST_PROPERTY_REF f_rProperty );

MKI_INTERFACE_MEMBER void
MKI_FileWriteDimLine( MKI_CONST_DIMLINE_REF f_rDimLine );

MKI_INTERFACE_MEMBER void
MKI_FileWriteDimSmall( MKI_CONST_DIMSMALL_REF f_rDimSmall );

MKI_INTERFACE_MEMBER void
MKI_FileWriteDimLarge( MKI_CONST_DIMLARGE_REF f_rDimLarge );

MKI_INTERFACE_MEMBER void
MKI_FileWriteTextStandard( MKI_CONST_TEXTSTANDARD_REF f_rTextStandard );

MKI_INTERFACE_MEMBER void
MKI_FileWriteTextFrame( MKI_CONST_TEXTFRAME_REF f_rTextFrame );

MKI_INTERFACE_MEMBER void
MKI_FileWriteTextReference( MKI_CONST_TEXTREFERENCE_REF f_rTextReference );

MKI_INTERFACE_MEMBER void
MKI_FileWriteCommentData( MKI_CONST_COMMENTDATA_REF f_rCommentData );

MKI_INTERFACE_MEMBER void
MKI_FileWriteBitmapRef( MKI_CONST_BITMAPREF_REF f_rBitmapRef );

MKI_INTERFACE_MEMBER void
MKI_FileWriteBitmapExport( MKI_CONST_BITMAP_EXPORT_REF f_rBitmapExport );

MKI_INTERFACE_MEMBER void
MKI_FileWritePrintEntry( MKI_CONST_PRINT_ENTRY_REF f_rPrintEntry );

MKI_INTERFACE_MEMBER void
MKI_FileWritePrintPDF( MKI_CONST_PRINT_PDF_REF f_rPrintPDF );

MKI_INTERFACE_MEMBER void
MKI_FileWritePartsListData( MKI_CONST_PARTSLIST_DATA_REF f_rPartsListData );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER void
MKI_FileWriteProfileEntry( __int32 f_nEntry );

MKI_INTERFACE_MEMBER void
MKI_FileWriteInfoEntry( __int32 f_nEntry );

MKI_INTERFACE_MEMBER bool
MKI_FileWriteEndOfFile( void );

MKI_INTERFACE_MEMBER void
MKI_FileWriteZeros( bool f_fValue );

MKI_INTERFACE_MEMBER void
MKI_FileWriteDelimiters( MKI_CONST_TOKEN_DATA_PTR f_pData );

MKI_INTERFACE_MEMBER bool
MKI_FileWriteError( void );

MKI_INTERFACE_MEMBER __int64
MKI_FileWriteCurrentSize( void );

MKI_INTERFACE_MEMBER __int32
MKI_FileWriteCurrentLine( void );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER bool
MKI_FileReadInitDisk( HANDLE f_hFileHandle, bool f_fGenerateCRC );

MKI_INTERFACE_MEMBER bool
MKI_FileReadInitDiskOffset( HANDLE f_hFileHandle, const size_t f_nOffset );

MKI_INTERFACE_MEMBER bool
MKI_FileReadInitMemory( HGLOBAL f_hMemory );

MKI_INTERFACE_MEMBER bool
MKI_FileReadIsUTF8( void );

MKI_INTERFACE_MEMBER bool
MKI_FileReadData( void* f_pData, const size_t f_unSize );

MKI_INTERFACE_MEMBER __int32
MKI_FileReadChar( void );

MKI_INTERFACE_MEMBER bool
MKI_FileReadLine( LPSTR f_pszData, const size_t f_unMaxSize );

MKI_INTERFACE_MEMBER void
MKI_FileReadExit( void );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER void
MKI_FileReadSemi( void );

MKI_INTERFACE_MEMBER void
MKI_FileReadComma( void );

MKI_INTERFACE_MEMBER bool
MKI_FileReadContinue( void );

MKI_INTERFACE_MEMBER __int32
MKI_FileReadLastKeyword( void );

MKI_INTERFACE_MEMBER void
MKI_FileReadKeyword( void );

MKI_INTERFACE_MEMBER void
MKI_FileReadNextKeyword( void );

MKI_INTERFACE_MEMBER void
MKI_FileReadNextSection( void );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER void
MKI_FileReadString( LPWSTR f_pszValue, const size_t f_unMaxSize );

#endif

    template <size_t f_unSize> __forceinline void
    MKI_FileReadString( wchar_t (&f_rDestin)[f_unSize] )
    {
      MKI_FileReadString( (LPWSTR) &f_rDestin, f_unSize );
    }

#ifndef MKI_SERVER

MKI_INTERFACE_MEMBER void
MKI_FileReadBinary( void* f_pValue, const size_t f_unMaxSize );

MKI_INTERFACE_MEMBER void
MKI_FileReadBool( bool& f_rValue );

MKI_INTERFACE_MEMBER void
MKI_FileReadInt16( __int16& f_rValue );

MKI_INTERFACE_MEMBER void
MKI_FileReadInt32( __int32& f_rValue );

MKI_INTERFACE_MEMBER void
MKI_FileReadInt64( __int64& f_rValue );

MKI_INTERFACE_MEMBER void
MKI_FileReadDouble( double& f_rValue );

MKI_INTERFACE_MEMBER void
MKI_FileReadColor( MKI_COLORVALUE_REF f_rValue, MKI_COLORMODEL_PTR f_pColorMode );

MKI_INTERFACE_MEMBER void
MKI_FileReadFontdef( MKI_FONTDEF_REF f_rFont );

MKI_INTERFACE_MEMBER void
MKI_FileReadXProperty( MKI_XPROPERTY_REF f_rXProperty );

MKI_INTERFACE_MEMBER void
MKI_FileReadProperty( MKI_PROPERTY_REF f_rProperty );

MKI_INTERFACE_MEMBER void
MKI_FileReadDimLine( MKI_DIMLINE_REF f_rDimLine );

MKI_INTERFACE_MEMBER void
MKI_FileReadDimSmall( MKI_DIMSMALL_REF f_rDimSmall );

MKI_INTERFACE_MEMBER void
MKI_FileReadDimLarge( MKI_DIMLARGE_REF f_rDimLarge );

MKI_INTERFACE_MEMBER void
MKI_FileReadTextStandard( MKI_TEXTSTANDARD_REF f_rTextStandard );

MKI_INTERFACE_MEMBER void
MKI_FileReadTextFrame( MKI_TEXTFRAME_REF f_rTextFrame );

MKI_INTERFACE_MEMBER void
MKI_FileReadTextReference( MKI_TEXTREFERENCE_REF f_rTextReference );

MKI_INTERFACE_MEMBER void
MKI_FileReadCommentData( MKI_COMMENTDATA_REF f_rCommentData );

MKI_INTERFACE_MEMBER void
MKI_FileReadBitmapRef( MKI_BITMAPREF_REF f_rBitmapRef );

MKI_INTERFACE_MEMBER void
MKI_FileReadBitmapExport( MKI_BITMAP_EXPORT_REF f_rBitmapExport );

MKI_INTERFACE_MEMBER void
MKI_FileReadPrintEntry( MKI_PRINT_ENTRY_REF f_rPrintEntry );

MKI_INTERFACE_MEMBER void
MKI_FileReadPrintPDF( MKI_PRINT_PDF_REF f_rPrintPDF );

MKI_INTERFACE_MEMBER void
MKI_FileReadPartsListData( MKI_PARTSLIST_DATA_REF f_rPartsListData );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER __int32
MKI_FileReadProfileEntry( void );

MKI_INTERFACE_MEMBER __int32
MKI_FileReadInfoEntry( void );

MKI_INTERFACE_MEMBER bool
MKI_FileReadEndOfFile( void );

MKI_INTERFACE_MEMBER void
MKI_FileReadDelimiters( MKI_CONST_TOKEN_DATA_PTR f_pData );

MKI_INTERFACE_MEMBER bool
MKI_FileReadError( void );

MKI_INTERFACE_MEMBER __int64
MKI_FileReadTotalSize( void );

MKI_INTERFACE_MEMBER __int64
MKI_FileReadCurrentSize( void );

MKI_INTERFACE_MEMBER __int32
MKI_FileReadCurrentLine( void );

//---------------------------------------------------------------------------------------------------------------------------------------------------

MKI_INTERFACE_MEMBER void
MKI_FeedbackLog( const LPCWSTR f_pszText );

MKI_INTERFACE_MEMBER bool
MKI_FeedbackNotification( __int32 f_nMode, DWORD f_dwDuration, const LPCWSTR f_pszShortText, const LPCWSTR f_pszLongText );

MKI_INTERFACE_MEMBER void
MKI_FeedbackHiddenGeo( void );

MKI_INTERFACE_MEMBER void
MKI_FeedbackHiddenMark( void );

MKI_INTERFACE_MEMBER bool
MKI_FeedbackLocation( const LPCWSTR f_pszText1, const LPCWSTR f_pszText2, MKI_CONST_POINT_REF f_rPoint );

MKI_INTERFACE_MEMBER bool
MKI_FeedbackLocation( __int32 f_nFeedbackCode, MKI_CONST_POINT_REF f_rPoint );

MKI_INTERFACE_MEMBER bool
MKI_FeedbackTooltip( __int32 f_nFeedbackCode );

//---------------------------------------------------------------------------------------------------------------------------------------------------

#endif // ifndef MKI_SERVER


//---------------------------------------------------------------------------------------------------------------------------------------------------
// Outdated prototypes and definitions of previous Malz++Kassner Open CAD6interface versions for backward compatibility.
//
// Definitions for Interface Version 4.70
//---------------------------------------------------------------------------------------------------------------------------------------------------
constexpr __int32 MKI_470_OBJ_CLIPSURFACE     = 42;
constexpr __int32 MKI_470_DB_INFO_CLIPSURFACE = 242;


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_470_CLIPSURFACE
{
  public:

  MKI_XPROPERTY   m_cXProperty;
  MKI_LIBRARYNAME m_szLibraryName;
  MKI_BLOCKNAME   m_szBlockName;

  MKI_MATRIX  m_cDisplayMatrix;
  __int32     m_nIgnoreBlock;

  __int32     m_nListIndex,
              m_nBlockIndex;
  bool        m_fNotFound;
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_470_BLOCK_CLIPSURFACE
{
  public:

  MKI_BLOCK_HEADER    m_cHeader;   // ElemType = 0, ElemCount = 0
  MKI_470_CLIPSURFACE m_cData;
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_470_PARSE_OBJ_CLIPSURFACE
{
  public:

  __forceinline
  MKI_470_PARSE_OBJ_CLIPSURFACE( MKI_CONST_ENTITY_OBJECT_PTR f_pObj )
  {
    m_pData = ( (MKI_470_BLOCK_CLIPSURFACE*) ( f_pObj->m_pData ) );
    m_pCurve = (MKI_BLOCK_PTR) ( (LPBYTE) ( m_pData ) + m_pData->m_cHeader.m_nSize );
  }

  MKI_470_BLOCK_CLIPSURFACE* m_pData;
  MKI_BLOCK_PTR              m_pCurve;

  __forceinline MKI_470_CLIPSURFACE& Data( void ) { return( m_pData->m_cData ); }
  __forceinline MKI_BLOCK_PTR CurveData( void ) { return( m_pCurve ); }

  private:

  // Disable default constructor
  MKI_470_PARSE_OBJ_CLIPSURFACE( void ) = delete;

};


//---------------------------------------------------------------------------------------------------------------------------------------------------
// Outdated prototypes and definitions of previous Malz++Kassner Open CAD6interface versions for backward compatibility.
//
// Definitions for Interface Version 6.41
//---------------------------------------------------------------------------------------------------------------------------------------------------
constexpr __int32 MKI_641_TOOLBOX_MAX          = 100;
constexpr __int32 MKI_641_BLOCKLIST_MAX        = 100;
constexpr __int32 MKI_641_DRAWING_PREVIEW_SIZE = 160;


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_641_FONTDEF
{
  public:

  __forceinline void
  CopyTo( MKI_FONTDEF_REF f_rDestin ) const
  {
    f_rDestin.m_nFontStyle  = m_nFontStyle;
    f_rDestin.m_nFontType   = m_nFontType;
    f_rDestin.m_nFontWeight = m_nFontWeight;
    MKI_ConvertAnsiToUTF16( m_szFontName, f_rDestin.m_szFontName );
  };

  __int32     m_nFontType,
              m_nFontStyle,
              m_nFontWeight;
  MKI_STR64A  m_szFontName;
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_641_XPROPERTY
{
  public:

  __forceinline void
  CopyTo( MKI_XPROPERTY_REF f_rDestin ) const
  {
    f_rDestin.m_nFlag          = m_nFlag;
    f_rDestin.m_nPen           = m_nPen;
    f_rDestin.m_nFillMode      = m_nFillMode;
    f_rDestin.m_cFillColor     = m_cFillColor;
    f_rDestin.m_cLineColor     = m_cLineColor;
    f_rDestin.m_nLineType      = m_nLineType;
    f_rDestin.m_dLineTypeScale = 1.0;
    f_rDestin.m_dLineWidth     = m_dLineWidth;
    f_rDestin.m_nLineMode      = m_nLineCaps;
    f_rDestin.m_nLayer         = m_nLayer;
    f_rDestin.m_nPage          = MKI_PAGE_MODEL;

    f_rDestin.m_cFillColorMode = MKI_COLORMODE_RGB100;
    f_rDestin.m_cFillAlpha     = MKI_ALPHA_MIN;
    f_rDestin.m_cLineColorMode = MKI_COLORMODE_RGB100;
    f_rDestin.m_cLineAlpha     = MKI_ALPHA_MIN;
  };

  __int32  m_nFlag,
           m_nPen,
           m_nFillMode;
  COLORREF m_cFillColor,
           m_cLineColor;
  __int32  m_nLineType;
  double   m_dLineWidth;
  __int32  m_nLineCaps,
           m_nLayer;
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_641_DIMSMALL
{
  public:

  __forceinline void
  CopyTo( MKI_DIMSMALL_REF f_rDestin ) const
  {
    m_cTextFont.CopyTo( f_rDestin.m_cTextFont );
    m_cTextXProperty.CopyTo( f_rDestin.m_cTextXProperty );
    f_rDestin.m_dTextSize1    = m_dTextSize1;
    f_rDestin.m_dTextSize2    = m_dTextSize2;
    f_rDestin.m_dCharDistance = m_dCharDistance;
    f_rDestin.m_dTabDistance  = m_dTabDistance;
    f_rDestin.m_nTextMode     = m_nTextMode;
    f_rDestin.m_nTextVertical = MKI_TEXTVERTICAL_LINETOP;
    f_rDestin.m_nTextOrder    = m_nTextOrder;
    f_rDestin.m_nNumMode      = m_nNumMode;
    f_rDestin.m_nNumPrecision = m_nNumPrecision;
    f_rDestin.m_nNumRefresh   = m_nNumRefresh;
    f_rDestin.m_nNumRotate    = m_nNumRotate;
    f_rDestin.m_nSystem       = m_nSystem;
  };

  MKI_641_FONTDEF   m_cTextFont;
  MKI_641_XPROPERTY m_cTextXProperty;

  double  m_dTextSize1,
          m_dTextSize2,
          m_dCharDistance,
          m_dTabDistance;
  __int32 m_nTextMode,
          m_nTextOrder,
          m_nNumMode,
          m_nNumPrecision,
          m_nNumRefresh,
          m_nNumRotate,
          m_nSystem;
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_641_DIMLARGE : public MKI_641_DIMSMALL
{
  public:

  __forceinline void
  CopyTo( MKI_DIMLARGE_REF f_rDestin ) const
  {
    ((MKI_641_DIMSMALL*)this)->CopyTo( (MKI_DIMSMALL_REF) f_rDestin );

    f_rDestin.m_fNumCentered     = m_fNumCentered;
    f_rDestin.m_nNumPlacing      = m_nNumPlacing;
    f_rDestin.m_nArrowStartForm  = m_nArrowStartForm;
    f_rDestin.m_nArrowStartMode  = m_nArrowStartMode;
    f_rDestin.m_nArrowEndForm    = m_nArrowEndForm;
    f_rDestin.m_nArrowEndMode    = m_nArrowEndMode;
    f_rDestin.m_fExtStartDisplay = m_fExtStartDisplay;
    f_rDestin.m_fExtEndDisplay   = m_fExtEndDisplay;
    f_rDestin.m_fLineDisplay     = m_fLineDisplay;
    f_rDestin.m_nLineOrientation = m_nLineOrientation;
    f_rDestin.m_nLineDimType     = m_nLineDimType;
    f_rDestin.m_nLineDistMode    = m_nLineDistMode;
    f_rDestin.m_dLineDistance    = m_dLineDistance;
    f_rDestin.m_dLineStartOffset = m_dLineStartOffset;
    f_rDestin.m_dLineEndOffset   = m_dLineEndOffset;
  };

  bool    m_fNumCentered;
  __int32 m_nNumPlacing,
          m_nArrowStartForm,
          m_nArrowStartMode,
          m_nArrowEndForm,
          m_nArrowEndMode;
  bool    m_fExtStartDisplay,
          m_fExtEndDisplay,
          m_fLineDisplay;
  __int32 m_nLineOrientation,
          m_nLineDimType,
          m_nLineDistMode;
  double  m_dLineDistance,
          m_dLineStartOffset,
          m_dLineEndOffset;
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_641_TEXTSTANDARD
{
  public:

  __forceinline void
  CopyTo( MKI_TEXTSTANDARD_REF f_rDestin ) const
  {
    m_cTextFont.CopyTo( f_rDestin.m_cTextFont );
    m_cTextXProperty.CopyTo( f_rDestin.m_cTextXProperty );
    f_rDestin.m_cTextMatrix   = m_cTextMatrix;
    f_rDestin.m_dCharDistance = m_dCharDistance;
    f_rDestin.m_dTabDistance  = m_dTabDistance;
    f_rDestin.m_dLineDistance = m_dLineDistance;
    f_rDestin.m_nTextMode     = m_nTextMode;
    f_rDestin.m_nTextVertical = MKI_TEXTVERTICAL_LINETOP;
    f_rDestin.m_nTextOrder    = m_nTextOrder;
    f_rDestin.m_nDisplayMode  = m_nDisplayMode;
  };

  MKI_641_FONTDEF   m_cTextFont;
  MKI_641_XPROPERTY m_cTextXProperty;
  MKI_MATRIX        m_cTextMatrix;
  double            m_dCharDistance,
                    m_dTabDistance,
                    m_dLineDistance;
  __int32           m_nTextMode,
                    m_nTextOrder,
                    m_nDisplayMode;
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_641_TEXTFRAME
{
  public:

  __forceinline void
  CopyTo( MKI_TEXTFRAME_REF f_rDestin ) const
  {
    m_cTextFont.CopyTo( f_rDestin.m_cTextFont );
    m_cTextXProperty.CopyTo( f_rDestin.m_cTextXProperty );
    f_rDestin.m_dTextSize     = m_dTextSize;
    f_rDestin.m_dCharDistance = m_dCharDistance;
    f_rDestin.m_dTabDistance  = m_dTabDistance;
    f_rDestin.m_dLineDistance = m_dLineDistance;
    f_rDestin.m_nTextMode     = m_nTextMode;
    f_rDestin.m_nTextVertical = MKI_TEXTVERTICAL_FRAMETOP;
    f_rDestin.m_nTextOrder    = m_nTextOrder;
  };

  MKI_641_FONTDEF   m_cTextFont;
  MKI_641_XPROPERTY m_cTextXProperty;
  double            m_dTextSize,
                    m_dCharDistance,
                    m_dTabDistance,
                    m_dLineDistance;
  __int32           m_nTextMode,
                    m_nTextOrder;
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
// Outdated prototypes and definitions of previous Malz++Kassner Open CAD6interface versions for backward compatibility.
//
// Definitions for Interface Version 6150
//---------------------------------------------------------------------------------------------------------------------------------------------------
typedef MKI_STR64A  MKI_6150_BLOCKNAME;
typedef MKI_STR64A  MKI_6150_LAYERNAME;
typedef MKI_STR64A  MKI_6150_LINENAME;
typedef MKI_STR64A  MKI_6150_HATCHNAME;
typedef MKI_STR64A  MKI_6150_LIBRARYNAME;
typedef MKI_STR64A  MKI_6150_MULTILINENAME;
typedef MKI_STR64A  MKI_6150_PENNAME;
typedef MKI_STR64A  MKI_6150_SYSTEMNAME;


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_6150_HATCHDEF
{
  public:

  __forceinline void
  CopyTo( MKI_HATCHDEF_REF f_rDestin ) const
  {
    MKI_ConvertAnsiToUTF16( m_szHatchName, f_rDestin.m_szHatchName );
    f_rDestin.m_fHatchLines1Active = m_fHatchLine1Active;
    f_rDestin.m_nHatchLineIndex1   = m_nHatchLine1;
    f_rDestin.m_dHatchLines1Rotate = m_dHatchLine1Rotate;
    f_rDestin.m_fHatchLines2Active = m_fHatchLine2Active;
    f_rDestin.m_nHatchLineIndex2   = m_nHatchLine2;
    f_rDestin.m_dHatchLines2Rotate = m_dHatchLine2Rotate;
    f_rDestin.m_fHatchBlockActive  = m_fHatchBlockActive;
    MKI_ConvertAnsiToUTF16( m_szHatchLibraryName, f_rDestin.m_szHatchLibraryName );
    MKI_ConvertAnsiToUTF16( m_szHatchBlockName,   f_rDestin.m_szHatchBlockName );
    f_rDestin.m_dHatchBlockRotate  = m_dHatchBlockRotate,
    f_rDestin.m_dHatchBlockScale   = m_dHatchBlockScale,
    f_rDestin.m_dHatchXStep1       = m_dHatchXStep1,
    f_rDestin.m_dHatchXStep2       = m_dHatchXStep2,
    f_rDestin.m_dHatchYStep1       = m_dHatchYStep1,
    f_rDestin.m_dHatchYStep2       = m_dHatchYStep2,
    f_rDestin.m_dHatchLineStep1    = m_dHatchLineStep1,
    f_rDestin.m_dHatchLineStep2    = m_dHatchLineStep2,
    f_rDestin.m_dHatchRotate       = m_dHatchRotate,
    f_rDestin.m_dHatchOffset1      = m_dHatchOffset1,
    f_rDestin.m_dHatchOffset2      = m_dHatchOffset2;
  };

  MKI_6150_HATCHNAME  m_szHatchName;

  bool    m_fHatchLine1Active;
  __int32 m_nHatchLine1;
  double  m_dHatchLine1Rotate;
  bool    m_fHatchLine2Active;
  __int32 m_nHatchLine2;
  double  m_dHatchLine2Rotate;
  bool    m_fHatchBlockActive;

  MKI_6150_LIBRARYNAME  m_szHatchLibraryName;
  MKI_6150_BLOCKNAME    m_szHatchBlockName;

  double  m_dHatchBlockRotate,
          m_dHatchBlockScale,
          m_dHatchXStep1,
          m_dHatchXStep2,
          m_dHatchYStep1,
          m_dHatchYStep2,
          m_dHatchLineStep1,
          m_dHatchLineStep2,
          m_dHatchRotate,
          m_dHatchOffset1,
          m_dHatchOffset2;
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
// Outdated prototypes and definitions of previous Malz++Kassner Open CAD6interface versions for backward compatibility.
//
// Definitions for Interface Version 6160
//---------------------------------------------------------------------------------------------------------------------------------------------------
constexpr __int32 MKI_6160_PAGE_MAX = 100;
constexpr __int32 MKI_6160_MULTILINE_MAX = 50;
constexpr __int32 MKI_6160_MKD_KEY_MULTILINE = 18;


//---------------------------------------------------------------------------------------------------------------------------------------------------
// Outdated prototypes and definitions of previous Malz++Kassner Open CAD6interface versions for backward compatibility.
//
// Definitions for Interface Version 6190
//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_6190_PARSE_OBJ_COMMENT
{
  public:

  __forceinline
  MKI_6190_PARSE_OBJ_COMMENT( MKI_CONST_ENTITY_OBJECT_PTR f_pObj )
  {
    m_pArrow  = (MKI_BLOCK_POINT_PTR)       f_pObj->m_pData;                                            // b1
    m_pCenter = (MKI_BLOCK_POINT_PTR)       ( (LPBYTE) ( m_pArrow ) + m_pArrow->m_cHeader.m_nSize );    // b2
    m_pText   = (MKI_BLOCK_TEXT_PTR)        ( (LPBYTE) ( m_pCenter ) + m_pCenter->m_cHeader.m_nSize );  // b110
    m_pData   = (MKI_BLOCK_COMMENTDATA_PTR) ( (LPBYTE) ( m_pText ) + m_pText->m_cHeader.m_nSize );      // b240
  }

  MKI_BLOCK_POINT_PTR m_pArrow,
                      m_pCenter;
  MKI_BLOCK_TEXT_PTR m_pText;
  MKI_BLOCK_COMMENTDATA_PTR m_pData;

  __forceinline MKI_POINT_REF PointArrow( void ) { return( m_pArrow->m_pData[0] ); }
  __forceinline MKI_POINT_REF PointCenter( void ) { return( m_pCenter->m_pData[0] ); }
  __forceinline LPWSTR Text( void ) { return( m_pText->m_pText ); }
  __forceinline MKI_COMMENTDATA_REF CommentData( void ) { return( m_pData->m_cData ); }

  private:

  // Disable default constructor
  MKI_6190_PARSE_OBJ_COMMENT( void ) = delete;

};


//---------------------------------------------------------------------------------------------------------------------------------------------------
// Outdated prototypes and definitions of previous Malz++Kassner Open CAD6interface versions for backward compatibility.
//
// Definitions for Interface Version 6191
//---------------------------------------------------------------------------------------------------------------------------------------------------
constexpr __int32 MKI_6191_BLOCKS_PER_DRAWING = 100000; // Maximum number of blocks per drawing / library


//---------------------------------------------------------------------------------------------------------------------------------------------------
class MKI_6191_XPROPERTY
{
  public:

  __forceinline void
  CopyTo( MKI_XPROPERTY_REF f_rDestin ) const
  {
    f_rDestin.m_nFlag          = m_nFlag;
    f_rDestin.m_nPen           = m_nPen;
    f_rDestin.m_nFillMode      = m_nFillMode;
    f_rDestin.m_cFillColor     = m_cFillColor;
    f_rDestin.m_cLineColor     = m_cLineColor;
    f_rDestin.m_nLineType      = m_nLineType;
    f_rDestin.m_dLineWidth     = m_dLineWidth;
    f_rDestin.m_nLineMode      = m_nLineMode;
    f_rDestin.m_nLayer         = m_nLayer;
    f_rDestin.m_nPage          = m_nPage;
    f_rDestin.m_cFillColorMode = m_cFillColorMode;
    f_rDestin.m_cFillAlpha     = m_cFillAlpha;
    f_rDestin.m_cLineColorMode = m_cLineColorMode;
    f_rDestin.m_cLineAlpha     = m_cLineAlpha;
    f_rDestin.m_dLineTypeScale = m_dLineTypeScale;
    f_rDestin.m_nFromPen       = MKI_USE_NULL;
    f_rDestin.m_nFromLayer     = MKI_USE_NULL;
  };

  __int32        m_nFlag,
                 m_nPen,
                 m_nFillMode;
  MKI_COLORVALUE m_cFillColor,
                 m_cLineColor;
  __int32        m_nLineType;
  double         m_dLineWidth;
  __int32        m_nLineMode,
                 m_nLayer,
                 m_nPage;
  MKI_COLORMODEL m_cFillColorMode;
  MKI_ALPHAVALUE m_cFillAlpha;
  MKI_COLORMODEL m_cLineColorMode;
  MKI_ALPHAVALUE m_cLineAlpha;
  double         m_dLineTypeScale;
};


//---------------------------------------------------------------------------------------------------------------------------------------------------
// Outdated prototypes and definitions of previous Malz++Kassner Open CAD6interface versions for backward compatibility.
//
// Definitions for Interface Version 6222
//---------------------------------------------------------------------------------------------------------------------------------------------------
constexpr __int32 MKI_6222_SYSTEM_MAX = 50;


//---------------------------------------------------------------------------------------------------------------------------------------------------
// Outdated prototypes and definitions of previous Malz++Kassner Open CAD6interface versions for backward compatibility.
//
// Definitions for Interface Version 6231
//---------------------------------------------------------------------------------------------------------------------------------------------------
constexpr __int32 MKI_6231_HATCH_MAX = 100;


#endif // ifndef MKI_RESOURCE
